# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `numdiff` Chapter.

``numdiff`` - Numerical Differentiation

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with calculating approximations to derivatives
of a function f.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d04/d04intro.html
"""

# NAG Copyright 2017-2019.

import sys as _sys

from ..base import utils

def fwd(xval, nder, hbase, der, erest, f, data=None):
    u"""
    Numerical differentiation, derivatives up to order 14, function of
    one real variable.

    For full information please refer to the NAG Library document for
    d04aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d04/d04aaf.html

    Parameters
    ----------
    xval : float
        The point at which the derivatives are required, x_0.

    nder : int
        Must be set so that its absolute value is the highest order
        derivative required.

        `nder` > 0

          All derivatives up to order min(`nder`, 14) are calculated.

        `nder` < 0 and `nder` is even

          Only even order derivatives up to order min(-`nder`, 14) are
          calculated.

        `nder` < 0 and `nder` is odd

          Only odd order derivatives up to order min(-`nder`, 13) are
          calculated.

    hbase : float
        The initial step length which may be positive or negative. For
        advice on the choice of `hbase` see [Further Comments].

    der : float, shape (14), modified in place
        On exit: `der`[j-1] contains an approximation to the jth
        derivative of f(x) at x = `xval`, so long as the jth derivative
        is one of those requested by you when specifying `nder`. For
        other values of j, `der`[j-1] is unused.

    erest : float, shape (14), modified in place
        On exit: an estimate of the absolute error in the corresponding
        result `der`[j-1] so long as the jth derivative is one of those
        requested by you when specifying `nder`. The sign of
        `erest`[j-1] is positive unless the result `der`[j-1] is
        questionable. It is set negative when |`der`[j-1]| <
        |`erest`[j-1]| or when for some other reason there is doubt
        about the validity of the result `der`[j-1] (see [Errors]). For
        other values of j, `erest`[j-1] is unused.

    f : callable retval = f(x, data=None)
        `f` must evaluate the function f(x) at a specified point.

        Parameters
        ~~~~~~~~~~
        x : float
            The value of the argument x.

            If you have equally spaced tabular data, the following
            information may be useful:

            - in any call of ``fwd`` the only values of x for which f(x)
              will be required are x = `xval` and x =
              `xval`+/-(2*j-1)`hbase`, for j = 1,2,...,10; and

            - f(x_0) is always computed, but it is disregarded when only
              odd order derivatives are required.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f(x) at the specified point.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nder` = 0.

            Constraint: `nder` != 0.

        (`errno` 1)
            On entry, `hbase` = 0.0.

            Constraint: `hbase` != 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.numdiff.fwd'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('d04aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, xval.
        utils._BYREF_INT, # Input, nder.
        utils._BYREF_FLOAT64, # Input, hbase.
        utils._EngineFloat64ArrayType.c_type, # Output, der[14].
        utils._EngineFloat64ArrayType.c_type, # Output, erest[14].
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _xval = utils._EngineFloat64ScalarType(
        xval, {'fun_name': fun_name, 'entity': 'xval'},
    ).to_ctypes()
    _nder = utils._EngineIntScalarType(
        nder, {'fun_name': fun_name, 'entity': 'nder'},
    ).to_ctypes()
    _hbase = utils._EngineFloat64ScalarType(
        hbase, {'fun_name': fun_name, 'entity': 'hbase'},
    ).to_ctypes()
    _der_cls = utils._EngineFloat64ArrayType(
        der, {'fun_name': fun_name, 'entity': 'der'},
        exp_shape=(14,),
        check_contents=False,
    )
    _der = _der_cls.to_ctypes(can_reference_contents=False)
    _erest_cls = utils._EngineFloat64ArrayType(
        erest, {'fun_name': fun_name, 'entity': 'erest'},
        exp_shape=(14,),
        check_contents=False,
    )
    _erest = _erest_cls.to_ctypes(can_reference_contents=False)
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _xval,
        _nder,
        _hbase,
        _der,
        _erest,
        _f,
        _fsub,
        _fh,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nder = 0.',
                    'Constraint: nder != 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, hbase = 0.0.',
                    'Constraint: hbase != 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _der_cls.refresh_py(_der)
        _erest_cls.refresh_py(_erest)
        return_data = None
    _err_class.manage()
    return return_data

def rcomm(xval, fval, der, erest):
    u"""
    Numerical differentiation, user-supplied function values,
    derivatives up to order 14, derivatives with respect to one real
    variable.

    For full information please refer to the NAG Library document for
    d04ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d04/d04baf.html

    Parameters
    ----------
    xval : float, shape (21)
        The abscissae at which the function has been evaluated, as
        described in [Description]. These can be generated by calling
        ``sample``. The order of the abscissae is irrelevant.

    fval : float, shape (21)
        `fval`[j-1] must contain the function value at `xval`[j-1], for
        j = 1,2,...,21.

    der : float, shape (14), modified in place
        On exit: the 14 derivative estimates.

    erest : float, shape (14), modified in place
        On exit: the 14 error estimates for the derivatives.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The derived h is below tolerance.

            Derived h > *<value>* is required. Derived h = *<value>*.

        (`errno` 1)
            On entry, the values of `xval` are not correctly spaced.

            Derived h = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.numdiff.rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d04baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, xval[21].
        utils._EngineFloat64ArrayType.c_type, # Input, fval[21].
        utils._EngineFloat64ArrayType.c_type, # Output, der[14].
        utils._EngineFloat64ArrayType.c_type, # Output, erest[14].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _xval = utils._EngineFloat64ArrayType(
        xval, {'fun_name': fun_name, 'entity': 'xval'},
        exp_shape=(21,),
    ).to_ctypes()
    _fval = utils._EngineFloat64ArrayType(
        fval, {'fun_name': fun_name, 'entity': 'fval'},
        exp_shape=(21,),
    ).to_ctypes()
    _der_cls = utils._EngineFloat64ArrayType(
        der, {'fun_name': fun_name, 'entity': 'der'},
        exp_shape=(14,),
        check_contents=False,
    )
    _der = _der_cls.to_ctypes(can_reference_contents=False)
    _erest_cls = utils._EngineFloat64ArrayType(
        erest, {'fun_name': fun_name, 'entity': 'erest'},
        exp_shape=(14,),
        check_contents=False,
    )
    _erest = _erest_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _xval,
        _fval,
        _der,
        _erest,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The derived h is below tolerance.',
                    'Derived h > {:s} is required.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Derived h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the values of xval are not correctly spaced.',
                    'Derived h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _der_cls.refresh_py(_der)
        _erest_cls.refresh_py(_erest)
        return_data = None
    _err_class.manage()
    return return_data

def sample(x_0, hbase, xval):
    u"""
    Generates sample points for function evaluations by ``rcomm``.

    For full information please refer to the NAG Library document for
    d04bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/d04/d04bbf.html

    Parameters
    ----------
    x_0 : float
        The abscissa x_0 at which derivatives are required.

    hbase : float
        The chosen step size h. If h < 10 epsilon, where epsilon =
        machine.precision(), the default h = epsilon^(1/4) will be used.

    xval : float, shape (21), modified in place
        On exit: the abscissae for passing to ``rcomm``.
    """

    excs = []
    fun_name = 'naginterfaces.base.numdiff.sample'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('d04bbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x_0.
        utils._BYREF_FLOAT64, # Input, hbase.
        utils._EngineFloat64ArrayType.c_type, # Output, xval[21].
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x_0 = utils._EngineFloat64ScalarType(
        x_0, {'fun_name': fun_name, 'entity': 'x_0'},
    ).to_ctypes()
    _hbase = utils._EngineFloat64ScalarType(
        hbase, {'fun_name': fun_name, 'entity': 'hbase'},
    ).to_ctypes()
    _xval_cls = utils._EngineFloat64ArrayType(
        xval, {'fun_name': fun_name, 'entity': 'xval'},
        exp_shape=(21,),
        check_contents=False,
    )
    _xval = _xval_cls.to_ctypes(can_reference_contents=False)
    engine_ctypes(
        _en_data,
        _x_0,
        _hbase,
        _xval,
    )
    utils._handle_cb_exception(excs)
    _xval_cls.refresh_py(_xval)
    return_data = None
    return return_data
