# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `sort` Chapter.

``sort`` - Sorting and Searching

This module is concerned with sorting and searching numeric or character
data.
It handles only the simplest types of data structure and it is concerned
only with internal sorting and searching -- that is, sorting and
searching a set of data which can all be stored within the program.

If you have large files of data or complicated data structures to be
sorted or searched you should use a comprehensive sorting or searching
program or package.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def realvec_sort(rv, m1, order):
    u"""
    Sort a vector, real numbers.

    For full information please refer to the NAG Library document for
    m01ca

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01caf.html

    Parameters
    ----------
    rv : float, array-like, shape (m2)
        Elements `m1` to m2 of `rv` must contain double values to be
        sorted.

    m1 : int
        The index of the first element of `rv` to be sorted.

    order : str, length 1
        If `order` = 'A', the values will be sorted into ascending
        order.

        If `order` = 'D', into descending order.

    Returns
    -------
    rv : float, ndarray, shape (m2)
        These values are rearranged into sorted order.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, `order` has an illegal value: `order` = *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.sort import realvec_sort as b_realvec_sort

    fun_name = 'naginterfaces.library.sort.realvec_sort'

    _rv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rv'},
        rv,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m2_py = _b_utils._EngineFloat64ArrayType.get_shape(_rv, {'fun_name': fun_name, 'entity': 'rv'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    b_realvec_sort(
        _rv,
        m1,
        _m2,
        order,
    )
    _rv_py = _rv
    return _rv_py

def intvec_sort(iv, m1, order):
    u"""
    Sort a vector, integer numbers.

    For full information please refer to the NAG Library document for
    m01cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01cbf.html

    Parameters
    ----------
    iv : int, array-like, shape (m2)
        Elements `m1` to m2 of `iv` must contain integer values to be
        sorted.

    m1 : int
        The index of the first element of `iv` to be sorted.

    order : str, length 1
        If `order` = 'A', the values will be sorted into ascending
        order.

        If `order` = 'D', into descending order.

    Returns
    -------
    iv : int, ndarray, shape (m2)
        These values are rearranged into sorted order.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, `order` has an illegal value: `order` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import intvec_sort as b_intvec_sort

    fun_name = 'naginterfaces.library.sort.intvec_sort'

    _iv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iv'},
        iv,
        copy=True,
    )
    _m2_py = _b_utils._EngineIntArrayType.get_shape(_iv, {'fun_name': fun_name, 'entity': 'iv'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    b_intvec_sort(
        _iv,
        m1,
        _m2,
        order,
    )
    _iv_py = _iv
    return _iv_py

def charvec_sort(ch, m1, l1, l2, order):
    u"""
    Sort a vector, character data.

    For full information please refer to the NAG Library document for
    m01cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01ccf.html

    Parameters
    ----------
    ch : str, array-like, shape (m2)
        Elements `m1` to m2 of `ch` must contain character data to be
        sorted.

    m1 : int
        The index of the first element of `ch` to be sorted.

    l1 : int
        Only the substring (`l1`:`l2`) of each element of `ch` is to be
        used in determining the sorted order.

    l2 : int
        Only the substring (`l1`:`l2`) of each element of `ch` is to be
        used in determining the sorted order.

    order : str, length 1
        If `order` = 'A', the values will be sorted into ASCII order.

        If `order` = 'R', into reverse ASCII order.

    Returns
    -------
    ch : str, ndarray, shape (m2)
        These values are rearranged into sorted order.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `l2` = *<value>* and len(`ch`[0]) = *<value>*.

            Constraint: `l2` <= len(`ch`[0]).

        (`errno` 1)
            On entry, `l1` = *<value>* and `l2` = *<value>*.

            Constraint: `l1` <= `l2`.

        (`errno` 1)
            On entry, `l1` = *<value>*.

            Constraint: `l1` >= 1.

        (`errno` 1)
            On entry, `l2` = *<value>*.

            Constraint: `l2` >= 1.

        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, `order` has an illegal value: `order` = *<value>*.

        (`errno` 3)
            On entry, len(`ch`[0]) = *<value>*.

            Constraint: len(`ch`[0]) <= 255.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import charvec_sort as b_charvec_sort

    fun_name = 'naginterfaces.library.sort.charvec_sort'

    _ch = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ch'},
        ch,
        copy=True,
    )
    _m2_py = _b_utils._EngineCharArrayType.get_shape(_ch, {'fun_name': fun_name, 'entity': 'ch'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    b_charvec_sort(
        _ch,
        m1,
        _m2,
        l1,
        l2,
        order,
    )
    _ch_py = _ch
    return _ch_py

def realvec_rank(rv, m1, order):
    u"""
    Rank a vector, real numbers.

    For full information please refer to the NAG Library document for
    m01da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01daf.html

    Parameters
    ----------
    rv : float, array-like, shape (m2)
        Elements `m1` to m2 of `rv` must contain double values to be
        ranked.

    m1 : int
        The index of the first element of `rv` to be ranked.

    order : str, length 1
        If `order` = 'A', the values will be ranked in ascending order.

        If `order` = 'D', into descending order.

    Returns
    -------
    irank : int, ndarray, shape (m2)
        Elements `m1` to m2 of `irank` contain the ranks of the
        corresponding elements of `rv`. Note that the ranks are in the
        range `m1` to m2: thus, if `rv`[i-1] is the first element in the
        rank order, `irank`[i-1] is set to `m1`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, `order` has an illegal value: `order` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import realvec_rank as b_realvec_rank

    fun_name = 'naginterfaces.library.sort.realvec_rank'

    _rv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rv'},
        rv,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m2_py = _b_utils._EngineFloat64ArrayType.get_shape(_rv, {'fun_name': fun_name, 'entity': 'rv'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _irank = _b_utils._EngineIntArrayType.empty_ndarray(
        (_m2_py,),
    )
    b_realvec_rank(
        _rv,
        m1,
        _m2,
        order,
        _irank,
    )
    _irank_py = _irank
    return _irank_py

def intvec_rank(iv, m1, order):
    u"""
    Rank a vector, integer numbers.

    For full information please refer to the NAG Library document for
    m01db

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01dbf.html

    Parameters
    ----------
    iv : int, array-like, shape (m2)
        Elements `m1` to m2 of `iv` must contain integer values to be
        ranked.

    m1 : int
        The index of the first element of `iv` to be ranked.

    order : str, length 1
        If `order` = 'A', the values will be ranked in ascending order.

        If `order` = 'D', into descending order.

    Returns
    -------
    irank : int, ndarray, shape (m2)
        Elements `m1` to m2 of `irank` contain the ranks of the
        corresponding elements of `iv`. Note that the ranks are in the
        range `m1` to m2: thus, if `iv`[i-1] is the first element in the
        rank order, `irank`[i-1] is set to `m1`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, `order` has an illegal value: `order` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import intvec_rank as b_intvec_rank

    fun_name = 'naginterfaces.library.sort.intvec_rank'

    _iv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iv'},
        iv,
        copy=False,
    )
    _m2_py = _b_utils._EngineIntArrayType.get_shape(_iv, {'fun_name': fun_name, 'entity': 'iv'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _irank = _b_utils._EngineIntArrayType.empty_ndarray(
        (_m2_py,),
    )
    b_intvec_rank(
        _iv,
        m1,
        _m2,
        order,
        _irank,
    )
    _irank_py = _irank
    return _irank_py

def charvec_rank(ch, m1, l1, l2, order):
    u"""
    Rank a vector, character data.

    For full information please refer to the NAG Library document for
    m01dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01dcf.html

    Parameters
    ----------
    ch : str, array-like, shape (m2)
        Elements `m1` to m2 of `ch` must contain character data to be
        ranked.

    m1 : int
        The index of the first element of `ch` to be ranked.

    l1 : int
        Only the substring (`l1`:`l2`) of each element of `ch` is to be
        used in determining the rank order.

    l2 : int
        Only the substring (`l1`:`l2`) of each element of `ch` is to be
        used in determining the rank order.

    order : str, length 1
        If `order` = 'A', the values will be ranked in ASCII order.

        If `order` = 'R', in reverse ASCII order.

    Returns
    -------
    irank : int, ndarray, shape (m2)
        Elements `m1` to m2 of `irank` contain the ranks of the
        corresponding elements of `ch`. Note that the ranks are in the
        range `m1` to m2: thus, if `ch`[i-1] is the first element in the
        rank order, `irank`[i-1] is set to `m1`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `l2` = *<value>* and len(`ch`[0]) = *<value>*.

            Constraint: `l2` <= len(`ch`[0]).

        (`errno` 1)
            On entry, `l1` = *<value>* and `l2` = *<value>*.

            Constraint: `l1` <= `l2`.

        (`errno` 1)
            On entry, `l1` = *<value>*.

            Constraint: `l1` >= 1.

        (`errno` 1)
            On entry, `l2` = *<value>*.

            Constraint: `l2` >= 1.

        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, `order` has an illegal value: `order` = *<value>*.

        (`errno` 3)
            On entry, len(`ch`[0]) = *<value>*.

            Constraint: len(`ch`[0]) <= 255.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import charvec_rank as b_charvec_rank

    fun_name = 'naginterfaces.library.sort.charvec_rank'

    _ch = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ch'},
        ch,
        copy=False,
    )
    _m2_py = _b_utils._EngineCharArrayType.get_shape(_ch, {'fun_name': fun_name, 'entity': 'ch'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _irank = _b_utils._EngineIntArrayType.empty_ndarray(
        (_m2_py,),
    )
    b_charvec_rank(
        _ch,
        m1,
        _m2,
        l1,
        l2,
        order,
        _irank,
    )
    _irank_py = _irank
    return _irank_py

def realmat_rank_rows(rm, m1, n1, order):
    u"""
    Rank rows of a matrix, real numbers.

    For full information please refer to the NAG Library document for
    m01de

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01def.html

    Parameters
    ----------
    rm : float, array-like, shape (m2, n2)
        Columns `n1` to n2 of rows `m1` to m2 of `rm` must contain
        double data to be ranked.

    m1 : int
        Note: this argument represents an array index; the value you
        supply must be base-1 for compatibility with the NAG Engine.

        The index of the first row of `rm` to be ranked.

    n1 : int
        The index of the first column of `rm` to be used.

    order : str, length 1
        If `order` = 'A', the rows will be ranked in ascending order.

        If `order` = 'D', into descending order.

    Returns
    -------
    irank : int, ndarray, shape (m2)
        Elements `m1` to m2 of `irank` contain the ranks of the
        corresponding rows of `rm`. Note that the ranks are in the range
        `m1` to m2: thus, if the ith row of `rm` is the first in the
        rank order, `irank`[i-1] is set to `m1`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m2 = *<value>* and ldm = *<value>*.

            Constraint: m2 <= ldm.

        (`errno` 1)
            On entry, `n1` = *<value>* and n2 = *<value>*.

            Constraint: `n1` <= n2.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 1)
            On entry, n2 = *<value>*.

            Constraint: n2 >= 1.

        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constratint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, `order` has an illegal value: `order` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import realmat_rank_rows as b_realmat_rank_rows

    fun_name = 'naginterfaces.library.sort.realmat_rank_rows'

    _rm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rm'},
        rm,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m2_py = _b_utils._EngineFloat64ArrayType.get_shape(_rm, {'fun_name': fun_name, 'entity': 'rm'}, exp_rank=2)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _n2_py = _b_utils._EngineFloat64ArrayType.get_shape(_rm, {'fun_name': fun_name, 'entity': 'rm'}, exp_rank=2)[1]
    _n2 = _b_utils._EngineIntScalarType(
        _n2_py, {'fun_name': fun_name, 'entity': '_n2'},
    ).to_ctypes()
    _irank = _b_utils._EngineIntArrayType.empty_ndarray(
        (_m2_py,),
    )
    b_realmat_rank_rows(
        _rm,
        m1,
        _m2,
        n1,
        _n2,
        order,
        _irank,
    )
    _irank_py = _irank
    return _irank_py

def intmat_rank_rows(im, m1, n1, order):
    u"""
    Rank rows of a matrix, integer numbers.

    For full information please refer to the NAG Library document for
    m01df

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01dff.html

    Parameters
    ----------
    im : int, array-like, shape (m2, n2)
        Columns `n1` to n2 of rows `m1` to m2 of `im` must contain int
        data to be ranked.

    m1 : int
        Note: this argument represents an array index; the value you
        supply must be base-1 for compatibility with the NAG Engine.

        The index of the first row of `im` to be ranked.

    n1 : int
        The index of the first column of `im` to be used.

    order : str, length 1
        If `order` = 'A', the rows will be ranked in ascending order.

        If `order` = 'D', into descending order.

    Returns
    -------
    irank : int, ndarray, shape (m2)
        Elements `m1` to m2 of `irank` contain the ranks of the
        corresponding rows of `im`. Note that the ranks are in the range
        `m1` to m2: thus, if the ith row of `im` is the first in the
        rank order, `irank`[i-1] is set to `m1`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m2 = *<value>* and ldm = *<value>*.

            Constraint: m2 <= ldm.

        (`errno` 1)
            On entry, `n1` = *<value>* and n2 = *<value>*.

            Constraint: `n1` <= n2.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 1)
            On entry, n2 = *<value>*.

            Constraint: n2 >= 1.

        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, `order` has an illegal value: `order` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import intmat_rank_rows as b_intmat_rank_rows

    fun_name = 'naginterfaces.library.sort.intmat_rank_rows'

    _im = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'im'},
        im,
        copy=False,
    )
    _m2_py = _b_utils._EngineIntArrayType.get_shape(_im, {'fun_name': fun_name, 'entity': 'im'}, exp_rank=2)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _n2_py = _b_utils._EngineIntArrayType.get_shape(_im, {'fun_name': fun_name, 'entity': 'im'}, exp_rank=2)[1]
    _n2 = _b_utils._EngineIntScalarType(
        _n2_py, {'fun_name': fun_name, 'entity': '_n2'},
    ).to_ctypes()
    _irank = _b_utils._EngineIntArrayType.empty_ndarray(
        (_m2_py,),
    )
    b_intmat_rank_rows(
        _im,
        m1,
        _m2,
        n1,
        _n2,
        order,
        _irank,
    )
    _irank_py = _irank
    return _irank_py

def realmat_rank_columns(rm, m1, n1, order):
    u"""
    Rank columns of a matrix, real numbers.

    For full information please refer to the NAG Library document for
    m01dj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01djf.html

    Parameters
    ----------
    rm : float, array-like, shape (m2, n2)
        Rows `m1` to m2 of columns `n1` to n2 of `rm` must contain
        double data to be ranked.

    m1 : int
        The index of the first row of `rm` to be used.

    n1 : int
        The index of the first column of `rm` to be ranked.

    order : str, length 1
        If `order` = 'A', the columns will be ranked in ascending order.

        If `order` = 'D', into descending order.

    Returns
    -------
    irank : int, ndarray, shape (n2)
        Elements `n1` to n2 of `irank` contain the ranks of the
        corresponding columns of `rm`. Note that the ranks are in the
        range `n1` to n2: thus, if the ith column of `rm` is the first
        in the rank order, `irank`[i-1] is set to `n1`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m2 = *<value>* and ldm = *<value>*.

            Constraint: m2 <= ldm.

        (`errno` 1)
            On entry, `n1` = *<value>* and n2 = *<value>*.

            Constraint: `n1` <= n2.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 1)
            On entry, n2 = *<value>*.

            Constraint: n2 >= 1.

        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, `order` has an illegal value: `order` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import realmat_rank_columns as b_realmat_rank_columns

    fun_name = 'naginterfaces.library.sort.realmat_rank_columns'

    _rm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rm'},
        rm,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m2_py = _b_utils._EngineFloat64ArrayType.get_shape(_rm, {'fun_name': fun_name, 'entity': 'rm'}, exp_rank=2)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _n2_py = _b_utils._EngineFloat64ArrayType.get_shape(_rm, {'fun_name': fun_name, 'entity': 'rm'}, exp_rank=2)[1]
    _n2 = _b_utils._EngineIntScalarType(
        _n2_py, {'fun_name': fun_name, 'entity': '_n2'},
    ).to_ctypes()
    _irank = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n2_py,),
    )
    b_realmat_rank_columns(
        _rm,
        m1,
        _m2,
        n1,
        _n2,
        order,
        _irank,
    )
    _irank_py = _irank
    return _irank_py

def intmat_rank_columns(im, m1, n1, order):
    u"""
    Rank columns of a matrix, integer numbers.

    For full information please refer to the NAG Library document for
    m01dk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01dkf.html

    Parameters
    ----------
    im : int, array-like, shape (m2, n2)
        Rows `m1` to m2 of columns `n1` to n2 of `im` must contain
        integer data to be ranked.

    m1 : int
        The index of the first row of `im` to be used.

    n1 : int
        The index of the first column of `im` to be ranked.

    order : str, length 1
        If `order` = 'A', the columns will be ranked in ascending order.

        If `order` = 'D', into descending order.

    Returns
    -------
    irank : int, ndarray, shape (n2)
        Elements `n1` to n2 of `irank` contain the ranks of the
        corresponding columns of `im`. Note that the ranks are in the
        range `n1` to n2: thus, if the ith column of `im` is the first
        in the rank order, `irank`[i-1] is set to `n1`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m2 = *<value>* and ldm = *<value>*.

            Constraint: m2 <= ldm.

        (`errno` 1)
            On entry, `n1` = *<value>* and n2 = *<value>*.

            Constraint: `n1` <= n2.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 1)
            On entry, n2 = *<value>*.

            Constraint: n2 >= 1.

        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, `order` has an illegal value: `order` = *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import intmat_rank_columns as b_intmat_rank_columns

    fun_name = 'naginterfaces.library.sort.intmat_rank_columns'

    _im = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'im'},
        im,
        copy=False,
    )
    _m2_py = _b_utils._EngineIntArrayType.get_shape(_im, {'fun_name': fun_name, 'entity': 'im'}, exp_rank=2)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _n2_py = _b_utils._EngineIntArrayType.get_shape(_im, {'fun_name': fun_name, 'entity': 'im'}, exp_rank=2)[1]
    _n2 = _b_utils._EngineIntScalarType(
        _n2_py, {'fun_name': fun_name, 'entity': '_n2'},
    ).to_ctypes()
    _irank = _b_utils._EngineIntArrayType.empty_ndarray(
        (_n2_py,),
    )
    b_intmat_rank_columns(
        _im,
        m1,
        _m2,
        n1,
        _n2,
        order,
        _irank,
    )
    _irank_py = _irank
    return _irank_py

def arbitrary_rank(compar, m1, m2, data=None):
    u"""
    Rank arbitrary data.

    For full information please refer to the NAG Library document for
    m01dz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01dzf.html

    Parameters
    ----------
    compar : callable retval = compar(i, j, data=None)
        `compar` must specify the relative ordering of any two data
        items; it must return True if item `i` must come strictly after
        item `j` in the rank ordering.

        Parameters
        ~~~~~~~~~~
        i : int
            `i` and `j` identify the data items to be compared.

        j : int
            `i` and `j` identify the data items to be compared.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : bool
            Must be True if item `i` must come strictly after item `j`
            in the rank ordering.

    m1 : int
        `m1` must specify the lower bound of the range of data items to
        be ranked, and the range of ranks to be assigned. Specifically,
        ``arbitrary_rank`` ranks the data items identified by integers
        in the range `m1` to `m2`, and assigns ranks in the range `m1`
        to `m2` which are stored in elements `m1` to `m2` of `irank`.

    m2 : int
        `m2` must specify the upper bound of the range of data items to
        be ranked, and the range of ranks to be assigned. Specifically,
        ``arbitrary_rank`` ranks the data items identified by integers
        in the range `m1` to `m2`, and assigns ranks in the range `m1`
        to `m2` which are stored in elements `m1` to `m2` of `irank`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    irank : int, ndarray, shape (`m2`)
        Elements `m1` to `m2` of `irank` contain the ranks of the data
        items `m1` to `m2`. Note that the ranks are in the range `m1` to
        `m2`: thus, if item i is first in the rank ordering,
        `irank`[i-1] contains `m1`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and `m2` = *<value>*.

            Constraint: `m1` <= `m2`.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, `m2` = *<value>*.

            Constraint: `m2` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import arbitrary_rank as b_arbitrary_rank

    def cb_compar(i, j, data=None):
        """Wrapping user's compar. Called as base compar."""
        if data is not None:
            _retval_compar = compar(
                i,
                j,
                data,
            )
        else:
            _retval_compar = compar(
                i,
                j,
            )
        return _retval_compar

    fun_name = 'naginterfaces.library.sort.arbitrary_rank'

    _b_utils._check_callable(compar, {'fun_name': fun_name, 'entity': 'compar'})
    _compar = cb_compar
    _m2 = _b_utils._EngineIntScalarType(
        m2, {'fun_name': fun_name, 'entity': 'm2'},
    ).to_ctypes()
    _m2_py = _b_utils._EngineIntScalarType.to_py(_m2)
    _irank = _b_utils._EngineIntArrayType.empty_ndarray(
        (_m2_py,),
    )
    b_arbitrary_rank(
        _compar,
        m1,
        _m2,
        _irank,
        data,
    )
    _irank_py = _irank
    return _irank_py

def realvec_rank_rearrange(rv, m1, irank):
    u"""
    Rearrange a vector according to given ranks, real numbers.

    For full information please refer to the NAG Library document for
    m01ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01eaf.html

    Parameters
    ----------
    rv : float, array-like, shape (m2)
        Elements `m1` to m2 of `rv` must contain real values to be
        rearranged.

    m1 : int
        `m1` and m2 must specify the range of the ranks supplied in
        `irank` and the elements of `rv` to be rearranged.

    irank : int, array-like, shape (m2)
        Elements `m1` to m2 of `irank` must contain a permutation of the
        integers `m1` to m2, which are interpreted as a vector of ranks.

    Returns
    -------
    rv : float, ndarray, shape (m2)
        These values are rearranged into rank order. For example, if
        `irank`[i-1] = `m1`, then the initial value of `rv`[i-1] is
        moved to `rv`[`m1`-1].

    irank : int, ndarray, shape (m2)
        Used as internal workspace prior to being restored and hence is
        unchanged.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            `irank`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `irank`[I-1] contains an out-of-range
            value: I = *<value>*, `irank`[I-1] = *<value>*.

        (`errno` 3)
            `irank`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `irank` contains a repeated value:
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import realvec_rank_rearrange as b_realvec_rank_rearrange

    fun_name = 'naginterfaces.library.sort.realvec_rank_rearrange'

    _rv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rv'},
        rv,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m2_py = _b_utils._EngineFloat64ArrayType.get_shape(_rv, {'fun_name': fun_name, 'entity': 'rv'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _irank = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irank'},
        irank,
        copy=True,
    )
    b_realvec_rank_rearrange(
        _rv,
        m1,
        _m2,
        _irank,
    )
    _rv_py = _rv
    _irank_py = _irank
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['rv', 'irank'],
    )(
        _rv_py,
        _irank_py,
    )
    return _return_data

def intvec_rank_rearrange(iv, m1, irank):
    u"""
    Rearrange a vector according to given ranks, integer numbers.

    For full information please refer to the NAG Library document for
    m01eb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01ebf.html

    Parameters
    ----------
    iv : int, array-like, shape (m2)
        Elements `m1` to m2 of `iv` must contain integer values to be
        rearranged.

    m1 : int
        `m1` and m2 specify the range of the ranks supplied in `irank`
        and the elements of `iv` to be rearranged.

    irank : int, array-like, shape (m2)
        Elements `m1` to m2 of `irank` must contain a permutation of the
        integers `m1` to m2, which are interpreted as a vector of ranks.

    Returns
    -------
    iv : int, ndarray, shape (m2)
        These values are rearranged into rank order. For example, if
        `irank`[i-1] = `m1`, then the initial value of `iv`[i-1] is
        moved to `iv`[`m1`-1].

    irank : int, ndarray, shape (m2)
        Used as internal workspace prior to being restored and hence is
        unchanged.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            `irank`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `irank`[I-1] contains an out-of-range
            value: I = *<value>*, `irank`[I-1] = *<value>*.

        (`errno` 3)
            `irank`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `irank` contains a repeated value:
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import intvec_rank_rearrange as b_intvec_rank_rearrange

    fun_name = 'naginterfaces.library.sort.intvec_rank_rearrange'

    _iv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iv'},
        iv,
        copy=True,
    )
    _m2_py = _b_utils._EngineIntArrayType.get_shape(_iv, {'fun_name': fun_name, 'entity': 'iv'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _irank = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irank'},
        irank,
        copy=True,
    )
    b_intvec_rank_rearrange(
        _iv,
        m1,
        _m2,
        _irank,
    )
    _iv_py = _iv
    _irank_py = _irank
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['iv', 'irank'],
    )(
        _iv_py,
        _irank_py,
    )
    return _return_data

def charvec_rank_rearrange(ch, m1, irank):
    u"""
    Rearrange a vector according to given ranks, character data.

    For full information please refer to the NAG Library document for
    m01ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01ecf.html

    Parameters
    ----------
    ch : str, array-like, shape (m2)
        Elements `m1` to m2 of `ch` must contain character data to be
        rearranged.

    m1 : int
        The range of the ranks supplied in `irank` and the elements of
        `ch` to be rearranged.

    irank : int, array-like, shape (m2)
        Elements `m1` to m2 of `irank` must contain a permutation of the
        integers `m1` to m2, which are interpreted as a vector of ranks.

    Returns
    -------
    ch : str, ndarray, shape (m2)
        These values are rearranged into rank order. For example, if
        `irank`[i-1] = `m1`, then the initial value of `ch`[i-1] is
        moved to `ch`[`m1`-1].

    irank : int, ndarray, shape (m2)
        Used as internal workspace prior to being restored and hence is
        unchanged.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            On entry, len(`ch`[0]) = *<value>*.

            Constraint: len(`ch`[0]) <= 255.

        (`errno` 3)
            `irank`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `irank`[I-1] contains an out-of-range
            value: I = *<value>*, `irank`[I-1] = *<value>*.

        (`errno` 4)
            `irank`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `irank` contains a repeated value:
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import charvec_rank_rearrange as b_charvec_rank_rearrange

    fun_name = 'naginterfaces.library.sort.charvec_rank_rearrange'

    _ch = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ch'},
        ch,
        copy=True,
    )
    _m2_py = _b_utils._EngineCharArrayType.get_shape(_ch, {'fun_name': fun_name, 'entity': 'ch'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _irank = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irank'},
        irank,
        copy=True,
    )
    b_charvec_rank_rearrange(
        _ch,
        m1,
        _m2,
        _irank,
    )
    _ch_py = _ch
    _irank_py = _irank
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ch', 'irank'],
    )(
        _ch_py,
        _irank_py,
    )
    return _return_data

def cmplxvec_rank_rearrange(cv, m1, irank):
    u"""
    Rearrange a vector according to given ranks, complex numbers.

    For full information please refer to the NAG Library document for
    m01ed

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01edf.html

    Parameters
    ----------
    cv : complex, array-like, shape (m2)
        Elements `m1` to m2 of `cv` must contain complex values to be
        rearranged.

    m1 : int
        `m1` and m2 must specify the range of the ranks supplied in
        `irank` and the elements of `cv` to be rearranged.

    irank : int, array-like, shape (m2)
        Elements `m1` to m2 of `irank` must contain a permutation of the
        integers `m1` to m2, which are interpreted as a vector of ranks.

    Returns
    -------
    cv : complex, ndarray, shape (m2)
        These values are rearranged into rank order. For example, if
        `irank`[i-1] = `m1`, then the initial value of `cv`[i-1] is
        moved to `cv`[`m1`-1].

    irank : int, ndarray, shape (m2)
        Used as internal workspace prior to being restored and hence is
        unchanged.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            `irank`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `irank`[I-1] contains an out-of-range
            value: I = *<value>*, `irank`[I-1] = *<value>*.

        (`errno` 3)
            `irank`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `irank` contains a repeated value:
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import cmplxvec_rank_rearrange as b_cmplxvec_rank_rearrange

    fun_name = 'naginterfaces.library.sort.cmplxvec_rank_rearrange'

    _cv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'cv'},
        cv,
        copy=True,
    )
    _m2_py = _b_utils._EngineComplex128ArrayType.get_shape(_cv, {'fun_name': fun_name, 'entity': 'cv'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _irank = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'irank'},
        irank,
        copy=True,
    )
    b_cmplxvec_rank_rearrange(
        _cv,
        m1,
        _m2,
        _irank,
    )
    _cv_py = _cv
    _irank_py = _irank
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['cv', 'irank'],
    )(
        _cv_py,
        _irank_py,
    )
    return _return_data

def realvec_search(valid, rv, item, m1=1):
    u"""
    Binary search in set of real numbers.

    For full information please refer to the NAG Library document for
    m01na

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01naf.html

    Parameters
    ----------
    valid : bool
        If `valid` is set to True argument checking will be performed.
        If `valid` is set to False ``realvec_search`` will be called
        without argument checking (which includes checking that array
        `rv` is sorted in ascending order) and the function will return
        with no exception or warning is raised. See [Further Comments]
        for further details.

    rv : float, array-like, shape (m2)
        Elements `m1` to m2 contain double values to be searched.

    item : float
        The sought-after item.

    m1 : int, optional
        Note: this argument represents an array index; the value you
        supply must be base-1 for compatibility with the NAG Engine.

        The index of the first element of `rv` to be searched.

    Returns
    -------
    irv : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        The index of the first value equal to the sought-after item.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `rv` must be sorted in ascending order: `rv`
            element *<value>* > element *<value>*.

        (`errno` 3)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 4)
            On entry, `m1` = *<value>*, m2 = *<value>*.

            Constraint: `m1` <= m2.
    """

    from ..base.sort import realvec_search as b_realvec_search

    fun_name = 'naginterfaces.library.sort.realvec_search'

    _rv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rv'},
        rv,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m2_py = _b_utils._EngineFloat64ArrayType.get_shape(_rv, {'fun_name': fun_name, 'entity': 'rv'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _return_scalar = b_realvec_search(
        valid,
        _rv,
        m1,
        _m2,
        item,
    )
    return _return_scalar

def intvec_search(valid, iv, item, m1=1):
    u"""
    Binary search in set of integer numbers.

    For full information please refer to the NAG Library document for
    m01nb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01nbf.html

    Parameters
    ----------
    valid : bool
        If `valid` is set to True argument checking will be performed.
        If `valid` is set to False ``intvec_search`` will be called
        without argument checking (which includes checking that array
        `iv` is sorted in ascending order) and the function will return
        with no exception or warning is raised. See [Further Comments]
        for further details.

    iv : int, array-like, shape (m2)
        Elements `m1` to m2 contain integer values to be searched.

    item : int
        The sought-after item.

    m1 : int, optional
        Note: this argument represents an array index; the value you
        supply must be base-1 for compatibility with the NAG Engine.

        The index of the first element of `iv` to be searched.

    Returns
    -------
    iiv : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        The index of the first value equal to the sought-after item.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `iv` must be sorted in ascending order: `iv`
            element *<value>* > element *<value>*.

        (`errno` 3)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 4)
            On entry, `m1` = *<value>*, m2 = *<value>*.

            Constraint: `m1` <= m2.
    """

    from ..base.sort import intvec_search as b_intvec_search

    fun_name = 'naginterfaces.library.sort.intvec_search'

    _iv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iv'},
        iv,
        copy=False,
    )
    _m2_py = _b_utils._EngineIntArrayType.get_shape(_iv, {'fun_name': fun_name, 'entity': 'iv'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _return_scalar = b_intvec_search(
        valid,
        _iv,
        m1,
        _m2,
        item,
    )
    return _return_scalar

def charvec_search(valid, ch, item, m1=1):
    u"""
    Binary search in set of character data.

    For full information please refer to the NAG Library document for
    m01nc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01ncf.html

    Parameters
    ----------
    valid : bool
        If `valid` is set to True argument checking will be performed.
        If `valid` is set to False ``charvec_search`` will be called
        without argument checking, which includes checking that array
        `ch` is sorted in ascending order and the function will return
        with no exception or warning is raised. See [Further Comments]
        for further details.

    ch : str, array-like, shape (m2)
        Elements `m1` to m2 contain character data to be searched.

    item : str
        The sought-after item. Trailing space characters are ignored.

    m1 : int, optional
        Note: this argument represents an array index; the value you
        supply must be base-1 for compatibility with the NAG Engine.

        The index of the first element of `ch` to be searched.

    Returns
    -------
    ich : int
        Note: this argument represents an array index; the value
        returned will be base-1.

        The index of the first value equal to the sought-after item.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `ch` must be sorted in ascending order: `ch`
            element *<value>* > element *<value>*.

        (`errno` 3)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 4)
            On entry, `m1` = *<value>*, m2 = *<value>*.

            Constraint: m2 >= `m1`.

        (`errno` 5)
            On entry, the length of each element of `ch` must be at most
            255: maximum string length = *<value>*.
    """

    from ..base.sort import charvec_search as b_charvec_search

    fun_name = 'naginterfaces.library.sort.charvec_search'

    _ch = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'ch'},
        ch,
        copy=False,
    )
    _m2_py = _b_utils._EngineCharArrayType.get_shape(_ch, {'fun_name': fun_name, 'entity': 'ch'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _return_scalar = b_charvec_search(
        valid,
        _ch,
        m1,
        _m2,
        item,
    )
    return _return_scalar

def realvec_vec_search(valid, rv, m1, m2, item, comm=None):
    u"""
    Searches an ordered set of real numbers using an O(1) method.

    For full information please refer to the NAG Library document for
    m01nd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01ndf.html

    Parameters
    ----------
    valid : bool
        If `valid` is set to True argument checking will be performed.
        If `valid` is set to False ``realvec_vec_search`` will be called
        without argument checking (which includes checking that array
        `rv` is sorted in strictly ascending order). See [Further
        Comments] for further details.

    rv : float, array-like, shape (n)
        X, the double values to be searched.

    m1 : int
        Note: this argument represents an array index; the value you
        supply must be base-1 for compatibility with the NAG Engine.

        The index of the first element of `rv` to be searched.

    m2 : int
        Note: this argument represents an array index; the value you
        supply must be base-1 for compatibility with the NAG Engine.

        The index of the last element of `rv` to be searched.

    item : float, array-like, shape (m)
        Z, the sought-after items.

    comm : None or dict, communication object, optional, modified in
    place
        Communication structure.

        If `comm` is None then searching is performed using an
        offset-based binary search.

        Otherwise, `comm` may be supplied uninitialized.

        It will then be populated and the routine will perform a direct
        search of `rv`.

        The populated `comm` can then be supplied again to subsequent
        calls for repeated searching of `rv`.

    Returns
    -------
    idx : int, ndarray, shape (m)
        Note: this argument represents array indices; the values
        returned will be base-1.

        `idx`[i-1] contains the index of the largest entry in `rv` which
        is smaller than or equal to `item`[i-1].

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `rv` must be sorted in strictly ascending order:
            `rv` element *<value>* > element *<value>*.

        (`errno` 3)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 0.

        (`errno` 4)
            On entry, `m1` = *<value>*, `m2` = *<value>*, n = *<value>*.

            Constraint: `m1` <= `m2` <= n.

        (`errno` 5)
            On entry, n = *<value>*.

            Constraint: n >= 2.

        (`errno` 6)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 7)
            On entry, the values in `rv` are such that the required
            value of lk would overflow the current platform's largest
            positive integer value.

        (`errno` 8)
            `comm` may have become corrupted.
    """

    from ..base.sort import realvec_vec_search as b_realvec_vec_search

    fun_name = 'naginterfaces.library.sort.realvec_vec_search'

    _rv = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'rv'},
        rv,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_rv, {'fun_name': fun_name, 'entity': 'rv'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _item = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'item'},
        item,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_item, {'fun_name': fun_name, 'entity': 'item'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _idx = _b_utils._EngineIntArrayType.empty_ndarray(
        (_m_py,),
    )
    b_realvec_vec_search(
        valid,
        _rv,
        _n,
        m1,
        m2,
        _item,
        _m,
        _idx,
        comm,
    )
    _idx_py = _idx
    return _idx_py

def permute_invert(iperm, m1):
    u"""
    Invert a permutation.

    For full information please refer to the NAG Library document for
    m01za

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01zaf.html

    Parameters
    ----------
    iperm : int, array-like, shape (m2)
        Elements `m1` to m2 of `iperm` must contain a permutation of the
        integers `m1` to m2.

    m1 : int
        `m1` and m2 must specify the range of elements used in the array
        `iperm` and the range of values in the permutation, as specified
        under `iperm`.

    Returns
    -------
    iperm : int, ndarray, shape (m2)
        These elements contain the inverse permutation of the integers
        `m1` to m2.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            `iperm`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `iperm`[I-1] contains an out-of-range
            value: I = *<value>*, `iperm`[I-1] = *<value>*.

        (`errno` 3)
            `iperm`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `iperm` contains a repeated value:
            *<value>*.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.sort import permute_invert as b_permute_invert

    fun_name = 'naginterfaces.library.sort.permute_invert'

    _iperm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iperm'},
        iperm,
        copy=True,
    )
    _m2_py = _b_utils._EngineIntArrayType.get_shape(_iperm, {'fun_name': fun_name, 'entity': 'iperm'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    b_permute_invert(
        _iperm,
        m1,
        _m2,
    )
    _iperm_py = _iperm
    return _iperm_py

def permute_check(iperm, m1):
    u"""
    Check validity of a permutation.

    For full information please refer to the NAG Library document for
    m01zb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01zbf.html

    Parameters
    ----------
    iperm : int, array-like, shape (m2)
        Elements `m1` to m2 of `iperm` must be set to values which are
        supposed to be a permutation of the integers `m1` to m2. If they
        are a valid permutation, the function exits with no exception or
        warning is raised.

    m1 : int
        The range of elements used in the array `iperm` and the range of
        values in the permutation, as specified under `iperm`.

    Returns
    -------
    iperm : int, ndarray, shape (m2)
        Used as internal workpsace prior to being restored and hence is
        unchanged.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            `iperm`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `iperm`[I-1] contains an out-of-range
            value: I = *<value>*, `iperm`[I-1] = *<value>*.

        (`errno` 3)
            `iperm`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `iperm` contains a repeated value:
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import permute_check as b_permute_check

    fun_name = 'naginterfaces.library.sort.permute_check'

    _iperm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iperm'},
        iperm,
        copy=True,
    )
    _m2_py = _b_utils._EngineIntArrayType.get_shape(_iperm, {'fun_name': fun_name, 'entity': 'iperm'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    b_permute_check(
        _iperm,
        m1,
        _m2,
    )
    _iperm_py = _iperm
    return _iperm_py

def permute_decompose(iperm, m1):
    u"""
    Decompose a permutation into cycles.

    For full information please refer to the NAG Library document for
    m01zc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/m01/m01zcf.html

    Parameters
    ----------
    iperm : int, array-like, shape (m2)
        Elements `m1` to m2 of `iperm` must contain a permutation of the
        integers `m1` to m2.

    m1 : int
        `m1` and m2 must specify the range of elements used in the array
        `iperm` and the range of values in the permutation, as specified
        under `iperm`.

    Returns
    -------
    iperm : int, ndarray, shape (m2)
        Is used as internal workpsace prior to being restored and hence
        is unchanged.

    icycl : int, ndarray, shape (m2)
        Elements `m1` to m2 of `icycl` contain a representation of the
        permutation as a list of cycles, with the first integer in each
        cycle negated. (See [Description].)

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m1` = *<value>* and m2 = *<value>*.

            Constraint: `m1` <= m2.

        (`errno` 1)
            On entry, `m1` = *<value>*.

            Constraint: `m1` >= 1.

        (`errno` 1)
            On entry, m2 = *<value>*.

            Constraint: m2 >= 1.

        (`errno` 2)
            `iperm`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `iperm`[I-1] contains an out-of-range
            value: I = *<value>*, `iperm`[I-1] = *<value>*.

        (`errno` 3)
            `iperm`[`m1`:m2-1] does not contain a permutation of the
            integers `m1` to m2. `iperm` contains a repeated value:
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sort import permute_decompose as b_permute_decompose

    fun_name = 'naginterfaces.library.sort.permute_decompose'

    _iperm = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'iperm'},
        iperm,
        copy=True,
    )
    _m2_py = _b_utils._EngineIntArrayType.get_shape(_iperm, {'fun_name': fun_name, 'entity': 'iperm'}, exp_rank=1)[0]
    _m2 = _b_utils._EngineIntScalarType(
        _m2_py, {'fun_name': fun_name, 'entity': '_m2'},
    ).to_ctypes()
    _icycl = _b_utils._EngineIntArrayType.empty_ndarray(
        (_m2_py,),
    )
    b_permute_decompose(
        _iperm,
        m1,
        _m2,
        _icycl,
    )
    _iperm_py = _iperm
    _icycl_py = _icycl
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['iperm', 'icycl'],
    )(
        _iperm_py,
        _icycl_py,
    )
    return _return_data
