# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unused-import
u"""
Interfaces for the NAG Mark 27.0 `sum` Chapter.

``sum`` - Summation of Series

This module is concerned with the following tasks:

- calculating the discrete Fourier transform of a sequence of real or
  complex data values;

- calculating the discrete convolution or the discrete correlation of
  two sequences of real or complex data values using discrete Fourier
  transforms;

- calculating the inverse Laplace transform of a user-supplied function;

- calculating the fast Gauss transform approximation to the discrete
  Gauss transform;

- direct summation of orthogonal series;

- acceleration of convergence of a sequence of real values.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06intro.html

See Also
--------
``naginterfaces.library.examples.sum`` :
    This subpackage contains examples for the ``sum`` module.
    See also the :ref:`library_sum_ex` subsection.
"""

# NAG Copyright 2017-2019.

import collections as _collections

from ..base import utils as _b_utils
from ..library import _utils

def accelerate(seqn, ncall, comm, nterms):
    u"""
    Acceleration of convergence of sequence, Shanks' transformation and
    epsilon algorithm.

    For full information please refer to the NAG Library document for
    c06ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06baf.html

    Parameters
    ----------
    seqn : float
        The next term of the sequence to be considered.

    ncall : int
        On the first call `ncall` must be set to 0. Thereafter `ncall`
        must not be changed between calls.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    nterms : int
        The maximum number of terms in the sequence.

    Returns
    -------
    ncall : int
        The number of terms in the sequence that have been considered.

    result : float
        The estimate of the limit of the sequence. For the first two
        calls, `result` = `seqn`.

    abserr : float
        An estimate of the absolute error in `result`. For the first
        three calls, `abserr` is set to a large machine-dependent
        constant.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ncall` = *<value>*.

            Constraint: `ncall` >= 0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import accelerate as b_accelerate

    _return_scalars = b_accelerate(
        seqn,
        ncall,
        comm,
        nterms,
    )
    _ncall_py = _return_scalars.ncall
    _result_py = _return_scalars.result
    _abserr_py = _return_scalars.abserr
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['ncall', 'result', 'abserr'],
    )(
        _ncall_py,
        _result_py,
        _abserr_py,
    )
    return _return_data

def chebyshev(x, xmin, xmax, c, s):
    u"""
    Sum of a Chebyshev series at a set of points.

    For full information please refer to the NAG Library document for
    c06dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06dcf.html

    Parameters
    ----------
    x : float, array-like, shape (lx)
        x in X, the set of arguments of the series.

    xmin : float
        The lower end point of the interval [x_min,x_max].

    xmax : float
        The upper end point of the interval [x_min,x_max].

    c : float, array-like, shape (n)
        `c`[j-1] must contain the coefficient c_j of the Chebyshev
        series, for j = 1,2,...,n.

    s : int
        Determines the series (see [Description]).

        `s` = 1

          The series is general.

        `s` = 2

          The series is even.

        `s` = 3

          The series is odd.

    Returns
    -------
    res : float, ndarray, shape (lx)
        The Chebyshev series evaluated at the set of points X.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, lx = *<value>*.

            Constraint: lx >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, `s` = *<value>*.

            Constraint: `s` = 1, 2 or 3.

        (`errno` 4)
            On entry, `xmax` = *<value>* and `xmin` = *<value>*.

            Constraint: `xmin` < `xmax`.

        (`errno` 5)
            On entry, element `x`[*<value>*] = *<value>*, `xmin` =
            *<value>* and `xmax` = *<value>*.

            Constraint: `xmin` <= `x`[i] <= `xmax`, for all i.
    """

    from ..base.sum import chebyshev as b_chebyshev

    fun_name = 'naginterfaces.library.sum.chebyshev'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _lx_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _lx = _b_utils._EngineIntScalarType(
        _lx_py, {'fun_name': fun_name, 'entity': '_lx'},
    ).to_ctypes()
    _c = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'c'},
        c,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _res = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_lx_py,),
    )
    b_chebyshev(
        _x,
        _lx,
        xmin,
        xmax,
        _c,
        _n,
        s,
        _res,
    )
    _res_py = _res
    return _res_py

def fft_real_1d_rfmt(x):
    u"""
    Single one-dimensional real discrete Fourier transform, extra
    workspace for greater speed.

    For full information please refer to the NAG Library document for
    c06fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06faf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        If `x` is declared with bounds (0:n-1) in the function from
        which ``fft_real_1d_rfmt`` is called, `x`[j] must contain x_j,
        for j = 0,1,...,n-1.

    Returns
    -------
    x : float, ndarray, shape (n)
        The discrete Fourier transform stored in Hermitian form. If the
        components of the transform z^_k are written as a_k+ib_k, and if
        `x` is declared with bounds (0:n-1) in the function from which
        ``fft_real_1d_rfmt`` is called, then for 0 <= k <= n/2, a_k is
        contained in `x`[k-1], and for 1 <= k <= (n-1)/2, b_k is
        contained in `x`[n-k]. (See also [the C06 Introduction] and
        [Example].)

    Raises
    ------
    NagValueError
        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n > 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_real_1d_rfmt as b_fft_real_1d_rfmt

    fun_name = 'naginterfaces.library.sum.fft_real_1d_rfmt'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_fft_real_1d_rfmt(
        _x,
        _n,
    )
    _x_py = _x
    return _x_py

def fft_hermitian_1d_rfmt(x):
    u"""
    Single one-dimensional Hermitian discrete Fourier transform, extra
    workspace for greater speed.

    For full information please refer to the NAG Library document for
    c06fb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06fbf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        The sequence to be transformed stored in Hermitian form. If the
        data values z_j are written as x_j+iy_j, and if `x` is declared
        with bounds (0:n-1) in the function from which
        ``fft_hermitian_1d_rfmt`` is called, then for 0 <= j <= n/2, x_j
        is contained in `x`[j-1], and for 1 <= j <= (n-1)/2, y_j is
        contained in `x`[n-j]. (See also [the C06 Introduction] and
        [Example].)

    Returns
    -------
    x : float, ndarray, shape (n)
        The components of the discrete Fourier transform x^_k. If `x` is
        declared with bounds (0:n-1) in the function from which
        ``fft_hermitian_1d_rfmt`` is called, x^_k is stored in `x`[k],
        for k = 0,1,...,n-1.

    Raises
    ------
    NagValueError
        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n > 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_hermitian_1d_rfmt as b_fft_hermitian_1d_rfmt

    fun_name = 'naginterfaces.library.sum.fft_hermitian_1d_rfmt'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_fft_hermitian_1d_rfmt(
        _x,
        _n,
    )
    _x_py = _x
    return _x_py

def fft_complex_1d_sep(x, y):
    u"""
    Single one-dimensional complex discrete Fourier transform, extra
    workspace for greater speed.

    For full information please refer to the NAG Library document for
    c06fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06fcf.html

    Parameters
    ----------
    x : float, array-like, shape (n)
        If `x` is declared with bounds (0:n-1) in the function from
        which ``fft_complex_1d_sep`` is called, `x`[j-1] must contain
        x_j, the real part of z_j, for j = 0,1,...,n-1.

    y : float, array-like, shape (n)
        If `y` is declared with bounds (0:n-1) in the function from
        which ``fft_complex_1d_sep`` is called, `y`[j-1] must contain
        y_j, the imaginary part of z_j, for j = 0,1,...,n-1.

    Returns
    -------
    x : float, ndarray, shape (n)
        The real parts a_k of the components of the discrete Fourier
        transform. If `x` is declared with bounds (0:n-1) in the
        function from which ``fft_complex_1d_sep`` is called, for 0 <= k
        <= n-1, a_k is contained in `x`[k-1].

    y : float, ndarray, shape (n)
        The imaginary parts b_k of the components of the discrete
        Fourier transform. If `y` is declared with bounds (0:n-1) in the
        function from which ``fft_complex_1d_sep`` is called, then for 0
        <= k <= n-1, b_k is contained in `y`[k-1].

    Raises
    ------
    NagValueError
        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n > 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_complex_1d_sep as b_fft_complex_1d_sep

    fun_name = 'naginterfaces.library.sum.fft_complex_1d_sep'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_fft_complex_1d_sep(
        _x,
        _y,
        _n,
    )
    _x_py = _x
    _y_py = _y
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'y'],
    )(
        _x_py,
        _y_py,
    )
    return _return_data

def fft_complex_multid_1d_sep(l, nd, x, y):
    u"""
    One-dimensional complex discrete Fourier transform of
    multidimensional data.

    For full information please refer to the NAG Library document for
    c06ff

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06fff.html

    Parameters
    ----------
    l : int
        l, the index of the variable (or dimension) on which the
        discrete Fourier transform is to be performed.

    nd : int, array-like, shape (ndim)
        `nd`[i-1] must contain n_i (the dimension of the ith variable),
        for i = 1,2,...,m.

    x : float, array-like, shape (n)
        `x`[ 1 + j_1 + n_1 j_2 + n_1 n_2 j_3 + ... -1] must contain the
        real part of the complex data value z_{j_1 j_2...j_m}, for 0 <=
        j_1 <= n_1-1,0 <= j_2 <= n_2-1,...; i.e., the values are stored
        in consecutive elements of the array according to the Fortran
        convention for storing multidimensional arrays.

    y : float, array-like, shape (n)
        The imaginary parts of the complex data values, stored in the
        same way as the real parts in the array `x`.

    Returns
    -------
    x : float, ndarray, shape (n)
        The real parts of the corresponding elements of the computed
        transform.

    y : float, ndarray, shape (n)
        The imaginary parts of the corresponding elements of the
        computed transform.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n = `nd`[0]*`nd`[1]*...*`nd`[ndim-1].

        (`errno` 3)
            On entry, `l` = *<value>* and ndim = *<value>*.

            Constraint: 1 <= `l` <= ndim.

        (`errno` 10*`l`+3)
            On entry, l = *<value>*.

            Constraint: `nd`[l-1] >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_complex_multid_1d_sep as b_fft_complex_multid_1d_sep

    fun_name = 'naginterfaces.library.sum.fft_complex_multid_1d_sep'

    _nd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'nd'},
        nd,
        copy=False,
    )
    _ndim_py = _b_utils._EngineIntArrayType.get_shape(_nd, {'fun_name': fun_name, 'entity': 'nd'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_fft_complex_multid_1d_sep(
        _ndim,
        l,
        _nd,
        _n,
        _x,
        _y,
    )
    _x_py = _x
    _y_py = _y
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'y'],
    )(
        _x_py,
        _y_py,
    )
    return _return_data

def fft_complex_multid_sep(nd, x, y):
    u"""
    Multidimensional complex discrete Fourier transform of
    multidimensional data.

    For full information please refer to the NAG Library document for
    c06fj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06fjf.html

    Parameters
    ----------
    nd : int, array-like, shape (ndim)
        `nd`[i-1] must contain n_i (the dimension of the ith variable),
        for i = 1,2,...,m.

    x : float, array-like, shape (n)
        `x`[ 1 + j_1 + n_1 j_2 + n_1 n_2 j_3 + ... -1] must contain the
        real part of the complex data value z_{j_1 j_2...j_m}, for 0 <=
        j_1 <= n_1-1,0 <= j_2 <= n_2-1,...; i.e., the values are stored
        in consecutive elements of the array according to the Fortran
        convention for storing multidimensional arrays.

    y : float, array-like, shape (n)
        The imaginary parts of the complex data values, stored in the
        same way as the real parts in the array `x`.

    Returns
    -------
    x : float, ndarray, shape (n)
        The real parts of the corresponding elements of the computed
        transform.

    y : float, ndarray, shape (n)
        The imaginary parts of the corresponding elements of the
        computed transform.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n = `nd`[0]*`nd`[1]*...*`nd`[ndim-1].

        (`errno` i mod 10 = 3)
            On entry, l = *<value>*.

            Constraint: `nd`[l-1] >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_complex_multid_sep as b_fft_complex_multid_sep

    fun_name = 'naginterfaces.library.sum.fft_complex_multid_sep'

    _nd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'nd'},
        nd,
        copy=False,
    )
    _ndim_py = _b_utils._EngineIntArrayType.get_shape(_nd, {'fun_name': fun_name, 'entity': 'nd'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_fft_complex_multid_sep(
        _ndim,
        _nd,
        _n,
        _x,
        _y,
    )
    _x_py = _x
    _y_py = _y
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'y'],
    )(
        _x_py,
        _y_py,
    )
    return _return_data

def convcorr_real(job, x, y):
    u"""
    Circular convolution or correlation of two real vectors, no
    restrictions on n.

    For full information please refer to the NAG Library document for
    c06fk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06fkf.html

    Parameters
    ----------
    job : int
        The computation to be performed.

        `job` = 1

          z_k = sum_j = 0^n-1x_j y_{k-j} (convolution);

        `job` = 2

          w_k = sum_j = 0^n-1x_j y_{k+j} (correlation).

    x : float, array-like, shape (n)
        The elements of one period of the vector x. If `x` is declared
        with bounds (0:n-1) in the function from which ``convcorr_real``
        is called, `x`[j] must contain x_j, for j = 0,1,...,n-1.

    y : float, array-like, shape (n)
        The elements of one period of the vector y. If `y` is declared
        with bounds (0:n-1) in the function from which ``convcorr_real``
        is called, `y`[j] must contain y_j, for j = 0,1,...,n-1.

    Returns
    -------
    x : float, ndarray, shape (n)
        The corresponding elements of the discrete convolution or
        correlation.

    y : float, ndarray, shape (n)
        The discrete Fourier transform of the convolution or correlation
        returned in the array `x`; the transform is stored in Hermitian
        form; if the components of the transform z_k are written as
        a_k+ib_k, then for 0 <= k <= n/2, a_k is contained in `y`[k],
        and for 1 <= k <= n/2-1, b_k is contained in `y`[n-k]. (See also
        [the C06 Introduction].)

    Raises
    ------
    NagValueError
        (`errno` 3)
            On entry, n = *<value>*.

            Constraint: n > 1.

        (`errno` 4)
            On entry, `job` = *<value>*.

            Constraint: `job` = 1 or 2.

    See Also
    --------
    :meth:`naginterfaces.library.examples.sum.convcorr_real_ex.main`
    """

    from ..base.sum import convcorr_real as b_convcorr_real

    fun_name = 'naginterfaces.library.sum.convcorr_real'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_convcorr_real(
        job,
        _x,
        _y,
        _n,
    )
    _x_py = _x
    _y_py = _y
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'y'],
    )(
        _x_py,
        _y_py,
    )
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.sum.withdraw_fft_real_1d_multi_rfmt',
    mkwdrn='28',
    advice=[
        'Please use fft_realherm_1d_multi_row or fft_realherm_1d_multi_col instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def withdraw_fft_real_1d_multi_rfmt(m, n, x, init, trig):
    u"""
    Multiple one-dimensional real discrete Fourier transforms.

    Note: ``withdraw_fft_real_1d_multi_rfmt`` will be removed in
    naginterfaces 28.0.0.
    Please use ``fft_realherm_1d_multi_row`` or
    ``fft_realherm_1d_multi_col`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    c06fp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06fpf.html

    Parameters
    ----------
    m : int
        m, the number of sequences to be transformed.

    n : int
        n, the number of real values in each sequence.

    x : float, array-like, shape (`m`*`n`)
        The data must be stored in `x` as if in a two-dimensional array
        of dimension (1:`m`,0:`n`-1); each of the m sequences is stored
        in a row of the array. In other words, if the data values of the
        pth sequence to be transformed are denoted by x_j^p, for j =
        0,1,...,n-1, the mn elements of the array `x` must contain the
        values

x_0^1,x_0^2,...,x_0^m,x_1^1,x_1^2,...,x_1^m,...,x_n-1^1,x_n-1^2,...,x_n-1^m.

    init : str, length 1
        Indicates whether trigonometric coefficients are to be
        calculated.

        `init` = 'I'

          Calculate the required trigonometric coefficients for the
          given value of n, and store in the array `trig`.

        `init` = 'S' or 'R'

          The required trigonometric coefficients are assumed to have
          been calculated and stored in the array `trig` in a prior call
          to one of ``withdraw_fft_real_1d_multi_rfmt`` or
          ``withdraw_fft_hermitian_1d_multi_rfmt``. The function
          performs a simple check that the current value of n is
          consistent with the values stored in `trig`.

    trig : float, array-like, shape (2*`n`)
        If `init` = 'S' or 'R', `trig` must contain the required
        trigonometric coefficients that have been previously calculated.
        Otherwise `trig` need not be set.

    Returns
    -------
    x : float, ndarray, shape (`m`*`n`)
        The m discrete Fourier transforms stored as if in a
        two-dimensional array of dimension (1:`m`,0:`n`-1). Each of the
        m transforms is stored in a row of the array in Hermitian form,
        overwriting the corresponding original sequence. If the n
        components of the discrete Fourier transform z^_k^p are written
        as a_k^p+ib_k^p, then for 0 <= k <= n/2, a_k^p is contained in
        `x`[p-1,k-1], and for 1 <= k <= (n-1)/2, b_k^p is contained in
        `x`[p-1,n-k-1]. (See also [the C06 Introduction].)

    trig : float, ndarray, shape (2*`n`)
        Contains the required coefficients (computed by the function if
        `init` = 'I').

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `init` = *<value>*.

            Constraint: `init` = 'I', 'S' or 'R'.

        (`errno` 5)
            On entry, `init` = *<value>* but `n` and `trig` array
            incompatible.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.sum import withdraw_fft_real_1d_multi_rfmt as b_withdraw_fft_real_1d_multi_rfmt

    fun_name = 'naginterfaces.library.sum.withdraw_fft_real_1d_multi_rfmt'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _trig = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'trig'},
        trig,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_withdraw_fft_real_1d_multi_rfmt(
        m,
        n,
        _x,
        init,
        _trig,
    )
    _x_py = _x
    _trig_py = _trig
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'trig'],
    )(
        _x_py,
        _trig_py,
    )
    return _return_data

@_b_utils._withdrawal(
    'naginterfaces.library.sum.withdraw_fft_hermitian_1d_multi_rfmt',
    mkwdrn='28',
    advice=[
        'Please use fft_realherm_1d_multi_row or fft_realherm_1d_multi_col instead.',
        'See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html',
    ]
)
def withdraw_fft_hermitian_1d_multi_rfmt(m, n, x, init, trig):
    u"""
    Multiple one-dimensional Hermitian discrete Fourier transforms.

    Note: ``withdraw_fft_hermitian_1d_multi_rfmt`` will be removed in
    naginterfaces 28.0.0.
    Please use ``fft_realherm_1d_multi_row`` or
    ``fft_realherm_1d_multi_col`` instead.
    See also https://www.nag.com/numeric/py/nagdoc_latest/replace.html

    For full information please refer to the NAG Library document for
    c06fq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06fqf.html

    Parameters
    ----------
    m : int
        m, the number of sequences to be transformed.

    n : int
        n, the number of data values in each sequence.

    x : float, array-like, shape (`m`*`n`)
        The data must be stored in `x` as if in a two-dimensional array
        of dimension (1:`m`,0:`n`-1); each of the m sequences is stored
        in a row of the array in Hermitian form. If the n data values
        z_j^p are written as x_j^p+iy_j^p, then for 0 <= j <= n/2, x_j^p
        is contained in `x`[p-1,j], and for 1 <= j <= (n-1)/2, y_j^p is
        contained in `x`[p,n-j]. (See also [the C06 Introduction].)

    init : str, length 1
        Indicates whether trigonometric coefficients are to be
        calculated.

        `init` = 'I'

          Calculate the required trigonometric coefficients for the
          given value of n, and store in the array `trig`.

        `init` = 'S' or 'R'

          The required trigonometric coefficients are assumed to have
          been calculated and stored in the array `trig` in a prior call
          to one of ``withdraw_fft_real_1d_multi_rfmt`` or
          ``withdraw_fft_hermitian_1d_multi_rfmt``. The function
          performs a simple check that the current value of n is
          consistent with the values stored in `trig`.

    trig : float, array-like, shape (2*`n`)
        If `init` = 'S' or 'R', `trig` must contain the required
        trigonometric coefficients that have been previously calculated.
        Otherwise `trig` need not be set.

    Returns
    -------
    x : float, ndarray, shape (`m`*`n`)
        The components of the m discrete Fourier transforms, stored as
        if in a two-dimensional array of dimension (1:`m`,0:`n`-1). Each
        of the m transforms is stored as a row of the array, overwriting
        the corresponding original sequence. If the n components of the
        discrete Fourier transform are denoted by x^_k^p, for k =
        0,1,...,n-1, the mn elements of the array `x` contain the values

x^_0^1,x^_0^2,...,x^_0^m,x^_1^1,x^_1^2,...,x^_1^m,...,x^_n-1^1,x^_n-1^2,...,x^_n-1^m.

    trig : float, ndarray, shape (2*`n`)
        Contains the required coefficients (computed by the function if
        `init` = 'I').

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `init` = *<value>*.

            Constraint: `init` = 'I', 'S' or 'R'.

        (`errno` 5)
            On entry, `init` = *<value>* but `n` and `trig` array
            incompatible.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    from ..base.sum import withdraw_fft_hermitian_1d_multi_rfmt as b_withdraw_fft_hermitian_1d_multi_rfmt

    fun_name = 'naginterfaces.library.sum.withdraw_fft_hermitian_1d_multi_rfmt'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _trig = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'trig'},
        trig,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_withdraw_fft_hermitian_1d_multi_rfmt(
        m,
        n,
        _x,
        init,
        _trig,
    )
    _x_py = _x
    _trig_py = _trig
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'trig'],
    )(
        _x_py,
        _trig_py,
    )
    return _return_data

def fft_complex_3d_sep(n1, n2, n3, x, y):
    u"""
    Three-dimensional complex discrete Fourier transform.

    For full information please refer to the NAG Library document for
    c06fx

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06fxf.html

    Parameters
    ----------
    n1 : int
        n_1, the first dimension of the transform.

    n2 : int
        n_2, the second dimension of the transform.

    n3 : int
        n_3, the third dimension of the transform.

    x : float, array-like, shape (`n1`*`n2`*`n3`)
        The real and imaginary parts of the complex data values must be
        stored in arrays `x` and `y` respectively. If `x` and `y` are
        regarded as three-dimensional arrays of dimension
        (0:`n1`-1,0:`n2`-1,0:`n3`-1), `x`[j_1-1,j_2-1,j_3-1] and
        `y`[j_1-1,j_2-1,j_3-1] must contain the real and imaginary parts
        of z_{j_1 j_2 j_3}.

    y : float, array-like, shape (`n1`*`n2`*`n3`)
        The real and imaginary parts of the complex data values must be
        stored in arrays `x` and `y` respectively. If `x` and `y` are
        regarded as three-dimensional arrays of dimension
        (0:`n1`-1,0:`n2`-1,0:`n3`-1), `x`[j_1-1,j_2-1,j_3-1] and
        `y`[j_1-1,j_2-1,j_3-1] must contain the real and imaginary parts
        of z_{j_1 j_2 j_3}.

    Returns
    -------
    x : float, ndarray, shape (`n1`*`n2`*`n3`)
        The real and imaginary parts respectively of the corresponding
        elements of the computed transform.

    y : float, ndarray, shape (`n1`*`n2`*`n3`)
        The real and imaginary parts respectively of the corresponding
        elements of the computed transform.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 2)
            On entry, `n2` = *<value>*.

            Constraint: `n2` >= 1.

        (`errno` 3)
            On entry, `n3` = *<value>*.

            Constraint: `n3` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_complex_3d_sep as b_fft_complex_3d_sep

    fun_name = 'naginterfaces.library.sum.fft_complex_3d_sep'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_fft_complex_3d_sep(
        n1,
        n2,
        n3,
        _x,
        _y,
    )
    _x_py = _x
    _y_py = _y
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'y'],
    )(
        _x_py,
        _y_py,
    )
    return _return_data

def invlaplace_crump(f, t, relerr, alphab, tfac=0.8, mxterm=100, data=None):
    u"""
    Inverse Laplace transform, Crump's method.

    For full information please refer to the NAG Library document for
    c06la

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06laf.html

    Parameters
    ----------
    f : callable fp = f(p, data=None)
        `f` must evaluate the function F(p) for a given value of p.

        Parameters
        ~~~~~~~~~~
        p : complex
            The argument p.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        fp : complex
            The value F(p).

    t : float, array-like, shape (n)
        Each `t`[j-1] must specify a point at which the inverse Laplace
        transform is required, for j = 1,2,...,n.

    relerr : float
        The required relative error in the values of the inverse Laplace
        transform. If the absolute value of the inverse is less than
        `relerr`, absolute accuracy is used instead. `relerr` must be in
        the range 0.0 <= `relerr` < 1.0. If `relerr` is set too small or
        to 0.0, the function uses a value sufficiently larger than
        machine precision.

    alphab : float
        alpha_b, an upper bound for alpha (see [Description]). Usually,
        alpha_b should be specified equal to, or slightly larger than,
        the value of alpha. If alpha_b < alpha then the prescribed
        accuracy may not be achieved or completely incorrect results may
        be obtained. If alpha_b is too large ``invlaplace_crump`` will
        be inefficient and convergence may not be achieved.

        Note: it is as important to specify alpha_b correctly as it is
        to specify the correct function for inversion.

    tfac : float, optional
        t_fac, a factor to be used in calculating the parameter tau.
        Larger values (e.g., 5.0) may be specified for difficult
        problems, but these may require very large values of `mxterm`.

    mxterm : int, optional
        The maximum number of (complex) terms to be used in the
        evaluation of the Fourier series.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    valinv : float, ndarray, shape (n)
        An estimate of the value of the inverse Laplace transform at t =
        `t`[j-1], for j = 1,2,...,n.

    errest : float, ndarray, shape (n)
        An estimate of the error in `valinv`[j]. This is usually an
        estimate of relative error but, if `valinv`[j-1] < `relerr`,
        `errest`[j] estimates the absolute error. `errest`[j-1] is
        unreliable when `valinv`[j-1] is small but slightly greater than
        `relerr`.

    nterms : int
        The number of (complex) terms actually used.

    na : int
        The number of values of a used by the function. See [Further
        Comments].

    alow : float
        The smallest value of a used in the algorithm. This may be used
        for checking the value of `alphab`- see [Further Comments].

    ahigh : float
        The largest value of a used in the algorithm. This may be used
        for checking the value of `alphab`- see [Further Comments].

    nfeval : int
        The number of calls to `f` made by the function.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tfac` = *<value>*.

            Constraint: `tfac` > 0.5.

        (`errno` 1)
            On entry, `relerr` = *<value>*.

            Constraint: 0.0 <= `relerr` < 1.0.

        (`errno` 1)
            On entry, `mxterm` = *<value>*.

            Constraint: `mxterm` >= 1.

        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, the elements of `t` are not strictly increasing.

        (`errno` 2)
            On entry, `t`[0] = *<value>*.

            Constraint: 0.0 <= `t`[0] < `t`[1] < ... < `t`[n-1].

        (`errno` 3)
            On entry, `t`[n-1] is too large: `t`[n-1] = *<value>*. If
            necessary, scale the problem as described in [Further
            Comments].

        (`errno` 4)
            Required accuracy cannot be obtained. Try increasing `tfac`,
            `alphab`, or both. `tfac` = *<value>* and `alphab` =
            *<value>*.

        (`errno` 5)
            Convergence failure in epsilon algorithm: `mxterm` =
            *<value>*. Some values of `valinv`[j-1] may be calculated to
            the desired accuracy; this may be determined by examining
            the values of `errest`[j-1]. Try reducing the range of `t`
            or increasing `mxterm`. If `errno` = 5 still results, try
            reducing `tfac`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            All values of `valinv` have been calculated, but are not of
            the required accuracy; the values of `errest`[j-1] should be
            examined carefully. Try reducing the range of `t`, or
            increasing `tfac`, `alphab` or both.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import invlaplace_crump as b_invlaplace_crump

    def cb_f(p, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _fp_f = f(
                p,
                data,
            )
        else:
            _fp_f = f(
                p,
            )
        _fp_f_base = _fp_f
        return _fp_f_base

    fun_name = 'naginterfaces.library.sum.invlaplace_crump'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _t = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 't'},
        t,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_t, {'fun_name': fun_name, 'entity': 't'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _valinv = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    _errest = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n_py,),
    )
    try:
        _return_scalars = b_invlaplace_crump(
            _f,
            _n,
            _t,
            _valinv,
            _errest,
            relerr,
            alphab,
            tfac,
            mxterm,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _valinv_py = _valinv
    _errest_py = _errest
    _nterms_py = _return_scalars.nterms
    _na_py = _return_scalars.na
    _alow_py = _return_scalars.alow
    _ahigh_py = _return_scalars.ahigh
    _nfeval_py = _return_scalars.nfeval
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['valinv', 'errest', 'nterms', 'na', 'alow', 'ahigh', 'nfeval'],
    )(
        _valinv_py,
        _errest_py,
        _nterms_py,
        _na_py,
        _alow_py,
        _ahigh_py,
        _nfeval_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def invlaplace_weeks(f, sigma0, sigma, b, epstol, mmax=1024, data=None):
    u"""
    Inverse Laplace transform, modified Weeks' method.

    For full information please refer to the NAG Library document for
    c06lb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06lbf.html

    Parameters
    ----------
    f : callable retval = f(s, data=None)
        `f` must return the value of the Laplace transform function F(s)
        for a given complex value of s.

        Parameters
        ~~~~~~~~~~
        s : complex
            The value of s for which F(s) must be evaluated. The real
            part of `s` is greater than sigma_0.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : complex
            The value of the Laplace transform function F(s) for the
            given complex value, s.

    sigma0 : float
        The abscissa of convergence of the Laplace transform, sigma_0.

    sigma : float
        The parameter sigma of the Laguerre expansion. If on entry
        `sigma` <= sigma_0, `sigma` is reset to sigma_0+0.7.

    b : float
        The parameter b of the Laguerre expansion. If on entry `b` <
        2*(sigma-sigma_0), `b` is reset to 2.5*(sigma-sigma_0).

    epstol : float
        The required relative pseudo-accuracy, that is, an upper bound
        on |f (t) - f~ (t)|e^(-sigma t).

    mmax : int, optional
        An upper bound on the number of Laguerre expansion coefficients
        to be computed. The number of coefficients actually computed is
        always a power of 2, so `mmax` should be a power of 2; if `mmax`
        is not a power of 2 then the maximum number of coefficients
        calculated will be the largest power of 2 less than `mmax`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    sigma : float
        The value actually used for sigma, as just described.

    b : float
        The value actually used for b, as just described.

    m : int
        The number of Laguerre expansion coefficients actually computed.
        The number of calls to `f` is `m`/2+2.

    acoef : float, ndarray, shape (`mmax`)
        The first `m` elements contain the computed Laguerre expansion
        coefficients, a_i.

    errvec : float, ndarray, shape (8)
        An 8-component vector of diagnostic information.

        `errvec`[0]

          Overall estimate of the pseudo-error |f (t) - f~ (t)|e^(-sigma
          t) = `errvec`[1]+`errvec`[2]+`errvec`[3].

        `errvec`[1]

          Estimate of the discretization pseudo-error.

        `errvec`[2]

          Estimate of the truncation pseudo-error.

        `errvec`[3]

          Estimate of the condition pseudo-error on the basis of minimal
          noise levels in function values.

        `errvec`[4]

          K, coefficient of a heuristic decay function for the expansion
          coefficients.

        `errvec`[5]

          R, base of the decay function for the expansion coefficients.

        `errvec`[6]

          Logarithm of the largest expansion coefficient.

        `errvec`[7]

          Logarithm of the smallest nonzero expansion coefficient.

        The values K and R returned in `errvec`[4] and `errvec`[5]
        define a decay function KR^(-i) constructed by the function for
        the purposes of error estimation.

        It satisfies

            |a_i| < KR^(-i), i = 1,2,...,m.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `mmax` = *<value>*.

            Constraint: `mmax` >= 8.

        (`errno` 4)
            The decay rate of the coefficients is too small. Increasing
            `mmax` may help. `mmax` = *<value>*. Pseudo-error estimate
            `errvec`[0] = *<value>* and `epstol` = *<value>*.

        (`errno` 5)
            The decay rate of the coefficients is too small and
            round-off error is such that the required accuracy cannot be
            obtained. Increasing `mmax` or `epstol` may help. `mmax` =
            *<value>*. Pseudo-error estimate `errvec`[0] = *<value>* and
            `epstol` = *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The estimate of the pseudo-error is slightly larger than
            `epstol`. Pseudo-error estimate `errvec`[0] = *<value>* and
            `epstol` = *<value>*.

        (`errno` 3)
            The round-off error level is larger than `epstol`.
            Increasing `epstol` may help. Pseudo-error estimate
            `errvec`[0] = *<value>* and `epstol` = *<value>*.

        (`errno` 6)
            Error bounds cannot be predicted. Check `sigma0`. `sigma0` =
            *<value>*.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import invlaplace_weeks as b_invlaplace_weeks

    def cb_f(s, data=None):
        """Wrapping user's f. Called as base f."""
        if data is not None:
            _retval_f = f(
                s,
                data,
            )
        else:
            _retval_f = f(
                s,
            )
        return _retval_f

    fun_name = 'naginterfaces.library.sum.invlaplace_weeks'

    _b_utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = cb_f
    _mmax = _b_utils._EngineIntScalarType(
        mmax, {'fun_name': fun_name, 'entity': 'mmax'},
    ).to_ctypes()
    _mmax_py = _b_utils._EngineIntScalarType.to_py(_mmax)
    _acoef = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_mmax_py,),
    )
    _errvec = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (8,),
    )
    try:
        _return_scalars = b_invlaplace_weeks(
            _f,
            sigma0,
            sigma,
            b,
            epstol,
            _mmax,
            _acoef,
            _errvec,
            data,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _sigma_py = _return_scalars.sigma
    _b_py = _return_scalars.b
    _m_py = _return_scalars.m
    _acoef_py = _acoef
    _errvec_py = _errvec
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['sigma', 'b', 'm', 'acoef', 'errvec'],
    )(
        _sigma_py,
        _b_py,
        _m_py,
        _acoef_py,
        _errvec_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data

def invlaplace_weeks_eval(t, sigma, b, acoef, errvec):
    u"""
    Evaluate inverse Laplace transform as computed by
    ``invlaplace_weeks``.

    For full information please refer to the NAG Library document for
    c06lc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06lcf.html

    Parameters
    ----------
    t : float
        The value t for which the inverse Laplace transform f(t) must be
        evaluated.

    sigma : float
        `sigma` must be unchanged from the previous call of
        ``invlaplace_weeks``

    b : float
        `b` must be unchanged from the previous call of
        ``invlaplace_weeks``

    acoef : float, array-like, shape (m)
        `acoef` must be unchanged from the previous call of
        ``invlaplace_weeks``

    errvec : float, array-like, shape (8)
        `errvec` must be unchanged from the previous call of
        ``invlaplace_weeks``

    Returns
    -------
    finv : float
        The approximation to the inverse Laplace transform at t.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            The approximation to f(t) is too large to be representable.

        (`errno` 2)
            The approximation to f(t) is too small to be representable.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import invlaplace_weeks_eval as b_invlaplace_weeks_eval

    fun_name = 'naginterfaces.library.sum.invlaplace_weeks_eval'

    _acoef = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'acoef'},
        acoef,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_acoef, {'fun_name': fun_name, 'entity': 'acoef'}, exp_rank=1)[0]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _errvec = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'errvec'},
        errvec,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    try:
        _return_scalar = b_invlaplace_weeks_eval(
            t,
            sigma,
            b,
            _m,
            _acoef,
            _errvec,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalar = _the_exc.return_data[0]
    _finv_py = _return_scalar
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['finv'],
    )(
        _finv_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _finv_py

def fft_realherm_1d(direct, x):
    u"""
    Single one-dimensional real and Hermitian complex discrete Fourier
    transform, using complex storage format for Hermitian sequences.

    For full information please refer to the NAG Library document for
    c06pa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06paf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    x : float, array-like, shape (n+2)
        If `x` is declared with bounds (0:n+1) in the function from
        which ``fft_realherm_1d`` is called:

        - if `direct` = 'F', `x`[j] must contain x_j, for j =
          0,1,...,n-1;

        - if `direct` = 'B', `x`[2*k] and `x`[2*k+1] must contain the
          real and imaginary parts respectively of z_k, for k =
          0,1,...,n/2. (Note that for the sequence z_k to be Hermitian,
          the imaginary part of z_0, and of z_{n/2} for n even, must be
          zero.)

    Returns
    -------
    x : float, ndarray, shape (n+2)
        - If `direct` = 'F' and `x` is declared with bounds (0:n+1),
          `x`[2*k] and `x`[2*k+1] will contain the real and imaginary
          parts respectively of z^_k, for k = 0,1,...,n/2;

        - if `direct` = 'B' and `x` is declared with bounds (0:n+1),
          `x`[j] will contain x^_j, for j = 0,1,...,n-1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 3)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_realherm_1d as b_fft_realherm_1d

    fun_name = 'naginterfaces.library.sum.fft_realherm_1d'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]-2, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_fft_realherm_1d(
        direct,
        _x,
        _n,
    )
    _x_py = _x
    return _x_py

def fft_complex_1d(direct, x):
    u"""
    Single one-dimensional complex discrete Fourier transform, complex
    data type.

    For full information please refer to the NAG Library document for
    c06pc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06pcf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    x : complex, array-like, shape (n)
        If `x` is declared with bounds (0:n-1) in the function from
        which ``fft_complex_1d`` is called, `x`[j] must contain z_j, for
        j = 0,1,...,n-1.

    Returns
    -------
    x : complex, ndarray, shape (n)
        The components of the discrete Fourier transform. If `x` is
        declared with bounds (0:n-1) in the function from which
        ``fft_complex_1d`` is called, z^_k is contained in `x`[k], for 0
        <= k <= n-1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 4)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_complex_1d as b_fft_complex_1d

    fun_name = 'naginterfaces.library.sum.fft_complex_1d'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_fft_complex_1d(
        direct,
        _x,
        _n,
    )
    _x_py = _x
    return _x_py

def fft_complex_multid_1d(direct, l, nd, x):
    u"""
    One-dimensional complex discrete Fourier transform of
    multidimensional data (using complex data type).

    For full information please refer to the NAG Library document for
    c06pf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06pff.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    l : int
        l, the index of the variable (or dimension) on which the
        discrete Fourier transform is to be performed.

    nd : int, array-like, shape (ndim)
        The elements of `nd` must contain the dimensions of the ndim
        variables; that is, `nd`[i-1] must contain the dimension of the
        ith variable.

    x : complex, array-like, shape (n)
        The complex data values. Data values are stored in `x` using
        column-major ordering for storing multidimensional arrays; that
        is, z_{j_1 j_2...j_m} is stored in `x`[ 1 + j_1 + n_1 j_2 + n_1
        n_2 j_3 + ... ].

    Returns
    -------
    x : complex, ndarray, shape (n)
        The corresponding elements of the computed transform.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim >= 1.

        (`errno` 2)
            On entry, `l` = *<value>*.

            Constraint: 1 <= `l` <= ndim.

        (`errno` 3)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 4)
            On entry, `nd`[*<value>*] = *<value>*.

            Constraint: `nd`[i-1] >= 1, for all i.

        (`errno` 5)
            On entry, n = *<value>*, product of `nd` elements is
            *<value>*.

            Constraint: n must equal the product of the dimensions held
            in array `nd`.

        (`errno` 8)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_complex_multid_1d as b_fft_complex_multid_1d

    fun_name = 'naginterfaces.library.sum.fft_complex_multid_1d'

    _nd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'nd'},
        nd,
        copy=False,
    )
    _ndim_py = _b_utils._EngineIntArrayType.get_shape(_nd, {'fun_name': fun_name, 'entity': 'nd'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_fft_complex_multid_1d(
        direct,
        _ndim,
        l,
        _nd,
        _n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_complex_multid(direct, nd, x):
    u"""
    Multidimensional complex discrete Fourier transform of
    multidimensional data (using complex data type).

    For full information please refer to the NAG Library document for
    c06pj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06pjf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    nd : int, array-like, shape (ndim)
        The elements of `nd` must contain the dimensions of the ndim
        variables; that is, `nd`[i-1] must contain the dimension of the
        ith variable.

    x : complex, array-like, shape (n)
        The complex data values. Data values are stored in `x` using
        column-major ordering for storing multidimensional arrays; that
        is, z_{j_1 j_2...j_m} is stored in `x`[ 1 + j_1 + n_1 j_2 + n_1
        n_2 j_3 + ... ].

    Returns
    -------
    x : complex, ndarray, shape (n)
        The corresponding elements of the computed transform.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, ndim = *<value>*.

            Constraint: ndim >= 1.

        (`errno` 2)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 3)
            On entry, `nd`[*<value>*] = *<value>*.

            Constraint: `nd`[i-1] >= 1, for all i.

        (`errno` 4)
            On entry, n = *<value>*, product of `nd` elements is
            *<value>*.

            Constraint: n must equal the product of the dimensions held
            in array `nd`.

        (`errno` 7)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_complex_multid as b_fft_complex_multid

    fun_name = 'naginterfaces.library.sum.fft_complex_multid'

    _nd = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'nd'},
        nd,
        copy=False,
    )
    _ndim_py = _b_utils._EngineIntArrayType.get_shape(_nd, {'fun_name': fun_name, 'entity': 'nd'}, exp_rank=1)[0]
    _ndim = _b_utils._EngineIntScalarType(
        _ndim_py, {'fun_name': fun_name, 'entity': '_ndim'},
    ).to_ctypes()
    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_fft_complex_multid(
        direct,
        _ndim,
        _nd,
        _n,
        _x,
    )
    _x_py = _x
    return _x_py

def convcorr_complex(job, x, y):
    u"""
    Circular convolution or correlation of two complex vectors.

    For full information please refer to the NAG Library document for
    c06pk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06pkf.html

    Parameters
    ----------
    job : int
        The computation to be performed:

        `job` = 1

          z_k = sum_j = 0^n-1x_j y_{k-j} (convolution);

        `job` = 2

          w_k = sum_j = 0^n-1x__j y_{k+j} (correlation).

    x : complex, array-like, shape (n)
        The elements of one period of the vector x. If `x` is declared
        with bounds (0:n-1) in the function from which
        ``convcorr_complex`` is called, `x`[j] must contain x_j, for j =
        0,1,...,n-1.

    y : complex, array-like, shape (n)
        The elements of one period of the vector y. If `y` is declared
        with bounds (0:n-1) in the function from which
        ``convcorr_complex`` is called, `y`[j-1] must contain y_j, for j
        = 0,1,...,n-1.

    Returns
    -------
    x : complex, ndarray, shape (n)
        The corresponding elements of the discrete convolution or
        correlation.

    y : complex, ndarray, shape (n)
        The discrete Fourier transform of the convolution or correlation
        returned in the array `x`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 2)
            On entry, argument `job` had an illegal value.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import convcorr_complex as b_convcorr_complex

    fun_name = 'naginterfaces.library.sum.convcorr_complex'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    _n_py = _b_utils._EngineComplex128ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=1)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=True,
    )
    b_convcorr_complex(
        job,
        _x,
        _y,
        _n,
    )
    _x_py = _x
    _y_py = _y
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['x', 'y'],
    )(
        _x_py,
        _y_py,
    )
    return _return_data

def fft_realherm_1d_multi_row(direct, m, n, x):
    u"""
    Multiple one-dimensional real and Hermitian complex discrete Fourier
    transforms, using row ordered complex storage format for Hermitian
    sequences.

    For full information please refer to the NAG Library document for
    c06pp

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06ppf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    m : int
        m, the number of sequences to be transformed.

    n : int
        n, the number of real or complex values in each sequence.

    x : float, array-like, shape (`m`*(`n`+2))
        The data must be stored such that consecutive elements of the
        same sequence are stored with a stride of `m` and corresponding
        elements of different sequences are stored consecutively. An
        additional two spaces are reserved for each sequence to allow
        for the pairwise storage of real and imaginary parts in the
        transformed domain. In other words, if the data values of the
        pth sequence to be transformed are denoted by x_j^p, for j =
        0,1,...,n-1:

        - if `direct` = 'F', `x`[j*`m`+p-1] must contain x_j^p, for p =
          1,2,...,m, for j = 0,1,...,n-1;

        - if `direct` = 'B', `x`[2*k*`m`+p-1] and `x`[ (2*k+1) *
          `m`+p-1] must contain the real and imaginary parts
          respectively of z^_k^p, for p = 1,2,...,m, for k =
          0,1,...,n/2. (Note that for the sequence z^_k^p to be
          Hermitian, the imaginary part of z^_0^p, and of z^_n/2^p for n
          even, must be zero.)

    Returns
    -------
    x : float, ndarray, shape (`m`*(`n`+2))
        - If `direct` = 'F' then `x`[2*k*`m`+p-1] and
          `x`[(2*k+1)*`m`+p-1] will contain the real and imaginary parts
          respectively of z^_k^p, for p = 1,2,...,m, for k =
          0,1,...,n/2;

        - if `direct` = 'B' then `x`[j*`m`+p-1] will contain x_j^p, for
          p = 1,2,...,m, for j = 0,1,...,n-1;

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 4)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_realherm_1d_multi_row as b_fft_realherm_1d_multi_row

    fun_name = 'naginterfaces.library.sum.fft_realherm_1d_multi_row'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_fft_realherm_1d_multi_row(
        direct,
        m,
        n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_realherm_1d_multi_col(direct, n, m, x):
    u"""
    Multiple one-dimensional real and Hermitian complex discrete Fourier
    transforms, using column ordered complex storage format for
    Hermitian sequences.

    For full information please refer to the NAG Library document for
    c06pq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06pqf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    n : int
        n, the number of real or complex values in each sequence.

    m : int
        m, the number of sequences to be transformed.

    x : float, array-like, shape ((`n`+2)*`m`)
        The m real or Hermitian data sequences to be transformed.

        - if `direct` = 'F', the m real data sequences, x^p =
          (x_1^p,x_2^p,...,x_n^p), for p = 1,2,...,m, should be stored
          sequentially in `x`, with a stride of n+2 between sequences.

        - if `direct` = 'B', the m Hermitian data sequences, z^^p =
          (z^_1^p,z^_2^p,...,z^_n/2+1^p) =
          (Re(z^_1^p),Im(z^_1^p),Re(z^_2^p),Im(z^_2^p),...,Re(z^_n/2+1^p),Im(z^_n/2+1^p)),
          for p = 1,2,...,m, should be stored sequentially in `x`, with
          a stride of n+2 between sequences.

        In other words:

        - if `direct` = 'F', `x`[(p-1)*(`n`+2)+j-1] must contain x_j^p,
          for p = 1,2,...,m, for j = 1,2,...,n;

        - if `direct` = 'B', `x`[(p-1) * (`n`+2) + (2*k-1)-1] and
          `x`[(p-1) * (`n`+2) + (2*k)-1] must contain the real and
          imaginary parts respectively of z^_k^p, for p = 1,2,...,m, for
          k = 1,2,...,n/2+1. (Note that for the sequence z^_k^p to be
          Hermitian, the imaginary part of z^_1^p, and of z^_n/2+1^p for
          n even, must be zero.)

    Returns
    -------
    x : float, ndarray, shape ((`n`+2)*`m`)
        - If `direct` = 'F' then the m sequences, z^^p, for p =
          1,2,...,m stored as described on entry for `direct` = 'B'

        - if `direct` = 'B' then the m sequences, x^p, for p = 1,2,...,m
          stored as described on entry for `direct` = 'F'

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 4)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_realherm_1d_multi_col as b_fft_realherm_1d_multi_col

    fun_name = 'naginterfaces.library.sum.fft_realherm_1d_multi_col'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_fft_realherm_1d_multi_col(
        direct,
        n,
        m,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_complex_1d_multi_row(direct, m, n, x):
    u"""
    Multiple one-dimensional complex discrete Fourier transforms using
    complex data type.

    For full information please refer to the NAG Library document for
    c06pr

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06prf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    m : int
        m, the number of sequences to be transformed.

    n : int
        n, the number of complex values in each sequence.

    x : complex, array-like, shape (`m`*`n`)
        The complex data must be stored in `x` as if in a
        two-dimensional array of dimension (1:`m`,0:`n`-1); each of the
        m sequences is stored in a row of each array. In other words, if
        the elements of the pth sequence to be transformed are denoted
        by z_j^p, for j = 0,1,...,n-1, `x`[j*`m`+p-1] must contain
        z_j^p.

    Returns
    -------
    x : complex, ndarray, shape (`m`*`n`)
        Is overwritten by the complex transforms.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 5)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_complex_1d_multi_row as b_fft_complex_1d_multi_row

    fun_name = 'naginterfaces.library.sum.fft_complex_1d_multi_row'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    b_fft_complex_1d_multi_row(
        direct,
        m,
        n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_complex_1d_multi_col(direct, n, m, x):
    u"""
    Multiple one-dimensional complex discrete Fourier transforms,
    complex data type.

    For full information please refer to the NAG Library document for
    c06ps

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06psf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    n : int
        n, the number of complex values in each sequence.

    m : int
        m, the number of sequences to be transformed.

    x : complex, array-like, shape (`n`*`m`)
        The complex data values z_j^p stored in `x`[(p-1)*`n`+j], for p
        = 1,2,...,`m`, for j = 0,1,...,`n`-1.

    Returns
    -------
    x : complex, ndarray, shape (`n`*`m`)
        Is overwritten by the complex transforms.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 5)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_complex_1d_multi_col as b_fft_complex_1d_multi_col

    fun_name = 'naginterfaces.library.sum.fft_complex_1d_multi_col'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    b_fft_complex_1d_multi_col(
        direct,
        n,
        m,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_complex_2d(direct, m, n, x):
    u"""
    Two-dimensional complex discrete Fourier transform, complex data
    type.

    For full information please refer to the NAG Library document for
    c06pu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06puf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    m : int
        m, the first dimension of the transform.

    n : int
        n, the second dimension of the transform.

    x : complex, array-like, shape (`m`*`n`)
        The complex data values. `x`[`m`*j_2+j_1] must contain z_{j_1
        j_2}, for j_2 = 0,...,`n`-1, for j_1 = 0,...,`m`-1.

    Returns
    -------
    x : complex, ndarray, shape (`m`*`n`)
        The corresponding elements of the computed transform.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 6)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_complex_2d as b_fft_complex_2d

    fun_name = 'naginterfaces.library.sum.fft_complex_2d'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    b_fft_complex_2d(
        direct,
        m,
        n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_real_2d(m, n, x):
    u"""
    Two-dimensional real-to-complex discrete Fourier transform.

    For full information please refer to the NAG Library document for
    c06pv

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06pvf.html

    Parameters
    ----------
    m : int
        m, the first dimension of the transform.

    n : int
        n, the second dimension of the transform.

    x : float, array-like, shape (`m`*`n`)
        The real input dataset x, where x_{j_1j_2} is stored in `x`[ j_2
        * m + j_1], for j_2 = 0,1,...,n-1, for j_1 = 0,1,...,m-1.

    Returns
    -------
    y : complex, ndarray, shape ((`m`/2+1)*`n`)
        The complex output dataset z^, where z^_{k_1k_2} is stored in
        `y`[ k_2 * (m/2+1) + k_1], for k_2 = 0,1,...,n-1, for k_1 =
        0,1,...,m/2. Note the first dimension is cut roughly by half to
        remove the redundant information due to conjugate symmetry.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_real_2d as b_fft_real_2d

    fun_name = 'naginterfaces.library.sum.fft_real_2d'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _y = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        ((_m_py//2+1)*_n_py,),
    )
    b_fft_real_2d(
        _m,
        _n,
        _x,
        _y,
    )
    _y_py = _y
    return _y_py

def fft_hermitian_2d(m, n, y):
    u"""
    Two-dimensional complex-to-real discrete Fourier transform.

    For full information please refer to the NAG Library document for
    c06pw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06pwf.html

    Parameters
    ----------
    m : int
        m, the first dimension of the transform.

    n : int
        n, the second dimension of the transform.

    y : complex, array-like, shape ((`m`/2+1)*`n`)
        The Hermitian sequence of complex input dataset z, where
        z_{j_1j_2} is stored in `y`[ j_2 * (m/2+1) + j_1], for j_2 =
        0,1,...,n-1, for j_1 = 0,1,...,m/2.

    Returns
    -------
    x : float, ndarray, shape (`m`*`n`)
        The real output dataset x^, where x^_{k_1k_2} is stored in `x`[
        k_2 * m + k_1], for k_2 = 0,1,...,n-1, for k_1 = 0,1,...,m-1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_hermitian_2d as b_fft_hermitian_2d

    fun_name = 'naginterfaces.library.sum.fft_hermitian_2d'

    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _m = _b_utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _m_py = _b_utils._EngineIntScalarType.to_py(_m)
    _n = _b_utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = _b_utils._EngineIntScalarType.to_py(_n)
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py*_n_py,),
    )
    b_fft_hermitian_2d(
        _m,
        _n,
        _y,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_complex_3d(direct, n1, n2, n3, x):
    u"""
    Three-dimensional complex discrete Fourier transform, complex data
    type.

    For full information please refer to the NAG Library document for
    c06px

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06pxf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    n1 : int
        n_1, the first dimension of the transform.

    n2 : int
        n_2, the second dimension of the transform.

    n3 : int
        n_3, the third dimension of the transform.

    x : complex, array-like, shape (`n1`*`n2`*`n3`)
        The complex data values. Data values are stored in `x` using
        column-major ordering for storing multidimensional arrays; that
        is, z_{j_1 j_2 j_3} is stored in `x`[ 1 + j_1 + n_1 j_2 + n_1
        n_2 j_3 ].

    Returns
    -------
    x : complex, ndarray, shape (`n1`*`n2`*`n3`)
        The corresponding elements of the computed transform.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 2)
            On entry, `n2` = *<value>*.

            Constraint: `n2` >= 1.

        (`errno` 3)
            On entry, `n3` = *<value>*.

            Constraint: `n3` >= 1.

        (`errno` 4)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 8)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_complex_3d as b_fft_complex_3d

    fun_name = 'naginterfaces.library.sum.fft_complex_3d'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
    )
    b_fft_complex_3d(
        direct,
        n1,
        n2,
        n3,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_real_3d(n1, n2, n3, x):
    u"""
    Three-dimensional real-to-complex discrete Fourier transform.

    For full information please refer to the NAG Library document for
    c06py

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06pyf.html

    Parameters
    ----------
    n1 : int
        n_1, the first dimension of the transform.

    n2 : int
        n_2, the second dimension of the transform.

    n3 : int
        n_3, the third dimension of the transform.

    x : float, array-like, shape (`n1`*`n2`*`n3`)
        The real input dataset x, where x_{j_1j_2j_3} is stored in `x`[
        j_3 * n_1 n_2 + j_2 * n_1 + j_1 +1 -1], for j_3 = 0,1,...,n_3-1,
        for j_2 = 0,1,...,n_2-1, for j_1 = 0,1,...,n_1-1.

    Returns
    -------
    y : complex, ndarray, shape ((`n1`/2+1)*`n2`*`n3`)
        The complex output dataset z^, where z^_{k_1k_2k_3} is stored in
        `y`[ k_3 * (n_1/2+1) n_2 + k_2 * (n_1/2+1) + k_1 +1 -1], for k_3
        = 0,1,...,n_3-1, for k_2 = 0,1,...,n_2-1, for k_1 =
        0,1,...,n_1/2. Note the first dimension is cut roughly by half
        to remove the redundant information due to conjugate symmetry.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 2)
            On entry, `n2` = *<value>*.

            Constraint: `n2` >= 1.

        (`errno` 3)
            On entry, `n3` = *<value>*.

            Constraint: `n3` >= 1.

        (`errno` 4)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_real_3d as b_fft_real_3d

    fun_name = 'naginterfaces.library.sum.fft_real_3d'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _n1 = _b_utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n1_py = _b_utils._EngineIntScalarType.to_py(_n1)
    _n2 = _b_utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _n2_py = _b_utils._EngineIntScalarType.to_py(_n2)
    _n3 = _b_utils._EngineIntScalarType(
        n3, {'fun_name': fun_name, 'entity': 'n3'},
    ).to_ctypes()
    _n3_py = _b_utils._EngineIntScalarType.to_py(_n3)
    _y = _b_utils._EngineComplex128ArrayType.empty_ndarray(
        ((_n1_py//2+1)*_n2_py*_n3_py,),
    )
    b_fft_real_3d(
        _n1,
        _n2,
        _n3,
        _x,
        _y,
    )
    _y_py = _y
    return _y_py

def fft_hermitian_3d(n1, n2, n3, y):
    u"""
    Three-dimensional complex-to-real discrete Fourier transform.

    For full information please refer to the NAG Library document for
    c06pz

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06pzf.html

    Parameters
    ----------
    n1 : int
        n_1, the first dimension of the transform.

    n2 : int
        n_2, the second dimension of the transform.

    n3 : int
        n_3, the third dimension of the transform.

    y : complex, array-like, shape ((`n1`/2+1)*`n2`*`n3`)
        The Hermitian sequence of complex input dataset z, where
        z_{j_1j_2j_3} is stored in `y`[ j_3 * (n_1/2+1) n_2 + j_2 *
        (n_1/2+1) + j_1 +1 -1], for j_3 = 0,1,...,n_3-1, for j_2 =
        0,1,...,n_2-1, for j_1 = 0,1,...,n_1/2.

    Returns
    -------
    x : float, ndarray, shape (`n1`*`n2`*`n3`)
        The real output dataset x^, where x^_{k_1k_2k_3} is stored in
        `x`[ k_3 * n_1 n_2 + k_2 * n_1 + k_1 +1 -1], for k_3 =
        0,1,...,n_3-1, for k_2 = 0,1,...,n_2-1, for k_1 = 0,1,...,n_1-1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 2)
            On entry, `n2` = *<value>*.

            Constraint: `n2` >= 1.

        (`errno` 3)
            On entry, `n3` = *<value>*.

            Constraint: `n3` >= 1.

        (`errno` 4)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_hermitian_3d as b_fft_hermitian_3d

    fun_name = 'naginterfaces.library.sum.fft_hermitian_3d'

    _y = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'y'},
        y,
        copy=False,
    )
    _n1 = _b_utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n1_py = _b_utils._EngineIntScalarType.to_py(_n1)
    _n2 = _b_utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _n2_py = _b_utils._EngineIntScalarType.to_py(_n2)
    _n3 = _b_utils._EngineIntScalarType(
        n3, {'fun_name': fun_name, 'entity': 'n3'},
    ).to_ctypes()
    _n3_py = _b_utils._EngineIntScalarType.to_py(_n3)
    _x = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_n1_py*_n2_py*_n3_py,),
    )
    b_fft_hermitian_3d(
        _n1,
        _n2,
        _n3,
        _y,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_real_sine_simple(m, n, x):
    u"""
    Discrete sine transform (easy-to-use).

    For full information please refer to the NAG Library document for
    c06ra

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06raf.html

    Parameters
    ----------
    m : int
        m, the number of sequences to be transformed.

    n : int
        One more than the number of real values in each sequence, i.e.,
        the number of values in each sequence is n-1.

    x : float, array-like, shape (`m`*(`n`+2))
        The data must be stored in `x` as if in a two-dimensional array
        of dimension (1:`m`,1:`n`+2); each of the m sequences is stored
        in a row of the array. In other words, if the n-1 data values of
        the pth sequence to be transformed are denoted by x_j^p, for p =
        1,2,...,m, for j = 1,2,...,n-1, the first m(n-1) elements of the
        array `x` must contain the values

x_1^1,x_1^2,...,x_1^m,x_2^1,x_2^2,...,x_2^m,...,x_n-1^1,x_n-1^2,...,x_n-1^m.

        The nth to (n+2)th elements of each row x_n^p,...,x_n+2^p, for p
        = 1,2,...,m, are required as workspace. These 3m elements may
        contain arbitrary values as they are set to zero by the
        function.

    Returns
    -------
    x : float, ndarray, shape (`m`*(`n`+2))
        The m Fourier sine transforms stored as if in a two-dimensional
        array of dimension (1:`m`,1:`n`+2). Each of the m transforms is
        stored in a row of the array, overwriting the corresponding
        original sequence. If the (n-1) components of the pth Fourier
        sine transform are denoted by x^_k^p, for p = 1,2,...,m, for k =
        1,2,...,n-1, the m(n+2) elements of the array `x` contain the
        values

x^_1^1,x^_1^2,...,x^_1^m,x^_2^1,x^_2^2,...,x^_2^m,...,x^_n-1^1,x^_n-1^2,...,x^_n-1^m,0,0,...,0
            (3m times).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_real_sine_simple as b_fft_real_sine_simple

    fun_name = 'naginterfaces.library.sum.fft_real_sine_simple'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_fft_real_sine_simple(
        m,
        n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_real_cosine_simple(m, n, x):
    u"""
    Discrete cosine transform (easy-to-use).

    For full information please refer to the NAG Library document for
    c06rb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06rbf.html

    Parameters
    ----------
    m : int
        m, the number of sequences to be transformed.

    n : int
        One less than the number of real values in each sequence, i.e.,
        the number of values in each sequence is n+1.

    x : float, array-like, shape (`m`*(`n`+3))
        The data must be stored in `x` as if in a two-dimensional array
        of dimension (1:`m`,0:`n`+2); each of the m sequences is stored
        in a row of the array. In other words, if the (n+1) data values
        of the pth sequence to be transformed are denoted by x_j^p, for
        p = 1,2,...,m, for j = 0,1,...,n, the first m(n+1) elements of
        the array `x` must contain the values

x_0^1,x_0^2,...,x_0^m,x_1^1,x_1^2,...,x_1^m,...,x_n^1,x_n^2,...,x_n^m.

        The (n+2)th and (n+3)th elements of each row x_n+1^p,x_n+2^p,
        for p = 1,2,...,m, are required as workspace. These 2m elements
        may contain arbitrary values as they are set to zero by the
        function.

    Returns
    -------
    x : float, ndarray, shape (`m`*(`n`+3))
        The m Fourier cosine transforms stored as if in a
        two-dimensional array of dimension (1:`m`,0:`n`+2). Each of the
        m transforms is stored in a row of the array, overwriting the
        corresponding original data. If the (n+1) components of the pth
        Fourier cosine transform are denoted by x^_k^p, for p =
        1,2,...,m, for k = 0,1,...,n, the m(n+3) elements of the array
        `x` contain the values

x^_0^1,x^_0^2,...,x^_0^m,x^_1^1,x^_1^2,...,x^_1^m,...,x^_n^1,x^_n^2,...,x^_n^m,0,0,...,0
            (2m times).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_real_cosine_simple as b_fft_real_cosine_simple

    fun_name = 'naginterfaces.library.sum.fft_real_cosine_simple'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_fft_real_cosine_simple(
        m,
        n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_real_qtrsine_simple(direct, m, n, x):
    u"""
    Discrete quarter-wave sine transform (easy-to-use).

    For full information please refer to the NAG Library document for
    c06rc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06rcf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    m : int
        m, the number of sequences to be transformed.

    n : int
        n, the number of real values in each sequence.

    x : float, array-like, shape (`m`*(`n`+2))
        The data must be stored in `x` as if in a two-dimensional array
        of dimension (1:`m`,1:`n`+2); each of the m sequences is stored
        in a row of the array. In other words, if the data values of the
        pth sequence to be transformed are denoted by x_j^p, for p =
        1,2,...,m, for j = 1,2,...,n, the first mn elements of the array
        `x` must contain the values

x_1^1,x_1^2,...,x_1^m,x_2^1,x_2^2,...,x_2^m,...,x_n^1,x_n^2,...,x_n^m.

        The (n+1)th and (n+2)th elements of each row x_n+1^p,x_n+2^p,
        for p = 1,2,...,m, are required as workspace. These 2m elements
        may contain arbitrary values as they are set to zero by the
        function.

    Returns
    -------
    x : float, ndarray, shape (`m`*(`n`+2))
        The m quarter-wave sine transforms stored as if in a
        two-dimensional array of dimension (1:`m`,1:`n`+2). Each of the
        m transforms is stored in a row of the array, overwriting the
        corresponding original sequence. If the n components of the pth
        quarter-wave sine transform are denoted by x^_k^p, for p =
        1,2,...,m, for k = 1,2,...,n, the m(n+2) elements of the array
        `x` contain the values

x^_1^1,x^_1^2,...,x^_1^m,x^_2^1,x^_2^2,...,x^_2^m,...,x^_n^1,x^_n^2,...,x^_n^m,0,0,...,0
            (2m times).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 4)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_real_qtrsine_simple as b_fft_real_qtrsine_simple

    fun_name = 'naginterfaces.library.sum.fft_real_qtrsine_simple'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_fft_real_qtrsine_simple(
        direct,
        m,
        n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_real_qtrcosine_simple(direct, m, n, x):
    u"""
    Discrete quarter-wave cosine transform (easy-to-use).

    For full information please refer to the NAG Library document for
    c06rd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06rdf.html

    Parameters
    ----------
    direct : str, length 1
        If the forward transform as defined in [Description] is to be
        computed, `direct` must be set equal to 'F'.

        If the backward transform is to be computed, `direct` must be
        set equal to 'B'.

    m : int
        m, the number of sequences to be transformed.

    n : int
        n, the number of real values in each sequence.

    x : float, array-like, shape (`m`*(`n`+2))
        The data must be stored in `x` as if in a two-dimensional array
        of dimension (1:`m`,0:`n`+1); each of the m sequences is stored
        in a row of the array. In other words, if the data values of the
        pth sequence to be transformed are denoted by x_j^p, for p =
        1,2,...,m, for j = 0,1,...,n-1, the first mn elements of the
        array `x` must contain the values

x_0^1,x_0^2,...,x_0^m,x_1^1,x_1^2,...,x_1^m,...,x_n-1^1,x_n-1^2,...,x_n-1^m.

        The (n+1)th and (n+2)th elements of each row x_n^p,x_n+1^p, for
        p = 1,2,...,m, are required as workspace. These 2m elements may
        contain arbitrary values as they are set to zero by the
        function.

    Returns
    -------
    x : float, ndarray, shape (`m`*(`n`+2))
        The m quarter-wave cosine transforms stored as if in a
        two-dimensional array of dimension (1:`m`,0:`n`+1). Each of the
        m transforms is stored in a row of the array, overwriting the
        corresponding original sequence. If the n components of the pth
        quarter-wave cosine transform are denoted by x^_k^p, for p =
        1,2,...,m, for k = 0,1,...,n-1, the m(n+2) elements of the array
        `x` contain the values

x^_0^1,x^_0^2,...,x^_0^m,x^_1^1,x^_1^2,...,x^_1^m,...,x^_n-1^1,x^_n-1^2,...,x^_n-1^m,0,0,...,0
            (2m times).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `direct` = *<value>*.

            Constraint: `direct` = 'F' or 'B'.

        (`errno` 4)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base.sum import fft_real_qtrcosine_simple as b_fft_real_qtrcosine_simple

    fun_name = 'naginterfaces.library.sum.fft_real_qtrcosine_simple'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    b_fft_real_qtrcosine_simple(
        direct,
        m,
        n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_sine(x):
    u"""
    Multiple discrete sine transforms, simple.

    For full information please refer to the NAG Library document for
    c06re

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06ref.html

    Parameters
    ----------
    x : float, array-like, shape (n-1, m)
        The data values of the pth sequence to be transformed, denoted
        by x_j^p, for p = 1,2,...,m, for j = 1,2,...,n-1, must be stored
        in `x`[j-1,p-1].

    Returns
    -------
    x : float, ndarray, shape (n-1, m)
        The (n-1) components of the pth Fourier sine transform, denoted
        by x^_k^p, for p = 1,2,...,m, for k = 1,2,...,n-1, are stored in
        `x`[k-1,p-1], overwriting the corresponding original values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_sine as b_fft_sine

    fun_name = 'naginterfaces.library.sum.fft_sine'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]+1, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_fft_sine(
        _m,
        _n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_cosine(x):
    u"""
    Multiple discrete cosine transforms, simple.

    For full information please refer to the NAG Library document for
    c06rf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06rff.html

    Parameters
    ----------
    x : float, array-like, shape (n+1, m)
        The data values of the pth sequence to be transformed, denoted
        by x_j^p, for p = 1,2,...,m, for j = 0,1,...,n, must be stored
        in `x`[j,p-1].

    Returns
    -------
    x : float, ndarray, shape (n+1, m)
        The (n+1) components of the pth Fourier cosine transform,
        denoted by x^_k^p, for p = 1,2,...,m, for k = 0,1,...,n, are
        stored in `x`[k,p-1], overwriting the corresponding original
        values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_cosine as b_fft_cosine

    fun_name = 'naginterfaces.library.sum.fft_cosine'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n = _b_utils._EngineIntScalarType(
        _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]-1, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_fft_cosine(
        _m,
        _n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_qtrsine(idir, x):
    u"""
    Multiple discrete quarter-wave sine transforms, simple.

    For full information please refer to the NAG Library document for
    c06rg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06rgf.html

    Parameters
    ----------
    idir : int
        Indicates the transform, as defined in [Description], to be
        computed.

        `idir` = 1

          Forward transform.

        `idir` = -1

          Inverse transform.

    x : float, array-like, shape (n, m)
        The data values of the pth sequence to be transformed, denoted
        by x_j^p, for p = 1,2,...,m, for j = 1,2,...,n, must be stored
        in `x`[j-1,p-1].

    Returns
    -------
    x : float, ndarray, shape (n, m)
        The n components of the pth quarter-wave sine transform, denoted
        by x^_k^p, for p = 1,2,...,m, for k = 1,2,...,n, are stored in
        `x`[k-1,p-1], overwriting the corresponding original values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, `idir` = *<value>*.

            Constraint: `idir` = -1 or 1.

        (`errno` 4)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_qtrsine as b_fft_qtrsine

    fun_name = 'naginterfaces.library.sum.fft_qtrsine'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_fft_qtrsine(
        idir,
        _m,
        _n,
        _x,
    )
    _x_py = _x
    return _x_py

def fft_qtrcosine(idir, x):
    u"""
    Multiple discrete quarter-wave cosine transforms, simple.

    For full information please refer to the NAG Library document for
    c06rh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06rhf.html

    Parameters
    ----------
    idir : int
        Indicates the transform, as defined in [Description], to be
        computed.

        `idir` = 1

          Forward transform.

        `idir` = -1

          Inverse transform.

    x : float, array-like, shape (n, m)
        The data values of the pth sequence to be transformed, denoted
        by x_j^p, for p = 1,2,...,m, for j = 0,1,...,n-1, must be stored
        in `x`[j,p-1].

    Returns
    -------
    x : float, ndarray, shape (n, m)
        The n components of the pth quarter-wave cosine transform,
        denoted by x^_k^p, for p = 1,2,...,m, for k = 0,1,...,n-1, are
        stored in `x`[k,p-1], overwriting the corresponding original
        values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, m = *<value>*.

            Constraint: m >= 1.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n >= 1.

        (`errno` 3)
            On entry, `idir` = *<value>*.

            Constraint: `idir` = -1 or 1.

        (`errno` 4)
            An internal error has occurred in this function. Check the
            function call and any array sizes. If the call is correct
            then please contact NAG for assistance.
    """

    from ..base.sum import fft_qtrcosine as b_fft_qtrcosine

    fun_name = 'naginterfaces.library.sum.fft_qtrcosine'

    _x = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'x'},
        x,
        copy=True,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2)[0]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    b_fft_qtrcosine(
        idir,
        _m,
        _n,
        _x,
    )
    _x_py = _x
    return _x_py

def fast_gauss(srcs, trgs, q, p1, p2, k, hin, tol):
    u"""
    Multidimensional fast Gauss transform.

    For full information please refer to the NAG Library document for
    c06sa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c06/c06saf.html

    Parameters
    ----------
    srcs : float, array-like, shape (d, n)
        x, the locations of the Gaussian sources.

    trgs : float, array-like, shape (d, m)
        y, the locations of the target points at which the FGT will be
        evaluated.

    q : float, array-like, shape (n)
        q, the weights of the Gaussian sources.

    p1 : int
        p_1, the number of terms of the first Taylor series to be
        evaluated.

    p2 : int
        p_2, the number of terms of the second Taylor series to be
        evaluated.

    k : int
        k, the number of clusters into which the source points will be
        aggregated.

    hin : float, array-like, shape (lhin)
        h, the standard deviations of the Gaussian sources. If lhin < n,
        the array will be expanded automatically by repeating `hin`
        until it is of length n. See [the G01 Introduction] for further
        information.

    tol : float
        epsilon, the desired accuracy of the FGT approximation of the
        DGT. Determines the radius of the source clusters: the
        contribution of a source point to the FGT approximation at a
        target point is disregarded if the source is outside the
        corresponding cluster radius.

    Returns
    -------
    p1 : int
        This argument is unchanged.

    p2 : int
        This argument is unchanged.

    k : int
        This argument is unchanged.

    v : float, ndarray, shape (m)
        G^(y), the value of the FGT evaluated at y.

    term : float, ndarray, shape (m)
        `term`[j-1] contains the absolute value of the final Taylor
        series term that is largest, relative to the size of the sum of
        the corresponding series, across all clusters that contribute to
        the FGT at target point `v`[j-1].

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, d = *<value>*.

            Constraint: d > 0.

        (`errno` 2)
            On entry, n = *<value>*.

            Constraint: n > 0.

        (`errno` 3)
            On entry, m = *<value>*.

            Constraint: m > 0.

        (`errno` 4)
            On entry, `p1` = *<value>*.

            Constraint: `p1` > 0.

        (`errno` 5)
            On entry, `p2` = *<value>*.

            Constraint: `p2` > 0.

        (`errno` 6)
            On entry, `k` = *<value>* and n = *<value>*.

            Constraint: 1 <= `k` <= n.

        (`errno` 7)
            On entry, `hin`[*<value>*] = *<value>*.

            Constraint: `hin`[i] > 0.0, for i = 0,...,lhin-1.

        (`errno` 8)
            On entry, lhin = *<value>*.

            Constraint: 1 <= lhin <= n.

        (`errno` 9)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 10)
            On exit, `p1` = *<value>*, `p2` = *<value>* and `k` =
            *<value>*.

            `p1`, `p2` or `k` may have been too small to calculate
            `v`[m-1] to the required accuracy `tol`.
    """

    from ..base.sum import fast_gauss as b_fast_gauss

    fun_name = 'naginterfaces.library.sum.fast_gauss'

    _srcs = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'srcs'},
        srcs,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _d_py = _b_utils._EngineFloat64ArrayType.get_shape(_srcs, {'fun_name': fun_name, 'entity': 'srcs'}, exp_rank=2)[0]
    _d = _b_utils._EngineIntScalarType(
        _d_py, {'fun_name': fun_name, 'entity': '_d'},
    ).to_ctypes()
    _n_py = _b_utils._EngineFloat64ArrayType.get_shape(_srcs, {'fun_name': fun_name, 'entity': 'srcs'}, exp_rank=2)[1]
    _n = _b_utils._EngineIntScalarType(
        _n_py, {'fun_name': fun_name, 'entity': '_n'},
    ).to_ctypes()
    _trgs = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'trgs'},
        trgs,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _m_py = _b_utils._EngineFloat64ArrayType.get_shape(_trgs, {'fun_name': fun_name, 'entity': 'trgs'}, exp_rank=2)[1]
    _m = _b_utils._EngineIntScalarType(
        _m_py, {'fun_name': fun_name, 'entity': '_m'},
    ).to_ctypes()
    _q = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'q'},
        q,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _hin = _utils._to_ndarray(
        {'fun_name': fun_name, 'entity': 'hin'},
        hin,
        copy=False,
        exp_class=_b_utils._EngineFloat64ArrayType,
    )
    _lhin_py = _b_utils._EngineFloat64ArrayType.get_shape(_hin, {'fun_name': fun_name, 'entity': 'hin'}, exp_rank=1)[0]
    _lhin = _b_utils._EngineIntScalarType(
        _lhin_py, {'fun_name': fun_name, 'entity': '_lhin'},
    ).to_ctypes()
    _v = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    _term = _b_utils._EngineFloat64ArrayType.empty_ndarray(
        (_m_py,),
    )
    try:
        _return_scalars = b_fast_gauss(
            _d,
            _srcs,
            _n,
            _trgs,
            _m,
            _q,
            p1,
            p2,
            k,
            _hin,
            _lhin,
            tol,
            _v,
            _term,
        )
        _reraise = False
    except _b_utils.NagAlgorithmicWarning as _exc:
        _the_exc = _exc
        _reraise = True
        _return_scalars = _the_exc.return_data
    _p1_py = _return_scalars.p1
    _p2_py = _return_scalars.p2
    _k_py = _return_scalars.k
    _v_py = _v
    _term_py = _term
    _return_data = _collections.namedtuple(
        'nag_return_data',
        ['p1', 'p2', 'k', 'v', 'term'],
    )(
        _p1_py,
        _p2_py,
        _k_py,
        _v_py,
        _term_py,
    )
    if _reraise:
        _the_exc.return_data = _return_data
        raise _the_exc
    return _return_data
