"""
Deprecated wrappers for the NAG C Library (Mark 26.0) d05 Chapter.

d05 - Integral Equations.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/D05/d05conts.html

NAG Copyright 2017.
"""

from ctypes import c_double
from ctypes import POINTER
from .util import get_library
from .util import Integer
from .util import NAG_D05AAC_K1
from .util import NAG_D05AAC_K2
from .util import NAG_D05AAC_G
from .util import Nag_KernelForm
from .util import Nag_Comm
from .util import NagError
from .util import NAG_D05ABC_K
from .util import NAG_D05ABC_G
from .util import Nag_Boolean
from .util import NAG_D05BAC_CK
from .util import NAG_D05BAC_CG
from .util import NAG_D05BAC_CF
from .util import Nag_ODEMethod
from .util import NAG_D05BDC_CK
from .util import NAG_D05BDC_CF
from .util import NAG_D05BDC_CG
from .util import Nag_WeightMode
from .util import NAG_D05BEC_CK
from .util import NAG_D05BEC_CF
from .util import NAG_D05BEC_CG

_library_cl = get_library()

d05aac_ctypes = _library_cl.d05aac
d05aac_ctypes.restype = None
d05aac_ctypes.argtypes = [c_double, c_double, c_double, Integer, NAG_D05AAC_K1, NAG_D05AAC_K2, NAG_D05AAC_G, Nag_KernelForm, POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def d05aac(_lambda, a, b, n, k1, k2, g, kform, f, c, comm, fail):
    """
    Linear nonsingular Fredholm integral equation, second kind, split
    kernel.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d05/d05aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        f, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        f, c
    """
    from .util import nag_double_type_check_and_cast
    _f = nag_double_type_check_and_cast('d05aac', f, 'f')
    _c = nag_double_type_check_and_cast('d05aac', c, 'c')
    d05aac_ctypes(_lambda, a, b, n, k1, k2, g, kform, _f, _c, comm, fail)

d05abc_ctypes = _library_cl.d05abc
d05abc_ctypes.restype = None
d05abc_ctypes.argtypes = [c_double, c_double, c_double, Integer, NAG_D05ABC_K, NAG_D05ABC_G, Nag_Boolean, Nag_Boolean, POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def d05abc(_lambda, a, b, n, k, g, odorev, ev, f, c, comm, fail):
    """
    Linear nonsingular Fredholm integral equation, second kind, smooth
    kernel.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d05/d05abc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        f, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        f, c
    """
    from .util import nag_double_type_check_and_cast
    _f = nag_double_type_check_and_cast('d05abc', f, 'f')
    _c = nag_double_type_check_and_cast('d05abc', c, 'c')
    d05abc_ctypes(_lambda, a, b, n, k, g, odorev, ev, _f, _c, comm, fail)

d05bac_ctypes = _library_cl.d05bac
d05bac_ctypes.restype = None
d05bac_ctypes.argtypes = [NAG_D05BAC_CK, NAG_D05BAC_CG, NAG_D05BAC_CF, Nag_ODEMethod, Integer, c_double, c_double, c_double, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def d05bac(ck, cg, cf, method, iorder, alim, tlim, tol, nmesh, thresh, work, lwk, yn, errest, comm, fail):
    """
    Nonlinear Volterra convolution equation, second kind.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d05/d05bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        work, yn, errest
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        work, yn, errest
    """
    from .util import nag_double_type_check_and_cast
    _work = nag_double_type_check_and_cast('d05bac', work, 'work')
    _yn = nag_double_type_check_and_cast('d05bac', yn, 'yn')
    _errest = nag_double_type_check_and_cast('d05bac', errest, 'errest')
    d05bac_ctypes(ck, cg, cf, method, iorder, alim, tlim, tol, nmesh, thresh, _work, lwk, _yn, _errest, comm, fail)

d05bdc_ctypes = _library_cl.d05bdc
d05bdc_ctypes.restype = None
d05bdc_ctypes.argtypes = [NAG_D05BDC_CK, NAG_D05BDC_CF, NAG_D05BDC_CG, Nag_WeightMode, Integer, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Nag_Comm), POINTER(NagError)]


def d05bdc(ck, cf, cg, wtmode, iorder, tlim, tolnl, nmesh, yn, rwsav, lrwsav, comm, fail):
    """
    Nonlinear convolution Volterra--Abel equation, second kind, weakly
    singular.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d05/d05bdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        yn, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        yn, rwsav
    """
    from .util import nag_double_type_check_and_cast
    _yn = nag_double_type_check_and_cast('d05bdc', yn, 'yn')
    _rwsav = nag_double_type_check_and_cast('d05bdc', rwsav, 'rwsav')
    d05bdc_ctypes(ck, cf, cg, wtmode, iorder, tlim, tolnl, nmesh, _yn, _rwsav, lrwsav, comm, fail)

d05bec_ctypes = _library_cl.d05bec
d05bec_ctypes.restype = None
d05bec_ctypes.argtypes = [NAG_D05BEC_CK, NAG_D05BEC_CF, NAG_D05BEC_CG, Nag_WeightMode, Integer, c_double, c_double, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(Nag_Comm), POINTER(NagError)]


def d05bec(ck, cf, cg, wtmode, iorder, tlim, tolnl, nmesh, yn, rwsav, lrwsav, comm, fail):
    """
    Nonlinear convolution Volterra--Abel equation, first kind, weakly
    singular.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d05/d05bec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        yn, rwsav
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        yn, rwsav
    """
    from .util import nag_double_type_check_and_cast
    _yn = nag_double_type_check_and_cast('d05bec', yn, 'yn')
    _rwsav = nag_double_type_check_and_cast('d05bec', rwsav, 'rwsav')
    d05bec_ctypes(ck, cf, cg, wtmode, iorder, tlim, tolnl, nmesh, _yn, _rwsav, lrwsav, comm, fail)

d05bwc_ctypes = _library_cl.d05bwc
d05bwc_ctypes.restype = None
d05bwc_ctypes.argtypes = [Nag_ODEMethod, Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d05bwc(method, iorder, nomg, omega, sw, fail):
    """
    Generate weights for use in solving Volterra equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d05/d05bwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        omega, sw
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        omega, sw
    """
    from .util import nag_double_type_check_and_cast
    _omega = nag_double_type_check_and_cast('d05bwc', omega, 'omega')
    _sw = nag_double_type_check_and_cast('d05bwc', sw, 'sw')
    d05bwc_ctypes(method, iorder, nomg, _omega, _sw, fail)

d05byc_ctypes = _library_cl.d05byc
d05byc_ctypes.restype = None
d05byc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def d05byc(iorder, iq, omega, sw, fail):
    """
    Generate weights for use in solving weakly singular Abel-type equations.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/d05/d05byc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        omega, sw
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        omega, sw
    """
    from .util import nag_double_type_check_and_cast
    _omega = nag_double_type_check_and_cast('d05byc', omega, 'omega')
    _sw = nag_double_type_check_and_cast('d05byc', sw, 'sw')
    d05byc_ctypes(iorder, iq, _omega, _sw, fail)

nag_inteq_fredholm2_split = d05aac
nag_inteq_fredholm2_smooth = d05abc
nag_inteq_volterra2 = d05bac
nag_inteq_abel2_weak = d05bdc
nag_inteq_abel1_weak = d05bec
nag_inteq_volterra_weights = d05bwc
nag_inteq_abel_weak_weights = d05byc
__all__ = ['d05aac', 'd05abc', 'd05bac', 'd05bdc', 'd05bec', 'd05bwc',
           'd05byc', 'nag_inteq_abel1_weak', 'nag_inteq_abel2_weak',
           'nag_inteq_abel_weak_weights',
           'nag_inteq_fredholm2_smooth', 'nag_inteq_fredholm2_split',
           'nag_inteq_volterra2', 'nag_inteq_volterra_weights']
