"""
Deprecated wrappers for the NAG C Library (Mark 26.0) f01 Chapter.

f01 - Matrix Operations, Including Inversion.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/F01/f01conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER
from .util import Nag_Complex_Type

from .util import get_library
from .util import Nag_OrderType
from .util import Integer
from .util import NagError
from .util import Complex
from .util import Nag_UploType
from .util import NAG_F01EFC_F
from .util import Nag_Comm
from .util import Nag_MatFunType
from .util import NAG_F01ELC_F
from .util import NAG_F01EMC_F
from .util import NAG_F01FFC_F
from .util import NAG_F01FLC_F
from .util import NAG_F01FMC_F
from .util import NAG_F01JBC_F
from .util import NAG_F01JCC_F
from .util import NAG_F01KBC_F
from .util import NAG_F01KCC_F
from .util import Nag_RFP_Store

_library_cl = get_library()

f01ecc_ctypes = _library_cl.f01ecc
f01ecc_ctypes.restype = None
f01ecc_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f01ecc(order, n, a, pda, fail):
    """
    Real matrix exponential.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01ecc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01ecc', a, 'a')
    f01ecc_ctypes(order, n, _a, pda, fail)

f01edc_ctypes = _library_cl.f01edc
f01edc_ctypes.restype = None
f01edc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f01edc(order, uplo, n, a, pda, fail):
    """
    Real symmetric matrix exponential.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01edc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01edc', a, 'a')
    f01edc_ctypes(order, uplo, n, _a, pda, fail)

f01efc_ctypes = _library_cl.f01efc
f01efc_ctypes.restype = None
f01efc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(c_double), Integer, NAG_F01EFC_F, POINTER(Nag_Comm), POINTER(Integer), POINTER(NagError)]


def f01efc(order, uplo, n, a, pda, f, comm, flag, fail):
    """
    Function of a real symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01efc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, flag
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        flag
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01efc', a, 'a')
    _flag = nag_integer_type_check_and_cast('f01efc', flag, 'flag')
    f01efc_ctypes(order, uplo, n, _a, pda, f, comm, _flag, fail)

f01ejc_ctypes = _library_cl.f01ejc
f01ejc_ctypes.restype = None
f01ejc_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f01ejc(order, n, a, pda, imnorm, fail):
    """
    Real matrix logarithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01ejc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, imnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, imnorm
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01ejc', a, 'a')
    _imnorm = nag_double_type_check_and_cast('f01ejc', imnorm, 'imnorm')
    f01ejc_ctypes(order, n, _a, pda, _imnorm, fail)

f01ekc_ctypes = _library_cl.f01ekc
f01ekc_ctypes.restype = None
f01ekc_ctypes.argtypes = [Nag_OrderType, Nag_MatFunType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f01ekc(order, fun, n, a, pda, imnorm, fail):
    """
    Exponential, sine, cosine, sinh or cosh of a real matrix (Schur--Parlett
    algorithm).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01ekc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, imnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, imnorm
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01ekc', a, 'a')
    _imnorm = nag_double_type_check_and_cast('f01ekc', imnorm, 'imnorm')
    f01ekc_ctypes(order, fun, n, _a, pda, _imnorm, fail)

f01elc_ctypes = _library_cl.f01elc
f01elc_ctypes.restype = None
f01elc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, NAG_F01ELC_F, POINTER(Nag_Comm), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f01elc(n, a, pda, f, comm, iflag, imnorm, fail):
    """
    Function of a real matrix (using numerical differentiation).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01elc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, iflag, imnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, imnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iflag
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01elc', a, 'a')
    _iflag = nag_integer_type_check_and_cast('f01elc', iflag, 'iflag')
    _imnorm = nag_double_type_check_and_cast('f01elc', imnorm, 'imnorm')
    f01elc_ctypes(n, _a, pda, f, comm, _iflag, _imnorm, fail)

f01emc_ctypes = _library_cl.f01emc
f01emc_ctypes.restype = None
f01emc_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(c_double), Integer, NAG_F01EMC_F, POINTER(Nag_Comm), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f01emc(order, n, a, pda, f, comm, iflag, imnorm, fail):
    """
    Function of a real matrix (using user-supplied derivatives).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01emc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, iflag, imnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, imnorm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iflag
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01emc', a, 'a')
    _iflag = nag_integer_type_check_and_cast('f01emc', iflag, 'iflag')
    _imnorm = nag_double_type_check_and_cast('f01emc', imnorm, 'imnorm')
    f01emc_ctypes(order, n, _a, pda, f, comm, _iflag, _imnorm, fail)

f01enc_ctypes = _library_cl.f01enc
f01enc_ctypes.restype = None
f01enc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f01enc(n, a, tda, fail):
    """
    Real matrix square root.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01enc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01enc', a, 'a')
    f01enc_ctypes(n, _a, tda, fail)

f01epc_ctypes = _library_cl.f01epc
f01epc_ctypes.restype = None
f01epc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f01epc(n, a, tda, fail):
    """
    Real upper quasi-triangular matrix square root.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01epc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01epc', a, 'a')
    f01epc_ctypes(n, _a, tda, fail)

f01eqc_ctypes = _library_cl.f01eqc
f01eqc_ctypes.restype = None
f01eqc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, c_double, POINTER(NagError)]


def f01eqc(n, a, tda, p, fail):
    """
    General power of a real matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01eqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01eqc', a, 'a')
    f01eqc_ctypes(n, _a, tda, p, fail)

f01fcc_ctypes = _library_cl.f01fcc
f01fcc_ctypes.restype = None
f01fcc_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f01fcc(order, n, a, pda, fail):
    """
    Complex matrix exponential.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01fcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01fcc', a, 'a')
    f01fcc_ctypes(order, n, _a, pda, fail)

f01fdc_ctypes = _library_cl.f01fdc
f01fdc_ctypes.restype = None
f01fdc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f01fdc(order, uplo, n, a, pda, fail):
    """
    Complex Hermitian matrix exponential.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01fdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01fdc', a, 'a')
    f01fdc_ctypes(order, uplo, n, _a, pda, fail)

f01ffc_ctypes = _library_cl.f01ffc
f01ffc_ctypes.restype = None
f01ffc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(Complex), Integer, NAG_F01FFC_F, POINTER(Nag_Comm), POINTER(Integer), POINTER(NagError)]


def f01ffc(order, uplo, n, a, pda, f, comm, flag, fail):
    """
    Function of a complex Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01ffc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, flag
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        flag
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01ffc', a, 'a')
    _flag = nag_integer_type_check_and_cast('f01ffc', flag, 'flag')
    f01ffc_ctypes(order, uplo, n, _a, pda, f, comm, _flag, fail)

f01fjc_ctypes = _library_cl.f01fjc
f01fjc_ctypes.restype = None
f01fjc_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f01fjc(order, n, a, pda, fail):
    """
    Complex matrix logarithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01fjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01fjc', a, 'a')
    f01fjc_ctypes(order, n, _a, pda, fail)

f01fkc_ctypes = _library_cl.f01fkc
f01fkc_ctypes.restype = None
f01fkc_ctypes.argtypes = [Nag_OrderType, Nag_MatFunType, Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f01fkc(order, fun, n, a, pda, fail):
    """
    Exponential, sine, cosine, sinh or cosh of a complex matrix (Schur--
    Parlett algorithm).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01fkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01fkc', a, 'a')
    f01fkc_ctypes(order, fun, n, _a, pda, fail)

f01flc_ctypes = _library_cl.f01flc
f01flc_ctypes.restype = None
f01flc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, NAG_F01FLC_F, POINTER(Nag_Comm), POINTER(Integer), POINTER(NagError)]


def f01flc(n, a, pda, f, comm, iflag, fail):
    """
    Function of a complex matrix (using numerical differentiation).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01flc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, iflag
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iflag
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01flc', a, 'a')
    _iflag = nag_integer_type_check_and_cast('f01flc', iflag, 'iflag')
    f01flc_ctypes(n, _a, pda, f, comm, _iflag, fail)

f01fmc_ctypes = _library_cl.f01fmc
f01fmc_ctypes.restype = None
f01fmc_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(Complex), Integer, NAG_F01FMC_F, POINTER(Nag_Comm), POINTER(Integer), POINTER(NagError)]


def f01fmc(order, n, a, pda, f, comm, iflag, fail):
    """
    Function of a complex matrix (using user-supplied derivatives).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01fmc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, iflag
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iflag
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01fmc', a, 'a')
    _iflag = nag_integer_type_check_and_cast('f01fmc', iflag, 'iflag')
    f01fmc_ctypes(order, n, _a, pda, f, comm, _iflag, fail)

f01fnc_ctypes = _library_cl.f01fnc
f01fnc_ctypes.restype = None
f01fnc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f01fnc(n, a, tda, fail):
    """
    Complex matrix square root.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01fnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01fnc', a, 'a')
    f01fnc_ctypes(n, _a, tda, fail)

f01fpc_ctypes = _library_cl.f01fpc
f01fpc_ctypes.restype = None
f01fpc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f01fpc(n, a, tda, fail):
    """
    Complex upper triangular matrix square root.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01fpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01fpc', a, 'a')
    f01fpc_ctypes(n, _a, tda, fail)

f01fqc_ctypes = _library_cl.f01fqc
f01fqc_ctypes.restype = None
f01fqc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, c_double, POINTER(NagError)]


def f01fqc(n, a, tda, p, fail):
    """
    General power of a complex matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01fqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01fqc', a, 'a')
    f01fqc_ctypes(n, _a, tda, p, fail)

f01gac_ctypes = _library_cl.f01gac
f01gac_ctypes.restype = None
f01gac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(NagError)]


def f01gac(n, m, a, pda, b, pdb, t, fail):
    """
    Action of a real matrix exponential on a real matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01gac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01gac', a, 'a')
    _b = nag_double_type_check_and_cast('f01gac', b, 'b')
    f01gac_ctypes(n, m, _a, pda, _b, pdb, t, fail)

f01hac_ctypes = _library_cl.f01hac
f01hac_ctypes.restype = None
f01hac_ctypes.argtypes = [Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(NagError)]


def f01hac(n, m, a, pda, b, pdb, t, fail):
    """
    Action of a complex matrix exponential on a complex matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01hac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01hac', a, 'a')
    _b = nag_complex_type_check_and_cast('f01hac', b, 'b')
    f01hac_ctypes(n, m, _a, pda, _b, pdb, t, fail)

f01gbc_ctypes = _library_cl.f01gbc
f01gbc_ctypes.restype = None
f01gbc_ctypes.argtypes = [POINTER(Integer), Integer, Integer, POINTER(c_double), Integer, c_double, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f01gbc(irevcm, n, m, b, pdb, t, tr, b2, pdb2, x, pdx, y, pdy, p, r, z, comm, icomm, fail):
    """
    Action of a real matrix exponential on a real matrix (reverse
    communication).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01gbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, b, b2, x, y, p, r, z, comm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        b, b2, x, y, p, r, z, comm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm, icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('f01gbc', irevcm, 'irevcm')
    _b = nag_double_type_check_and_cast('f01gbc', b, 'b')
    _b2 = nag_double_type_check_and_cast('f01gbc', b2, 'b2')
    _x = nag_double_type_check_and_cast('f01gbc', x, 'x')
    _y = nag_double_type_check_and_cast('f01gbc', y, 'y')
    _p = nag_double_type_check_and_cast('f01gbc', p, 'p')
    _r = nag_double_type_check_and_cast('f01gbc', r, 'r')
    _z = nag_double_type_check_and_cast('f01gbc', z, 'z')
    _comm = nag_double_type_check_and_cast('f01gbc', comm, 'comm')
    _icomm = nag_integer_type_check_and_cast('f01gbc', icomm, 'icomm')
    f01gbc_ctypes(_irevcm, n, m, _b, pdb, t, tr, _b2, pdb2, _x, pdx, _y, pdy, _p, _r, _z, _comm, _icomm, fail)

f01hbc_ctypes = _library_cl.f01hbc
f01hbc_ctypes.restype = None
f01hbc_ctypes.argtypes = [POINTER(Integer), Integer, Integer, POINTER(Complex), Integer, Nag_Complex_Type, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), POINTER(Complex), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f01hbc(irevcm, n, m, b, pdb, t, tr, b2, pdb2, x, pdx, y, pdy, p, r, z, ccomm, comm, icomm, fail):
    """
    Action of a complex matrix exponential on a complex matrix (reverse
    communication).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01hbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        irevcm, b, b2, x, y, p, r, z, ccomm, comm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        comm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irevcm, icomm
    TypeError will be raised when the following arguments are not of data type
    Complex:
        b, b2, x, y, p, r, z, ccomm
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _irevcm = nag_integer_type_check_and_cast('f01hbc', irevcm, 'irevcm')
    _b = nag_complex_type_check_and_cast('f01hbc', b, 'b')
    _b2 = nag_complex_type_check_and_cast('f01hbc', b2, 'b2')
    _x = nag_complex_type_check_and_cast('f01hbc', x, 'x')
    _y = nag_complex_type_check_and_cast('f01hbc', y, 'y')
    _p = nag_complex_type_check_and_cast('f01hbc', p, 'p')
    _r = nag_complex_type_check_and_cast('f01hbc', r, 'r')
    _z = nag_complex_type_check_and_cast('f01hbc', z, 'z')
    _ccomm = nag_complex_type_check_and_cast('f01hbc', ccomm, 'ccomm')
    _comm = nag_double_type_check_and_cast('f01hbc', comm, 'comm')
    _icomm = nag_integer_type_check_and_cast('f01hbc', icomm, 'icomm')
    f01hbc_ctypes(_irevcm, n, m, _b, pdb, t, tr, _b2, pdb2, _x, pdx, _y, pdy, _p, _r, _z, _ccomm, _comm, _icomm, fail)

f01jac_ctypes = _library_cl.f01jac
f01jac_ctypes.restype = None
f01jac_ctypes.argtypes = [Nag_MatFunType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01jac(fun, n, a, pda, conda, norma, normfa, fail):
    """
    Condition number for the exponential, logarithm, sine, cosine, sinh or
    cosh of a real matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01jac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, conda, norma, normfa
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01jac', a, 'a')
    _conda = nag_double_type_check_and_cast('f01jac', conda, 'conda')
    _norma = nag_double_type_check_and_cast('f01jac', norma, 'norma')
    _normfa = nag_double_type_check_and_cast('f01jac', normfa, 'normfa')
    f01jac_ctypes(fun, n, _a, pda, _conda, _norma, _normfa, fail)

f01jbc_ctypes = _library_cl.f01jbc
f01jbc_ctypes.restype = None
f01jbc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, NAG_F01JBC_F, POINTER(Nag_Comm), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01jbc(n, a, pda, f, comm, iflag, conda, norma, normfa, fail):
    """
    Condition number for a function of a real matrix (using numerical
    differentiation).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01jbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, iflag, conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iflag
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01jbc', a, 'a')
    _iflag = nag_integer_type_check_and_cast('f01jbc', iflag, 'iflag')
    _conda = nag_double_type_check_and_cast('f01jbc', conda, 'conda')
    _norma = nag_double_type_check_and_cast('f01jbc', norma, 'norma')
    _normfa = nag_double_type_check_and_cast('f01jbc', normfa, 'normfa')
    f01jbc_ctypes(n, _a, pda, f, comm, _iflag, _conda, _norma, _normfa, fail)

f01jcc_ctypes = _library_cl.f01jcc
f01jcc_ctypes.restype = None
f01jcc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, NAG_F01JCC_F, POINTER(Nag_Comm), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01jcc(n, a, pda, f, comm, iflag, conda, norma, normfa, fail):
    """
    Condition number for a function of a real matrix (using user-supplied
    derivatives).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01jcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, iflag, conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iflag
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01jcc', a, 'a')
    _iflag = nag_integer_type_check_and_cast('f01jcc', iflag, 'iflag')
    _conda = nag_double_type_check_and_cast('f01jcc', conda, 'conda')
    _norma = nag_double_type_check_and_cast('f01jcc', norma, 'norma')
    _normfa = nag_double_type_check_and_cast('f01jcc', normfa, 'normfa')
    f01jcc_ctypes(n, _a, pda, f, comm, _iflag, _conda, _norma, _normfa, fail)

f01jdc_ctypes = _library_cl.f01jdc
f01jdc_ctypes.restype = None
f01jdc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01jdc(n, a, pda, alpha, condsa, fail):
    """
    Condition number for square root of real matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01jdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, alpha, condsa
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, alpha, condsa
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01jdc', a, 'a')
    _alpha = nag_double_type_check_and_cast('f01jdc', alpha, 'alpha')
    _condsa = nag_double_type_check_and_cast('f01jdc', condsa, 'condsa')
    f01jdc_ctypes(n, _a, pda, _alpha, _condsa, fail)

f01jec_ctypes = _library_cl.f01jec
f01jec_ctypes.restype = None
f01jec_ctypes.argtypes = [Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(NagError)]


def f01jec(n, a, pda, p, condpa, fail):
    """
    Condition number for real matrix power.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01jec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, condpa
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, condpa
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01jec', a, 'a')
    _condpa = nag_double_type_check_and_cast('f01jec', condpa, 'condpa')
    f01jec_ctypes(n, _a, pda, p, _condpa, fail)

f01jfc_ctypes = _library_cl.f01jfc
f01jfc_ctypes.restype = None
f01jfc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(NagError)]


def f01jfc(n, a, pda, e, pde, p, fail):
    """
    Frechet derivative of real matrix power.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01jfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, e
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, e
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01jfc', a, 'a')
    _e = nag_double_type_check_and_cast('f01jfc', e, 'e')
    f01jfc_ctypes(n, _a, pda, _e, pde, p, fail)

f01jgc_ctypes = _library_cl.f01jgc
f01jgc_ctypes.restype = None
f01jgc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f01jgc(n, a, pda, condea, fail):
    """
    Condition number for real matrix exponential.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01jgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, condea
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, condea
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01jgc', a, 'a')
    _condea = nag_double_type_check_and_cast('f01jgc', condea, 'condea')
    f01jgc_ctypes(n, _a, pda, _condea, fail)

f01jhc_ctypes = _library_cl.f01jhc
f01jhc_ctypes.restype = None
f01jhc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f01jhc(n, a, pda, e, pde, fail):
    """
    Frechet derivative of real matrix exponential.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01jhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, e
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, e
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01jhc', a, 'a')
    _e = nag_double_type_check_and_cast('f01jhc', e, 'e')
    f01jhc_ctypes(n, _a, pda, _e, pde, fail)

f01jjc_ctypes = _library_cl.f01jjc
f01jjc_ctypes.restype = None
f01jjc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f01jjc(n, a, pda, condla, fail):
    """
    Condition number for real matrix logarithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01jjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, condla
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, condla
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01jjc', a, 'a')
    _condla = nag_double_type_check_and_cast('f01jjc', condla, 'condla')
    f01jjc_ctypes(n, _a, pda, _condla, fail)

f01jkc_ctypes = _library_cl.f01jkc
f01jkc_ctypes.restype = None
f01jkc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f01jkc(n, a, pda, e, pde, fail):
    """
    Frechet derivative of real matrix logarithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01jkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, e
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, e
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01jkc', a, 'a')
    _e = nag_double_type_check_and_cast('f01jkc', e, 'e')
    f01jkc_ctypes(n, _a, pda, _e, pde, fail)

f01kac_ctypes = _library_cl.f01kac
f01kac_ctypes.restype = None
f01kac_ctypes.argtypes = [Nag_MatFunType, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01kac(fun, n, a, pda, conda, norma, normfa, fail):
    """
    Condition number for the exponential, logarithm, sine, cosine, sinh or
    cosh of a complex matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01kac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01kac', a, 'a')
    _conda = nag_double_type_check_and_cast('f01kac', conda, 'conda')
    _norma = nag_double_type_check_and_cast('f01kac', norma, 'norma')
    _normfa = nag_double_type_check_and_cast('f01kac', normfa, 'normfa')
    f01kac_ctypes(fun, n, _a, pda, _conda, _norma, _normfa, fail)

f01kbc_ctypes = _library_cl.f01kbc
f01kbc_ctypes.restype = None
f01kbc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, NAG_F01KBC_F, POINTER(Nag_Comm), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01kbc(n, a, pda, f, comm, iflag, conda, norma, normfa, fail):
    """
    Condition number for a function of a complex matrix (using numerical
    differentiation).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01kbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, iflag, conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iflag
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01kbc', a, 'a')
    _iflag = nag_integer_type_check_and_cast('f01kbc', iflag, 'iflag')
    _conda = nag_double_type_check_and_cast('f01kbc', conda, 'conda')
    _norma = nag_double_type_check_and_cast('f01kbc', norma, 'norma')
    _normfa = nag_double_type_check_and_cast('f01kbc', normfa, 'normfa')
    f01kbc_ctypes(n, _a, pda, f, comm, _iflag, _conda, _norma, _normfa, fail)

f01kcc_ctypes = _library_cl.f01kcc
f01kcc_ctypes.restype = None
f01kcc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, NAG_F01KCC_F, POINTER(Nag_Comm), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01kcc(n, a, pda, f, comm, iflag, conda, norma, normfa, fail):
    """
    Condition number for a function of a complex matrix (using user-supplied
    derivatives).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01kcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, iflag, conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        conda, norma, normfa
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iflag
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01kcc', a, 'a')
    _iflag = nag_integer_type_check_and_cast('f01kcc', iflag, 'iflag')
    _conda = nag_double_type_check_and_cast('f01kcc', conda, 'conda')
    _norma = nag_double_type_check_and_cast('f01kcc', norma, 'norma')
    _normfa = nag_double_type_check_and_cast('f01kcc', normfa, 'normfa')
    f01kcc_ctypes(n, _a, pda, f, comm, _iflag, _conda, _norma, _normfa, fail)

f01kdc_ctypes = _library_cl.f01kdc
f01kdc_ctypes.restype = None
f01kdc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01kdc(n, a, pda, alpha, condsa, fail):
    """
    Condition number for square root of complex matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01kdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, alpha, condsa
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        alpha, condsa
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01kdc', a, 'a')
    _alpha = nag_double_type_check_and_cast('f01kdc', alpha, 'alpha')
    _condsa = nag_double_type_check_and_cast('f01kdc', condsa, 'condsa')
    f01kdc_ctypes(n, _a, pda, _alpha, _condsa, fail)

f01kec_ctypes = _library_cl.f01kec
f01kec_ctypes.restype = None
f01kec_ctypes.argtypes = [Integer, POINTER(Complex), Integer, c_double, POINTER(c_double), POINTER(NagError)]


def f01kec(n, a, pda, p, condpa, fail):
    """
    Condition number for complex matrix power.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01kec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, condpa
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        condpa
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01kec', a, 'a')
    _condpa = nag_double_type_check_and_cast('f01kec', condpa, 'condpa')
    f01kec_ctypes(n, _a, pda, p, _condpa, fail)

f01kfc_ctypes = _library_cl.f01kfc
f01kfc_ctypes.restype = None
f01kfc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, c_double, POINTER(NagError)]


def f01kfc(n, a, pda, e, pde, p, fail):
    """
    Frechet derivative of complex matrix power.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01kfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, e
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01kfc', a, 'a')
    _e = nag_complex_type_check_and_cast('f01kfc', e, 'e')
    f01kfc_ctypes(n, _a, pda, _e, pde, p, fail)

f01kgc_ctypes = _library_cl.f01kgc
f01kgc_ctypes.restype = None
f01kgc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f01kgc(n, a, pda, condea, fail):
    """
    Condition number for complex matrix exponential.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01kgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, condea
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        condea
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01kgc', a, 'a')
    _condea = nag_double_type_check_and_cast('f01kgc', condea, 'condea')
    f01kgc_ctypes(n, _a, pda, _condea, fail)

f01khc_ctypes = _library_cl.f01khc
f01khc_ctypes.restype = None
f01khc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f01khc(n, a, pda, e, pde, fail):
    """
    Frechet derivative of complex matrix exponential.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01khc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, e
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01khc', a, 'a')
    _e = nag_complex_type_check_and_cast('f01khc', e, 'e')
    f01khc_ctypes(n, _a, pda, _e, pde, fail)

f01kjc_ctypes = _library_cl.f01kjc
f01kjc_ctypes.restype = None
f01kjc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f01kjc(n, a, pda, condla, fail):
    """
    Condition number for complex matrix logarithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01kjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, condla
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        condla
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01kjc', a, 'a')
    _condla = nag_double_type_check_and_cast('f01kjc', condla, 'condla')
    f01kjc_ctypes(n, _a, pda, _condla, fail)

f01kkc_ctypes = _library_cl.f01kkc
f01kkc_ctypes.restype = None
f01kkc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f01kkc(n, a, pda, e, pde, fail):
    """
    Frechet derivative of complex matrix logarithm.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01kkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, e
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, e
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01kkc', a, 'a')
    _e = nag_complex_type_check_and_cast('f01kkc', e, 'e')
    f01kkc_ctypes(n, _a, pda, _e, pde, fail)

f01mcc_ctypes = _library_cl.f01mcc
f01mcc_ctypes.restype = None
f01mcc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01mcc(n, a, lal, row, al, d, fail):
    """
    LDL^T factorization of real symmetric positive definite variable-
    bandwidth (skyline) matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01mcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, row, al, d
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, al, d
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        row
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01mcc', a, 'a')
    _row = nag_integer_type_check_and_cast('f01mcc', row, 'row')
    _al = nag_double_type_check_and_cast('f01mcc', al, 'al')
    _d = nag_double_type_check_and_cast('f01mcc', d, 'd')
    f01mcc_ctypes(n, _a, lal, _row, _al, _d, fail)

f01vac_ctypes = _library_cl.f01vac
f01vac_ctypes.restype = None
f01vac_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f01vac(order, uplo, n, a, pda, ap, fail):
    """
    Copies a real triangular matrix from full format to packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ap
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, ap
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01vac', a, 'a')
    _ap = nag_double_type_check_and_cast('f01vac', ap, 'ap')
    f01vac_ctypes(order, uplo, n, _a, pda, _ap, fail)

nag_dtrttp = f01vac

f01vbc_ctypes = _library_cl.f01vbc
f01vbc_ctypes.restype = None
f01vbc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f01vbc(order, uplo, n, a, pda, ap, fail):
    """
    Copies a complex triangular matrix from full format to packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ap
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, ap
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01vbc', a, 'a')
    _ap = nag_complex_type_check_and_cast('f01vbc', ap, 'ap')
    f01vbc_ctypes(order, uplo, n, _a, pda, _ap, fail)

nag_ztrttp = f01vbc

f01vcc_ctypes = _library_cl.f01vcc
f01vcc_ctypes.restype = None
f01vcc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f01vcc(order, uplo, n, ap, a, pda, fail):
    """
    Copies a real triangular matrix from packed format to full format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, a
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f01vcc', ap, 'ap')
    _a = nag_double_type_check_and_cast('f01vcc', a, 'a')
    f01vcc_ctypes(order, uplo, n, _ap, _a, pda, fail)

nag_dtpttr = f01vcc

f01vdc_ctypes = _library_cl.f01vdc
f01vdc_ctypes.restype = None
f01vdc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f01vdc(order, uplo, n, ap, a, pda, fail):
    """
    Copies a complex triangular matrix from packed format to full format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, a
    """
    from .util import nag_complex_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f01vdc', ap, 'ap')
    _a = nag_complex_type_check_and_cast('f01vdc', a, 'a')
    f01vdc_ctypes(order, uplo, n, _ap, _a, pda, fail)

nag_ztpttr = f01vdc

f01vec_ctypes = _library_cl.f01vec
f01vec_ctypes.restype = None
f01vec_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f01vec(order, transr, uplo, n, a, pda, ar, fail):
    """
    Copies a real triangular matrix from full format to Rectangular Full
    Packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ar
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, ar
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f01vec', a, 'a')
    _ar = nag_double_type_check_and_cast('f01vec', ar, 'ar')
    f01vec_ctypes(order, transr, uplo, n, _a, pda, _ar, fail)

nag_dtrttf = f01vec

f01vfc_ctypes = _library_cl.f01vfc
f01vfc_ctypes.restype = None
f01vfc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(NagError)]


def f01vfc(order, transr, uplo, n, a, pda, ar, fail):
    """
    Copies a complex triangular matrix from full format to Rectangular Full
    Packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, ar
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, ar
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f01vfc', a, 'a')
    _ar = nag_complex_type_check_and_cast('f01vfc', ar, 'ar')
    f01vfc_ctypes(order, transr, uplo, n, _a, pda, _ar, fail)

nag_ztrttf = f01vfc

f01vgc_ctypes = _library_cl.f01vgc
f01vgc_ctypes.restype = None
f01vgc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f01vgc(order, transr, uplo, n, ar, a, pda, fail):
    """
    Copies a real triangular matrix from Rectangular Full Packed format to
    full format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ar, a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ar, a
    """
    from .util import nag_double_type_check_and_cast
    _ar = nag_double_type_check_and_cast('f01vgc', ar, 'ar')
    _a = nag_double_type_check_and_cast('f01vgc', a, 'a')
    f01vgc_ctypes(order, transr, uplo, n, _ar, _a, pda, fail)

nag_dtfttr = f01vgc

f01vhc_ctypes = _library_cl.f01vhc
f01vhc_ctypes.restype = None
f01vhc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_UploType, Integer, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f01vhc(order, transr, uplo, n, ar, a, pda, fail):
    """
    Copies a complex triangular matrix from Rectangular Full Packed format
    to full format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ar, a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ar, a
    """
    from .util import nag_complex_type_check_and_cast
    _ar = nag_complex_type_check_and_cast('f01vhc', ar, 'ar')
    _a = nag_complex_type_check_and_cast('f01vhc', a, 'a')
    f01vhc_ctypes(order, transr, uplo, n, _ar, _a, pda, fail)

nag_ztfttr = f01vhc

f01vjc_ctypes = _library_cl.f01vjc
f01vjc_ctypes.restype = None
f01vjc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01vjc(order, transr, uplo, n, ap, ar, fail):
    """
    Copies a real triangular matrix from packed format to Rectangular Full
    Packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, ar
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, ar
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f01vjc', ap, 'ap')
    _ar = nag_double_type_check_and_cast('f01vjc', ar, 'ar')
    f01vjc_ctypes(order, transr, uplo, n, _ap, _ar, fail)

nag_dtpttf = f01vjc

f01vkc_ctypes = _library_cl.f01vkc
f01vkc_ctypes.restype = None
f01vkc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_UploType, Integer, POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f01vkc(order, transr, uplo, n, ap, ar, fail):
    """
    Copies a complex triangular matrix from packed format to Rectangular
    Full Packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, ar
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, ar
    """
    from .util import nag_complex_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f01vkc', ap, 'ap')
    _ar = nag_complex_type_check_and_cast('f01vkc', ar, 'ar')
    f01vkc_ctypes(order, transr, uplo, n, _ap, _ar, fail)

nag_ztpttf = f01vkc

f01vlc_ctypes = _library_cl.f01vlc
f01vlc_ctypes.restype = None
f01vlc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f01vlc(order, transr, uplo, n, ar, ap, fail):
    """
    Copies a real triangular matrix from Rectangular Full Packed format to
    packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ar, ap
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ar, ap
    """
    from .util import nag_double_type_check_and_cast
    _ar = nag_double_type_check_and_cast('f01vlc', ar, 'ar')
    _ap = nag_double_type_check_and_cast('f01vlc', ap, 'ap')
    f01vlc_ctypes(order, transr, uplo, n, _ar, _ap, fail)

nag_dtfttp = f01vlc

f01vmc_ctypes = _library_cl.f01vmc
f01vmc_ctypes.restype = None
f01vmc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_UploType, Integer, POINTER(Complex), POINTER(Complex), POINTER(NagError)]


def f01vmc(order, transr, uplo, n, ar, ap, fail):
    """
    Copies a complex triangular matrix from Rectangular Full Packed format
    to packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f01/f01vmc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ar, ap
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ar, ap
    """
    from .util import nag_complex_type_check_and_cast
    _ar = nag_complex_type_check_and_cast('f01vmc', ar, 'ar')
    _ap = nag_complex_type_check_and_cast('f01vmc', ap, 'ap')
    f01vmc_ctypes(order, transr, uplo, n, _ar, _ap, fail)

nag_ztfttp = f01vmc

nag_real_gen_matrix_exp = f01ecc
nag_real_symm_matrix_exp = f01edc
nag_matop_real_symm_matrix_fun = f01efc
nag_matop_real_gen_matrix_log = f01ejc
nag_matop_real_gen_matrix_fun_std = f01ekc
nag_matop_real_gen_matrix_fun_num = f01elc
nag_matop_real_gen_matrix_fun_usd = f01emc
nag_matop_real_gen_matrix_sqrt = f01enc
nag_matop_real_tri_matrix_sqrt = f01epc
nag_matop_real_gen_matrix_pow = f01eqc
nag_matop_complex_gen_matrix_exp = f01fcc
nag_matop_complex_herm_matrix_exp = f01fdc
nag_matop_complex_herm_matrix_fun = f01ffc
nag_matop_complex_gen_matrix_log = f01fjc
nag_matop_complex_gen_matrix_fun_std = f01fkc
nag_matop_complex_gen_matrix_fun_num = f01flc
nag_matop_complex_gen_matrix_fun_usd = f01fmc
nag_matop_complex_gen_matrix_sqrt = f01fnc
nag_matop_complex_tri_matrix_sqrt = f01fpc
nag_matop_complex_gen_matrix_pow = f01fqc
nag_matop_real_gen_matrix_actexp = f01gac
nag_matop_real_gen_matrix_actexp_rcomm = f01gbc
nag_matop_complex_gen_matrix_actexp = f01hac
nag_matop_complex_gen_matrix_actexp_rcomm = f01hbc
nag_matop_real_gen_matrix_cond_std = f01jac
nag_matop_real_gen_matrix_cond_num = f01jbc
nag_matop_real_gen_matrix_cond_usd = f01jcc
nag_matop_real_gen_matrix_cond_sqrt = f01jdc
nag_matop_real_gen_matrix_cond_pow = f01jec
nag_matop_real_gen_matrix_frcht_pow = f01jfc
nag_matop_real_gen_matrix_cond_exp = f01jgc
nag_matop_real_gen_matrix_frcht_exp = f01jhc
nag_matop_real_gen_matrix_cond_log = f01jjc
nag_matop_real_gen_matrix_frcht_log = f01jkc
nag_matop_complex_gen_matrix_cond_std = f01kac
nag_matop_complex_gen_matrix_cond_num = f01kbc
nag_matop_complex_gen_matrix_cond_usd = f01kcc
nag_matop_complex_gen_matrix_cond_sqrt = f01kdc
nag_matop_complex_gen_matrix_cond_pow = f01kec
nag_matop_complex_gen_matrix_frcht_pow = f01kfc
nag_matop_complex_gen_matrix_cond_exp = f01kgc
nag_matop_complex_gen_matrix_frcht_exp = f01khc
nag_matop_complex_gen_matrix_cond_log = f01kjc
nag_matop_complex_gen_matrix_frcht_log = f01kkc
nag_real_cholesky_skyline = f01mcc
__all__ = ['f01ecc', 'f01edc', 'f01efc', 'f01ejc',
           'f01ekc', 'f01elc', 'f01emc', 'f01enc', 'f01epc', 'f01eqc',
           'f01fcc', 'f01fdc', 'f01ffc', 'f01fjc', 'f01fkc', 'f01flc',
           'f01fmc', 'f01fnc', 'f01fpc', 'f01fqc', 'f01gac', 'f01gbc',
           'f01hac', 'f01hbc', 'f01jac', 'f01jbc', 'f01jcc', 'f01jdc',
           'f01jec', 'f01jfc', 'f01jgc', 'f01jhc', 'f01jjc', 'f01jkc',
           'f01kac', 'f01kbc', 'f01kcc', 'f01kdc', 'f01kec', 'f01kfc',
           'f01kgc', 'f01khc', 'f01kjc', 'f01kkc', 'f01mcc',
           'nag_matop_complex_gen_matrix_actexp',
           'nag_matop_complex_gen_matrix_actexp_rcomm',
           'nag_matop_complex_gen_matrix_cond_exp',
           'nag_matop_complex_gen_matrix_cond_log',
           'nag_matop_complex_gen_matrix_cond_num',
           'nag_matop_complex_gen_matrix_cond_pow',
           'nag_matop_complex_gen_matrix_cond_sqrt',
           'nag_matop_complex_gen_matrix_cond_std',
           'nag_matop_complex_gen_matrix_cond_usd',
           'nag_matop_complex_gen_matrix_exp',
           'nag_matop_complex_gen_matrix_frcht_exp',
           'nag_matop_complex_gen_matrix_frcht_log',
           'nag_matop_complex_gen_matrix_frcht_pow',
           'nag_matop_complex_gen_matrix_fun_num',
           'nag_matop_complex_gen_matrix_fun_std',
           'nag_matop_complex_gen_matrix_fun_usd',
           'nag_matop_complex_gen_matrix_log',
           'nag_matop_complex_gen_matrix_pow',
           'nag_matop_complex_gen_matrix_sqrt',
           'nag_matop_complex_herm_matrix_exp',
           'nag_matop_complex_herm_matrix_fun',
           'nag_matop_complex_tri_matrix_sqrt',
           'nag_matop_real_gen_matrix_actexp',
           'nag_matop_real_gen_matrix_actexp_rcomm',
           'nag_matop_real_gen_matrix_cond_exp',
           'nag_matop_real_gen_matrix_cond_log',
           'nag_matop_real_gen_matrix_cond_num',
           'nag_matop_real_gen_matrix_cond_pow',
           'nag_matop_real_gen_matrix_cond_sqrt',
           'nag_matop_real_gen_matrix_cond_std',
           'nag_matop_real_gen_matrix_cond_usd',
           'nag_matop_real_gen_matrix_frcht_exp',
           'nag_matop_real_gen_matrix_frcht_log',
           'nag_matop_real_gen_matrix_frcht_pow',
           'nag_matop_real_gen_matrix_fun_num',
           'nag_matop_real_gen_matrix_fun_std',
           'nag_matop_real_gen_matrix_fun_usd',
           'nag_matop_real_gen_matrix_log',
           'nag_matop_real_gen_matrix_pow',
           'nag_matop_real_gen_matrix_sqrt',
           'nag_matop_real_symm_matrix_fun',
           'nag_matop_real_tri_matrix_sqrt',
           'nag_real_cholesky_skyline',
           'nag_real_gen_matrix_exp',
           'nag_real_symm_matrix_exp']
