"""
Deprecated wrappers for the NAG C Library (Mark 26.0) f02 Chapter.

f02 - Eigenvalues and Eigenvectors.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/F02/f02conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER
from .util import NAG_F02FKC_MONIT
from .util import NAG_F02FKC_OPTION

from .util import get_library
from .util import Integer
from .util import NagError
from .util import Complex
from .util import Nag_Select_Eigenvalues
from .util import NAG_F02EKC_MONIT
from .util import NAG_F02EKC_OPTION
from .util import Nag_Comm
from .util import Nag_ScaleType
from .util import Nag_LeftVecsType
from .util import Nag_RightVecsType
from .util import Nag_CondErrType
from .util import Nag_OrderType
from .util import NAG_F02WGC_AV

_library_cl = get_library()


f02ecc_ctypes = _library_cl.f02ecc
f02ecc_ctypes.restype = None
f02ecc_ctypes.argtypes = [Nag_Select_Eigenvalues, Integer, POINTER(c_double), Integer, c_double, c_double, Integer, POINTER(Integer), POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f02ecc(crit, n, a, tda, wl, wu, mest, m, w, v, tdv, fail):
    """
    Computes selected eigenvalues and eigenvectors of a real general matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f02/f02ecc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, m, w, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    TypeError will be raised when the following arguments are not of data type
    Complex:
        w, v
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f02ecc', a, 'a')
    _m = nag_integer_type_check_and_cast('f02ecc', m, 'm')
    _w = nag_complex_type_check_and_cast('f02ecc', w, 'w')
    _v = nag_complex_type_check_and_cast('f02ecc', v, 'v')
    f02ecc_ctypes(crit, n, _a, tda, wl, wu, mest, _m, _w, _v, tdv, fail)

f02ekc_ctypes = _library_cl.f02ekc
f02ekc_ctypes.restype = None
f02ekc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), Integer, Integer, c_double, NAG_F02EKC_MONIT, NAG_F02EKC_OPTION, POINTER(Integer), POINTER(Complex), POINTER(c_double), Integer, POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def f02ekc(n, nnz, a, icolzp, irowix, nev, ncv, sigma, monit, option, nconv, w, v, tdv, resid, comm, fail):
    """
    Selected eigenvalues and eigenvectors of a real sparse general matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f02/f02ekc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, icolzp, irowix, nconv, w, v, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, v, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        icolzp, irowix, nconv
    TypeError will be raised when the following arguments are not of data type
    Complex:
        w
    """
    from ctypes import cast
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f02ekc', a, 'a')
    _icolzp = nag_integer_type_check_and_cast('f02ekc', icolzp, 'icolzp')
    _irowix = nag_integer_type_check_and_cast('f02ekc', irowix, 'irowix')
    if monit is None:
        monit = cast(None, NAG_F02EKC_MONIT)
    if option is None:
        option = cast(None, NAG_F02EKC_OPTION)
    _nconv = nag_integer_type_check_and_cast('f02ekc', nconv, 'nconv')
    _w = nag_complex_type_check_and_cast('f02ekc', w, 'w')
    _v = nag_double_type_check_and_cast('f02ekc', v, 'v')
    _resid = nag_double_type_check_and_cast('f02ekc', resid, 'resid')
    f02ekc_ctypes(n, nnz, _a, _icolzp, _irowix, nev, ncv, sigma, monit, option, _nconv, _w, _v, tdv, _resid, comm, fail)

f02fkc_ctypes = _library_cl.f02fkc
f02fkc_ctypes.restype = None
f02fkc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), Integer, Integer, c_double, NAG_F02FKC_MONIT, NAG_F02FKC_OPTION, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def f02fkc(n, nnz, a, irow, icol, nev, ncv, sigma, monit, option, nconv, w, v, pdv, resid, comm, fail):
    """
    Selected eigenvalues and eigenvectors of a real symmetric sparse matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f02/f02fkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, irow, icol, nconv, w, v, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, w, v, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        irow, icol, nconv
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f02fkc', a, 'a')
    _w = nag_double_type_check_and_cast('f02fkc', w, 'w')
    _v = nag_double_type_check_and_cast('f02fkc', v, 'v')
    _resid = nag_double_type_check_and_cast('f02fkc', resid, 'resid')
    _irow = nag_integer_type_check_and_cast('f02fkc', irow, 'irow')
    _icol = nag_integer_type_check_and_cast('f02fkc', icol, 'icol')
    if monit is None:
        monit = cast(None, NAG_F02FKC_MONIT)
    if option is None:
        option = cast(None, NAG_F02FKC_OPTION)
    _nconv = nag_integer_type_check_and_cast('f02fkc', nconv, 'nconv')
    f02fkc_ctypes(n, nnz, _a, _irow, _icol, nev, ncv, sigma, monit, option, _nconv, _w, _v, pdv, _resid, comm, fail)

nag_eigen_real_symm_sparse_arnoldi = f02fkc

f02gcc_ctypes = _library_cl.f02gcc
f02gcc_ctypes.restype = None
f02gcc_ctypes.argtypes = [Nag_Select_Eigenvalues, Integer, POINTER(Complex), Integer, c_double, c_double, Integer, POINTER(Integer), POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f02gcc(crit, n, a, tda, wl, wu, mest, m, w, v, tdv, fail):
    """
    Computes selected eigenvalues and eigenvectors of a complex general
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f02/f02gcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, m, w, v
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        m
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, w, v
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f02gcc', a, 'a')
    _m = nag_integer_type_check_and_cast('f02gcc', m, 'm')
    _w = nag_complex_type_check_and_cast('f02gcc', w, 'w')
    _v = nag_complex_type_check_and_cast('f02gcc', v, 'v')
    f02gcc_ctypes(crit, n, _a, tda, wl, wu, mest, _m, _w, _v, tdv, fail)

f02jcc_ctypes = _library_cl.f02jcc
f02jcc_ctypes.restype = None
f02jcc_ctypes.argtypes = [Nag_ScaleType, Nag_LeftVecsType, Nag_RightVecsType, Nag_CondErrType, c_double, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f02jcc(scale, jobvl, jobvr, sense, tol, n, a, pda, b, pdb, c, pdc, alphar, alphai, beta, vl, pdvl, vr, pdvr, s, bevl, bevr, iwarn, fail):
    """
    Solves the quadratic eigenvalue problem for real matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f02/f02jcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c, alphar, alphai, beta, vl, vr, s, bevl, bevr, iwarn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c, alphar, alphai, beta, vl, vr, s, bevl, bevr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwarn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_double_type_check_and_cast('f02jcc', a, 'a')
    _b = nag_double_type_check_and_cast('f02jcc', b, 'b')
    _c = nag_double_type_check_and_cast('f02jcc', c, 'c')
    _alphar = nag_double_type_check_and_cast('f02jcc', alphar, 'alphar')
    _alphai = nag_double_type_check_and_cast('f02jcc', alphai, 'alphai')
    _beta = nag_double_type_check_and_cast('f02jcc', beta, 'beta')
    _vl = nag_double_type_check_and_cast('f02jcc', vl, 'vl')
    _vr = nag_double_type_check_and_cast('f02jcc', vr, 'vr')
    _s = nag_double_type_check_and_cast('f02jcc', s, 's')
    _bevl = nag_double_type_check_and_cast('f02jcc', bevl, 'bevl')
    _bevr = nag_double_type_check_and_cast('f02jcc', bevr, 'bevr')
    _iwarn = nag_integer_type_check_and_cast('f02jcc', iwarn, 'iwarn')
    f02jcc_ctypes(scale, jobvl, jobvr, sense, tol, n, _a, pda, _b, pdb, _c, pdc, _alphar, _alphai, _beta, _vl, pdvl, _vr, pdvr, _s, _bevl, _bevr, _iwarn, fail)

f02jqc_ctypes = _library_cl.f02jqc
f02jqc_ctypes.restype = None
f02jqc_ctypes.argtypes = [Nag_ScaleType, Nag_LeftVecsType, Nag_RightVecsType, Nag_CondErrType, c_double, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(Complex), POINTER(Complex), POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def f02jqc(scale, jobvl, jobvr, sense, tol, n, a, pda, b, pdb, c, pdc, alpha, beta, vl, pdvl, vr, pdvr, s, bevl, bevr, iwarn, fail):
    """
    Solves the quadratic eigenvalue problem for complex matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f02/f02jqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c, alpha, beta, vl, vr, s, bevl, bevr, iwarn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        s, bevl, bevr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iwarn
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, c, alpha, beta, vl, vr
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f02jqc', a, 'a')
    _b = nag_complex_type_check_and_cast('f02jqc', b, 'b')
    _c = nag_complex_type_check_and_cast('f02jqc', c, 'c')
    _alpha = nag_complex_type_check_and_cast('f02jqc', alpha, 'alpha')
    _beta = nag_complex_type_check_and_cast('f02jqc', beta, 'beta')
    _vl = nag_complex_type_check_and_cast('f02jqc', vl, 'vl')
    _vr = nag_complex_type_check_and_cast('f02jqc', vr, 'vr')
    _s = nag_double_type_check_and_cast('f02jqc', s, 's')
    _bevl = nag_double_type_check_and_cast('f02jqc', bevl, 'bevl')
    _bevr = nag_double_type_check_and_cast('f02jqc', bevr, 'bevr')
    _iwarn = nag_integer_type_check_and_cast('f02jqc', iwarn, 'iwarn')
    f02jqc_ctypes(scale, jobvl, jobvr, sense, tol, n, _a, pda, _b, pdb, _c, pdc, _alpha, _beta, _vl, pdvl, _vr, pdvr, _s, _bevl, _bevr, _iwarn, fail)

f02wgc_ctypes = _library_cl.f02wgc
f02wgc_ctypes.restype = None
f02wgc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, Integer, NAG_F02WGC_AV, POINTER(Integer), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def f02wgc(order, m, n, k, ncv, av, nconv, sigma, u, pdu, v, pdv, resid, comm, fail):
    """
    Computes leading terms in the singular value decomposition of a real
    general matrix; also computes corresponding left and right singular
    vectors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f02/f02wgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nconv, sigma, u, v, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        sigma, u, v, resid
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nconv
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nconv = nag_integer_type_check_and_cast('f02wgc', nconv, 'nconv')
    _sigma = nag_double_type_check_and_cast('f02wgc', sigma, 'sigma')
    _u = nag_double_type_check_and_cast('f02wgc', u, 'u')
    _v = nag_double_type_check_and_cast('f02wgc', v, 'v')
    _resid = nag_double_type_check_and_cast('f02wgc', resid, 'resid')
    f02wgc_ctypes(order, m, n, k, ncv, av, _nconv, _sigma, _u, pdu, _v, pdv, _resid, comm, fail)

nag_real_eigensystem_sel = f02ecc
nag_eigen_real_gen_sparse_arnoldi = f02ekc
nag_complex_eigensystem_sel = f02gcc
nag_eigen_real_gen_quad = f02jcc
nag_eigen_complex_gen_quad = f02jqc
nag_real_partial_svd = f02wgc
__all__ = ['f02ecc', 'f02ekc',
           'f02gcc', 'f02jcc', 'f02jqc', 'f02wgc',
           'nag_complex_eigensystem_sel',
           'nag_eigen_complex_gen_quad', 'nag_eigen_real_gen_quad',
           'nag_eigen_real_gen_sparse_arnoldi',
           'nag_real_eigensystem_sel',
           'nag_real_partial_svd']
