"""
Deprecated wrappers for the NAG C Library (Mark 26.0) f16 Chapter.

f16 - NAG Interface to BLAS.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/F16/f16conts.html

NAG Copyright 2017.
"""
from ctypes import POINTER
from ctypes import c_double

from .util import get_library

from .util import Integer
from .util import NagError
from .util import Nag_ConjType
from .util import Complex
from .util import Nag_OrderType
from .util import Nag_TransType
from .util import Nag_UploType
from .util import Nag_DiagType
from .util import Nag_NormType
from .util import Nag_SideType
from .util import Nag_Complex_Type
from .util import Nag_RFP_Store

_library_cl = get_library()

f16dbc_ctypes = _library_cl.f16dbc
f16dbc_ctypes.restype = None
f16dbc_ctypes.argtypes = [Integer, Integer, POINTER(Integer), Integer, POINTER(NagError)]


def f16dbc(n, alpha, x, incx, fail):
    """
    Broadcast scalar into integer vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16dbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        x
    """
    from .util import nag_integer_type_check_and_cast
    _x = nag_integer_type_check_and_cast('f16dbc', x, 'x')
    f16dbc_ctypes(n, alpha, _x, incx, fail)

f16dlc_ctypes = _library_cl.f16dlc
f16dlc_ctypes.restype = Integer
f16dlc_ctypes.argtypes = [Integer, POINTER(Integer), Integer, POINTER(NagError)]


def f16dlc(n, x, incx, fail):
    """
    Sum elements of integer vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16dlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        x
    """
    from .util import nag_integer_type_check_and_cast
    _x = nag_integer_type_check_and_cast('f16dlc', x, 'x')
    return_value = f16dlc_ctypes(n, _x, incx, fail)
    return return_value

f16dnc_ctypes = _library_cl.f16dnc
f16dnc_ctypes.restype = None
f16dnc_ctypes.argtypes = [Integer, POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f16dnc(n, x, incx, k, i, fail):
    """
    Maximum value and location, integer vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16dnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, k, i
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        x, k, i
    """
    from .util import nag_integer_type_check_and_cast
    _x = nag_integer_type_check_and_cast('f16dnc', x, 'x')
    _k = nag_integer_type_check_and_cast('f16dnc', k, 'k')
    _i = nag_integer_type_check_and_cast('f16dnc', i, 'i')
    f16dnc_ctypes(n, _x, incx, _k, _i, fail)

f16dpc_ctypes = _library_cl.f16dpc
f16dpc_ctypes.restype = None
f16dpc_ctypes.argtypes = [Integer, POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f16dpc(n, x, incx, k, i, fail):
    """
    Minimum value and location, integer vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16dpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, k, i
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        x, k, i
    """
    from .util import nag_integer_type_check_and_cast
    _x = nag_integer_type_check_and_cast('f16dpc', x, 'x')
    _k = nag_integer_type_check_and_cast('f16dpc', k, 'k')
    _i = nag_integer_type_check_and_cast('f16dpc', i, 'i')
    f16dpc_ctypes(n, _x, incx, _k, _i, fail)

f16dqc_ctypes = _library_cl.f16dqc
f16dqc_ctypes.restype = None
f16dqc_ctypes.argtypes = [Integer, POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f16dqc(n, x, incx, k, i, fail):
    """
    Maximum absolute value and location, integer vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16dqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, k, i
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        x, k, i
    """
    from .util import nag_integer_type_check_and_cast
    _x = nag_integer_type_check_and_cast('f16dqc', x, 'x')
    _k = nag_integer_type_check_and_cast('f16dqc', k, 'k')
    _i = nag_integer_type_check_and_cast('f16dqc', i, 'i')
    f16dqc_ctypes(n, _x, incx, _k, _i, fail)

f16drc_ctypes = _library_cl.f16drc
f16drc_ctypes.restype = None
f16drc_ctypes.argtypes = [Integer, POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def f16drc(n, x, incx, k, i, fail):
    """
    Minimum absolute value and location, integer vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16drc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, k, i
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        x, k, i
    """
    from .util import nag_integer_type_check_and_cast
    _x = nag_integer_type_check_and_cast('f16drc', x, 'x')
    _k = nag_integer_type_check_and_cast('f16drc', k, 'k')
    _i = nag_integer_type_check_and_cast('f16drc', i, 'i')
    f16drc_ctypes(n, _x, incx, _k, _i, fail)

f16eac_ctypes = _library_cl.f16eac
f16eac_ctypes.restype = None
f16eac_ctypes.argtypes = [Nag_ConjType, Integer, c_double, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f16eac(conj, n, alpha, x, incx, beta, y, incy, r, fail):
    """
    Dot product of two vectors, allows scaling and accumulation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16eac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, r
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16eac', x, 'x')
    _y = nag_double_type_check_and_cast('f16eac', y, 'y')
    _r = nag_double_type_check_and_cast('f16eac', r, 'r')
    f16eac_ctypes(conj, n, alpha, _x, incx, beta, _y, incy, _r, fail)

f16ecc_ctypes = _library_cl.f16ecc
f16ecc_ctypes.restype = None
f16ecc_ctypes.argtypes = [Integer, c_double, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16ecc(n, alpha, x, incx, beta, y, incy, fail):
    """
    Real weighted vector addition.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ecc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16ecc', x, 'x')
    _y = nag_double_type_check_and_cast('f16ecc', y, 'y')
    f16ecc_ctypes(n, alpha, _x, incx, beta, _y, incy, fail)

f16ehc_ctypes = _library_cl.f16ehc
f16ehc_ctypes.restype = None
f16ehc_ctypes.argtypes = [Integer, c_double, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f16ehc(n, alpha, x, incx, beta, y, incy, w, incw, fail):
    """
    Real weighted vector addition preserving input.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ehc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, w
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, w
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16ehc', x, 'x')
    _y = nag_double_type_check_and_cast('f16ehc', y, 'y')
    _w = nag_double_type_check_and_cast('f16ehc', w, 'w')
    f16ehc_ctypes(n, alpha, _x, incx, beta, _y, incy, _w, incw, fail)

f16elc_ctypes = _library_cl.f16elc
f16elc_ctypes.restype = c_double
f16elc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f16elc(n, x, incx, fail):
    """
    Sum elements of real vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16elc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16elc', x, 'x')
    return_value = f16elc_ctypes(n, _x, incx, fail)
    return return_value

f16fbc_ctypes = _library_cl.f16fbc
f16fbc_ctypes.restype = None
f16fbc_ctypes.argtypes = [Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16fbc(n, alpha, x, incx, fail):
    """
    Broadcast scalar into real vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16fbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16fbc', x, 'x')
    f16fbc_ctypes(n, alpha, _x, incx, fail)

f16gcc_ctypes = _library_cl.f16gcc
f16gcc_ctypes.restype = None
f16gcc_ctypes.argtypes = [Integer, Nag_Complex_Type, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16gcc(n, alpha, x, incx, beta, y, incy, fail):
    """
    Complex weighted vector addition.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16gcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x, y
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16gcc', x, 'x')
    _y = nag_complex_type_check_and_cast('f16gcc', y, 'y')
    f16gcc_ctypes(n, alpha, _x, incx, beta, _y, incy, fail)

f16ghc_ctypes = _library_cl.f16ghc
f16ghc_ctypes.restype = None
f16ghc_ctypes.argtypes = [Integer, Nag_Complex_Type, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f16ghc(n, alpha, x, incx, beta, y, incy, w, incw, fail):
    """
    Complex weighted vector addition preserving input.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ghc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, w
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x, y, w
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16ghc', x, 'x')
    _y = nag_complex_type_check_and_cast('f16ghc', y, 'y')
    _w = nag_complex_type_check_and_cast('f16ghc', w, 'w')
    f16ghc_ctypes(n, alpha, _x, incx, beta, _y, incy, _w, incw, fail)

f16glc_ctypes = _library_cl.f16glc
f16glc_ctypes.restype = Complex
f16glc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f16glc(n, x, incx, fail):
    """
    Sum elements of complex vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16glc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16glc', x, 'x')
    return_value = f16glc_ctypes(n, _x, incx, fail)
    return return_value

f16hbc_ctypes = _library_cl.f16hbc
f16hbc_ctypes.restype = None
f16hbc_ctypes.argtypes = [Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16hbc(n, alpha, x, incx, fail):
    """
    Broadcast scalar into complex vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16hbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16hbc', x, 'x')
    f16hbc_ctypes(n, alpha, _x, incx, fail)

f16jnc_ctypes = _library_cl.f16jnc
f16jnc_ctypes.restype = None
f16jnc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f16jnc(n, x, incx, k, r, fail):
    """
    Maximum value and location, real vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16jnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, k, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, r
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16jnc', x, 'x')
    _k = nag_integer_type_check_and_cast('f16jnc', k, 'k')
    _r = nag_double_type_check_and_cast('f16jnc', r, 'r')
    f16jnc_ctypes(n, _x, incx, _k, _r, fail)

f16jpc_ctypes = _library_cl.f16jpc
f16jpc_ctypes.restype = None
f16jpc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f16jpc(n, x, incx, k, r, fail):
    """
    Minimum value and location, real vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16jpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, k, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, r
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16jpc', x, 'x')
    _k = nag_integer_type_check_and_cast('f16jpc', k, 'k')
    _r = nag_double_type_check_and_cast('f16jpc', r, 'r')
    f16jpc_ctypes(n, _x, incx, _k, _r, fail)

f16jqc_ctypes = _library_cl.f16jqc
f16jqc_ctypes.restype = None
f16jqc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f16jqc(n, x, incx, k, r, fail):
    """
    Maximum absolute value and location, real vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16jqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, k, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, r
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16jqc', x, 'x')
    _k = nag_integer_type_check_and_cast('f16jqc', k, 'k')
    _r = nag_double_type_check_and_cast('f16jqc', r, 'r')
    f16jqc_ctypes(n, _x, incx, _k, _r, fail)

f16jrc_ctypes = _library_cl.f16jrc
f16jrc_ctypes.restype = None
f16jrc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f16jrc(n, x, incx, k, r, fail):
    """
    Minimum absolute value and location, real vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16jrc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, k, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, r
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16jrc', x, 'x')
    _k = nag_integer_type_check_and_cast('f16jrc', k, 'k')
    _r = nag_double_type_check_and_cast('f16jrc', r, 'r')
    f16jrc_ctypes(n, _x, incx, _k, _r, fail)

f16jsc_ctypes = _library_cl.f16jsc
f16jsc_ctypes.restype = None
f16jsc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f16jsc(n, x, incx, k, r, fail):
    """
    Maximum absolute value and location, complex vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16jsc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, k, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16jsc', x, 'x')
    _k = nag_integer_type_check_and_cast('f16jsc', k, 'k')
    _r = nag_double_type_check_and_cast('f16jsc', r, 'r')
    f16jsc_ctypes(n, _x, incx, _k, _r, fail)

f16jtc_ctypes = _library_cl.f16jtc
f16jtc_ctypes.restype = None
f16jtc_ctypes.argtypes = [Integer, POINTER(Complex), Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def f16jtc(n, x, incx, k, r, fail):
    """
    Minimum absolute value and location, complex vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16jtc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, k, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16jtc', x, 'x')
    _k = nag_integer_type_check_and_cast('f16jtc', k, 'k')
    _r = nag_double_type_check_and_cast('f16jtc', r, 'r')
    f16jtc_ctypes(n, _x, incx, _k, _r, fail)

f16pac_ctypes = _library_cl.f16pac
f16pac_ctypes.restype = None
f16pac_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16pac(order, trans, m, n, alpha, a, pda, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, real rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, x, y
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16pac', a, 'a')
    _x = nag_double_type_check_and_cast('f16pac', x, 'x')
    _y = nag_double_type_check_and_cast('f16pac', y, 'y')
    f16pac_ctypes(order, trans, m, n, alpha, _a, pda, _x, incx, beta, _y, incy, fail)

f16pbc_ctypes = _library_cl.f16pbc
f16pbc_ctypes.restype = None
f16pbc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16pbc(order, trans, m, n, kl, ku, alpha, ab, pdab, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, real rectangular band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, x, y
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f16pbc', ab, 'ab')
    _x = nag_double_type_check_and_cast('f16pbc', x, 'x')
    _y = nag_double_type_check_and_cast('f16pbc', y, 'y')
    f16pbc_ctypes(order, trans, m, n, kl, ku, alpha, _ab, pdab, _x, incx, beta, _y, incy, fail)

f16pcc_ctypes = _library_cl.f16pcc
f16pcc_ctypes.restype = None
f16pcc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16pcc(order, uplo, n, alpha, a, pda, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, real symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, x, y
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16pcc', a, 'a')
    _x = nag_double_type_check_and_cast('f16pcc', x, 'x')
    _y = nag_double_type_check_and_cast('f16pcc', y, 'y')
    f16pcc_ctypes(order, uplo, n, alpha, _a, pda, _x, incx, beta, _y, incy, fail)

f16pdc_ctypes = _library_cl.f16pdc
f16pdc_ctypes.restype = None
f16pdc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16pdc(order, uplo, n, k, alpha, ab, pdab, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, real symmetric band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, x, y
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f16pdc', ab, 'ab')
    _x = nag_double_type_check_and_cast('f16pdc', x, 'x')
    _y = nag_double_type_check_and_cast('f16pdc', y, 'y')
    f16pdc_ctypes(order, uplo, n, k, alpha, _ab, pdab, _x, incx, beta, _y, incy, fail)

f16pec_ctypes = _library_cl.f16pec
f16pec_ctypes.restype = None
f16pec_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, c_double, POINTER(c_double), POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16pec(order, uplo, n, alpha, ap, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, real symmetric packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, x, y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, x, y
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f16pec', ap, 'ap')
    _x = nag_double_type_check_and_cast('f16pec', x, 'x')
    _y = nag_double_type_check_and_cast('f16pec', y, 'y')
    f16pec_ctypes(order, uplo, n, alpha, _ap, _x, incx, beta, _y, incy, fail)

f16pfc_ctypes = _library_cl.f16pfc
f16pfc_ctypes.restype = None
f16pfc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f16pfc(order, uplo, trans, diag, n, alpha, a, pda, x, incx, fail):
    """
    Matrix-vector product, real triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, x
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16pfc', a, 'a')
    _x = nag_double_type_check_and_cast('f16pfc', x, 'x')
    f16pfc_ctypes(order, uplo, trans, diag, n, alpha, _a, pda, _x, incx, fail)

f16pgc_ctypes = _library_cl.f16pgc
f16pgc_ctypes.restype = None
f16pgc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f16pgc(order, uplo, trans, diag, n, k, alpha, ab, pdab, x, incx, fail):
    """
    Matrix-vector product, real triangular band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, x
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f16pgc', ab, 'ab')
    _x = nag_double_type_check_and_cast('f16pgc', x, 'x')
    f16pgc_ctypes(order, uplo, trans, diag, n, k, alpha, _ab, pdab, _x, incx, fail)

f16phc_ctypes = _library_cl.f16phc
f16phc_ctypes.restype = None
f16phc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, c_double, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f16phc(order, uplo, trans, diag, n, alpha, ap, x, incx, fail):
    """
    Matrix-vector product, real triangular packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16phc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, x
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f16phc', ap, 'ap')
    _x = nag_double_type_check_and_cast('f16phc', x, 'x')
    f16phc_ctypes(order, uplo, trans, diag, n, alpha, _ap, _x, incx, fail)

f16pjc_ctypes = _library_cl.f16pjc
f16pjc_ctypes.restype = None
f16pjc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f16pjc(order, uplo, trans, diag, n, alpha, a, pda, x, incx, fail):
    """
    System of equations, real triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, x
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16pjc', a, 'a')
    _x = nag_double_type_check_and_cast('f16pjc', x, 'x')
    f16pjc_ctypes(order, uplo, trans, diag, n, alpha, _a, pda, _x, incx, fail)

f16pkc_ctypes = _library_cl.f16pkc
f16pkc_ctypes.restype = None
f16pkc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f16pkc(order, uplo, trans, diag, n, k, alpha, ab, pdab, x, incx, fail):
    """
    System of equations, real triangular band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, x
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f16pkc', ab, 'ab')
    _x = nag_double_type_check_and_cast('f16pkc', x, 'x')
    f16pkc_ctypes(order, uplo, trans, diag, n, k, alpha, _ab, pdab, _x, incx, fail)

f16plc_ctypes = _library_cl.f16plc
f16plc_ctypes.restype = None
f16plc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, c_double, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f16plc(order, uplo, trans, diag, n, alpha, ap, x, incx, fail):
    """
    System of equations, real triangular packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16plc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, x
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f16plc', ap, 'ap')
    _x = nag_double_type_check_and_cast('f16plc', x, 'x')
    f16plc_ctypes(order, uplo, trans, diag, n, alpha, _ap, _x, incx, fail)

f16pmc_ctypes = _library_cl.f16pmc
f16pmc_ctypes.restype = None
f16pmc_ctypes.argtypes = [Nag_OrderType, Nag_ConjType, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16pmc(order, conj, m, n, alpha, x, incx, y, incy, beta, a, pda, fail):
    """
    Rank-1 update, real rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pmc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, a
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16pmc', x, 'x')
    _y = nag_double_type_check_and_cast('f16pmc', y, 'y')
    _a = nag_double_type_check_and_cast('f16pmc', a, 'a')
    f16pmc_ctypes(order, conj, m, n, alpha, _x, incx, _y, incy, beta, _a, pda, fail)

f16ppc_ctypes = _library_cl.f16ppc
f16ppc_ctypes.restype = None
f16ppc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, c_double, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16ppc(order, uplo, n, alpha, x, incx, beta, a, pda, fail):
    """
    Rank-1 update, real symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ppc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, a
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16ppc', x, 'x')
    _a = nag_double_type_check_and_cast('f16ppc', a, 'a')
    f16ppc_ctypes(order, uplo, n, alpha, _x, incx, beta, _a, pda, fail)

f16pqc_ctypes = _library_cl.f16pqc
f16pqc_ctypes.restype = None
f16pqc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, c_double, POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(NagError)]


def f16pqc(order, uplo, n, alpha, x, incx, beta, ap, fail):
    """
    Rank-1 update, real symmetric packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16pqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, ap
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, ap
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16pqc', x, 'x')
    _ap = nag_double_type_check_and_cast('f16pqc', ap, 'ap')
    f16pqc_ctypes(order, uplo, n, alpha, _x, incx, beta, _ap, fail)

f16prc_ctypes = _library_cl.f16prc
f16prc_ctypes.restype = None
f16prc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16prc(order, uplo, n, alpha, x, incx, y, incy, beta, a, pda, fail):
    """
    Rank-2 update, real symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16prc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, a
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16prc', x, 'x')
    _y = nag_double_type_check_and_cast('f16prc', y, 'y')
    _a = nag_double_type_check_and_cast('f16prc', a, 'a')
    f16prc_ctypes(order, uplo, n, alpha, _x, incx, _y, incy, beta, _a, pda, fail)

f16psc_ctypes = _library_cl.f16psc
f16psc_ctypes.restype = None
f16psc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(NagError)]


def f16psc(order, uplo, n, alpha, x, incx, y, incy, beta, ap, fail):
    """
    Rank-2 update, real symmetric packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16psc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, ap
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, ap
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('f16psc', x, 'x')
    _y = nag_double_type_check_and_cast('f16psc', y, 'y')
    _ap = nag_double_type_check_and_cast('f16psc', ap, 'ap')
    f16psc_ctypes(order, uplo, n, alpha, _x, incx, _y, incy, beta, _ap, fail)

f16qec_ctypes = _library_cl.f16qec
f16qec_ctypes.restype = None
f16qec_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f16qec(order, uplo, trans, diag, n, a, pda, b, pdb, fail):
    """
    Matrix copy, real triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16qec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16qec', a, 'a')
    _b = nag_double_type_check_and_cast('f16qec', b, 'b')
    f16qec_ctypes(order, uplo, trans, diag, n, _a, pda, _b, pdb, fail)

f16qfc_ctypes = _library_cl.f16qfc
f16qfc_ctypes.restype = None
f16qfc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f16qfc(order, trans, m, n, a, pda, b, pdb, fail):
    """
    Matrix copy, real rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16qfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16qfc', a, 'a')
    _b = nag_double_type_check_and_cast('f16qfc', b, 'b')
    f16qfc_ctypes(order, trans, m, n, _a, pda, _b, pdb, fail)

f16qgc_ctypes = _library_cl.f16qgc
f16qgc_ctypes.restype = None
f16qgc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, c_double, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16qgc(order, uplo, n, alpha, diag, a, pda, fail):
    """
    Matrix initialization, real triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16qgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16qgc', a, 'a')
    f16qgc_ctypes(order, uplo, n, alpha, diag, _a, pda, fail)

f16qhc_ctypes = _library_cl.f16qhc
f16qhc_ctypes.restype = None
f16qhc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, c_double, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16qhc(order, m, n, alpha, diag, a, pda, fail):
    """
    Matrix initialization, real rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16qhc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16qhc', a, 'a')
    f16qhc_ctypes(order, m, n, alpha, diag, _a, pda, fail)

f16rac_ctypes = _library_cl.f16rac
f16rac_ctypes.restype = None
f16rac_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f16rac(order, norm, m, n, a, pda, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, real
    general matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16rac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, r
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16rac', a, 'a')
    _r = nag_double_type_check_and_cast('f16rac', r, 'r')
    f16rac_ctypes(order, norm, m, n, _a, pda, _r, fail)

f16rbc_ctypes = _library_cl.f16rbc
f16rbc_ctypes.restype = None
f16rbc_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f16rbc(order, norm, m, n, kl, ku, ab, pdab, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, real
    band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16rbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, r
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f16rbc', ab, 'ab')
    _r = nag_double_type_check_and_cast('f16rbc', r, 'r')
    f16rbc_ctypes(order, norm, m, n, kl, ku, _ab, pdab, _r, fail)

f16rcc_ctypes = _library_cl.f16rcc
f16rcc_ctypes.restype = None
f16rcc_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Nag_UploType, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f16rcc(order, norm, uplo, n, a, pda, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, real
    symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16rcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, r
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16rcc', a, 'a')
    _r = nag_double_type_check_and_cast('f16rcc', r, 'r')
    f16rcc_ctypes(order, norm, uplo, n, _a, pda, _r, fail)

f16rdc_ctypes = _library_cl.f16rdc
f16rdc_ctypes.restype = None
f16rdc_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f16rdc(order, norm, uplo, n, ap, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, real
    symmetric matrix, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16rdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ap, r
    """
    from .util import nag_double_type_check_and_cast
    _ap = nag_double_type_check_and_cast('f16rdc', ap, 'ap')
    _r = nag_double_type_check_and_cast('f16rdc', r, 'r')
    f16rdc_ctypes(order, norm, uplo, n, _ap, _r, fail)

f16rec_ctypes = _library_cl.f16rec
f16rec_ctypes.restype = None
f16rec_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Nag_UploType, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def f16rec(order, norm, uplo, n, k, ab, pdab, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, real
    symmetric band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16rec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ab, r
    """
    from .util import nag_double_type_check_and_cast
    _ab = nag_double_type_check_and_cast('f16rec', ab, 'ab')
    _r = nag_double_type_check_and_cast('f16rec', r, 'r')
    f16rec_ctypes(order, norm, uplo, n, k, _ab, pdab, _r, fail)

f16rkc_ctypes = _library_cl.f16rkc
f16rkc_ctypes.restype = None
f16rkc_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Nag_RFP_Store, Nag_UploType, Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def f16rkc(order, norm, transr, uplo, n, ar, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, real
    symmetric matrix, Rectangular Full Packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16rkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ar, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ar, r
    """
    from .util import nag_double_type_check_and_cast
    _ar = nag_double_type_check_and_cast('f16rkc', ar, 'ar')
    _r = nag_double_type_check_and_cast('f16rkc', r, 'r')
    f16rkc_ctypes(order, norm, transr, uplo, n, _ar, _r, fail)

nag_dsf_norm = f16rkc

f16sac_ctypes = _library_cl.f16sac
f16sac_ctypes.restype = None
f16sac_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16sac(order, trans, m, n, alpha, a, pda, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, complex rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16sac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, x, y
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, x, y
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16sac', a, 'a')
    _x = nag_complex_type_check_and_cast('f16sac', x, 'x')
    _y = nag_complex_type_check_and_cast('f16sac', y, 'y')
    f16sac_ctypes(order, trans, m, n, alpha, _a, pda, _x, incx, beta, _y, incy, fail)

f16sbc_ctypes = _library_cl.f16sbc
f16sbc_ctypes.restype = None
f16sbc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16sbc(order, trans, m, n, kl, ku, alpha, ab, pdab, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, complex rectangular band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16sbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, x, y
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, x, y
    """
    from .util import nag_complex_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f16sbc', ab, 'ab')
    _x = nag_complex_type_check_and_cast('f16sbc', x, 'x')
    _y = nag_complex_type_check_and_cast('f16sbc', y, 'y')
    f16sbc_ctypes(order, trans, m, n, kl, ku, alpha, _ab, pdab, _x, incx, beta, _y, incy, fail)

f16scc_ctypes = _library_cl.f16scc
f16scc_ctypes.restype = None
f16scc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16scc(order, uplo, n, alpha, a, pda, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, complex Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16scc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, x, y
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, x, y
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16scc', a, 'a')
    _x = nag_complex_type_check_and_cast('f16scc', x, 'x')
    _y = nag_complex_type_check_and_cast('f16scc', y, 'y')
    f16scc_ctypes(order, uplo, n, alpha, _a, pda, _x, incx, beta, _y, incy, fail)

f16sdc_ctypes = _library_cl.f16sdc
f16sdc_ctypes.restype = None
f16sdc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16sdc(order, uplo, n, k, alpha, ab, pdab, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, complex Hermitian band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16sdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, x, y
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, x, y
    """
    from .util import nag_complex_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f16sdc', ab, 'ab')
    _x = nag_complex_type_check_and_cast('f16sdc', x, 'x')
    _y = nag_complex_type_check_and_cast('f16sdc', y, 'y')
    f16sdc_ctypes(order, uplo, n, k, alpha, _ab, pdab, _x, incx, beta, _y, incy, fail)

f16sec_ctypes = _library_cl.f16sec
f16sec_ctypes.restype = None
f16sec_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Nag_Complex_Type, POINTER(Complex), POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16sec(order, uplo, n, alpha, ap, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, complex Hermitian packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16sec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, x, y
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, x, y
    """
    from .util import nag_complex_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f16sec', ap, 'ap')
    _x = nag_complex_type_check_and_cast('f16sec', x, 'x')
    _y = nag_complex_type_check_and_cast('f16sec', y, 'y')
    f16sec_ctypes(order, uplo, n, alpha, _ap, _x, incx, beta, _y, incy, fail)

f16sfc_ctypes = _library_cl.f16sfc
f16sfc_ctypes.restype = None
f16sfc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f16sfc(order, uplo, trans, diag, n, alpha, a, pda, x, incx, fail):
    """
    Matrix-vector product, complex triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16sfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, x
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16sfc', a, 'a')
    _x = nag_complex_type_check_and_cast('f16sfc', x, 'x')
    f16sfc_ctypes(order, uplo, trans, diag, n, alpha, _a, pda, _x, incx, fail)

f16sgc_ctypes = _library_cl.f16sgc
f16sgc_ctypes.restype = None
f16sgc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f16sgc(order, uplo, trans, diag, n, k, alpha, ab, pdab, x, incx, fail):
    """
    Matrix-vector product, complex triangular band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16sgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, x
    """
    from .util import nag_complex_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f16sgc', ab, 'ab')
    _x = nag_complex_type_check_and_cast('f16sgc', x, 'x')
    f16sgc_ctypes(order, uplo, trans, diag, n, k, alpha, _ab, pdab, _x, incx, fail)

f16shc_ctypes = _library_cl.f16shc
f16shc_ctypes.restype = None
f16shc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Nag_Complex_Type, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f16shc(order, uplo, trans, diag, n, alpha, ap, x, incx, fail):
    """
    Matrix-vector product, complex triangular packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16shc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, x
    """
    from .util import nag_complex_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f16shc', ap, 'ap')
    _x = nag_complex_type_check_and_cast('f16shc', x, 'x')
    f16shc_ctypes(order, uplo, trans, diag, n, alpha, _ap, _x, incx, fail)

f16sjc_ctypes = _library_cl.f16sjc
f16sjc_ctypes.restype = None
f16sjc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f16sjc(order, uplo, trans, diag, n, alpha, a, pda, x, incx, fail):
    """
    System of equations, complex triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16sjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, x
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16sjc', a, 'a')
    _x = nag_complex_type_check_and_cast('f16sjc', x, 'x')
    f16sjc_ctypes(order, uplo, trans, diag, n, alpha, _a, pda, _x, incx, fail)

f16skc_ctypes = _library_cl.f16skc
f16skc_ctypes.restype = None
f16skc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f16skc(order, uplo, trans, diag, n, k, alpha, ab, pdab, x, incx, fail):
    """
    System of equations, complex triangular band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16skc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab, x
    """
    from .util import nag_complex_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f16skc', ab, 'ab')
    _x = nag_complex_type_check_and_cast('f16skc', x, 'x')
    f16skc_ctypes(order, uplo, trans, diag, n, k, alpha, _ab, pdab, _x, incx, fail)

f16slc_ctypes = _library_cl.f16slc
f16slc_ctypes.restype = None
f16slc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Nag_Complex_Type, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f16slc(order, uplo, trans, diag, n, alpha, ap, x, incx, fail):
    """
    System of equations, complex triangular packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16slc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, x
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, x
    """
    from .util import nag_complex_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f16slc', ap, 'ap')
    _x = nag_complex_type_check_and_cast('f16slc', x, 'x')
    f16slc_ctypes(order, uplo, trans, diag, n, alpha, _ap, _x, incx, fail)

f16smc_ctypes = _library_cl.f16smc
f16smc_ctypes.restype = None
f16smc_ctypes.argtypes = [Nag_OrderType, Nag_ConjType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16smc(order, conj, m, n, alpha, x, incx, y, incy, beta, a, pda, fail):
    """
    Rank-1 update, complex rectangular matrix, unconjugated vector.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16smc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x, y, a
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16smc', x, 'x')
    _y = nag_complex_type_check_and_cast('f16smc', y, 'y')
    _a = nag_complex_type_check_and_cast('f16smc', a, 'a')
    f16smc_ctypes(order, conj, m, n, alpha, _x, incx, _y, incy, beta, _a, pda, fail)

f16spc_ctypes = _library_cl.f16spc
f16spc_ctypes.restype = None
f16spc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, c_double, POINTER(Complex), Integer, c_double, POINTER(Complex), Integer, POINTER(NagError)]


def f16spc(order, uplo, n, alpha, x, incx, beta, a, pda, fail):
    """
    Rank-1 update, complex Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16spc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x, a
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16spc', x, 'x')
    _a = nag_complex_type_check_and_cast('f16spc', a, 'a')
    f16spc_ctypes(order, uplo, n, alpha, _x, incx, beta, _a, pda, fail)

f16sqc_ctypes = _library_cl.f16sqc
f16sqc_ctypes.restype = None
f16sqc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, c_double, POINTER(Complex), Integer, c_double, POINTER(Complex), POINTER(NagError)]


def f16sqc(order, uplo, n, alpha, x, incx, beta, ap, fail):
    """
    Rank-1 update, complex Hermitian packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16sqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, ap
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x, ap
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16sqc', x, 'x')
    _ap = nag_complex_type_check_and_cast('f16sqc', ap, 'ap')
    f16sqc_ctypes(order, uplo, n, alpha, _x, incx, beta, _ap, fail)

f16src_ctypes = _library_cl.f16src
f16src_ctypes.restype = None
f16src_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, c_double, POINTER(Complex), Integer, POINTER(NagError)]


def f16src(order, uplo, n, alpha, x, incx, y, incy, beta, a, pda, fail):
    """
    Rank-2 update, complex Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16src.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x, y, a
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16src', x, 'x')
    _y = nag_complex_type_check_and_cast('f16src', y, 'y')
    _a = nag_complex_type_check_and_cast('f16src', a, 'a')
    f16src_ctypes(order, uplo, n, alpha, _x, incx, _y, incy, beta, _a, pda, fail)

f16ssc_ctypes = _library_cl.f16ssc
f16ssc_ctypes.restype = None
f16ssc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, c_double, POINTER(Complex), POINTER(NagError)]


def f16ssc(order, uplo, n, alpha, x, incx, y, incy, beta, ap, fail):
    """
    Rank-2 update, complex Hermitian packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ssc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, ap
    TypeError will be raised when the following arguments are not of data type
    Complex:
        x, y, ap
    """
    from .util import nag_complex_type_check_and_cast
    _x = nag_complex_type_check_and_cast('f16ssc', x, 'x')
    _y = nag_complex_type_check_and_cast('f16ssc', y, 'y')
    _ap = nag_complex_type_check_and_cast('f16ssc', ap, 'ap')
    f16ssc_ctypes(order, uplo, n, alpha, _x, incx, _y, incy, beta, _ap, fail)

f16tac_ctypes = _library_cl.f16tac
f16tac_ctypes.restype = None
f16tac_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16tac(order, uplo, n, alpha, a, pda, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, complex symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16tac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, x, y
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, x, y
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16tac', a, 'a')
    _x = nag_complex_type_check_and_cast('f16tac', x, 'x')
    _y = nag_complex_type_check_and_cast('f16tac', y, 'y')
    f16tac_ctypes(order, uplo, n, alpha, _a, pda, _x, incx, beta, _y, incy, fail)

f16tcc_ctypes = _library_cl.f16tcc
f16tcc_ctypes.restype = None
f16tcc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Nag_Complex_Type, POINTER(Complex), POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16tcc(order, uplo, n, alpha, ap, x, incx, beta, y, incy, fail):
    """
    Matrix-vector product, complex symmetric packed matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16tcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, x, y
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap, x, y
    """
    from .util import nag_complex_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f16tcc', ap, 'ap')
    _x = nag_complex_type_check_and_cast('f16tcc', x, 'x')
    _y = nag_complex_type_check_and_cast('f16tcc', y, 'y')
    f16tcc_ctypes(order, uplo, n, alpha, _ap, _x, incx, beta, _y, incy, fail)

f16tec_ctypes = _library_cl.f16tec
f16tec_ctypes.restype = None
f16tec_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f16tec(order, uplo, trans, diag, n, a, pda, b, pdb, fail):
    """
    Matrix copy, complex triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16tec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16tec', a, 'a')
    _b = nag_complex_type_check_and_cast('f16tec', b, 'b')
    f16tec_ctypes(order, uplo, trans, diag, n, _a, pda, _b, pdb, fail)

f16tfc_ctypes = _library_cl.f16tfc
f16tfc_ctypes.restype = None
f16tfc_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Integer, Integer, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f16tfc(order, trans, m, n, a, pda, b, pdb, fail):
    """
    Matrix copy, complex rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16tfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16tfc', a, 'a')
    _b = nag_complex_type_check_and_cast('f16tfc', b, 'b')
    f16tfc_ctypes(order, trans, m, n, _a, pda, _b, pdb, fail)

f16tgc_ctypes = _library_cl.f16tgc
f16tgc_ctypes.restype = None
f16tgc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Integer, Nag_Complex_Type, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16tgc(order, uplo, n, alpha, diag, a, pda, fail):
    """
    Matrix initialization, complex triangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16tgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16tgc', a, 'a')
    f16tgc_ctypes(order, uplo, n, alpha, diag, _a, pda, fail)

f16thc_ctypes = _library_cl.f16thc
f16thc_ctypes.restype = None
f16thc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Nag_Complex_Type, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16thc(order, m, n, alpha, diag, a, pda, fail):
    """
    Matrix initialization, complex rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16thc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16thc', a, 'a')
    f16thc_ctypes(order, m, n, alpha, diag, _a, pda, fail)

f16uac_ctypes = _library_cl.f16uac
f16uac_ctypes.restype = None
f16uac_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Integer, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f16uac(order, norm, m, n, a, pda, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, complex
    general matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16uac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16uac', a, 'a')
    _r = nag_double_type_check_and_cast('f16uac', r, 'r')
    f16uac_ctypes(order, norm, m, n, _a, pda, _r, fail)

f16ubc_ctypes = _library_cl.f16ubc
f16ubc_ctypes.restype = None
f16ubc_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f16ubc(order, norm, m, n, kl, ku, ab, pdab, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, complex
    band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ubc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f16ubc', ab, 'ab')
    _r = nag_double_type_check_and_cast('f16ubc', r, 'r')
    f16ubc_ctypes(order, norm, m, n, kl, ku, _ab, pdab, _r, fail)

f16ucc_ctypes = _library_cl.f16ucc
f16ucc_ctypes.restype = None
f16ucc_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f16ucc(order, norm, uplo, n, a, pda, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, complex
    Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ucc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16ucc', a, 'a')
    _r = nag_double_type_check_and_cast('f16ucc', r, 'r')
    f16ucc_ctypes(order, norm, uplo, n, _a, pda, _r, fail)

f16udc_ctypes = _library_cl.f16udc
f16udc_ctypes.restype = None
f16udc_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Nag_UploType, Integer, POINTER(Complex), POINTER(c_double), POINTER(NagError)]


def f16udc(order, norm, uplo, n, ap, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, complex
    Hermitian matrix, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16udc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f16udc', ap, 'ap')
    _r = nag_double_type_check_and_cast('f16udc', r, 'r')
    f16udc_ctypes(order, norm, uplo, n, _ap, _r, fail)

f16uec_ctypes = _library_cl.f16uec
f16uec_ctypes.restype = None
f16uec_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Nag_UploType, Integer, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f16uec(order, norm, uplo, n, k, ab, pdab, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, complex
    Hermitian band matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16uec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ab, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ab
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ab = nag_complex_type_check_and_cast('f16uec', ab, 'ab')
    _r = nag_double_type_check_and_cast('f16uec', r, 'r')
    f16uec_ctypes(order, norm, uplo, n, k, _ab, pdab, _r, fail)

f16ufc_ctypes = _library_cl.f16ufc
f16ufc_ctypes.restype = None
f16ufc_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Nag_UploType, Integer, POINTER(Complex), Integer, POINTER(c_double), POINTER(NagError)]


def f16ufc(order, norm, uplo, n, a, pda, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, complex
    symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ufc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16ufc', a, 'a')
    _r = nag_double_type_check_and_cast('f16ufc', r, 'r')
    f16ufc_ctypes(order, norm, uplo, n, _a, pda, _r, fail)

f16ugc_ctypes = _library_cl.f16ugc
f16ugc_ctypes.restype = None
f16ugc_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Nag_UploType, Integer, POINTER(Complex), POINTER(c_double), POINTER(NagError)]


def f16ugc(order, norm, uplo, n, ap, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, complex
    symmetric matrix, packed storage.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ugc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ap, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ap
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    _ap = nag_complex_type_check_and_cast('f16ugc', ap, 'ap')
    _r = nag_double_type_check_and_cast('f16ugc', r, 'r')
    f16ugc_ctypes(order, norm, uplo, n, _ap, _r, fail)

f16ukc_ctypes = _library_cl.f16ukc
f16ukc_ctypes.restype = None
f16ukc_ctypes.argtypes = [Nag_OrderType, Nag_NormType, Nag_RFP_Store, Nag_UploType, Integer, Nag_Complex_Type, POINTER(c_double), POINTER(NagError)]


def f16ukc(order, norm, transr, uplo, n, ar, r, fail):
    """
    1-norm, infinity-norm, Frobenius norm, largest absolute element, complex
    Hermitian matrix, Rectangular Full Packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ukc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ar, r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    TypeError will be raised when the following arguments are not of data type
    Complex:
        ar
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_complex_type_check_and_cast
    _r = nag_double_type_check_and_cast('f16ukc', r, 'r')
    _ar = nag_complex_type_check_and_cast('f16ukc', ar, 'ar')
    f16ukc_ctypes(order, norm, transr, uplo, n, _ar, _r, fail)

nag_zhf_norm = f16ukc

f16yac_ctypes = _library_cl.f16yac
f16yac_ctypes.restype = None
f16yac_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Nag_TransType, Integer, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16yac(order, transa, transb, m, n, k, alpha, a, pda, b, pdb, beta, c, pdc, fail):
    """
    Matrix-matrix product, two real rectangular matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16yac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16yac', a, 'a')
    _b = nag_double_type_check_and_cast('f16yac', b, 'b')
    _c = nag_double_type_check_and_cast('f16yac', c, 'c')
    f16yac_ctypes(order, transa, transb, m, n, k, alpha, _a, pda, _b, pdb, beta, _c, pdc, fail)

f16ycc_ctypes = _library_cl.f16ycc
f16ycc_ctypes.restype = None
f16ycc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16ycc(order, side, uplo, m, n, alpha, a, pda, b, pdb, beta, c, pdc, fail):
    """
    Matrix-matrix product, one real symmetric matrix, one real rectangular
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ycc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16ycc', a, 'a')
    _b = nag_double_type_check_and_cast('f16ycc', b, 'b')
    _c = nag_double_type_check_and_cast('f16ycc', c, 'c')
    f16ycc_ctypes(order, side, uplo, m, n, alpha, _a, pda, _b, pdb, beta, _c, pdc, fail)

f16yfc_ctypes = _library_cl.f16yfc
f16yfc_ctypes.restype = None
f16yfc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f16yfc(order, side, uplo, trans, diag, m, n, alpha, a, pda, b, pdb, fail):
    """
    Matrix-matrix product, one real triangular matrix, one real rectangular
    matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16yfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16yfc', a, 'a')
    _b = nag_double_type_check_and_cast('f16yfc', b, 'b')
    f16yfc_ctypes(order, side, uplo, trans, diag, m, n, alpha, _a, pda, _b, pdb, fail)

f16yjc_ctypes = _library_cl.f16yjc
f16yjc_ctypes.restype = None
f16yjc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def f16yjc(order, side, uplo, trans, diag, m, n, alpha, a, pda, b, pdb, fail):
    """
    Solves a system of equations with multiple right-hand sides, real
    triangular coefficient matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16yjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16yjc', a, 'a')
    _b = nag_double_type_check_and_cast('f16yjc', b, 'b')
    f16yjc_ctypes(order, side, uplo, trans, diag, m, n, alpha, _a, pda, _b, pdb, fail)

f16ylc_ctypes = _library_cl.f16ylc
f16ylc_ctypes.restype = None
f16ylc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_SideType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Integer, c_double, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def f16ylc(order, transr, side, uplo, trans, diag, m, n, alpha, ar, b, pdb, fail):
    """
    Solves a system of equations with multiple right-hand sides, real
    triangular coefficient matrix, Rectangular Full Packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ylc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ar, b
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        ar, b
    """
    from .util import nag_double_type_check_and_cast
    _ar = nag_double_type_check_and_cast('f16ylc', ar, 'ar')
    _b = nag_double_type_check_and_cast('f16ylc', b, 'b')
    f16ylc_ctypes(order, transr, side, uplo, trans, diag, m, n, alpha, _ar, _b, pdb, fail)

nag_dtfsm = f16ylc

f16ypc_ctypes = _library_cl.f16ypc
f16ypc_ctypes.restype = None
f16ypc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Integer, Integer, c_double, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16ypc(order, uplo, trans, n, k, alpha, a, pda, beta, c, pdc, fail):
    """
    Rank-k update of a real symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ypc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16ypc', a, 'a')
    _c = nag_double_type_check_and_cast('f16ypc', c, 'c')
    f16ypc_ctypes(order, uplo, trans, n, k, alpha, _a, pda, beta, _c, pdc, fail)

f16yqc_ctypes = _library_cl.f16yqc
f16yqc_ctypes.restype = None
f16yqc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_UploType, Nag_TransType, Integer, Integer, c_double, POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(NagError)]


def f16yqc(order, transr, uplo, trans, n, k, alpha, a, pda, beta, cr, fail):
    """
    Rank-k update of a real symmetric matrix, Rectangular Full Packed
    format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16yqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, cr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, cr
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16yqc', a, 'a')
    _cr = nag_double_type_check_and_cast('f16yqc', cr, 'cr')
    f16yqc_ctypes(order, transr, uplo, trans, n, k, alpha, _a, pda, beta, _cr, fail)

nag_dsfrk = f16yqc

f16yrc_ctypes = _library_cl.f16yrc
f16yrc_ctypes.restype = None
f16yrc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, POINTER(NagError)]


def f16yrc(order, uplo, trans, n, k, alpha, a, pda, b, pdb, beta, c, pdc, fail):
    """
    Rank-2k update of a real symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16yrc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a, b, c
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('f16yrc', a, 'a')
    _b = nag_double_type_check_and_cast('f16yrc', b, 'b')
    _c = nag_double_type_check_and_cast('f16yrc', c, 'c')
    f16yrc_ctypes(order, uplo, trans, n, k, alpha, _a, pda, _b, pdb, beta, _c, pdc, fail)

f16zac_ctypes = _library_cl.f16zac
f16zac_ctypes.restype = None
f16zac_ctypes.argtypes = [Nag_OrderType, Nag_TransType, Nag_TransType, Integer, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16zac(order, transa, transb, m, n, k, alpha, a, pda, b, pdb, beta, c, pdc, fail):
    """
    Matrix-matrix product, two complex rectangular matrices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16zac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16zac', a, 'a')
    _b = nag_complex_type_check_and_cast('f16zac', b, 'b')
    _c = nag_complex_type_check_and_cast('f16zac', c, 'c')
    f16zac_ctypes(order, transa, transb, m, n, k, alpha, _a, pda, _b, pdb, beta, _c, pdc, fail)

f16zcc_ctypes = _library_cl.f16zcc
f16zcc_ctypes.restype = None
f16zcc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16zcc(order, side, uplo, m, n, alpha, a, pda, b, pdb, beta, c, pdc, fail):
    """
    Matrix-matrix product, one complex Hermitian matrix, one complex
    rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16zcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16zcc', a, 'a')
    _b = nag_complex_type_check_and_cast('f16zcc', b, 'b')
    _c = nag_complex_type_check_and_cast('f16zcc', c, 'c')
    f16zcc_ctypes(order, side, uplo, m, n, alpha, _a, pda, _b, pdb, beta, _c, pdc, fail)

f16zfc_ctypes = _library_cl.f16zfc
f16zfc_ctypes.restype = None
f16zfc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f16zfc(order, side, uplo, trans, diag, m, n, alpha, a, pda, b, pdb, fail):
    """
    Matrix-matrix product, one complex triangular matrix, one complex
    rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16zfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16zfc', a, 'a')
    _b = nag_complex_type_check_and_cast('f16zfc', b, 'b')
    f16zfc_ctypes(order, side, uplo, trans, diag, m, n, alpha, _a, pda, _b, pdb, fail)

f16zjc_ctypes = _library_cl.f16zjc
f16zjc_ctypes.restype = None
f16zjc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, POINTER(NagError)]


def f16zjc(order, side, uplo, trans, diag, m, n, alpha, a, pda, b, pdb, fail):
    """
    Solves system of equations with multiple right-hand sides, complex
    triangular coefficient matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16zjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16zjc', a, 'a')
    _b = nag_complex_type_check_and_cast('f16zjc', b, 'b')
    f16zjc_ctypes(order, side, uplo, trans, diag, m, n, alpha, _a, pda, _b, pdb, fail)

f16zlc_ctypes = _library_cl.f16zlc
f16zlc_ctypes.restype = None
f16zlc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_SideType, Nag_UploType, Nag_TransType, Nag_DiagType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), POINTER(Complex), Integer, POINTER(NagError)]


def f16zlc(order, transr, side, uplo, trans, diag, m, n, alpha, ar, b, pdb, fail):
    """
    Solves system of equations with multiple right-hand sides, complex
    triangular coefficient matrix, Rectangular Full Packed format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16zlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ar, b
    TypeError will be raised when the following arguments are not of data type
    Complex:
        alpha, ar, b
    """
    from .util import nag_complex_type_check_and_cast
    _ar = nag_complex_type_check_and_cast('f16zlc', ar, 'ar')
    _b = nag_complex_type_check_and_cast('f16zlc', b, 'b')
    f16zlc_ctypes(order, transr, side, uplo, trans, diag, m, n, alpha, _ar, _b, pdb, fail)

nag_ztfsm = f16zlc

f16zpc_ctypes = _library_cl.f16zpc
f16zpc_ctypes.restype = None
f16zpc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Integer, Integer, c_double, POINTER(Complex), Integer, c_double, POINTER(Complex), Integer, POINTER(NagError)]


def f16zpc(order, uplo, trans, n, k, alpha, a, pda, beta, c, pdc, fail):
    """
    Rank-k update of a complex Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16zpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16zpc', a, 'a')
    _c = nag_complex_type_check_and_cast('f16zpc', c, 'c')
    f16zpc_ctypes(order, uplo, trans, n, k, alpha, _a, pda, beta, _c, pdc, fail)

f16zqc_ctypes = _library_cl.f16zqc
f16zqc_ctypes.restype = None
f16zqc_ctypes.argtypes = [Nag_OrderType, Nag_RFP_Store, Nag_UploType, Nag_TransType, Integer, Integer, c_double, POINTER(Complex), Integer, c_double, POINTER(Complex), POINTER(NagError)]


def f16zqc(order, transr, uplo, trans, n, k, alpha, a, pda, beta, cr, fail):
    """
    Rank-k update of a complex Hermitian matrix, Rectangular Full Packed
    format.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16zqc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, cr
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, cr
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16zqc', a, 'a')
    _cr = nag_complex_type_check_and_cast('f16zqc', cr, 'cr')
    f16zqc_ctypes(order, transr, uplo, trans, n, k, alpha, _a, pda, beta, _cr, fail)

nag_zhfrk = f16zqc

f16zrc_ctypes = _library_cl.f16zrc
f16zrc_ctypes.restype = None
f16zrc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, c_double, POINTER(Complex), Integer, POINTER(NagError)]


def f16zrc(order, uplo, trans, n, k, alpha, a, pda, b, pdb, beta, c, pdc, fail):
    """
    Rank-2k update of a complex Hermitian matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16zrc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16zrc', a, 'a')
    _b = nag_complex_type_check_and_cast('f16zrc', b, 'b')
    _c = nag_complex_type_check_and_cast('f16zrc', c, 'c')
    f16zrc_ctypes(order, uplo, trans, n, k, alpha, _a, pda, _b, pdb, beta, _c, pdc, fail)

f16ztc_ctypes = _library_cl.f16ztc
f16ztc_ctypes.restype = None
f16ztc_ctypes.argtypes = [Nag_OrderType, Nag_SideType, Nag_UploType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16ztc(order, side, uplo, m, n, alpha, a, pda, b, pdb, beta, c, pdc, fail):
    """
    Matrix-matrix product, one complex symmetric matrix, one complex
    rectangular matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16ztc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16ztc', a, 'a')
    _b = nag_complex_type_check_and_cast('f16ztc', b, 'b')
    _c = nag_complex_type_check_and_cast('f16ztc', c, 'c')
    f16ztc_ctypes(order, side, uplo, m, n, alpha, _a, pda, _b, pdb, beta, _c, pdc, fail)

f16zuc_ctypes = _library_cl.f16zuc
f16zuc_ctypes.restype = None
f16zuc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16zuc(order, uplo, trans, n, k, alpha, a, pda, beta, c, pdc, fail):
    """
    Rank-k update of a complex symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16zuc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16zuc', a, 'a')
    _c = nag_complex_type_check_and_cast('f16zuc', c, 'c')
    f16zuc_ctypes(order, uplo, trans, n, k, alpha, _a, pda, beta, _c, pdc, fail)

f16zwc_ctypes = _library_cl.f16zwc
f16zwc_ctypes.restype = None
f16zwc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_TransType, Integer, Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(Complex), Integer, Nag_Complex_Type, POINTER(Complex), Integer, POINTER(NagError)]


def f16zwc(order, uplo, trans, n, k, alpha, a, pda, b, pdb, beta, c, pdc, fail):
    """
    Rank-2k update of a complex symmetric matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/f16/f16zwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a, b, c
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a, b, c
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('f16zwc', a, 'a')
    _b = nag_complex_type_check_and_cast('f16zwc', b, 'b')
    _c = nag_complex_type_check_and_cast('f16zwc', c, 'c')
    f16zwc_ctypes(order, uplo, trans, n, k, alpha, _a, pda, _b, pdb, beta, _c, pdc, fail)

nag_iload = f16dbc
nag_isum = f16dlc
nag_imax_val = f16dnc
nag_imin_val = f16dpc
nag_iamax_val = f16dqc
nag_iamin_val = f16drc
nag_ddot = f16eac
nag_daxpby = f16ecc
nag_dwaxpby = f16ehc
nag_dsum = f16elc
nag_dload = f16fbc
nag_zaxpby = f16gcc
nag_zwaxpby = f16ghc
nag_zsum = f16glc
nag_zload = f16hbc
nag_dmax_val = f16jnc
nag_dmin_val = f16jpc
nag_damax_val = f16jqc
nag_damin_val = f16jrc
nag_zamax_val = f16jsc
nag_zamin_val = f16jtc
nag_dgemv = f16pac
nag_dgbmv = f16pbc
nag_dsymv = f16pcc
nag_dsbmv = f16pdc
nag_dspmv = f16pec
nag_dtrmv = f16pfc
nag_dtbmv = f16pgc
nag_dtpmv = f16phc
nag_dtrsv = f16pjc
nag_dtbsv = f16pkc
nag_dtpsv = f16plc
nag_dger = f16pmc
nag_dsyr = f16ppc
nag_dspr = f16pqc
nag_dsyr2 = f16prc
nag_dspr2 = f16psc
nag_dtr_copy = f16qec
nag_dge_copy = f16qfc
nag_dtr_load = f16qgc
nag_dge_load = f16qhc
nag_dge_norm = f16rac
nag_dgb_norm = f16rbc
nag_dsy_norm = f16rcc
nag_dsp_norm = f16rdc
nag_dsb_norm = f16rec
nag_zgemv = f16sac
nag_zgbmv = f16sbc
nag_zhemv = f16scc
nag_zhbmv = f16sdc
nag_zhpmv = f16sec
nag_ztrmv = f16sfc
nag_ztbmv = f16sgc
nag_ztpmv = f16shc
nag_ztrsv = f16sjc
nag_ztbsv = f16skc
nag_ztpsv = f16slc
nag_zger = f16smc
nag_zher = f16spc
nag_zhpr = f16sqc
nag_zher2 = f16src
nag_zhpr2 = f16ssc
nag_zsymv = f16tac
nag_zspmv = f16tcc
nag_ztr_copy = f16tec
nag_zge_copy = f16tfc
nag_ztr_load = f16tgc
nag_zge_load = f16thc
nag_zge_norm = f16uac
nag_zgb_norm = f16ubc
nag_zhe_norm = f16ucc
nag_zhp_norm = f16udc
nag_zhb_norm = f16uec
nag_zsy_norm = f16ufc
nag_zsp_norm = f16ugc
nag_dgemm = f16yac
nag_dsymm = f16ycc
nag_dtrmm = f16yfc
nag_dtrsm = f16yjc
nag_dsyrk = f16ypc
nag_dsyr2k = f16yrc
nag_zgemm = f16zac
nag_zhemm = f16zcc
nag_ztrmm = f16zfc
nag_ztrsm = f16zjc
nag_zherk = f16zpc
nag_zher2k = f16zrc
nag_zsymm = f16ztc
nag_zsyrk = f16zuc
nag_zsyr2k = f16zwc
__all__ = ['f16dbc', 'f16dlc', 'f16dnc', 'f16dpc', 'f16dqc', 'f16drc',
           'f16eac', 'f16ecc', 'f16ehc', 'f16elc', 'f16fbc', 'f16gcc',
           'f16ghc', 'f16glc', 'f16hbc', 'f16jnc', 'f16jpc', 'f16jqc',
           'f16jrc', 'f16jsc', 'f16jtc', 'f16pac', 'f16pbc', 'f16pcc',
           'f16pdc', 'f16pec', 'f16pfc', 'f16pgc', 'f16phc', 'f16pjc',
           'f16pkc', 'f16plc', 'f16pmc', 'f16ppc', 'f16pqc', 'f16prc',
           'f16psc', 'f16qec', 'f16qfc', 'f16qgc', 'f16qhc', 'f16rac',
           'f16rbc', 'f16rcc', 'f16rdc', 'f16rec', 'f16sac', 'f16sbc',
           'f16scc', 'f16sdc', 'f16sec', 'f16sfc', 'f16sgc', 'f16shc',
           'f16sjc', 'f16skc', 'f16slc', 'f16smc', 'f16spc', 'f16sqc',
           'f16src', 'f16ssc', 'f16tac', 'f16tcc', 'f16tec', 'f16tfc',
           'f16tgc', 'f16thc', 'f16uac', 'f16ubc', 'f16ucc', 'f16udc',
           'f16uec', 'f16ufc', 'f16ugc',
           'f16yac', 'f16ycc', 'f16yfc',
           'f16yjc', 'f16ypc', 'f16yrc', 'f16zac', 'f16zcc', 'f16zfc',
           'f16zjc', 'f16zpc', 'f16zrc', 'f16ztc', 'f16zuc', 'f16zwc',
           'nag_damax_val', 'nag_damin_val', 'nag_daxpby', 'nag_ddot',
           'nag_dgb_norm', 'nag_dgbmv', 'nag_dge_copy',
           'nag_dge_load', 'nag_dge_norm', 'nag_dgemm', 'nag_dgemv',
           'nag_dger', 'nag_dload', 'nag_dmax_val',
           'nag_dmin_val', 'nag_dsb_norm', 'nag_dsbmv',
           'nag_dsp_norm', 'nag_dspmv', 'nag_dspr', 'nag_dspr2',
           'nag_dsum', 'nag_dsy_norm', 'nag_dsymm', 'nag_dsymv',
           'nag_dsyr', 'nag_dsyr2', 'nag_dsyr2k', 'nag_dsyrk',
           'nag_dtbmv', 'nag_dtbsv', 'nag_dtpmv',
           'nag_dtpsv', 'nag_dtr_copy', 'nag_dtr_load', 'nag_dtrmm',
           'nag_dtrmv', 'nag_dtrsm', 'nag_dtrsv', 'nag_dwaxpby',
           'nag_iamax_val', 'nag_iamin_val', 'nag_iload',
           'nag_imax_val', 'nag_imin_val', 'nag_isum',
           'nag_zamax_val', 'nag_zamin_val', 'nag_zaxpby',
           'nag_zgb_norm', 'nag_zgbmv', 'nag_zge_copy',
           'nag_zge_load', 'nag_zge_norm', 'nag_zgemm', 'nag_zgemv',
           'nag_zger', 'nag_zhb_norm', 'nag_zhbmv', 'nag_zhe_norm',
           'nag_zhemm', 'nag_zhemv', 'nag_zher', 'nag_zher2',
           'nag_zher2k', 'nag_zherk', 'nag_zhp_norm', 'nag_zhpmv',
           'nag_zhpr', 'nag_zhpr2', 'nag_zload',
           'nag_zsp_norm', 'nag_zspmv', 'nag_zsum', 'nag_zsy_norm',
           'nag_zsymm', 'nag_zsymv', 'nag_zsyr2k', 'nag_zsyrk',
           'nag_ztbmv', 'nag_ztbsv', 'nag_ztpmv',
           'nag_ztpsv', 'nag_ztr_copy', 'nag_ztr_load', 'nag_ztrmm',
           'nag_ztrmv', 'nag_ztrsm', 'nag_ztrsv', 'nag_zwaxpby']
