"""
Deprecated wrappers for the NAG C Library (Mark 26.0) g02 Chapter.

g02 - Correlation and Regression Analysis.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/G02/g02conts.html

NAG Copyright 2017.
"""
from ctypes import c_char_p
from ctypes import c_double
from ctypes import POINTER
from .util import _reset_units
from .util import Nag_LARSModelType
from .util import Nag_LARSPreProcess
from .util import Nag_LARSTargetType
from .util import get_library

from .util import Integer
from .util import Nag_Comm
from .util import Nag_OrderType
from .util import NagError

from .util import Nag_NearCorr_ProbType
from .util import Nag_SumSquare
from .util import Nag_IncludeMean
from .util import Nag_Boolean
from .util import Nag_UpdateObserv
from .util import NAG_G02EFC_MONFUN
from .util import Nag_Link
from .util import Nag_Distributions
from .util import Nag_RegType
from .util import Nag_PsiFun
from .util import Nag_SigmaEst
from .util import Nag_CovMatrixEst
from .util import NAG_G02HBC_UCV
from .util import NAG_G02HDC_CHI
from .util import NAG_G02HDC_PSI
from .util import NAG_G02HFC_PSI
from .util import NAG_G02HFC_PSP
from .util import NAG_G02HLC_UCV
from .util import NAG_G02HMC_UCV
from .util import Nag_PredictError
from .util import Nag_EstimatesOption
from .util import Nag_OptionLOO
from .util import Nag_ParaOption
from .util import Nag_VIFOption
from .util import Nag_ScalePredictor
from .util import Nag_IncludeIntercept
from .util import Nag_VariableType
from .util import Nag_FullStepwise

_library_cl = get_library()

STRING = c_char_p

g02efg_ctypes = _library_cl.g02efg
g02efg_ctypes.restype = None
g02efg_ctypes.argtypes = [Nag_FullStepwise, Integer, c_double, POINTER(Nag_Comm)]


def g02efg(flag, var, val, comm):
    """
    Monitor function for full stepwise regression.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02efc.html#monfun
    """
    g02efg_ctypes(flag, var, val, comm)

g02aac_ctypes = _library_cl.g02aac
g02aac_ctypes.restype = None
g02aac_ctypes.argtypes = [Nag_OrderType, POINTER(c_double), Integer, Integer, c_double, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g02aac(order, g, pdg, n, errtol, maxits, maxit, x, pdx, _iter, feval, nrmgrd, fail):
    """
    Computes the nearest correlation matrix to a real square matrix, using
    the method of Qi and Sun.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        g, x, _iter, feval, nrmgrd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        g, x, nrmgrd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        _iter, feval
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _g = nag_double_type_check_and_cast('g02aac', g, 'g')
    _x = nag_double_type_check_and_cast('g02aac', x, 'x')
    __iter = nag_integer_type_check_and_cast('g02aac', _iter, '_iter')
    _feval = nag_integer_type_check_and_cast('g02aac', feval, 'feval')
    _nrmgrd = nag_double_type_check_and_cast('g02aac', nrmgrd, 'nrmgrd')
    g02aac_ctypes(order, _g, pdg, n, errtol, maxits, maxit, _x, pdx, __iter, _feval, _nrmgrd, fail)

g02abc_ctypes = _library_cl.g02abc
g02abc_ctypes.restype = None
g02abc_ctypes.argtypes = [Nag_OrderType, POINTER(c_double), Integer, Integer, Nag_NearCorr_ProbType, c_double, POINTER(c_double), c_double, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g02abc(order, g, pdg, n, opt, alpha, w, errtol, maxits, maxit, x, pdx, _iter, feval, nrmgrd, fail):
    """
    Computes the nearest correlation matrix to a real square matrix,
    augmented g02aac to incorporate weights and bounds.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02abc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        g, w, x, _iter, feval, nrmgrd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        g, w, x, nrmgrd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        _iter, feval
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _g = nag_double_type_check_and_cast('g02abc', g, 'g')
    _w = nag_double_type_check_and_cast('g02abc', w, 'w')
    _x = nag_double_type_check_and_cast('g02abc', x, 'x')
    __iter = nag_integer_type_check_and_cast('g02abc', _iter, '_iter')
    _feval = nag_integer_type_check_and_cast('g02abc', feval, 'feval')
    _nrmgrd = nag_double_type_check_and_cast('g02abc', nrmgrd, 'nrmgrd')
    g02abc_ctypes(order, _g, pdg, n, opt, alpha, _w, errtol, maxits, maxit, _x, pdx, __iter, _feval, _nrmgrd, fail)

g02aec_ctypes = _library_cl.g02aec
g02aec_ctypes.restype = None
g02aec_ctypes.argtypes = [Nag_OrderType, POINTER(c_double), Integer, Integer, Integer, c_double, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g02aec(order, g, pdg, n, k, errtol, maxit, x, pdx, _iter, feval, nrmpgd, fail):
    """
    Computes the nearest correlation matrix with k-factor structure to a
    real square matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02aec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        g, x, _iter, feval, nrmpgd
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        g, x, nrmpgd
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        _iter, feval
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _g = nag_double_type_check_and_cast('g02aec', g, 'g')
    _x = nag_double_type_check_and_cast('g02aec', x, 'x')
    __iter = nag_integer_type_check_and_cast('g02aec', _iter, '_iter')
    _feval = nag_integer_type_check_and_cast('g02aec', feval, 'feval')
    _nrmpgd = nag_double_type_check_and_cast('g02aec', nrmpgd, 'nrmpgd')
    g02aec_ctypes(order, _g, pdg, n, k, errtol, maxit, _x, pdx, __iter, _feval, _nrmpgd, fail)

g02ajc_ctypes = _library_cl.g02ajc
g02ajc_ctypes.restype = None
g02ajc_ctypes.argtypes = [POINTER(c_double), Integer, Integer, c_double, POINTER(c_double), Integer, c_double, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g02ajc(g, pdg, n, alpha, h, pdh, errtol, maxit, x, pdx, _iter, norm, fail):
    """
    Computes the nearest correlation matrix to a real square matrix, using
    element-wise weighting.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02ajc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        g, h, x, _iter, norm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        g, h, x, norm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        _iter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _g = nag_double_type_check_and_cast('g02ajc', g, 'g')
    _h = nag_double_type_check_and_cast('g02ajc', h, 'h')
    _x = nag_double_type_check_and_cast('g02ajc', x, 'x')
    __iter = nag_integer_type_check_and_cast('g02ajc', _iter, '_iter')
    _norm = nag_double_type_check_and_cast('g02ajc', norm, 'norm')
    g02ajc_ctypes(_g, pdg, n, alpha, _h, pdh, errtol, maxit, _x, pdx, __iter, _norm, fail)

g02anc_ctypes = _library_cl.g02anc
g02anc_ctypes.restype = None
g02anc_ctypes.argtypes = [POINTER(c_double), Integer, Integer, Integer, c_double, c_double, POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02anc(g, pdg, n, k, errtol, eigtol, x, pdx, alpha, _iter, eigmin, norm, fail):
    """
    Computes a correlation matrix from an approximate matrix with fixed
    submatrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02anc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        g, x, alpha, _iter, eigmin, norm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        g, x, alpha, eigmin, norm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        _iter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _g = nag_double_type_check_and_cast('g02anc', g, 'g')
    _x = nag_double_type_check_and_cast('g02anc', x, 'x')
    _alpha = nag_double_type_check_and_cast('g02anc', alpha, 'alpha')
    _eigmin = nag_double_type_check_and_cast('g02anc', eigmin, 'eigmin')
    _norm = nag_double_type_check_and_cast('g02anc', norm, 'norm')
    __iter = nag_integer_type_check_and_cast('g02anc', _iter, '_iter')
    g02anc_ctypes(_g, pdg, n, k, errtol, eigtol, _x, pdx, _alpha, __iter, _eigmin, _norm, fail)


g02apc_ctypes = _library_cl.g02apc
g02apc_ctypes.restype = None
g02apc_ctypes.argtypes = [POINTER(c_double),
                          Integer,
                          Integer,
                          c_double,
                          POINTER(c_double),
                          Integer,
                          c_double,
                          c_double,
                          POINTER(c_double),
                          Integer,
                          POINTER(c_double),
                          POINTER(Integer),
                          POINTER(c_double),
                          POINTER(c_double),
                          POINTER(NagError)]


def g02apc(g, pdg, n, theta, h, pdh, errtol, eigtol, x, pdx, alpha,
           _iter, eigmin, norm, fail):
    """
    Computes a correlation matrix from an approximate one using a
    specified target matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02apc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        g, h, x, alpha, eigmin, norm, _iter
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        g, h, x, alpha, eigmin, norm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        _iter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _g = nag_double_type_check_and_cast('g02anc', g, 'g')
    _h = nag_double_type_check_and_cast('g02anc', h, 'h')
    _x = nag_double_type_check_and_cast('g02anc', x, 'x')
    _alpha = nag_double_type_check_and_cast('g02anc', alpha, 'alpha')
    _eigmin = nag_double_type_check_and_cast('g02anc', eigmin, 'eigmin')
    _norm = nag_double_type_check_and_cast('g02anc', norm, 'norm')
    __iter = nag_integer_type_check_and_cast('g02anc', _iter, '_iter')
    g02apc_ctypes(_g, pdg, n, theta, _h, pdh, errtol, eigtol, _x, pdx, _alpha,
                  __iter, _eigmin, _norm, fail)


g02brc_ctypes = _library_cl.g02brc
g02brc_ctypes.restype = None
g02brc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(NagError)]

def g02brc(n, m, x, tdx, svar, sobs, corr, tdc, fail):
    """
    Kendall and/or Spearman non-parametric rank correlation coefficients,
    allows variables and observations to be selectively disregarded.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02brc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, svar, sobs, corr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, corr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        svar, sobs
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02brc', x, 'x')
    _svar = nag_integer_type_check_and_cast('g02brc', svar, 'svar')
    _sobs = nag_integer_type_check_and_cast('g02brc', sobs, 'sobs')
    _corr = nag_double_type_check_and_cast('g02brc', corr, 'corr')
    g02brc_ctypes(n, m, _x, tdx, _svar, _sobs, _corr, tdc, fail)

g02btc_ctypes = _library_cl.g02btc
g02btc_ctypes.restype = None
g02btc_ctypes.argtypes = [Nag_SumSquare, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02btc(mean, m, wt, x, incx, sw, xbar, c, fail):
    """
    Update a weighted sum of squares matrix with a new observation.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02btc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, sw, xbar, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, sw, xbar, c
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02btc', x, 'x')
    _sw = nag_double_type_check_and_cast('g02btc', sw, 'sw')
    _xbar = nag_double_type_check_and_cast('g02btc', xbar, 'xbar')
    _c = nag_double_type_check_and_cast('g02btc', c, 'c')
    g02btc_ctypes(mean, m, wt, _x, incx, _sw, _xbar, _c, fail)

g02buc_ctypes = _library_cl.g02buc
g02buc_ctypes.restype = None
g02buc_ctypes.argtypes = [Nag_OrderType, Nag_SumSquare, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02buc(order, mean, n, m, x, pdx, wt, sw, wmean, c, fail):
    """
    Computes a weighted sum of squares matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02buc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, wt, sw, wmean, c
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, wt, sw, wmean, c
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02buc', x, 'x')
    _wt = nag_double_type_check_and_cast('g02buc', wt, 'wt')
    _sw = nag_double_type_check_and_cast('g02buc', sw, 'sw')
    _wmean = nag_double_type_check_and_cast('g02buc', wmean, 'wmean')
    _c = nag_double_type_check_and_cast('g02buc', c, 'c')
    g02buc_ctypes(order, mean, n, m, _x, pdx, _wt, _sw, _wmean, _c, fail)

g02bwc_ctypes = _library_cl.g02bwc
g02bwc_ctypes.restype = None
g02bwc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(NagError)]


def g02bwc(m, r, fail):
    """
    Computes a correlation matrix from a sum of squares matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02bwc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        r
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r
    """
    from .util import nag_double_type_check_and_cast
    _r = nag_double_type_check_and_cast('g02bwc', r, 'r')
    g02bwc_ctypes(m, _r, fail)

g02bxc_ctypes = _library_cl.g02bxc
g02bxc_ctypes.restype = None
g02bxc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g02bxc(n, m, x, tdx, sx, wt, sw, wmean, std, r, tdr, v, tdv, fail):
    """
    Product-moment correlation, unweighted/weighted correlation and
    covariance matrix, allows variables to be disregarded.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02bxc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, sx, wt, sw, wmean, std, r, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, wt, sw, wmean, std, r, v
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02bxc', x, 'x')
    _sx = nag_integer_type_check_and_cast('g02bxc', sx, 'sx')
    _wt = nag_double_type_check_and_cast('g02bxc', wt, 'wt')
    _sw = nag_double_type_check_and_cast('g02bxc', sw, 'sw')
    _wmean = nag_double_type_check_and_cast('g02bxc', wmean, 'wmean')
    _std = nag_double_type_check_and_cast('g02bxc', std, 'std')
    _r = nag_double_type_check_and_cast('g02bxc', r, 'r')
    _v = nag_double_type_check_and_cast('g02bxc', v, 'v')
    g02bxc_ctypes(n, m, _x, tdx, _sx, _wt, _sw, _wmean, _std, _r, tdr, _v, tdv, fail)

g02byc_ctypes = _library_cl.g02byc
g02byc_ctypes.restype = None
g02byc_ctypes.argtypes = [Integer, Integer, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g02byc(m, ny, nx, sz, r, tdr, p, tdp, fail):
    """
    Computes partial correlation/variance-covariance matrix from correlation
    /variance-covariance matrix computed by g02bxc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02byc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        sz, r, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        r, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sz
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _sz = nag_integer_type_check_and_cast('g02byc', sz, 'sz')
    _r = nag_double_type_check_and_cast('g02byc', r, 'r')
    _p = nag_double_type_check_and_cast('g02byc', p, 'p')
    g02byc_ctypes(m, ny, nx, _sz, _r, tdr, _p, tdp, fail)

g02bzc_ctypes = _library_cl.g02bzc
g02bzc_ctypes.restype = None
g02bzc_ctypes.argtypes = [Nag_SumSquare, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02bzc(mean, m, xsw, xmean, xc, ysw, ymean, yc, fail):
    """
    Combines two sums of squares matrices, for use after g02buc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02bzc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xsw, xmean, xc, ymean, yc
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xsw, xmean, xc, ymean, yc
    """
    from .util import nag_double_type_check_and_cast
    _xsw = nag_double_type_check_and_cast('g02bzc', xsw, 'xsw')
    _xmean = nag_double_type_check_and_cast('g02bzc', xmean, 'xmean')
    _xc = nag_double_type_check_and_cast('g02bzc', xc, 'xc')
    _ymean = nag_double_type_check_and_cast('g02bzc', ymean, 'ymean')
    _yc = nag_double_type_check_and_cast('g02bzc', yc, 'yc')
    g02bzc_ctypes(mean, m, _xsw, _xmean, _xc, ysw, _ymean, _yc, fail)

g02cac_ctypes = _library_cl.g02cac
g02cac_ctypes.restype = None
g02cac_ctypes.argtypes = [Nag_SumSquare, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02cac(mean, n, x, y, wt, a, b, aserr, bserr, rsq, rss, df, fail):
    """
    Simple linear regression with or without a constant term, data may be
    weighted.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, wt, a, b, aserr, bserr, rsq, rss, df
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, wt, a, b, aserr, bserr, rsq, rss, df
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02cac', x, 'x')
    _y = nag_double_type_check_and_cast('g02cac', y, 'y')
    _wt = nag_double_type_check_and_cast('g02cac', wt, 'wt')
    _a = nag_double_type_check_and_cast('g02cac', a, 'a')
    _b = nag_double_type_check_and_cast('g02cac', b, 'b')
    _aserr = nag_double_type_check_and_cast('g02cac', aserr, 'aserr')
    _bserr = nag_double_type_check_and_cast('g02cac', bserr, 'bserr')
    _rsq = nag_double_type_check_and_cast('g02cac', rsq, 'rsq')
    _rss = nag_double_type_check_and_cast('g02cac', rss, 'rss')
    _df = nag_double_type_check_and_cast('g02cac', df, 'df')
    g02cac_ctypes(mean, n, _x, _y, _wt, _a, _b, _aserr, _bserr, _rsq, _rss, _df, fail)

g02cbc_ctypes = _library_cl.g02cbc
g02cbc_ctypes.restype = None
g02cbc_ctypes.argtypes = [Nag_SumSquare, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02cbc(mean, n, x, y, wt, clm, clp, yhat, yml, ymu, yl, yu, h, res, rms, fail):
    """
    Simple linear regression confidence intervals for the regression line
    and individual points.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02cbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, wt, yhat, yml, ymu, yl, yu, h, res, rms
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, wt, yhat, yml, ymu, yl, yu, h, res, rms
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02cbc', x, 'x')
    _y = nag_double_type_check_and_cast('g02cbc', y, 'y')
    _wt = nag_double_type_check_and_cast('g02cbc', wt, 'wt')
    _yhat = nag_double_type_check_and_cast('g02cbc', yhat, 'yhat')
    _yml = nag_double_type_check_and_cast('g02cbc', yml, 'yml')
    _ymu = nag_double_type_check_and_cast('g02cbc', ymu, 'ymu')
    _yl = nag_double_type_check_and_cast('g02cbc', yl, 'yl')
    _yu = nag_double_type_check_and_cast('g02cbc', yu, 'yu')
    _h = nag_double_type_check_and_cast('g02cbc', h, 'h')
    _res = nag_double_type_check_and_cast('g02cbc', res, 'res')
    _rms = nag_double_type_check_and_cast('g02cbc', rms, 'rms')
    g02cbc_ctypes(mean, n, _x, _y, _wt, clm, clp, _yhat, _yml, _ymu, _yl, _yu, _h, _res, _rms, fail)

g02dac_ctypes = _library_cl.g02dac
g02dac_ctypes.restype = None
g02dac_ctypes.argtypes = [Nag_IncludeMean, Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(Nag_Boolean), POINTER(Integer), POINTER(c_double), c_double, POINTER(c_double), POINTER(NagError)]


def g02dac(mean, n, x, tdx, m, sx, ip, y, wt, rss, df, b, se, cov, res, h, q, tdq, svd, rank, p, tol, comar, fail):
    """
    Fits a general (multiple) linear regression model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02dac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, sx, y, wt, rss, df, b, se, cov, res, h, q, rank, p, comar
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, wt, rss, df, b, se, cov, res, h, q, p, comar
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sx, rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02dac', x, 'x')
    _sx = nag_integer_type_check_and_cast('g02dac', sx, 'sx')
    _y = nag_double_type_check_and_cast('g02dac', y, 'y')
    _wt = nag_double_type_check_and_cast('g02dac', wt, 'wt')
    _rss = nag_double_type_check_and_cast('g02dac', rss, 'rss')
    _df = nag_double_type_check_and_cast('g02dac', df, 'df')
    _b = nag_double_type_check_and_cast('g02dac', b, 'b')
    _se = nag_double_type_check_and_cast('g02dac', se, 'se')
    _cov = nag_double_type_check_and_cast('g02dac', cov, 'cov')
    _res = nag_double_type_check_and_cast('g02dac', res, 'res')
    _h = nag_double_type_check_and_cast('g02dac', h, 'h')
    _q = nag_double_type_check_and_cast('g02dac', q, 'q')
    _rank = nag_integer_type_check_and_cast('g02dac', rank, 'rank')
    _p = nag_double_type_check_and_cast('g02dac', p, 'p')
    _comar = nag_double_type_check_and_cast('g02dac', comar, 'comar')
    g02dac_ctypes(mean, n, _x, tdx, m, _sx, ip, _y, _wt, _rss, _df, _b, _se, _cov, _res, _h, _q, tdq, svd, _rank, _p, tol, _comar, fail)

g02dcc_ctypes = _library_cl.g02dcc
g02dcc_ctypes.restype = None
g02dcc_ctypes.argtypes = [Nag_UpdateObserv, Nag_IncludeMean, Integer, POINTER(Integer), POINTER(c_double), Integer, Integer, POINTER(c_double), Integer, Integer, Integer, c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02dcc(update, mean, m, sx, q, tdq, ip, x, nr, tdx, ix, y, wt, rss, fail):
    """
    Add/delete an observation to/from a general linear regression model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02dcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        sx, q, x, wt, rss
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        q, x, wt, rss
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _sx = nag_integer_type_check_and_cast('g02dcc', sx, 'sx')
    _q = nag_double_type_check_and_cast('g02dcc', q, 'q')
    _x = nag_double_type_check_and_cast('g02dcc', x, 'x')
    _wt = nag_double_type_check_and_cast('g02dcc', wt, 'wt')
    _rss = nag_double_type_check_and_cast('g02dcc', rss, 'rss')
    g02dcc_ctypes(update, mean, m, _sx, _q, tdq, ip, _x, nr, tdx, ix, y, _wt, _rss, fail)

g02ddc_ctypes = _library_cl.g02ddc
g02ddc_ctypes.restype = None
g02ddc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_Boolean), POINTER(Integer), POINTER(c_double), c_double, POINTER(NagError)]


def g02ddc(n, ip, q, tdq, rss, df, b, se, cov, svd, rank, p, tol, fail):
    """
    Estimates of regression parameters from an updated model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02ddc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        q, rss, df, b, se, cov, rank, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        q, rss, df, b, se, cov, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _q = nag_double_type_check_and_cast('g02ddc', q, 'q')
    _rss = nag_double_type_check_and_cast('g02ddc', rss, 'rss')
    _df = nag_double_type_check_and_cast('g02ddc', df, 'df')
    _b = nag_double_type_check_and_cast('g02ddc', b, 'b')
    _se = nag_double_type_check_and_cast('g02ddc', se, 'se')
    _cov = nag_double_type_check_and_cast('g02ddc', cov, 'cov')
    _rank = nag_integer_type_check_and_cast('g02ddc', rank, 'rank')
    _p = nag_double_type_check_and_cast('g02ddc', p, 'p')
    g02ddc_ctypes(n, ip, _q, tdq, _rss, _df, _b, _se, _cov, svd, _rank, _p, tol, fail)

g02dec_ctypes = _library_cl.g02dec
g02dec_ctypes.restype = None
g02dec_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(NagError)]


def g02dec(n, ip, q, tdq, p, wt, x, rss, tol, fail):
    """
    Add a new independent variable to a general linear regression model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02dec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        q, p, wt, x, rss
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        q, p, wt, x, rss
    """
    from .util import nag_double_type_check_and_cast
    _q = nag_double_type_check_and_cast('g02dec', q, 'q')
    _p = nag_double_type_check_and_cast('g02dec', p, 'p')
    _wt = nag_double_type_check_and_cast('g02dec', wt, 'wt')
    _x = nag_double_type_check_and_cast('g02dec', x, 'x')
    _rss = nag_double_type_check_and_cast('g02dec', rss, 'rss')
    g02dec_ctypes(n, ip, _q, tdq, _p, _wt, _x, _rss, tol, fail)

g02dfc_ctypes = _library_cl.g02dfc
g02dfc_ctypes.restype = None
g02dfc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(NagError)]


def g02dfc(ip, q, tdq, indx, rss, fail):
    """
    Delete an independent variable from a general linear regression model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02dfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        q, rss
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        q, rss
    """
    from .util import nag_double_type_check_and_cast
    _q = nag_double_type_check_and_cast('g02dfc', q, 'q')
    _rss = nag_double_type_check_and_cast('g02dfc', rss, 'rss')
    g02dfc_ctypes(ip, _q, tdq, indx, _rss, fail)

g02dgc_ctypes = _library_cl.g02dgc
g02dgc_ctypes.restype = None
g02dgc_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, Nag_Boolean, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02dgc(n, wt, rss, ip, rank, cov, q, tdq, svd, p, y, b, se, res, comar, fail):
    """
    Fits a general linear regression model to new dependent variable.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02dgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        wt, rss, cov, q, p, y, b, se, res, comar
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        wt, rss, cov, q, p, y, b, se, res, comar
    """
    from .util import nag_double_type_check_and_cast
    _wt = nag_double_type_check_and_cast('g02dgc', wt, 'wt')
    _rss = nag_double_type_check_and_cast('g02dgc', rss, 'rss')
    _cov = nag_double_type_check_and_cast('g02dgc', cov, 'cov')
    _q = nag_double_type_check_and_cast('g02dgc', q, 'q')
    _p = nag_double_type_check_and_cast('g02dgc', p, 'p')
    _y = nag_double_type_check_and_cast('g02dgc', y, 'y')
    _b = nag_double_type_check_and_cast('g02dgc', b, 'b')
    _se = nag_double_type_check_and_cast('g02dgc', se, 'se')
    _res = nag_double_type_check_and_cast('g02dgc', res, 'res')
    _comar = nag_double_type_check_and_cast('g02dgc', comar, 'comar')
    g02dgc_ctypes(n, _wt, _rss, ip, rank, _cov, _q, tdq, svd, _p, _y, _b, _se, _res, _comar, fail)

g02dkc_ctypes = _library_cl.g02dkc
g02dkc_ctypes.restype = None
g02dkc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02dkc(ip, iconst, p, c, tdc, b, rss, df, se, cov, fail):
    """
    Estimates of parameters of a general linear regression model for given
    constraints.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02dkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        p, c, b, se, cov
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, c, b, se, cov
    """
    from .util import nag_double_type_check_and_cast
    _p = nag_double_type_check_and_cast('g02dkc', p, 'p')
    _c = nag_double_type_check_and_cast('g02dkc', c, 'c')
    _b = nag_double_type_check_and_cast('g02dkc', b, 'b')
    _se = nag_double_type_check_and_cast('g02dkc', se, 'se')
    _cov = nag_double_type_check_and_cast('g02dkc', cov, 'cov')
    g02dkc_ctypes(ip, iconst, _p, _c, tdc, _b, rss, df, _se, _cov, fail)

g02dnc_ctypes = _library_cl.g02dnc
g02dnc_ctypes.restype = None
g02dnc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Nag_Boolean), POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(NagError)]


def g02dnc(ip, rank, b, cov, p, f, est, stat, sestat, t, tol, fail):
    """
    Estimate of an estimable function for a general linear regression model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02dnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        b, cov, p, f, stat, sestat, t
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        b, cov, p, f, stat, sestat, t
    """
    from .util import nag_double_type_check_and_cast
    _b = nag_double_type_check_and_cast('g02dnc', b, 'b')
    _cov = nag_double_type_check_and_cast('g02dnc', cov, 'cov')
    _p = nag_double_type_check_and_cast('g02dnc', p, 'p')
    _f = nag_double_type_check_and_cast('g02dnc', f, 'f')
    _stat = nag_double_type_check_and_cast('g02dnc', stat, 'stat')
    _sestat = nag_double_type_check_and_cast('g02dnc', sestat, 'sestat')
    _t = nag_double_type_check_and_cast('g02dnc', t, 't')
    g02dnc_ctypes(ip, rank, _b, _cov, _p, _f, est, _stat, _sestat, _t, tol, fail)

g02eac_ctypes = _library_cl.g02eac
g02eac_ctypes.restype = None
g02eac_ctypes.argtypes = [Nag_OrderType, Nag_IncludeMean, Integer, Integer, POINTER(c_double), Integer, POINTER(STRING), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(STRING), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def g02eac(order, mean, n, m, x, pdx, varnames, sx, y, wt, nmod, model, rss, nterms, mrank, fail):
    """
    Computes residual sums of squares for all possible linear regressions
    for a set of independent variables.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02eac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, sx, y, wt, nmod, model, rss, nterms, mrank
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, wt, rss
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sx, nmod, nterms, mrank
    TypeError will be raised when the following arguments are not arrays of
    strings:
        varnames, model
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_char_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02eac', x, 'x')
    _varnames = nag_char_type_check_and_cast('g02eac', varnames, 'varnames')
    _sx = nag_integer_type_check_and_cast('g02eac', sx, 'sx')
    _y = nag_double_type_check_and_cast('g02eac', y, 'y')
    _wt = nag_double_type_check_and_cast('g02eac', wt, 'wt')
    _nmod = nag_integer_type_check_and_cast('g02eac', nmod, 'nmod')
    _model = nag_char_type_check_and_cast('g02eac', model, 'model')
    _rss = nag_double_type_check_and_cast('g02eac', rss, 'rss')
    _nterms = nag_integer_type_check_and_cast('g02eac', nterms, 'nterms')
    _mrank = nag_integer_type_check_and_cast('g02eac', mrank, 'mrank')
    g02eac_ctypes(order, mean, n, m, _x, pdx, varnames, _sx, _y, _wt, _nmod, _model, _rss, _nterms, _mrank, fail)

g02ecc_ctypes = _library_cl.g02ecc
g02ecc_ctypes.restype = None
g02ecc_ctypes.argtypes = [Nag_IncludeMean, Integer, c_double, c_double, Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02ecc(mean, n, sigsq, tss, nmod, nterms, rss, rsq, cp, fail):
    """
    Calculates R**2 and C_P values from residual sums of squares.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02ecc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nterms, rss, rsq, cp
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rss, rsq, cp
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nterms
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nterms = nag_integer_type_check_and_cast('g02ecc', nterms, 'nterms')
    _rss = nag_double_type_check_and_cast('g02ecc', rss, 'rss')
    _rsq = nag_double_type_check_and_cast('g02ecc', rsq, 'rsq')
    _cp = nag_double_type_check_and_cast('g02ecc', cp, 'cp')
    g02ecc_ctypes(mean, n, sigsq, tss, nmod, _nterms, _rss, _rsq, _cp, fail)

g02eec_ctypes = _library_cl.g02eec
g02eec_ctypes.restype = None
g02eec_ctypes.argtypes = [Nag_OrderType, POINTER(Integer), Nag_IncludeMean, Integer, Integer, POINTER(c_double), Integer, POINTER(STRING), POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), c_double, POINTER(Nag_Boolean), POINTER(STRING), POINTER(c_double), POINTER(c_double), POINTER(STRING), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(STRING), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(NagError)]


def g02eec(order, istep, mean, n, m, x, pdx, varnames, sx, maxip, y, wt, fin, addvar, newvar, chrss, f, model, nterm, rss, idf, ifr, freevars, exss, q, pdq, p, fail):
    """
    Fits a linear regression model by forward selection.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02eec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        istep, x, sx, y, wt, chrss, f, nterm, rss, idf, ifr, exss, q, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, wt, chrss, f, rss, exss, q, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        istep, sx, nterm, idf, ifr
    TypeError will be raised when the following arguments are not arrays of
    strings:
        varnames, newvar, model, freevars
    """
    from .util import nag_char_type_check_and_cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _istep = nag_integer_type_check_and_cast('g02eec', istep, 'istep')
    _x = nag_double_type_check_and_cast('g02eec', x, 'x')
    _varnames = nag_char_type_check_and_cast('g02eec', varnames, 'varnames')
    _sx = nag_integer_type_check_and_cast('g02eec', sx, 'sx')
    _y = nag_double_type_check_and_cast('g02eec', y, 'y')
    _wt = nag_double_type_check_and_cast('g02eec', wt, 'wt')
    _newvar = nag_char_type_check_and_cast('g02eec', newvar, 'newvar')
    _chrss = nag_double_type_check_and_cast('g02eec', chrss, 'chrss')
    _f = nag_double_type_check_and_cast('g02eec', f, 'f')
    _model = nag_char_type_check_and_cast('g02eec', model, 'model')
    _nterm = nag_integer_type_check_and_cast('g02eec', nterm, 'nterm')
    _rss = nag_double_type_check_and_cast('g02eec', rss, 'rss')
    _idf = nag_integer_type_check_and_cast('g02eec', idf, 'idf')
    _ifr = nag_integer_type_check_and_cast('g02eec', ifr, 'ifr')
    _freevars = nag_char_type_check_and_cast('g02eec', freevars, 'freevars')
    _exss = nag_double_type_check_and_cast('g02eec', exss, 'exss')
    _q = nag_double_type_check_and_cast('g02eec', q, 'q')
    _p = nag_double_type_check_and_cast('g02eec', p, 'p')
    g02eec_ctypes(order, _istep, mean, n, m, _x, pdx, _varnames, _sx, maxip, _y, _wt, fin, addvar, _newvar, _chrss, _f, _model, _nterm, _rss, _idf, _ifr, _freevars, _exss, _q, pdq, _p, fail)

g02efc_ctypes = _library_cl.g02efc
g02efc_ctypes.restype = None
g02efc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), c_double, POINTER(Integer), c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), NAG_G02EFC_MONFUN, POINTER(Nag_Comm), POINTER(NagError)]


@_reset_units
def g02efc(m, n, wmean, c, sw, isx, fin, fout, tau, b, se, rsq, rms, df, monfun, comm, fail):
    """
    Stepwise linear regression.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02efc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        wmean, c, isx, b, se, rsq, rms, df
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        wmean, c, b, se, rsq, rms
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx, df
    """
    from ctypes import cast
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _wmean = nag_double_type_check_and_cast('g02efc', wmean, 'wmean')
    _c = nag_double_type_check_and_cast('g02efc', c, 'c')
    _isx = nag_integer_type_check_and_cast('g02efc', isx, 'isx')
    _b = nag_double_type_check_and_cast('g02efc', b, 'b')
    _se = nag_double_type_check_and_cast('g02efc', se, 'se')
    _rsq = nag_double_type_check_and_cast('g02efc', rsq, 'rsq')
    _rms = nag_double_type_check_and_cast('g02efc', rms, 'rms')
    _df = nag_integer_type_check_and_cast('g02efc', df, 'df')
    if monfun is None:
        monfun = cast(None, NAG_G02EFC_MONFUN)
    g02efc_ctypes(m, n, _wmean, _c, sw, _isx, fin, fout, tau, _b, _se, _rsq, _rms, _df, monfun, comm, fail)

g02fac_ctypes = _library_cl.g02fac
g02fac_ctypes.restype = None
g02fac_ctypes.argtypes = [Integer, Integer, Integer, POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), POINTER(NagError)]


def g02fac(n, ip, nres, res, h, rms, sres, fail):
    """
    Calculates standardized residuals and influence statistics.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02fac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        res, h, sres
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        res, h, sres
    """
    from .util import nag_double_type_check_and_cast
    _res = nag_double_type_check_and_cast('g02fac', res, 'res')
    _h = nag_double_type_check_and_cast('g02fac', h, 'h')
    _sres = nag_double_type_check_and_cast('g02fac', sres, 'sres')
    g02fac_ctypes(n, ip, nres, _res, _h, rms, _sres, fail)

g02fcc_ctypes = _library_cl.g02fcc
g02fcc_ctypes.restype = None
g02fcc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02fcc(n, p, res, d, pdl, pdu, fail):
    """
    Computes Durbin--Watson test statistic.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02fcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        res, d, pdl, pdu
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        res, d, pdl, pdu
    """
    from .util import nag_double_type_check_and_cast
    _res = nag_double_type_check_and_cast('g02fcc', res, 'res')
    _d = nag_double_type_check_and_cast('g02fcc', d, 'd')
    _pdl = nag_double_type_check_and_cast('g02fcc', pdl, 'pdl')
    _pdu = nag_double_type_check_and_cast('g02fcc', pdu, 'pdu')
    g02fcc_ctypes(n, p, _res, _d, _pdl, _pdu, fail)

g02gac_ctypes = _library_cl.g02gac
g02gac_ctypes.restype = None
g02gac_ctypes.argtypes = [Nag_Link, Nag_IncludeMean, Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, c_double, Integer, Integer, STRING, c_double, POINTER(NagError)]


def g02gac(link, mean, n, x, tdx, m, sx, ip, y, wt, offset, scale, expower, rss, df, b, rank, se, cov, v, tdv, tol, maxiter, printiter, outfile, eps, fail):
    """
    Fits a generalized linear model with Normal errors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02gac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, sx, y, wt, offset, scale, rss, df, b, rank, se, cov, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, wt, offset, scale, rss, df, b, se, cov, v
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sx, rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02gac', x, 'x')
    _sx = nag_integer_type_check_and_cast('g02gac', sx, 'sx')
    _y = nag_double_type_check_and_cast('g02gac', y, 'y')
    _wt = nag_double_type_check_and_cast('g02gac', wt, 'wt')
    _offset = nag_double_type_check_and_cast('g02gac', offset, 'offset')
    _scale = nag_double_type_check_and_cast('g02gac', scale, 'scale')
    _rss = nag_double_type_check_and_cast('g02gac', rss, 'rss')
    _df = nag_double_type_check_and_cast('g02gac', df, 'df')
    _b = nag_double_type_check_and_cast('g02gac', b, 'b')
    _rank = nag_integer_type_check_and_cast('g02gac', rank, 'rank')
    _se = nag_double_type_check_and_cast('g02gac', se, 'se')
    _cov = nag_double_type_check_and_cast('g02gac', cov, 'cov')
    _v = nag_double_type_check_and_cast('g02gac', v, 'v')
    g02gac_ctypes(link, mean, n, _x, tdx, m, _sx, ip, _y, _wt, _offset, _scale, expower, _rss, _df, _b, _rank, _se, _cov, _v, tdv, tol, maxiter, printiter, outfile, eps, fail)

g02gbc_ctypes = _library_cl.g02gbc
g02gbc_ctypes.restype = None
g02gbc_ctypes.argtypes = [Nag_Link, Nag_IncludeMean, Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, c_double, Integer, Integer, STRING, c_double, POINTER(NagError)]


def g02gbc(link, mean, n, x, tdx, m, sx, ip, y, binomt, wt, offset, dev, df, b, rank, se, cov, v, tdv, tol, maxiter, printiter, outfile, eps, fail):
    """
    Fits a generalized linear model with binomial errors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02gbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, sx, y, binomt, wt, offset, dev, df, b, rank, se, cov, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, binomt, wt, offset, dev, df, b, se, cov, v
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sx, rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02gbc', x, 'x')
    _sx = nag_integer_type_check_and_cast('g02gbc', sx, 'sx')
    _y = nag_double_type_check_and_cast('g02gbc', y, 'y')
    _binomt = nag_double_type_check_and_cast('g02gbc', binomt, 'binomt')
    _wt = nag_double_type_check_and_cast('g02gbc', wt, 'wt')
    _offset = nag_double_type_check_and_cast('g02gbc', offset, 'offset')
    _dev = nag_double_type_check_and_cast('g02gbc', dev, 'dev')
    _df = nag_double_type_check_and_cast('g02gbc', df, 'df')
    _b = nag_double_type_check_and_cast('g02gbc', b, 'b')
    _rank = nag_integer_type_check_and_cast('g02gbc', rank, 'rank')
    _se = nag_double_type_check_and_cast('g02gbc', se, 'se')
    _cov = nag_double_type_check_and_cast('g02gbc', cov, 'cov')
    _v = nag_double_type_check_and_cast('g02gbc', v, 'v')
    g02gbc_ctypes(link, mean, n, _x, tdx, m, _sx, ip, _y, _binomt, _wt, _offset, _dev, _df, _b, _rank, _se, _cov, _v, tdv, tol, maxiter, printiter, outfile, eps, fail)

g02gcc_ctypes = _library_cl.g02gcc
g02gcc_ctypes.restype = None
g02gcc_ctypes.argtypes = [Nag_Link, Nag_IncludeMean, Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, c_double, Integer, Integer, STRING, c_double, POINTER(NagError)]


def g02gcc(link, mean, n, x, tdx, m, sx, ip, y, wt, offset, expower, dev, df, b, rank, se, cov, v, tdv, tol, maxiter, printiter, outfile, eps, fail):
    """
    Fits a generalized linear model with Poisson errors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02gcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, sx, y, wt, offset, dev, df, b, rank, se, cov, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, wt, offset, dev, df, b, se, cov, v
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sx, rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02gcc', x, 'x')
    _sx = nag_integer_type_check_and_cast('g02gcc', sx, 'sx')
    _y = nag_double_type_check_and_cast('g02gcc', y, 'y')
    _wt = nag_double_type_check_and_cast('g02gcc', wt, 'wt')
    _offset = nag_double_type_check_and_cast('g02gcc', offset, 'offset')
    _dev = nag_double_type_check_and_cast('g02gcc', dev, 'dev')
    _df = nag_double_type_check_and_cast('g02gcc', df, 'df')
    _b = nag_double_type_check_and_cast('g02gcc', b, 'b')
    _rank = nag_integer_type_check_and_cast('g02gcc', rank, 'rank')
    _se = nag_double_type_check_and_cast('g02gcc', se, 'se')
    _cov = nag_double_type_check_and_cast('g02gcc', cov, 'cov')
    _v = nag_double_type_check_and_cast('g02gcc', v, 'v')
    g02gcc_ctypes(link, mean, n, _x, tdx, m, _sx, ip, _y, _wt, _offset, expower, _dev, _df, _b, _rank, _se, _cov, _v, tdv, tol, maxiter, printiter, outfile, eps, fail)

g02gdc_ctypes = _library_cl.g02gdc
g02gdc_ctypes.restype = None
g02gdc_ctypes.argtypes = [Nag_Link, Nag_IncludeMean, Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, c_double, Integer, Integer, STRING, c_double, POINTER(NagError)]


def g02gdc(link, mean, n, x, tdx, m, sx, ip, y, wt, offset, scale, expower, dev, df, b, rank, se, cov, v, tdv, tol, maxiter, printiter, outfile, eps, fail):
    """
    Fits a generalized linear model with gamma errors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02gdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, sx, y, wt, offset, scale, dev, df, b, rank, se, cov, v
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, wt, offset, scale, dev, df, b, se, cov, v
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sx, rank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02gdc', x, 'x')
    _sx = nag_integer_type_check_and_cast('g02gdc', sx, 'sx')
    _y = nag_double_type_check_and_cast('g02gdc', y, 'y')
    _wt = nag_double_type_check_and_cast('g02gdc', wt, 'wt')
    _offset = nag_double_type_check_and_cast('g02gdc', offset, 'offset')
    _scale = nag_double_type_check_and_cast('g02gdc', scale, 'scale')
    _dev = nag_double_type_check_and_cast('g02gdc', dev, 'dev')
    _df = nag_double_type_check_and_cast('g02gdc', df, 'df')
    _b = nag_double_type_check_and_cast('g02gdc', b, 'b')
    _rank = nag_integer_type_check_and_cast('g02gdc', rank, 'rank')
    _se = nag_double_type_check_and_cast('g02gdc', se, 'se')
    _cov = nag_double_type_check_and_cast('g02gdc', cov, 'cov')
    _v = nag_double_type_check_and_cast('g02gdc', v, 'v')
    g02gdc_ctypes(link, mean, n, _x, tdx, m, _sx, ip, _y, _wt, _offset, _scale, expower, _dev, _df, _b, _rank, _se, _cov, _v, tdv, tol, maxiter, printiter, outfile, eps, fail)

g02gkc_ctypes = _library_cl.g02gkc
g02gkc_ctypes.restype = None
g02gkc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), c_double, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02gkc(ip, nclin, v, tdv, c, tdc, b, scale, se, cov, fail):
    """
    Estimates and standard errors of parameters of a general linear model
    for given constraints.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02gkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        v, c, b, se, cov
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        v, c, b, se, cov
    """
    from .util import nag_double_type_check_and_cast
    _v = nag_double_type_check_and_cast('g02gkc', v, 'v')
    _c = nag_double_type_check_and_cast('g02gkc', c, 'c')
    _b = nag_double_type_check_and_cast('g02gkc', b, 'b')
    _se = nag_double_type_check_and_cast('g02gkc', se, 'se')
    _cov = nag_double_type_check_and_cast('g02gkc', cov, 'cov')
    g02gkc_ctypes(ip, nclin, _v, tdv, _c, tdc, _b, scale, _se, _cov, fail)

g02gnc_ctypes = _library_cl.g02gnc
g02gnc_ctypes.restype = None
g02gnc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(Nag_Boolean), POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, POINTER(NagError)]


def g02gnc(ip, rank, b, cov, v, tdv, f, est, stat, sestat, z, tol, fail):
    """
    Estimable function and the standard error of a generalized linear model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02gnc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        b, cov, v, f, stat, sestat, z
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        b, cov, v, f, stat, sestat, z
    """
    from .util import nag_double_type_check_and_cast
    _b = nag_double_type_check_and_cast('g02gnc', b, 'b')
    _cov = nag_double_type_check_and_cast('g02gnc', cov, 'cov')
    _v = nag_double_type_check_and_cast('g02gnc', v, 'v')
    _f = nag_double_type_check_and_cast('g02gnc', f, 'f')
    _stat = nag_double_type_check_and_cast('g02gnc', stat, 'stat')
    _sestat = nag_double_type_check_and_cast('g02gnc', sestat, 'sestat')
    _z = nag_double_type_check_and_cast('g02gnc', z, 'z')
    g02gnc_ctypes(ip, rank, _b, _cov, _v, tdv, _f, est, _stat, _sestat, _z, tol, fail)

g02gpc_ctypes = _library_cl.g02gpc
g02gpc_ctypes.restype = None
g02gpc_ctypes.argtypes = [Nag_Distributions, Nag_Link, Nag_IncludeMean, Integer, POINTER(c_double), Integer, Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, c_double, POINTER(c_double), POINTER(c_double), Nag_Boolean, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g02gpc(errfn, link, mean, n, x, tdx, m, sx, ip, binomt, offset, wt, scale, expower, b, cov, vfobs, eta, seeta, pred, sepred, fail):
    """
    Computes a predicted value and its associated standard error based on a
    previously fitted generalized linear model.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02gpc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, sx, binomt, offset, wt, b, cov, eta, seeta, pred, sepred
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, binomt, offset, wt, b, cov, eta, seeta, pred, sepred
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        sx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02gpc', x, 'x')
    _sx = nag_integer_type_check_and_cast('g02gpc', sx, 'sx')
    _binomt = nag_double_type_check_and_cast('g02gpc', binomt, 'binomt')
    _offset = nag_double_type_check_and_cast('g02gpc', offset, 'offset')
    _wt = nag_double_type_check_and_cast('g02gpc', wt, 'wt')
    _b = nag_double_type_check_and_cast('g02gpc', b, 'b')
    _cov = nag_double_type_check_and_cast('g02gpc', cov, 'cov')
    _eta = nag_double_type_check_and_cast('g02gpc', eta, 'eta')
    _seeta = nag_double_type_check_and_cast('g02gpc', seeta, 'seeta')
    _pred = nag_double_type_check_and_cast('g02gpc', pred, 'pred')
    _sepred = nag_double_type_check_and_cast('g02gpc', sepred, 'sepred')
    g02gpc_ctypes(errfn, link, mean, n, _x, tdx, m, _sx, ip, _binomt, _offset, _wt, scale, expower, _b, _cov, vfobs, _eta, _seeta, _pred, _sepred, fail)

g02hac_ctypes = _library_cl.g02hac
g02hac_ctypes.restype = None
g02hac_ctypes.argtypes = [Nag_RegType, Nag_PsiFun, Nag_SigmaEst, Nag_CovMatrixEst, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), c_double, POINTER(c_double), c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), c_double, Integer, Integer, STRING, POINTER(c_double), POINTER(NagError)]


def g02hac(regtype, psifun, sigmaest, covmatest, n, m, x, tdx, y, cpsi, hpsi, cucv, dchi, theta, sigma, c, tdc, rs, wt, tol, maxiter, printiter, outfile, info, fail):
    """
    Robust regression, standard M-estimates.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02hac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, hpsi, theta, sigma, c, rs, wt, info
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, hpsi, theta, sigma, c, rs, wt, info
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02hac', x, 'x')
    _y = nag_double_type_check_and_cast('g02hac', y, 'y')
    _hpsi = nag_double_type_check_and_cast('g02hac', hpsi, 'hpsi')
    _theta = nag_double_type_check_and_cast('g02hac', theta, 'theta')
    _sigma = nag_double_type_check_and_cast('g02hac', sigma, 'sigma')
    _c = nag_double_type_check_and_cast('g02hac', c, 'c')
    _rs = nag_double_type_check_and_cast('g02hac', rs, 'rs')
    _wt = nag_double_type_check_and_cast('g02hac', wt, 'wt')
    _info = nag_double_type_check_and_cast('g02hac', info, 'info')
    g02hac_ctypes(regtype, psifun, sigmaest, covmatest, n, m, _x, tdx, _y, cpsi, _hpsi, cucv, dchi, _theta, _sigma, _c, tdc, _rs, _wt, tol, maxiter, printiter, outfile, _info, fail)

g02hbc_ctypes = _library_cl.g02hbc
g02hbc_ctypes.restype = None
g02hbc_ctypes.argtypes = [Nag_OrderType, NAG_G02HBC_UCV, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), c_double, c_double, c_double, Integer, Integer, STRING, POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def g02hbc(order, ucv, n, m, x, pdx, a, z, bl, bd, tol, maxit, nitmon, outfile, nit, comm, fail):
    """
    Robust regression, compute weights for use with g02hdc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02hbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, a, z, nit
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, a, z
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nit
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02hbc', x, 'x')
    _a = nag_double_type_check_and_cast('g02hbc', a, 'a')
    _z = nag_double_type_check_and_cast('g02hbc', z, 'z')
    _nit = nag_integer_type_check_and_cast('g02hbc', nit, 'nit')
    g02hbc_ctypes(order, ucv, n, m, _x, pdx, _a, _z, bl, bd, tol, maxit, nitmon, outfile, _nit, comm, fail)

g02hdc_ctypes = _library_cl.g02hdc
g02hdc_ctypes.restype = None
g02hdc_ctypes.argtypes = [Nag_OrderType, NAG_G02HDC_CHI, NAG_G02HDC_PSI, c_double, c_double, Nag_RegType, Nag_SigmaEst, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), c_double, c_double, Integer, Integer, STRING, POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def g02hdc(order, chi, psi, psip0, beta, regtype, sigmaest, n, m, x, pdx, y, wgt, theta, k, sigma, rs, tol, eps, maxit, nitmon, outfile, nit, comm, fail):
    """
    Robust regression, compute regression with user-supplied functions and
    weights.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02hdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, wgt, theta, k, sigma, rs, nit
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, wgt, theta, sigma, rs
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        k, nit
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02hdc', x, 'x')
    _y = nag_double_type_check_and_cast('g02hdc', y, 'y')
    _wgt = nag_double_type_check_and_cast('g02hdc', wgt, 'wgt')
    _theta = nag_double_type_check_and_cast('g02hdc', theta, 'theta')
    _k = nag_integer_type_check_and_cast('g02hdc', k, 'k')
    _sigma = nag_double_type_check_and_cast('g02hdc', sigma, 'sigma')
    _rs = nag_double_type_check_and_cast('g02hdc', rs, 'rs')
    _nit = nag_integer_type_check_and_cast('g02hdc', nit, 'nit')
    g02hdc_ctypes(order, chi, psi, psip0, beta, regtype, sigmaest, n, m, _x, pdx, _y, _wgt, _theta, _k, _sigma, _rs, tol, eps, maxit, nitmon, outfile, _nit, comm, fail)

g02hfc_ctypes = _library_cl.g02hfc
g02hfc_ctypes.restype = None
g02hfc_ctypes.argtypes = [Nag_OrderType, NAG_G02HFC_PSI, NAG_G02HFC_PSP, Nag_RegType, Nag_CovMatrixEst, c_double, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(Nag_Comm), POINTER(NagError)]


def g02hfc(order, psi, psp, regtype, covmatest, sigma, n, m, x, pdx, rs, wgt, cov, pdc, commarr, comm, fail):
    """
    Robust regression, variance-covariance matrix following g02hdc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02hfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, rs, wgt, cov, commarr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, rs, wgt, cov, commarr
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02hfc', x, 'x')
    _rs = nag_double_type_check_and_cast('g02hfc', rs, 'rs')
    _wgt = nag_double_type_check_and_cast('g02hfc', wgt, 'wgt')
    _cov = nag_double_type_check_and_cast('g02hfc', cov, 'cov')
    _commarr = nag_double_type_check_and_cast('g02hfc', commarr, 'commarr')
    g02hfc_ctypes(order, psi, psp, regtype, covmatest, sigma, n, m, _x, pdx, _rs, _wgt, _cov, pdc, _commarr, comm, fail)

g02hkc_ctypes = _library_cl.g02hkc
g02hkc_ctypes.restype = None
g02hkc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), POINTER(c_double), Integer, Integer, STRING, c_double, POINTER(Integer), POINTER(NagError)]


def g02hkc(n, m, x, tdx, eps, cov, theta, maxiter, printiter, outfile, tol, _iter, fail):
    """
    Robust estimation of a correlation matrix, Huber's weight function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02hkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, cov, theta, _iter
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, cov, theta
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        _iter
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02hkc', x, 'x')
    _cov = nag_double_type_check_and_cast('g02hkc', cov, 'cov')
    _theta = nag_double_type_check_and_cast('g02hkc', theta, 'theta')
    __iter = nag_integer_type_check_and_cast('g02hkc', _iter, '_iter')
    g02hkc_ctypes(n, m, _x, tdx, eps, _cov, _theta, maxiter, printiter, outfile, tol, __iter, fail)

g02hlc_ctypes = _library_cl.g02hlc
g02hlc_ctypes.restype = None
g02hlc_ctypes.argtypes = [Nag_OrderType, NAG_G02HLC_UCV, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, c_double, Integer, Integer, STRING, c_double, POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def g02hlc(order, ucv, indm, n, m, x, pdx, cov, a, wt, theta, bl, bd, maxit, nitmon, outfile, tol, nit, comm, fail):
    """
    Calculates a robust estimation of a correlation matrix, user-supplied
    weight function plus derivatives.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02hlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, cov, a, wt, theta, nit
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, cov, a, wt, theta
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nit
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02hlc', x, 'x')
    _cov = nag_double_type_check_and_cast('g02hlc', cov, 'cov')
    _a = nag_double_type_check_and_cast('g02hlc', a, 'a')
    _wt = nag_double_type_check_and_cast('g02hlc', wt, 'wt')
    _theta = nag_double_type_check_and_cast('g02hlc', theta, 'theta')
    _nit = nag_integer_type_check_and_cast('g02hlc', nit, 'nit')
    g02hlc_ctypes(order, ucv, indm, n, m, _x, pdx, _cov, _a, _wt, _theta, bl, bd, maxit, nitmon, outfile, tol, _nit, comm, fail)

g02hmc_ctypes = _library_cl.g02hmc
g02hmc_ctypes.restype = None
g02hmc_ctypes.argtypes = [Nag_OrderType, NAG_G02HMC_UCV, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), c_double, c_double, Integer, Integer, STRING, c_double, POINTER(Integer), POINTER(Nag_Comm), POINTER(NagError)]


def g02hmc(order, ucv, indm, n, m, x, pdx, cov, a, wt, theta, bl, bd, maxit, nitmon, outfile, tol, nit, comm, fail):
    """
    Calculates a robust estimation of a correlation matrix, user-supplied
    weight function.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02hmc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, cov, a, wt, theta, nit
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, cov, a, wt, theta
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nit
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02hmc', x, 'x')
    _cov = nag_double_type_check_and_cast('g02hmc', cov, 'cov')
    _a = nag_double_type_check_and_cast('g02hmc', a, 'a')
    _wt = nag_double_type_check_and_cast('g02hmc', wt, 'wt')
    _theta = nag_double_type_check_and_cast('g02hmc', theta, 'theta')
    _nit = nag_integer_type_check_and_cast('g02hmc', nit, 'nit')
    g02hmc_ctypes(order, ucv, indm, n, m, _x, pdx, _cov, _a, _wt, _theta, bl, bd, maxit, nitmon, outfile, tol, _nit, comm, fail)

g02jac_ctypes = _library_cl.g02jac
g02jac_ctypes.restype = None
g02jac_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, Integer, Integer, POINTER(Integer), Integer, Integer, POINTER(Integer), Integer, POINTER(Integer), Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, c_double, POINTER(Integer), POINTER(NagError)]


def g02jac(n, ncol, dat, tddat, levels, yvid, cwid, nfv, fvid, fint, nrv, rvid, nvpr, vpr, rint, svid, gamma, nff, nrf, df, reml, lb, b, se, maxit, tol, warn, fail):
    """
    Linear mixed effects regression using Restricted Maximum Likelihood
    (REML).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02jac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dat, levels, fvid, rvid, vpr, gamma, nff, nrf, df, reml, b, se, warn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dat, gamma, reml, b, se
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        levels, fvid, rvid, vpr, nff, nrf, df, warn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _dat = nag_double_type_check_and_cast('g02jac', dat, 'dat')
    _levels = nag_integer_type_check_and_cast('g02jac', levels, 'levels')
    _fvid = nag_integer_type_check_and_cast('g02jac', fvid, 'fvid')
    _rvid = nag_integer_type_check_and_cast('g02jac', rvid, 'rvid')
    _vpr = nag_integer_type_check_and_cast('g02jac', vpr, 'vpr')
    _gamma = nag_double_type_check_and_cast('g02jac', gamma, 'gamma')
    _nff = nag_integer_type_check_and_cast('g02jac', nff, 'nff')
    _nrf = nag_integer_type_check_and_cast('g02jac', nrf, 'nrf')
    _df = nag_integer_type_check_and_cast('g02jac', df, 'df')
    _reml = nag_double_type_check_and_cast('g02jac', reml, 'reml')
    _b = nag_double_type_check_and_cast('g02jac', b, 'b')
    _se = nag_double_type_check_and_cast('g02jac', se, 'se')
    _warn = nag_integer_type_check_and_cast('g02jac', warn, 'warn')
    g02jac_ctypes(n, ncol, _dat, tddat, _levels, yvid, cwid, nfv, _fvid, fint, nrv, _rvid, nvpr, _vpr, rint, svid, _gamma, _nff, _nrf, _df, _reml, lb, _b, _se, maxit, tol, _warn, fail)

g02jbc_ctypes = _library_cl.g02jbc
g02jbc_ctypes.restype = None
g02jbc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, Integer, Integer, POINTER(Integer), Integer, Integer, POINTER(Integer), Integer, POINTER(Integer), Integer, Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, c_double, POINTER(Integer), POINTER(NagError)]


def g02jbc(n, ncol, dat, tddat, levels, yvid, cwid, nfv, fvid, fint, nrv, rvid, nvpr, vpr, rint, svid, gamma, nff, nrf, df, ml, lb, b, se, maxit, tol, warn, fail):
    """
    Linear mixed effects regression using Maximum Likelihood (ML).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02jbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dat, levels, fvid, rvid, vpr, gamma, nff, nrf, df, ml, b, se, warn
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dat, gamma, ml, b, se
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        levels, fvid, rvid, vpr, nff, nrf, df, warn
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _dat = nag_double_type_check_and_cast('g02jbc', dat, 'dat')
    _levels = nag_integer_type_check_and_cast('g02jbc', levels, 'levels')
    _fvid = nag_integer_type_check_and_cast('g02jbc', fvid, 'fvid')
    _rvid = nag_integer_type_check_and_cast('g02jbc', rvid, 'rvid')
    _vpr = nag_integer_type_check_and_cast('g02jbc', vpr, 'vpr')
    _gamma = nag_double_type_check_and_cast('g02jbc', gamma, 'gamma')
    _nff = nag_integer_type_check_and_cast('g02jbc', nff, 'nff')
    _nrf = nag_integer_type_check_and_cast('g02jbc', nrf, 'nrf')
    _df = nag_integer_type_check_and_cast('g02jbc', df, 'df')
    _ml = nag_double_type_check_and_cast('g02jbc', ml, 'ml')
    _b = nag_double_type_check_and_cast('g02jbc', b, 'b')
    _se = nag_double_type_check_and_cast('g02jbc', se, 'se')
    _warn = nag_integer_type_check_and_cast('g02jbc', warn, 'warn')
    g02jbc_ctypes(n, ncol, _dat, tddat, _levels, yvid, cwid, nfv, _fvid, fint, nrv, _rvid, nvpr, _vpr, rint, svid, _gamma, _nff, _nrf, _df, _ml, lb, _b, _se, maxit, tol, _warn, fail)

g02jcc_ctypes = _library_cl.g02jcc
g02jcc_ctypes.restype = None
g02jcc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(Integer), Integer, Integer, POINTER(Integer), Integer, POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(NagError)]


def g02jcc(order, n, ncol, dat, pddat, levels, y, wt, fixed, lfixed, nrndm, rndm, lrndm, nff, nlsv, nrf, rcomm, lrcomm, icomm, licomm, fail):
    """
    Hierarchical mixed effects regression, initialization function for
    g02jdc and g02jec.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02jcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dat, levels, y, wt, fixed, rndm, nff, nlsv, nrf, rcomm, icomm
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dat, y, wt, rcomm
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        levels, fixed, rndm, nff, nlsv, nrf, icomm
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _dat = nag_double_type_check_and_cast('g02jcc', dat, 'dat')
    _levels = nag_integer_type_check_and_cast('g02jcc', levels, 'levels')
    _y = nag_double_type_check_and_cast('g02jcc', y, 'y')
    _wt = nag_double_type_check_and_cast('g02jcc', wt, 'wt')
    _fixed = nag_integer_type_check_and_cast('g02jcc', fixed, 'fixed')
    _rndm = nag_integer_type_check_and_cast('g02jcc', rndm, 'rndm')
    _nff = nag_integer_type_check_and_cast('g02jcc', nff, 'nff')
    _nlsv = nag_integer_type_check_and_cast('g02jcc', nlsv, 'nlsv')
    _nrf = nag_integer_type_check_and_cast('g02jcc', nrf, 'nrf')
    _rcomm = nag_double_type_check_and_cast('g02jcc', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('g02jcc', icomm, 'icomm')
    g02jcc_ctypes(order, n, ncol, _dat, pddat, _levels, _y, _wt, _fixed, lfixed, nrndm, _rndm, lrndm, _nff, _nlsv, _nrf, _rcomm, lrcomm, _icomm, licomm, fail)

g02jdc_ctypes = _library_cl.g02jdc
g02jdc_ctypes.restype = None
g02jdc_ctypes.argtypes = [Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g02jdc(lvpr, vpr, nvpr, gamma, effn, rnkx, ncov, lnlike, lb, _id, pdid, b, se, czz, pdczz, cxx, pdcxx, cxz, pdcxz, rcomm, icomm, iopt, liopt, ropt, lropt, fail):
    """
    Hierarchical mixed effects regression using Restricted Maximum
    Likelihood (REML).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02jdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        vpr, gamma, effn, rnkx, ncov, lnlike, _id, b, se, czz, cxx, cxz, rcomm, icomm, iopt, ropt
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        gamma, lnlike, b, se, czz, cxx, cxz, rcomm, ropt
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        vpr, effn, rnkx, ncov, _id, icomm, iopt
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _vpr = nag_integer_type_check_and_cast('g02jdc', vpr, 'vpr')
    _gamma = nag_double_type_check_and_cast('g02jdc', gamma, 'gamma')
    _effn = nag_integer_type_check_and_cast('g02jdc', effn, 'effn')
    _rnkx = nag_integer_type_check_and_cast('g02jdc', rnkx, 'rnkx')
    _ncov = nag_integer_type_check_and_cast('g02jdc', ncov, 'ncov')
    _lnlike = nag_double_type_check_and_cast('g02jdc', lnlike, 'lnlike')
    __id = nag_integer_type_check_and_cast('g02jdc', _id, '_id')
    _b = nag_double_type_check_and_cast('g02jdc', b, 'b')
    _se = nag_double_type_check_and_cast('g02jdc', se, 'se')
    _czz = nag_double_type_check_and_cast('g02jdc', czz, 'czz')
    _cxx = nag_double_type_check_and_cast('g02jdc', cxx, 'cxx')
    _cxz = nag_double_type_check_and_cast('g02jdc', cxz, 'cxz')
    _rcomm = nag_double_type_check_and_cast('g02jdc', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('g02jdc', icomm, 'icomm')
    _iopt = nag_integer_type_check_and_cast('g02jdc', iopt, 'iopt')
    _ropt = nag_double_type_check_and_cast('g02jdc', ropt, 'ropt')
    g02jdc_ctypes(lvpr, _vpr, nvpr, _gamma, _effn, _rnkx, _ncov, _lnlike, lb, __id, pdid, _b, _se, _czz, pdczz, _cxx, pdcxx, _cxz, pdcxz, _rcomm, _icomm, _iopt, liopt, _ropt, lropt, fail)

g02jec_ctypes = _library_cl.g02jec
g02jec_ctypes.restype = None
g02jec_ctypes.argtypes = [Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g02jec(lvpr, vpr, nvpr, gamma, effn, rnkx, ncov, lnlike, lb, _id, pdid, b, se, czz, pdczz, cxx, pdcxx, cxz, pdcxz, rcomm, icomm, iopt, liopt, ropt, lropt, fail):
    """
    Hierarchical mixed effects regression using Maximum Likelihood (ML).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02jec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        vpr, gamma, effn, rnkx, ncov, lnlike, _id, b, se, czz, cxx, cxz, rcomm, icomm, iopt, ropt
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        gamma, lnlike, b, se, czz, cxx, cxz, rcomm, ropt
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        vpr, effn, rnkx, ncov, _id, icomm, iopt
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _vpr = nag_integer_type_check_and_cast('g02jec', vpr, 'vpr')
    _gamma = nag_double_type_check_and_cast('g02jec', gamma, 'gamma')
    _effn = nag_integer_type_check_and_cast('g02jec', effn, 'effn')
    _rnkx = nag_integer_type_check_and_cast('g02jec', rnkx, 'rnkx')
    _ncov = nag_integer_type_check_and_cast('g02jec', ncov, 'ncov')
    _lnlike = nag_double_type_check_and_cast('g02jec', lnlike, 'lnlike')
    __id = nag_integer_type_check_and_cast('g02jec', _id, '_id')
    _b = nag_double_type_check_and_cast('g02jec', b, 'b')
    _se = nag_double_type_check_and_cast('g02jec', se, 'se')
    _czz = nag_double_type_check_and_cast('g02jec', czz, 'czz')
    _cxx = nag_double_type_check_and_cast('g02jec', cxx, 'cxx')
    _cxz = nag_double_type_check_and_cast('g02jec', cxz, 'cxz')
    _rcomm = nag_double_type_check_and_cast('g02jec', rcomm, 'rcomm')
    _icomm = nag_integer_type_check_and_cast('g02jec', icomm, 'icomm')
    _iopt = nag_integer_type_check_and_cast('g02jec', iopt, 'iopt')
    _ropt = nag_double_type_check_and_cast('g02jec', ropt, 'ropt')
    g02jec_ctypes(lvpr, _vpr, nvpr, _gamma, _effn, _rnkx, _ncov, _lnlike, lb, __id, pdid, _b, _se, _czz, pdczz, _cxx, pdcxx, _cxz, pdcxz, _rcomm, _icomm, _iopt, liopt, _ropt, lropt, fail)

g02kac_ctypes = _library_cl.g02kac
g02kac_ctypes.restype = None
g02kac_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, c_double, POINTER(c_double), POINTER(c_double), Nag_PredictError, POINTER(Integer), c_double, POINTER(c_double), Nag_EstimatesOption, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), Nag_OptionLOO, POINTER(c_double), POINTER(NagError)]


def g02kac(order, n, m, x, pdx, isx, ip, tau, y, h, opt, niter, tol, nep, orig, b, vif, res, rss, df, optloo, perr, fail):
    """
    Ridge regression, optimizing a ridge regression parameter.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02kac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, y, h, niter, nep, b, vif, res, rss, df, perr
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, h, nep, b, vif, res, rss, perr
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx, niter, df
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02kac', x, 'x')
    _isx = nag_integer_type_check_and_cast('g02kac', isx, 'isx')
    _y = nag_double_type_check_and_cast('g02kac', y, 'y')
    _h = nag_double_type_check_and_cast('g02kac', h, 'h')
    _niter = nag_integer_type_check_and_cast('g02kac', niter, 'niter')
    _nep = nag_double_type_check_and_cast('g02kac', nep, 'nep')
    _b = nag_double_type_check_and_cast('g02kac', b, 'b')
    _vif = nag_double_type_check_and_cast('g02kac', vif, 'vif')
    _res = nag_double_type_check_and_cast('g02kac', res, 'res')
    _rss = nag_double_type_check_and_cast('g02kac', rss, 'rss')
    _df = nag_integer_type_check_and_cast('g02kac', df, 'df')
    _perr = nag_double_type_check_and_cast('g02kac', perr, 'perr')
    g02kac_ctypes(order, n, m, _x, pdx, _isx, ip, tau, _y, _h, opt, _niter, tol, _nep, orig, _b, _vif, _res, _rss, _df, optloo, _perr, fail)

g02kbc_ctypes = _library_cl.g02kbc
g02kbc_ctypes.restype = None
g02kbc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Nag_ParaOption, POINTER(c_double), Integer, Nag_VIFOption, POINTER(c_double), Integer, Integer, POINTER(Nag_PredictError), POINTER(c_double), Integer, POINTER(NagError)]


def g02kbc(order, n, m, x, pdx, isx, ip, y, lh, h, nep, wantb, b, pdb, wantvf, vf, pdvf, lpec, pec, pe, pdpe, fail):
    """
    Ridge regression using a number of supplied ridge regression parameters.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02kbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, y, h, nep, b, vf, pec, pe
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, h, nep, b, vf, pe
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx
    TypeError will be raised when the following arguments are not of data type
    Nag_PredictError:
        pec
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    from .util import nag_enum_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02kbc', x, 'x')
    _isx = nag_integer_type_check_and_cast('g02kbc', isx, 'isx')
    _y = nag_double_type_check_and_cast('g02kbc', y, 'y')
    _h = nag_double_type_check_and_cast('g02kbc', h, 'h')
    _nep = nag_double_type_check_and_cast('g02kbc', nep, 'nep')
    _b = nag_double_type_check_and_cast('g02kbc', b, 'b')
    _vf = nag_double_type_check_and_cast('g02kbc', vf, 'vf')
    _pe = nag_double_type_check_and_cast('g02kbc', pe, 'pe')
    _pec = nag_enum_type_check_and_cast('g02kbc', pec, 'pec', Nag_PredictError)
    g02kbc_ctypes(order, n, m, _x, pdx, _isx, ip, _y, lh, _h, _nep, wantb, _b, pdb, wantvf, _vf, pdvf, lpec, _pec, _pe, pdpe, fail)

g02lac_ctypes = _library_cl.g02lac
g02lac_ctypes.restype = None
g02lac_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Nag_ScalePredictor, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def g02lac(order, n, mx, x, pdx, isx, ip, my, y, pdy, xbar, ybar, iscale, xstd, ystd, maxfac, xres, pdxres, yres, pdyres, w, pdw, p, pdp, t, pdt, c, pdc, u, pdu, xcv, ycv, pdycv, fail):
    """
    Partial least squares (PLS) regression using singular value
    decomposition.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02lac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, y, xbar, ybar, xstd, ystd, xres, yres, w, p, t, c, u, xcv, ycv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, xbar, ybar, xstd, ystd, xres, yres, w, p, t, c, u, xcv, ycv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02lac', x, 'x')
    _isx = nag_integer_type_check_and_cast('g02lac', isx, 'isx')
    _y = nag_double_type_check_and_cast('g02lac', y, 'y')
    _xbar = nag_double_type_check_and_cast('g02lac', xbar, 'xbar')
    _ybar = nag_double_type_check_and_cast('g02lac', ybar, 'ybar')
    _xstd = nag_double_type_check_and_cast('g02lac', xstd, 'xstd')
    _ystd = nag_double_type_check_and_cast('g02lac', ystd, 'ystd')
    _xres = nag_double_type_check_and_cast('g02lac', xres, 'xres')
    _yres = nag_double_type_check_and_cast('g02lac', yres, 'yres')
    _w = nag_double_type_check_and_cast('g02lac', w, 'w')
    _p = nag_double_type_check_and_cast('g02lac', p, 'p')
    _t = nag_double_type_check_and_cast('g02lac', t, 't')
    _c = nag_double_type_check_and_cast('g02lac', c, 'c')
    _u = nag_double_type_check_and_cast('g02lac', u, 'u')
    _xcv = nag_double_type_check_and_cast('g02lac', xcv, 'xcv')
    _ycv = nag_double_type_check_and_cast('g02lac', ycv, 'ycv')
    g02lac_ctypes(order, n, mx, _x, pdx, _isx, ip, my, _y, pdy, _xbar, _ybar, iscale, _xstd, _ystd, maxfac, _xres, pdxres, _yres, pdyres, _w, pdw, _p, pdp, _t, pdt, _c, pdc, _u, pdu, _xcv, _ycv, pdycv, fail)

g02lbc_ctypes = _library_cl.g02lbc
g02lbc_ctypes.restype = None
g02lbc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Nag_ScalePredictor, POINTER(c_double), POINTER(c_double), Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def g02lbc(order, n, mx, x, pdx, isx, ip, my, y, pdy, xbar, ybar, iscale, xstd, ystd, maxfac, maxit, tau, xres, pdxres, yres, pdyres, w, pdw, p, pdp, t, pdt, c, pdc, u, pdu, xcv, ycv, pdycv, fail):
    """
    Partial least squares (PLS) regression using Wold's iterative method.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02lbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, isx, y, xbar, ybar, xstd, ystd, xres, yres, w, p, t, c, u, xcv, ycv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, xbar, ybar, xstd, ystd, xres, yres, w, p, t, c, u, xcv, ycv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02lbc', x, 'x')
    _isx = nag_integer_type_check_and_cast('g02lbc', isx, 'isx')
    _y = nag_double_type_check_and_cast('g02lbc', y, 'y')
    _xbar = nag_double_type_check_and_cast('g02lbc', xbar, 'xbar')
    _ybar = nag_double_type_check_and_cast('g02lbc', ybar, 'ybar')
    _xstd = nag_double_type_check_and_cast('g02lbc', xstd, 'xstd')
    _ystd = nag_double_type_check_and_cast('g02lbc', ystd, 'ystd')
    _xres = nag_double_type_check_and_cast('g02lbc', xres, 'xres')
    _yres = nag_double_type_check_and_cast('g02lbc', yres, 'yres')
    _w = nag_double_type_check_and_cast('g02lbc', w, 'w')
    _p = nag_double_type_check_and_cast('g02lbc', p, 'p')
    _t = nag_double_type_check_and_cast('g02lbc', t, 't')
    _c = nag_double_type_check_and_cast('g02lbc', c, 'c')
    _u = nag_double_type_check_and_cast('g02lbc', u, 'u')
    _xcv = nag_double_type_check_and_cast('g02lbc', xcv, 'xcv')
    _ycv = nag_double_type_check_and_cast('g02lbc', ycv, 'ycv')
    g02lbc_ctypes(order, n, mx, _x, pdx, _isx, ip, my, _y, pdy, _xbar, _ybar, iscale, _xstd, _ystd, maxfac, maxit, tau, _xres, pdxres, _yres, pdyres, _w, pdw, _p, pdp, _t, pdt, _c, pdc, _u, pdu, _xcv, _ycv, pdycv, fail)

g02lcc_ctypes = _library_cl.g02lcc
g02lcc_ctypes.restype = None
g02lcc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, c_double, POINTER(c_double), Integer, Nag_EstimatesOption, POINTER(c_double), POINTER(c_double), Nag_ScalePredictor, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g02lcc(order, ip, my, maxfac, nfact, p, pdp, c, pdc, w, pdw, rcond, b, pdb, orig, xbar, ybar, iscale, xstd, ystd, ob, pdob, vipopt, ycv, pdycv, vip, pdvip, fail):
    """
    PLS parameter estimates following partial least squares regression by
    g02lac or g02lbc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02lcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        p, c, w, b, xbar, ybar, xstd, ystd, ob, ycv, vip
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        p, c, w, b, xbar, ybar, xstd, ystd, ob, ycv, vip
    """
    from .util import nag_double_type_check_and_cast
    _p = nag_double_type_check_and_cast('g02lcc', p, 'p')
    _c = nag_double_type_check_and_cast('g02lcc', c, 'c')
    _w = nag_double_type_check_and_cast('g02lcc', w, 'w')
    _b = nag_double_type_check_and_cast('g02lcc', b, 'b')
    _xbar = nag_double_type_check_and_cast('g02lcc', xbar, 'xbar')
    _ybar = nag_double_type_check_and_cast('g02lcc', ybar, 'ybar')
    _xstd = nag_double_type_check_and_cast('g02lcc', xstd, 'xstd')
    _ystd = nag_double_type_check_and_cast('g02lcc', ystd, 'ystd')
    _ob = nag_double_type_check_and_cast('g02lcc', ob, 'ob')
    _ycv = nag_double_type_check_and_cast('g02lcc', ycv, 'ycv')
    _vip = nag_double_type_check_and_cast('g02lcc', vip, 'vip')
    g02lcc_ctypes(order, ip, my, maxfac, nfact, _p, pdp, _c, pdc, _w, pdw, rcond, _b, pdb, orig, _xbar, _ybar, iscale, _xstd, _ystd, _ob, pdob, vipopt, _ycv, pdycv, _vip, pdvip, fail)

g02ldc_ctypes = _library_cl.g02ldc
g02ldc_ctypes.restype = None
g02ldc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Nag_EstimatesOption, POINTER(c_double), POINTER(c_double), Nag_ScalePredictor, POINTER(c_double), POINTER(c_double), POINTER(c_double), Integer, Integer, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g02ldc(order, ip, my, orig, xbar, ybar, iscale, xstd, ystd, b, pdb, n, mz, isz, z, pdz, yhat, pdyhat, fail):
    """
    PLS predictions based on parameter estimates from g02lcc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02ldc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        xbar, ybar, xstd, ystd, b, isz, z, yhat
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        xbar, ybar, xstd, ystd, b, z, yhat
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isz
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _xbar = nag_double_type_check_and_cast('g02ldc', xbar, 'xbar')
    _ybar = nag_double_type_check_and_cast('g02ldc', ybar, 'ybar')
    _xstd = nag_double_type_check_and_cast('g02ldc', xstd, 'xstd')
    _ystd = nag_double_type_check_and_cast('g02ldc', ystd, 'ystd')
    _b = nag_double_type_check_and_cast('g02ldc', b, 'b')
    _isz = nag_integer_type_check_and_cast('g02ldc', isz, 'isz')
    _z = nag_double_type_check_and_cast('g02ldc', z, 'z')
    _yhat = nag_double_type_check_and_cast('g02ldc', yhat, 'yhat')
    g02ldc_ctypes(order, ip, my, orig, _xbar, _ybar, iscale, _xstd, _ystd, _b, pdb, n, mz, _isz, _z, pdz, _yhat, pdyhat, fail)

g02mac_ctypes = _library_cl.g02mac
g02mac_ctypes.restype = None
g02mac_ctypes.argtypes = [Nag_LARSModelType, Nag_LARSPreProcess, Nag_LARSPreProcess, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def g02mac(mtype, pred, prey, n, m, d, pdd, isx, y, mnstep, ip, nstep, b, pdb, fitsum, ropt, lropt, fail):
    """
    Least angle regression (LARS), least absolute shrinkage and selection
    operator (LASSO) and forward stagewise regression.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02mac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        d, isx, y, ip, nstep, b, fitsum, ropt
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        d, y, b, fitsum, ropt
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx, ip, nstep
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _d = nag_double_type_check_and_cast('g02mac', d, 'd')
    _y = nag_double_type_check_and_cast('g02mac', y, 'y')
    _b = nag_double_type_check_and_cast('g02mac', b, 'b')
    _fitsum = nag_double_type_check_and_cast('g02mac', fitsum, 'fitsum')
    _ropt = nag_double_type_check_and_cast('g02mac', ropt, 'ropt')
    _isx = nag_integer_type_check_and_cast('g02mac', isx, 'isx')
    _ip = nag_integer_type_check_and_cast('g02mac', ip, 'ip')
    _nstep = nag_integer_type_check_and_cast('g02mac', nstep, 'nstep')
    g02mac_ctypes(mtype, pred, prey, n, m, _d, pdd, _isx, _y, mnstep, _ip, _nstep, _b, pdb, _fitsum, _ropt, lropt, fail)

nag_lars = g02mac

g02mbc_ctypes = _library_cl.g02mbc
g02mbc_ctypes.restype = None
g02mbc_ctypes.argtypes = [Nag_LARSModelType, Nag_LARSPreProcess, Nag_LARSPreProcess, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), c_double, Integer, POINTER(Integer), POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(NagError)]


def g02mbc(mtype, pred, intcpt, n, m, dtd, pddtd, isx, dty, yty, mnstep, ip, nstep, b, pdb, fitsum, ropt, lropt, fail):
    """
    Least Angle Regression (LARS), Least Absolute Shrinkage and Selection
    Operator (LASSO) and forward stagewise regression using the cross-
    products matrix.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02mbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dtd, isx, dty, ip, nstep, b, fitsum, ropt
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dtd, dty, b, fitsum, ropt
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx, ip, nstep
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _dtd = nag_double_type_check_and_cast('g02mbc', dtd, 'dtd')
    _dty = nag_double_type_check_and_cast('g02mbc', dty, 'dty')
    _b = nag_double_type_check_and_cast('g02mbc', b, 'b')
    _fitsum = nag_double_type_check_and_cast('g02mbc', fitsum, 'fitsum')
    _ropt = nag_double_type_check_and_cast('g02mbc', ropt, 'ropt')
    _isx = nag_integer_type_check_and_cast('g02mbc', isx, 'isx')
    _ip = nag_integer_type_check_and_cast('g02mbc', ip, 'ip')
    _nstep = nag_integer_type_check_and_cast('g02mbc', nstep, 'nstep')
    g02mbc_ctypes(mtype, pred, intcpt, n, m, _dtd, pddtd, _isx, _dty, yty, mnstep, _ip, _nstep, _b, pdb, _fitsum, _ropt, lropt, fail)

nag_lars_xtx = g02mbc

g02mcc_ctypes = _library_cl.g02mcc
g02mcc_ctypes.restype = None
g02mcc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), Nag_LARSTargetType, POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g02mcc(nstep, ip, b, pdb, fitsum, ktype, nk, lnk, nb, pdnb, fail):
    """
    Calculates additional parameter estimates following Least Angle
    Regression (LARS), Least Absolute Shrinkage and Selection Operator
    (LASSO) or forward stagewise regression.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02mcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        b, fitsum, nk, nb
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        b, fitsum, nk, nb
    """
    from .util import nag_double_type_check_and_cast
    _b = nag_double_type_check_and_cast('g02mcc', b, 'b')
    _fitsum = nag_double_type_check_and_cast('g02mcc', fitsum, 'fitsum')
    _nk = nag_double_type_check_and_cast('g02mcc', nk, 'nk')
    _nb = nag_double_type_check_and_cast('g02mcc', nb, 'nb')
    g02mcc_ctypes(nstep, ip, _b, pdb, _fitsum, ktype, _nk, lnk, _nb, pdnb, fail)

nag_lars_param = g02mcc

g02qfc_ctypes = _library_cl.g02qfc
g02qfc_ctypes.restype = None
g02qfc_ctypes.argtypes = [Integer, Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g02qfc(n, m, x, y, ntau, tau, df, b, bl, bu, info, fail):
    """
    Linear quantile regression, simple interface, independent, identically
    distributed (IID) errors.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02qfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, tau, df, b, bl, bu, info
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, tau, df, b, bl, bu
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        info
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g02qfc', x, 'x')
    _y = nag_double_type_check_and_cast('g02qfc', y, 'y')
    _tau = nag_double_type_check_and_cast('g02qfc', tau, 'tau')
    _df = nag_double_type_check_and_cast('g02qfc', df, 'df')
    _b = nag_double_type_check_and_cast('g02qfc', b, 'b')
    _bl = nag_double_type_check_and_cast('g02qfc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('g02qfc', bu, 'bu')
    _info = nag_integer_type_check_and_cast('g02qfc', info, 'info')
    g02qfc_ctypes(n, m, _x, _y, ntau, _tau, _df, _b, _bl, _bu, _info, fail)

g02qgc_ctypes = _library_cl.g02qgc
g02qgc_ctypes.restype = None
g02qgc_ctypes.argtypes = [Nag_OrderType, Nag_IncludeIntercept, Integer, Integer, POINTER(c_double), Integer, POINTER(Integer), Integer, POINTER(c_double), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(NagError)]


def g02qgc(order, intcpt, n, m, dat, pddat, isx, ip, y, wt, ntau, tau, df, b, bl, bu, ch, res, iopts, opts, state, info, fail):
    """
    Linear quantile regression, comprehensive interface.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02qgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        dat, isx, y, wt, tau, df, b, bl, bu, ch, res, iopts, opts, state, info
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        dat, y, wt, tau, df, b, bl, bu, ch, res, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        isx, iopts, state, info
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _dat = nag_double_type_check_and_cast('g02qgc', dat, 'dat')
    _isx = nag_integer_type_check_and_cast('g02qgc', isx, 'isx')
    _y = nag_double_type_check_and_cast('g02qgc', y, 'y')
    _wt = nag_double_type_check_and_cast('g02qgc', wt, 'wt')
    _tau = nag_double_type_check_and_cast('g02qgc', tau, 'tau')
    _df = nag_double_type_check_and_cast('g02qgc', df, 'df')
    _b = nag_double_type_check_and_cast('g02qgc', b, 'b')
    _bl = nag_double_type_check_and_cast('g02qgc', bl, 'bl')
    _bu = nag_double_type_check_and_cast('g02qgc', bu, 'bu')
    _ch = nag_double_type_check_and_cast('g02qgc', ch, 'ch')
    _res = nag_double_type_check_and_cast('g02qgc', res, 'res')
    _iopts = nag_integer_type_check_and_cast('g02qgc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('g02qgc', opts, 'opts')
    _state = nag_integer_type_check_and_cast('g02qgc', state, 'state')
    _info = nag_integer_type_check_and_cast('g02qgc', info, 'info')
    g02qgc_ctypes(order, intcpt, n, m, _dat, pddat, _isx, ip, _y, _wt, ntau, _tau, _df, _b, _bl, _bu, _ch, _res, _iopts, _opts, _state, _info, fail)

g02zkc_ctypes = _library_cl.g02zkc
g02zkc_ctypes.restype = None
g02zkc_ctypes.argtypes = [STRING, POINTER(Integer), Integer, POINTER(c_double), Integer, POINTER(NagError)]


def g02zkc(optstr, iopts, liopts, opts, lopts, fail):
    """
    Option setting function for g02qgc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02zkc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _iopts = nag_integer_type_check_and_cast('g02zkc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('g02zkc', opts, 'opts')
    g02zkc_ctypes(optstr, _iopts, liopts, _opts, lopts, fail)

g02zlc_ctypes = _library_cl.g02zlc
g02zlc_ctypes.restype = None
g02zlc_ctypes.argtypes = [STRING, POINTER(Integer), POINTER(c_double), STRING, Integer, POINTER(Nag_VariableType), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g02zlc(optstr, ivalue, rvalue, cvalue, lcvalue, optype, iopts, opts, fail):
    """
    Option getting function for g02qgc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g02/g02zlc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ivalue, rvalue, iopts, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rvalue, opts
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ivalue, iopts
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ivalue = nag_integer_type_check_and_cast('g02zlc', ivalue, 'ivalue')
    _rvalue = nag_double_type_check_and_cast('g02zlc', rvalue, 'rvalue')
    _iopts = nag_integer_type_check_and_cast('g02zlc', iopts, 'iopts')
    _opts = nag_double_type_check_and_cast('g02zlc', opts, 'opts')
    g02zlc_ctypes(optstr, _ivalue, _rvalue, cvalue, lcvalue, optype, _iopts, _opts, fail)

nag_nearest_correlation = g02aac
nag_nearest_correlation_bounded = g02abc
nag_nearest_correlation_k_factor = g02aec
nag_nearest_correlation_h_weight = g02ajc
nag_nearest_correlation_shrinking = g02anc
nag_nearest_correlation_target = g02apc
nag_ken_spe_corr_coeff = g02brc
nag_sum_sqs_update = g02btc
nag_sum_sqs = g02buc
nag_cov_to_corr = g02bwc
nag_corr_cov = g02bxc
nag_partial_corr = g02byc
nag_sum_sqs_combine = g02bzc
nag_simple_linear_regression = g02cac
nag_regress_confid_interval = g02cbc
nag_regsn_mult_linear = g02dac
nag_regsn_mult_linear_addrem_obs = g02dcc
nag_regsn_mult_linear_upd_model = g02ddc
nag_regsn_mult_linear_add_var = g02dec
nag_regsn_mult_linear_delete_var = g02dfc
nag_regsn_mult_linear_newyvar = g02dgc
nag_regsn_mult_linear_tran_model = g02dkc
nag_regsn_mult_linear_est_func = g02dnc
nag_all_regsn = g02eac
nag_cp_stat = g02ecc
nag_step_regsn = g02eec
nag_full_step_regsn = g02efc
nag_regsn_std_resid_influence = g02fac
nag_durbin_watson_stat = g02fcc
nag_glm_normal = g02gac
nag_glm_binomial = g02gbc
nag_glm_poisson = g02gcc
nag_glm_gamma = g02gdc
nag_glm_tran_model = g02gkc
nag_glm_est_func = g02gnc
nag_glm_predict = g02gpc
nag_robust_m_regsn_estim = g02hac
nag_robust_m_regsn_wts = g02hbc
nag_robust_m_regsn_user_fn = g02hdc
nag_robust_m_regsn_param_var = g02hfc
nag_robust_corr_estim = g02hkc
nag_robust_m_corr_user_fn = g02hlc
nag_robust_m_corr_user_fn_no_derr = g02hmc
nag_reml_mixed_regsn = g02jac
nag_ml_mixed_regsn = g02jbc
nag_hier_mixed_init = g02jcc
nag_reml_hier_mixed_regsn = g02jdc
nag_ml_hier_mixed_regsn = g02jec
nag_regsn_ridge_opt = g02kac
nag_regsn_ridge = g02kbc
nag_pls_orth_scores_svd = g02lac
nag_pls_orth_scores_wold = g02lbc
nag_pls_orth_scores_fit = g02lcc
nag_pls_orth_scores_pred = g02ldc
nag_regsn_quant_linear_iid = g02qfc
nag_regsn_quant_linear = g02qgc
nag_g02_opt_set = g02zkc
nag_g02_opt_get = g02zlc
__all__ = ['g02aac', 'g02abc', 'g02aec', 'g02ajc', 'g02anc', 'g02apc',
           'g02brc', 'g02btc', 'g02buc', 'g02bwc',
           'g02bxc', 'g02byc', 'g02bzc', 'g02cac', 'g02cbc',
           'g02dac',
           'g02dcc', 'g02ddc', 'g02dec', 'g02dfc', 'g02dgc', 'g02dkc',
           'g02dnc', 'g02eac', 'g02ecc', 'g02eec', 'g02efc', 'g02efg',
           'g02fac', 'g02fcc', 'g02gac', 'g02gbc', 'g02gcc',
           'g02gdc', 'g02gkc', 'g02gnc', 'g02gpc', 'g02hac', 'g02hbc',
           'g02hdc', 'g02hfc', 'g02hkc', 'g02hlc', 'g02hmc', 'g02jac',
           'g02jbc', 'g02jcc', 'g02jdc', 'g02jec', 'g02kac', 'g02kbc',
           'g02lac', 'g02lbc', 'g02lcc', 'g02ldc', 'g02qfc', 'g02qgc',
           'g02zkc', 'g02zlc', 'nag_all_regsn', 'nag_corr_cov',
           'nag_cov_to_corr',
           'nag_cp_stat', 'nag_durbin_watson_stat',
           'nag_full_step_regsn',
           'nag_g02_opt_get', 'nag_g02_opt_set', 'nag_glm_binomial',
           'nag_glm_est_func', 'nag_glm_gamma', 'nag_glm_normal',
           'nag_glm_poisson', 'nag_glm_predict', 'nag_glm_tran_model',
           'nag_hier_mixed_init', 'nag_ken_spe_corr_coeff',
           'nag_ml_hier_mixed_regsn', 'nag_ml_mixed_regsn',
           'nag_nearest_correlation',
           'nag_nearest_correlation_bounded',
           'nag_nearest_correlation_h_weight',
           'nag_nearest_correlation_k_factor', 'nag_partial_corr',
           'nag_pls_orth_scores_fit', 'nag_pls_orth_scores_pred',
           'nag_pls_orth_scores_svd', 'nag_pls_orth_scores_wold',
           'nag_regress_confid_interval', 'nag_regsn_mult_linear',
           'nag_regsn_mult_linear_add_var',
           'nag_regsn_mult_linear_addrem_obs',
           'nag_regsn_mult_linear_delete_var',
           'nag_regsn_mult_linear_est_func',
           'nag_regsn_mult_linear_newyvar',
           'nag_regsn_mult_linear_tran_model',
           'nag_regsn_mult_linear_upd_model',
           'nag_regsn_quant_linear', 'nag_regsn_quant_linear_iid',
           'nag_regsn_ridge', 'nag_regsn_ridge_opt',
           'nag_regsn_std_resid_influence',
           'nag_reml_hier_mixed_regsn', 'nag_reml_mixed_regsn',
           'nag_robust_corr_estim', 'nag_robust_m_corr_user_fn',
           'nag_robust_m_corr_user_fn_no_derr',
           'nag_robust_m_regsn_estim', 'nag_robust_m_regsn_param_var',
           'nag_robust_m_regsn_user_fn', 'nag_robust_m_regsn_wts',
           'nag_simple_linear_regression', 'nag_step_regsn',
           'nag_sum_sqs', 'nag_sum_sqs_combine', 'nag_sum_sqs_update',
           'nag_nearest_correlation_shrinking',
           'nag_nearest_correlation_target']
