"""
Deprecated wrappers for the NAG C Library (Mark 26.0) g08 Chapter.

g08 - Nonparametric Statistics.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/G08/g08conts.html

NAG Copyright 2017.
"""
from ctypes import c_double
from ctypes import POINTER

from .util import get_library

from .util import Integer
from .util import NagError
from .util import Nag_TailProbability
from .util import Nag_IncSignZeros
from .util import Nag_Boolean
from .util import Nag_CompProb
from .util import Nag_TestStatistics
from .util import Nag_Distributions
from .util import Nag_ParaEstimates
from .util import Nag_OrderType

_library_cl = get_library()

g08aac_ctypes = _library_cl.g08aac
g08aac_ctypes.restype = None
g08aac_ctypes.argtypes = [Integer, POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g08aac(n, x, y, s, p, nontied, fail):
    """
    Sign test on two paired samples.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08aac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, s, p, nontied
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        s, nontied
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08aac', x, 'x')
    _y = nag_double_type_check_and_cast('g08aac', y, 'y')
    _s = nag_integer_type_check_and_cast('g08aac', s, 's')
    _p = nag_double_type_check_and_cast('g08aac', p, 'p')
    _nontied = nag_integer_type_check_and_cast('g08aac', nontied, 'nontied')
    g08aac_ctypes(n, _x, _y, _s, _p, _nontied, fail)

g08acc_ctypes = _library_cl.g08acc
g08acc_ctypes.restype = None
g08acc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(Integer), POINTER(Integer), POINTER(c_double), POINTER(NagError)]


def g08acc(n1, x, n2, y, below, above, p, fail):
    """
    Median test on two samples of unequal size.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08acc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, below, above, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        below, above
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08acc', x, 'x')
    _y = nag_double_type_check_and_cast('g08acc', y, 'y')
    _below = nag_integer_type_check_and_cast('g08acc', below, 'below')
    _above = nag_integer_type_check_and_cast('g08acc', above, 'above')
    _p = nag_double_type_check_and_cast('g08acc', p, 'p')
    g08acc_ctypes(n1, _x, n2, _y, _below, _above, _p, fail)

g08aec_ctypes = _library_cl.g08aec
g08aec_ctypes.restype = None
g08aec_ctypes.argtypes = [Integer, Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08aec(k, n, x, tdx, fr, p, fail):
    """
    Friedman two-way analysis of variance on k matched samples.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08aec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, fr, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, fr, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08aec', x, 'x')
    _fr = nag_double_type_check_and_cast('g08aec', fr, 'fr')
    _p = nag_double_type_check_and_cast('g08aec', p, 'p')
    g08aec_ctypes(k, n, _x, tdx, _fr, _p, fail)

g08afc_ctypes = _library_cl.g08afc
g08afc_ctypes.restype = None
g08afc_ctypes.argtypes = [Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08afc(k, l, x, lx, h, p, fail):
    """
    Kruskal--Wallis one-way analysis of variance on k samples of unequal
    size.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08afc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        l, x, h, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, h, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        l
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _l = nag_integer_type_check_and_cast('g08afc', l, 'l')
    _x = nag_double_type_check_and_cast('g08afc', x, 'x')
    _h = nag_double_type_check_and_cast('g08afc', h, 'h')
    _p = nag_double_type_check_and_cast('g08afc', p, 'p')
    g08afc_ctypes(k, _l, _x, lx, _h, _p, fail)

g08agc_ctypes = _library_cl.g08agc
g08agc_ctypes.restype = None
g08agc_ctypes.argtypes = [Integer, POINTER(c_double), c_double, Nag_TailProbability, Nag_IncSignZeros, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(NagError)]


def g08agc(n, x, median, tail, zeros, w, z, p, nonzero, fail):
    """
    Performs the Wilcoxon one-sample (matched pairs) signed rank test.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08agc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, w, z, p, nonzero
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, w, z, p
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nonzero
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08agc', x, 'x')
    _w = nag_double_type_check_and_cast('g08agc', w, 'w')
    _z = nag_double_type_check_and_cast('g08agc', z, 'z')
    _p = nag_double_type_check_and_cast('g08agc', p, 'p')
    _nonzero = nag_integer_type_check_and_cast('g08agc', nonzero, 'nonzero')
    g08agc_ctypes(n, _x, median, tail, zeros, _w, _z, _p, _nonzero, fail)

g08amc_ctypes = _library_cl.g08amc
g08amc_ctypes.restype = None
g08amc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), Nag_TailProbability, Nag_CompProb, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08amc(n1, x, n2, y, tail, exact, u, z, p, fail):
    """
    Performs the Mann--Whitney U test on two independent samples.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08amc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, u, z, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, u, z, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08amc', x, 'x')
    _y = nag_double_type_check_and_cast('g08amc', y, 'y')
    _u = nag_double_type_check_and_cast('g08amc', u, 'u')
    _z = nag_double_type_check_and_cast('g08amc', z, 'z')
    _p = nag_double_type_check_and_cast('g08amc', p, 'p')
    g08amc_ctypes(n1, _x, n2, _y, tail, exact, _u, _z, _p, fail)

g08cbc_ctypes = _library_cl.g08cbc
g08cbc_ctypes.restype = None
g08cbc_ctypes.argtypes = [Integer, POINTER(c_double), Nag_Distributions, POINTER(c_double), Nag_ParaEstimates, Nag_TestStatistics, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08cbc(n, x, dist, par, estima, dtype, d, z, p, fail):
    """
    Performs the one-sample Kolmogorov--Smirnov test for standard
    distributions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08cbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, par, d, z, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, par, d, z, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08cbc', x, 'x')
    _par = nag_double_type_check_and_cast('g08cbc', par, 'par')
    _d = nag_double_type_check_and_cast('g08cbc', d, 'd')
    _z = nag_double_type_check_and_cast('g08cbc', z, 'z')
    _p = nag_double_type_check_and_cast('g08cbc', p, 'p')
    g08cbc_ctypes(n, _x, dist, _par, estima, dtype, _d, _z, _p, fail)

g08cdc_ctypes = _library_cl.g08cdc
g08cdc_ctypes.restype = None
g08cdc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), Nag_TestStatistics, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08cdc(n1, x, n2, y, dtype, d, z, p, fail):
    """
    Performs the two-sample Kolmogorov--Smirnov test.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08cdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, y, d, z, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, y, d, z, p
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08cdc', x, 'x')
    _y = nag_double_type_check_and_cast('g08cdc', y, 'y')
    _d = nag_double_type_check_and_cast('g08cdc', d, 'd')
    _z = nag_double_type_check_and_cast('g08cdc', z, 'z')
    _p = nag_double_type_check_and_cast('g08cdc', p, 'p')
    g08cdc_ctypes(n1, _x, n2, _y, dtype, _d, _z, _p, fail)

g08cgc_ctypes = _library_cl.g08cgc
g08cgc_ctypes.restype = None
g08cgc_ctypes.argtypes = [Integer, POINTER(Integer), POINTER(c_double), Nag_Distributions, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08cgc(nclass, ifreq, cint, dist, par, npest, prob, chisq, p, ndf, _eval, chisqi, fail):
    """
    Performs the chi**2 goodness-of-fit test, for standard continuous
    distributions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08cgc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        ifreq, cint, par, prob, chisq, p, ndf, _eval, chisqi
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        cint, par, prob, chisq, p, _eval, chisqi
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        ifreq, ndf
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _ifreq = nag_integer_type_check_and_cast('g08cgc', ifreq, 'ifreq')
    _cint = nag_double_type_check_and_cast('g08cgc', cint, 'cint')
    _par = nag_double_type_check_and_cast('g08cgc', par, 'par')
    _prob = nag_double_type_check_and_cast('g08cgc', prob, 'prob')
    _chisq = nag_double_type_check_and_cast('g08cgc', chisq, 'chisq')
    _p = nag_double_type_check_and_cast('g08cgc', p, 'p')
    _ndf = nag_integer_type_check_and_cast('g08cgc', ndf, 'ndf')
    __eval = nag_double_type_check_and_cast('g08cgc', _eval, '_eval')
    _chisqi = nag_double_type_check_and_cast('g08cgc', chisqi, 'chisqi')
    g08cgc_ctypes(nclass, _ifreq, _cint, dist, _par, npest, _prob, _chisq, _p, _ndf, __eval, _chisqi, fail)

g08chc_ctypes = _library_cl.g08chc
g08chc_ctypes.restype = c_double
g08chc_ctypes.argtypes = [Integer, Nag_Boolean, POINTER(c_double), POINTER(NagError)]


def g08chc(n, issort, y, fail):
    """
    Calculates the Anderson--Darling goodness-of-fit test statistic.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08chc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y
    """
    from .util import nag_double_type_check_and_cast
    _y = nag_double_type_check_and_cast('g08chc', y, 'y')
    return_value = g08chc_ctypes(n, issort, _y, fail)
    return return_value

g08cjc_ctypes = _library_cl.g08cjc
g08cjc_ctypes.restype = None
g08cjc_ctypes.argtypes = [Integer, Nag_Boolean, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08cjc(n, issort, y, a2, p, fail):
    """
    Calculates the Anderson--Darling goodness-of-fit test statistic and its
    probability for the case of uniformly distributed data.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08cjc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, a2, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, a2, p
    """
    from .util import nag_double_type_check_and_cast
    _y = nag_double_type_check_and_cast('g08cjc', y, 'y')
    _a2 = nag_double_type_check_and_cast('g08cjc', a2, 'a2')
    _p = nag_double_type_check_and_cast('g08cjc', p, 'p')
    g08cjc_ctypes(n, issort, _y, _a2, _p, fail)

g08ckc_ctypes = _library_cl.g08ckc
g08ckc_ctypes.restype = None
g08ckc_ctypes.argtypes = [Integer, Nag_Boolean, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08ckc(n, issort, y, ybar, yvar, a2, aa2, p, fail):
    """
    Calculates the Anderson--Darling goodness-of-fit test statistic and its
    probability for the case of a fully-unspecified Normal distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08ckc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, ybar, yvar, a2, aa2, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, ybar, yvar, a2, aa2, p
    """
    from .util import nag_double_type_check_and_cast
    _y = nag_double_type_check_and_cast('g08ckc', y, 'y')
    _ybar = nag_double_type_check_and_cast('g08ckc', ybar, 'ybar')
    _yvar = nag_double_type_check_and_cast('g08ckc', yvar, 'yvar')
    _a2 = nag_double_type_check_and_cast('g08ckc', a2, 'a2')
    _aa2 = nag_double_type_check_and_cast('g08ckc', aa2, 'aa2')
    _p = nag_double_type_check_and_cast('g08ckc', p, 'p')
    g08ckc_ctypes(n, issort, _y, _ybar, _yvar, _a2, _aa2, _p, fail)

g08clc_ctypes = _library_cl.g08clc
g08clc_ctypes.restype = None
g08clc_ctypes.argtypes = [Integer, Nag_Boolean, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08clc(n, issort, y, ybar, a2, aa2, p, fail):
    """
    Calculates the Anderson--Darling goodness-of-fit test statistic and its
    probability for the case of an unspecified exponential distribution.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08clc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        y, ybar, a2, aa2, p
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, ybar, a2, aa2, p
    """
    from .util import nag_double_type_check_and_cast
    _y = nag_double_type_check_and_cast('g08clc', y, 'y')
    _ybar = nag_double_type_check_and_cast('g08clc', ybar, 'ybar')
    _a2 = nag_double_type_check_and_cast('g08clc', a2, 'a2')
    _aa2 = nag_double_type_check_and_cast('g08clc', aa2, 'aa2')
    _p = nag_double_type_check_and_cast('g08clc', p, 'p')
    g08clc_ctypes(n, issort, _y, _ybar, _a2, _aa2, _p, fail)

g08eac_ctypes = _library_cl.g08eac
g08eac_ctypes.restype = None
g08eac_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08eac(n, x, maxrun, nruns, chi, df, prob, fail):
    """
    Performs the runs up or runs down test for randomness.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08eac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, nruns, chi, df, prob
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, chi, df, prob
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nruns
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08eac', x, 'x')
    _nruns = nag_integer_type_check_and_cast('g08eac', nruns, 'nruns')
    _chi = nag_double_type_check_and_cast('g08eac', chi, 'chi')
    _df = nag_double_type_check_and_cast('g08eac', df, 'df')
    _prob = nag_double_type_check_and_cast('g08eac', prob, 'prob')
    g08eac_ctypes(n, _x, maxrun, _nruns, _chi, _df, _prob, fail)

g08ebc_ctypes = _library_cl.g08ebc
g08ebc_ctypes.restype = None
g08ebc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08ebc(n, x, maxcount, lag, chi, df, prob, fail):
    """
    Performs the pairs (serial) test for randomness.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08ebc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, chi, df, prob
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, chi, df, prob
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08ebc', x, 'x')
    _chi = nag_double_type_check_and_cast('g08ebc', chi, 'chi')
    _df = nag_double_type_check_and_cast('g08ebc', df, 'df')
    _prob = nag_double_type_check_and_cast('g08ebc', prob, 'prob')
    g08ebc_ctypes(n, _x, maxcount, lag, _chi, _df, _prob, fail)

g08ecc_ctypes = _library_cl.g08ecc
g08ecc_ctypes.restype = None
g08ecc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08ecc(n, x, maxcount, chi, df, prob, fail):
    """
    Performs the triplets test for randomness.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08ecc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, chi, df, prob
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, chi, df, prob
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08ecc', x, 'x')
    _chi = nag_double_type_check_and_cast('g08ecc', chi, 'chi')
    _df = nag_double_type_check_and_cast('g08ecc', df, 'df')
    _prob = nag_double_type_check_and_cast('g08ecc', prob, 'prob')
    g08ecc_ctypes(n, _x, maxcount, _chi, _df, _prob, fail)

g08edc_ctypes = _library_cl.g08edc
g08edc_ctypes.restype = None
g08edc_ctypes.argtypes = [Integer, POINTER(c_double), Integer, Integer, c_double, c_double, c_double, POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08edc(n, x, numgaps, maxgap, lower, upper, length, chi, df, prob, fail):
    """
    Performs the gaps test for randomness.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08edc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x, chi, df, prob
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x, chi, df, prob
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('g08edc', x, 'x')
    _chi = nag_double_type_check_and_cast('g08edc', chi, 'chi')
    _df = nag_double_type_check_and_cast('g08edc', df, 'df')
    _prob = nag_double_type_check_and_cast('g08edc', prob, 'prob')
    g08edc_ctypes(n, _x, numgaps, maxgap, lower, upper, length, _chi, _df, _prob, fail)

g08rac_ctypes = _library_cl.g08rac
g08rac_ctypes.restype = None
g08rac_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, Integer, Integer, c_double, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08rac(order, ns, nv, y, p, x, pdx, idist, nmax, tol, prvr, pdparvar, irank, zin, eta, vapvec, parest, fail):
    """
    Regression using ranks, uncensored data.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08rac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nv, y, x, prvr, irank, zin, eta, vapvec, parest
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, x, prvr, zin, eta, vapvec, parest
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nv, irank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nv = nag_integer_type_check_and_cast('g08rac', nv, 'nv')
    _y = nag_double_type_check_and_cast('g08rac', y, 'y')
    _x = nag_double_type_check_and_cast('g08rac', x, 'x')
    _prvr = nag_double_type_check_and_cast('g08rac', prvr, 'prvr')
    _irank = nag_integer_type_check_and_cast('g08rac', irank, 'irank')
    _zin = nag_double_type_check_and_cast('g08rac', zin, 'zin')
    _eta = nag_double_type_check_and_cast('g08rac', eta, 'eta')
    _vapvec = nag_double_type_check_and_cast('g08rac', vapvec, 'vapvec')
    _parest = nag_double_type_check_and_cast('g08rac', parest, 'parest')
    g08rac_ctypes(order, ns, _nv, _y, p, _x, pdx, idist, nmax, tol, _prvr, pdparvar, _irank, _zin, _eta, _vapvec, _parest, fail)

g08rbc_ctypes = _library_cl.g08rbc
g08rbc_ctypes.restype = None
g08rbc_ctypes.argtypes = [Nag_OrderType, Integer, POINTER(Integer), POINTER(c_double), Integer, POINTER(c_double), Integer, POINTER(Integer), c_double, Integer, c_double, POINTER(c_double), Integer, POINTER(Integer), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(c_double), POINTER(NagError)]


def g08rbc(order, ns, nv, y, p, x, pdx, icen, gamma, nmax, tol, prvr, pdprvr, irank, zin, eta, vapvec, parest, fail):
    """
    Regression using ranks, right-censored data.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/g08/g08rbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        nv, y, x, icen, prvr, irank, zin, eta, vapvec, parest
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        y, x, prvr, zin, eta, vapvec, parest
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        nv, icen, irank
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_integer_type_check_and_cast
    _nv = nag_integer_type_check_and_cast('g08rbc', nv, 'nv')
    _y = nag_double_type_check_and_cast('g08rbc', y, 'y')
    _x = nag_double_type_check_and_cast('g08rbc', x, 'x')
    _icen = nag_integer_type_check_and_cast('g08rbc', icen, 'icen')
    _prvr = nag_double_type_check_and_cast('g08rbc', prvr, 'prvr')
    _irank = nag_integer_type_check_and_cast('g08rbc', irank, 'irank')
    _zin = nag_double_type_check_and_cast('g08rbc', zin, 'zin')
    _eta = nag_double_type_check_and_cast('g08rbc', eta, 'eta')
    _vapvec = nag_double_type_check_and_cast('g08rbc', vapvec, 'vapvec')
    _parest = nag_double_type_check_and_cast('g08rbc', parest, 'parest')
    g08rbc_ctypes(order, ns, _nv, _y, p, _x, pdx, _icen, gamma, nmax, tol, _prvr, pdprvr, _irank, _zin, _eta, _vapvec, _parest, fail)

nag_sign_test = g08aac
nag_median_test = g08acc
nag_friedman_test = g08aec
nag_kruskal_wallis_test = g08afc
nag_wilcoxon_test = g08agc
nag_mann_whitney = g08amc
nag_1_sample_ks_test = g08cbc
nag_2_sample_ks_test = g08cdc
nag_chi_sq_goodness_of_fit_test = g08cgc
nag_anderson_darling_stat = g08chc
nag_anderson_darling_uniform_prob = g08cjc
nag_anderson_darling_normal_prob = g08ckc
nag_anderson_darling_exp_prob = g08clc
nag_runs_test = g08eac
nag_pairs_test = g08ebc
nag_triplets_test = g08ecc
nag_gaps_test = g08edc
nag_rank_regsn = g08rac
nag_rank_regsn_censored = g08rbc
__all__ = ['g08aac', 'g08acc', 'g08aec', 'g08afc', 'g08agc',
           'g08amc', 'g08cbc',
           'g08cdc', 'g08cgc', 'g08chc', 'g08cjc', 'g08ckc',
           'g08clc', 'g08eac', 'g08ebc', 'g08ecc', 'g08edc',
           'g08rac', 'g08rbc', 'nag_1_sample_ks_test',
           'nag_2_sample_ks_test', 'nag_anderson_darling_exp_prob',
           'nag_anderson_darling_normal_prob',
           'nag_anderson_darling_stat',
           'nag_anderson_darling_uniform_prob',
           'nag_chi_sq_goodness_of_fit_test', 'nag_friedman_test',
           'nag_gaps_test', 'nag_kruskal_wallis_test',
           'nag_mann_whitney', 'nag_median_test',
           'nag_pairs_test', 'nag_rank_regsn',
           'nag_rank_regsn_censored', 'nag_runs_test',
           'nag_sign_test', 'nag_triplets_test', 'nag_wilcoxon_test']
