"""
Deprecated wrappers for the NAG C Library (Mark 26.0) m01 Chapter.

m01 - Sorting and Searching.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/M01/m01conts.html

NAG Copyright 2017.
"""
from ctypes import c_char_p
from ctypes import c_ulong
from ctypes import c_double
from ctypes import POINTER
from ctypes import c_long
from .util import get_library

from .util import Nag_SortOrder
from .util import NagError
from .util import Pointer
from .util import NAG_M01CSC_COMPARE
from .util import NAG_M01CTC_COMPARE
from .util import NAG_M01CUC_COMPARE
from .util import NAG_M01DSC_COMPARE
from .util import Nag_Boolean
from .util import NAG_M01FSC_COMPARE
from .util import Nag_SearchMatch
from .util import Integer

_library_cl = get_library()

STRING = c_char_p


size_t = c_ulong
m01cac_ctypes = _library_cl.m01cac
m01cac_ctypes.restype = None
m01cac_ctypes.argtypes = [POINTER(c_double), size_t, Nag_SortOrder, POINTER(NagError)]


def m01cac(vec, n, order, fail):
    """
    Quicksort of set of values of data type double.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        vec
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        vec
    """
    from .util import nag_double_type_check_and_cast
    _vec = nag_double_type_check_and_cast('m01cac', vec, 'vec')
    m01cac_ctypes(_vec, n, order, fail)

ptrdiff_t = c_long
m01csc_ctypes = _library_cl.m01csc
m01csc_ctypes.restype = None
m01csc_ctypes.argtypes = [Pointer, size_t, size_t, ptrdiff_t, NAG_M01CSC_COMPARE, Nag_SortOrder, POINTER(NagError)]


def m01csc(vec, n, size, stride, compare, order, fail):
    """
    Quicksort of set of values of arbitrary data type.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01csc.html
    """
    m01csc_ctypes(vec, n, size, stride, compare, order, fail)

m01ctc_ctypes = _library_cl.m01ctc
m01ctc_ctypes.restype = None
m01ctc_ctypes.argtypes = [Pointer, size_t, size_t, ptrdiff_t, NAG_M01CTC_COMPARE, Nag_SortOrder, POINTER(NagError)]


def m01ctc(vec, n, size, stride, compare, order, fail):
    """
    Stable sort of set of values of arbitrary data type.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01ctc.html
    """
    m01ctc_ctypes(vec, n, size, stride, compare, order, fail)

m01cuc_ctypes = _library_cl.m01cuc
m01cuc_ctypes.restype = None
m01cuc_ctypes.argtypes = [POINTER(Pointer), ptrdiff_t, NAG_M01CUC_COMPARE, Nag_SortOrder, POINTER(NagError)]


def m01cuc(base, offset, compare, order, fail):
    """
    Chain sort of linked list.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01cuc.html
    """
    m01cuc_ctypes(base, offset, compare, order, fail)

m01dsc_ctypes = _library_cl.m01dsc
m01dsc_ctypes.restype = None
m01dsc_ctypes.argtypes = [Pointer, size_t, ptrdiff_t, NAG_M01DSC_COMPARE, Nag_SortOrder, POINTER(size_t), POINTER(NagError)]


def m01dsc(vec, n, stride, compare, order, ranks, fail):
    """
    Rank sort of set of values of arbitrary data type.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01dsc.html
    """
    m01dsc_ctypes(vec, n, stride, compare, order, ranks, fail)

m01esc_ctypes = _library_cl.m01esc
m01esc_ctypes.restype = None
m01esc_ctypes.argtypes = [Pointer, size_t, size_t, ptrdiff_t, POINTER(size_t), POINTER(NagError)]


def m01esc(vec, n, size, stride, indices, fail):
    """
    Reorders set of values of arbitrary data type into the order specified
    by a set of indices.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01esc.html
    """
    m01esc_ctypes(vec, n, size, stride, indices, fail)

m01fsc_ctypes = _library_cl.m01fsc
m01fsc_ctypes.restype = Nag_Boolean
m01fsc_ctypes.argtypes = [Pointer, Pointer, size_t, ptrdiff_t, NAG_M01FSC_COMPARE, Nag_SortOrder, Nag_SearchMatch, POINTER(Pointer), POINTER(NagError)]


def m01fsc(key, vec, n, stride, compare, order, final, match, fail):
    """
    Searches a vector for either the first or last match to a given value.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01fsc.html
    """
    return_value = m01fsc_ctypes(key, vec, n, stride, compare, order, final, match, fail)
    return return_value

m01nac_ctypes = _library_cl.m01nac
m01nac_ctypes.restype = Integer
m01nac_ctypes.argtypes = [Nag_Boolean, POINTER(c_double), Integer, Integer, c_double, POINTER(NagError)]


def m01nac(validate, rv, m1, m2, item, fail):
    """
    Binary search in set of real numbers.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01nac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        rv
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        rv
    """
    from .util import nag_double_type_check_and_cast
    _rv = nag_double_type_check_and_cast('m01nac', rv, 'rv')
    return_value = m01nac_ctypes(validate, _rv, m1, m2, item, fail)
    return return_value

m01nbc_ctypes = _library_cl.m01nbc
m01nbc_ctypes.restype = Integer
m01nbc_ctypes.argtypes = [Nag_Boolean, POINTER(Integer), Integer, Integer, Integer, POINTER(NagError)]


def m01nbc(validate, iv, m1, m2, item, fail):
    """
    Binary search in set of integer numbers.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01nbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        iv
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        iv
    """
    from .util import nag_integer_type_check_and_cast
    _iv = nag_integer_type_check_and_cast('m01nbc', iv, 'iv')
    return_value = m01nbc_ctypes(validate, _iv, m1, m2, item, fail)
    return return_value

m01ncc_ctypes = _library_cl.m01ncc
m01ncc_ctypes.restype = Integer
m01ncc_ctypes.argtypes = [Nag_Boolean, POINTER(STRING), Integer, Integer, STRING, POINTER(NagError)]


def m01ncc(validate, ch, m1, m2, item, fail):
    """
    Binary search in set of character data.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01ncc.html

    TypeError will be raised when the following arguments are not arrays of
    strings:
        ch
    """
    from .util import nag_char_type_check_and_cast
    _ch = nag_char_type_check_and_cast('m01ncc', ch, 'ch')
    return_value = m01ncc_ctypes(validate, _ch, m1, m2, item, fail)
    return return_value

m01zac_ctypes = _library_cl.m01zac
m01zac_ctypes.restype = None
m01zac_ctypes.argtypes = [POINTER(size_t), size_t, POINTER(NagError)]


def m01zac(ranks, n, fail):
    """
    Inverts a permutation converting a rank vector to an index vector or
    vice versa.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/m01/m01zac.html
    """
    m01zac_ctypes(ranks, n, fail)

nag_double_sort = m01cac
nag_quicksort = m01csc
nag_stable_sort = m01ctc
nag_chain_sort = m01cuc
nag_rank_sort = m01dsc
nag_reorder_vector = m01esc
nag_search_vector = m01fsc
nag_search_double = m01nac
nag_search_int = m01nbc
nag_search_char = m01ncc
nag_make_indices = m01zac
__all__ = ['m01cac', 'm01csc', 'm01ctc', 'm01cuc', 'm01dsc', 'm01esc',
           'm01fsc', 'm01nac', 'm01nbc', 'm01ncc', 'm01zac',
           'nag_chain_sort', 'nag_double_sort', 'nag_make_indices',
           'nag_quicksort', 'nag_rank_sort', 'nag_reorder_vector',
           'nag_search_char', 'nag_search_double', 'nag_search_int',
           'nag_search_vector', 'nag_stable_sort']
