"""
Deprecated wrappers for the NAG C Library (Mark 26.0) x04 Chapter.

x04 - Input/Output Utilities.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/X04/x04conts.html

NAG Copyright 2017.
"""
from ctypes import c_char_p
from ctypes import c_ulong
from ctypes import POINTER
from ctypes import c_double
from ctypes import c_int
from .util import get_library
from .util import Pointer
from .util import Integer
from .util import Nag_FileID
from .util import NagError

from .util import Nag_OrderType
from .util import Nag_MatrixType
from .util import Nag_DiagType
from .util import Nag_LabelType
from .util import Nag_UploType
from .util import Complex
from .util import Nag_ComplexFormType

_library_cl = get_library()

STRING = c_char_p

size_t = c_ulong
x04bcc_ctypes = _library_cl.x04bcc
x04bcc_ctypes.restype = Pointer
x04bcc_ctypes.argtypes = [Pointer, size_t]

def x04bcc(ptr, newsize):
    """
    Wrapper for realloc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/genint/essint.html#usingmemory
    """
    return_value = x04bcc_ctypes(ptr, newsize)
    return return_value

x04bdc_ctypes = _library_cl.x04bdc
x04bdc_ctypes.restype = None
x04bdc_ctypes.argtypes = [POINTER(Pointer)]

def x04bdc(ptr):
    """
    Wrapper for free.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/genint/essint.html#usingmemory
    """
    x04bdc_ctypes(ptr)

x04bjc_ctypes = _library_cl.x04bjc
x04bjc_ctypes.restype = Pointer
x04bjc_ctypes.argtypes = [size_t]

def x04bjc(size):
    """
    Wrapper for malloc.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/genint/essint.html#usingmemory
    """
    return_value = x04bjc_ctypes(size)
    return return_value

x04acc_ctypes = _library_cl.x04acc
x04acc_ctypes.restype = None
x04acc_ctypes.argtypes = [STRING, Integer, POINTER(Nag_FileID), POINTER(NagError)]

def x04acc(filename, mode, fileid, fail):
    """
    Open unit number for reading, writing or appending, and associate unit
    with named file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04acc.html
    """
    x04acc_ctypes(filename, mode, fileid, fail)

x04adc_ctypes = _library_cl.x04adc
x04adc_ctypes.restype = None
x04adc_ctypes.argtypes = [Nag_FileID, POINTER(NagError)]

def x04adc(fileid, fail):
    """
    Close file associated with given unit number.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04adc.html
    """
    x04adc_ctypes(fileid, fail)

x04bac_ctypes = _library_cl.x04bac
x04bac_ctypes.restype = None
x04bac_ctypes.argtypes = [Nag_FileID, STRING]

def x04bac(fileid, line):
    """
    Write formatted record to external file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04bac.html
    """
    x04bac_ctypes(fileid, line)

x04bbc_ctypes = _library_cl.x04bbc
x04bbc_ctypes.restype = None
x04bbc_ctypes.argtypes = [Nag_FileID, STRING, Integer, POINTER(NagError)]

def x04bbc(fileid, line, linelen, fail):
    """
    Read formatted record from external file.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04bbc.html
    """
    x04bbc_ctypes(fileid, line, linelen, fail)

x04cac_ctypes = _library_cl.x04cac
x04cac_ctypes.restype = None
x04cac_ctypes.argtypes = [Nag_OrderType, Nag_MatrixType, Nag_DiagType, Integer, Integer, POINTER(c_double), Integer, STRING, STRING, POINTER(NagError)]

def x04cac(order, matrix, diag, m, n, a, pda, title, outfile, fail):
    """
    Print real general matrix (easy-to-use).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('x04cac', a, 'a')
    x04cac_ctypes(order, matrix, diag, m, n, _a, pda, title, outfile, fail)

x04cbc_ctypes = _library_cl.x04cbc
x04cbc_ctypes.restype = None
x04cbc_ctypes.argtypes = [Nag_OrderType, Nag_MatrixType, Nag_DiagType, Integer, Integer, POINTER(c_double), Integer, STRING, STRING, Nag_LabelType, POINTER(STRING), Nag_LabelType, POINTER(STRING), Integer, Integer, STRING, POINTER(NagError)]

def x04cbc(order, matrix, diag, m, n, a, pda, form, title, labrow, rlabs, labcol, clabs, ncols, indent, outfile, fail):
    """
    Print real general matrix (comprehensive).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04cbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not arrays of
    strings:
        rlabs, clabs
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_char_type_check_and_cast
    _a = nag_double_type_check_and_cast('x04cbc', a, 'a')
    _rlabs = nag_char_type_check_and_cast('x04cbc', rlabs, 'rlabs')
    _clabs = nag_char_type_check_and_cast('x04cbc', clabs, 'clabs')
    x04cbc_ctypes(order, matrix, diag, m, n, _a, pda, form, title, labrow, _rlabs, labcol, _clabs, ncols, indent, outfile, fail)

x04ccc_ctypes = _library_cl.x04ccc
x04ccc_ctypes.restype = None
x04ccc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_DiagType, Integer, POINTER(c_double), STRING, STRING, POINTER(NagError)]

def x04ccc(order, uplo, diag, n, a, title, outfile, fail):
    """
    Print real packed triangular matrix (easy-to-use).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04ccc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('x04ccc', a, 'a')
    x04ccc_ctypes(order, uplo, diag, n, _a, title, outfile, fail)

x04cdc_ctypes = _library_cl.x04cdc
x04cdc_ctypes.restype = None
x04cdc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_DiagType, Integer, POINTER(c_double), STRING, STRING, Nag_LabelType, POINTER(STRING), Nag_LabelType, POINTER(STRING), Integer, Integer, STRING, POINTER(NagError)]

def x04cdc(order, uplo, diag, n, a, form, title, labrow, rlabs, labcol, clabs, ncols, indent, outfile, fail):
    """
    Print real packed triangular matrix (comprehensive).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04cdc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not arrays of
    strings:
        rlabs, clabs
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_char_type_check_and_cast
    _a = nag_double_type_check_and_cast('x04cdc', a, 'a')
    _rlabs = nag_char_type_check_and_cast('x04cdc', rlabs, 'rlabs')
    _clabs = nag_char_type_check_and_cast('x04cdc', clabs, 'clabs')
    x04cdc_ctypes(order, uplo, diag, n, _a, form, title, labrow, _rlabs, labcol, _clabs, ncols, indent, outfile, fail)

x04cec_ctypes = _library_cl.x04cec
x04cec_ctypes.restype = None
x04cec_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, STRING, STRING, POINTER(NagError)]

def x04cec(order, m, n, kl, ku, a, pda, title, outfile, fail):
    """
    Print real packed banded matrix (easy-to-use).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04cec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    """
    from .util import nag_double_type_check_and_cast
    _a = nag_double_type_check_and_cast('x04cec', a, 'a')
    x04cec_ctypes(order, m, n, kl, ku, _a, pda, title, outfile, fail)

x04cfc_ctypes = _library_cl.x04cfc
x04cfc_ctypes.restype = None
x04cfc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, Integer, POINTER(c_double), Integer, STRING, STRING, Nag_LabelType, POINTER(STRING), Nag_LabelType, POINTER(STRING), Integer, Integer, STRING, POINTER(NagError)]

def x04cfc(order, m, n, kl, ku, a, pda, form, title, labrow, rlabs, labcol, clabs, ncols, indent, outfile, fail):
    """
    Print real packed banded matrix (comprehensive).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04cfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        a
    TypeError will be raised when the following arguments are not arrays of
    strings:
        rlabs, clabs
    """
    from .util import nag_double_type_check_and_cast
    from .util import nag_char_type_check_and_cast
    _a = nag_double_type_check_and_cast('x04cfc', a, 'a')
    _rlabs = nag_char_type_check_and_cast('x04cfc', rlabs, 'rlabs')
    _clabs = nag_char_type_check_and_cast('x04cfc', clabs, 'clabs')
    x04cfc_ctypes(order, m, n, kl, ku, _a, pda, form, title, labrow, _rlabs, labcol, _clabs, ncols, indent, outfile, fail)

x04dac_ctypes = _library_cl.x04dac
x04dac_ctypes.restype = None
x04dac_ctypes.argtypes = [Nag_OrderType, Nag_MatrixType, Nag_DiagType, Integer, Integer, POINTER(Complex), Integer, STRING, STRING, POINTER(NagError)]

def x04dac(order, matrix, diag, m, n, a, pda, title, outfile, fail):
    """
    Print complex general matrix (easy-to-use).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04dac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('x04dac', a, 'a')
    x04dac_ctypes(order, matrix, diag, m, n, _a, pda, title, outfile, fail)

x04dbc_ctypes = _library_cl.x04dbc
x04dbc_ctypes.restype = None
x04dbc_ctypes.argtypes = [Nag_OrderType, Nag_MatrixType, Nag_DiagType, Integer, Integer, POINTER(Complex), Integer, Nag_ComplexFormType, STRING, STRING, Nag_LabelType, POINTER(STRING), Nag_LabelType, POINTER(STRING), Integer, Integer, STRING, POINTER(NagError)]

def x04dbc(order, matrix, diag, m, n, a, pda, cmplxform, form, title, labrow, rlabs, labcol, clabs, ncols, indent, outfile, fail):
    """
    Print complex general matrix (comprehensive).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04dbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    TypeError will be raised when the following arguments are not arrays of
    strings:
        rlabs, clabs
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_char_type_check_and_cast
    _a = nag_complex_type_check_and_cast('x04dbc', a, 'a')
    _rlabs = nag_char_type_check_and_cast('x04dbc', rlabs, 'rlabs')
    _clabs = nag_char_type_check_and_cast('x04dbc', clabs, 'clabs')
    x04dbc_ctypes(order, matrix, diag, m, n, _a, pda, cmplxform, form, title, labrow, _rlabs, labcol, _clabs, ncols, indent, outfile, fail)

x04dcc_ctypes = _library_cl.x04dcc
x04dcc_ctypes.restype = None
x04dcc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_DiagType, Integer, POINTER(Complex), STRING, STRING, POINTER(NagError)]

def x04dcc(order, uplo, diag, n, a, title, outfile, fail):
    """
    Print complex packed triangular matrix (easy-to-use).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04dcc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('x04dcc', a, 'a')
    x04dcc_ctypes(order, uplo, diag, n, _a, title, outfile, fail)

x04ddc_ctypes = _library_cl.x04ddc
x04ddc_ctypes.restype = None
x04ddc_ctypes.argtypes = [Nag_OrderType, Nag_UploType, Nag_DiagType, Integer, POINTER(Complex), Nag_ComplexFormType, STRING, STRING, Nag_LabelType, POINTER(STRING), Nag_LabelType, POINTER(STRING), Integer, Integer, STRING, POINTER(NagError)]

def x04ddc(order, uplo, diag, n, a, cmplxform, form, title, labrow, rlabs, labcol, clabs, ncols, indent, outfile, fail):
    """
    Print complex packed triangular matrix (comprehensive).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04ddc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    TypeError will be raised when the following arguments are not arrays of
    strings:
        rlabs, clabs
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_char_type_check_and_cast
    _a = nag_complex_type_check_and_cast('x04ddc', a, 'a')
    _rlabs = nag_char_type_check_and_cast('x04ddc', rlabs, 'rlabs')
    _clabs = nag_char_type_check_and_cast('x04ddc', clabs, 'clabs')
    x04ddc_ctypes(order, uplo, diag, n, _a, cmplxform, form, title, labrow, _rlabs, labcol, _clabs, ncols, indent, outfile, fail)

x04dec_ctypes = _library_cl.x04dec
x04dec_ctypes.restype = None
x04dec_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, STRING, STRING, POINTER(NagError)]

def x04dec(order, m, n, kl, ku, a, pda, title, outfile, fail):
    """
    Print complex packed banded matrix (easy-to-use).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04dec.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    """
    from .util import nag_complex_type_check_and_cast
    _a = nag_complex_type_check_and_cast('x04dec', a, 'a')
    x04dec_ctypes(order, m, n, kl, ku, _a, pda, title, outfile, fail)

x04dfc_ctypes = _library_cl.x04dfc
x04dfc_ctypes.restype = None
x04dfc_ctypes.argtypes = [Nag_OrderType, Integer, Integer, Integer, Integer, POINTER(Complex), Integer, Nag_ComplexFormType, STRING, STRING, Nag_LabelType, POINTER(STRING), Nag_LabelType, POINTER(STRING), Integer, Integer, STRING, POINTER(NagError)]

def x04dfc(order, m, n, kl, ku, a, pda, cmplxform, form, title, labrow, rlabs, labcol, clabs, ncols, indent, outfile, fail):
    """
    Print complex packed banded matrix (comprehensive).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04dfc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        a
    TypeError will be raised when the following arguments are not of data type
    Complex:
        a
    TypeError will be raised when the following arguments are not arrays of
    strings:
        rlabs, clabs
    """
    from .util import nag_complex_type_check_and_cast
    from .util import nag_char_type_check_and_cast
    _a = nag_complex_type_check_and_cast('x04dfc', a, 'a')
    _rlabs = nag_char_type_check_and_cast('x04dfc', rlabs, 'rlabs')
    _clabs = nag_char_type_check_and_cast('x04dfc', clabs, 'clabs')
    x04dfc_ctypes(order, m, n, kl, ku, _a, pda, cmplxform, form, title, labrow, _rlabs, labcol, _clabs, ncols, indent, outfile, fail)

x04nac_ctypes = _library_cl.x04nac
x04nac_ctypes.restype = c_int
x04nac_ctypes.argtypes = [STRING]

def x04nac(enumstring):
    """
    Converts NAG enum member name to a value.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04nac.html
    """
    return_value = x04nac_ctypes(enumstring)
    return return_value

x04nbc_ctypes = _library_cl.x04nbc
x04nbc_ctypes.restype = STRING
x04nbc_ctypes.argtypes = [c_int]

def x04nbc(enumvalue):
    """
    Converts NAG enum member value to its name.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04nbc.html
    """
    return_value = x04nbc_ctypes(enumvalue)
    return return_value

x04ncc_ctypes = _library_cl.x04ncc
x04ncc_ctypes.restype = c_int
x04ncc_ctypes.argtypes = [STRING]

def x04ncc(errorname):
    """
    Converts NAG error name to its code value.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04ncc.html
    """
    return_value = x04ncc_ctypes(errorname)
    return return_value

x04ndc_ctypes = _library_cl.x04ndc
x04ndc_ctypes.restype = STRING
x04ndc_ctypes.argtypes = [c_int]

def x04ndc(code):
    """
    Returns the string error name corresponding to a C Libary exit error
    code.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x04/x04ndc.html
    """
    return_value = x04ndc_ctypes(code)
    return return_value

nag_open_file = x04acc
nag_close_file = x04adc
nag_write_line = x04bac
nag_read_line = x04bbc
nag_gen_real_mat_print = x04cac
nag_gen_real_mat_print_comp = x04cbc
nag_pack_real_mat_print = x04ccc
nag_pack_real_mat_print_comp = x04cdc
nag_band_real_mat_print = x04cec
nag_band_real_mat_print_comp = x04cfc
nag_gen_complx_mat_print = x04dac
nag_gen_complx_mat_print_comp = x04dbc
nag_pack_complx_mat_print = x04dcc
nag_pack_complx_mat_print_comp = x04ddc
nag_band_complx_mat_print = x04dec
nag_band_complx_mat_print_comp = x04dfc
nag_enum_name_to_value = x04nac
nag_enum_value_to_name = x04nbc
nag_error_name_to_code = x04ncc
nag_code_to_error_name = x04ndc
__all__ = ['nag_band_complx_mat_print',
           'nag_band_complx_mat_print_comp',
           'nag_band_real_mat_print', 'nag_band_real_mat_print_comp',
           'nag_close_file', 'nag_code_to_error_name',
           'nag_enum_name_to_value', 'nag_enum_value_to_name',
           'nag_error_name_to_code',
           'nag_gen_complx_mat_print',
           'nag_gen_complx_mat_print_comp',
           'nag_gen_real_mat_print',
           'nag_gen_real_mat_print_comp', 'nag_open_file',
           'nag_pack_complx_mat_print',
           'nag_pack_complx_mat_print_comp',
           'nag_pack_real_mat_print', 'nag_pack_real_mat_print_comp',
           'nag_read_line', 'nag_write_line', 'x04acc', 'x04adc',
           'x04bac', 'x04bbc', 'x04bcc', 'x04bdc', 'x04bjc',
           'x04cac', 'x04cbc', 'x04ccc', 'x04cdc', 'x04cec', 'x04cfc',
           'x04dac', 'x04dbc', 'x04dcc', 'x04ddc', 'x04dec', 'x04dfc',
           'x04nac', 'x04nbc', 'x04ncc', 'x04ndc']
