"""
Deprecated wrappers for the NAG C Library (Mark 26.0) x07 Chapter.

x07 - IEEE Arithmetic.

https://www.nag.com/numeric/CL/nagdoc_cl26/html/X07/x07conts.html

NAG Copyright 2017.
"""

from ctypes import c_double
from ctypes import POINTER
from .util import get_library
from .util import Nag_Boolean
from .util import Integer

_library_cl = get_library()


x07aac_ctypes = _library_cl.x07aac
x07aac_ctypes.restype = Nag_Boolean
x07aac_ctypes.argtypes = [c_double]

def x07aac(x):
    """
    Determines whether its argument has a finite value.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x07/x07aac.html
    """
    return_value = x07aac_ctypes(x)
    return return_value

x07abc_ctypes = _library_cl.x07abc
x07abc_ctypes.restype = Nag_Boolean
x07abc_ctypes.argtypes = [c_double]

def x07abc(x):
    """
    Determines whether its argument is a NaN (Not A Number).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x07/x07abc.html
    """
    return_value = x07abc_ctypes(x)
    return return_value

x07bac_ctypes = _library_cl.x07bac
x07bac_ctypes.restype = None
x07bac_ctypes.argtypes = [Integer, POINTER(c_double)]

def x07bac(isign, x):
    """
    Creates a signed infinite value.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x07/x07bac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('x07bac', x, 'x')
    x07bac_ctypes(isign, _x)

x07bbc_ctypes = _library_cl.x07bbc
x07bbc_ctypes.restype = None
x07bbc_ctypes.argtypes = [Integer, POINTER(c_double)]

def x07bbc(quiet, x):
    """
    Creates a NaN (Not A Number).
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x07/x07bbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        x
    TypeError will be raised when the following arguments are not of data type
    numpy.float:
        x
    """
    from .util import nag_double_type_check_and_cast
    _x = nag_double_type_check_and_cast('x07bbc', x, 'x')
    x07bbc_ctypes(quiet, _x)

x07cac_ctypes = _library_cl.x07cac
x07cac_ctypes.restype = None
x07cac_ctypes.argtypes = [POINTER(Integer)]

def x07cac(exceptionmode):
    """
    Gets current behaviour of floating-point exceptions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x07/x07cac.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        exceptionmode
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        exceptionmode
    """
    from .util import nag_integer_type_check_and_cast
    _exceptionmode = nag_integer_type_check_and_cast('x07cac', exceptionmode, 'exceptionmode')
    x07cac_ctypes(_exceptionmode)

x07cbc_ctypes = _library_cl.x07cbc
x07cbc_ctypes.restype = None
x07cbc_ctypes.argtypes = [POINTER(Integer)]

def x07cbc(exceptionmode):
    """
    Sets behaviour of floating-point exceptions.
    https://www.nag.com/numeric/CL/nagdoc_cl26/html/x07/x07cbc.html

    TypeError will be raised when the following arguments are not instances
    of numpy.ndarray:
        exceptionmode
    TypeError will be raised when the following arguments are not of data type
    numpy.int32 or numpy.int64:
        exceptionmode
    """
    from .util import nag_integer_type_check_and_cast
    _exceptionmode = nag_integer_type_check_and_cast('x07cbc', exceptionmode, 'exceptionmode')
    x07cbc_ctypes(_exceptionmode)

nag_is_finite = x07aac
nag_is_nan = x07abc
nag_create_infinity = x07bac
nag_create_nan = x07bbc
nag_get_ieee_exception_mode = x07cac
nag_set_ieee_exception_mode = x07cbc
__all__ = ['nag_create_infinity', 'nag_create_nan',
           'nag_get_ieee_exception_mode', 'nag_is_finite',
           'nag_is_nan', 'nag_set_ieee_exception_mode', 'x07aac',
           'x07abc', 'x07bac', 'x07bbc', 'x07cac', 'x07cbc']
