#!/usr/bin/env python
"""
naginterfaces: Python interfaces for the NAG algorithmic library.

Raises RuntimeError if the host system is not supported.
Raises RuntimeError if the underlying NAG libraries are missing
from the distribution.
Raises RuntimeError if the version-identification text file is missing
from the distribution.
"""

# NAG Copyright 2017-2019.

import os
from platform import system
from struct import calcsize

from setuptools import find_packages, setup

THIS_SYSTEM = system()

if THIS_SYSTEM == "Darwin":
    SHLIB_EXT = "dylib"
elif THIS_SYSTEM == "Linux":
    SHLIB_EXT = "so"
elif THIS_SYSTEM == "Windows":
    SHLIB_EXT = "dll"
else:
    raise RuntimeError("Unexpected system " + THIS_SYSTEM + ".")

PACKAGE_NAME = "naginterfaces"

NAGLIB_PLATDIR = (
    "_".join(["naglib", THIS_SYSTEM]) +
    ('_32' if calcsize('P') == 4 else '')
)

if not os.path.isdir(os.path.join(PACKAGE_NAME, NAGLIB_PLATDIR)):
    raise RuntimeError(
        "Missing Library directory for distribution on " + THIS_SYSTEM + "."
        " Are you using the correct source archive for this platform?"
    )

NAGLIBS = "/".join([
    NAGLIB_PLATDIR,
    ".".join(["*", SHLIB_EXT])
])

VERSION_FILE = os.path.join(PACKAGE_NAME, 'version.txt')

if not os.path.isfile(VERSION_FILE):
    raise RuntimeError("Missing version file.")

with open(VERSION_FILE, 'r') as fo:
    VERSION = fo.readline().rstrip('\n')

SHORT_VERSION = '.'.join(VERSION.split('.')[:2])

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    author="Numerical Algorithms Group",
    author_email="support@nag.com",
    description=(
        "Python interfaces for the NAG algorithmic library, Mark " +
        SHORT_VERSION
    ),
    long_description=open('README.rst').read().replace(':ref:', ''),
    url="https://www.nag.com/",
    license='Other/Proprietary License',
    keywords='nag numerical algorithms',
    packages=find_packages(),
    package_data={PACKAGE_NAME: [NAGLIBS]},
    platforms=["Darwin", "Linux", "Windows"],
    install_requires=["numpy>=1.15"],
    python_requires=">=2.7, !=3.0, !=3.1, !=3.2, !=3.3, <4",
    include_package_data=True,
    zip_safe=False,
)
