#!/usr/bin/env python
"""
Integration tests for wrapper functionality in the roots submodule.

Public use of this module is not supported by NAG.
"""

# NAG Copyright 2018.

# pylint: disable=invalid-name

import unittest

import numpy as np

from naginterfaces.base import utils as b_utils
from test_functionality_core import IntegrationTest

class TestC05bb(IntegrationTest): # pylint: disable=too-few-public-methods
    """
    Abstract suite of integration tests for the c05bb wrapper.
    """

class TestC05bbBase(TestC05bb, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import roots as chapter
        TestC05bb.set_up(self)
        self.fun = chapter.lambertw_complex

    def test_good(self):
        """
        Simple call.
        """
        w, resid = self.fun(0, False, 0.5 - 1.j)
        self.assertAlmostEqual(w, 0.516511 - 0.422053j, places=6)
        self.assertAlmostEqual(resid, 0.)

class TestC05bbNp(TestC05bb, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import roots as chapter
        TestC05bb.set_up(self)
        self.fun = chapter.lambertw_complex

    def test_good(self):
        """
        Simple call.
        """
        w, resid = self.fun(0, False, 0.5 - 1.j)
        self.assertAlmostEqual(w, 0.516511 - 0.422053j, places=6)
        self.assertAlmostEqual(resid, 0.)

class TestC05md(IntegrationTest):
    """
    Abstract suite of integration tests for the c05md wrapper.
    """

    # pylint: disable=too-few-public-methods

    init_x = np.array([2., 0.5, 2., 0.5])
    exp_x = [0.7682, 0.7862, 0.6948, 0.6180]

class TestC05mdBase(TestC05md, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import roots as chapter
        TestC05md.set_up(self)
        self.fun = chapter.sys_func_aa_rcomm

    def test_good(self):
        """
        Good call.
        """
        from math import cos, sin, sqrt

        from naginterfaces.base import machine

        irevcm = 0
        x = np.array([2., 0.5, 2., 0.5])
        n = len(x)
        fvec = np.empty(n)
        comm = {}

        while True:
            irevcm = self.fun(
                irevcm, n, x, fvec,
                sqrt(machine.precision()),
                sqrt(machine.precision()),
                2,
                0.,
                0,
                comm,
            )
            if irevcm == 1:
                continue
            if irevcm == 2:
                fvec = [
                    cos(x[2]) - x[0],
                    sqrt(1.-x[3]**2) - x[1],
                    sin(x[0]) - x[2],
                    x[1]**2 - x[3],
                ]
                continue
            break

        np.testing.assert_allclose(
            x, self.exp_x, rtol=1e-4,
        )

class TestC05mdNp(TestC05md, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import roots as chapter
        TestC05md.set_up(self)
        self.fun = chapter.sys_func_aa_rcomm

    def test_good(self):
        """
        Good call.
        """
        from math import cos, sin, sqrt

        irevcm = 0
        x = np.array([2., 0.5, 2., 0.5])
        fvec = np.empty(len(x))
        comm = {}

        while True:
            irevcm = self.fun(
                irevcm, x, fvec, comm, m=2,
            )
            if irevcm == 1:
                continue
            if irevcm == 2:
                fvec = np.array([
                    cos(x[2]) - x[0],
                    sqrt(1.-x[3]**2) - x[1],
                    sin(x[0]) - x[2],
                    x[1]**2 - x[3],
                ])
                continue
            break

        np.testing.assert_allclose(
            x, self.exp_x, rtol=1e-4,
        )

class TestC05hybr(IntegrationTest):
    """
    Abstract suite of integration tests for the modified Powell nonlinear
    system solvers.
    """

    # pylint: disable=too-few-public-methods

    exp_x = [
        -0.5706545116006593, -0.6816283422912304, -0.7017324525634708,
        -0.7042129400837523, -0.7013690476272891, -0.6918656433799145,
        -0.6657920121546894, -0.5960342012808165, -0.4164120629984719,
    ]
    init_x = [-1.]*len(exp_x)

    @staticmethod
    def trid_fcn(x):
        # pylint: disable=missing-docstring
        fvec = (3. - 2.*x)*x + 1.
        fvec[1:] -= x[:-1]
        fvec[:-1] -= 2.*x[1:]
        return fvec

    @staticmethod
    def trid_fcnj(x):
        # pylint: disable=missing-docstring
        fjac = np.zeros((len(x), len(x)))
        for k in range(1, len(x)):
            fjac[k, k-1] = -1.
        for k, x_el in enumerate(x):
            fjac[k, k] = 3. - 4.*x_el
        for k in range(len(x) - 1):
            fjac[k, k+1] = -2.
        return fjac

class TestC05qbBase(TestC05hybr, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import roots as chapter
        TestC05hybr.set_up(self)
        self.fun = chapter.sys_func_easy

    def test_good(self):
        """
        Good call.
        """
        from copy import copy
        from math import sqrt

        from naginterfaces.base import machine

        def base_fcn(_, x, fvec):
            # pylint: disable=missing-docstring
            fvec[:] = self.trid_fcn(x)

        n = len(self.init_x)
        x = copy(self.init_x)
        xtol = sqrt(machine.precision())
        fvec = np.empty(n)

        self.fun(base_fcn, n, x, fvec, xtol)

        np.testing.assert_allclose(
            x, self.exp_x,
        )

class TestC05qbNp(TestC05hybr, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import roots as chapter
        TestC05hybr.set_up(self)
        self.fun = chapter.sys_func_easy

    def test_good(self):
        """
        Good call.
        """
        np.testing.assert_allclose(
            self.fun(self.trid_fcn, self.init_x).x, self.exp_x,
        )

class TestC05qd(IntegrationTest):
    """
    Abstract suite of integration tests for the c05qd wrapper.
    """

    # pylint: disable=too-few-public-methods

    def default_args(self, n=9):
        """
        A default argument set.
        """
        # pylint: disable=attribute-defined-outside-init
        import math
        self.args = [
            0, # irevcm, 0
            n, # n, 1
            [-1.]*n, # x, 2
            [-42.]*n, # fvec, 3
            math.sqrt(np.finfo(float).eps), # xtol, 4
            1, # ml, 5
            1, # mu, 6
            0., # epsfcn, 7
            2, # mode, 8
            [1.]*n, # diag, 9
            100., # factor, 10
            np.empty((n, n)), # fjac, 11
            [-42.]*(n*(n+1)//2), # r, 12
            [-42.]*n, # qtf, 13
            {}, # comm, 14
        ]

class TestC05qdBase(TestC05qd, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import roots as chapter
        TestC05qd.set_up(self)
        self.fun = chapter.sys_func_rcomm
        self.lineno = 0

    def test_bad_irevcm(self):
        """
        Illegal irevcm.

        Coverage for: handling an Engine error in the base wrappers.
        Coverage for _STR_FMT_INT_AS_STR.
        """
        self.default_args()
        irevcm = -1
        self.args[0] = irevcm
        with self.my_assert_raises_regex( # pylint: disable=deprecated-method
            b_utils.NagValueError, # pylint: disable=bad-continuation
            '.* On entry, irevcm = -1.\n.* Constraint: irevcm.*$' # pylint: disable=bad-continuation
        ) as w_ctxt:
            self.fun(*self.args)
        self.assertEqual(
            w_ctxt.exception.errno,
            2,
        )
        self.assertEqual(
            w_ctxt.exception._errcodes, # pylint: disable=protected-access
            (2, 2, 99991),
        )

    def test_bad_factor(self):
        """
        Illegal factor.

        Coverage for: _STR_FMT_FLOAT_AS_STR.
        """
        self.default_args()
        self.args[10] = -1.
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            r'factor = -0.100000000000000E\+01.\n',
            self.fun, *self.args
        )

    def stagnant_problem(self):
        """
        The Powell Singular problem.
        """
        import inspect
        import math

        self.default_args(n=4)

        self.args[2][:] = [3., -1., 0., 1.]

        irevcm = 0

        while True:
            self.args[0] = irevcm
            self.lineno = inspect.getframeinfo(inspect.currentframe())[1] + 1
            irevcm = self.fun(*self.args)
            if irevcm == 1:
                continue
            if irevcm == 2:
                x, y, z, w = self.args[2]
                self.args[3][:] = [
                    x + 10.*y,
                    math.sqrt(5.)*(-w + z),
                    (y - 2.*z)**2,
                    math.sqrt(10.)*(-w + x)**2
                ]
                continue
            break

    def test_warn_as_warning(self):
        """
        A warning is displayed.

        Coverage for: handling an Engine warning in the base wrappers.
        """
        import inspect
        import warnings

        with warnings.catch_warnings(record=True) as w_ctxt:
            self.stagnant_problem()

        self.assertEqual(len(w_ctxt), 1)
        self.assertIs(w_ctxt[0].category, b_utils.NagAlgorithmicWarning)
        self.assertEqual(
            w_ctxt[0].filename,
            inspect.getframeinfo(inspect.currentframe())[0]
        )
        self.assertEqual(
            w_ctxt[0].lineno,
            self.lineno
        )

    def test_warn_as_exception(self):
        """
        A warning is raised.
        """
        import warnings

        warnings.simplefilter('error', b_utils.NagAlgorithmicWarning)

        with self.assertRaises(b_utils.NagAlgorithmicWarning) as w_ctxt:
            self.stagnant_problem()

        self.assertIn(
            w_ctxt.exception.errno,
            [4, 5],
        )
        self.assertIsInstance(w_ctxt.exception.return_data, tuple)
        self.assertEqual(
            w_ctxt.exception.return_data.__class__.__name__,
            'nag_return_data'
        )
        self.assertEqual(len(w_ctxt.exception.return_data), 1)
        self.assertEqual(
            w_ctxt.exception.return_data.irevcm,
            0
        )
        self.assertTrue(np.linalg.norm(self.args[3]) < 1e-4)
        warnings.simplefilter('always', b_utils.NagAlgorithmicWarning)

class TestC05qsBase(TestC05hybr, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import roots as chapter
        TestC05hybr.set_up(self)
        self.fun = chapter.sparsys_func_easy

    def test_good(self):
        """
        Good call.
        """
        from copy import copy
        from math import sqrt

        from naginterfaces.base import machine

        def base_fcn(_n, _lindf, indf, x, fvec):
            # pylint: disable=missing-docstring
            full_fvec = self.trid_fcn(x)
            for ind in indf:
                fvec[ind - 1] = full_fvec[ind - 1]

        n = len(self.init_x)
        x = copy(self.init_x)
        fvec = np.empty(n)
        xtol = sqrt(machine.precision())
        nnz = n**2
        comm = {}

        self.fun(base_fcn, n, x, fvec, xtol, nnz, comm)

        np.testing.assert_allclose(
            x, self.exp_x,
        )

class TestC05qsNp(TestC05hybr, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.library import roots as chapter
        TestC05hybr.set_up(self)
        self.fun = chapter.sparsys_func_easy

    def test_bad_nnz(self):
        """
        nnz too small.
        """
        self.my_assert_raises_regex(
            b_utils.NagValueError,
            'the supplied nnz.*was too small',
            self.fun,
            self.trid_fcn, self.init_x, nnz=-42,
        )

    def test_good(self):
        """
        Good call.
        """
        def np_fcn(indf, x, fvec):
            # pylint: disable=missing-docstring
            full_fvec = self.trid_fcn(x)
            for ind in indf:
                fvec[ind - 1] = full_fvec[ind - 1]

        np.testing.assert_allclose(
            self.fun(np_fcn, self.init_x).x, self.exp_x,
        )

    def test_good_contin(self):
        """
        Good call.
        """
        import warnings

        ncalls = [0]
        first = True

        def np_fcn(indf, x, fvec):
            # pylint: disable=missing-docstring
            full_fvec = self.trid_fcn(x)
            for ind in indf:
                fvec[ind - 1] = full_fvec[ind - 1]
            if first:
                ncalls[0] += 1
                if ncalls[0] == 20:
                    raise b_utils.UserCallbackTerminate

        comm = {}
        with warnings.catch_warnings(record=True):
            x, _, actual_nnz = self.fun(np_fcn, self.init_x, comm=comm)
        self.assertEqual(actual_nnz, 25)
        first = False
        np.testing.assert_allclose(
            self.fun(np_fcn, x, comm=comm).x, self.exp_x,
        )

class TestC05rbBase(TestC05hybr, unittest.TestCase):
    """
    Concrete base tests.
    """

    def setUp(self):
        from naginterfaces.base import roots as chapter
        TestC05hybr.set_up(self)
        self.fun = chapter.sys_deriv_easy

    def test_good(self):
        """
        Good call.
        """
        from copy import copy
        from math import sqrt

        from naginterfaces.base import machine

        def base_fcn(_, x, fvec):
            # pylint: disable=missing-docstring
            fvec[:] = self.trid_fcn(x)

        def base_fcnj(_, x, fjac):
            # pylint: disable=missing-docstring
            fjac[:, :] = self.trid_fcnj(x)

        n = len(self.init_x)
        x = copy(self.init_x)
        xtol = sqrt(machine.precision())
        fvec = np.empty(n)
        fjac = np.empty((n, n))

        self.fun(base_fcn, base_fcnj, n, x, fvec, fjac, xtol)

        np.testing.assert_allclose(
            x, self.exp_x,
        )

class TestC05rbNp(TestC05hybr, unittest.TestCase):
    """
    Concrete level-2 tests.
    """

    def setUp(self):
        from naginterfaces.library import roots as chapter
        TestC05hybr.set_up(self)
        self.fun = chapter.sys_deriv_easy

    def test_good(self):
        """
        Good call.
        """
        np.testing.assert_allclose(
            self.fun(self.trid_fcn, self.trid_fcnj, self.init_x).x, self.exp_x,
        )

if __name__ == '__main__': # pragma: no cover
    unittest.main()
