# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `nonpar` Chapter.

``nonpar`` - Nonparametric Statistics

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

The functions in this module perform nonparametric statistical tests
which are based on distribution-free methods of analysis.
For convenience, the module contents are divided into five types of
test: tests of location, tests of dispersion, tests of distribution,
tests of association and correlation, and tests of randomness.
There are also functions to fit linear regression models using the ranks
of the observations.

The emphasis in this module is on testing; if you wish to compute
nonparametric correlations you are referred to submodule ``correg``,
which contains several functions for that purpose.

There are a large number of nonparametric tests available.
A selection of some of the more commonly used tests are included in this
module.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def test_sign(x, y, n):
    u"""
    Sign test on two paired samples.

    For full information please refer to the NAG Library document for
    g08aa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08aaf.html

    Parameters
    ----------
    x : float, shape (`n`)
        `x`[i-1] and `y`[i-1] must be set to the ith pair of data
        values, {x_i,y_i}, for i = 1,2,...,n.

    y : float, shape (`n`)
        `x`[i-1] and `y`[i-1] must be set to the ith pair of data
        values, {x_i,y_i}, for i = 1,2,...,n.

    n : int
        n, the size of each sample.

    Returns
    -------
    isgn : int
        The Sign test statistic, S.

    n1 : int
        The number of non-tied pairs, n_1.

    p : float
        The lower tail probability, p, corresponding to S.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, the samples are identical, i.e., n_1 = 0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_sign'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08aaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Output, isgn.
        utils._BYREF_INT, # Output, n1.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _isgn = utils._EngineIntScalarType.empty_ctypes()
    _n1 = utils._EngineIntScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y,
        _n,
        _isgn,
        _n1,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the samples are identical, i.e., n_1 = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _isgn_py = utils._EngineIntScalarType.to_py(_isgn)
        _n1_py = utils._EngineIntScalarType.to_py(_n1)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['isgn', 'n1', 'p'],
        )(
            _isgn_py,
            _n1_py,
            _p_py,
        )
    _err_class.manage()
    return return_data

def test_median(x, n, n1):
    u"""
    Median test on two samples of unequal size.

    For full information please refer to the NAG Library document for
    g08ac

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08acf.html

    Parameters
    ----------
    x : float, shape (`n`)
        The first n_1 elements of `x` must be set to the data values in
        the first sample, and the next n_2 (= `n`-n_1) elements to the
        data values in the second sample.

    n : int
        The total of the two sample sizes, n (= n_1+n_2).

    n1 : int
        The size of the first sample n_1.

    Returns
    -------
    i1 : int
        The number of scores in the first sample which lie below the
        pooled median, i_1.

    i2 : int
        The number of scores in the second sample which lie below the
        pooled median, i_2.

    p : float
        The tail probability p corresponding to the observed dichotomy
        of the two samples.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 2)
            On entry, `n1` = *<value>* and `n` = *<value>*.

            Constraint: 1 <= `n1` < `n`.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_median'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08acft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, n1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_INT, # Output, i1.
        utils._BYREF_INT, # Output, i2.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _i1 = utils._EngineIntScalarType.empty_ctypes()
    _i2 = utils._EngineIntScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _n,
        _n1,
        _w,
        _i1,
        _i2,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 1 <= n1 < n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _i1_py = utils._EngineIntScalarType.to_py(_i1)
        _i2_py = utils._EngineIntScalarType.to_py(_i2)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['i1', 'i2', 'p'],
        )(
            _i1_py,
            _i2_py,
            _p_py,
        )
    _err_class.manage()
    return return_data

def test_friedman(x, k, n):
    u"""
    Friedman two-way analysis of variance on k matched samples.

    For full information please refer to the NAG Library document for
    g08ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08aef.html

    Parameters
    ----------
    x : float, shape (`k`, `n`)
        `x`[i-1,j-1] must be set to the value, x_{ij}, of observation j
        in sample i, for j = 1,2,...,n, for i = 1,2,...,k.

    k : int
        k, the number of samples.

    n : int
        n, the size of each sample.

    Returns
    -------
    fr : float
        The value of the Friedman test statistic, F.

    p : float
        The approximate significance, p, of the Friedman test statistic.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 2.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_friedman'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, n].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Workspace, w1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, w2.
        utils._BYREF_FLOAT64, # Output, fr.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_k_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _w1 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _w2 = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _fr = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _ldx,
        _k,
        _n,
        _w1,
        _w2,
        _fr,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _fr_py = utils._EngineFloat64ScalarType.to_py(_fr)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['fr', 'p'],
        )(
            _fr_py,
            _p_py,
        )
    _err_class.manage()
    return return_data

def test_kruskal(x, lx, l, k):
    u"""
    Kruskal--Wallis one-way analysis of variance on k samples of unequal
    size.

    For full information please refer to the NAG Library document for
    g08af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08aff.html

    Parameters
    ----------
    x : float, shape (`lx`)
        The elements of `x` must contain the observations in the `k`
        samples. The first l_1 elements must contain the scores in the
        first sample, the next l_2 those in the second sample, and so
        on.

    lx : int
        N, the total number of observations.

    l : int, shape (`k`)
        `l`[i] must contain the number of observations l_i in sample i,
        for i = 1,2,...,k.

    k : int
        k, the number of samples.

    Returns
    -------
    h : float
        The value of the Kruskal--Wallis test statistic, H.

    p : float
        The approximate significance, p, of the Kruskal--Wallis test
        statistic.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 2.

        (`errno` 2)
            On entry, i = *<value>* and `l`[i-1] = *<value>*.

            Constraint: `l`[i-1] > 0.

        (`errno` 3)
            On entry, sum_i(`l`[i-1]) = *<value>* and `lx` = *<value>*.

            Constraint: sum_i(`l`[i-1]) = `lx`.

        (`errno` 4)
            On entry, all the observations were equal.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_kruskal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[lx].
        utils._BYREF_INT, # Input, lx.
        utils._EngineIntArrayType.c_type, # Input, l[k].
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Workspace, w.
        utils._BYREF_FLOAT64, # Output, h.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _lx = utils._EngineIntScalarType(
        lx, {'fun_name': fun_name, 'entity': 'lx'},
    ).to_ctypes()
    _lx_py = utils._EngineIntScalarType.to_py(_lx)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_lx_py,),
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _l = utils._EngineIntArrayType(
        l, {'fun_name': fun_name, 'entity': 'l'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _w = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _h = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _lx,
        _l,
        _k,
        _w,
        _h,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and l[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: l[i-1] > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sum_i(l[i-1]) = {:s} and lx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sum_i(l[i-1]) = lx.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all the observations were equal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _h_py = utils._EngineFloat64ScalarType.to_py(_h)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['h', 'p'],
        )(
            _h_py,
            _p_py,
        )
    _err_class.manage()
    return return_data

def test_wilcoxon(n, x, xme, tail, zer):
    u"""
    Performs the Wilcoxon one-sample (matched pairs) signed rank test.

    For full information please refer to the NAG Library document for
    g08ag

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08agf.html

    Parameters
    ----------
    n : int
        n, the size of the sample.

    x : float, shape (`n`)
        The sample observations, x_1,x_2,...,x_n.

    xme : float
        The median test value, X_med.

    tail : str, length 1
        Indicates the choice of tail probability, and hence the
        alternative hypothesis.

        `tail` = 'T'

          A two tailed probability is calculated and the alternative
          hypothesis is H_1: population median != X_med.

        `tail` = 'U'

          An upper tailed probability is calculated and the alternative
          hypothesis is H_1: population median > X_med.

        `tail` = 'L'

          A lower tailed probability is calculated and the alternative
          hypothesis is H_1: population median < X_med.

    zer : str, length 1
        Indicates whether or not to include the cases where d_i = 0.0 in
        the ranking of the d_is.

        `zer` = 'Y'

          All d_i = 0.0 are included when ranking.

        `zer` = 'N'

          All d_i = 0.0, are ignored, that is all cases where d_i = 0.0
          are removed before ranking.

    Returns
    -------
    w : float
        The Wilcoxon rank sum statistic, W, being the sum of the
        positive ranks.

    wnor : float
        The approximate Normal test statistic, z, as described in
        [Description].

    p : float
        The tail probability, p, as specified by the argument `tail`.

    n1 : int
        The number of nonzero d_i's, n_1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `zer` = *<value>*.

            Constraint: `zer` = 'Y' or 'N'.

        (`errno` 1)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'T', 'U' or 'L'.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            All elements of the sample are equal to `xme`, i.e.,
            variance = 0.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_wilcoxon'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08agft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_FLOAT64, # Input, xme.
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_STRING, # Input, zer(len=1).
        utils._BYREF_FLOAT64, # Output, w.
        utils._BYREF_FLOAT64, # Output, wnor.
        utils._BYREF_FLOAT64, # Output, p.
        utils._BYREF_INT, # Output, n1.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of zer.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _xme = utils._EngineFloat64ScalarType(
        xme, {'fun_name': fun_name, 'entity': 'xme'},
    ).to_ctypes()
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _zer = utils._EngineCharScalarType(
        zer, {'fun_name': fun_name, 'entity': 'zer'}, exp_chrlen=1,
    ).to_ctypes()
    _w = utils._EngineFloat64ScalarType.empty_ctypes()
    _wnor = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _n1 = utils._EngineIntScalarType.empty_ctypes()
    _wrk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _xme,
        _tail,
        _zer,
        _w,
        _wnor,
        _p,
        _n1,
        _wrk,
        _errbuf,
        _ierr,
        _tail._length_,
        _zer._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, zer = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: zer = \'Y\' or \'N\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'T\', \'U\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'All elements of the sample are equal to xme, i.e., variance = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_py = utils._EngineFloat64ScalarType.to_py(_w)
        _wnor_py = utils._EngineFloat64ScalarType.to_py(_wnor)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        _n1_py = utils._EngineIntScalarType.to_py(_n1)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['w', 'wnor', 'p', 'n1'],
        )(
            _w_py,
            _wnor_py,
            _p_py,
            _n1_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def test_mwu(n1, x, n2, y, tail, ranks):
    u"""
    Performs the Mann--Whitney U test on two independent samples.

    For full information please refer to the NAG Library document for
    g08ah

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08ahf.html

    Parameters
    ----------
    n1 : int
        The size of the first sample, n_1.

    x : float, shape (`n1`)
        The first vector of observations, x_1,x_2,...,x_n_1.

    n2 : int
        The size of the second sample, n_2.

    y : float, shape (`n2`)
        The second vector of observations. y_1,y_2,...,y_n_2.

    tail : str, length 1
        Indicates the choice of tail probability, and hence the
        alternative hypothesis.

        `tail` = 'T'

          A two tailed probability is calculated and the alternative
          hypothesis is H_1:F(x) != G(y).

        `tail` = 'U'

          An upper tailed probability is calculated and the alternative
          hypothesis H_1:F(x) < G(y), i.e., the x's tend to be greater
          than the y's.

        `tail` = 'L'

          A lower tailed probability is calculated and the alternative
          hypothesis H_1:F(x) > G(y), i.e., the x's tend to be less than
          the y's.

    ranks : float, shape (`n1`+`n2`), modified in place
        On exit: contains the ranks of the pooled sample. The ranks of
        the first sample are contained in the first `n1` elements and
        those of the second sample are contained in the next `n2`
        elements.

    Returns
    -------
    u : float
        The Mann--Whitney rank sum statistic, U.

    unor : float
        The approximate Normal test statistic, z, as described in
        [Description].

    p : float
        The tail probability, p, as specified by the argument `tail`.

    ties : bool
        Indicates whether the pooled sample contained ties or not. This
        will be useful in checking which function to use should one wish
        to calculate an exact tail probability.

        `ties` = False, no ties were present (use ``prob_mwu_noties``
        for an exact probability).

        `ties` = True, ties were present (use ``prob_mwu_ties`` for an
        exact probability).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n2` = *<value>*.

            Constraint: `n2` >= 1.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 2)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'T', 'U' or 'L'.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The pooled sample values are all the same, i.e., the
            variance of `u` = 0.0.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_mwu'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08ahft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n1].
        utils._BYREF_INT, # Input, n2.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n2].
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_FLOAT64, # Output, u.
        utils._BYREF_FLOAT64, # Output, unor.
        utils._BYREF_FLOAT64, # Output, p.
        utils._BYREF_LOGICAL, # Output, ties.
        utils._EngineFloat64ArrayType.c_type, # Output, ranks[n1+n2].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n1_py = utils._EngineIntScalarType.to_py(_n1)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n1_py,),
    ).to_ctypes()
    _n2 = utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _n2_py = utils._EngineIntScalarType.to_py(_n2)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n2_py,),
    ).to_ctypes()
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _u = utils._EngineFloat64ScalarType.empty_ctypes()
    _unor = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _ties = utils._EngineLogicalScalarType.empty_ctypes()
    _ranks_cls = utils._EngineFloat64ArrayType(
        ranks, {'fun_name': fun_name, 'entity': 'ranks'},
        exp_shape=(_n1_py+_n2_py,),
        check_contents=False,
    )
    _ranks = _ranks_cls.to_ctypes(can_reference_contents=False)
    _wrk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n1,
        _x,
        _n2,
        _y,
        _tail,
        _u,
        _unor,
        _p,
        _ties,
        _ranks,
        _wrk,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 89999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n2 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'T\', \'U\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The pooled sample values are all the same, i.e., the variance of u = 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ranks_cls.refresh_py(_ranks)
        _u_py = utils._EngineFloat64ScalarType.to_py(_u)
        _unor_py = utils._EngineFloat64ScalarType.to_py(_unor)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        _ties_py = utils._EngineLogicalScalarType.to_py(_ties)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['u', 'unor', 'p', 'ties'],
        )(
            _u_py,
            _unor_py,
            _p_py,
            _ties_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_mwu_noties(n1, n2, tail, u):
    u"""
    Computes the exact probabilities for the Mann--Whitney U statistic,
    no ties in pooled sample.

    For full information please refer to the NAG Library document for
    g08aj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08ajf.html

    Parameters
    ----------
    n1 : int
        The number of non-tied pairs, n_1.

    n2 : int
        The size of the second sample, n_2.

    tail : str, length 1
        Indicates the choice of tail probability, and hence the
        alternative hypothesis.

        `tail` = 'T'

          A two tailed probability is calculated and the alternative
          hypothesis is H_1:F(x) != G(y).

        `tail` = 'U'

          An upper tailed probability is calculated and the alternative
          hypothesis H_1:F(x) < G(y), i.e., the x's tend to be greater
          than the y's.

        `tail` = 'L'

          A lower tailed probability is calculated and the alternative
          hypothesis H_1:F(x) > G(y), i.e., the x's tend to be less than
          the y's.

    u : float
        U, the value of the Mann--Whitney rank sum test statistic. This
        is the statistic returned through the argument `u` by
        ``test_mwu``.

    Returns
    -------
    p : float
        The exact tail probability, p, as specified by the argument
        `tail`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n2` = *<value>*.

            Constraint: `n2` >= 1.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 2)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'T', 'U' or 'L'.

        (`errno` 3)
            On entry, `u` = *<value>*.

            Constraint: `u` >= 0.0.

        (`errno` 4)
            On entry, lwrk = *<value>* and the minimum workspace is
            *<value>*.

            Constraint: lwrk >= (`n1`*`n2`)/2+1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.prob_mwu_noties'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08ajft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_INT, # Input, n2.
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_FLOAT64, # Input, u.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrk.
        utils._BYREF_INT, # Input, lwrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n2 = utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _u = utils._EngineFloat64ScalarType(
        u, {'fun_name': fun_name, 'entity': 'u'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _wrk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwrk = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n1,
        _n2,
        _tail,
        _u,
        _p,
        _wrk,
        _lwrk,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 89999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n2 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'T\', \'U\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, u = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: u >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lwrk = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'the minimum workspace is {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lwrk >= (n1*n2)/2+1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_mwu_ties(n1, n2, tail, ranks, u):
    u"""
    Computes the exact probabilities for the Mann--Whitney U statistic,
    ties in pooled sample.

    For full information please refer to the NAG Library document for
    g08ak

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08akf.html

    Parameters
    ----------
    n1 : int
        The number of non-tied pairs, n_1.

    n2 : int
        The size of the second sample, n_2.

    tail : str, length 1
        Indicates the choice of tail probability, and hence the
        alternative hypothesis.

        `tail` = 'T'

          A two tailed probability is calculated and the alternative
          hypothesis is H_1:F(x) != G(y).

        `tail` = 'U'

          An upper tailed probability is calculated and the alternative
          hypothesis H_1:F(x) < G(y), i.e., the x's tend to be greater
          than the y's.

        `tail` = 'L'

          A lower tailed probability is calculated and the alternative
          hypothesis H_1:F(x) > G(y), i.e., the x's tend to be less than
          the y's.

    ranks : float, shape (`n1`+`n2`)
        The ranks of the pooled sample. These ranks are output in the
        array `ranks` by ``test_mwu`` and should not be altered in any
        way if you are using the same n_1, n_2 and `u` as used in
        ``test_mwu``.

    u : float
        U, the value of the Mann--Whitney rank sum test statistic. This
        is the statistic returned through the argument `u` by
        ``test_mwu``.

    Returns
    -------
    p : float
        The tail probability, p, as specified by the argument `tail`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n2` = *<value>*.

            Constraint: `n2` >= 1.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 2)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'T', 'U' or 'L'.

        (`errno` 3)
            On entry, `u` = *<value>*.

            Constraint: `u` >= 0.0.

        (`errno` 5)
            On entry, at least one rank, given in `ranks`, was outside
            the expected range.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.prob_mwu_ties'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08akft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_INT, # Input, n2.
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input, ranks[n1+n2].
        utils._BYREF_FLOAT64, # Input, u.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrk.
        utils._BYREF_INT, # Input, lwrk.
        utils._EngineIntArrayType.c_type, # Workspace, iwrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n2 = utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _n1_py = utils._EngineIntScalarType.to_py(_n1)
    _n2_py = utils._EngineIntScalarType.to_py(_n2)
    _ranks = utils._EngineFloat64ArrayType(
        ranks, {'fun_name': fun_name, 'entity': 'ranks'},
        exp_shape=(_n1_py+_n2_py,),
    ).to_ctypes()
    _u = utils._EngineFloat64ScalarType(
        u, {'fun_name': fun_name, 'entity': 'u'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _wrk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwrk = utils._EngineIntScalarType.c_type(0)
    _iwrk = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n1,
        _n2,
        _tail,
        _ranks,
        _u,
        _p,
        _wrk,
        _lwrk,
        _iwrk,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 89999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n2 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'T\', \'U\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, u = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: u >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one rank, given in ranks, was outside the expected range.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def test_cochranq(n, k, x):
    u"""
    Performs the Cochran Q test on cross-classified binary data.

    For full information please refer to the NAG Library document for
    g08al

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08alf.html

    Parameters
    ----------
    n : int
        n, the number of blocks.

    k : int
        k, the number of treatments.

    x : float, shape (`n`, `k`)
        The matrix of observed zero-one data. `x`[i-1,j-1] must contain
        the value x_{ij}, for j = 1,2,...,k, for i = 1,2,...,n.

    Returns
    -------
    q : float
        The value of the Cochran Q-test statistic.

    prob : float
        The upper tail probability, p, associated with the Cochran
        Q-test statistic, that is the probability of obtaining a value
        greater than or equal to the observed value (the output value of
        `q`).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 2.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 2)
            On entry, i = *<value>*, j = *<value>* and `x`[i-1,j-1] =
            *<value>*.

            Constraint: `x`[i-1,j-1] = 0.0 or 1.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The solution has failed to converge while calculating the
            tail probability.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_cochranq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08alft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, k].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_FLOAT64, # Output, q.
        utils._BYREF_FLOAT64, # Output, prob.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _k_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _q = utils._EngineFloat64ScalarType.empty_ctypes()
    _prob = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _k,
        _x,
        _ldx,
        _q,
        _prob,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, j = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[i-1,j-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[i-1,j-1] = 0.0 or 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solution has failed to converge while calculating the tail probability.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _q_py = utils._EngineFloat64ScalarType.to_py(_q)
        _prob_py = utils._EngineFloat64ScalarType.to_py(_prob)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['q', 'prob'],
        )(
            _q_py,
            _prob_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def test_mooddavid(x, n, n1, r, itest):
    u"""
    Mood's and David's tests on two samples of unequal size.

    For full information please refer to the NAG Library document for
    g08ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08baf.html

    Parameters
    ----------
    x : float, shape (`n`)
        The first n_1 elements of `x` must be set to the data values in
        the first sample, and the next n_2 (= `n`-n_1) elements to the
        data values in the second sample.

    n : int
        The total of the two sample sizes, n (= n_1+n_2).

    n1 : int
        The size of the first sample, n_1.

    r : float, shape (`n`), modified in place
        On exit: the ranks r_i, assigned to the data values x_i, for i =
        1,2,...,n.

    itest : int
        The test(s) to be carried out.

        `itest` = 0

          Both Mood's and David's tests.

        `itest` = 1

          David's test only.

        `itest` = 2

          Mood's test only.

    Returns
    -------
    w : float
        Mood's test statistic, W, if requested.

    v : float
        David's test statistic, V, if requested.

    pw : float
        The lower tail probability, p_w, corresponding to the value of
        W, if Mood's test was requested.

    pv : float
        The lower tail probability, p_v, corresponding to the value of
        V, if David's test was requested.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 2.

        (`errno` 2)
            On entry, `n1` = *<value>* and `n` = *<value>*.

            Constraint: 1 < `n1` < `n`.

        (`errno` 3)
            On entry, `itest` = *<value>*.

            Constraint: `itest` = 0, 1 or 2.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_mooddavid'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, n1.
        utils._EngineFloat64ArrayType.c_type, # Output, r[n].
        utils._BYREF_INT, # Input, itest.
        utils._BYREF_FLOAT64, # Output, w.
        utils._BYREF_FLOAT64, # Output, v.
        utils._BYREF_FLOAT64, # Output, pw.
        utils._BYREF_FLOAT64, # Output, pv.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _itest = utils._EngineIntScalarType(
        itest, {'fun_name': fun_name, 'entity': 'itest'},
    ).to_ctypes()
    _w = utils._EngineFloat64ScalarType.empty_ctypes()
    _v = utils._EngineFloat64ScalarType.empty_ctypes()
    _pw = utils._EngineFloat64ScalarType.empty_ctypes()
    _pv = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _n,
        _n1,
        _r,
        _itest,
        _w,
        _v,
        _pw,
        _pv,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 1 < n1 < n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, itest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: itest = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _r_cls.refresh_py(_r)
        _w_py = utils._EngineFloat64ScalarType.to_py(_w)
        _v_py = utils._EngineFloat64ScalarType.to_py(_v)
        _pw_py = utils._EngineFloat64ScalarType.to_py(_pw)
        _pv_py = utils._EngineFloat64ScalarType.to_py(_pv)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['w', 'v', 'pw', 'pv'],
        )(
            _w_py,
            _v_py,
            _pw_py,
            _pv_py,
        )
    _err_class.manage()
    return return_data

def test_ks_1sample(n, x, dist, par, estima, ntype, sx):
    u"""
    Performs the one-sample Kolmogorov--Smirnov test for standard
    distributions.

    For full information please refer to the NAG Library document for
    g08cb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08cbf.html

    Parameters
    ----------
    n : int
        n, the number of observations in the sample.

    x : float, shape (`n`)
        The sample observations x_1,x_2,...,x_n.

    dist : str
        The theoretical (null) distribution from which it is suspected
        the data may arise.

        `dist` = 'U'

          The uniform distribution over (a,b).

        `dist` = 'N'

          The Normal distribution with mean mu and variance sigma^2.

        `dist` = 'G'

          The gamma distribution with shape parameteralpha and scale
          parameter beta, where the mean = alphabeta.

        `dist` = 'BE'

          The beta distribution with shape parameters alpha and beta,
          where the mean = alpha/(alpha+beta).

        `dist` = 'BI'

          The binomial distribution with the number of trials, m, and
          the probability of a success, p.

        `dist` = 'E'

          The exponential distribution with parameter lambda, where the
          mean = 1/lambda.

        `dist` = 'P'

          The Poisson distribution with parameter mu, where the mean =
          mu.

        `dist` = 'NB'

          The negative binomial distribution with the number of trials,
          m, and the probability of success, p.

        `dist` = 'GP'

          The generalized Pareto distribution with shape parameter xi
          and scale beta.

        Any number of characters may be supplied as the actual
        parameter, however only the characters, maximum 2, required to
        uniquely identify the distribution are referenced.

    par : float, shape (2), modified in place
        On entry: if `estima` = 'S', `par` must contain the known values
        of the parameter(s) of the null distribution as follows.

        If a uniform distribution is used, `par`[0] and `par`[1] must
        contain the boundaries a and b respectively.

        If a Normal distribution is used, `par`[0] and `par`[1] must
        contain the mean, mu, and the variance, sigma^2, respectively.

        If a gamma distribution is used, `par`[0] and `par`[1] must
        contain the parameters alpha and beta respectively.

        If a beta distribution is used, `par`[0] and `par`[1] must
        contain the parameters alpha and beta respectively.

        If a binomial distribution is used, `par`[0] and `par`[1] must
        contain the parameters m and p respectively.

        If an exponential distribution is used, `par`[0] must contain
        the parameter lambda.

        If a Poisson distribution is used, `par`[0] must contain the
        parameter mu.

        If a negative binomial distribution is used, `par`[0] and
        `par`[1] must contain the parameters m and p respectively.

        If a generalized Pareto distribution is used, `par`[0] and
        `par`[1] must contain the parameters xi and beta respectively.

        If `estima` = 'E', `par` need not be set except when the null
        distribution requested is either the binomial or the negative
        binomial distribution in which case `par`[0] must contain the
        parameter m.

        On exit: if `estima` = 'S', `par` is unchanged; if `estima` =
        'E', and `dist` = 'BI' or `dist` = 'NB' then `par`[1] is
        estimated from the data; otherwise `par`[0] and `par`[1] are
        estimated from the data.

    estima : str, length 1
        `estima` must specify whether values of the parameters of the
        null distribution are known or are to be estimated from the
        data.

        `estima` = 'S'

          Values of the parameters will be supplied in the array `par`
          described above.

        `estima` = 'E'

          Parameters are to be estimated from the data except when the
          null distribution requested is the binomial distribution or
          the negative binomial distribution in which case the first
          parameter, m, must be supplied in `par`[0] and only the second
          parameter, p, is estimated from the data.

    ntype : int
        The test statistic to be calculated, i.e., the choice of
        alternative hypothesis.

        `ntype` = 1

          Computes D_n, to test H_0 against H_1,

        `ntype` = 2

          Computes D_n^+, to test H_0 against H_2,

        `ntype` = 3

          Computes D_n^-, to test H_0 against H_3.

    sx : float, shape (`n`), modified in place
        On exit: the sample observations, x_1,x_2,...,x_n, sorted in
        ascending order.

    Returns
    -------
    d : float
        The Kolmogorov--Smirnov test statistic (D_n, D_n^+ or D_n^-
        according to the value of `ntype`).

    z : float
        A standardized value, Z, of the test statistic, D, without any
        correction for continuity.

    p : float
        The probability, p, associated with the observed value of D
        where D may be D_n,D_n^+ or D_n^- depending on the value of
        `ntype` (see [Description]).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 3.

        (`errno` 2)
            On entry, `dist` = *<value>*.

            Constraint: `dist` = 'U', 'N', 'G', 'BE', 'BI', 'E', 'P',
            'NB' or 'GP'.

        (`errno` 3)
            On entry, `ntype` = *<value>*.

            Constraint: `ntype` = 1, 2 or 3.

        (`errno` 4)
            On entry, `estima` = *<value>*.

            Constraint: `estima` = 'E' or 'S'.

        (`errno` 5)
            On entry, `estima` = 'S' and `par`[0] = *<value>*; `par`[1]
            = *<value>*.

            Constraint: for the generalized Pareto distribution with
            `par`[0] < 0, 0 <= `x`[i-1] <= -`par`[1]/`par`[0], for i =
            1,2,...,`n`.

        (`errno` 5)
            On entry, `estima` = 'S' and `par`[1] = *<value>*.

            Constraint: for the generalized Pareto distribution,
            `par`[1] > 0.

        (`errno` 5)
            On entry, `estima` = 'S' and `par`[1] = *<value>*.

            Constraint: for the negative binomial distribution, 0 <
            `par`[1] < 1.

        (`errno` 5)
            On entry, `dist` = 'NB' and m = `par`[0] = *<value>*.

            Note that m must always be supplied.

            Constraint: for the negative binomial distribution, 1 <=
            `par`[0] < 1/eps, where eps = machine precision, see
            ``machine.precision``.

        (`errno` 5)
            On entry, `estima` = 'S' and `par`[0] = *<value>*.

            Constraint: for the Poisson distribution, 0 < `par`[0] <
            1000000.

        (`errno` 5)
            On entry, `estima` = 'S' and `par`[0] = *<value>*.

            Constraint: for the exponential distribution, `par`[0] > 0.

        (`errno` 5)
            On entry, `estima` = 'S' and `par`[1] = *<value>*.

            Constraint: for the binomial distribution, 0 < `par`[1] < 1.

        (`errno` 5)
            On entry, `dist` = 'BI' and m = `par`[0] = *<value>*.

            Note that m must always be supplied.

            Constraint: for the binomial distribution, 1 <= `par`[0] <
            1/eps, where eps = machine precision, see
            ``machine.precision``.

        (`errno` 5)
            On entry, `estima` = 'S' and `par`[0] = *<value>*; `par`[1]
            = *<value>*.

            Constraint: for the beta distribution, 0 < `par`[0] and
            `par`[1] <= 1000000.

        (`errno` 5)
            On entry, `estima` = 'S' and `par`[0] = *<value>*; `par`[1]
            = *<value>*.

            Constraint: for the gamma distribution, `par`[0] and
            `par`[1] > 0.

        (`errno` 5)
            On entry, `estima` = 'S' and `par`[1] = *<value>*.

            Constraint: for the Normal distribution, `par`[1] > 0.

        (`errno` 5)
            On entry, `estima` = 'S' and `par`[0] = *<value>*; `par`[1]
            = *<value>*.

            Constraint: for the uniform distribution, `par`[0] <
            `par`[1].

        (`errno` 6)
            On entry, `dist` = 'GP' and `estima` = 'E'.

            The parameter estimates are invalid; the data may not be
            from the generalized Pareto distribution.

        (`errno` 6)
            On entry, `dist` = 'E' or 'P' and all observations are zero.

            Constraint: at least one `x`[i-1] > 0, for i = 1,2,...,`n`.

        (`errno` 6)
            On entry, `dist` = 'BI' and all observations are zero or m.

            Constraint: at least one 0.0 < `x`[i-1] < `par`[0], for i =
            1,2,...,`n`.

        (`errno` 6)
            On entry, `dist` = 'G', 'E', 'P', 'NB' or 'GP' and at least
            one observation is negative.

            Constraint: `x`[i-1] >= 0, for i = 1,2,...,`n`.

        (`errno` 6)
            On entry, `dist` = 'BI' and at least one observation is
            illegal.

            Constraint: 0 <= `x`[i-1] <= `par`[0], for i = 1,2,...,`n`.

        (`errno` 6)
            On entry, `dist` = 'BE' and at least one observation is
            illegal.

            Constraint: 0 <= `x`[i-1] <= 1, for i = 1,2,...,`n`.

        (`errno` 6)
            On entry, `dist` = 'U' and at least one observation is
            illegal.

            Constraint: `par`[0] <= `x`[i-1] <= `par`[1], for i =
            1,2,...,`n`.

        (`errno` 7)
            On entry, `dist` = 'U', 'N', 'G', 'BE' or 'GP', `estima` =
            'E' and the whole sample is constant. Thus the variance is
            zero.

        (`errno` 8)
            On entry, `dist` = 'NB', `par`[0] = *<value>*, `par`[1] =
            *<value>*.

            The variance `par`[0]*(1-`par`[1])/(`par`[1]*`par`[1])
            exceeds 1000000.

        (`errno` 8)
            On entry, `dist` = 'BI', `par`[0] = *<value>*, `par`[1] =
            *<value>*.

            The variance `par`[0]*`par`[1]*(1-`par`[1]) exceeds 1000000.

        (`errno` 9)
            On entry, `dist` = 'G' and in the computation of the
            incomplete gamma function by ``specfun.gamma_incomplete``
            the convergence of the Taylor series or Legendre continued
            fraction fails within 600 iterations.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_ks_1sample'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08cbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, idist.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, par[2].
        utils._BYREF_INT, # Input, iest.
        utils._BYREF_INT, # Input, ntype.
        utils._BYREF_FLOAT64, # Output, d.
        utils._BYREF_FLOAT64, # Output, z.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineFloat64ArrayType.c_type, # Output, sx[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _dist = utils._EngineCharScalarType(
        dist, {'fun_name': fun_name, 'entity': 'dist'}, exp_chrlen=None,
    ).to_ctypes()
    _dist_py = utils._EngineCharScalarType.to_py(_dist)
    if _dist_py.upper() == 'U':
        _idist = utils._EngineIntScalarType.c_type(1)
    elif _dist_py.upper() == 'UN':
        _idist = utils._EngineIntScalarType.c_type(1)
    elif _dist_py.upper() == 'N':
        _idist = utils._EngineIntScalarType.c_type(2)
    elif _dist_py.upper() == 'NO':
        _idist = utils._EngineIntScalarType.c_type(2)
    elif _dist_py.upper() == 'G':
        _idist = utils._EngineIntScalarType.c_type(3)
    elif _dist_py.upper() == 'GA':
        _idist = utils._EngineIntScalarType.c_type(3)
    elif _dist_py.upper() == 'BE':
        _idist = utils._EngineIntScalarType.c_type(4)
    elif _dist_py.upper() == 'BI':
        _idist = utils._EngineIntScalarType.c_type(5)
    elif _dist_py.upper() == 'E':
        _idist = utils._EngineIntScalarType.c_type(6)
    elif _dist_py.upper() == 'EX':
        _idist = utils._EngineIntScalarType.c_type(6)
    elif _dist_py.upper() == 'P':
        _idist = utils._EngineIntScalarType.c_type(7)
    elif _dist_py.upper() == 'PO':
        _idist = utils._EngineIntScalarType.c_type(7)
    elif _dist_py.upper() == 'NB':
        _idist = utils._EngineIntScalarType.c_type(8)
    elif _dist_py.upper() == 'GP':
        _idist = utils._EngineIntScalarType.c_type(9)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, dist = {:s}.'.format(
                    utils._STR_FMT_STR.format(_dist_py),
                ),
                'Constraint: dist = \'U\', \'N\', \'G\', \'BE\', \'BI\', \'E\', \'P\', \'NB\' or \'GP\'.',
            ],
            errcodes=(2,),
        )
        _err_class.manage()
    _par_cls = utils._EngineFloat64ArrayType(
        par, {'fun_name': fun_name, 'entity': 'par'},
        exp_shape=(2,),
    )
    _par = _par_cls.to_ctypes()
    _estima = utils._EngineCharScalarType(
        estima, {'fun_name': fun_name, 'entity': 'estima'}, exp_chrlen=1,
    ).to_ctypes()
    _estima_py = utils._EngineCharScalarType.to_py(_estima)
    if _estima_py.upper() == 'S':
        _iest = utils._EngineIntScalarType.c_type(2)
    elif _estima_py.upper() == 'E':
        _iest = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineErrorExit(
            fun_name=fun_name,
            msg_lines=[
                'On entry, estima = {:s}.'.format(
                    utils._STR_FMT_STR.format(_estima_py),
                ),
                'Constraint: estima = \'E\' or \'S\'.',
            ],
            errcodes=(4,),
        )
        _err_class.manage()
    _ntype = utils._EngineIntScalarType(
        ntype, {'fun_name': fun_name, 'entity': 'ntype'},
    ).to_ctypes()
    _d = utils._EngineFloat64ScalarType.empty_ctypes()
    _z = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _sx_cls = utils._EngineFloat64ArrayType(
        sx, {'fun_name': fun_name, 'entity': 'sx'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _sx = _sx_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _idist,
        _par,
        _iest,
        _ntype,
        _d,
        _z,
        _p,
        _sx,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ntype = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ntype = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99969:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, estima = \'S\' and par[0] = {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for the generalized Pareto distribution with par[0] < 0,',
                    '0 <= x[i-1] <= -par[1]/par[0], for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99971:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, estima = \'S\' and par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: for the generalized Pareto distribution, par[1] > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99974:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, estima = \'S\' and par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: for the negative binomial distribution, 0 < par[1] < 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99975:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'NB\' and m = par[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Note that m must always be supplied.',
                    'Constraint: for the negative binomial distribution,',
                    '1 <= par[0] < 1/eps, where eps = machine precision, see machine.precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, estima = \'S\' and par[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: for the Poisson distribution, 0 < par[0] < 1000000.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, estima = \'S\' and par[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: for the exponential distribution, par[0] > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, estima = \'S\' and par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: for the binomial distribution, 0 < par[1] < 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'BI\' and m = par[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Note that m must always be supplied.',
                    'Constraint: for the binomial distribution,',
                    '1 <= par[0] < 1/eps, where eps = machine precision, see machine.precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, estima = \'S\' and par[0] = {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for the beta distribution, 0 < par[0] and par[1] <= 1000000.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, estima = \'S\' and par[0] = {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for the gamma distribution, par[0] and par[1] > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, estima = \'S\' and par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: for the Normal distribution, par[1] > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, estima = \'S\' and par[0] = {:s};'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for the uniform distribution, par[0] < par[1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99968:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'GP\' and estima = \'E\'.',
                    'The parameter estimates are invalid; the data may not be from the',
                    'generalized Pareto distribution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99976:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'E\' or \'P\' and all observations are zero.',
                    'Constraint: at least one x[i-1] > 0, for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99977:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'BI\' and all observations are zero or m.',
                    'Constraint: at least one 0.0 < x[i-1] < par[0], for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99982:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'G\', \'E\', \'P\', \'NB\' or \'GP\'',
                    'and at least one observation is negative.',
                    'Constraint: x[i-1] >= 0, for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'G\', \'E\', \'P\', \'NB\' or \'GP\'',
                    'and at least one observation is negative.',
                    'Constraint: x[i-1] >= 0, for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99984:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'BI\' and at least one observation is illegal.',
                    'Constraint: 0 <= x[i-1] <= par[0], for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'BE\' and at least one observation is illegal.',
                    'Constraint: 0 <= x[i-1] <= 1, for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'G\', \'E\', \'P\', \'NB\' or \'GP\'',
                    'and at least one observation is negative.',
                    'Constraint: x[i-1] >= 0, for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'U\' and at least one observation is illegal.',
                    'Constraint: par[0] <= x[i-1] <= par[1], for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99981:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'U\', \'N\', \'G\', \'BE\' or \'GP\', estima = \'E\' and',
                    'the whole sample is constant. Thus the variance is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99972:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'NB\', par[0] = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'The variance par[0]*(1-par[1])/(par[1]*par[1]) exceeds 1000000.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99980:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'BI\', par[0] = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'The variance par[0]*par[1]*(1-par[1]) exceeds 1000000.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99979:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'G\' and in the computation of the incomplete',
                    'gamma function by specfun.gamma_incomplete the convergence of the Taylor series or Legendre',
                    'continued fraction fails within 600 iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _par_cls.refresh_py(_par)
        _sx_cls.refresh_py(_sx)
        _d_py = utils._EngineFloat64ScalarType.to_py(_d)
        _z_py = utils._EngineFloat64ScalarType.to_py(_z)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['d', 'z', 'p'],
        )(
            _d_py,
            _z_py,
            _p_py,
        )
    _err_class.manage()
    return return_data

def test_ks_1sample_user(n, x, cdf, ntype, data=None):
    u"""
    Performs the one-sample Kolmogorov--Smirnov test for a user-supplied
    distribution.

    For full information please refer to the NAG Library document for
    g08cc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08ccf.html

    Parameters
    ----------
    n : int
        n, the number of observations in the sample.

    x : float, shape (`n`)
        The sample observations, x_1,x_2,...,x_n.

    cdf : callable retval = cdf(x, data=None)
        `cdf` must return the value of the theoretical (null) cumulative
        distribution function for a given value of its argument.

        Parameters
        ~~~~~~~~~~
        x : float
            The argument for which `cdf` must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of the theoretical (null) cumulative distribution
            function evaluated at `x`.

    ntype : int
        The statistic to be calculated, i.e., the choice of alternative
        hypothesis.

        `ntype` = 1

          Computes D_n, to test H_0 against H_1.

        `ntype` = 2

          Computes D_n^+, to test H_0 against H_2.

        `ntype` = 3

          Computes D_n^-, to test H_0 against H_3.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    d : float
        The Kolmogorov--Smirnov test statistic (D_n, D_n^+ or D_n^-
        according to the value of `ntype`).

    z : float
        A standardized value, Z, of the test statistic, D, without the
        continuity correction applied.

    p : float
        The probability, p, associated with the observed value of D,
        where D may D_n, D_n^+ or D_n^- depending on the value of
        `ntype` (see [Description]).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, `ntype` = *<value>*.

            Constraint: `ntype` = 1, 2 or 3.

        (`errno` 3)
            On entry, at x = *<value>*, F_0(x) = *<value>*.

            Constraint: 0.0 <= F_0(x) <= 1.

        (`errno` 4)
            On entry, at x = *<value>*, F_0(x) = *<value>* and at y =
            *<value>*, F_0(y) = *<value>*

            Constraint: when x < y, F_0(x) <= F_0(y).

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_ks_1sample_user'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_cdf_name = fun_name + ':cdf'

    cdfh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_cdfh(_cdf, _cdfsub, en_data, x, retval, _iuser, _ruser):
        """Helper for cdf."""
        try:
            _x_cdfh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_cdfh = cdf(
                    _x_cdfh,
                    data,
                )
            else:
                _retval_cdfh = cdf(
                    _x_cdfh,
                )
            utils._EngineFloat64ScalarType(
                _retval_cdfh, {'fun_name': cb_cdf_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('g08ccft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        cdfh,
        utils._BYREF_INT, # Input, ntype.
        utils._BYREF_FLOAT64, # Output, d.
        utils._BYREF_FLOAT64, # Output, z.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineFloat64ArrayType.c_type, # Output, sx[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    utils._check_callable(cdf, {'fun_name': fun_name, 'entity': 'cdf'})
    _cdf = utils._UNUSED_CALLBACK
    _cdfsub = utils._UNUSED_CALLBACK
    _cdfh = utils._cast_callback(
        cdfh, cb_cdfh,
        {'fun_name': fun_name, 'entity': 'cdf'},
    )
    _ntype = utils._EngineIntScalarType(
        ntype, {'fun_name': fun_name, 'entity': 'ntype'},
    ).to_ctypes()
    _d = utils._EngineFloat64ScalarType.empty_ctypes()
    _z = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _sx = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _cdf,
        _cdfsub,
        _cdfh,
        _ntype,
        _d,
        _z,
        _p,
        _sx,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ntype = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ntype = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at x = {:s}, F_0(x) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0.0 <= F_0(x) <= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at x = {:s}, F_0(x) = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'at y = {:s}, F_0(y) = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: when x < y, F_0(x) <= F_0(y).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _d_py = utils._EngineFloat64ScalarType.to_py(_d)
        _z_py = utils._EngineFloat64ScalarType.to_py(_z)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['d', 'z', 'p'],
        )(
            _d_py,
            _z_py,
            _p_py,
        )
    _err_class.manage()
    return return_data

def test_ks_2sample(n1, x, n2, y, ntype, sx, sy):
    u"""
    Performs the two-sample Kolmogorov--Smirnov test.

    For full information please refer to the NAG Library document for
    g08cd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08cdf.html

    Parameters
    ----------
    n1 : int
        The number of observations in the first sample, n_1.

    x : float, shape (`n1`)
        The observations from the first sample, x_1,x_2,...,x_n_1.

    n2 : int
        The number of observations in the second sample, n_2.

    y : float, shape (`n2`)
        The observations from the second sample, y_1,y_2,...,y_n_2.

    ntype : int
        The statistic to be computed, i.e., the choice of alternative
        hypothesis.

        `ntype` = 1

          Computes D_{n_1 n_2}, to test against H_1.

        `ntype` = 2

          Computes D_n_1 n_2^+, to test against H_2.

        `ntype` = 3

          Computes D_n_1 n_2^-, to test against H_3.

    sx : float, shape (`n1`), modified in place
        On exit: the observations from the first sample sorted in
        ascending order.

    sy : float, shape (`n2`), modified in place
        On exit: the observations from the second sample sorted in
        ascending order.

    Returns
    -------
    d : float
        The Kolmogorov--Smirnov test statistic (D_{n_1 n_2}, D_n_1 n_2^+
        or D_n_1 n_2^- according to the value of `ntype`).

    z : float
        A standardized value, Z, of the test statistic, D, without any
        correction for continuity.

    p : float
        The tail probability associated with the observed value of D,
        where D may be D_{n_1,n_2},D_n_1,n_2^+ or D_n_1,n_2^- depending
        on the value of `ntype` (see [Description]).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n2` = *<value>*.

            Constraint: `n2` >= 1.

        (`errno` 1)
            On entry, `n1` = *<value>*.

            Constraint: `n1` >= 1.

        (`errno` 2)
            On entry, `ntype` = *<value>*.

            Constraint: `ntype` = 1, 2 or 3.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The iterative process used in the approximation of the
            probability for large n_1 and n_2 did not converge. For the
            two sided test p = 1 is returned. For the one-sided test p =
            0.5 is returned.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_ks_2sample'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08cdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n1].
        utils._BYREF_INT, # Input, n2.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n2].
        utils._BYREF_INT, # Input, ntype.
        utils._BYREF_FLOAT64, # Output, d.
        utils._BYREF_FLOAT64, # Output, z.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineFloat64ArrayType.c_type, # Output, sx[n1].
        utils._EngineFloat64ArrayType.c_type, # Output, sy[n2].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n1_py = utils._EngineIntScalarType.to_py(_n1)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n1_py,),
    ).to_ctypes()
    _n2 = utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _n2_py = utils._EngineIntScalarType.to_py(_n2)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n2_py,),
    ).to_ctypes()
    _ntype = utils._EngineIntScalarType(
        ntype, {'fun_name': fun_name, 'entity': 'ntype'},
    ).to_ctypes()
    _d = utils._EngineFloat64ScalarType.empty_ctypes()
    _z = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _sx_cls = utils._EngineFloat64ArrayType(
        sx, {'fun_name': fun_name, 'entity': 'sx'},
        exp_shape=(_n1_py,),
        check_contents=False,
    )
    _sx = _sx_cls.to_ctypes(can_reference_contents=False)
    _sy_cls = utils._EngineFloat64ArrayType(
        sy, {'fun_name': fun_name, 'entity': 'sy'},
        exp_shape=(_n2_py,),
        check_contents=False,
    )
    _sy = _sy_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n1,
        _x,
        _n2,
        _y,
        _ntype,
        _d,
        _z,
        _p,
        _sx,
        _sy,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 89999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n2 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ntype = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ntype = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iterative process used in the approximation of the probability for',
                    'large n_1 and n_2 did not converge. For the two sided test p = 1 is',
                    'returned. For the one-sided test p = 0.5 is returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _sx_cls.refresh_py(_sx)
        _sy_cls.refresh_py(_sy)
        _d_py = utils._EngineFloat64ScalarType.to_py(_d)
        _z_py = utils._EngineFloat64ScalarType.to_py(_z)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['d', 'z', 'p'],
        )(
            _d_py,
            _z_py,
            _p_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def test_chisq(nclass, ifreq, cb, dist, par, npest, prob, efreq, chisqi):
    u"""
    Performs the chi^2 goodness-of-fit test, for standard continuous
    distributions.

    For full information please refer to the NAG Library document for
    g08cg

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08cgf.html

    Parameters
    ----------
    nclass : int
        k, the number of classes into which the data is divided.

    ifreq : int, shape (`nclass`)
        `ifreq`[i] must specify the frequency of the ith class, O_i, for
        i = 1,2,...,k.

    cb : float, shape (`nclass`-1)
        `cb`[i-1] must specify the upper boundary value for the ith
        class, for i = 1,2,...,k-1.

    dist : str, length 1
        Indicates for which distribution the test is to be carried out.

        `dist` = 'N'

          The Normal distribution is used.

        `dist` = 'U'

          The uniform distribution is used.

        `dist` = 'E'

          The exponential distribution is used.

        `dist` = 'C'

          The chi^2-distribution is used.

        `dist` = 'G'

          The gamma distribution is used.

        `dist` = 'A'

          You must supply the class probabilities in the array `prob`.

    par : float, shape (2)
        Must contain the parameters of the distribution which is being
        tested. If you supply the probabilities (i.e., `dist` = 'A') the
        array `par` is not referenced.

        If a Normal distribution is used then `par`[0] and `par`[1] must
        contain the mean, mu, and the variance, sigma^2, respectively.

        If a uniform distribution is used then `par`[0] and `par`[1]
        must contain the boundaries a and b respectively.

        If an exponential distribution is used then `par`[0] must
        contain the parameter lambda. `par`[1] is not used.

        If a chi^2-distribution is used then `par`[0] must contain the
        number of degrees of freedom. `par`[1] is not used.

        If a gamma distribution is used `par`[0] and `par`[1] must
        contain the parameters alpha and beta respectively.

    npest : int
        The number of estimated parameters of the distribution.

    prob : float, shape (`nclass`)
        If you are supplying the probability distribution (i.e., `dist`
        = 'A') then `prob`[i] must contain the probability that X lies
        in the ith class.

        If `dist` != 'A', `prob` is not referenced.

    efreq : float, shape (`nclass`), modified in place
        On exit: `efreq`[i-1] contains the expected frequency for the
        ith class, E_i, for i = 1,2,...,k.

    chisqi : float, shape (`nclass`), modified in place
        On exit: `chisqi`[i-1] contains the contribution from the ith
        class to the test statistic, that is, (O_i-E_i)^2/E_i, for i =
        1,2,...,k.

    Returns
    -------
    chisq : float
        The test statistic, X^2, for the chi^2 goodness-of-fit test.

    p : float
        The upper tail probability from the chi^2-distribution
        associated with the test statistic, X^2, and the number of
        degrees of freedom.

    ndf : int
        Contains (`nclass`-1-`npest`), the degrees of freedom associated
        with the test.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nclass` = *<value>*.

            Constraint: `nclass` >= 2.

        (`errno` 2)
            On entry, `dist` = *<value>*.

            Constraint: `dist` = 'N', 'U', 'E', 'C', 'G' or 'A'.

        (`errno` 3)
            On entry, `npest` = *<value>*.

            Constraint: 0 <= `npest` < `nclass`-1.

        (`errno` 4)
            On entry, i = *<value>* and `ifreq`[i-1] = *<value>*.

            Constraint: `ifreq`[i-1] >= 0.

        (`errno` 5)
            On entry, i = *<value>*, `cb`[i-2] = *<value>* and `cb`[i-1]
            = *<value>*.

            Constraint: `cb`[i-2] < `cb`[i-1].

        (`errno` 6)
            On entry, `cb`[0] = *<value>*.

            Constraint: `cb`[0] >= 0.0.

        (`errno` 7)
            On entry, `par`[0] = *<value>* and `par`[1] = *<value>*.

            Constraint: for the gamma distribution, `par`[0] > 0.0 and
            `par`[1] > 0.0.

        (`errno` 7)
            On entry, `par`[0] = *<value>*.

            Constraint: for the chi^2 distribution, `par`[0] > 0.0.

        (`errno` 7)
            On entry, `par`[0] = *<value>*.

            Constraint: for the exponential distribution, `par`[0] >
            0.0.

        (`errno` 7)
            On entry, `par`[0] = *<value>* and `par`[1] = *<value>*.

            Constraint: for the uniform distribution, `par`[0] <
            `par`[1], `par`[0] <= `cb`[0] and `par`[1] >=
            `cb`[`nclass`-2].

        (`errno` 7)
            On entry, `par`[1] = *<value>*.

            Constraint: for the Normal distribution, `par`[1] > 0.0.

        (`errno` 8)
            On entry, sum_i(`prob`[i]) = *<value>*.

            Constraint: sum_i(`prob`[i]) = 1.0.

        (`errno` 8)
            On entry, i = *<value>* and `prob`[i-1] = *<value>*.

            Constraint: `prob` > 0.0

        (`errno` 9)
            An expected frequency equals zero, when the observed
            frequency was not.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 10)
            At least one class has an expected frequency less than 1.

        (`errno` 11)
            The solution has failed to converge whilst computing the
            expected values. The returned solution may be an adequate
            approximation.

    Notes
    -----
    In the NAG Library the traditional C interface for this routine
    uses a different algorithmic base.
    Please contact NAG if you have any questions about compatibility.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.test_chisq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08cgft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nclass.
        utils._EngineIntArrayType.c_type, # Input, ifreq[nclass].
        utils._EngineFloat64ArrayType.c_type, # Input, cb[nclass-1].
        utils._BYREF_STRING, # Input, dist(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input, par[2].
        utils._BYREF_INT, # Input, npest.
        utils._EngineFloat64ArrayType.c_type, # Input, prob[nclass].
        utils._BYREF_FLOAT64, # Output, chisq.
        utils._BYREF_FLOAT64, # Output, p.
        utils._BYREF_INT, # Output, ndf.
        utils._EngineFloat64ArrayType.c_type, # Output, efreq[nclass].
        utils._EngineFloat64ArrayType.c_type, # Output, chisqi[nclass].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of dist.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _nclass = utils._EngineIntScalarType(
        nclass, {'fun_name': fun_name, 'entity': 'nclass'},
    ).to_ctypes()
    _nclass_py = utils._EngineIntScalarType.to_py(_nclass)
    _ifreq = utils._EngineIntArrayType(
        ifreq, {'fun_name': fun_name, 'entity': 'ifreq'},
        exp_shape=(_nclass_py,),
    ).to_ctypes()
    _cb = utils._EngineFloat64ArrayType(
        cb, {'fun_name': fun_name, 'entity': 'cb'},
        exp_shape=(_nclass_py-1,),
    ).to_ctypes()
    _dist = utils._EngineCharScalarType(
        dist, {'fun_name': fun_name, 'entity': 'dist'}, exp_chrlen=1,
    ).to_ctypes()
    _par = utils._EngineFloat64ArrayType(
        par, {'fun_name': fun_name, 'entity': 'par'},
        exp_shape=(2,),
    ).to_ctypes()
    _npest = utils._EngineIntScalarType(
        npest, {'fun_name': fun_name, 'entity': 'npest'},
    ).to_ctypes()
    _prob = utils._EngineFloat64ArrayType(
        prob, {'fun_name': fun_name, 'entity': 'prob'},
        exp_shape=(_nclass_py,),
    ).to_ctypes()
    _chisq = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _ndf = utils._EngineIntScalarType.empty_ctypes()
    _efreq_cls = utils._EngineFloat64ArrayType(
        efreq, {'fun_name': fun_name, 'entity': 'efreq'},
        exp_shape=(_nclass_py,),
        check_contents=False,
    )
    _efreq = _efreq_cls.to_ctypes(can_reference_contents=False)
    _chisqi_cls = utils._EngineFloat64ArrayType(
        chisqi, {'fun_name': fun_name, 'entity': 'chisqi'},
        exp_shape=(_nclass_py,),
        check_contents=False,
    )
    _chisqi = _chisqi_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nclass,
        _ifreq,
        _cb,
        _dist,
        _par,
        _npest,
        _prob,
        _chisq,
        _p,
        _ndf,
        _efreq,
        _chisqi,
        _errbuf,
        _ierr,
        _dist._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nclass = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nclass >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, dist = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: dist = \'N\', \'U\', \'E\', \'C\', \'G\' or \'A\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, npest = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0 <= npest < nclass-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and ifreq[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: ifreq[i-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s}, cb[i-2] = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'cb[i-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: cb[i-2] < cb[i-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cb[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cb[0] >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, par[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for the gamma distribution, par[0] > 0.0 and par[1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, par[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: for the chi^2 distribution, par[0] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, par[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: for the exponential distribution, par[0] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, par[0] = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: for the uniform distribution, par[0] < par[1],',
                    'par[0] <= cb[0] and par[1] >= cb[nclass-2].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, par[1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: for the Normal distribution, par[1] > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sum_i(prob[i]) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: sum_i(prob[i]) = 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, i = {:s} and prob[i-1] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: prob > 0.0',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An expected frequency equals zero, when the observed frequency was not.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99984:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'At least one class has an expected frequency less than 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99986:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solution has failed to converge whilst computing the expected values.',
                    'The returned solution may be an adequate approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _efreq_cls.refresh_py(_efreq)
        _chisqi_cls.refresh_py(_chisqi)
        _chisq_py = utils._EngineFloat64ScalarType.to_py(_chisq)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        _ndf_py = utils._EngineIntScalarType.to_py(_ndf)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['chisq', 'p', 'ndf'],
        )(
            _chisq_py,
            _p_py,
            _ndf_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def gofstat_anddar(n, issort, y):
    u"""
    Calculates the Anderson--Darling goodness-of-fit test statistic.

    For full information please refer to the NAG Library document for
    g08ch

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08chf.html

    Parameters
    ----------
    n : int
        n, the number of observations.

    issort : bool
        Set `issort` = True if the observations are sorted in ascending
        order; otherwise the function will sort the observations.

    y : float, shape (`n`), modified in place
        On entry: y_i, for i = 1,2,...,n, the n observations.

        On exit: if `issort` = False, the data sorted in ascending
        order; otherwise the array is unchanged.

    Returns
    -------
    gofs : float
        The Anderson--Darling goodness-of-fit test statistic.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 1.

        (`errno` 3)
            `issort` = True and the data in `y` is not sorted in
            ascending order.

        (`errno` 9)
            The data in `y` must lie in the interval (0,1).
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.gofstat_anddar'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08chft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_LOGICAL, # Input, issort.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
        utils._BYREF_FLOAT64, # Output, gofs.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _issort = utils._EngineLogicalScalarType(
        issort, {'fun_name': fun_name, 'entity': 'issort'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    )
    _y = _y_cls.to_ctypes()
    _gofs = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _issort,
        _y,
        _gofs,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'issort = True and the data in y is not sorted in ascending order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The data in y must lie in the interval (0,1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _y_cls.refresh_py(_y)
        _gofs_py = utils._EngineFloat64ScalarType.to_py(_gofs)
        return_data = _gofs_py
    _err_class.manage()
    return return_data

def gofstat_anddar_unif(n, issort, y):
    u"""
    Calculates the Anderson--Darling goodness-of-fit test statistic and
    its probability for the case of uniformly distributed data.

    For full information please refer to the NAG Library document for
    g08cj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08cjf.html

    Parameters
    ----------
    n : int
        n, the number of observations.

    issort : bool
        Set `issort` = True if the observations are sorted in ascending
        order; otherwise the function will sort the observations.

    y : float, shape (`n`), modified in place
        On entry: y_i, for i = 1,2,...,n, the n observations.

        On exit: if `issort` = False, the data sorted in ascending
        order; otherwise the array is unchanged.

    Returns
    -------
    a2 : float
        A^2, the Anderson--Darling test statistic.

    p : float
        p, the upper tail probability for A^2.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 1.

        (`errno` 3)
            `issort` = True and the data in `y` is not sorted in
            ascending order.

        (`errno` 9)
            The data in `y` must lie in the interval (0,1).
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.gofstat_anddar_unif'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08cjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_LOGICAL, # Input, issort.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, y[n].
        utils._BYREF_FLOAT64, # Output, a2.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _issort = utils._EngineLogicalScalarType(
        issort, {'fun_name': fun_name, 'entity': 'issort'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y_cls = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    )
    _y = _y_cls.to_ctypes()
    _a2 = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _issort,
        _y,
        _a2,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'issort = True and the data in y is not sorted in ascending order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The data in y must lie in the interval (0,1).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _y_cls.refresh_py(_y)
        _a2_py = utils._EngineFloat64ScalarType.to_py(_a2)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['a2', 'p'],
        )(
            _a2_py,
            _p_py,
        )
    _err_class.manage()
    return return_data

def gofstat_anddar_normal(n, issort, y):
    u"""
    Calculates the Anderson--Darling goodness-of-fit test statistic and
    its probability for the case of a fully-unspecified Normal
    distribution.

    For full information please refer to the NAG Library document for
    g08ck

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08ckf.html

    Parameters
    ----------
    n : int
        n, the number of observations.

    issort : bool
        Set `issort` = True if the observations are sorted in ascending
        order; otherwise the function will sort the observations.

    y : float, shape (`n`)
        y_i, for i = 1,2,...,n, the n observations.

    Returns
    -------
    ybar : float
        The maximum likelihood estimate of mean.

    yvar : float
        The maximum likelihood estimate of variance.

    a2 : float
        A^2, the Anderson--Darling test statistic.

    aa2 : float
        The adjusted A^2.

    p : float
        p, the upper tail probability for the adjusted A^2.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 1.

        (`errno` 3)
            `issort` = True and the data in `y` is not sorted in
            ascending order.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.gofstat_anddar_normal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08ckft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_LOGICAL, # Input, issort.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_FLOAT64, # Output, ybar.
        utils._BYREF_FLOAT64, # Output, yvar.
        utils._BYREF_FLOAT64, # Output, a2.
        utils._BYREF_FLOAT64, # Output, aa2.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _issort = utils._EngineLogicalScalarType(
        issort, {'fun_name': fun_name, 'entity': 'issort'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ybar = utils._EngineFloat64ScalarType.empty_ctypes()
    _yvar = utils._EngineFloat64ScalarType.empty_ctypes()
    _a2 = utils._EngineFloat64ScalarType.empty_ctypes()
    _aa2 = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _issort,
        _y,
        _ybar,
        _yvar,
        _a2,
        _aa2,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'issort = True and the data in y is not sorted in ascending order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ybar_py = utils._EngineFloat64ScalarType.to_py(_ybar)
        _yvar_py = utils._EngineFloat64ScalarType.to_py(_yvar)
        _a2_py = utils._EngineFloat64ScalarType.to_py(_a2)
        _aa2_py = utils._EngineFloat64ScalarType.to_py(_aa2)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ybar', 'yvar', 'a2', 'aa2', 'p'],
        )(
            _ybar_py,
            _yvar_py,
            _a2_py,
            _aa2_py,
            _p_py,
        )
    _err_class.manage()
    return return_data

def gofstat_anddar_exp(n, issort, y):
    u"""
    Calculates the Anderson--Darling goodness-of-fit test statistic and
    its probability for the case of an unspecified exponential
    distribution.

    For full information please refer to the NAG Library document for
    g08cl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08clf.html

    Parameters
    ----------
    n : int
        n, the number of observations.

    issort : bool
        Set `issort` = True if the observations are sorted in ascending
        order; otherwise the function will sort the observations.

    y : float, shape (`n`)
        y_i, for i = 1,2,...,n, the n observations.

    Returns
    -------
    ybar : float
        The maximum likelihood estimate of mean.

    a2 : float
        A^2, the Anderson--Darling test statistic.

    aa2 : float
        The adjusted A^2.

    p : float
        p, the upper tail probability for the adjusted A^2.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 1.

        (`errno` 3)
            `issort` = True and the data in `y` is not sorted in
            ascending order.

        (`errno` 9)
            The data in `y` must be greater than zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.gofstat_anddar_exp'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08clft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_LOGICAL, # Input, issort.
        utils._EngineFloat64ArrayType.c_type, # Input, y[n].
        utils._BYREF_FLOAT64, # Output, ybar.
        utils._BYREF_FLOAT64, # Output, a2.
        utils._BYREF_FLOAT64, # Output, aa2.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _issort = utils._EngineLogicalScalarType(
        issort, {'fun_name': fun_name, 'entity': 'issort'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ybar = utils._EngineFloat64ScalarType.empty_ctypes()
    _a2 = utils._EngineFloat64ScalarType.empty_ctypes()
    _aa2 = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _issort,
        _y,
        _ybar,
        _a2,
        _aa2,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'issort = True and the data in y is not sorted in ascending order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The data in y must be greater than zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ybar_py = utils._EngineFloat64ScalarType.to_py(_ybar)
        _a2_py = utils._EngineFloat64ScalarType.to_py(_a2)
        _aa2_py = utils._EngineFloat64ScalarType.to_py(_aa2)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ybar', 'a2', 'aa2', 'p'],
        )(
            _ybar_py,
            _a2_py,
            _aa2_py,
            _p_py,
        )
    _err_class.manage()
    return return_data

def concordance_kendall(x, k, n):
    u"""
    Kendall's coefficient of concordance.

    For full information please refer to the NAG Library document for
    g08da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08daf.html

    Parameters
    ----------
    x : float, shape (`k`, `n`)
        `x`[i-1,j-1] must be set to the value x_{ij} of object j in
        comparison i, for j = 1,2,...,n, for i = 1,2,...,k.

    k : int
        k, the number of comparisons.

    n : int
        n, the number of objects.

    Returns
    -------
    w : float
        The value of Kendall's coefficient of concordance, W.

    p : float
        The approximate significance, p, of W.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 3)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 2.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    from ..base import _nag_enum_consts

    excs = []
    fun_name = 'naginterfaces.base.nonpar.concordance_kendall'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08daft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, sorder.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, sdx].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_INT, # Input, sdx.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Workspace, rnk.
        utils._BYREF_INT, # Input, ldrnk.
        utils._BYREF_INT, # Input, sdrnk.
        utils._BYREF_FLOAT64, # Output, w.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _sorder = utils._EngineIntScalarType.c_type(_nag_enum_consts.nag_col_major)
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _sdx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='s')
    _sdx = utils._EngineIntScalarType(
        _sdx_py, {'fun_name': fun_name, 'entity': '_sdx'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_k_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _rnk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _ldrnk = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldrnk'},
    ).to_ctypes()
    _sdrnk = utils._EngineIntScalarType(
        _sdx_py, {'fun_name': fun_name, 'entity': '_sdrnk'},
    ).to_ctypes()
    _w = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _sorder,
        _x,
        _ldx,
        _sdx,
        _k,
        _n,
        _rnk,
        _ldrnk,
        _sdrnk,
        _w,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _w_py = utils._EngineFloat64ScalarType.to_py(_w)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['w', 'p'],
        )(
            _w_py,
            _p_py,
        )
    _err_class.manage()
    return return_data

def randtest_runs(cl, n, x, m, maxr, nruns, ncount, ex, cov, comm):
    u"""
    Performs the runs up or runs down test for randomness.

    For full information please refer to the NAG Library document for
    g08ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08eaf.html

    Parameters
    ----------
    cl : str, length 1
        Must specify the type of call to ``randtest_runs``.

        `cl` = 'S'

          This is the one and only call to ``randtest_runs`` (single
          call mode). All data are to be input at once. All test
          statistics are computed after the counting of runs is
          complete.

        `cl` = 'F'

          This is the first call to the function. All initializations
          are carried out and the counting of runs begins. The final
          test statistics are not computed since further calls will be
          made to ``randtest_runs``.

        `cl` = 'I'

          This is an intermediate call during which the counts of runs
          are updated. The final test statistics are not computed since
          further calls will be made to ``randtest_runs``.

        `cl` = 'L'

          This is the last call to ``randtest_runs``. The test
          statistics are computed after the final counting of runs is
          completed.

    n : int
        n, the length of the current sequence of observations.

    x : float, shape (`n`)
        The sequence of observations.

    m : int
        The maximum number of runs to be sought. If `m` <= 0 then no
        limit is placed on the number of runs that are found.

        `m` must not be changed between calls to ``randtest_runs``.

    maxr : int
        r, the length of the longest run for which tabulation is
        desired. That is, all runs with length greater than or equal to
        r are counted together.

        `maxr` must not be changed between calls to ``randtest_runs``.

    nruns : int
        If `cl` = 'S' or 'F', `nruns` need not be set.

        If `cl` = 'I' or 'L', `nruns` must contain the value returned by
        the previous call to ``randtest_runs``.

    ncount : int, shape (`maxr`), modified in place
        On entry: if `cl` = 'S' or 'F', `ncount` need not be set.

        If `cl` = 'I' or 'L', `ncount` must contain the values returned
        by the previous call to ``randtest_runs``.

        On exit: the counts of runs of the different lengths, c_i, for i
        = 1,2,...,r.

    ex : float, shape (`maxr`), modified in place
        On exit: if `cl` = 'S' or 'L', then `ex` contains the expected
        values of the counts, e_i, for i = 1,2,...,r.

        Otherwise the elements of `ex` are not set.

    cov : float, shape (`maxr`, `maxr`), modified in place
        On exit: if `cl` = 'S' or 'L' then `cov` contains the covariance
        matrix of the counts, Sigma_c.

        Otherwise the elements of `cov` are not set.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    nruns : int
        The number of runs actually found.

    chi : float
        If `cl` = 'S' or 'L', `chi` contains the approximate chi^2 test
        statistic, X^2.

        Otherwise `chi` is not set.

    df : float
        If `cl` = 'S' or 'L' then `df` contains the degrees of freedom
        of the chi^2 statistic.

        Otherwise `df` is not set.

    prob : float
        If `cl` = 'S' or 'L', then `prob` contains the upper tail
        probability corresponding to the chi^2 test statistic, i.e., the
        significance level.

        Otherwise `prob` is not set.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `cl` = *<value>*.

            Constraint: `cl` = 'S', 'F', 'I' or 'L'.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: if `cl` = 'S', `n` >= 3, otherwise `n` >= 1.

        (`errno` 3)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: if `cl` = 'S', `m` <= `n`.

        (`errno` 4)
            On entry, `maxr` = *<value>* and `n` = *<value>*.

            Constraint: if `cl` = 'S', `maxr` < `n`.

        (`errno` 4)
            On entry, `maxr` = *<value>*.

            Constraint: `maxr` >= 1.

        (`errno` 7)
            There is a tie in the sequence of observations.

        (`errno` 8)
            The total length of the runs found is less than `maxr`.

            `maxr` = *<value>* whereas the total length of all runs is
            *<value>*.

        (`errno` 9)
            The covariance matrix stored in `cov` is not positive
            definite, thus the approximate chi^2 test statistic cannot
            be computed.

            This may be because `maxr` is too large relative to the
            length of the full sequence.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 10)
            The number of runs requested were not found, only *<value>*
            out of the requested *<value>* where found.

            All statistics are returned and may still be of use.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.randtest_runs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08eaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, cl(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, maxr.
        utils._BYREF_INT, # Input/Output, nruns.
        utils._EngineIntArrayType.c_type, # Input/Output, ncount[maxr].
        utils._EngineFloat64ArrayType.c_type, # Output, ex[maxr].
        utils._EngineFloat64ArrayType.c_type, # Output, cov[ldcov, maxr].
        utils._BYREF_INT, # Input, ldcov.
        utils._BYREF_FLOAT64, # Output, chi.
        utils._BYREF_FLOAT64, # Output, df.
        utils._BYREF_FLOAT64, # Output, prob.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrk.
        utils._BYREF_INT, # Input, lwrk.
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[10].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[5].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((cov, 'cov'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _cl = utils._EngineCharScalarType(
        cl, {'fun_name': fun_name, 'entity': 'cl'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _maxr = utils._EngineIntScalarType(
        maxr, {'fun_name': fun_name, 'entity': 'maxr'},
    ).to_ctypes()
    _nruns = utils._EngineIntScalarType(
        nruns, {'fun_name': fun_name, 'entity': 'nruns'},
    ).to_ctypes()
    _maxr_py = utils._EngineIntScalarType.to_py(_maxr)
    _ncount_cls = utils._EngineIntArrayType(
        ncount, {'fun_name': fun_name, 'entity': 'ncount'},
        exp_shape=(_maxr_py,),
    )
    _ncount = _ncount_cls.to_ctypes()
    _ex_cls = utils._EngineFloat64ArrayType(
        ex, {'fun_name': fun_name, 'entity': 'ex'},
        exp_shape=(_maxr_py,),
        check_contents=False,
    )
    _ex = _ex_cls.to_ctypes(can_reference_contents=False)
    _ldcov_py = utils._EngineFloat64ArrayType.get_nd(cov, {'fun_name': fun_name, 'entity': 'cov'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldcov = utils._EngineIntScalarType(
        _ldcov_py, {'fun_name': fun_name, 'entity': '_ldcov'},
    ).to_ctypes()
    _cov_cls = utils._EngineFloat64ArrayType(
        cov, {'fun_name': fun_name, 'entity': 'cov'},
        exp_shape=(_maxr_py, _maxr_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _cov = _cov_cls.to_ctypes(can_reference_contents=False)
    _chi = utils._EngineFloat64ScalarType.empty_ctypes()
    _df = utils._EngineFloat64ScalarType.empty_ctypes()
    _prob = utils._EngineFloat64ScalarType.empty_ctypes()
    _wrk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwrk = utils._EngineIntScalarType.c_type(0)
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav', 'rwsav'),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(10)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(5)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _cl,
        _n,
        _x,
        _m,
        _maxr,
        _nruns,
        _ncount,
        _ex,
        _cov,
        _ldcov,
        _chi,
        _df,
        _prob,
        _wrk,
        _lwrk,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _cl._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cl = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cl = \'S\', \'F\', \'I\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if cl = \'S\', n >= 3, otherwise n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if cl = \'S\', m <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxr = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if cl = \'S\', maxr < n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxr = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxr >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'There is a tie in the sequence of observations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The total length of the runs found is less than maxr.',
                    'maxr = {:s} whereas'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'the total length of all runs is {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The covariance matrix stored in cov is not positive definite,',
                    'thus the approximate chi^2 test statistic cannot be computed.',
                    'This may be because maxr is too large relative to',
                    'the length of the full sequence.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of runs requested were not found, only {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'out of the requested {:s} where found.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'All statistics are returned and may still be of use.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _nruns_py = utils._EngineIntScalarType.to_py(_nruns)
        _ncount_cls.refresh_py(_ncount)
        _ex_cls.refresh_py(_ex)
        _cov_cls.refresh_py(_cov)
        _chi_py = utils._EngineFloat64ScalarType.to_py(_chi)
        _df_py = utils._EngineFloat64ScalarType.to_py(_df)
        _prob_py = utils._EngineFloat64ScalarType.to_py(_prob)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nruns', 'chi', 'df', 'prob'],
        )(
            _nruns_py,
            _chi_py,
            _df_py,
            _prob_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def randtest_pairs(cl, n, x, msize, lag, ncount, comm):
    u"""
    Performs the pairs (serial) test for randomness.

    For full information please refer to the NAG Library document for
    g08eb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08ebf.html

    Parameters
    ----------
    cl : str, length 1
        Indicates the type of call to ``randtest_pairs``.

        `cl` = 'S'

          This is the one and only call to ``randtest_pairs`` (single
          call mode). All data are to be input at once. All test
          statistics are computed after the counting of pairs is
          complete.

        `cl` = 'F'

          This is the first call to the function. All initializations
          are carried out and the counting of pairs begins. The final
          test statistics are not computed since further calls will be
          made to ``randtest_pairs``.

        `cl` = 'I'

          This is an intermediate call during which the counts of pairs
          are updated. The final test statistics are not computed since
          further calls will be made to ``randtest_pairs``.

        `cl` = 'L'

          This is the last call to ``randtest_pairs``. The test
          statistics are computed after the final counting of runs is
          complete.

    n : int
        n, the number of observations.

    x : float, shape (`n`)
        The sequence of observations.

    msize : int
        m, the size of the matrix of counts.

        `msize` must not be changed between calls to ``randtest_pairs``.

    lag : int
        l, the lag to be used in choosing pairs.

        If `lag` = 1, then we consider the pairs (`x`[i-1],`x`[i]), for
        i = 1,3,...,n-1, where n is the number of observations.

        If `lag` > 1, then we consider the pairs (`x`[i-1],`x`[i+l-1]),
        for i = 1,2,...,l,2l+1,2l+2,...,3l,4l+1,...,n-l, where n is the
        number of observations.

        `lag` must not be changed between calls to ``randtest_pairs``.

    ncount : int, shape (`msize`, `msize`), modified in place
        On entry: if `cl` = 'S' or 'F', `ncount` need not be set.

        If `cl` = 'I' or 'L', `ncount` must contain the values returned
        by the previous call to ``randtest_pairs``.

        On exit: is an `msize` by `msize` matrix containing the counts
        of the number of pairs in each cell, c_{ij}, for j = 1,2,...,m,
        for i = 1,2,...,m.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    ex : float
        If `cl` = 'S' or 'L' then `ex` contains the expected number of
        counts in each cell, e.

        Otherwise `ex` is not set.

    chi : float
        If `cl` = 'S' or 'L' then `chi` contains the chi^2 test
        statistic, X^2, for testing the null hypothesis of randomness.

        Otherwise `chi` is not set.

    df : float
        If `cl` = 'S' or 'L' then `df` contains the degrees of freedom
        for the chi^2 statistic.

        Otherwise `df` is not set.

    prob : float
        If `cl` = 'S' or 'L' then `prob` contains the upper tail
        probability associated with the chi^2 test statistic, i.e., the
        significance level.

        Otherwise `prob` is not set.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `cl` = *<value>*.

            Constraint: `cl` = 'S', 'F', 'I' or 'L'.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: if `cl` = 'S', `n` >= 2, otherwise `n` >= 1.

        (`errno` 3)
            On entry, `msize` = *<value>*.

            Constraint: `msize` >= 2

        (`errno` 4)
            On entry, `lag` = *<value>* and `n` = *<value>*.

            Constraint: `lag` > 0 and if `cl` = 'S', `lag` < `n`.

        (`errno` 6)
            On entry, at least one element of `x` is out of range.

            Constraint: 0 <= `x`[i-1] <= 1, for i = 1,2,...,`n`.

        (`errno` 7)
            No pairs were found. This will occur if the value of `lag`
            is greater than or equal to the total number of
            observations.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 8)
            `msize` is too large relative to the number of pairs,
            therefore the expected value for at least one cell is less
            than or equal to 5.0.

            This implies that the chi^2 distribution may not be a very
            good approximation to the distribution of test statistic.

            `msize` = *<value>*, number of pairs = *<value>* and
            expected value = *<value>*.

            All statistics are returned and may still be of use.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.randtest_pairs'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08ebft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, cl(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, msize.
        utils._BYREF_INT, # Input, lag.
        utils._EngineIntArrayType.c_type, # Input/Output, ncount[ldc, msize].
        utils._BYREF_INT, # Input, ldc.
        utils._BYREF_FLOAT64, # Output, ex.
        utils._BYREF_FLOAT64, # Output, chi.
        utils._BYREF_FLOAT64, # Output, df.
        utils._BYREF_FLOAT64, # Output, prob.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wrk[2*lag].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[5].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ncount, 'ncount'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _cl = utils._EngineCharScalarType(
        cl, {'fun_name': fun_name, 'entity': 'cl'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _msize = utils._EngineIntScalarType(
        msize, {'fun_name': fun_name, 'entity': 'msize'},
    ).to_ctypes()
    _lag = utils._EngineIntScalarType(
        lag, {'fun_name': fun_name, 'entity': 'lag'},
    ).to_ctypes()
    _ldc_py = utils._EngineIntArrayType.get_nd(ncount, {'fun_name': fun_name, 'entity': 'ncount'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _msize_py = utils._EngineIntScalarType.to_py(_msize)
    _ncount_cls = utils._EngineIntArrayType(
        ncount, {'fun_name': fun_name, 'entity': 'ncount'},
        exp_shape=(_msize_py, _msize_py),
        mutual_sorder=_m_sorder,
    )
    _ncount = _ncount_cls.to_ctypes()
    _ex = utils._EngineFloat64ScalarType.empty_ctypes()
    _chi = utils._EngineFloat64ScalarType.empty_ctypes()
    _df = utils._EngineFloat64ScalarType.empty_ctypes()
    _prob = utils._EngineFloat64ScalarType.empty_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('wrk', 'iwsav'),
    )
    _lag_py = utils._EngineIntScalarType.to_py(_lag)
    if _comm_needs_init:
        _wrk = utils._EngineFloat64ArrayType.empty_ctypes(2*_lag_py)
        comm['wrk'] = _wrk
    else:
        _wrk = comm['wrk']
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(5)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _cl,
        _n,
        _x,
        _msize,
        _lag,
        _ncount,
        _ldc,
        _ex,
        _chi,
        _df,
        _prob,
        _wrk,
        _iwsav,
        _errbuf,
        _ierr,
        _cl._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cl = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cl = \'S\', \'F\', \'I\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if cl = \'S\', n >= 2, otherwise n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, msize = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: msize >= 2',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lag = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: lag > 0 and if cl = \'S\', lag < n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one element of x is out of range.',
                    'Constraint: 0 <= x[i-1] <= 1, for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one element of x is out of range.',
                    'Constraint: 0 <= x[i-1] <= 1, for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No pairs were found. This will occur if the value of lag is',
                    'greater than or equal to the total number of observations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'msize is too large relative to the number of pairs, therefore the',
                    'expected value for at least one cell is less than or equal to 5.0.',
                    'This implies that the chi^2 distribution may not be a very good',
                    'approximation to the distribution of test statistic.',
                    'msize = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'number of pairs = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'expected value = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'All statistics are returned and may still be of use.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ncount_cls.refresh_py(_ncount)
        _ex_py = utils._EngineFloat64ScalarType.to_py(_ex)
        _chi_py = utils._EngineFloat64ScalarType.to_py(_chi)
        _df_py = utils._EngineFloat64ScalarType.to_py(_df)
        _prob_py = utils._EngineFloat64ScalarType.to_py(_prob)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ex', 'chi', 'df', 'prob'],
        )(
            _ex_py,
            _chi_py,
            _df_py,
            _prob_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def randtest_triplets(cl, n, x, msize, ncount, comm):
    u"""
    Performs the triplets test for randomness.

    For full information please refer to the NAG Library document for
    g08ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08ecf.html

    Parameters
    ----------
    cl : str, length 1
        Indicates the type of call to ``randtest_triplets``.

        `cl` = 'S'

          This is the one and only call to ``randtest_triplets`` (single
          call mode). All data are to be input at once. All test
          statistics are computed after counting of the triplets is
          complete.

        `cl` = 'F'

          This is the first call to the function. All initializations
          are carried out and the counting of triplets begins. The final
          test statistics are not computed since further calls will be
          made to ``randtest_triplets``.

        `cl` = 'I'

          This is an intermediate call during which counts of the
          triplets are updated. The final test statistics are not
          computed since further calls will be made to
          ``randtest_triplets``.

        `cl` = 'L'

          This is the last call to ``randtest_triplets``. The test
          statistics are computed after the final counting of the
          triplets is complete.

    n : int
        n, the number of observations.

    x : float, shape (`n`)
        The sequence of observations.

    msize : int
        m, the size of the count matrix to be formed.

        `msize` must not be changed between calls to
        ``randtest_triplets``.

    ncount : int, shape (`msize`, `msize`, `msize`), modified in place
        On entry: if `cl` = 'S' or 'F', `ncount` need not be set.

        If `cl` = 'I' or 'L', `ncount` must contain the values returned
        by the previous call to ``randtest_triplets``.

        On exit: is an `msize` by `msize` by `msize` matrix containing
        the counts of the number of triplets, c_{jkl}, for l =
        1,2,...,m, for k = 1,2,...,m, for j = 1,2,...,m.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    ex : float
        If `cl` = 'S' or 'L' then `ex` contains the expected number of
        counts for each element of the count matrix.

        Otherwise `ex` is not set.

    chi : float
        If `cl` = 'S' or 'L' then `chi` contains the chi^2 test
        statistic, X^2, for testing the null hypothesis of randomness.

        Otherwise `chi` is not set.

    df : float
        If `cl` = 'S' or 'L' then `df` contains the degrees of freedom
        for the chi^2 statistic.

        Otherwise `df` is not set.

    prob : float
        If `cl` = 'S' or 'L' then `prob` contains the upper tail
        probability associated with the chi^2 test statistic, i.e., the
        significance level.

        Otherwise `prob` is not set.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `cl` = *<value>*.

            Constraint: `cl` = 'S', 'F', 'I' or 'L'.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: if `cl` = 'S', `n` >= 3, otherwise `n` >= 1.

        (`errno` 3)
            On entry, `msize` = *<value>*.

            Constraint: `msize` >= 2.

        (`errno` 5)
            On entry, at least one element of `x` is out of range.

            Constraint: 0 <= `x`[i-1] <= 1, for i = 1,2,...,`n`.

        (`errno` 6)
            No triplets were found because less than 3 observations were
            provided in total.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 7)
            `msize` is too large relative to the number of triplets,
            therefore the expected value for at least one cell is less
            than or equal to 5.0.

            This implies that the chi^2 distribution may not be a very
            good approximation to the distribution of the test
            statistic.

            `msize` = *<value>*, number of triplets = *<value>* and
            expected value = *<value>*.

            All statistics are returned and may still be of use.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.randtest_triplets'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08ecft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, cl(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, msize.
        utils._EngineIntArrayType.c_type, # Input/Output, ncount[ldc, ldc, msize].
        utils._BYREF_INT, # Input, ldc.
        utils._BYREF_FLOAT64, # Output, ex.
        utils._BYREF_FLOAT64, # Output, chi.
        utils._BYREF_FLOAT64, # Output, df.
        utils._BYREF_FLOAT64, # Output, prob.
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[10].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((ncount, 'ncount'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _cl = utils._EngineCharScalarType(
        cl, {'fun_name': fun_name, 'entity': 'cl'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _msize = utils._EngineIntScalarType(
        msize, {'fun_name': fun_name, 'entity': 'msize'},
    ).to_ctypes()
    _ldc_py = utils._EngineIntArrayType.get_shape(ncount, {'fun_name': fun_name, 'entity': 'ncount'}, exp_rank=3)[0]
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    _msize_py = utils._EngineIntScalarType.to_py(_msize)
    _ncount_cls = utils._EngineIntArrayType(
        ncount, {'fun_name': fun_name, 'entity': 'ncount'},
        exp_shape=(_msize_py, _msize_py, _msize_py),
        mutual_sorder=_m_sorder,
    )
    _ncount = _ncount_cls.to_ctypes()
    _ex = utils._EngineFloat64ScalarType.empty_ctypes()
    _chi = utils._EngineFloat64ScalarType.empty_ctypes()
    _df = utils._EngineFloat64ScalarType.empty_ctypes()
    _prob = utils._EngineFloat64ScalarType.empty_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav',),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(10)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _cl,
        _n,
        _x,
        _msize,
        _ncount,
        _ldc,
        _ex,
        _chi,
        _df,
        _prob,
        _iwsav,
        _errbuf,
        _ierr,
        _cl._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cl = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cl = \'S\', \'F\', \'I\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if cl = \'S\', n >= 3, otherwise n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, msize = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: msize >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one element of x is out of range.',
                    'Constraint: 0 <= x[i-1] <= 1, for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one element of x is out of range.',
                    'Constraint: 0 <= x[i-1] <= 1, for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No triplets were found because less than 3 observations',
                    'were provided in total.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'msize is too large relative to the number of triplets, therefore the',
                    'expected value for at least one cell is less than or equal to 5.0.',
                    'This implies that the chi^2 distribution may not be a very good',
                    'approximation to the distribution of the test statistic.',
                    'msize = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'number of triplets = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'expected value = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'All statistics are returned and may still be of use.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ncount_cls.refresh_py(_ncount)
        _ex_py = utils._EngineFloat64ScalarType.to_py(_ex)
        _chi_py = utils._EngineFloat64ScalarType.to_py(_chi)
        _df_py = utils._EngineFloat64ScalarType.to_py(_df)
        _prob_py = utils._EngineFloat64ScalarType.to_py(_prob)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ex', 'chi', 'df', 'prob'],
        )(
            _ex_py,
            _chi_py,
            _df_py,
            _prob_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def randtest_gaps(cl, n, x, m, maxg, rlo, rup, totlen, ngaps, ncount, ex, comm):
    u"""
    Performs the gaps test for randomness.

    For full information please refer to the NAG Library document for
    g08ed

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08edf.html

    Parameters
    ----------
    cl : str, length 1
        Indicates the type of call to ``randtest_gaps``.

        `cl` = 'S'

          This is the one and only call to ``randtest_gaps`` (single
          call mode). All data are to be input at once. All test
          statistics are computed after the counting of gaps is
          complete.

        `cl` = 'F'

          This is the first call to the function. All initializations
          are carried out before the counting of gaps begins. The final
          test statistics are not computed since further calls will be
          made to ``randtest_gaps``.

        `cl` = 'I'

          This is an intermediate call during which the counts of gaps
          are updated. The final test statistics are not computed since
          further calls will be made to ``randtest_gaps``.

        `cl` = 'L'

          This is the last call to ``randtest_gaps``. The test
          statistics are computed after the final counting of gaps is
          complete.

    n : int
        n, the length of the current sequence of observations.

    x : float, shape (`n`)
        The sequence of observations.

    m : int
        The maximum number of gaps to be sought. If `m` <= 0 then there
        is no limit placed on the number of gaps that are found.

        `m` should not be changed between calls to ``randtest_gaps``.

    maxg : int
        k, the length of the longest gap for which tabulation is
        desired.

        `maxg` must not be changed between calls to ``randtest_gaps``.

    rlo : float
        The lower limit of the interval to be used to define the gaps,
        r_l.

        `rlo` must not be changed between calls to ``randtest_gaps``.

    rup : float
        The upper limit of the interval to be used to define the gaps,
        r_u.

        `rup` must not be changed between calls to ``randtest_gaps``.

    totlen : float
        The total length of the interval which contains all possible
        numbers that may arise in the sequence.

    ngaps : int
        If `cl` = 'S' or 'F', `ngaps` need not be set.

        If `cl` = 'I' or 'L', `ngaps` must contain the value returned by
        the previous call to ``randtest_gaps``.

    ncount : int, shape (`maxg`), modified in place
        On entry: if `cl` = 'S' or 'F', `ncount` need not be set.

        If `cl` = 'I' or 'L', `ncount` must contain the values returned
        by the previous call to ``randtest_gaps``.

        On exit: the counts of gaps of the different lengths, c_i, for i
        = 1,2,...,k.

    ex : float, shape (`maxg`), modified in place
        On exit: if `cl` = 'S' or 'L' then `ex` contains the expected
        values of the counts.

        Otherwise the elements of `ex` are not set.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    ngaps : int
        The number of gaps actually found, ngaps.

    chi : float
        If `cl` = 'S' or 'L' then `chi` contains the chi^2 test
        statistic, X^2, for testing the null hypothesis of randomness.

        Otherwise `chi` is not set.

    df : float
        If `cl` = 'S' or 'L' then `df` contains the degrees of freedom
        for the chi^2 statistic.

        Otherwise `df` is not set.

    prob : float
        If `cl` = 'S' or 'L' then `prob` contains the upper tail
        probability associated with the chi^2 test statistic, i.e., the
        significance level.

        Otherwise `prob` is not set.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `cl` = *<value>*.

            Constraint: `cl` = 'S', 'F', 'I' or 'L'.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: if `cl` = 'S', `m` <= `n`.

        (`errno` 4)
            On entry, `maxg` = *<value>* and `n` = *<value>*.

            Constraint: if `cl` = 'S', `maxg` <= `n`.

        (`errno` 4)
            On entry, `maxg` = *<value>*.

            Constraint: `maxg` > 1.

        (`errno` 5)
            On entry, `rlo` = *<value>*, `rup` = *<value>* and `totlen`
            = *<value>*.

            Constraint: `rup`-`rlo` < `totlen`.

        (`errno` 5)
            On entry, `totlen` = *<value>*.

            Constraint: `totlen` > 0.0.

        (`errno` 5)
            On entry, `rlo` = *<value>* and `rup` = *<value>*.

            Constraint: `rup` > `rlo`.

        (`errno` 6)
            No gaps were found. Try using a longer sequence, or increase
            the size of the interval `rup`-`rlo`.

        (`errno` 7)
            The expected frequency in class i = *<value>* is zero. The
            value of (`rup`-`rlo`)/`totlen` may be too close to 0.0 or
            1.0. or `maxg` is too large relative to the number of gaps
            found.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 8)
            The number of gaps requested were not found, only *<value>*
            out of the requested *<value>* where found.

            All statistics are returned and may still be of use.

        (`errno` 9)
            The expected frequency of at least one class is less than 1.

            This implies that the chi^2 may not be a very good
            approximation to the distribution of the test statistics.

            All statistics are returned and may still be of use.
    """

    excs = []
    fun_name = 'naginterfaces.base.nonpar.randtest_gaps'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08edft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, cl(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, maxg.
        utils._BYREF_FLOAT64, # Input, rlo.
        utils._BYREF_FLOAT64, # Input, rup.
        utils._BYREF_FLOAT64, # Input, totlen.
        utils._BYREF_INT, # Input/Output, ngaps.
        utils._EngineIntArrayType.c_type, # Input/Output, ncount[maxg].
        utils._EngineFloat64ArrayType.c_type, # Output, ex[maxg].
        utils._BYREF_FLOAT64, # Output, chi.
        utils._BYREF_FLOAT64, # Output, df.
        utils._BYREF_FLOAT64, # Output, prob.
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[5].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of cl.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _cl = utils._EngineCharScalarType(
        cl, {'fun_name': fun_name, 'entity': 'cl'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _maxg = utils._EngineIntScalarType(
        maxg, {'fun_name': fun_name, 'entity': 'maxg'},
    ).to_ctypes()
    _rlo = utils._EngineFloat64ScalarType(
        rlo, {'fun_name': fun_name, 'entity': 'rlo'},
    ).to_ctypes()
    _rup = utils._EngineFloat64ScalarType(
        rup, {'fun_name': fun_name, 'entity': 'rup'},
    ).to_ctypes()
    _totlen = utils._EngineFloat64ScalarType(
        totlen, {'fun_name': fun_name, 'entity': 'totlen'},
    ).to_ctypes()
    _ngaps = utils._EngineIntScalarType(
        ngaps, {'fun_name': fun_name, 'entity': 'ngaps'},
    ).to_ctypes()
    _maxg_py = utils._EngineIntScalarType.to_py(_maxg)
    _ncount_cls = utils._EngineIntArrayType(
        ncount, {'fun_name': fun_name, 'entity': 'ncount'},
        exp_shape=(_maxg_py,),
    )
    _ncount = _ncount_cls.to_ctypes()
    _ex_cls = utils._EngineFloat64ArrayType(
        ex, {'fun_name': fun_name, 'entity': 'ex'},
        exp_shape=(_maxg_py,),
        check_contents=False,
    )
    _ex = _ex_cls.to_ctypes(can_reference_contents=False)
    _chi = utils._EngineFloat64ScalarType.empty_ctypes()
    _df = utils._EngineFloat64ScalarType.empty_ctypes()
    _prob = utils._EngineFloat64ScalarType.empty_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav',),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(5)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _cl,
        _n,
        _x,
        _m,
        _maxg,
        _rlo,
        _rup,
        _totlen,
        _ngaps,
        _ncount,
        _ex,
        _chi,
        _df,
        _prob,
        _iwsav,
        _errbuf,
        _ierr,
        _cl._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cl = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cl = \'S\', \'F\', \'I\' or \'L\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if cl = \'S\', m <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxg = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if cl = \'S\', maxg <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxg = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxg > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rlo = {:s}, rup = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'totlen = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: rup-rlo < totlen.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, totlen = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: totlen > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rlo = {:s} and rup = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: rup > rlo.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'No gaps were found. Try using a longer sequence, or increase the size',
                    'of the interval rup-rlo.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The expected frequency in class i = {:s} is zero.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The value of (rup-rlo)/totlen may be too close to 0.0 or 1.0.',
                    'or maxg is too large relative to the number of gaps found.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of gaps requested were not found, only {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'out of the requested {:s} where found.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'All statistics are returned and may still be of use.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The expected frequency of at least one class is less than 1.',
                    'This implies that the chi^2 may not be a very good',
                    'approximation to the distribution of the test statistics.',
                    'All statistics are returned and may still be of use.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _ngaps_py = utils._EngineIntScalarType.to_py(_ngaps)
        _ncount_cls.refresh_py(_ncount)
        _ex_cls.refresh_py(_ex)
        _chi_py = utils._EngineFloat64ScalarType.to_py(_chi)
        _df_py = utils._EngineFloat64ScalarType.to_py(_df)
        _prob_py = utils._EngineFloat64ScalarType.to_py(_prob)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ngaps', 'chi', 'df', 'prob'],
        )(
            _ngaps_py,
            _chi_py,
            _df_py,
            _prob_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def rank_regsn(ns, nv, nsum, y, ip, x, idist, nmax, tol, prvr, irank, zin, eta, vapvec, parest):
    u"""
    Regression using ranks, uncensored data.

    For full information please refer to the NAG Library document for
    g08ra

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08raf.html

    Parameters
    ----------
    ns : int
        The number of samples.

    nv : int, shape (`ns`), modified in place
        The number of observations in the ith sample, for i =
        1,2,...,`ns`.

    nsum : int
        The total number of observations.

    y : float, shape (`nsum`)
        The observations in each sample. Specifically, `y`[ sum_k =
        1^i-1`nv`[k-1]+j -1] must contain the jth observation in the ith
        sample.

    ip : int
        The number of parameters to be fitted.

    x : float, shape (`nsum`, `ip`)
        The design matrices for each sample. Specifically, `x`[ sum_k =
        1^i-1 `nv`[k-1] +j -1,l-1] must contain the value of the lth
        explanatory variable for the jth observation in the ith sample.

    idist : int
        The error distribution to be used in the analysis.

        `idist` = 1

          Normal.

        `idist` = 2

          Logistic.

        `idist` = 3

          Extreme value.

        `idist` = 4

          Double-exponential.

    nmax : int
        The value of the largest sample size.

    tol : float
        The tolerance for judging whether two observations are tied.
        Thus, observations Y_i and Y_j are adjudged to be tied if
        |Y_i-Y_j| < `tol`.

    prvr : float, shape (`ip`+1, `ip`), modified in place
        On exit: the variance-covariance matrices of the score
        statistics and the parameter estimates, the former being stored
        in the upper triangle and the latter in the lower triangle. Thus
        for 1 <= i <= j <= `ip`, `prvr`[i-1,j-1] contains an estimate of
        the covariance between the ith and jth score statistics. For 1
        <= j <= i <= `ip`-1, `prvr`[i,j-1] contains an estimate of the
        covariance between the ith and jth parameter estimates.

    irank : int, shape (`nmax`), modified in place
        On exit: for the one sample case, `irank` contains the ranks of
        the observations.

    zin : float, shape (`nmax`), modified in place
        On exit: for the one sample case, `zin` contains the expected
        values of the function g(.) of the order statistics.

    eta : float, shape (`nmax`), modified in place
        On exit: for the one sample case, `eta` contains the expected
        values of the function g'(.) of the order statistics.

    vapvec : float, shape (`nmax`*(`nmax`+1)/2), modified in place
        On exit: for the one sample case, `vapvec` contains the upper
        triangle of the variance-covariance matrix of the function g(.)
        of the order statistics stored column-wise.

    parest : float, shape (4*`ip`+1), modified in place
        On exit: the statistics calculated by the function.

        The first `ip` components of `parest` contain the score
        statistics.

        The next `ip` elements contain the parameter estimates.

        `parest`[2*`ip`] contains the value of the chi^2 statistic.

        The next `ip` elements of `parest` contain the standard errors
        of the parameter estimates.

        Finally, the remaining `ip` elements of `parest` contain the
        z-statistics.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, max_i(`nv`[i]) = *<value>* and `nmax` = *<value>*.

            Constraint: max_i(`nv`[i]) = `nmax`.

        (`errno` 1)
            On entry, sum_i(`nv`[i]) = *<value>* and `nsum` = *<value>*.

            Constraint: sum_i(`nv`[i]) = `nsum`.

        (`errno` 1)
            On entry, *<value>* elements of `nv` are < 1.

            Constraint: `nv`[i] >= 1.

        (`errno` 1)
            On entry, `nmax` = *<value>* and `ip` = *<value>*.

            Constraint: `nmax` > `ip`.

        (`errno` 1)
            On entry, `ip` = *<value>*.

            Constraint: `ip` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `ns` = *<value>*.

            Constraint: `ns` >= 1.

        (`errno` 2)
            On entry, `idist` = *<value>*.

            On entry, `idist` = 1, 2, 3 or 4.

        (`errno` 3)
            On entry, all the observations were adjudged to be tied.

        (`errno` 4)
            The matrix X^T(B-A)X is either ill-conditioned or not
            positive definite.

        (`errno` 5)
            On entry, for j = *<value>*, `x`[i,j-1] = *<value>* for all
            i.

            Constraint: `x`[i,j-1] != `x`[i+1,j-1] for at least one i.
    """

    from ..base import _nag_enum_consts

    excs = []
    fun_name = 'naginterfaces.base.nonpar.rank_regsn'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08raft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, sorder.
        utils._BYREF_INT, # Input, ns.
        utils._EngineIntArrayType.c_type, # Input/Output, nv[ns].
        utils._BYREF_INT, # Input, nsum.
        utils._EngineFloat64ArrayType.c_type, # Input, y[nsum].
        utils._BYREF_INT, # Input, ip.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._BYREF_INT, # Input, idist.
        utils._BYREF_INT, # Input, nmax.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Output, prvr[ldprvr, ip].
        utils._BYREF_INT, # Input, ldprvr.
        utils._EngineIntArrayType.c_type, # Output, irank[nmax].
        utils._EngineFloat64ArrayType.c_type, # Output, zin[nmax].
        utils._EngineFloat64ArrayType.c_type, # Output, eta[nmax].
        utils._EngineFloat64ArrayType.c_type, # Output, vapvec[nmax*(nmax+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, parest[4*ip+1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (prvr, 'prvr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _sorder = utils._EngineIntScalarType.c_type(_nag_enum_consts.nag_col_major)
    _ns = utils._EngineIntScalarType(
        ns, {'fun_name': fun_name, 'entity': 'ns'},
    ).to_ctypes()
    _ns_py = utils._EngineIntScalarType.to_py(_ns)
    _nv_cls = utils._EngineIntArrayType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
        exp_shape=(_ns_py,),
    )
    _nv = _nv_cls.to_ctypes()
    _nsum = utils._EngineIntScalarType(
        nsum, {'fun_name': fun_name, 'entity': 'nsum'},
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_nv_cls.get_sum(0, _ns_py-1),),
    ).to_ctypes()
    _ip = utils._EngineIntScalarType(
        ip, {'fun_name': fun_name, 'entity': 'ip'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _nsum_py = utils._EngineIntScalarType.to_py(_nsum)
    _sorder_py = utils._EngineIntScalarType.to_py(_sorder)
    _ip_py = utils._EngineIntScalarType.to_py(_ip)
    if _sorder_py == 1:
        _x_dim2 = _ip_py
    elif _sorder_py == 2:
        _x_dim2 = _nsum_py
    else:
        _x_dim2 = 0
    exp_shape_x = (_nsum_py, _x_dim2)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _idist = utils._EngineIntScalarType(
        idist, {'fun_name': fun_name, 'entity': 'idist'},
    ).to_ctypes()
    _nmax = utils._EngineIntScalarType(
        nmax, {'fun_name': fun_name, 'entity': 'nmax'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _ldprvr_py = utils._EngineFloat64ArrayType.get_nd(prvr, {'fun_name': fun_name, 'entity': 'prvr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldprvr = utils._EngineIntScalarType(
        _ldprvr_py, {'fun_name': fun_name, 'entity': '_ldprvr'},
    ).to_ctypes()
    _prvr_cls = utils._EngineFloat64ArrayType(
        prvr, {'fun_name': fun_name, 'entity': 'prvr'},
        exp_shape=(_ip_py+1, _ip_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _prvr = _prvr_cls.to_ctypes(can_reference_contents=False)
    _nmax_py = utils._EngineIntScalarType.to_py(_nmax)
    _irank_cls = utils._EngineIntArrayType(
        irank, {'fun_name': fun_name, 'entity': 'irank'},
        exp_shape=(_nmax_py,),
        check_contents=False,
    )
    _irank = _irank_cls.to_ctypes(can_reference_contents=False)
    _zin_cls = utils._EngineFloat64ArrayType(
        zin, {'fun_name': fun_name, 'entity': 'zin'},
        exp_shape=(_nmax_py,),
        check_contents=False,
    )
    _zin = _zin_cls.to_ctypes(can_reference_contents=False)
    _eta_cls = utils._EngineFloat64ArrayType(
        eta, {'fun_name': fun_name, 'entity': 'eta'},
        exp_shape=(_nmax_py,),
        check_contents=False,
    )
    _eta = _eta_cls.to_ctypes(can_reference_contents=False)
    _vapvec_cls = utils._EngineFloat64ArrayType(
        vapvec, {'fun_name': fun_name, 'entity': 'vapvec'},
        exp_shape=(_nmax_py*(_nmax_py+1)//2,),
        check_contents=False,
    )
    _vapvec = _vapvec_cls.to_ctypes(can_reference_contents=False)
    _parest_cls = utils._EngineFloat64ArrayType(
        parest, {'fun_name': fun_name, 'entity': 'parest'},
        exp_shape=(4*_ip_py+1,),
        check_contents=False,
    )
    _parest = _parest_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _sorder,
        _ns,
        _nv,
        _nsum,
        _y,
        _ip,
        _x,
        _ldx,
        _idist,
        _nmax,
        _tol,
        _prvr,
        _ldprvr,
        _irank,
        _zin,
        _eta,
        _vapvec,
        _parest,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, max_i(nv[i]) = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: max_i(nv[i]) = nmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sum_i(nv[i]) = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nsum = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sum_i(nv[i]) = nsum.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, {:s} elements of nv are < 1.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv[i] >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nmax = {:s} and ip = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nmax > ip.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ip = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ip >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ns = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ns >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, idist = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On entry, idist = 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all the observations were adjudged to be tied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix X^T(B-A)X is either ill-conditioned or not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, for j = {:s}, x[i,j-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'for all i.',
                    'Constraint: x[i,j-1] != x[i+1,j-1] for at least one i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _nv_cls.refresh_py(_nv)
        _prvr_cls.refresh_py(_prvr)
        _irank_cls.refresh_py(_irank)
        _zin_cls.refresh_py(_zin)
        _eta_cls.refresh_py(_eta)
        _vapvec_cls.refresh_py(_vapvec)
        _parest_cls.refresh_py(_parest)
        return_data = None
    _err_class.manage()
    return return_data

def rank_regsn_censored(ns, nv, nsum, y, ip, x, icen, gamma, nmax, tol, prvr, irank, zin, eta, vapvec, parest):
    u"""
    Regression using ranks, right-censored data.

    For full information please refer to the NAG Library document for
    g08rb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g08/g08rbf.html

    Parameters
    ----------
    ns : int
        The number of samples.

    nv : int, shape (`ns`)
        The number of observations in the ith sample, for i =
        1,2,...,`ns`.

    nsum : int
        The total number of observations.

    y : float, shape (`nsum`)
        The observations in each sample. Specifically, `y`[ sum_k =
        1^i-1`nv`[k-1]+j -1] must contain the jth observation in the ith
        sample.

    ip : int
        The number of parameters to be fitted.

    x : float, shape (`nsum`, `ip`)
        The design matrices for each sample. Specifically, `x`[ sum_k =
        1^i-1 `nv`[k-1] + j -1,l-1] must contain the value of the lth
        explanatory variable for the jth observations in the ith sample.

    icen : int, shape (`nsum`)
        Defines the censoring variable for the observations in `y`.

        `icen`[i-1] = 0

          If `y`[i-1] is uncensored.

        `icen`[i-1] = 1

          If `y`[i-1] is censored.

    gamma : float
        The value of the parameter defining the generalized logistic
        distribution. For `gamma` <= 0.0001, the limiting extreme value
        distribution is assumed.

    nmax : int
        The value of the largest sample size.

    tol : float
        The tolerance for judging whether two observations are tied.
        Thus, observations Y_i and Y_j are adjudged to be tied if
        |Y_i-Y_j| < `tol`.

    prvr : float, shape (`ip`+1, `ip`), modified in place
        On exit: the variance-covariance matrices of the score
        statistics and the parameter estimates, the former being stored
        in the upper triangle and the latter in the lower triangle. Thus
        for 1 <= i <= j <= `ip`, `prvr`[i-1,j-1] contains an estimate of
        the covariance between the ith and jth score statistics. For 1
        <= j <= i <= `ip`-1, `prvr`[i,j-1] contains an estimate of the
        covariance between the ith and jth parameter estimates.

    irank : int, shape (`nmax`), modified in place
        On exit: for the one sample case, `irank` contains the ranks of
        the observations.

    zin : float, shape (`nmax`), modified in place
        On exit: for the one sample case, `zin` contains the expected
        values of the function g(.) of the order statistics.

    eta : float, shape (`nmax`), modified in place
        On exit: for the one sample case, `eta` contains the expected
        values of the function g'(.) of the order statistics.

    vapvec : float, shape (`nmax`*(`nmax`+1)/2), modified in place
        On exit: for the one sample case, `vapvec` contains the upper
        triangle of the variance-covariance matrix of the function g(.)
        of the order statistics stored column-wise.

    parest : float, shape (4*`ip`+1), modified in place
        On exit: the statistics calculated by the function.

        The first `ip` components of `parest` contain the score
        statistics.

        The next `ip` elements contain the parameter estimates.

        `parest`[2*`ip`] contains the value of the chi^2 statistic.

        The next `ip` elements of `parest` contain the standard errors
        of the parameter estimates.

        Finally, the remaining `ip` elements of `parest` contain the
        z-statistics.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `gamma` = *<value>*.

            Constraint: `gamma` >= 0.0.

        (`errno` 1)
            On entry, `nmax` = *<value>* and `ip` = *<value>*.

            Constraint: `nmax` > `ip`.

        (`errno` 1)
            On entry, `ip` = *<value>*.

            Constraint: `ip` >= 1.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `ns` = *<value>*.

            Constraint: `ns` >= 1.

        (`errno` 1)
            On entry, max_i(`nv`[i]) = *<value>* and `nmax` = *<value>*.

            Constraint: max_i(`nv`[i]) = `nmax`.

        (`errno` 1)
            On entry, sum_i(`nv`[i]) = *<value>* and `nsum` = *<value>*.

            Constraint: sum_i(`nv`[i]) = `nsum`.

        (`errno` 1)
            On entry, *<value>* elements of `nv` are < 1.

            Constraint: `nv`[i] >= 1.

        (`errno` 2)
            On entry, *<value>* elements of `icen` are out of range.

            Constraint: `icen`[i] = 0 or 1, for all i.

        (`errno` 3)
            On entry, all the observations were adjudged to be tied.

        (`errno` 4)
            The matrix X^T(B-A)X is either ill-conditioned or not
            positive definite.

        (`errno` 5)
            On entry, for j = *<value>*, `x`[i,j-1] = *<value>* for all
            i.

            Constraint: `x`[i,j-1] != `x`[i+1,j-1] for at least one i.
    """

    from ..base import _nag_enum_consts

    excs = []
    fun_name = 'naginterfaces.base.nonpar.rank_regsn_censored'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g08rbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, sorder.
        utils._BYREF_INT, # Input, ns.
        utils._EngineIntArrayType.c_type, # Input, nv[ns].
        utils._BYREF_INT, # Input, nsum.
        utils._EngineFloat64ArrayType.c_type, # Input, y[nsum].
        utils._BYREF_INT, # Input, ip.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, *].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineIntArrayType.c_type, # Input, icen[nsum].
        utils._BYREF_FLOAT64, # Input, gamma.
        utils._BYREF_INT, # Input, nmax.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Output, prvr[ldprvr, ip].
        utils._BYREF_INT, # Input, ldprvr.
        utils._EngineIntArrayType.c_type, # Output, irank[nmax].
        utils._EngineFloat64ArrayType.c_type, # Output, zin[nmax].
        utils._EngineFloat64ArrayType.c_type, # Output, eta[nmax].
        utils._EngineFloat64ArrayType.c_type, # Output, vapvec[nmax*(nmax+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, parest[4*ip+1].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (prvr, 'prvr')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _sorder = utils._EngineIntScalarType.c_type(_nag_enum_consts.nag_col_major)
    _ns = utils._EngineIntScalarType(
        ns, {'fun_name': fun_name, 'entity': 'ns'},
    ).to_ctypes()
    _ns_py = utils._EngineIntScalarType.to_py(_ns)
    _nv_cls = utils._EngineIntArrayType(
        nv, {'fun_name': fun_name, 'entity': 'nv'},
        exp_shape=(_ns_py,),
    )
    _nv = _nv_cls.to_ctypes()
    _nsum = utils._EngineIntScalarType(
        nsum, {'fun_name': fun_name, 'entity': 'nsum'},
    ).to_ctypes()
    _y = utils._EngineFloat64ArrayType(
        y, {'fun_name': fun_name, 'entity': 'y'},
        exp_shape=(_nv_cls.get_sum(0, _ns_py-1),),
    ).to_ctypes()
    _ip = utils._EngineIntScalarType(
        ip, {'fun_name': fun_name, 'entity': 'ip'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _nsum_py = utils._EngineIntScalarType.to_py(_nsum)
    _sorder_py = utils._EngineIntScalarType.to_py(_sorder)
    _ip_py = utils._EngineIntScalarType.to_py(_ip)
    if _sorder_py == 1:
        _x_dim2 = _ip_py
    elif _sorder_py == 2:
        _x_dim2 = _nsum_py
    else:
        _x_dim2 = 0
    exp_shape_x = (_nsum_py, _x_dim2)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=exp_shape_x,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _icen = utils._EngineIntArrayType(
        icen, {'fun_name': fun_name, 'entity': 'icen'},
        exp_shape=(_nv_cls.get_sum(0, _ns_py-1),),
    ).to_ctypes()
    _gamma = utils._EngineFloat64ScalarType(
        gamma, {'fun_name': fun_name, 'entity': 'gamma'},
    ).to_ctypes()
    _nmax = utils._EngineIntScalarType(
        nmax, {'fun_name': fun_name, 'entity': 'nmax'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _ldprvr_py = utils._EngineFloat64ArrayType.get_nd(prvr, {'fun_name': fun_name, 'entity': 'prvr'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldprvr = utils._EngineIntScalarType(
        _ldprvr_py, {'fun_name': fun_name, 'entity': '_ldprvr'},
    ).to_ctypes()
    _prvr_cls = utils._EngineFloat64ArrayType(
        prvr, {'fun_name': fun_name, 'entity': 'prvr'},
        exp_shape=(_ip_py+1, _ip_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _prvr = _prvr_cls.to_ctypes(can_reference_contents=False)
    _nmax_py = utils._EngineIntScalarType.to_py(_nmax)
    _irank_cls = utils._EngineIntArrayType(
        irank, {'fun_name': fun_name, 'entity': 'irank'},
        exp_shape=(_nmax_py,),
        check_contents=False,
    )
    _irank = _irank_cls.to_ctypes(can_reference_contents=False)
    _zin_cls = utils._EngineFloat64ArrayType(
        zin, {'fun_name': fun_name, 'entity': 'zin'},
        exp_shape=(_nmax_py,),
        check_contents=False,
    )
    _zin = _zin_cls.to_ctypes(can_reference_contents=False)
    _eta_cls = utils._EngineFloat64ArrayType(
        eta, {'fun_name': fun_name, 'entity': 'eta'},
        exp_shape=(_nmax_py,),
        check_contents=False,
    )
    _eta = _eta_cls.to_ctypes(can_reference_contents=False)
    _vapvec_cls = utils._EngineFloat64ArrayType(
        vapvec, {'fun_name': fun_name, 'entity': 'vapvec'},
        exp_shape=(_nmax_py*(_nmax_py+1)//2,),
        check_contents=False,
    )
    _vapvec = _vapvec_cls.to_ctypes(can_reference_contents=False)
    _parest_cls = utils._EngineFloat64ArrayType(
        parest, {'fun_name': fun_name, 'entity': 'parest'},
        exp_shape=(4*_ip_py+1,),
        check_contents=False,
    )
    _parest = _parest_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _sorder,
        _ns,
        _nv,
        _nsum,
        _y,
        _ip,
        _x,
        _ldx,
        _icen,
        _gamma,
        _nmax,
        _tol,
        _prvr,
        _ldprvr,
        _irank,
        _zin,
        _eta,
        _vapvec,
        _parest,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, gamma = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: gamma >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nmax = {:s} and ip = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: nmax > ip.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ip = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ip >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ns = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ns >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, max_i(nv[i]) = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: max_i(nv[i]) = nmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sum_i(nv[i]) = {:s} and'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'nsum = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: sum_i(nv[i]) = nsum.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, {:s} elements of nv are < 1.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nv[i] >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, {:s} elements of icen are out of range.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: icen[i] = 0 or 1, for all i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all the observations were adjudged to be tied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix X^T(B-A)X is either ill-conditioned or not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, for j = {:s}, x[i,j-1] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'for all i.',
                    'Constraint: x[i,j-1] != x[i+1,j-1] for at least one i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _prvr_cls.refresh_py(_prvr)
        _irank_cls.refresh_py(_irank)
        _zin_cls.refresh_py(_zin)
        _eta_cls.refresh_py(_eta)
        _vapvec_cls.refresh_py(_vapvec)
        _parest_cls.refresh_py(_parest)
        return_data = None
    _err_class.manage()
    return return_data
