# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
# pylint: disable=simplifiable-if-statement
u"""
Interfaces for the NAG Mark 27.0 `roots` Chapter.

``roots`` - Roots of One or More Transcendental Equations

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module is concerned with the calculation of zeros of continuous
functions of one or more variables.
The majority of problems considered are for real-valued functions of
real variables, in which case complex equations must be expressed in
terms of the equivalent larger system of real equations.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def contfn_brent_interval(x, h, eps, eta, f, data=None):
    u"""
    Zero of continuous function, Brent algorithm, from a given starting
    value, binary search for interval.

    For full information please refer to the NAG Library document for
    c05au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05auf.html

    Parameters
    ----------
    x : float
        An initial approximation to the zero.

    h : float
        A step length for use in the binary search for an interval
        containing the zero. The maximum interval searched is
        [`x`-256.0*`h`,`x`+256.0*`h`].

    eps : float
        The termination tolerance on x (see [Description]).

    eta : float
        A value such that if |f(x)| <= `eta`, x is accepted as the zero.
        `eta` may be specified as 0.0 (see [Accuracy]).

    f : callable retval = f(x, data=None)
        `f` must evaluate the function f whose zero is to be determined.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f at the specified point.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float
        If the function exits successfully or `errno` = 4, this argument
        is the final approximation to the zero.

        If `errno` = 3, this argument is likely to be a pole of f(x).

        Otherwise, this argument contains no useful information.

    a : float
        The lower bound of the interval resulting from the binary search

    b : float
        The upper bound of the interval resulting from the binary search

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` > 0.0.

        (`errno` 1)
            On entry, `x` = *<value>* and `h` = *<value>*.

            Constraint: `x`+`h` != `x` (to machine accuracy).

        (`errno` 2)
            An interval containing the zero could not be found.
            Increasing `h` and calling ``contfn_brent_interval`` again
            will increase the range searched for the zero. Decreasing
            `h` and calling ``contfn_brent_interval`` again will refine
            the mesh used in the search for the zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            Solution may be a pole rather than a zero.

        (`errno` 4)
            The tolerance `eps` has been set too small for the problem
            being solved. However, the value `x` returned is a good
            approximation to the zero. `eps` = *<value>*.

    See Also
    --------
    :meth:`naginterfaces.library.examples.roots.contfn_brent_interval_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.contfn_brent_interval'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('c05auft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, x.
        utils._BYREF_FLOAT64, # Input, h.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._BYREF_FLOAT64, # Input, eta.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Output, a.
        utils._BYREF_FLOAT64, # Output, b.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _h = utils._EngineFloat64ScalarType(
        h, {'fun_name': fun_name, 'entity': 'h'},
    ).to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _eta = utils._EngineFloat64ScalarType(
        eta, {'fun_name': fun_name, 'entity': 'eta'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _a = utils._EngineFloat64ScalarType.empty_ctypes()
    _b = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _h,
        _eps,
        _eta,
        _f,
        _fsub,
        _fh,
        _a,
        _b,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: eps > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s} and h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: x+h != x (to machine accuracy).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1003:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An interval containing the zero could not be found.',
                    'Increasing h and calling contfn_brent_interval again will increase the range searched for',
                    'the zero. Decreasing h and calling contfn_brent_interval again will refine the mesh used',
                    'in the search for the zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1004:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Solution may be a pole rather than a zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1005:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The tolerance eps has been set too small for the problem',
                    'being solved. However, the value x returned is',
                    'a good approximation to the zero.',
                    'eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        _a_py = utils._EngineFloat64ScalarType.to_py(_a)
        _b_py = utils._EngineFloat64ScalarType.to_py(_b)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['x', 'a', 'b'],
        )(
            _x_py,
            _a_py,
            _b_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def contfn_interval_rcomm(x, fx, h, boundl, boundu, y, comm, ind):
    u"""
    Binary search for interval containing zero of continuous function
    (reverse communication).

    For full information please refer to the NAG Library document for
    c05av

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05avf.html

    Parameters
    ----------
    x : float
        On initial entry: the best available approximation to the zero.

    fx : float
        On initial entry: if `ind` = 1, `fx` need not be set.

        If `ind` = -1, `fx` must contain f(`x`) for the initial value of
        `x`.

        On intermediate entry: must contain f(`x`) for the current value
        of `x`.

    h : float
        On initial entry: a basic step size which is used in the binary
        search for an interval containing a zero. The basic step sizes
        `h`,0.1*`h`, 0.01*`h` and 0.001*`h` are used in turn when
        searching for the zero.

    boundl : float
        The lower bound for the interval of search for the zero.

    boundu : float
        The upper bound for the interval of search for the zero.

    y : float
        On initial entry: need not be set.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    ind : int
        On initial entry: must be set to 1 or -1.

        `ind` = 1

          `fx` need not be set.

        `ind` = -1

          `fx` must contain f(`x`).

    Returns
    -------
    x : float
        On intermediate exit: contains the point at which f must be
        evaluated before re-entry to the function.

        On final exit: contains one end of an interval containing the
        zero, the other end being in `y`, unless an error has occurred.
        If `errno` = 4, this argument and `y` are the end points of the
        largest interval searched. If a zero is located exactly, its
        value is returned in this argument (and in `y`).

    h : float
        On final exit: is undefined.

    y : float
        On final exit: contains the closest point found to the final
        value of `x`, such that f(`x`)*f(`y`) <= 0.0. If a value `x` is
        found such that f(`x`) = 0, `y` = `x`. On final exit with
        `errno` = 4, `x` and this argument are the end points of the
        largest interval searched.

    c1 : float
        On final exit: if the function exits successfully or `errno` =
        4, `c1` is set to f(`y`).

    ind : int
        On intermediate exit: contains 2 or 3. The calling program must
        evaluate f at `x`, storing the result in `fx`, and re-enter
        ``contfn_interval_rcomm`` with all other arguments unchanged.

        On final exit: contains 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `boundl` = *<value>* and `boundu` = *<value>*.

            Constraint: `boundl` < `boundu`.

        (`errno` 1)
            On entry, `x` = *<value>*, `boundl` = *<value>* and `boundu`
            = *<value>*.

            Constraint: `boundl` <= `x` <= `boundu`.

        (`errno` 1)
            On entry, `x` = *<value>*, `h` = *<value>*, `boundl` =
            *<value>* and `boundu` = *<value>*.

            Constraint: either `x`+`h` or `x`-`h` must lie inside the
            closed interval [`boundl`,`boundu`].

        (`errno` 2)
            On entry, `x` = *<value>* and `h` = *<value>*.

            Constraint: `h` must be sufficiently large that `x`+`h` !=
            `x` on the computer.

        (`errno` 3)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = -1, 1, 2 or 3.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            An interval containing the zero could not be found. Try
            restarting with modified `x` and `h`.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.contfn_interval_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c05avft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, x.
        utils._BYREF_FLOAT64, # Input, fx.
        utils._BYREF_FLOAT64, # Input/Output, h.
        utils._BYREF_FLOAT64, # Input, boundl.
        utils._BYREF_FLOAT64, # Input, boundu.
        utils._BYREF_FLOAT64, # Input/Output, y.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[11].
        utils._BYREF_INT, # Input/Output, ind.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _fx = utils._EngineFloat64ScalarType(
        fx, {'fun_name': fun_name, 'entity': 'fx'},
    ).to_ctypes()
    _h = utils._EngineFloat64ScalarType(
        h, {'fun_name': fun_name, 'entity': 'h'},
    ).to_ctypes()
    _boundl = utils._EngineFloat64ScalarType(
        boundl, {'fun_name': fun_name, 'entity': 'boundl'},
    ).to_ctypes()
    _boundu = utils._EngineFloat64ScalarType(
        boundu, {'fun_name': fun_name, 'entity': 'boundu'},
    ).to_ctypes()
    _y = utils._EngineFloat64ScalarType(
        y, {'fun_name': fun_name, 'entity': 'y'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('c',),
    )
    if _comm_needs_init:
        _c = utils._EngineFloat64ArrayType.empty_ctypes(11)
        comm['c'] = _c
    else:
        _c = comm['c']
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _fx,
        _h,
        _boundl,
        _boundu,
        _y,
        _c,
        _ind,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ind_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, boundl = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'boundu = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: boundl < boundu.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}, boundl = {:s}'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and boundu = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: boundl <= x <= boundu.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1003:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}, h = {:s},'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'boundl = {:s} and boundu = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: either x+h or x-h must lie inside the closed interval',
                    '[boundl,boundu].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1004:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s} and h = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: h must be sufficiently large that x+h != x on the computer.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1005:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = -1, 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1006:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'An interval containing the zero could not be found.',
                    'Try restarting with modified x and h.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        _h_py = utils._EngineFloat64ScalarType.to_py(_h)
        _y_py = utils._EngineFloat64ScalarType.to_py(_y)
        _c1_py = utils._EngineFloat64ArrayType.get_element(_c, {'fun_name': fun_name, 'entity': '_c'}, (0,), validate=False)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['x', 'h', 'y', 'c1', 'ind'],
        )(
            _x_py,
            _h_py,
            _y_py,
            _c1_py,
            _ind_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def contfn_cntin(x, eps, eta, f, nfmax, data=None):
    u"""
    Zero of continuous function, continuation method, from a given
    starting value.

    For full information please refer to the NAG Library document for
    c05aw

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05awf.html

    Parameters
    ----------
    x : float
        An initial approximation to the zero.

    eps : float
        An absolute tolerance to control the accuracy to which the zero
        is determined. In general, the smaller the value of `eps` the
        more accurate `x` will be as an approximation to alpha. Indeed,
        for very small positive values of `eps`, it is likely that the
        final approximation will satisfy |`x`-alpha| < `eps`. You are
        advised to call the function with more than one value for `eps`
        to check the accuracy obtained.

    eta : float
        A value such that if |f(x)| < `eta`, x is accepted as the zero.
        `eta` may be specified as 0.0 (see [Accuracy]).

    f : callable retval = f(x, data=None)
        `f` must evaluate the function f whose zero is to be determined.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f evaluated at `x`.

    nfmax : int
        The maximum permitted number of calls to `f` from
        ``contfn_cntin``. If `f` is inexpensive to evaluate, `nfmax`
        should be given a large value (say > 1000).

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float
        The final approximation to the zero, unless `errno` = 1, 2 or 5,
        in which case it contains no useful information.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `nfmax` = *<value>*.

            Constraint: `nfmax` > 0.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` > 0.0.

        (`errno` 2)
            Internal scale factor invalid for this problem. Consider
            using ``contfn_cntin_rcomm`` instead and setting scal.

        (`errno` 3)
            Either `f` has no zero near `x` or too much accuracy has
            been requested. Check the coding of `f` or increase `eps`.

        (`errno` 4)
            More than `nfmax` calls have been made to `f`.

        (`errno` 5)
            A serious error occurred in an internal call to an auxiliary
            function.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.contfn_cntin'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('c05awft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, x.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._BYREF_FLOAT64, # Input, eta.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_INT, # Input, nfmax.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _eta = utils._EngineFloat64ScalarType(
        eta, {'fun_name': fun_name, 'entity': 'eta'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _nfmax = utils._EngineIntScalarType(
        nfmax, {'fun_name': fun_name, 'entity': 'nfmax'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _eps,
        _eta,
        _f,
        _fsub,
        _fh,
        _nfmax,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nfmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nfmax > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: eps > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Internal scale factor invalid for this problem.',
                    'Consider using contfn_cntin_rcomm instead and setting scal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either f has no zero near x or too much accuracy has been requested.',
                    'Check the coding of f or increase eps.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'More than nfmax calls have been made to f.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'A serious error occurred in an internal call to an auxiliary function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        return_data = _x_py
    _err_class.manage()
    return return_data

def contfn_cntin_rcomm(x, fx, tol, ir, scal, comm, ind):
    u"""
    Zero of continuous function, continuation method, from a given
    starting value (reverse communication).

    For full information please refer to the NAG Library document for
    c05ax

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05axf.html

    Parameters
    ----------
    x : float
        On initial entry: an initial approximation to the zero.

    fx : float
        On initial entry: if `ind` = 1, `fx` need not be set.

        If `ind` = -1, `fx` must contain f(`x`) for the initial value of
        `x`.

        On intermediate entry: must contain f(`x`) for the current value
        of `x`.

    tol : float
        On initial entry: a value that controls the accuracy to which
        the zero is determined. `tol` is used in determining the
        convergence of the secant iteration used at each stage of the
        continuation process. It is used directly when solving the last
        problem (theta_m = 0 in [Description]), and sqrt(`tol`) is used
        for the problem defined by theta_r, r < m. Convergence to the
        accuracy specified by `tol` is not guaranteed, and so you are
        recommended to find the zero using at least two values for `tol`
        to check the accuracy obtained.

    ir : int
        On initial entry: indicates the type of error test required, as
        follows. Solving the problem defined by theta_r, 1 <= r <= m,
        involves computing a sequence of secant iterates
        x_r^0,x_r^1,.... This sequence will be considered to have
        converged only if:

        for `ir` = 0,

            |x_r^(i+1)-x_r^(i)| <= eps*max(1.0, |x_r^(i)|),

        for `ir` = 1,

            |x_r^(i+1)-x_r^(i)| <= eps,

        for `ir` = 2,

            |x_r^(i+1)-x_r^(i)| <= eps*|x_r^(i)|,

        for some i > 1; here eps is either `tol` or sqrt(`tol`) as
        discussed above.

        Note that there are other subsidiary conditions (not given here)
        which must also be satisfied before the secant iteration is
        considered to have converged.

    scal : float
        On initial entry: a factor for use in determining a significant
        approximation to the derivative of f(x) at x = x_0, the initial
        value. A number of difference approximations to f'(x_0) are
        calculated using

            f'(x_0)~(f(x_0+h)-f(x_0))/h

        where |h| < |`scal`| and h has the same sign as `scal`. A
        significance (cancellation) check is made on each difference
        approximation and the approximation is rejected if
        insignificant.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    ind : int
        On initial entry: must be set to 1 or -1.

        `ind` = 1

          `fx` need not be set.

        `ind` = -1

          `fx` must contain f(`x`).

    Returns
    -------
    x : float
        On intermediate exit: the point at which f must be evaluated
        before re-entry to the function.

        On final exit: the final approximation to the zero.

    ind : int
        On intermediate exit: contains 2, 3 or 4. The calling program
        must evaluate f at `x`, storing the result in `fx`, and re-enter
        ``contfn_cntin_rcomm`` with all other arguments unchanged.

        On final exit: contains 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 1)
            On entry, `ir` = *<value>*.

            Constraint: `ir` = 0, 1 or 2.

        (`errno` 2)
            On initial entry, `ind` = *<value>*.

            Constraint: `ind` = -1 or 1.

        (`errno` 2)
            On intermediate entry, `ind` = *<value>*.

            Constraint: `ind` = 2, 3 or 4.

        (`errno` 3)
            On entry, `scal` = *<value>*.

            Constraint: `x`+`scal` != `x` (to machine accuracy).

        (`errno` 3)
            Significant derivatives of f cannot be computed. This can
            happen when f is almost constant and nonzero, for any value
            of `scal`.

        (`errno` 4)
            Current problem in the continuation sequence cannot be
            solved. Perhaps the original problem had no solution or the
            continuation path passes through a set of insoluble
            problems: consider refining the initial approximation to the
            zero. Alternatively, `tol` is too small, and the accuracy
            requirement is too stringent, or too large and the initial
            approximation too poor.

        (`errno` 5)
            Continuation away from the initial point is not possible.
            This error exit will usually occur if the problem has not
            been properly posed or the error requirement is extremely
            stringent.

        (`errno` 6)
            Final problem (with theta_m = 0) cannot be solved. It is
            likely that too much accuracy has been requested, or that
            the zero is at alpha = 0 and `ir` = 2.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.contfn_cntin_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c05axft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, x.
        utils._BYREF_FLOAT64, # Input, fx.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, ir.
        utils._BYREF_FLOAT64, # Input, scal.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[26].
        utils._BYREF_INT, # Input/Output, ind.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _fx = utils._EngineFloat64ScalarType(
        fx, {'fun_name': fun_name, 'entity': 'fx'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _ir = utils._EngineIntScalarType(
        ir, {'fun_name': fun_name, 'entity': 'ir'},
    ).to_ctypes()
    _scal = utils._EngineFloat64ScalarType(
        scal, {'fun_name': fun_name, 'entity': 'scal'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('c',),
    )
    if _comm_needs_init:
        _c = utils._EngineFloat64ArrayType.empty_ctypes(26)
        comm['c'] = _c
    else:
        _c = comm['c']
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _fx,
        _tol,
        _ir,
        _scal,
        _c,
        _ind,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ind_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ir = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ir = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1021:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = -1 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1022:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1031:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scal = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x+scal != x (to machine accuracy).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1032:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Significant derivatives of f cannot be computed.',
                    'This can happen when f is almost constant and nonzero,',
                    'for any value of scal.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1041:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Current problem in the continuation sequence cannot be solved.',
                    'Perhaps the original problem had no solution or the',
                    'continuation path passes through a set of insoluble problems:',
                    'consider refining the initial approximation to the zero.',
                    'Alternatively, tol is too small, and the accuracy requirement is',
                    'too stringent, or too large and the initial approximation too poor.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1051:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Continuation away from the initial point is not possible.',
                    'This error exit will usually occur if the problem has not been properly',
                    'posed or the error requirement is extremely stringent.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 1061:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Final problem (with theta_m = 0) cannot be solved.',
                    'It is likely that too much accuracy has been requested,',
                    'or that the zero is at alpha = 0 and ir = 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['x', 'ind'],
        )(
            _x_py,
            _ind_py,
        )
    _err_class.manage()
    return return_data

def contfn_brent(a, b, eps, eta, f, data=None):
    u"""
    Zero of continuous function in a given interval, Brent algorithm.

    For full information please refer to the NAG Library document for
    c05ay

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05ayf.html

    Parameters
    ----------
    a : float
        a, the lower bound of the interval.

    b : float
        b, the upper bound of the interval.

    eps : float
        The termination tolerance on x (see [Description]).

    eta : float
        A value such that if |f(x)| <= `eta`, x is accepted as the zero.
        `eta` may be specified as 0.0 (see [Accuracy]).

    f : callable retval = f(x, data=None)
        `f` must evaluate the function f whose zero is to be determined.

        Parameters
        ~~~~~~~~~~
        x : float
            The point at which the function must be evaluated.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

        Returns
        ~~~~~~~
        retval : float
            The value of f evaluated at `x`.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    x : float
        If the function exits successfully or `errno` = 2, this argument
        is the final approximation to the zero. If `errno` = 3, this
        argument is likely to be a pole of f(x). Otherwise, this
        argument contains no useful information.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: `eps` > 0.0.

        (`errno` 1)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `a` != `b`.

        (`errno` 1)
            On entry, `f`(`a`) and `f`(`b`) have the same sign with
            neither equalling 0.0: `f`(`a`) = *<value>* and `f`(`b`) =
            *<value>*.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            No further improvement in the solution is possible. `eps` is
            too small: `eps` = *<value>*. The final value of `x`
            returned is an accurate approximation to the zero.

        (`errno` 3)
            The function values in the interval [`a`,`b`] might contain
            a pole rather than a zero. Reducing `eps` may help in
            distinguishing between a pole and a zero.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.contfn_brent'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_f_name = fun_name + ':f'

    fh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, retval.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
    )

    def cb_fh(_f, _fsub, en_data, x, retval, _iuser, _ruser):
        """Helper for f."""
        try:
            _x_fh = utils._EngineFloat64ScalarType.to_py(x.contents)
            if data is not None:
                _retval_fh = f(
                    _x_fh,
                    data,
                )
            else:
                _retval_fh = f(
                    _x_fh,
                )
            utils._EngineFloat64ScalarType(
                _retval_fh, {'fun_name': cb_f_name, 'returned': 'retval'},
            ).refresh_ctypes_buffer(retval)
        except:
            excs.append(_sys.exc_info())
            en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('c05ayft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._BYREF_FLOAT64, # Input, eta.
        utils._UNUSED_FUNCTYPE,
        utils._UNUSED_FUNCTYPE,
        fh,
        utils._BYREF_FLOAT64, # Output, x.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _eta = utils._EngineFloat64ScalarType(
        eta, {'fun_name': fun_name, 'entity': 'eta'},
    ).to_ctypes()
    utils._check_callable(f, {'fun_name': fun_name, 'entity': 'f'})
    _f = utils._UNUSED_CALLBACK
    _fsub = utils._UNUSED_CALLBACK
    _fh = utils._cast_callback(
        fh, cb_fh,
        {'fun_name': fun_name, 'entity': 'f'},
    )
    _x = utils._EngineFloat64ScalarType.empty_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _b,
        _eps,
        _eta,
        _f,
        _fsub,
        _fh,
        _x,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: eps > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a != b.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1003:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, f(a) and f(b) have the same sign with neither equalling 0.0:',
                    'f(a) = {:s} and f(b) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1004:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No further improvement in the solution is possible.',
                    'eps is too small: eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'The final value of x returned is an accurate approximation to the zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1005:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function values in the interval [a,b] might contain',
                    'a pole rather than a zero. Reducing eps may help in distinguishing between',
                    'a pole and a zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        return_data = _x_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['x'],
            )(
                _x_py,
            )
    _err_class.manage()
    return return_data

def contfn_brent_rcomm(x, y, fx, tolx, ir, c, ind):
    u"""
    Zero of continuous function in a given interval, Brent algorithm
    (reverse communication).

    For full information please refer to the NAG Library document for
    c05az

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05azf.html

    Parameters
    ----------
    x : float
        On initial entry: `x` and `y` must define an initial interval
        [a,b] containing the zero, such that f(`x`)*f(`y`) <= 0.0. It is
        not necessary that `x` < `y`. Otherwise, `x` and `y` must be the
        same entities output by a previous call to
        ``contfn_brent_rcomm``.

    y : float
        On initial entry: `x` and `y` must define an initial interval
        [a,b] containing the zero, such that f(`x`)*f(`y`) <= 0.0. It is
        not necessary that `x` < `y`. Otherwise, `x` and `y` must be the
        same entities output by a previous call to
        ``contfn_brent_rcomm``.

    fx : float
        On initial entry: if `ind` = 1, `fx` need not be set.

        If `ind` = -1, `fx` must contain f(`x`) for the initial value of
        `x`.

        On intermediate entry: must contain f(`x`) for the current value
        of `x`.

    tolx : float
        On initial entry: the accuracy to which the zero is required.
        The type of error test is specified by `ir`.

    ir : int
        On initial entry: indicates the type of error test.

        `ir` = 0

          The test is: |`x`-`y`| <= 2.0*`tolx`*max(1.0, |`x`|).

        `ir` = 1

          The test is: |`x`-`y`| <= 2.0*`tolx`.

        `ir` = 2

          The test is: |`x`-`y`| <= 2.0*`tolx`*|`x`|.

    c : float, shape (17), modified in place
        On initial entry: if `ind` = 1, no elements of `c` need be set.

        If `ind` = -1, `c`[0] must contain f(`y`), other elements of `c`
        need not be set.

        On final exit: is undefined.

    ind : int
        On initial entry: must be set to 1 or -1.

        `ind` = 1

          `fx` and `c`[0] need not be set.

        `ind` = -1

          `fx` and `c`[0] must contain f(`x`) and f(`y`) respectively.

    Returns
    -------
    x : float
        On intermediate exit: `x` contains the point at which f must be
        evaluated before re-entry to the function. `y` has no external
        significance but must be communicated back to the next call of
        ``contfn_brent_rcomm`` as argument `y`.

        On final exit: `x` and `y` define a smaller interval containing
        the zero, such that f(`x`)*f(`y`) <= 0.0, and |`x`-`y`|
        satisfies the accuracy specified by `tolx` and `ir`, unless an
        error has occurred. If `errno` = 4, `x` and `y` generally
        contain very good approximations to a pole; if `errno` = 5, `x`
        and `y` generally contain very good approximations to the zero
        (see [Errors]). If a point `x` is found such that f(`x`) = 0.0,
        on final exit `x` = `y` (in this case there is no guarantee that
        `x` is a simple zero). In all cases, the value returned in `x`
        is the better approximation to the zero.

    y : float
        On intermediate exit: `x` contains the point at which f must be
        evaluated before re-entry to the function. `y` has no external
        significance but must be communicated back to the next call of
        ``contfn_brent_rcomm`` as argument `y`.

        On final exit: `x` and `y` define a smaller interval containing
        the zero, such that f(`x`)*f(`y`) <= 0.0, and |`x`-`y`|
        satisfies the accuracy specified by `tolx` and `ir`, unless an
        error has occurred. If `errno` = 4, `x` and `y` generally
        contain very good approximations to a pole; if `errno` = 5, `x`
        and `y` generally contain very good approximations to the zero
        (see [Errors]). If a point `x` is found such that f(`x`) = 0.0,
        on final exit `x` = `y` (in this case there is no guarantee that
        `x` is a simple zero). In all cases, the value returned in `x`
        is the better approximation to the zero.

    ind : int
        On intermediate exit: contains 2, 3 or 4. The calling program
        must evaluate f at `x`, storing the result in `fx`, and re-enter
        ``contfn_brent_rcomm`` with all other arguments unchanged.

        On final exit: contains 0.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, f(`x`) and f(`y`) have the same sign with neither
            equalling 0.0: f(`x`) = *<value>* and f(`y`) = *<value>*.

        (`errno` 2)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = -1, 1, 2, 3 or 4.

        (`errno` 3)
            On entry, `tolx` = *<value>*.

            Constraint: `tolx` > 0.0.

        (`errno` 3)
            On entry, `ir` = *<value>*.

            Constraint: `ir` = 0, 1 or 2.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The final interval may contain a pole rather than a zero.
            Note that this error exit is not completely reliable: it may
            be taken in extreme cases when [`x`,`y`] contains a zero, or
            it may not be taken when [`x`,`y`] contains a pole. Both
            these cases occur most frequently when `tolx` is large.

        (`errno` 5)
            The tolerance `tolx` has been set too small for the problem
            being solved. However, the values `x` and `y` returned may
            well be good approximations to the zero. `tolx` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.contfn_brent_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c05azft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input/Output, x.
        utils._BYREF_FLOAT64, # Input/Output, y.
        utils._BYREF_FLOAT64, # Input, fx.
        utils._BYREF_FLOAT64, # Input, tolx.
        utils._BYREF_INT, # Input, ir.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, c[17].
        utils._BYREF_INT, # Input/Output, ind.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _y = utils._EngineFloat64ScalarType(
        y, {'fun_name': fun_name, 'entity': 'y'},
    ).to_ctypes()
    _fx = utils._EngineFloat64ScalarType(
        fx, {'fun_name': fun_name, 'entity': 'fx'},
    ).to_ctypes()
    _tolx = utils._EngineFloat64ScalarType(
        tolx, {'fun_name': fun_name, 'entity': 'tolx'},
    ).to_ctypes()
    _ir = utils._EngineIntScalarType(
        ir, {'fun_name': fun_name, 'entity': 'ir'},
    ).to_ctypes()
    _c_cls = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=(17,),
    )
    _c = _c_cls.to_ctypes()
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y,
        _fx,
        _tolx,
        _ir,
        _c,
        _ind,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ind_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, f(x) and f(y) have the same sign with neither equalling 0.0:',
                    'f(x) = {:s} and f(y) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1002:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = -1, 1, 2, 3 or 4.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1003:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tolx = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tolx > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1004:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ir = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ir = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1005:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The final interval may contain a pole rather than a zero.',
                    'Note that this error exit is not completely reliable: it may be taken',
                    'in extreme cases when [x,y] contains a zero, or it may not be taken',
                    'when [x,y] contains a pole.',
                    'Both these cases occur most frequently when tolx is large.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1006:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The tolerance tolx has been set too small for the problem',
                    'being solved. However, the values x and y returned may well',
                    'be good approximations to the zero.',
                    'tolx = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        _y_py = utils._EngineFloat64ScalarType.to_py(_y)
        _c_cls.refresh_py(_c)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['x', 'y', 'ind'],
        )(
            _x_py,
            _y_py,
            _ind_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def lambertw_real(x, branch, offset):
    u"""
    Real values of Lambert's W function, W(x).

    For full information please refer to the NAG Library document for
    c05ba

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05baf.html

    Parameters
    ----------
    x : float
        If `offset` = True, `x` is the offset Delta x from -exp(-1) of
        the intended argument to W; that is, W(beta) is computed, where
        beta = -exp(-1)+Delta x.

        If `offset` = False, `x` is the argument x of the function; that
        is, W(beta) is computed, where beta = x.

    branch : int
        The real branch required.

        `branch` = 0

          The branch W_0 is selected.

        `branch` = -1

          The branch W_{-1} is selected.

    offset : bool
        Controls whether or not `x` is being specified as an offset from
        -exp(-1).

    Returns
    -------
    lambw : float
        The real values of Lambert's W function W(x).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `branch` = *<value>*.

            Constraint: `branch` = 0 or -1.

        (`errno` 1)
            On entry, `offset` = True and `x` = *<value>*.

            Constraint: if `offset` = True then `x` >= 0.0.

        (`errno` 1)
            On entry, `offset` = False and `x` = *<value>*.

            Constraint: if `offset` = False then `x` >= -exp(-1.0).

        (`errno` 1)
            On entry, `branch` = -1, `offset` = True and `x` =
            *<value>*.

            Constraint: if `branch` = -1 and `offset` = True then `x` <
            exp(-1.0).

        (`errno` 1)
            On entry, `branch` = -1, `offset` = False and `x` =
            *<value>*.

            Constraint: if `branch` = -1 and `offset` = False then `x` <
            0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            For the given offset `x`, W is negligibly different from -1:
            `x` = *<value>*.

        (`errno` 2)
            `x` is close to -exp(-1). Enter `x` as an offset to -exp(-1)
            for greater accuracy: `x` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.lambertw_real'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c05baft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_INT, # Input, branch.
        utils._BYREF_LOGICAL, # Input, offset.
        utils._BYREF_FLOAT64, # Output, lambw.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _branch = utils._EngineIntScalarType(
        branch, {'fun_name': fun_name, 'entity': 'branch'},
    ).to_ctypes()
    _offset = utils._EngineLogicalScalarType(
        offset, {'fun_name': fun_name, 'entity': 'offset'},
    ).to_ctypes()
    _lambw = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _branch,
        _offset,
        _lambw,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, branch = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: branch = 0 or -1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, offset = True and x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if offset = True then x >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1012:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, offset = False and x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if offset = False then x >= -exp(-1.0).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1013:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, branch = -1, offset = True and x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if branch = -1 and offset = True then x < exp(-1.0).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1014:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, branch = -1, offset = False and x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if branch = -1 and offset = False then x < 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1110:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'For the given offset x, W is negligibly different from -1:',
                    'x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1111:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'x is close to -exp(-1).',
                    'Enter x as an offset to -exp(-1) for greater accuracy:',
                    'x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _lambw_py = utils._EngineFloat64ScalarType.to_py(_lambw)
        return_data = _lambw_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['lambw'],
            )(
                _lambw_py,
            )
    _err_class.manage()
    return return_data

def lambertw_complex(branch, offset, z):
    u"""
    Values of Lambert's W function, W(z).

    For full information please refer to the NAG Library document for
    c05bb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05bbf.html

    Parameters
    ----------
    branch : int
        The branch required.

    offset : bool
        Controls whether or not `z` is being specified as an offset from
        -exp(-1).

    z : complex
        If `offset` = True, `z` is the offset Delta z from -exp(-1) of
        the intended argument to W; that is, W(beta) is computed, where
        beta = -exp(-1)+Delta z.

        If `offset` = False, `z` is the argument z of the function; that
        is, W(beta) is computed, where beta = z.

    Returns
    -------
    w : complex
        The value W(beta): see also the description of `z`.

    resid : float
        The residual |W(beta)exp(W(beta))-beta|: see also the
        description of `z`.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            For the given offset `z`, W is negligibly different from -1:
            Re(`z`) = *<value>* and Im(`z`) = *<value>*.

        (`errno` 1)
            `z` is close to -exp(-1). Enter `z` as an offset to -exp(-1)
            for greater accuracy: Re(`z`) = *<value>* and Im(`z`) =
            *<value>*.

        (`errno` 2)
            The iterative procedure used internally did not converge in
            *<value>* iterations. Check the value of `resid` for the
            accuracy of `w`.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.lambertw_complex'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c05bbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, branch.
        utils._BYREF_INT, # Input, ioffset.
        utils._BYREF_COMPLEX128, # Input, z.
        utils._BYREF_COMPLEX128, # Output, w.
        utils._BYREF_FLOAT64, # Output, resid.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _branch = utils._EngineIntScalarType(
        branch, {'fun_name': fun_name, 'entity': 'branch'},
    ).to_ctypes()
    _offset = utils._EngineLogicalScalarType(
        offset, {'fun_name': fun_name, 'entity': 'offset'},
    ).to_ctypes()
    _offset_py = utils._EngineLogicalScalarType.to_py(_offset)
    if _offset_py is False:
        _ioffset = utils._EngineIntScalarType.c_type(0)
    elif _offset_py is True:
        _ioffset = utils._EngineIntScalarType.c_type(1)
    else:
        _err_class = utils._EngineUnexpectedExit(
            fun_name=fun_name,
            msg_lines=[
                'Invalid value for offset.',
            ],
            errcodes=(utils._IERR_UNEXPECTED,),
        )
        _err_class.manage()
    _z = utils._EngineComplex128ScalarType(
        z, {'fun_name': fun_name, 'entity': 'z'},
    ).to_ctypes()
    _w = utils._EngineComplex128ScalarType.empty_ctypes()
    _resid = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _branch,
        _ioffset,
        _z,
        _w,
        _resid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'For the given offset z, W is negligibly different from -1:',
                    'Re(z) = {:s} and Im(z) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1011:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'z is close to -exp(-1). Enter z as an offset to -exp(-1) for greater',
                    'accuracy: Re(z) = {:s} and'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Im(z) = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1110:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iterative procedure used internally did not converge in',
                    '{:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Check the value of resid for the accuracy of w.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _w_py = utils._EngineComplex128ScalarType.to_py(_w)
        _resid_py = utils._EngineFloat64ScalarType.to_py(_resid)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['w', 'resid'],
        )(
            _w_py,
            _resid_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def sys_func_aa(fcn, n, x, fvec, atol, rtol, m, cndtol, astart, data=None):
    u"""
    Solution of a system of nonlinear equations using Anderson
    acceleration.

    For full information please refer to the NAG Library document for
    c05mb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05mbf.html

    Parameters
    ----------
    fcn : callable fcn(n, x, fvec, data=None)
        `fcn` must return the values of the functions f_k at a point x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float, ndarray, shape (`n`)
            The components of the point x at which the functions must be
            evaluated.

        fvec : float, ndarray, shape (`n`), to be modified in place
            On exit: the function values f_k(x).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    n : int
        n, the number of equations.

    x : float, shape (`n`), modified in place
        On entry: an initial guess at the solution vector, x^_0.

        On exit: the final estimate of the solution vector.

    fvec : float, shape (`n`), modified in place
        On exit: the function values at the final point, `x`.

    atol : float
        The absolute convergence criterion; see `rtol`.

    rtol : float
        The relative convergence criterion. At each iteration
        ||f(x^_i)|| is computed. The iteration is deemed to have
        converged if ||f(x^_i)|| <= max(`atol`, `rtol`*||f(x^_0)||).

    m : int
        m, the number of previous iterates to use in Anderson
        acceleration. If m = 0, Anderson acceleration is not used.

    cndtol : float
        The maximum allowable condition number for the triangular QR
        factor generated during Anderson acceleration. At each
        iteration, if the condition number exceeds `cndtol`, columns are
        deleted until it is sufficiently small.

        If `cndtol` = 0.0, no condition number tests are performed.

    astart : int
        The number of iterations by which to delay the start of Anderson
        acceleration.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 3)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 4)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: 0 <= `m` <= `n`.

        (`errno` 5)
            On entry, `cndtol` = *<value>*.

            Constraint: `cndtol` >= 0.0.

        (`errno` 6)
            On entry, `astart` = *<value>*.

            Constraint: `astart` >= 0.

        (`errno` 7)
            An error occurred in evaluating the QR decomposition during
            Anderson acceleration. This may be due to slow convergence
            of the iteration. Try setting the value of `cndtol`. If
            condition number tests are already performed, try decreasing
            `cndtol`.

        (`errno` 11)
            The iteration has diverged and subsequent iterates are too
            large to be computed in floating-point arithmetic.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 8)
            The iteration is not making good progress. This failure exit
            may indicate that the system does not have a zero, or that
            the solution is very close to the origin. Rerunning
            ``sys_func_aa`` from a different starting point may avoid
            the region of difficulty.

        (`errno` 9)
            There have been at least 200*(`n`+1) calls to `fcn`.
            Consider restarting the calculation from the point held in
            `x`.

    NagCallbackTerminateWarning
        (`errno` 10)
            Termination requested in `fcn`.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.sys_func_aa'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fvec[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Input/Output, iflag.
    )

    def cb_fcnh(_fcn, en_data, n, x, fvec, _iuser, _ruser, iflag):
        """Helper for fcn."""
        try:
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_n_fcnh,),
            )
            _fvec_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                fvec,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _fvec_fcnh,
                    data,
                )
            else:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _fvec_fcnh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fvec_fcnh, {'fun_name': cb_fcn_name, 'returned': 'fvec'}, fvec,
            )
            utils._unlink_pointer(fvec)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('c05mbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fvec[n].
        utils._BYREF_FLOAT64, # Input, atol.
        utils._BYREF_FLOAT64, # Input, rtol.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_FLOAT64, # Input, cndtol.
        utils._BYREF_INT, # Input, astart.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _fvec_cls = utils._EngineFloat64ArrayType(
        fvec, {'fun_name': fun_name, 'entity': 'fvec'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _fvec = _fvec_cls.to_ctypes(can_reference_contents=False)
    _atol = utils._EngineFloat64ScalarType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
    ).to_ctypes()
    _rtol = utils._EngineFloat64ScalarType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _cndtol = utils._EngineFloat64ScalarType(
        cndtol, {'fun_name': fun_name, 'entity': 'cndtol'},
    ).to_ctypes()
    _astart = utils._EngineIntScalarType(
        astart, {'fun_name': fun_name, 'entity': 'astart'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fcn,
        _fcnh,
        _n,
        _x,
        _fvec,
        _atol,
        _rtol,
        _m,
        _cndtol,
        _astart,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: atol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0 <= m <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cndtol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cndtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, astart = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: astart >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error occurred in evaluating the QR decomposition during Anderson',
                    'acceleration. This may be due to slow convergence of the iteration.',
                    'Try setting the value of cndtol. If condition number tests',
                    'are already performed, try decreasing cndtol.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress.',
                    'This failure exit may indicate that the system does not have a zero, or',
                    'that the solution is very close to the origin.',
                    'Rerunning sys_func_aa from a different starting point may avoid',
                    'the region of difficulty.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There have been at least 200*(n+1) calls to fcn.',
                    'Consider restarting the calculation from the point held in x.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99990:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in fcn.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 11 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration has diverged and subsequent iterates are too large to be',
                    'computed in floating-point arithmetic.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _fvec_cls.refresh_py(_fvec)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def sys_func_aa_rcomm(irevcm, n, x, fvec, atol, rtol, m, cndtol, astart, comm):
    u"""
    Solution of a system of nonlinear equations using Anderson
    acceleration (reverse communication).

    For full information please refer to the NAG Library document for
    c05md

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05mdf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must have the value 0.

    n : int
        n, the number of equations.

    x : float, shape (`n`), modified in place
        On initial entry: an initial guess at the solution vector, x^_0.

        On intermediate exit: contains the current point.

        On final exit: the final estimate of the solution vector.

    fvec : float, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` = 1, `fvec` must not be
        changed.

        If `irevcm` = 2, `fvec` must be set to the values of the
        functions computed at the current point `x`, f(x^_i).

        On final exit: the function values at the final point, `x`.

    atol : float
        On initial entry: the absolute convergence criterion.

    rtol : float
        On initial entry: the relative convergence criterion. At each
        iteration ||f(x^_i)|| is computed. The iteration is deemed to
        have converged if ||f(x^_i)|| <= max(`atol`,
        `rtol`*||f(x^_0)||).

    m : int
        On initial entry: m, the number of previous iterates to use in
        Anderson acceleration. If m = 0, Anderson acceleration is not
        used.

    cndtol : float
        On initial entry: the maximum allowable condition number for the
        triangular QR factor generated during Anderson acceleration. At
        each iteration, if the condition number exceeds `cndtol`,
        columns are deleted until it is sufficiently small.

        If `cndtol` = 0.0, no condition number tests are performed.

    astart : int
        On initial entry: the number of iterations by which to delay the
        start of Anderson acceleration.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    irevcm : int
        On intermediate exit: specifies what action you must take before
        re-entering ``sys_func_aa_rcomm`` with `irevcm` set to this
        value. The value of this argument should be interpreted as
        follows:

        `irevcm` = 1

          Indicates the start of a new iteration. No action is required
          by you, but `x` and `fvec` are available for printing, and a
          limit on the number of iterations can be applied.

        `irevcm` = 2

          Indicates that before re-entry to ``sys_func_aa_rcomm``,
          `fvec` must contain the function values f(x^_i).

        On final exit: `irevcm` = 0 and the algorithm has terminated.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On initial entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0.

        (`errno` 1)
            On intermediate entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 1 or 2.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 3)
            On entry, `atol` = *<value>*.

            Constraint: `atol` >= 0.0.

        (`errno` 4)
            On entry, `rtol` = *<value>*.

            Constraint: `rtol` >= 0.0.

        (`errno` 5)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: 0 <= `m` <= `n`.

        (`errno` 6)
            On entry, `cndtol` = *<value>*.

            Constraint: `cndtol` >= 0.0.

        (`errno` 7)
            On entry, `astart` = *<value>*.

            Constraint: `astart` >= 0.

        (`errno` 8)
            An error occurred in evaluating the QR decomposition during
            Anderson acceleration. This may be due to slow convergence
            of the iteration. Try setting the value of `cndtol`. If
            condition number tests are already performed, try decreasing
            `cndtol`.

        (`errno` 9)
            The iteration is not making good progress, as measured by
            the reduction in the norm of f(x) in the last *<value>*
            iterations.

        (`errno` 10)
            The iteration has diverged and subsequent iterates are too
            large to be computed in floating-point arithmetic.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.sys_func_aa_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c05mdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fvec[n].
        utils._BYREF_FLOAT64, # Input, atol.
        utils._BYREF_FLOAT64, # Input, rtol.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_FLOAT64, # Input, cndtol.
        utils._BYREF_INT, # Input, astart.
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[14+m].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[2*m*n+m**2+m+2*n+1+min(m, 1)*max(n, 3*m)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _fvec_cls = utils._EngineFloat64ArrayType(
        fvec, {'fun_name': fun_name, 'entity': 'fvec'},
        exp_shape=(_n_py,),
    )
    _fvec = _fvec_cls.to_ctypes()
    _atol = utils._EngineFloat64ScalarType(
        atol, {'fun_name': fun_name, 'entity': 'atol'},
    ).to_ctypes()
    _rtol = utils._EngineFloat64ScalarType(
        rtol, {'fun_name': fun_name, 'entity': 'rtol'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _cndtol = utils._EngineFloat64ScalarType(
        cndtol, {'fun_name': fun_name, 'entity': 'cndtol'},
    ).to_ctypes()
    _astart = utils._EngineIntScalarType(
        astart, {'fun_name': fun_name, 'entity': 'astart'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav', 'rwsav'),
    )
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(14+_m_py)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(2*_m_py*_n_py+_m_py**2+_m_py+2*_n_py+1+min(_m_py, 1)*max(_n_py, 3*_m_py))
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _n,
        _x,
        _fvec,
        _atol,
        _rtol,
        _m,
        _cndtol,
        _astart,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On initial entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On intermediate entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, atol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: atol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: 0 <= m <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cndtol = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: cndtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, astart = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: astart >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 99989:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An error occurred in evaluating the QR decomposition during Anderson',
                    'acceleration. This may be due to slow convergence of the iteration.',
                    'Try setting the value of cndtol. If condition number tests',
                    'are already performed, try decreasing cndtol.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress, as measured by the reduction',
                    'in the norm of f(x) in the last {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration has diverged and subsequent iterates are too large to be',
                    'computed in floating-point arithmetic.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_cls.refresh_py(_x)
        _fvec_cls.refresh_py(_fvec)
        return_data = _irevcm_py
    _err_class.manage()
    return return_data

def sys_func_easy(fcn, n, x, fvec, xtol, data=None):
    u"""
    Solution of a system of nonlinear equations using function values
    only (easy-to-use).

    For full information please refer to the NAG Library document for
    c05qb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05qbf.html

    Parameters
    ----------
    fcn : callable fcn(n, x, fvec, data=None)
        `fcn` must return the values of the functions f_i at a point x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float, ndarray, shape (`n`)
            The components of the point x at which the functions must be
            evaluated.

        fvec : float, ndarray, shape (`n`), to be modified in place
            On exit: the function values f_i(x).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    n : int
        n, the number of equations.

    x : float, shape (`n`), modified in place
        On entry: an initial guess at the solution vector.

        On exit: the final estimate of the solution vector.

    fvec : float, shape (`n`), modified in place
        On exit: the function values at the final point returned in `x`.

    xtol : float
        The accuracy in `x` to which the solution is required.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There have been at least 200*(`n`+1) calls to `fcn`.
            Consider restarting the calculation from the point held in
            `x`.

        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress. This failure exit
            may indicate that the system does not have a zero, or that
            the solution is very close to the origin (see [Accuracy]).
            Otherwise, rerunning ``sys_func_easy`` from a different
            starting point may avoid the region of difficulty.

    NagCallbackTerminateWarning
        (`errno` 5)
            Termination requested in `fcn`.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.sys_func_easy'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fvec[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Input/Output, iflag.
    )

    def cb_fcnh(_fcn, en_data, n, x, fvec, _iuser, _ruser, iflag):
        """Helper for fcn."""
        try:
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _x_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_n_fcnh,),
            )
            _fvec_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                fvec,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _fvec_fcnh,
                    data,
                )
            else:
                fcn(
                    _n_fcnh,
                    _x_fcnh,
                    _fvec_fcnh,
                )
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fvec_fcnh, {'fun_name': cb_fcn_name, 'returned': 'fvec'}, fvec,
            )
            utils._unlink_pointer(fvec)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('c05qbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fvec[n].
        utils._BYREF_FLOAT64, # Input, xtol.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _fvec_cls = utils._EngineFloat64ArrayType(
        fvec, {'fun_name': fun_name, 'entity': 'fvec'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _fvec = _fvec_cls.to_ctypes(can_reference_contents=False)
    _xtol = utils._EngineFloat64ScalarType(
        xtol, {'fun_name': fun_name, 'entity': 'xtol'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fcn,
        _fcnh,
        _n,
        _x,
        _fvec,
        _xtol,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There have been at least 200*(n+1) calls to fcn.',
                    'Consider restarting the calculation from the point held in x.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No further improvement in the solution is possible.',
                    'xtol is too small: xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress.',
                    'This failure exit may indicate that the system does not have a zero, or',
                    'that the solution is very close to the origin',
                    '(see [Accuracy]).',
                    'Otherwise, rerunning sys_func_easy from a different starting point may avoid',
                    'the region of difficulty.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in fcn.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 11 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _fvec_cls.refresh_py(_fvec)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def sys_func_expert(fcnval, fcnmon, n, x, fvec, xtol, maxfev, ml, mu, epsfcn, mode, diag, factor, nprint, fjac, r, qtf, data=None):
    u"""
    Solution of a system of nonlinear equations using function values
    only (comprehensive).

    For full information please refer to the NAG Library document for
    c05qc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05qcf.html

    Parameters
    ----------
    fcnval : callable fcnval(n, x, fvec, data=None)
        `fcnval` must return the values of the functions f_i at a point
        x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float, ndarray, shape (`n`)
            The components of the point x at which the functions must be
            evaluated.

        fvec : float, ndarray, shape (`n`), to be modified in place
            On exit: this argument must contain the function values
            f_i(x).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    fcnmon : None or callable fcnmon(n, x, fvec, data=None)
        Note: if this argument is None then a NAG-supplied facility will
        be used.

        `fcnmon` may be used to monitor the progress of the algorithm.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float, ndarray, shape (`n`)
            The components of the current evaluation point x.

        fvec : float, ndarray, shape (`n`)
            This argument contains the function values f_i(x).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    n : int
        n, the number of equations.

    x : float, shape (`n`), modified in place
        On entry: an initial guess at the solution vector.

        On exit: the final estimate of the solution vector.

    fvec : float, shape (`n`), modified in place
        On exit: the function values at the final point returned in `x`.

    xtol : float
        The accuracy in `x` to which the solution is required.

    maxfev : int
        The maximum number of calls to `fcnval`. ``sys_func_expert``
        will exit with `errno` = 2, if, at the end of an iteration, the
        number of calls to `fcnval` exceeds `maxfev`.

    ml : int
        The number of subdiagonals within the band of the Jacobian
        matrix. (If the Jacobian is not banded, or you are unsure, set
        `ml` = `n`-1.)

    mu : int
        The number of superdiagonals within the band of the Jacobian
        matrix. (If the Jacobian is not banded, or you are unsure, set
        `mu` = `n`-1.)

    epsfcn : float
        A rough estimate of the largest relative error in the functions.
        It is used in determining a suitable step for a forward
        difference approximation to the Jacobian. If `epsfcn` is less
        than machine precision (returned by ``machine.precision``) then
        machine precision is used. Consequently a value of 0.0 will
        often be suitable.

    mode : int
        Indicates whether or not you have provided scaling factors in
        `diag`.

        If `mode` = 2, the scaling must have been specified in `diag`.

        Otherwise, if `mode` = 1, the variables will be scaled
        internally.

    diag : float, shape (`n`), modified in place
        On entry: if `mode` = 2, `diag` must contain multiplicative
        scale factors for the variables.

        If `mode` = 1, `diag` need not be set.

        On exit: the scale factors actually used (computed internally if
        `mode` = 1).

    factor : float
        A quantity to be used in determining the initial step bound. In
        most cases, `factor` should lie between 0.1 and 100.0. (The step
        bound is `factor`*||`diag`*`x`||_2 if this is nonzero; otherwise
        the bound is `factor`.)

    nprint : int
        Indicates whether (and how often) calls to `fcnmon` are to be
        made.

        `nprint` <= 0

          No calls are made.

        `nprint` > 0

          `fcnmon` is called at the beginning of the first iteration,
          every `nprint` iterations thereafter and immediately before
          the return from ``sys_func_expert``.

    fjac : float, shape (`n`, `n`), modified in place
        On exit: the orthogonal matrix Q produced by the QR
        factorization of the final approximate Jacobian.

    r : float, shape (`n`*(`n`+1)/2), modified in place
        On exit: the upper triangular matrix R produced by the QR
        factorization of the final approximate Jacobian, stored
        row-wise.

    qtf : float, shape (`n`), modified in place
        On exit: the vector Q^Tf.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    nfev : int
        The number of calls made to `fcnval`.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

        (`errno` 13)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 1 or 2.

        (`errno` 14)
            On entry, `factor` = *<value>*.

            Constraint: `factor` > 0.0.

        (`errno` 15)
            On entry, `mode` = 2 and `diag` contained a non-positive
            element.

        (`errno` 16)
            On entry, `ml` = *<value>*.

            Constraint: `ml` >= 0.

        (`errno` 17)
            On entry, `mu` = *<value>*.

            Constraint: `mu` >= 0.

        (`errno` 18)
            On entry, `maxfev` = *<value>*.

            Constraint: `maxfev` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There have been at least `maxfev` calls to `fcnval`:
            `maxfev` = *<value>*. Consider restarting the calculation
            from the final point held in `x`.

        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* Jacobian
            evaluations.

        (`errno` 5)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* iterations.

    NagCallbackTerminateWarning
        (`errno` 6)
            Termination requested in `fcnval` or `fcnmon`.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.sys_func_expert'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fvec[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Input/Output, iflag.
    )

    def cb_fcnh(_fcn, en_data, n, x, fvec, _iuser, _ruser, iflag):
        """Helper for fcnval and fcnmon."""
        try:
            _iflag_fcnh = utils._EngineIntScalarType.to_py(iflag.contents)
            if _iflag_fcnh > 0:
                _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
                _x_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    x,
                    (_n_fcnh,),
                )
                _fvec_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    fvec,
                    (_n_fcnh,),
                    can_reference_contents=False,
                )
                if data is not None:
                    fcnval(
                        _n_fcnh,
                        _x_fcnh,
                        _fvec_fcnh,
                        data,
                    )
                else:
                    fcnval(
                        _n_fcnh,
                        _x_fcnh,
                        _fvec_fcnh,
                    )
                utils._unlink_pointer(x)
                utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                    _fvec_fcnh, {'fun_name': fun_name + ':fcnval', 'returned': 'fvec'}, fvec,
                )
                utils._unlink_pointer(fvec)
            elif _iflag_fcnh == 0:
                if fcnmon is None:
                    return
                _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
                _x_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    x,
                    (_n_fcnh,),
                )
                _fvec_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    fvec,
                    (_n_fcnh,),
                )
                if data is not None:
                    fcnmon(
                        _n_fcnh,
                        _x_fcnh,
                        _fvec_fcnh,
                        data,
                    )
                else:
                    fcnmon(
                        _n_fcnh,
                        _x_fcnh,
                        _fvec_fcnh,
                    )
                utils._unlink_pointer(x)
                utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                    _fvec_fcnh, {'fun_name': fun_name + ':fcnmon', 'returned': 'fvec'}, fvec,
                )
                utils._unlink_pointer(fvec)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('c05qcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fvec[n].
        utils._BYREF_FLOAT64, # Input, xtol.
        utils._BYREF_INT, # Input, maxfev.
        utils._BYREF_INT, # Input, ml.
        utils._BYREF_INT, # Input, mu.
        utils._BYREF_FLOAT64, # Input, epsfcn.
        utils._BYREF_INT, # Input, mode.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, diag[n].
        utils._BYREF_FLOAT64, # Input, factor.
        utils._BYREF_INT, # Input, nprint.
        utils._BYREF_INT, # Output, nfev.
        utils._EngineFloat64ArrayType.c_type, # Output, fjac[ldfjac, n].
        utils._BYREF_INT, # Input, ldfjac.
        utils._EngineFloat64ArrayType.c_type, # Output, r[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, qtf[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((fjac, 'fjac'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    utils._check_callable(fcnval, {'fun_name': fun_name, 'entity': 'fcnval'})
    utils._check_callable(fcnmon, {'fun_name': fun_name, 'entity': 'fcnmon'}, allow_none=True)
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _fvec_cls = utils._EngineFloat64ArrayType(
        fvec, {'fun_name': fun_name, 'entity': 'fvec'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _fvec = _fvec_cls.to_ctypes(can_reference_contents=False)
    _xtol = utils._EngineFloat64ScalarType(
        xtol, {'fun_name': fun_name, 'entity': 'xtol'},
    ).to_ctypes()
    _maxfev = utils._EngineIntScalarType(
        maxfev, {'fun_name': fun_name, 'entity': 'maxfev'},
    ).to_ctypes()
    _ml = utils._EngineIntScalarType(
        ml, {'fun_name': fun_name, 'entity': 'ml'},
    ).to_ctypes()
    _mu = utils._EngineIntScalarType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
    ).to_ctypes()
    _epsfcn = utils._EngineFloat64ScalarType(
        epsfcn, {'fun_name': fun_name, 'entity': 'epsfcn'},
    ).to_ctypes()
    _mode = utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _diag_cls = utils._EngineFloat64ArrayType(
        diag, {'fun_name': fun_name, 'entity': 'diag'},
        exp_shape=(_n_py,),
    )
    _diag = _diag_cls.to_ctypes()
    _factor = utils._EngineFloat64ScalarType(
        factor, {'fun_name': fun_name, 'entity': 'factor'},
    ).to_ctypes()
    _nprint = utils._EngineIntScalarType(
        nprint, {'fun_name': fun_name, 'entity': 'nprint'},
    ).to_ctypes()
    _nfev = utils._EngineIntScalarType.empty_ctypes()
    _ldfjac_py = utils._EngineFloat64ArrayType.get_nd(fjac, {'fun_name': fun_name, 'entity': 'fjac'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldfjac = utils._EngineIntScalarType(
        _ldfjac_py, {'fun_name': fun_name, 'entity': '_ldfjac'},
    ).to_ctypes()
    _fjac_cls = utils._EngineFloat64ArrayType(
        fjac, {'fun_name': fun_name, 'entity': 'fjac'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _fjac = _fjac_cls.to_ctypes(can_reference_contents=False)
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _qtf_cls = utils._EngineFloat64ArrayType(
        qtf, {'fun_name': fun_name, 'entity': 'qtf'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _qtf = _qtf_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fcn,
        _fcnh,
        _n,
        _x,
        _fvec,
        _xtol,
        _maxfev,
        _ml,
        _mu,
        _epsfcn,
        _mode,
        _diag,
        _factor,
        _nprint,
        _nfev,
        _fjac,
        _ldfjac,
        _r,
        _qtf,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There have been at least maxfev calls to fcnval:',
                    'maxfev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Consider restarting the calculation from the final point held in x.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No further improvement in the solution is possible.',
                    'xtol is too small: xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress, as measured by the improvement',
                    'from the last {:s} Jacobian evaluations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99987:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress, as measured by the improvement',
                    'from the last {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99991:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in fcnval or fcnmon.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 11 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, factor = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: factor > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = 2 and diag contained a non-positive element.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ml = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ml >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mu = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mu >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxfev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxfev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _fvec_cls.refresh_py(_fvec)
        _diag_cls.refresh_py(_diag)
        _fjac_cls.refresh_py(_fjac)
        _r_cls.refresh_py(_r)
        _qtf_cls.refresh_py(_qtf)
        _nfev_py = utils._EngineIntScalarType.to_py(_nfev)
        return_data = _nfev_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nfev'],
            )(
                _nfev_py,
            )
    _err_class.manage()
    return return_data

def sys_func_rcomm(irevcm, n, x, fvec, xtol, ml, mu, epsfcn, mode, diag, factor, fjac, r, qtf, comm):
    u"""
    Solution of a system of nonlinear equations using function values
    only (reverse communication).

    For full information please refer to the NAG Library document for
    c05qd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05qdf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must have the value 0.

    n : int
        n, the number of equations.

    x : float, shape (`n`), modified in place
        On initial entry: an initial guess at the solution vector.

        On intermediate exit: contains the current point.

        On final exit: the final estimate of the solution vector.

    fvec : float, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` = 1, `fvec` must not be
        changed.

        If `irevcm` = 2, `fvec` must be set to the values of the
        functions computed at the current point `x`.

        On final exit: the function values at the final point, `x`.

    xtol : float
        On initial entry: the accuracy in `x` to which the solution is
        required.

    ml : int
        On initial entry: the number of subdiagonals within the band of
        the Jacobian matrix. (If the Jacobian is not banded, or you are
        unsure, set `ml` = `n`-1.)

    mu : int
        On initial entry: the number of superdiagonals within the band
        of the Jacobian matrix. (If the Jacobian is not banded, or you
        are unsure, set `mu` = `n`-1.)

    epsfcn : float
        On initial entry: the order of the largest relative error in the
        functions. It is used in determining a suitable step for a
        forward difference approximation to the Jacobian. If `epsfcn` is
        less than machine precision (returned by ``machine.precision``)
        then machine precision is used. Consequently a value of 0.0 will
        often be suitable.

    mode : int
        On initial entry: indicates whether or not you have provided
        scaling factors in `diag`.

        If `mode` = 2, the scaling must have been supplied in `diag`.

        Otherwise, if `mode` = 1, the variables will be scaled
        internally.

    diag : float, shape (`n`), modified in place
        On entry: if `mode` = 2, `diag` must contain multiplicative
        scale factors for the variables.

        If `mode` = 1, `diag` need not be set.

        On exit: the scale factors actually used (computed internally if
        `mode` = 1).

    factor : float
        On initial entry: a quantity to be used in determining the
        initial step bound. In most cases, `factor` should lie between
        0.1 and 100.0. (The step bound is `factor`*||`diag`*`x`||_2 if
        this is nonzero; otherwise the bound is `factor`.)

    fjac : float, shape (`n`, `n`), modified in place
        On initial entry: need not be set.

        On intermediate exit: must not be changed.

        On final exit: the orthogonal matrix Q produced by the QR
        factorization of the final approximate Jacobian.

    r : float, shape (`n`*(`n`+1)/2), modified in place
        On initial entry: need not be set.

        On intermediate exit: must not be changed.

        On final exit: the upper triangular matrix R produced by the QR
        factorization of the final approximate Jacobian, stored
        row-wise.

    qtf : float, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate exit: must not be changed.

        On final exit: the vector Q^Tf.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    irevcm : int
        On intermediate exit: specifies what action you must take before
        re-entering ``sys_func_rcomm`` with `irevcm` set to this value.
        The value of this argument should be interpreted as follows:

        `irevcm` = 1

          Indicates the start of a new iteration. No action is required
          by you, but `x` and `fvec` are available for printing.

        `irevcm` = 2

          Indicates that before re-entry to ``sys_func_rcomm``, `fvec`
          must contain the function values f_i(x).

        On final exit: `irevcm` = 0 and the algorithm has terminated.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0, 1 or 2.

        (`errno` 11)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

        (`errno` 13)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 1 or 2.

        (`errno` 14)
            On entry, `factor` = *<value>*.

            Constraint: `factor` > 0.0.

        (`errno` 15)
            On entry, `mode` = 2 and `diag` contained a non-positive
            element.

        (`errno` 16)
            On entry, `ml` = *<value>*.

            Constraint: `ml` >= 0.

        (`errno` 17)
            On entry, `mu` = *<value>*.

            Constraint: `mu` >= 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* Jacobian
            evaluations.

        (`errno` 5)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* iterations.

    See Also
    --------
    :meth:`naginterfaces.library.examples.roots.sys_func_rcomm_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.sys_func_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c05qdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fvec[n].
        utils._BYREF_FLOAT64, # Input, xtol.
        utils._BYREF_INT, # Input, ml.
        utils._BYREF_INT, # Input, mu.
        utils._BYREF_FLOAT64, # Input, epsfcn.
        utils._BYREF_INT, # Input, mode.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, diag[n].
        utils._BYREF_FLOAT64, # Input, factor.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fjac[n, n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, r[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, qtf[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[17].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[4*n+10].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((fjac, 'fjac'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _fvec_cls = utils._EngineFloat64ArrayType(
        fvec, {'fun_name': fun_name, 'entity': 'fvec'},
        exp_shape=(_n_py,),
        check_contents=_irevcm_py != 0,
    )
    _fvec = _fvec_cls.to_ctypes(can_reference_contents=_irevcm_py != 0)
    _xtol = utils._EngineFloat64ScalarType(
        xtol, {'fun_name': fun_name, 'entity': 'xtol'},
    ).to_ctypes()
    _ml = utils._EngineIntScalarType(
        ml, {'fun_name': fun_name, 'entity': 'ml'},
    ).to_ctypes()
    _mu = utils._EngineIntScalarType(
        mu, {'fun_name': fun_name, 'entity': 'mu'},
    ).to_ctypes()
    _epsfcn = utils._EngineFloat64ScalarType(
        epsfcn, {'fun_name': fun_name, 'entity': 'epsfcn'},
    ).to_ctypes()
    _mode = utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _diag_cls = utils._EngineFloat64ArrayType(
        diag, {'fun_name': fun_name, 'entity': 'diag'},
        exp_shape=(_n_py,),
    )
    _diag = _diag_cls.to_ctypes()
    _factor = utils._EngineFloat64ScalarType(
        factor, {'fun_name': fun_name, 'entity': 'factor'},
    ).to_ctypes()
    _fjac_cls = utils._EngineFloat64ArrayType(
        fjac, {'fun_name': fun_name, 'entity': 'fjac'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=_irevcm_py != 0,
    )
    _fjac = _fjac_cls.to_ctypes(can_reference_contents=_irevcm_py != 0)
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=_irevcm_py != 0,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=_irevcm_py != 0)
    _qtf_cls = utils._EngineFloat64ArrayType(
        qtf, {'fun_name': fun_name, 'entity': 'qtf'},
        exp_shape=(_n_py,),
        check_contents=_irevcm_py != 0,
    )
    _qtf = _qtf_cls.to_ctypes(can_reference_contents=_irevcm_py != 0)
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav', 'rwsav'),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(17)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(4*_n_py+10)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _n,
        _x,
        _fvec,
        _xtol,
        _ml,
        _mu,
        _epsfcn,
        _mode,
        _diag,
        _factor,
        _fjac,
        _r,
        _qtf,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0, 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No further improvement in the solution is possible.',
                    'xtol is too small: xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress, as measured by the improvement',
                    'from the last {:s} Jacobian evaluations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99988:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress, as measured by the improvement',
                    'from the last {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, factor = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: factor > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99990:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = 2 and diag contained a non-positive element.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 16 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ml = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ml >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 17 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mu = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mu >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _fvec_cls.refresh_py(_fvec)
        _diag_cls.refresh_py(_diag)
        _fjac_cls.refresh_py(_fjac)
        _r_cls.refresh_py(_r)
        _qtf_cls.refresh_py(_qtf)
        return_data = _irevcm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['irevcm'],
            )(
                _irevcm_py,
            )
    _err_class.manage()
    return return_data

def sparsys_func_easy(fcn, n, x, fvec, xtol, nnz, comm, data=None):
    u"""
    Solution of a sparse system of nonlinear equations using function
    values only (easy-to-use).

    For full information please refer to the NAG Library document for
    c05qs

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05qsf.html

    Parameters
    ----------
    fcn : callable fcn(n, lindf, indf, x, fvec, data=None)
        `fcn` must return the values of the functions f_i at a point x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        lindf : int
            `lindf` specifies the number of indices i for which values
            of f_i(x) must be computed.

        indf : int, ndarray, shape (`lindf`)
            Note: this argument represents array indices; the values
            supplied will be base-1.

            This argument specifies the indices i for which values of
            f_i(x) must be computed. The indices are specified in
            strictly ascending order.

        x : float, ndarray, shape (`n`)
            The components of the point x at which the functions must be
            evaluated. `x`[i-1] contains the coordinate x_i.

        fvec : float, ndarray, shape (`n`), to be modified in place
            On exit: `fvec`[i-1] must contain the function values
            f_i(x), for all indices i in `indf`.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    n : int
        n, the number of equations.

    x : float, shape (`n`), modified in place
        On entry: an initial guess at the solution vector. `x`[i-1] must
        contain the coordinate x_i.

        On exit: the final estimate of the solution vector.

    fvec : float, shape (`n`), modified in place
        On exit: the function values at the final point returned in `x`.
        `fvec`[i-1] contains the function values f_i.

    xtol : float
        The accuracy in `x` to which the solution is required.

    nnz : int
        `nnz` should be an upper estimate of the number of nonzero
        entries found in the Jacobian.

        The size of the data that is self communicated to this routine
        via `comm` is a function of `nnz`.

        A suitable overestimate for `nnz` is `n`^2, but this may be
        suboptimal when `n` is large.

        If an existing sparsity pattern for the problem is communicated
        via `comm`, `nnz` will be ignored.

        The value that was supplied in the call to ``sparsys_func_easy``
        which computed that sparsity pattern will be extracted from
        `comm` and used instead.

    comm : None or dict, communication object, modified in place
        Optionally, on entry: an existing sparsity pattern for the
        Jacobian, computed by a subsequent call. Otherwise, supplying an
        uninitialized object indicates it is the first time
        ``sparsys_func_easy`` is called for this specific problem.

        On exit, if not None on entry: will be populated with the
        sparsity pattern of the Jacobian. On subsequent calls, the same
        communication object can be passed in again if the problem has a
        Jacobian of the same sparsity pattern.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    actual_nnz : int
        The number of nonzero entries found in the Jacobian.

    Raises
    ------
    NagValueError
        (`errno` 6)
            On entry, the supplied `nnz` = *<value>* was too small for
            this problem.

        (`errno` 7)
            On entry, the supplied `nnz` = *<value>* was too small for
            this problem.

        (`errno` 9)
            An internal error has occurred. Code = *<value>*.

        (`errno` 11)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There have been at least 200*(`n`+1) calls to `fcn`.
            Consider restarting the calculation from the point held in
            `x` and supplying the populated sparsity pattern from
            `comm`.

        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress. This failure exit
            may indicate that the system does not have a zero, or that
            the solution is very close to the origin (see [Accuracy]).
            Otherwise, rerunning ``sparsys_func_easy`` from a different
            starting point may avoid the region of difficulty. The
            condition number of the Jacobian is *<value>*.

    NagCallbackTerminateWarning
        (`errno` 5)
            Termination requested in `fcn`.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.sparsys_func_easy'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    cb_fcn_name = fun_name + ':fcn'

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, lindf.
        utils._EngineIntArrayType.c_type, # Input, indf[lindf].
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fvec[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Input/Output, iflag.
    )

    def cb_fcnh(_fcn, en_data, n, lindf, indf, x, fvec, _iuser, _ruser, iflag):
        """Helper for fcn."""
        try:
            _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
            _lindf_fcnh = utils._EngineIntScalarType.to_py(lindf.contents)
            _indf_fcnh = utils._EngineIntArrayType.from_pointer(
                indf,
                (_lindf_fcnh,),
            )
            _x_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                x,
                (_n_fcnh,),
            )
            _fvec_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                fvec,
                (_n_fcnh,),
                can_reference_contents=False,
            )
            if data is not None:
                fcn(
                    _n_fcnh,
                    _lindf_fcnh,
                    _indf_fcnh,
                    _x_fcnh,
                    _fvec_fcnh,
                    data,
                )
            else:
                fcn(
                    _n_fcnh,
                    _lindf_fcnh,
                    _indf_fcnh,
                    _x_fcnh,
                    _fvec_fcnh,
                )
            utils._unlink_pointer(indf)
            utils._unlink_pointer(x)
            utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                _fvec_fcnh, {'fun_name': cb_fcn_name, 'returned': 'fvec'}, fvec,
            )
            utils._unlink_pointer(fvec)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('c05qsft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fvec[n].
        utils._BYREF_FLOAT64, # Input, xtol.
        utils._BYREF_LOGICAL, # Input, init.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    utils._check_callable(fcn, {'fun_name': fun_name, 'entity': 'fcn'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _fvec_cls = utils._EngineFloat64ArrayType(
        fvec, {'fun_name': fun_name, 'entity': 'fvec'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _fvec = _fvec_cls.to_ctypes(can_reference_contents=False)
    _xtol = utils._EngineFloat64ScalarType(
        xtol, {'fun_name': fun_name, 'entity': 'xtol'},
    ).to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('rcomm', 'icomm'),
        allow_none=True,
    )
    if comm is None or _comm_needs_init:
        _init = utils._EngineLogicalScalarType(
            True, {'fun_name': fun_name, 'entity': '_init'},
        ).to_ctypes()
    else:
        _init = utils._EngineLogicalScalarType(
            False, {'fun_name': fun_name, 'entity': '_init'},
        ).to_ctypes()
    _init_py = utils._EngineLogicalScalarType.to_py(_init)
    _nnz = utils._EngineIntScalarType(
        nnz, {'fun_name': fun_name, 'entity': 'nnz'},
    ).to_ctypes()
    _nnz_py = utils._EngineIntScalarType.to_py(_nnz)
    if _comm_needs_init:
        if not _init_py:
            _rcomm_dim1 = 12+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (0,), validate=False)
        else:
            _rcomm_dim1 = 12+_nnz_py
        _rcomm = utils._EngineFloat64ArrayType.empty_ctypes(_rcomm_dim1)
        if comm is not None:
            comm['rcomm'] = _rcomm
    else:
        _rcomm = comm['rcomm']
    _lrcomm_py = utils._EngineFloat64ArrayType.get_shape(_rcomm, {'fun_name': fun_name, 'entity': '_rcomm'}, exp_rank=1, validate=False)[0]
    _lrcomm = utils._EngineIntScalarType(
        _lrcomm_py, {'fun_name': fun_name, 'entity': '_lrcomm'},
    ).to_ctypes()
    if _comm_needs_init:
        if not _init_py:
            _icomm_dim1 = 8*_n_py+19+utils._EngineIntArrayType.get_element(comm['icomm'], {'fun_name': fun_name, 'entity': 'comm[\'icomm\']'}, (0,), validate=False)
        else:
            _icomm_dim1 = 8*_n_py+19+_nnz_py
        _icomm = utils._EngineIntArrayType.empty_ctypes(_icomm_dim1)
        if comm is not None:
            comm['icomm'] = _icomm
    else:
        _icomm = comm['icomm']
    _licomm_py = utils._EngineIntArrayType.get_shape(_icomm, {'fun_name': fun_name, 'entity': '_icomm'}, exp_rank=1, validate=False)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fcn,
        _fcnh,
        _n,
        _x,
        _fvec,
        _xtol,
        _init,
        _rcomm,
        _lrcomm,
        _icomm,
        _licomm,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 201:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There have been at least 200*(n+1) calls to fcn.',
                    'Consider restarting the calculation from the',
                    'point held in x and supplying the populated sparsity pattern from comm.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 301:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No further improvement in the solution is possible.',
                    'xtol is too small: xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 401:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress.',
                    'This failure exit may indicate that the system does not have a zero, or',
                    'that the solution is very close to the origin',
                    '(see [Accuracy]).',
                    'Otherwise, rerunning sparsys_func_easy from a different starting point may avoid',
                    'the region of difficulty.',
                    'The condition number of the Jacobian is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 501:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in fcn.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 6 and _ifmt == 601:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the supplied nnz = {:s} was too small for this problem.'.format(
                        utils._STR_FMT_INT.format(_nnz_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 701:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the supplied nnz = {:s} was too small for this problem.'.format(
                        utils._STR_FMT_INT.format(_nnz_py),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 901:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'An internal error has occurred. Code = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 102:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _fvec_cls.refresh_py(_fvec)
        _actual_nnz_py = utils._EngineIntArrayType.get_element(_icomm, {'fun_name': fun_name, 'entity': '_icomm'}, (0,), validate=False)
        return_data = _actual_nnz_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['actual_nnz'],
            )(
                _actual_nnz_py,
            )
    _err_class.manage()
    return return_data

def sys_deriv_easy(fcnval, fcnjac, n, x, fvec, fjac, xtol, data=None):
    u"""
    Solution of a system of nonlinear equations using first derivatives
    (easy-to-use).

    For full information please refer to the NAG Library document for
    c05rb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05rbf.html

    Parameters
    ----------
    fcnval : callable fcnval(n, x, fvec, data=None)
        `fcnval` must return the values of the functions f_i at a point
        x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float, ndarray, shape (`n`)
            The components of the point x at which the functions must be
            evaluated.

        fvec : float, ndarray, shape (`n`), to be modified in place
            On exit: this argument must contain the function values
            f_i(x).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    fcnjac : callable fcnjac(n, x, fjac, data=None)
        `fcnjac` must return the Jacobian at x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float, ndarray, shape (`n`)
            The components of the point x at which the Jacobian must be
            evaluated.

        fjac : float, ndarray, shape (`n`, `n`), to be modified in place
            On exit: `fjac`[i-1,j-1] must contain the value of df_i/dx_j
            at the point x, for j = 1,2,...,n, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    n : int
        n, the number of equations.

    x : float, shape (`n`), modified in place
        On entry: an initial guess at the solution vector.

        On exit: the final estimate of the solution vector.

    fvec : float, shape (`n`), modified in place
        On exit: the function values at the final point returned in `x`.

    fjac : float, shape (`n`, `n`), modified in place
        On exit: the orthogonal matrix Q produced by the QR
        factorization of the final approximate Jacobian.

    xtol : float
        The accuracy in `x` to which the solution is required.

    data : arbitrary, optional
        User-communication data for callback functions.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There have been at least 100*(`n`+1) calls to `fcnval` and
            `fcnjac` combined. Consider restarting the calculation from
            the point held in `x`.

        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress. This failure exit
            may indicate that the system does not have a zero, or that
            the solution is very close to the origin (see [Accuracy]).
            Otherwise, rerunning ``sys_deriv_easy`` from a different
            starting point may avoid the region of difficulty.

    NagCallbackTerminateWarning
        (`errno` 5)
            Termination requested in `fcnval` or `fcnjac`.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.sys_deriv_easy'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fvec[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fjac[ldfjac, n].
        utils._BYREF_INT, # Input, ldfjac.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Input/Output, iflag.
    )

    def cb_fcnh(_fcn, en_data, n, x, fvec, fjac, ldfjac, _iuser, _ruser, iflag):
        """Helper for fcnval and fcnjac."""
        try:
            _iflag_fcnh = utils._EngineIntScalarType.to_py(iflag.contents)
            if _iflag_fcnh == 1:
                _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
                _x_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    x,
                    (_n_fcnh,),
                )
                _fvec_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    fvec,
                    (_n_fcnh,),
                    can_reference_contents=False,
                )
                if data is not None:
                    fcnval(
                        _n_fcnh,
                        _x_fcnh,
                        _fvec_fcnh,
                        data,
                    )
                else:
                    fcnval(
                        _n_fcnh,
                        _x_fcnh,
                        _fvec_fcnh,
                    )
                utils._unlink_pointer(x)
                utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                    _fvec_fcnh, {'fun_name': fun_name + ':fcnval', 'returned': 'fvec'}, fvec,
                )
                utils._unlink_pointer(fvec)
            elif _iflag_fcnh == 2:
                _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
                _x_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    x,
                    (_n_fcnh,),
                )
                _fjac_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    fjac,
                    (_n_fcnh, _n_fcnh),
                    can_reference_contents=False,
                    sorder=_m_sorder,
                )
                if data is not None:
                    fcnjac(
                        _n_fcnh,
                        _x_fcnh,
                        _fjac_fcnh,
                        data,
                    )
                else:
                    fcnjac(
                        _n_fcnh,
                        _x_fcnh,
                        _fjac_fcnh,
                    )
                utils._unlink_pointer(x)
                utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                    _fjac_fcnh, {'fun_name': fun_name + ':fcnjac', 'returned': 'fjac'}, fjac,
                )
                utils._unlink_pointer(fjac)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('c05rbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fvec[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fjac[ldfjac, n].
        utils._BYREF_INT, # Input, ldfjac.
        utils._BYREF_FLOAT64, # Input, xtol.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((fjac, 'fjac'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    utils._check_callable(fcnval, {'fun_name': fun_name, 'entity': 'fcnval'})
    utils._check_callable(fcnjac, {'fun_name': fun_name, 'entity': 'fcnjac'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _fvec_cls = utils._EngineFloat64ArrayType(
        fvec, {'fun_name': fun_name, 'entity': 'fvec'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _fvec = _fvec_cls.to_ctypes(can_reference_contents=False)
    _ldfjac_py = utils._EngineFloat64ArrayType.get_nd(fjac, {'fun_name': fun_name, 'entity': 'fjac'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldfjac = utils._EngineIntScalarType(
        _ldfjac_py, {'fun_name': fun_name, 'entity': '_ldfjac'},
    ).to_ctypes()
    _fjac_cls = utils._EngineFloat64ArrayType(
        fjac, {'fun_name': fun_name, 'entity': 'fjac'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _fjac = _fjac_cls.to_ctypes(can_reference_contents=False)
    _xtol = utils._EngineFloat64ScalarType(
        xtol, {'fun_name': fun_name, 'entity': 'xtol'},
    ).to_ctypes()
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fcn,
        _fcnh,
        _n,
        _x,
        _fvec,
        _fjac,
        _ldfjac,
        _xtol,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There have been at least 100*(n+1) calls to fcnval and fcnjac combined.',
                    'Consider restarting the calculation from the point held in x.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No further improvement in the solution is possible.',
                    'xtol is too small: xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress.',
                    'This failure exit may indicate that the system does not have a zero, or',
                    'that the solution is very close to the origin',
                    '(see [Accuracy]).',
                    'Otherwise, rerunning sys_deriv_easy from a different starting point may avoid',
                    'the region of difficulty.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in fcnval or fcnjac.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 11 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _fvec_cls.refresh_py(_fvec)
        _fjac_cls.refresh_py(_fjac)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def sys_deriv_expert(fcnval, fcnjac, fcnmon, n, x, fvec, fjac, xtol, maxfev, mode, diag, factor, nprint, r, qtf, data=None):
    u"""
    Solution of a system of nonlinear equations using first derivatives
    (comprehensive).

    For full information please refer to the NAG Library document for
    c05rc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05rcf.html

    Parameters
    ----------
    fcnval : callable fcnval(n, x, fvec, data=None)
        `fcnval` must return the values of the functions f_i at a point
        x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float, ndarray, shape (`n`)
            The components of the point x at which the functions must be
            evaluated.

        fvec : float, ndarray, shape (`n`), to be modified in place
            On exit: this argument must contain the function values
            f_i(x).

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    fcnjac : callable fcnjac(n, x, fjac, data=None)
        `fcnjac` must return the Jacobian at x.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float, ndarray, shape (`n`)
            The components of the point x at which the Jacobian must be
            evaluated.

        fjac : float, ndarray, shape (`n`, `n`), to be modified in place
            On exit: `fjac`[i-1,j-1] must contain the value of df_i/dx_j
            at the point x, for j = 1,2,...,n, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    fcnmon : callable fcnmon(n, x, fvec, fjac, data=None)
        `fcnmon` may be used to monitor the progress of the algorithm.

        Parameters
        ~~~~~~~~~~
        n : int
            n, the number of equations.

        x : float, ndarray, shape (`n`)
            The components of the current evaluation point x.

        fvec : float, ndarray, shape (`n`)
            This argument contains the function values f_i(x).

        fjac : float, ndarray, shape (`n`, `n`)
            `fjac`[i-1,j-1] contains the value of df_i/dx_j at the point
            x, for j = 1,2,...,n, for i = 1,2,...,n.

        data : arbitrary, optional, modifiable in place
            User-communication data for callback functions.

    n : int
        n, the number of equations.

    x : float, shape (`n`), modified in place
        On entry: an initial guess at the solution vector.

        On exit: the final estimate of the solution vector.

    fvec : float, shape (`n`), modified in place
        On exit: the function values at the final point returned in `x`.

    fjac : float, shape (`n`, `n`), modified in place
        On exit: the orthogonal matrix Q produced by the QR
        factorization of the final approximate Jacobian.

    xtol : float
        The accuracy in `x` to which the solution is required.

    maxfev : int
        The maximum number of calls to `fcnval` and `fcnjac` combined.
        ``sys_deriv_expert`` will exit with `errno` = 2, if, at the end
        of an iteration, the number of calls to `fcnval` and `fcnjac`
        combined exceeds `maxfev`.

    mode : int
        Indicates whether or not you have provided scaling factors in
        `diag`.

        If `mode` = 2, the scaling must have been specified in `diag`.

        Otherwise, if `mode` = 1, the variables will be scaled
        internally.

    diag : float, shape (`n`), modified in place
        On entry: if `mode` = 2, `diag` must contain multiplicative
        scale factors for the variables.

        If `mode` = 1, `diag` need not be set.

        On exit: the scale factors actually used (computed internally if
        `mode` = 1).

    factor : float
        A quantity to be used in determining the initial step bound. In
        most cases, `factor` should lie between 0.1 and 100.0. (The step
        bound is `factor`*||`diag`*`x`||_2 if this is nonzero; otherwise
        the bound is `factor`.)

    nprint : int
        Indicates whether (and how often) calls to `fcnmon` are to be
        made.

        `nprint` <= 0

          No calls are made.

        `nprint` > 0

          `fcnmon` is called at the beginning of the first iteration,
          every `nprint` iterations thereafter and immediately before
          the return from ``sys_deriv_expert``.

    r : float, shape (`n`*(`n`+1)/2), modified in place
        On exit: the upper triangular matrix R produced by the QR
        factorization of the final approximate Jacobian, stored
        row-wise.

    qtf : float, shape (`n`), modified in place
        On exit: the vector Q^Tf.

    data : arbitrary, optional
        User-communication data for callback functions.

    Returns
    -------
    nfev : int
        The number of calls made to `fcnval` and `fcnjac` combined to
        evaluate the functions.

    njev : int
        The number of calls made to `fcnjac`.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

        (`errno` 13)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 1 or 2.

        (`errno` 14)
            On entry, `factor` = *<value>*.

            Constraint: `factor` > 0.0.

        (`errno` 15)
            On entry, `mode` = 2 and `diag` contained a non-positive
            element.

        (`errno` 18)
            On entry, `maxfev` = *<value>*.

            Constraint: `maxfev` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There have been at least `maxfev` calls to `fcnval` and
            `fcnjac` combined: `maxfev` = *<value>*. Consider restarting
            the calculation from the final point held in `x`.

        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* Jacobian
            evaluations. This failure exit may indicate that the system
            does not have a zero, or that the solution is very close to
            the origin (see [Accuracy]). Otherwise, rerunning
            ``sys_deriv_expert`` from a different starting point may
            avoid the region of difficulty.

        (`errno` 5)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* iterations. This
            failure exit may indicate that the system does not have a
            zero, or that the solution is very close to the origin (see
            [Accuracy]). Otherwise, rerunning ``sys_deriv_expert`` from
            a different starting point may avoid the region of
            difficulty.

    NagCallbackTerminateWarning
        (`errno` 6)
            Termination requested in `fcnval`, `fcnjac` or `fcnmon`.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.sys_deriv_expert'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    fcnh = utils._EngineState._sys_functype(
        None,
        utils._UNUSED_FUNCTYPE,
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fvec[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fjac[ldfjac, n].
        utils._BYREF_INT, # Input, ldfjac.
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._BYREF_INT, # Input/Output, iflag.
    )

    def cb_fcnh(_fcn, en_data, n, x, fvec, fjac, ldfjac, _iuser, _ruser, iflag):
        """Helper for fcnval and fcnjac and fcnmon."""
        try:
            _iflag_fcnh = utils._EngineIntScalarType.to_py(iflag.contents)
            if _iflag_fcnh == 1:
                _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
                _x_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    x,
                    (_n_fcnh,),
                )
                _fvec_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    fvec,
                    (_n_fcnh,),
                    can_reference_contents=False,
                )
                if data is not None:
                    fcnval(
                        _n_fcnh,
                        _x_fcnh,
                        _fvec_fcnh,
                        data,
                    )
                else:
                    fcnval(
                        _n_fcnh,
                        _x_fcnh,
                        _fvec_fcnh,
                    )
                utils._unlink_pointer(x)
                utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                    _fvec_fcnh, {'fun_name': fun_name + ':fcnval', 'returned': 'fvec'}, fvec,
                )
                utils._unlink_pointer(fvec)
            elif _iflag_fcnh == 2:
                _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
                _x_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    x,
                    (_n_fcnh,),
                )
                _fjac_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    fjac,
                    (_n_fcnh, _n_fcnh),
                    can_reference_contents=False,
                    sorder=_m_sorder,
                )
                if data is not None:
                    fcnjac(
                        _n_fcnh,
                        _x_fcnh,
                        _fjac_fcnh,
                        data,
                    )
                else:
                    fcnjac(
                        _n_fcnh,
                        _x_fcnh,
                        _fjac_fcnh,
                    )
                utils._unlink_pointer(x)
                utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                    _fjac_fcnh, {'fun_name': fun_name + ':fcnjac', 'returned': 'fjac'}, fjac,
                )
                utils._unlink_pointer(fjac)
            elif _iflag_fcnh == 0:
                _n_fcnh = utils._EngineIntScalarType.to_py(n.contents)
                _x_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    x,
                    (_n_fcnh,),
                )
                _fvec_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    fvec,
                    (_n_fcnh,),
                )
                _fjac_fcnh = utils._EngineFloat64ArrayType.from_pointer(
                    fjac,
                    (_n_fcnh, _n_fcnh),
                    sorder=_m_sorder,
                )
                if data is not None:
                    fcnmon(
                        _n_fcnh,
                        _x_fcnh,
                        _fvec_fcnh,
                        _fjac_fcnh,
                        data,
                    )
                else:
                    fcnmon(
                        _n_fcnh,
                        _x_fcnh,
                        _fvec_fcnh,
                        _fjac_fcnh,
                    )
                utils._unlink_pointer(x)
                utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                    _fvec_fcnh, {'fun_name': fun_name + ':fcnmon', 'returned': 'fvec'}, fvec,
                )
                utils._unlink_pointer(fvec)
                utils._EngineFloat64ArrayType.refresh_ctypes_buffer(
                    _fjac_fcnh, {'fun_name': fun_name + ':fcnmon', 'returned': 'fjac'}, fjac,
                )
                utils._unlink_pointer(fjac)
        except:
            excs.append(_sys.exc_info())
            iflag.contents.value = -1
            if excs[-1][0] is not utils.UserCallbackTerminate:
                en_data.contents.hlperr = utils._IERR_HELPER

    engine_ctypes = utils._EngineState._get_symbol('c05rcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._UNUSED_FUNCTYPE,
        fcnh,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fvec[n].
        utils._EngineFloat64ArrayType.c_type, # Output, fjac[ldfjac, n].
        utils._BYREF_INT, # Input, ldfjac.
        utils._BYREF_FLOAT64, # Input, xtol.
        utils._BYREF_INT, # Input, maxfev.
        utils._BYREF_INT, # Input, mode.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, diag[n].
        utils._BYREF_FLOAT64, # Input, factor.
        utils._BYREF_INT, # Input, nprint.
        utils._BYREF_INT, # Output, nfev.
        utils._BYREF_INT, # Output, njev.
        utils._EngineFloat64ArrayType.c_type, # Output, r[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Output, qtf[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iuser[*].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, ruser[*].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((fjac, 'fjac'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    utils._check_callable(fcnval, {'fun_name': fun_name, 'entity': 'fcnval'})
    utils._check_callable(fcnjac, {'fun_name': fun_name, 'entity': 'fcnjac'})
    utils._check_callable(fcnmon, {'fun_name': fun_name, 'entity': 'fcnmon'})
    _fcn = utils._UNUSED_CALLBACK
    _fcnh = utils._cast_callback(
        fcnh, cb_fcnh,
        {'fun_name': fun_name, 'entity': 'fcn'},
    )
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _fvec_cls = utils._EngineFloat64ArrayType(
        fvec, {'fun_name': fun_name, 'entity': 'fvec'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _fvec = _fvec_cls.to_ctypes(can_reference_contents=False)
    _ldfjac_py = utils._EngineFloat64ArrayType.get_nd(fjac, {'fun_name': fun_name, 'entity': 'fjac'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldfjac = utils._EngineIntScalarType(
        _ldfjac_py, {'fun_name': fun_name, 'entity': '_ldfjac'},
    ).to_ctypes()
    _fjac_cls = utils._EngineFloat64ArrayType(
        fjac, {'fun_name': fun_name, 'entity': 'fjac'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _fjac = _fjac_cls.to_ctypes(can_reference_contents=False)
    _xtol = utils._EngineFloat64ScalarType(
        xtol, {'fun_name': fun_name, 'entity': 'xtol'},
    ).to_ctypes()
    _maxfev = utils._EngineIntScalarType(
        maxfev, {'fun_name': fun_name, 'entity': 'maxfev'},
    ).to_ctypes()
    _mode = utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _diag_cls = utils._EngineFloat64ArrayType(
        diag, {'fun_name': fun_name, 'entity': 'diag'},
        exp_shape=(_n_py,),
    )
    _diag = _diag_cls.to_ctypes()
    _factor = utils._EngineFloat64ScalarType(
        factor, {'fun_name': fun_name, 'entity': 'factor'},
    ).to_ctypes()
    _nprint = utils._EngineIntScalarType(
        nprint, {'fun_name': fun_name, 'entity': 'nprint'},
    ).to_ctypes()
    _nfev = utils._EngineIntScalarType.empty_ctypes()
    _njev = utils._EngineIntScalarType.empty_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _qtf_cls = utils._EngineFloat64ArrayType(
        qtf, {'fun_name': fun_name, 'entity': 'qtf'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _qtf = _qtf_cls.to_ctypes(can_reference_contents=False)
    _iuser = None
    _ruser = None
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _fcn,
        _fcnh,
        _n,
        _x,
        _fvec,
        _fjac,
        _ldfjac,
        _xtol,
        _maxfev,
        _mode,
        _diag,
        _factor,
        _nprint,
        _nfev,
        _njev,
        _r,
        _qtf,
        _iuser,
        _ruser,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There have been at least maxfev calls to fcnval and fcnjac combined:',
                    'maxfev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Consider restarting the calculation from the final point held in x.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No further improvement in the solution is possible.',
                    'xtol is too small: xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress, as measured by the improvement',
                    'from the last {:s} Jacobian evaluations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'This failure exit may indicate that the system does not have a zero, or',
                    'that the solution is very close to the origin',
                    '(see [Accuracy]).',
                    'Otherwise, rerunning sys_deriv_expert from a different starting point may avoid',
                    'the region of difficulty.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99989:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress,',
                    'as measured by the improvement from the last {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'iterations.',
                    'This failure exit may indicate that the system does not have a zero, or',
                    'that the solution is very close to the origin',
                    '(see [Accuracy]).',
                    'Otherwise, rerunning sys_deriv_expert from a different starting point may avoid',
                    'the region of difficulty.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99993:
            _err_class = utils._UserCallbackExit(
                fun_name=fun_name,
                msg_lines=[
                    'Termination requested in fcnval, fcnjac or fcnmon.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
                cb_excs=excs,
            )
        elif _ierr_py == 11 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, factor = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: factor > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = 2 and diag contained a non-positive element.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 18 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxfev = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxfev > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _fvec_cls.refresh_py(_fvec)
        _fjac_cls.refresh_py(_fjac)
        _diag_cls.refresh_py(_diag)
        _r_cls.refresh_py(_r)
        _qtf_cls.refresh_py(_qtf)
        _nfev_py = utils._EngineIntScalarType.to_py(_nfev)
        _njev_py = utils._EngineIntScalarType.to_py(_njev)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['nfev', 'njev'],
        )(
            _nfev_py,
            _njev_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def sys_deriv_rcomm(irevcm, n, x, fvec, fjac, xtol, mode, diag, factor, r, qtf, comm):
    u"""
    Solution of a system of nonlinear equations using first derivatives
    (reverse communication).

    For full information please refer to the NAG Library document for
    c05rd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05rdf.html

    Parameters
    ----------
    irevcm : int
        On initial entry: must have the value 0.

    n : int
        n, the number of equations.

    x : float, shape (`n`), modified in place
        On initial entry: an initial guess at the solution vector.

        On intermediate exit: contains the current point.

        On final exit: the final estimate of the solution vector.

    fvec : float, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` != 2, `fvec` must not be
        changed.

        If `irevcm` = 2, `fvec` must be set to the values of the
        functions computed at the current point `x`.

        On final exit: the function values at the final point, `x`.

    fjac : float, shape (`n`, `n`), modified in place
        On initial entry: need not be set.

        On intermediate entry: if `irevcm` != 3, `fjac` must not be
        changed.

        If `irevcm` = 3, `fjac`[i-1,j-1] must contain the value of
        df_i/dx_j at the point x, for j = 1,2,...,n, for i = 1,2,...,n.

        On final exit: the orthogonal matrix Q produced by the QR
        factorization of the final approximate Jacobian.

    xtol : float
        On initial entry: the accuracy in `x` to which the solution is
        required.

    mode : int
        On initial entry: indicates whether or not you have provided
        scaling factors in `diag`.

        If `mode` = 2, the scaling must have been supplied in `diag`.

        Otherwise, if `mode` = 1, the variables will be scaled
        internally.

    diag : float, shape (`n`), modified in place
        On initial entry: if `mode` = 2, `diag` must contain
        multiplicative scale factors for the variables.

        If `mode` = 1, `diag` need not be set.

        On intermediate exit: `diag` must not be changed.

        On final exit: the scale factors actually used (computed
        internally if `mode` = 1).

    factor : float
        On initial entry: a quantity to be used in determining the
        initial step bound. In most cases, `factor` should lie between
        0.1 and 100.0. (The step bound is `factor`*||`diag`*`x`||_2 if
        this is nonzero; otherwise the bound is `factor`.)

    r : float, shape (`n`*(`n`+1)/2), modified in place
        On initial entry: need not be set.

        On intermediate exit: must not be changed.

        On final exit: the upper triangular matrix R produced by the QR
        factorization of the final approximate Jacobian, stored
        row-wise.

    qtf : float, shape (`n`), modified in place
        On initial entry: need not be set.

        On intermediate exit: must not be changed.

        On final exit: the vector Q^Tf.

    comm : dict, communication object, modified in place
        Communication structure.

        On initial entry: need not be set.

    Returns
    -------
    irevcm : int
        On intermediate exit: specifies what action you must take before
        re-entering ``sys_deriv_rcomm`` with `irevcm` set to this value.
        The value of this argument should be interpreted as follows:

        `irevcm` = 1

          Indicates the start of a new iteration. No action is required
          by you, but `x` and `fvec` are available for printing.

        `irevcm` = 2

          Indicates that before re-entry to ``sys_deriv_rcomm``, `fvec`
          must contain the function values f_i(x).

        `irevcm` = 3

          Indicates that before re-entry to ``sys_deriv_rcomm``,
          `fjac`[i-1,j-1] must contain the value of df_i/dx_j at the
          point x, for j = 1,2,...,n, for i = 1,2,...,n.

        On final exit: `irevcm` = 0 and the algorithm has terminated.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `irevcm` = *<value>*.

            Constraint: `irevcm` = 0, 1, 2 or 3.

        (`errno` 11)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 12)
            On entry, `xtol` = *<value>*.

            Constraint: `xtol` >= 0.0.

        (`errno` 13)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 1 or 2.

        (`errno` 14)
            On entry, `factor` = *<value>*.

            Constraint: `factor` > 0.0.

        (`errno` 15)
            On entry, `mode` = 2 and `diag` contained a non-positive
            element.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            No further improvement in the solution is possible. `xtol`
            is too small: `xtol` = *<value>*.

        (`errno` 4)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* Jacobian
            evaluations. This failure exit may indicate that the system
            does not have a zero, or that the solution is very close to
            the origin (see [Accuracy]). Otherwise, rerunning
            ``sys_deriv_rcomm`` from a different starting point may
            avoid the region of difficulty.

        (`errno` 5)
            The iteration is not making good progress, as measured by
            the improvement from the last *<value>* iterations. This
            failure exit may indicate that the system does not have a
            zero, or that the solution is very close to the origin (see
            [Accuracy]). Otherwise, rerunning ``sys_deriv_rcomm`` from a
            different starting point may avoid the region of difficulty.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.sys_deriv_rcomm'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c05rdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, irevcm.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fvec[n].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, fjac[n, n].
        utils._BYREF_FLOAT64, # Input, xtol.
        utils._BYREF_INT, # Input, mode.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, diag[n].
        utils._BYREF_FLOAT64, # Input, factor.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, r[n*(n+1)//2].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, qtf[n].
        utils._EngineIntArrayType.c_type, # Input/Output, iwsav[17].
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rwsav[4*n+10].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((fjac, 'fjac'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _irevcm = utils._EngineIntScalarType(
        irevcm, {'fun_name': fun_name, 'entity': 'irevcm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    )
    _x = _x_cls.to_ctypes()
    _fvec_cls = utils._EngineFloat64ArrayType(
        fvec, {'fun_name': fun_name, 'entity': 'fvec'},
        exp_shape=(_n_py,),
    )
    _fvec = _fvec_cls.to_ctypes()
    _fjac_cls = utils._EngineFloat64ArrayType(
        fjac, {'fun_name': fun_name, 'entity': 'fjac'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _fjac = _fjac_cls.to_ctypes()
    _xtol = utils._EngineFloat64ScalarType(
        xtol, {'fun_name': fun_name, 'entity': 'xtol'},
    ).to_ctypes()
    _mode = utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _diag_cls = utils._EngineFloat64ArrayType(
        diag, {'fun_name': fun_name, 'entity': 'diag'},
        exp_shape=(_n_py,),
    )
    _diag = _diag_cls.to_ctypes()
    _factor = utils._EngineFloat64ScalarType(
        factor, {'fun_name': fun_name, 'entity': 'factor'},
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py*(_n_py+1)//2,),
    )
    _r = _r_cls.to_ctypes()
    _qtf_cls = utils._EngineFloat64ArrayType(
        qtf, {'fun_name': fun_name, 'entity': 'qtf'},
        exp_shape=(_n_py,),
    )
    _qtf = _qtf_cls.to_ctypes()
    _comm_needs_init = utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        opt_keys=('iwsav', 'rwsav'),
    )
    if _comm_needs_init:
        _iwsav = utils._EngineIntArrayType.empty_ctypes(17)
        comm['iwsav'] = _iwsav
    else:
        _iwsav = comm['iwsav']
    if _comm_needs_init:
        _rwsav = utils._EngineFloat64ArrayType.empty_ctypes(4*_n_py+10)
        comm['rwsav'] = _rwsav
    else:
        _rwsav = comm['rwsav']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _irevcm,
        _n,
        _x,
        _fvec,
        _fjac,
        _xtol,
        _mode,
        _diag,
        _factor,
        _r,
        _qtf,
        _iwsav,
        _rwsav,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _irevcm_py = utils._EngineIntScalarType.to_py(_irevcm)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _irevcm_py != 0 or _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, irevcm = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: irevcm = 0, 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'No further improvement in the solution is possible.',
                    'xtol is too small: xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99991:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress, as measured by the improvement',
                    'from the last {:s} Jacobian evaluations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'This failure exit may indicate that the system does not have a zero, or',
                    'that the solution is very close to the origin',
                    '(see [Accuracy]).',
                    'Otherwise, rerunning sys_deriv_rcomm from a different starting point may avoid',
                    'the region of difficulty.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The iteration is not making good progress,',
                    'as measured by the improvement from the last {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'iterations.',
                    'This failure exit may indicate that the system does not have a zero, or',
                    'that the solution is very close to the origin',
                    '(see [Accuracy]).',
                    'Otherwise, rerunning sys_deriv_rcomm from a different starting point may avoid',
                    'the region of difficulty.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xtol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xtol >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 1001:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, factor = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: factor > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 15 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = 2 and diag contained a non-positive element.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _fvec_cls.refresh_py(_fvec)
        _fjac_cls.refresh_py(_fjac)
        _diag_cls.refresh_py(_diag)
        _r_cls.refresh_py(_r)
        _qtf_cls.refresh_py(_qtf)
        return_data = _irevcm_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['irevcm'],
            )(
                _irevcm_py,
            )
    _err_class.manage()
    return return_data

def sys_deriv_check(mode, m, n, x, fvec, fjac, xp, fvecp, err):
    u"""
    Check user's function for calculating first derivatives of a set of
    nonlinear functions of several variables.

    For full information please refer to the NAG Library document for
    c05zd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/c05/c05zdf.html

    Parameters
    ----------
    mode : int
        The value 1 on the first call and the value 2 on the second call
        of ``sys_deriv_check``.

    m : int
        m, the number of functions.

    n : int
        n, the number of variables. For use with ``sys_deriv_easy``,
        ``sys_deriv_expert`` and ``sys_deriv_rcomm``, `m` = `n`.

    x : float, shape (`n`)
        The components of a point x, at which the consistency check is
        to be made. (See [Accuracy].)

    fvec : float, shape (`m`)
        If `mode` = 2, `fvec` must contain the value of the functions
        evaluated at x. If `mode` = 1, `fvec` is not referenced.

    fjac : float, shape (`m`, `n`)
        If `mode` = 2, `fjac` must contain the value of df_i/dx_j at the
        point x, for j = 1,2,...,n, for i = 1,2,...,m. If `mode` = 1,
        `fjac` is not referenced.

    xp : float, shape (`n`), modified in place
        On exit: if `mode` = 1, `xp` is set to a point neighbouring `x`.
        If `mode` = 2, `xp` is undefined.

    fvecp : float, shape (`m`)
        If `mode` = 2, `fvecp` must contain the value of the functions
        evaluated at `xp` (as output by a preceding call to
        ``sys_deriv_check`` with `mode` = 1). If `mode` = 1, `fvecp` is
        not referenced.

    err : float, shape (`m`), modified in place
        On exit: if `mode` = 2, `err` contains measures of correctness
        of the respective gradients. If `mode` = 1, `err` is undefined.
        If there is no loss of significance (see [Accuracy]), if
        `err`[i-1] is 1.0 the ith user-supplied gradient df_i/dx_j, for
        j = 1,2,...,n is correct, whilst if `err`[i-1] is 0.0 the ith
        gradient is incorrect. For values of `err`[i-1] between 0.0 and
        1.0 the categorisation is less certain. In general, a value of
        `err`[i-1] > 0.5 indicates that the ith gradient is probably
        correct.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `mode` = *<value>*.

            Constraint: `mode` = 1 or 2.

        (`errno` 2)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 3)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.roots.sys_deriv_check'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('c05zdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, mode.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Input, fvec[m].
        utils._EngineFloat64ArrayType.c_type, # Input, fjac[m, n].
        utils._EngineFloat64ArrayType.c_type, # Output, xp[n].
        utils._EngineFloat64ArrayType.c_type, # Input, fvecp[m].
        utils._EngineFloat64ArrayType.c_type, # Output, err[m].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((fjac, 'fjac'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _mode = utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _fvec = utils._EngineFloat64ArrayType(
        fvec, {'fun_name': fun_name, 'entity': 'fvec'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _fjac = utils._EngineFloat64ArrayType(
        fjac, {'fun_name': fun_name, 'entity': 'fjac'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _xp_cls = utils._EngineFloat64ArrayType(
        xp, {'fun_name': fun_name, 'entity': 'xp'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _xp = _xp_cls.to_ctypes(can_reference_contents=False)
    _fvecp = utils._EngineFloat64ArrayType(
        fvecp, {'fun_name': fun_name, 'entity': 'fvecp'},
        exp_shape=(_m_py,),
    ).to_ctypes()
    _err_cls = utils._EngineFloat64ArrayType(
        err, {'fun_name': fun_name, 'entity': 'err'},
        exp_shape=(_m_py,),
        check_contents=False,
    )
    _err = _err_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _mode,
        _m,
        _n,
        _x,
        _fvec,
        _fjac,
        _xp,
        _fvecp,
        _err,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mode = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mode = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _xp_cls.refresh_py(_xp)
        _err_cls.refresh_py(_err)
        return_data = None
    _err_class.manage()
    return return_data
