# -*- coding: utf-8 -*-
# pylint: disable=bare-except,chained-comparison,invalid-name,len-as-condition,line-too-long,protected-access,redefined-outer-name,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 27.0 `stat` Chapter.

``stat`` - Simple Calculations on Statistical Data

Base interfaces.

We do not intend for these lower-level base interfaces to be called
directly from user code. Please refer to the ``naginterfaces.library``
subpackage instead.

This module covers the following topics:

- descriptive statistics and exploratory data analysis;

- statistical distribution functions and their inverses;

- testing for Normality and other distributions.

For full information please refer to the NAG Library document

https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01intro.html
"""

# NAG Copyright 2017-2019.

import collections as _collections
import sys as _sys

from ..base import utils

def summary_2var(n, x1, x2, wt, res):
    u"""
    Means, corrected sums of squares and cross-products, etc., two
    variables, from raw data.

    For full information please refer to the NAG Library document for
    g01ab

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01abf.html

    Parameters
    ----------
    n : int
        n, the number of pairs of observations.

    x1 : float, shape (`n`)
        The observations from the first sample, x_i, for i = 1,2,...,n.

    x2 : float, shape (`n`)
        The observations from the second sample, y_i, for i = 1,2,...,n.

    wt : None or float, shape (`n`), modified in place
        Optionally, on entry: if weights are being supplied then the
        elements of `wt` must contain the weights associated with the
        observations, w_i, for i = 1,2,...,n.

        If weights are not supplied then `wt` may be None.

        On exit, if not None on entry: if iwt = 1, the elements of `wt`
        are unchanged, otherwise each element of `wt` will be assigned
        the value 1.0.

    res : float, shape (13), modified in place
        On exit: the elements of `res` contain the following results:

        [table omitted]

    Returns
    -------
    nval : int
        Is used to indicate the number of valid observations, m; see
        [Description] (g).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            The number of valid cases, m, is 0.

        (`errno` 3)
            On entry, `wt`[*<value>*] = *<value>*.

            Constraint: `wt`[i-1] >= 0.0, for i = 1,2,...,`n`

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The number of valid cases, m, is 1. In this case standard
            deviation and product-moment correlation coefficient cannot
            be calculated.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.summary_2var'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01abft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x1[n].
        utils._EngineFloat64ArrayType.c_type, # Input, x2[n].
        utils._BYREF_INT, # Input/Output, iwt.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, wt[n].
        utils._EngineFloat64ArrayType.c_type, # Output, res[13].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x1 = utils._EngineFloat64ArrayType(
        x1, {'fun_name': fun_name, 'entity': 'x1'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _x2 = utils._EngineFloat64ArrayType(
        x2, {'fun_name': fun_name, 'entity': 'x2'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    if wt is None:
        _iwt = utils._EngineIntScalarType.c_type(0)
    else:
        _iwt = utils._EngineIntScalarType.c_type(1)
    if wt is None:
        _wt = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    else:
        _wt_cls = utils._EngineFloat64ArrayType(
            wt, {'fun_name': fun_name, 'entity': 'wt'},
            exp_shape=(_n_py,),
            allow_none=True,
        )
        _wt = _wt_cls.to_ctypes()
    _res_cls = utils._EngineFloat64ArrayType(
        res, {'fun_name': fun_name, 'entity': 'res'},
        exp_shape=(13,),
        check_contents=False,
    )
    _res = _res_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x1,
        _x2,
        _iwt,
        _wt,
        _res,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of valid cases, m, is 1. In this case standard',
                    'deviation and product-moment correlation coefficient cannot',
                    'be calculated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 89998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of valid cases, m, is 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wt[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: wt[i-1] >= 0.0, for i = 1,2,...,n',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        if wt is not None:
            _wt_cls.refresh_py(_wt)
        _res_cls.refresh_py(_res)
        _iwt_py = utils._EngineIntScalarType.to_py(_iwt)
        _nval_py = _iwt_py
        return_data = _nval_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['nval'],
            )(
                _nval_py,
            )
    _err_class.manage()
    return return_data

def summary_freq(k, x, ifreq):
    u"""
    Mean, variance, skewness, kurtosis, etc., one variable, from
    frequency table.

    For full information please refer to the NAG Library document for
    g01ad

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01adf.html

    Parameters
    ----------
    k : int
        k, the number of class boundaries, which is one more than the
        number of classes of the frequency distribution.

    x : float, shape (`k`)
        The elements of `x` must contain the boundary values of the
        classes in ascending order, so that class i is bounded by the
        values in `x`[i-1] and `x`[i], for i = 1,2,...,k-1.

    ifreq : int, shape (`k`)
        The ith element of `ifreq` must contain the frequency associated
        with the ith class, for i = 1,2,...,k-1. `ifreq`[k-1] is not
        used by the function.

    Returns
    -------
    xmean : float
        The mean value, y_.

    s2 : float
        The standard deviation, s_2.

    s3 : float
        The coefficient of skewness, s_3.

    s4 : float
        The coefficient of kurtosis, s_4.

    n : int
        The total frequency, n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` > 1.

        (`errno` 2)
            On entry, I = *<value>*, `x`[I-2] = *<value>* and `x`[I-1] =
            *<value>*.

            Constraint: `x`[I-2] <= `x`[I-1].

        (`errno` 3)
            Either `ifreq`[i] < 0 for some i, or the sum of frequencies
            is zero.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The total frequency, n, is less than 2, hence the quantities
            s_2, s_3 and s_4 cannot be calculated.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.summary_freq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01adft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input, x[k].
        utils._EngineIntArrayType.c_type, # Input, ifreq[k].
        utils._BYREF_FLOAT64, # Output, xmean.
        utils._BYREF_FLOAT64, # Output, s2.
        utils._BYREF_FLOAT64, # Output, s3.
        utils._BYREF_FLOAT64, # Output, s4.
        utils._BYREF_INT, # Output, n.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _ifreq = utils._EngineIntArrayType(
        ifreq, {'fun_name': fun_name, 'entity': 'ifreq'},
        exp_shape=(_k_py,),
    ).to_ctypes()
    _xmean = utils._EngineFloat64ScalarType.empty_ctypes()
    _s2 = utils._EngineFloat64ScalarType.empty_ctypes()
    _s3 = utils._EngineFloat64ScalarType.empty_ctypes()
    _s4 = utils._EngineFloat64ScalarType.empty_ctypes()
    _n = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _k,
        _x,
        _ifreq,
        _xmean,
        _s2,
        _s3,
        _s4,
        _n,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, I = {:s}, x[I-2] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and x[I-1] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: x[I-2] <= x[I-1].',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either ifreq[i] < 0 for some i, or the sum of frequencies is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The total frequency, n, is less than 2, hence the quantities s_2,',
                    's_3 and s_4 cannot be calculated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _xmean_py = utils._EngineFloat64ScalarType.to_py(_xmean)
        _s2_py = utils._EngineFloat64ScalarType.to_py(_s2)
        _s3_py = utils._EngineFloat64ScalarType.to_py(_s3)
        _s4_py = utils._EngineFloat64ScalarType.to_py(_s4)
        _n_py = utils._EngineIntScalarType.to_py(_n)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['xmean', 's2', 's3', 's4', 'n'],
        )(
            _xmean_py,
            _s2_py,
            _s3_py,
            _s4_py,
            _n_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def frequency_table(n, k, x, iclass, cb, ifreq):
    u"""
    Frequency table from raw data.

    For full information please refer to the NAG Library document for
    g01ae

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01aef.html

    Parameters
    ----------
    n : int
        n, the number of observations.

    k : int
        k, the number of classes desired in the frequency distribution.
        Whether or not class boundary values are user-supplied, `k` must
        include the two extreme classes which stretch to +/ - infinity.

    x : float, shape (`n`)
        The sample of observations of the variable for which the
        frequency distribution is required, x_i, for i = 1,2,...,n. The
        values may be in any order.

    iclass : int
        Indicates whether class boundary values are to be calculated
        within ``frequency_table``, or are supplied by you.

        If `iclass` = 0, the class boundary values are to be calculated
        within the function.

        If `iclass` = 1, they are user-supplied.

    cb : float, shape (`k`), modified in place
        On entry: if `iclass` = 0, the elements of `cb` need not be
        assigned values, as ``frequency_table`` calculates k-1 class
        boundary values.

        If `iclass` = 1, the first k-1 elements of `cb` must contain the
        class boundary values you supplied, in ascending order.

        In both cases, the element `cb`[k-1] need not be assigned, as it
        is not used in the function.

        On exit: the first k-1 elements of `cb` contain the class
        boundary values in ascending order.

    ifreq : int, shape (`k`), modified in place
        On exit: the elements of `ifreq` contain the frequencies in each
        class, f_i, for i = 1,2,...,k. In particular `ifreq`[0] contains
        the frequency of the class up to `cb`[0], f_1, and `ifreq`[k-1]
        contains the frequency of the class greater than `cb`[k-2], f_k.

    Returns
    -------
    xmin : float
        The smallest value in the sample, a.

    xmax : float
        The largest value in the sample, b.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 2.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 3)
            On entry, `cb`[*<value>*] = *<value>* and `cb`[*<value>*] =
            *<value>*.

            Constraint: `cb`[*<value>*] < `cb`[*<value>*].
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.frequency_table'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01aeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, k.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, iclass.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, cb[k].
        utils._EngineIntArrayType.c_type, # Output, ifreq[k].
        utils._BYREF_FLOAT64, # Output, xmin.
        utils._BYREF_FLOAT64, # Output, xmax.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _iclass = utils._EngineIntScalarType(
        iclass, {'fun_name': fun_name, 'entity': 'iclass'},
    ).to_ctypes()
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _cb_cls = utils._EngineFloat64ArrayType(
        cb, {'fun_name': fun_name, 'entity': 'cb'},
        exp_shape=(_k_py,),
    )
    _cb = _cb_cls.to_ctypes()
    _ifreq_cls = utils._EngineIntArrayType(
        ifreq, {'fun_name': fun_name, 'entity': 'ifreq'},
        exp_shape=(_k_py,),
        check_contents=False,
    )
    _ifreq = _ifreq_cls.to_ctypes(can_reference_contents=False)
    _xmin = utils._EngineFloat64ScalarType.empty_ctypes()
    _xmax = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _k,
        _x,
        _iclass,
        _cb,
        _ifreq,
        _xmin,
        _xmax,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, cb[{:s}] = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'and cb[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[3]),
                    ),
                    'Constraint: cb[{:s}] < cb[{:s}].'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _cb_cls.refresh_py(_cb)
        _ifreq_cls.refresh_py(_ifreq)
        _xmin_py = utils._EngineFloat64ScalarType.to_py(_xmin)
        _xmax_py = utils._EngineFloat64ScalarType.to_py(_xmax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['xmin', 'xmax'],
        )(
            _xmin_py,
            _xmax_py,
        )
    _err_class.manage()
    return return_data

def contingency_table(m, n, nobs, num, pred, p):
    u"""
    Two-way contingency table analysis, with chi^2/Fisher's exact test.

    For full information please refer to the NAG Library document for
    g01af

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01aff.html

    Parameters
    ----------
    m : int
        m+1, one more than the number of rows of the frequency matrix.

    n : int
        n+1, one more than the number of columns of the frequency
        matrix.

    nobs : int, shape (`m`, `n`), modified in place
        On entry: the elements `nobs`[i-1,j-1], for j = 1,2,...,n, for i
        = 1,2,...,m, must contain the frequencies for the two-way
        classification. The (m+1)th row and the (n+1)th column of `nobs`
        need not be set.

        On exit: contains the following information:

        - `nobs`[i-1,j-1], for j = 1,2,...,n_1, for i = 1,2,...,m_1,
          contain the frequencies for the two-way classification after
          'shrinkage' has taken place (see [Description]).

        - `nobs`[i-1,n], for i = 1,2,...,m_1, contain the total
          frequencies in the remaining rows, R_i.

        - `nobs`[m,j-1], for j = 1,2,...,n_1, contain the total
          frequencies in the remaining columns, C_j.

        - `nobs`[m,n], contains the total frequency, T.

        If any 'shrinkage' has occurred, all other cells contain no
        useful information.

    num : int
        The value assigned to `num` must determine whether automatic
        'shrinkage' is required when any r_{ij} < 1, as outlined in
        [Description](1).

        If `num` = 1, shrinkage is required, otherwise shrinkage is not
        required.

    pred : float, shape (`m`, `n`), modified in place
        On exit: the elements `pred`[i-1,j-1], where i = 1,2,...,`m1`
        and j = 1,2,...,`n1` contain the expected frequencies, r_{ij}
        corresponding to the observed frequencies `nobs`[i-1,j-1],
        except in the case when Fisher's exact test for a 2*2
        classification is to be used, when `pred` is not used. No other
        elements are utilized.

    p : float, shape (21), modified in place
        On exit: the first `num` elements contain the probabilities
        associated with the various possible frequency tables, P_r, for
        r = 0,1,...,R_1, the remainder are unspecified.

    Returns
    -------
    num : int
        When Fisher's exact test for a 2*2 classification is used then
        `num` contains the number of elements used in the array `p`,
        otherwise `num` is set to zero.

    chis : float
        The value of the test statistic, chi^2, except when Fisher's
        exact test for a 2*2 classification is used in which case it is
        unspecified.

    npos : int
        `p`[`npos`-1] holds the probability associated with the given
        table of frequencies.

    ndf : int
        The value of `ndf` gives the number of degrees of freedom for
        the chi-square distribution, (m_1-1)*(n_1-1); when Fisher's
        exact test is used `ndf` = 1.

    m1 : int
        The number of rows of the two-way classification, after any
        'shrinkage', m_1.

    n1 : int
        The number of columns of the two-way classification, after any
        'shrinkage', n_1.

    Raises
    ------
    NagValueError
        (`errno` 1)
            The number of rows or columns of `nobs` is less than 2.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 2.

        (`errno` 1)
            On entry, `m` = *<value>*.

            Constraint: `m` > 2.

        (`errno` 2)
            At least one frequency is negative, or all frequencies are
            zero.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.contingency_table'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01afft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ldnob.
        utils._BYREF_INT, # Input, ldpred.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, n.
        utils._EngineIntArrayType.c_type, # Input/Output, nobs[ldnob, n].
        utils._BYREF_INT, # Input/Output, num.
        utils._EngineFloat64ArrayType.c_type, # Output, pred[ldpred, n].
        utils._BYREF_FLOAT64, # Output, chis.
        utils._EngineFloat64ArrayType.c_type, # Output, p[21].
        utils._BYREF_INT, # Output, npos.
        utils._BYREF_INT, # Output, ndf.
        utils._BYREF_INT, # Output, m1.
        utils._BYREF_INT, # Output, n1.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((nobs, 'nobs'), (pred, 'pred')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _ldnob_py = utils._EngineIntArrayType.get_nd(nobs, {'fun_name': fun_name, 'entity': 'nobs'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldnob = utils._EngineIntScalarType(
        _ldnob_py, {'fun_name': fun_name, 'entity': '_ldnob'},
    ).to_ctypes()
    _ldpred_py = utils._EngineFloat64ArrayType.get_nd(pred, {'fun_name': fun_name, 'entity': 'pred'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldpred = utils._EngineIntScalarType(
        _ldpred_py, {'fun_name': fun_name, 'entity': '_ldpred'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _m_py = utils._EngineIntScalarType.to_py(_m)
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _nobs_cls = utils._EngineIntArrayType(
        nobs, {'fun_name': fun_name, 'entity': 'nobs'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _nobs = _nobs_cls.to_ctypes()
    _num = utils._EngineIntScalarType(
        num, {'fun_name': fun_name, 'entity': 'num'},
    ).to_ctypes()
    _pred_cls = utils._EngineFloat64ArrayType(
        pred, {'fun_name': fun_name, 'entity': 'pred'},
        exp_shape=(_m_py, _n_py),
        mutual_sorder=_m_sorder,
        check_contents=False,
    )
    _pred = _pred_cls.to_ctypes(can_reference_contents=False)
    _chis = utils._EngineFloat64ScalarType.empty_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(21,),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _npos = utils._EngineIntScalarType.empty_ctypes()
    _ndf = utils._EngineIntScalarType.empty_ctypes()
    _m1 = utils._EngineIntScalarType.empty_ctypes()
    _n1 = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ldnob,
        _ldpred,
        _m,
        _n,
        _nobs,
        _num,
        _pred,
        _chis,
        _p,
        _npos,
        _ndf,
        _m1,
        _n1,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The number of rows or columns of nobs is less than 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m > 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'At least one frequency is negative, or all frequencies are zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _nobs_cls.refresh_py(_nobs)
        _num_py = utils._EngineIntScalarType.to_py(_num)
        _pred_cls.refresh_py(_pred)
        _p_cls.refresh_py(_p)
        _chis_py = utils._EngineFloat64ScalarType.to_py(_chis)
        _npos_py = utils._EngineIntScalarType.to_py(_npos)
        _ndf_py = utils._EngineIntScalarType.to_py(_ndf)
        _m1_py = utils._EngineIntScalarType.to_py(_m1)
        _n1_py = utils._EngineIntScalarType.to_py(_n1)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['num', 'chis', 'npos', 'ndf', 'm1', 'n1'],
        )(
            _num_py,
            _chis_py,
            _npos_py,
            _ndf_py,
            _m1_py,
            _n1_py,
        )
    _err_class.manage()
    return return_data

def five_point_summary(n, x, res):
    u"""
    Computes a five-point summary (median, hinges and extremes).

    For full information please refer to the NAG Library document for
    g01al

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01alf.html

    Parameters
    ----------
    n : int
        n, number of observations in the sample.

    x : float, shape (`n`)
        The sample observations, x_1,x_2,...,x_n.

    res : float, shape (5), modified in place
        On exit: `res` contains the five-point summary.

        `res`[0]

          The minimum.

        `res`[1]

          The lower hinge.

        `res`[2]

          The median.

        `res`[3]

          The upper hinge.

        `res`[4]

          The maximum.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 5.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.five_point_summary'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01alft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineIntArrayType.c_type, # Workspace, iwrk.
        utils._EngineFloat64ArrayType.c_type, # Output, res[5].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _iwrk = utils._EngineIntArrayType.empty_ctypes(0)
    _res_cls = utils._EngineFloat64ArrayType(
        res, {'fun_name': fun_name, 'entity': 'res'},
        exp_shape=(5,),
        check_contents=False,
    )
    _res = _res_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _x,
        _iwrk,
        _res,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _res_cls.refresh_py(_res)
        return_data = None
    _err_class.manage()
    return return_data

def quantiles(n, rv, nq, q, qv):
    u"""
    Find quantiles of an unordered vector, real numbers.

    For full information please refer to the NAG Library document for
    g01am

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01amf.html

    Parameters
    ----------
    n : int
        The number of elements in the input vector `rv`.

    rv : float, shape (`n`), modified in place
        On entry: the vector whose quantiles are to be determined.

        On exit: the order of the elements in `rv` is not, in general,
        preserved.

    nq : int
        The number of quantiles requested.

    q : float, shape (`nq`)
        The quantiles to be calculated, in ascending order. Note that
        these must be between 0.0 and 1.0, with 0.0 returning the
        smallest element and 1.0 the largest.

    qv : float, shape (`nq`), modified in place
        On exit: `qv`[i-1] contains the quantile specified by the value
        provided in `q`[i-1], or an interpolated value if the quantile
        falls between two data values.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `nq` = *<value>*.

            Constraint: `nq` > 0.

        (`errno` 3)
            On entry, an element of `q` was less than 0.0 or greater
            than 1.0.

        (`errno` 4)
            On entry, `q` was not in ascending order.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.quantiles'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01amft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rv[n].
        utils._BYREF_INT, # Input, nq.
        utils._EngineFloat64ArrayType.c_type, # Input, q[nq].
        utils._EngineFloat64ArrayType.c_type, # Output, qv[nq].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _rv_cls = utils._EngineFloat64ArrayType(
        rv, {'fun_name': fun_name, 'entity': 'rv'},
        exp_shape=(_n_py,),
    )
    _rv = _rv_cls.to_ctypes()
    _nq = utils._EngineIntScalarType(
        nq, {'fun_name': fun_name, 'entity': 'nq'},
    ).to_ctypes()
    _nq_py = utils._EngineIntScalarType.to_py(_nq)
    _q = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=(_nq_py,),
    ).to_ctypes()
    _qv_cls = utils._EngineFloat64ArrayType(
        qv, {'fun_name': fun_name, 'entity': 'qv'},
        exp_shape=(_nq_py,),
        check_contents=False,
    )
    _qv = _qv_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _rv,
        _nq,
        _q,
        _qv,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nq > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, an element of q was less than 0.0 or greater than 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q was not in ascending order.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rv_cls.refresh_py(_rv)
        _qv_cls.refresh_py(_qv)
        return_data = None
    _err_class.manage()
    return return_data

def quantiles_stream_fixed(ind, n, rv, nb, eps, q, qv, nq, rcomm, icomm):
    u"""
    Calculates approximate quantiles from a data stream of known size.

    For full information please refer to the NAG Library document for
    g01an

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01anf.html

    Parameters
    ----------
    ind : int
        Indicates the action required in the current call to
        ``quantiles_stream_fixed``.

        `ind` = 0

          Return the required length of `rcomm` and `icomm` in
          `icomm`[0] and `icomm`[1] respectively. `n` and `eps` must be
          set and licomm must be at least 2.

        `ind` = 1

          Initialise the communication arrays and process the first `nb`
          values from the data stream as supplied in `rv`.

        `ind` = 2

          Process the next block of `nb` values from the data stream.
          The calling program must update `rv` and (if required) `nb`,
          and re-enter ``quantiles_stream_fixed`` with all other
          parameters unchanged.

        `ind` = 3

          Calculate the `nq` epsilon-approximate quantiles specified in
          `q`. The calling program must set `q` and `nq` and re-enter
          ``quantiles_stream_fixed`` with all other parameters
          unchanged. This option can be chosen only when `np` >=
          ⌈exp(1.0)/`eps`⌉.

    n : int
        n, the total number of values in the data stream.

    rv : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `ind` in (1, 2): `nb`; otherwise: 0.

        If `ind` = 1 or 2, the vector containing the current block of
        data, otherwise `rv` is not referenced.

    nb : int
        If `ind` = 1 or 2, the size of the current block of data. The
        size of blocks of data in array `rv` can vary; therefore `nb`
        can change between calls to ``quantiles_stream_fixed``.

    eps : float
        Approximation factor epsilon.

    q : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `ind`=3: `nq`; otherwise: 0.

        If `ind` = 3, the quantiles to be calculated, otherwise `q` is
        not referenced. Note that `q`[i] = 0.0, corresponds to the
        minimum value and `q`[i] = 1.0 to the maximum value.

    qv : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `ind`=3: `nq`; otherwise: 0.

        On exit: if `ind` = 3, `qv`[i] contains the epsilon-approximate
        quantiles specified by the value provided in `q`[i].

    nq : int
        If `ind` = 3, the number of quantiles requested, otherwise `nq`
        is not referenced.

    rcomm : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `ind` != 0: the value returned in `icomm`[0] by a
        call to ``quantiles_stream_fixed`` with `ind` = 0; otherwise: 0.

        Communication array.

    icomm : int, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `ind` != 0: the value returned in `icomm`[1] by a
        call to ``quantiles_stream_fixed`` with `ind` = 0; otherwise: 2.

        Communication array.

    Returns
    -------
    ind : int
        Indicates output from a successful call.

        `ind` = 1

          Lengths of `rcomm` and `icomm` have been returned in
          `icomm`[0] and `icomm`[1] respectively.

        `ind` = 2

          ``quantiles_stream_fixed`` has processed `np` data points and
          expects to be called again with additional data (i.e., `np` <
          `n`).

        `ind` = 3

          ``quantiles_stream_fixed`` has returned the requested
          epsilon-approximate quantiles in `qv`. These quantiles are
          based on `np` data points.

        `ind` = 4

          Routine has processed all `n` data points (i.e., `np` = `n`).

    np : int
        The number of elements processed so far.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0, 1, 2 or 3.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 3)
            On entry, `eps` = *<value>*.

            Constraint: exp(1.0)/`n` <= `eps` <= 1.0.

        (`errno` 4)
            On entry, `ind` = 1 or 2 and `nb` = *<value>*.

            Constraint: if `ind` = 1 or 2 then `nb` > 0.

        (`errno` 7)
            Number of data elements streamed, *<value>* is not
            sufficient for a quantile query when `eps` = *<value>*.

            Supply more data or reprocess the data with a higher `eps`
            value.

        (`errno` 8)
            On entry, `ind` = 3 and `nq` = *<value>*.

            Constraint: if `ind` = 3 then `nq` > 0.

        (`errno` 9)
            On entry, `ind` = 3 and `q`[*<value>*] = *<value>*.

            Constraint: if `ind` = 3 then 0.0 <= `q`[i] <= 1.0 for all
            i.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.quantiles_stream_fixed'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01anft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, ind.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, rv[lrv].
        utils._BYREF_INT, # Input, nb.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._BYREF_INT, # Output, np.
        utils._EngineFloat64ArrayType.c_type, # Input, q[lq].
        utils._EngineFloat64ArrayType.c_type, # Output, qv[lq].
        utils._BYREF_INT, # Input, nq.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._BYREF_INT, # Input, lrv.
        utils._BYREF_INT, # Input, lq.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lrv_py = utils._EngineFloat64ArrayType.get_shape(rv, {'fun_name': fun_name, 'entity': 'rv'}, exp_rank=1)[0]
    _lrv = utils._EngineIntScalarType(
        _lrv_py, {'fun_name': fun_name, 'entity': '_lrv'},
    ).to_ctypes()
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    if _ind_py in (1, 2):
        _rv_dim1 = _nb_py
    else:
        _rv_dim1 = 0
    exp_shape_rv = (_rv_dim1,)
    _rv = utils._EngineFloat64ArrayType(
        rv, {'fun_name': fun_name, 'entity': 'rv'},
        exp_shape=exp_shape_rv,
    ).to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _np = utils._EngineIntScalarType.empty_ctypes()
    _lq_py = utils._EngineFloat64ArrayType.get_shape(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=1)[0]
    _lq = utils._EngineIntScalarType(
        _lq_py, {'fun_name': fun_name, 'entity': '_lq'},
    ).to_ctypes()
    _nq = utils._EngineIntScalarType(
        nq, {'fun_name': fun_name, 'entity': 'nq'},
    ).to_ctypes()
    _nq_py = utils._EngineIntScalarType.to_py(_nq)
    if _ind_py == 3:
        _q_dim1 = _nq_py
    else:
        _q_dim1 = 0
    exp_shape_q = (_q_dim1,)
    _q = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
    ).to_ctypes()
    if _ind_py == 3:
        _qv_dim1 = _nq_py
    else:
        _qv_dim1 = 0
    exp_shape_qv = (_qv_dim1,)
    _qv_cls = utils._EngineFloat64ArrayType(
        qv, {'fun_name': fun_name, 'entity': 'qv'},
        exp_shape=exp_shape_qv,
        check_contents=False,
    )
    _qv = _qv_cls.to_ctypes(can_reference_contents=False)
    _lrcomm_py = utils._EngineFloat64ArrayType.get_shape(rcomm, {'fun_name': fun_name, 'entity': 'rcomm'}, exp_rank=1)[0]
    _lrcomm = utils._EngineIntScalarType(
        _lrcomm_py, {'fun_name': fun_name, 'entity': '_lrcomm'},
    ).to_ctypes()
    _rcomm_cls = utils._EngineFloat64ArrayType(
        rcomm, {'fun_name': fun_name, 'entity': 'rcomm'},
        exp_shape=(_lrcomm_py,),
    )
    _rcomm = _rcomm_cls.to_ctypes()
    _licomm_py = utils._EngineIntArrayType.get_shape(icomm, {'fun_name': fun_name, 'entity': 'icomm'}, exp_rank=1)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm_cls = utils._EngineIntArrayType(
        icomm, {'fun_name': fun_name, 'entity': 'icomm'},
        exp_shape=(_licomm_py,),
    )
    _icomm = _icomm_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ind,
        _n,
        _rv,
        _nb,
        _eps,
        _np,
        _q,
        _qv,
        _nq,
        _rcomm,
        _lrcomm,
        _icomm,
        _licomm,
        _lrv,
        _lq,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0, 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: exp(1.0)/n <= eps <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = 1 or 2 and nb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if ind = 1 or 2 then nb > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Number of data elements streamed, {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'is not sufficient for a quantile query when eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Supply more data or reprocess the data with a higher eps value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = 3 and nq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if ind = 3 then nq > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = 3 and q[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if ind = 3 then 0.0 <= q[i] <= 1.0 for all i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        _qv_cls.refresh_py(_qv)
        _rcomm_cls.refresh_py(_rcomm)
        _icomm_cls.refresh_py(_icomm)
        _np_py = utils._EngineIntScalarType.to_py(_np)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ind', 'np'],
        )(
            _ind_py,
            _np_py,
        )
    _err_class.manage()
    return return_data

def quantiles_stream_arbitrary(ind, rv, nb, eps, q, qv, nq, rcomm, icomm):
    u"""
    Calculates approximate quantiles from a data stream of unknown size.

    For full information please refer to the NAG Library document for
    g01ap

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01apf.html

    Parameters
    ----------
    ind : int
        On initial entry: must be set to 0.Indicates the action required
        in the current call to ``quantiles_stream_arbitrary``.

        `ind` = 0

          Initialize the communication arrays and attempt to process the
          first `nb` values from the data stream. `eps`, `rv` and `nb`
          must be set and licomm must be at least 10.

        `ind` = 1

          Attempt to process the next block of `nb` values from the data
          stream. The calling program must update `rv` and (if required)
          `nb`, and re-enter ``quantiles_stream_arbitrary`` with all
          other parameters unchanged.

        `ind` = 2

          Continue calculation following the reallocation of either or
          both of the communication arrays `rcomm` and `icomm`.

        `ind` = 3

          Calculate the `nq` epsilon-approximate quantiles specified in
          `q`. The calling program must set `q` and `nq` and re-enter
          ``quantiles_stream_arbitrary`` with all other parameters
          unchanged. This option can be chosen only when `np` >=
          ⌈exp(1.0)/`eps`⌉.

    rv : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `ind` in (0, 1, 2): `nb`; otherwise: 0.

        If `ind` = 0, 1 or 2, the vector containing the current block of
        data, otherwise `rv` is not referenced.

    nb : int
        If `ind` = 0, 1 or 2, the size of the current block of data. The
        size of blocks of data in array `rv` can vary; therefore `nb`
        can change between calls to ``quantiles_stream_arbitrary``.

    eps : float
        Approximation factor epsilon.

    q : float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `ind`=3: `nq`; otherwise: 0.

        If `ind` = 3, the quantiles to be calculated, otherwise `q` is
        not referenced. Note that `q`[i] = 0.0, corresponds to the
        minimum value and `q`[i] = 1.0 to the maximum value.

    qv : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `ind`=3: `nq`; otherwise: 0.

        On exit: if `ind` = 3, `qv`[i] contains the epsilon-approximate
        quantiles specified by the value provided in `q`[i].

    nq : int
        If `ind` = 3, the number of quantiles requested, otherwise `nq`
        is not referenced.

    rcomm : float, shape (lrcomm), modified in place
        On entry: if `ind` = 1 or 2 then the first l elements of `rcomm`
        as supplied to ``quantiles_stream_arbitrary`` must be identical
        to the first l elements of `rcomm` returned from the last call
        to ``quantiles_stream_arbitrary``, where l is the value of
        lrcomm used in the last call. In other words, the contents of
        `rcomm` must not be altered between calls to this function. If
        `rcomm` needs to be reallocated then its contents must be
        preserved. If `ind` = 0 then `rcomm` need not be set.

        On exit: `rcomm` holds information required by subsequent calls
        to ``quantiles_stream_arbitrary``.

    icomm : int, shape (licomm), modified in place
        On entry: if `ind` = 1 or 2 then the first l elements of `icomm`
        as supplied to ``quantiles_stream_arbitrary`` must be identical
        to the first l elements of `icomm` returned from the last call
        to ``quantiles_stream_arbitrary``, where l is the value of
        licomm used in the last call. In other words, the contents of
        `icomm` must not be altered between calls to this function. If
        `icomm` needs to be reallocated then its contents must be
        preserved. If `ind` = 0 then `icomm` need not be set.

        On exit: `icomm`[0] holds the minimum required length for
        `rcomm` and `icomm`[1] holds the minimum required length for
        `icomm`. The remaining elements of `icomm` are used for
        communication between subsequent calls to
        ``quantiles_stream_arbitrary``.

    Returns
    -------
    ind : int
        Indicates output from the call.

        `ind` = 1

          ``quantiles_stream_arbitrary`` has processed `np` data points
          and expects to be called again with additional data.

        `ind` = 2

          Either one or more of the communication arrays `rcomm` and
          `icomm` is too small. The new minimum lengths of `rcomm` and
          `icomm` have been returned in `icomm`[0] and `icomm`[1]
          respectively. If the new minimum length is greater than the
          current length then the corresponding communication array
          needs to be reallocated, its contents preserved and
          ``quantiles_stream_arbitrary`` called again with all other
          parameters unchanged.

          If there is more data to be processed, it is recommended that
          lrcomm and licomm are made significantly bigger than the
          minimum to limit the number of reallocations.

        `ind` = 3

          ``quantiles_stream_arbitrary`` has returned the requested
          epsilon-approximate quantiles in `qv`. These quantiles are
          based on `np` data points.

    np : int
        m, the number of elements processed so far.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ind` = *<value>*.

            Constraint: `ind` = 0, 1, 2 or 3.

        (`errno` 2)
            On entry, `eps` = *<value>*.

            Constraint: 0.0 < `eps` <= 1.0.

        (`errno` 3)
            On entry, `ind` = 0, 1 or 2 and `nb` = *<value>*.

            Constraint: if `ind` = 0, 1 or 2 then `nb` > 0.

        (`errno` 6)
            The contents of `icomm` have been altered between calls to
            this function.

        (`errno` 7)
            The contents of `rcomm` have been altered between calls to
            this function.

        (`errno` 8)
            Number of data elements streamed, *<value>* is not
            sufficient for a quantile query when `eps` = *<value>*.

            Supply more data or reprocess the data with a higher `eps`
            value.

        (`errno` 9)
            On entry, `ind` = 3 and `nq` = *<value>*.

            Constraint: if `ind` = 3 then `nq` > 0.

        (`errno` 10)
            On entry, `ind` = 3 and `q`[*<value>*] = *<value>*.

            Constraint: if `ind` = 3 then 0.0 <= `q`[i] <= 1.0 for all
            i.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.quantiles_stream_arbitrary'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01apft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input/Output, ind.
        utils._EngineFloat64ArrayType.c_type, # Input, rv[lrv].
        utils._BYREF_INT, # Input, nb.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._BYREF_INT, # Output, np.
        utils._EngineFloat64ArrayType.c_type, # Input, q[lq].
        utils._EngineFloat64ArrayType.c_type, # Output, qv[lq].
        utils._BYREF_INT, # Input, nq.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._EngineIntArrayType.c_type, # Input/Output, icomm[licomm].
        utils._BYREF_INT, # Input, licomm.
        utils._BYREF_INT, # Input, lrv.
        utils._BYREF_INT, # Input, lq.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ind = utils._EngineIntScalarType(
        ind, {'fun_name': fun_name, 'entity': 'ind'},
    ).to_ctypes()
    _lrv_py = utils._EngineFloat64ArrayType.get_shape(rv, {'fun_name': fun_name, 'entity': 'rv'}, exp_rank=1)[0]
    _lrv = utils._EngineIntScalarType(
        _lrv_py, {'fun_name': fun_name, 'entity': '_lrv'},
    ).to_ctypes()
    _ind_py = utils._EngineIntScalarType.to_py(_ind)
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    if _ind_py in (0, 1, 2):
        _rv_dim1 = _nb_py
    else:
        _rv_dim1 = 0
    exp_shape_rv = (_rv_dim1,)
    _rv = utils._EngineFloat64ArrayType(
        rv, {'fun_name': fun_name, 'entity': 'rv'},
        exp_shape=exp_shape_rv,
    ).to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _np = utils._EngineIntScalarType.empty_ctypes()
    _lq_py = utils._EngineFloat64ArrayType.get_shape(q, {'fun_name': fun_name, 'entity': 'q'}, exp_rank=1)[0]
    _lq = utils._EngineIntScalarType(
        _lq_py, {'fun_name': fun_name, 'entity': '_lq'},
    ).to_ctypes()
    _nq = utils._EngineIntScalarType(
        nq, {'fun_name': fun_name, 'entity': 'nq'},
    ).to_ctypes()
    _nq_py = utils._EngineIntScalarType.to_py(_nq)
    if _ind_py == 3:
        _q_dim1 = _nq_py
    else:
        _q_dim1 = 0
    exp_shape_q = (_q_dim1,)
    _q = utils._EngineFloat64ArrayType(
        q, {'fun_name': fun_name, 'entity': 'q'},
        exp_shape=exp_shape_q,
    ).to_ctypes()
    if _ind_py == 3:
        _qv_dim1 = _nq_py
    else:
        _qv_dim1 = 0
    exp_shape_qv = (_qv_dim1,)
    _qv_cls = utils._EngineFloat64ArrayType(
        qv, {'fun_name': fun_name, 'entity': 'qv'},
        exp_shape=exp_shape_qv,
        check_contents=False,
    )
    _qv = _qv_cls.to_ctypes(can_reference_contents=False)
    _lrcomm_py = utils._EngineFloat64ArrayType.get_shape(rcomm, {'fun_name': fun_name, 'entity': 'rcomm'}, exp_rank=1)[0]
    _lrcomm = utils._EngineIntScalarType(
        _lrcomm_py, {'fun_name': fun_name, 'entity': '_lrcomm'},
    ).to_ctypes()
    _rcomm_cls = utils._EngineFloat64ArrayType(
        rcomm, {'fun_name': fun_name, 'entity': 'rcomm'},
        exp_shape=(_lrcomm_py,),
    )
    _rcomm = _rcomm_cls.to_ctypes()
    _licomm_py = utils._EngineIntArrayType.get_shape(icomm, {'fun_name': fun_name, 'entity': 'icomm'}, exp_rank=1)[0]
    _licomm = utils._EngineIntScalarType(
        _licomm_py, {'fun_name': fun_name, 'entity': '_licomm'},
    ).to_ctypes()
    _icomm_cls = utils._EngineIntArrayType(
        icomm, {'fun_name': fun_name, 'entity': 'icomm'},
        exp_shape=(_licomm_py,),
    )
    _icomm = _icomm_cls.to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ind,
        _rv,
        _nb,
        _eps,
        _np,
        _q,
        _qv,
        _nq,
        _rcomm,
        _lrcomm,
        _icomm,
        _licomm,
        _lrv,
        _lq,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ind = 0, 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 < eps <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = 0, 1 or 2 and nb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if ind = 0, 1 or 2 then nb > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 6:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The contents of icomm have been altered between calls to this function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 7:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The contents of rcomm have been altered between calls to this function.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Number of data elements streamed, {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'is not sufficient for a quantile query when eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Supply more data or reprocess the data with a higher eps value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = 3 and nq = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if ind = 3 then nq > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 10 and _ifmt == 10:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ind = 3 and q[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if ind = 3 then 0.0 <= q[i] <= 1.0 for all i.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _ind_py = utils._EngineIntScalarType.to_py(_ind)
        _qv_cls.refresh_py(_qv)
        _rcomm_cls.refresh_py(_rcomm)
        _icomm_cls.refresh_py(_icomm)
        _np_py = utils._EngineIntScalarType.to_py(_np)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['ind', 'np'],
        )(
            _ind_py,
            _np_py,
        )
    _err_class.manage()
    return return_data

def summary_onevar(nb, x, wt, pn, rcomm):
    u"""
    Computes univariate summary information: mean, variance, skewness,
    kurtosis.

    For full information please refer to the NAG Library document for
    g01at

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01atf.html

    Parameters
    ----------
    nb : int
        b, the number of observations in the current block of data. The
        size of the block of data supplied in `x` and `wt` can vary;
        therefore `nb` can change between calls to ``summary_onevar``.

    x : float, shape (`nb`)
        The current block of observations, corresponding to x_i, for i =
        k+1,...,k+b, where k is the number of observations processed so
        far and b is the size of the current block of data.

    wt : None or float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `wt` is not None: `nb`; otherwise: 0.

        If `wt` is not None, `wt` must contain the user-supplied weights
        corresponding to the block of data supplied in `x`, that is w_i,
        for i = k+1,...,k+b.

        If `wt` is None, w_i = 1 for all i.

    pn : int
        The number of valid observations processed so far, that is the
        number of observations with w_i > 0, for i = 1,2,...,k. On the
        first call to ``summary_onevar``, or when starting to summarise
        a new dataset, `pn` must be set to 0.

        If `pn` != 0, it must be the same value as returned by the last
        call to ``summary_onevar``.

    rcomm : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `rcomm` is not None: 20; otherwise: 0.

        Optionally, on entry: communication array, used to store
        information between calls to ``summary_onevar``. If `pn` = 0,
        `rcomm` need not be initialized, otherwise it must be unchanged
        since the last call to this function.

        If `rcomm` is None, `rcomm` is not referenced and all the data
        must be supplied in one go.

        On exit, if not None on entry: the updated communication array.
        The first five elements of `rcomm` hold information that may be
        of interest with

            [table omitted]

        the remaining elements of `rcomm` are used for workspace and so
        are undefined.

    Returns
    -------
    pn : int
        The updated number of valid observations processed, that is the
        number of observations with w_i > 0, for i = 1,2,...,k+b.

    xmean : float
        x_, the mean of the first k+b observations.

    xsd : float
        s_2, the standard deviation of the first k+b observations.

    xskew : float
        s_3, the coefficient of skewness for the first k+b observations.

    xkurt : float
        s_4, the coefficient of kurtosis for the first k+b observations.

    xmin : float
        The smallest value in the first k+b observations.

    xmax : float
        The largest value in the first k+b observations.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `nb` = *<value>*.

            Constraint: `nb` >= 0.

        (`errno` 41)
            On entry, `wt`[*<value>*] = *<value>*.

            Constraint: if `wt` is not None then `wt`[i-1] >= 0, for i =
            1,2,...,`nb`.

        (`errno` 51)
            On entry, `pn` = *<value>*.

            Constraint: `pn` >= 0.

        (`errno` 52)
            On entry, `pn` = *<value>*.

            On exit from previous call, `pn` = *<value>*.

            Constraint: if `pn` > 0, `pn` must be unchanged since
            previous call.

        (`errno` 54)
            On entry, `pn` = *<value>*.

            Constraint: if `rcomm` is None then `pn` = 0.

        (`errno` 121)
            `rcomm` has been corrupted between calls.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 53)
            On entry, the number of valid observations is zero.

        (`errno` 71)
            On exit we were unable to calculate `xskew` or `xkurt`. A
            value of 0 has been returned.

        (`errno` 72)
            On exit we were unable to calculate `xsd`, `xskew` or
            `xkurt`. A value of 0 has been returned.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.summary_onevar'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01atft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, nb.
        utils._EngineFloat64ArrayType.c_type, # Input, x[nb].
        utils._BYREF_INT, # Input, lwt.
        utils._EngineFloat64ArrayType.c_type, # Input, wt[lwt].
        utils._BYREF_INT, # Input/Output, pn.
        utils._BYREF_FLOAT64, # Output, xmean.
        utils._BYREF_FLOAT64, # Output, xsd.
        utils._BYREF_FLOAT64, # Output, xskew.
        utils._BYREF_FLOAT64, # Output, xkurt.
        utils._BYREF_FLOAT64, # Output, xmin.
        utils._BYREF_FLOAT64, # Output, xmax.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_nb_py,),
    ).to_ctypes()
    _lwt_py = utils._EngineFloat64ArrayType.get_shape(wt, {'fun_name': fun_name, 'entity': 'wt'}, exp_rank=1, allow_none=True)[0]
    _lwt = utils._EngineIntScalarType(
        _lwt_py, {'fun_name': fun_name, 'entity': '_lwt'},
    ).to_ctypes()
    if wt is not None:
        _wt_dim1 = _nb_py
    else:
        _wt_dim1 = 0
    exp_shape_wt = (_wt_dim1,)
    _wt = utils._EngineFloat64ArrayType(
        wt, {'fun_name': fun_name, 'entity': 'wt'},
        exp_shape=exp_shape_wt,
        allow_none=True,
    ).to_ctypes()
    _pn = utils._EngineIntScalarType(
        pn, {'fun_name': fun_name, 'entity': 'pn'},
    ).to_ctypes()
    _xmean = utils._EngineFloat64ScalarType.empty_ctypes()
    _xsd = utils._EngineFloat64ScalarType.empty_ctypes()
    _xskew = utils._EngineFloat64ScalarType.empty_ctypes()
    _xkurt = utils._EngineFloat64ScalarType.empty_ctypes()
    _xmin = utils._EngineFloat64ScalarType.empty_ctypes()
    _xmax = utils._EngineFloat64ScalarType.empty_ctypes()
    _lrcomm_py = utils._EngineFloat64ArrayType.get_shape(rcomm, {'fun_name': fun_name, 'entity': 'rcomm'}, exp_rank=1, allow_none=True)[0]
    _lrcomm = utils._EngineIntScalarType(
        _lrcomm_py, {'fun_name': fun_name, 'entity': '_lrcomm'},
    ).to_ctypes()
    _pn_py = utils._EngineIntScalarType.to_py(_pn)
    if _pn_py != 0:
        if rcomm is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'rcomm'},
                'must not be None when pn != 0',
            )
    if rcomm is not None:
        _rcomm_dim1 = 20
    else:
        _rcomm_dim1 = 0
    exp_shape_rcomm = (_rcomm_dim1,)
    _rcomm_cls = utils._EngineFloat64ArrayType(
        rcomm, {'fun_name': fun_name, 'entity': 'rcomm'},
        exp_shape=exp_shape_rcomm,
        check_contents=_pn_py != 0,
        allow_none=True,
    )
    _rcomm = _rcomm_cls.to_ctypes(can_reference_contents=_pn_py != 0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _nb,
        _x,
        _lwt,
        _wt,
        _pn,
        _xmean,
        _xsd,
        _xskew,
        _xkurt,
        _xmin,
        _xmax,
        _rcomm,
        _lrcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nb >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 41 and _ifmt == 41:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wt[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if wt is not None then wt[i-1] >= 0, for i = 1,2,...,nb.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 51 and _ifmt == 51:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, pn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: pn >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 52 and _ifmt == 52:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, pn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On exit from previous call, pn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if pn > 0, pn must be unchanged since previous call.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 53 and _ifmt == 53:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the number of valid observations is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 54 and _ifmt == 54:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, pn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if rcomm is None then pn = 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 71 and _ifmt == 71:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On exit we were unable to calculate xskew or xkurt.',
                    'A value of 0 has been returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 72 and _ifmt == 72:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On exit we were unable to calculate xsd, xskew or xkurt.',
                    'A value of 0 has been returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 121 and _ifmt == 121:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'rcomm has been corrupted between calls.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _pn_py = utils._EngineIntScalarType.to_py(_pn)
        if rcomm is not None:
            _rcomm_cls.refresh_py(_rcomm)
        _xmean_py = utils._EngineFloat64ScalarType.to_py(_xmean)
        _xsd_py = utils._EngineFloat64ScalarType.to_py(_xsd)
        _xskew_py = utils._EngineFloat64ScalarType.to_py(_xskew)
        _xkurt_py = utils._EngineFloat64ScalarType.to_py(_xkurt)
        _xmin_py = utils._EngineFloat64ScalarType.to_py(_xmin)
        _xmax_py = utils._EngineFloat64ScalarType.to_py(_xmax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['pn', 'xmean', 'xsd', 'xskew', 'xkurt', 'xmin', 'xmax'],
        )(
            _pn_py,
            _xmean_py,
            _xsd_py,
            _xskew_py,
            _xkurt_py,
            _xmin_py,
            _xmax_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def summary_onevar_combine(b, mrcomm, rcomm):
    u"""
    Combines multiple sets of summary information, for use after
    ``summary_onevar``.

    For full information please refer to the NAG Library document for
    g01au

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01auf.html

    Parameters
    ----------
    b : int
        b, the number of blocks the full dataset was split into.

    mrcomm : float, shape (20, `b`)
        The jth column of `mrcomm` must contain the information returned
        in rcomm from one of the runs of ``summary_onevar``.

    rcomm : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `rcomm` is not None: 20; otherwise: 0.

        On exit, if not None on entry: an amalgamation of the
        information held in `mrcomm`. This is in the same format as
        rcomm from ``summary_onevar``.

        If lrcomm = 0, `rcomm` is not referenced.

    Returns
    -------
    pn : int
        The number of valid observations, that is the number of
        observations with w_i > 0, for i = 1,2,...,n.

    xmean : float
        x_, the mean.

    xsd : float
        s_2, the standard deviation.

    xskew : float
        s_3, the coefficient of skewness.

    xkurt : float
        s_4, the coefficient of kurtosis.

    xmin : float
        The smallest value.

    xmax : float
        The largest value.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `b` = *<value>*.

            Constraint: `b` >= 1.

        (`errno` 21)
            On entry, `mrcomm` is not in the expected format.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 31)
            On entry, the number of valid observations is zero.

        (`errno` 51)
            On exit we were unable to calculate `xskew` or `xkurt`. A
            value of 0 has been returned.

        (`errno` 52)
            On exit we were unable to calculate `xsd`, `xskew` or
            `xkurt`. A value of 0 has been returned.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.summary_onevar_combine'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01auft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, b.
        utils._EngineFloat64ArrayType.c_type, # Input, mrcomm[ldmrcomm, b].
        utils._BYREF_INT, # Output, pn.
        utils._BYREF_FLOAT64, # Output, xmean.
        utils._BYREF_FLOAT64, # Output, xsd.
        utils._BYREF_FLOAT64, # Output, xskew.
        utils._BYREF_FLOAT64, # Output, xkurt.
        utils._BYREF_FLOAT64, # Output, xmin.
        utils._BYREF_FLOAT64, # Output, xmax.
        utils._EngineFloat64ArrayType.c_type, # Output, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._BYREF_INT, # Input, ldmrcomm.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((mrcomm, 'mrcomm'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _b = utils._EngineIntScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _ldmrcomm_py = utils._EngineFloat64ArrayType.get_nd(mrcomm, {'fun_name': fun_name, 'entity': 'mrcomm'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldmrcomm = utils._EngineIntScalarType(
        _ldmrcomm_py, {'fun_name': fun_name, 'entity': '_ldmrcomm'},
    ).to_ctypes()
    _b_py = utils._EngineIntScalarType.to_py(_b)
    _mrcomm = utils._EngineFloat64ArrayType(
        mrcomm, {'fun_name': fun_name, 'entity': 'mrcomm'},
        exp_shape=(20, _b_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _pn = utils._EngineIntScalarType.empty_ctypes()
    _xmean = utils._EngineFloat64ScalarType.empty_ctypes()
    _xsd = utils._EngineFloat64ScalarType.empty_ctypes()
    _xskew = utils._EngineFloat64ScalarType.empty_ctypes()
    _xkurt = utils._EngineFloat64ScalarType.empty_ctypes()
    _xmin = utils._EngineFloat64ScalarType.empty_ctypes()
    _xmax = utils._EngineFloat64ScalarType.empty_ctypes()
    if rcomm is not None:
        _lrcomm = utils._EngineIntScalarType.c_type(20)
    else:
        _lrcomm = utils._EngineIntScalarType.c_type(0)
    if rcomm is not None:
        _rcomm_dim1 = 20
    else:
        _rcomm_dim1 = 0
    exp_shape_rcomm = (_rcomm_dim1,)
    _rcomm_cls = utils._EngineFloat64ArrayType(
        rcomm, {'fun_name': fun_name, 'entity': 'rcomm'},
        exp_shape=exp_shape_rcomm,
        check_contents=False,
        allow_none=True,
    )
    _rcomm = _rcomm_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _b,
        _mrcomm,
        _pn,
        _xmean,
        _xsd,
        _xskew,
        _xkurt,
        _xmin,
        _xmax,
        _rcomm,
        _lrcomm,
        _ldmrcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: b >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mrcomm is not in the expected format.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 31 and _ifmt == 31:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the number of valid observations is zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 51 and _ifmt == 51:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On exit we were unable to calculate xskew or xkurt.',
                    'A value of 0 has been returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 52 and _ifmt == 52:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On exit we were unable to calculate xsd, xskew or xkurt.',
                    'A value of 0 has been returned.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        if rcomm is not None:
            _rcomm_cls.refresh_py(_rcomm)
        _pn_py = utils._EngineIntScalarType.to_py(_pn)
        _xmean_py = utils._EngineFloat64ScalarType.to_py(_xmean)
        _xsd_py = utils._EngineFloat64ScalarType.to_py(_xsd)
        _xskew_py = utils._EngineFloat64ScalarType.to_py(_xskew)
        _xkurt_py = utils._EngineFloat64ScalarType.to_py(_xkurt)
        _xmin_py = utils._EngineFloat64ScalarType.to_py(_xmin)
        _xmax_py = utils._EngineFloat64ScalarType.to_py(_xmax)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['pn', 'xmean', 'xsd', 'xskew', 'xkurt', 'xmin', 'xmax'],
        )(
            _pn_py,
            _xmean_py,
            _xsd_py,
            _xskew_py,
            _xkurt_py,
            _xmin_py,
            _xmax_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_binomial(n, p, k):
    u"""
    Binomial distribution function.

    For full information please refer to the NAG Library document for
    g01bj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01bjf.html

    Parameters
    ----------
    n : int
        The parameter n of the binomial distribution.

    p : float
        The parameter p of the binomial distribution.

    k : int
        The integer k which defines the required probabilities.

    Returns
    -------
    plek : float
        The lower tail probability, Prob{X <= k}.

    pgtk : float
        The upper tail probability, Prob{X > k}.

    peqk : float
        The point probability, Prob{X = k}.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 2)
            On entry, `p` = *<value>*.

            Constraint: `p` < 1.0.

        (`errno` 2)
            On entry, `p` = *<value>*.

            Constraint: `p` > 0.0.

        (`errno` 3)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 3)
            On entry, `k` = *<value>* and `n` = *<value>*.

            Constraint: `k` <= `n`.

        (`errno` 4)
            On entry, `n` is too large to be represented exactly as a
            double precision number.

        (`errno` 5)
            On entry, the variance (= np(1-p)) exceeds 10^6.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_binomial'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01bjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Output, plek.
        utils._BYREF_FLOAT64, # Output, pgtk.
        utils._BYREF_FLOAT64, # Output, peqk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _plek = utils._EngineFloat64ScalarType.empty_ctypes()
    _pgtk = utils._EngineFloat64ScalarType.empty_ctypes()
    _peqk = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _p,
        _k,
        _plek,
        _pgtk,
        _peqk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: k <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n is too large to be represented exactly as a double precision',
                    'number.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the variance (= np(1-p)) exceeds 10^6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _plek_py = utils._EngineFloat64ScalarType.to_py(_plek)
        _pgtk_py = utils._EngineFloat64ScalarType.to_py(_pgtk)
        _peqk_py = utils._EngineFloat64ScalarType.to_py(_peqk)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['plek', 'pgtk', 'peqk'],
        )(
            _plek_py,
            _pgtk_py,
            _peqk_py,
        )
    _err_class.manage()
    return return_data

def prob_poisson(rlamda, k):
    u"""
    Poisson distribution function.

    For full information please refer to the NAG Library document for
    g01bk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01bkf.html

    Parameters
    ----------
    rlamda : float
        The parameter lambda of the Poisson distribution.

    k : int
        The integer k which defines the required probabilities.

    Returns
    -------
    plek : float
        The lower tail probability, Prob{X <= k}.

    pgtk : float
        The upper tail probability, Prob{X > k}.

    peqk : float
        The point probability, Prob{X = k}.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `rlamda` = *<value>*.

            Constraint: `rlamda` > 0.0.

        (`errno` 2)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 3)
            On entry, `rlamda` = *<value>*.

            Constraint: `rlamda` <= 10^6.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_poisson'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01bkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, rlamda.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Output, plek.
        utils._BYREF_FLOAT64, # Output, pgtk.
        utils._BYREF_FLOAT64, # Output, peqk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _rlamda = utils._EngineFloat64ScalarType(
        rlamda, {'fun_name': fun_name, 'entity': 'rlamda'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _plek = utils._EngineFloat64ScalarType.empty_ctypes()
    _pgtk = utils._EngineFloat64ScalarType.empty_ctypes()
    _peqk = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _rlamda,
        _k,
        _plek,
        _pgtk,
        _peqk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rlamda = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rlamda > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rlamda = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rlamda <= 10^6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _plek_py = utils._EngineFloat64ScalarType.to_py(_plek)
        _pgtk_py = utils._EngineFloat64ScalarType.to_py(_pgtk)
        _peqk_py = utils._EngineFloat64ScalarType.to_py(_peqk)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['plek', 'pgtk', 'peqk'],
        )(
            _plek_py,
            _pgtk_py,
            _peqk_py,
        )
    _err_class.manage()
    return return_data

def prob_hypergeom(n, l, m, k):
    u"""
    Hypergeometric distribution function.

    For full information please refer to the NAG Library document for
    g01bl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01blf.html

    Parameters
    ----------
    n : int
        The parameter n of the hypergeometric distribution.

    l : int
        The parameter l of the hypergeometric distribution.

    m : int
        The parameter m of the hypergeometric distribution.

    k : int
        The integer k which defines the required probabilities.

    Returns
    -------
    plek : float
        The lower tail probability, Prob{X <= k}.

    pgtk : float
        The upper tail probability, Prob{X > k}.

    peqk : float
        The point probability, Prob{X = k}.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 0.

        (`errno` 2)
            On entry, `l` = *<value>* and `n` = *<value>*.

            Constraint: `l` <= `n`.

        (`errno` 2)
            On entry, `l` = *<value>*.

            Constraint: `l` >= 0.

        (`errno` 3)
            On entry, `m` = *<value>* and `n` = *<value>*.

            Constraint: `m` <= `n`.

        (`errno` 3)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 0.

        (`errno` 4)
            On entry, `k` = *<value>*, `l` = *<value>*, `m` = *<value>*
            and `l`+`m`-`n` = *<value>*.

            Constraint: `k` >= `l`+`m`-`n`.

        (`errno` 4)
            On entry, `k` = *<value>* and `m` = *<value>*.

            Constraint: `k` <= `m`.

        (`errno` 4)
            On entry, `k` = *<value>* and `l` = *<value>*.

            Constraint: `k` <= `l`.

        (`errno` 4)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 5)
            On entry, `n` is too large to be represented exactly as a
            double precision number.

        (`errno` 6)
            On entry, the variance = l*m*(n-l)*(n-m)/(n^2*(n-1)) exceeds
            10^6.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_hypergeom'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01blft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, l.
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_FLOAT64, # Output, plek.
        utils._BYREF_FLOAT64, # Output, pgtk.
        utils._BYREF_FLOAT64, # Output, peqk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _plek = utils._EngineFloat64ScalarType.empty_ctypes()
    _pgtk = utils._EngineFloat64ScalarType.empty_ctypes()
    _peqk = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _l,
        _m,
        _k,
        _plek,
        _pgtk,
        _peqk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 89998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, l = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: l <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: l >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 89997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s} and n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: m <= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 69996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}, l = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'm = {:s} and l+m-n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[4]),
                    ),
                    'Constraint: k >= l+m-n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 79996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s} and m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: k <= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 89996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s} and l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: k <= l.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n is too large to be represented exactly as a double precision',
                    'number.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the variance = l*m*(n-l)*(n-m)/(n^2*(n-1)) exceeds 10^6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _plek_py = utils._EngineFloat64ScalarType.to_py(_plek)
        _pgtk_py = utils._EngineFloat64ScalarType.to_py(_pgtk)
        _peqk_py = utils._EngineFloat64ScalarType.to_py(_peqk)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['plek', 'pgtk', 'peqk'],
        )(
            _plek_py,
            _pgtk_py,
            _peqk_py,
        )
    _err_class.manage()
    return return_data

def normal_scores_exact(n, pp, etol):
    u"""
    Normal scores, accurate values.

    For full information please refer to the NAG Library document for
    g01da

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01daf.html

    Parameters
    ----------
    n : int
        n, the size of the set.

    pp : float, shape (`n`), modified in place
        On exit: the Normal scores. `pp`[i-1] contains the value
        E(x_(i)), for i = 1,2,...,n.

    etol : float
        The maximum value for the estimated absolute error in the
        computed scores.

    Returns
    -------
    errest : float
        A computed estimate of the maximum error in the computed scores
        (see [Accuracy]).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.

        (`errno` 2)
            On entry, `etol` = *<value>*.

            Constraint: `etol` > 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The function was unable to estimate the scores with
            estimated error less than `etol`. The best result obtained
            is returned together with the associated value of `errest`.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.normal_scores_exact'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01daft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, pp[n].
        utils._BYREF_FLOAT64, # Input, etol.
        utils._BYREF_FLOAT64, # Output, errest.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._BYREF_INT, # Input, iw.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _iemode = utils._EngineIntScalarType.c_type(0)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _pp_cls = utils._EngineFloat64ArrayType(
        pp, {'fun_name': fun_name, 'entity': 'pp'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _pp = _pp_cls.to_ctypes(can_reference_contents=False)
    _etol = utils._EngineFloat64ScalarType(
        etol, {'fun_name': fun_name, 'entity': 'etol'},
    ).to_ctypes()
    _errest = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _iw = utils._EngineIntScalarType.c_type(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _n,
        _pp,
        _etol,
        _errest,
        _work,
        _iw,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, etol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: etol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function was unable to estimate the scores with estimated',
                    'error less than etol. The best result obtained is returned together',
                    'with the associated value of errest.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _pp_cls.refresh_py(_pp)
        _errest_py = utils._EngineFloat64ScalarType.to_py(_errest)
        return_data = _errest_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['errest'],
            )(
                _errest_py,
            )
    _err_class.manage()
    return return_data

def normal_scores_approx(n, pp):
    u"""
    Normal scores, approximate values.

    For full information please refer to the NAG Library document for
    g01db

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01dbf.html

    Parameters
    ----------
    n : int
        n, the size of the sample.

    pp : float, shape (`n`), modified in place
        On exit: the Normal scores. `pp`[i-1] contains the value
        E(x_(i)), for i = 1,2,...,n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

    Notes
    -----
    No equivalent traditional C interface for this routine exists in the
    NAG Library.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.normal_scores_approx'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01dbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Output, pp[n].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _pp_cls = utils._EngineFloat64ArrayType(
        pp, {'fun_name': fun_name, 'entity': 'pp'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _pp = _pp_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _pp,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _pp_cls.refresh_py(_pp)
        return_data = None
    _err_class.manage()
    return return_data

def normal_scores_var(n, exp1, exp2, sumssq, vec):
    u"""
    Normal scores, approximate variance-covariance matrix.

    For full information please refer to the NAG Library document for
    g01dc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01dcf.html

    Parameters
    ----------
    n : int
        n, the sample size.

    exp1 : float
        The expected value of the largest Normal order statistic, m_n,
        from a sample of size n.

    exp2 : float
        The expected value of the second largest Normal order statistic,
        m_{n-1}, from a sample of size n.

    sumssq : float
        The sum of squares of the expected values of the Normal order
        statistics from a sample of size n.

    vec : float, shape (`n`*(`n`+1)/2), modified in place
        On exit: the upper triangle of the n by n variance-covariance
        matrix packed by column. Thus element V_{ij} is stored in
        `vec`[i+j*(j-1)/2-1], for 1 <= i <= j <= n.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 0.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.normal_scores_var'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01dcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, exp1.
        utils._BYREF_FLOAT64, # Input, exp2.
        utils._BYREF_FLOAT64, # Input, sumssq.
        utils._EngineFloat64ArrayType.c_type, # Output, vec[n*(n+1)//2].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _exp1 = utils._EngineFloat64ScalarType(
        exp1, {'fun_name': fun_name, 'entity': 'exp1'},
    ).to_ctypes()
    _exp2 = utils._EngineFloat64ScalarType(
        exp2, {'fun_name': fun_name, 'entity': 'exp2'},
    ).to_ctypes()
    _sumssq = utils._EngineFloat64ScalarType(
        sumssq, {'fun_name': fun_name, 'entity': 'sumssq'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _vec_cls = utils._EngineFloat64ArrayType(
        vec, {'fun_name': fun_name, 'entity': 'vec'},
        exp_shape=(_n_py*(_n_py+1)//2,),
        check_contents=False,
    )
    _vec = _vec_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _exp1,
        _exp2,
        _sumssq,
        _vec,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _vec_cls.refresh_py(_vec)
        return_data = None
    _err_class.manage()
    return return_data

def test_shapiro_wilk(x, n, calwts, a):
    u"""
    Shapiro and Wilk's W test for Normality.

    For full information please refer to the NAG Library document for
    g01dd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01ddf.html

    Parameters
    ----------
    x : float, shape (`n`)
        The ordered sample values, x_i, for i = 1,2,...,n.

    n : int
        n, the sample size.

    calwts : bool
        Must be set to True if you wish ``test_shapiro_wilk`` to
        calculate the elements of `a`.

        `calwts` should be set to False if you have saved the values in
        `a` from a previous call to ``test_shapiro_wilk``.

        If in doubt, set `calwts` equal to True.

    a : float, shape (`n`), modified in place
        On entry: if `calwts` has been set to False then before entry
        `a` must contain the n weights as calculated in a previous call
        to ``test_shapiro_wilk``, otherwise `a` need not be set.

        On exit: the n weights required to calculate `w`.

    Returns
    -------
    w : float
        The value of the statistic, `w`.

    pw : float
        The significance level of `w`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 3.

        (`errno` 2)
            On entry, `n` = *<value>*.

            Constraint: `n` <= 5000.

        (`errno` 3)
            On entry, elements of `x` not in order. `x`[*<value>*] =
            *<value>*, `x`[*<value>*] = *<value>*, `x`[*<value>*] =
            *<value>*.

        (`errno` 3)
            On entry, all elements of `x` are equal.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.test_shapiro_wilk'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01ddft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_LOGICAL, # Input, calwts.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, a[n].
        utils._BYREF_FLOAT64, # Output, w.
        utils._BYREF_FLOAT64, # Output, pw.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _calwts = utils._EngineLogicalScalarType(
        calwts, {'fun_name': fun_name, 'entity': 'calwts'},
    ).to_ctypes()
    _a_cls = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py,),
    )
    _a = _a_cls.to_ctypes()
    _w = utils._EngineFloat64ScalarType.empty_ctypes()
    _pw = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _n,
        _calwts,
        _a,
        _w,
        _pw,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 2 and _ifmt == 2:
            _ierr_fl = 1
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 3.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _ierr_fl = 2
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n <= 5000.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 1:
            _ierr_fl = 3
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, elements of x not in order.',
                    'x[{:s}] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'x[{:s}] = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[3]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[4]),
                    ),
                    'x[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[5]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[6]),
                    ),
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 11 and _ifmt == 11:
            _ierr_fl = 3
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, all elements of x are equal.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _a_cls.refresh_py(_a)
        _w_py = utils._EngineFloat64ScalarType.to_py(_w)
        _pw_py = utils._EngineFloat64ScalarType.to_py(_pw)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['w', 'pw'],
        )(
            _w_py,
            _pw_py,
        )
    _err_class.manage()
    return return_data

def ranks_and_scores(scores, ties, n, x, r):
    u"""
    Ranks, Normal scores, approximate Normal scores or exponential
    (Savage) scores.

    For full information please refer to the NAG Library document for
    g01dh

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01dhf.html

    Parameters
    ----------
    scores : str, length 1
        Indicates which of the following scores are required.

        `scores` = 'R'

          The ranks.

        `scores` = 'N'

          The Normal scores, that is the expected value of the Normal
          order statistics.

        `scores` = 'B'

          The Blom version of the Normal scores.

        `scores` = 'T'

          The Tukey version of the Normal scores.

        `scores` = 'V'

          The van der Waerden version of the Normal scores.

        `scores` = 'S'

          The Savage scores, that is the expected value of the
          exponential order statistics.

    ties : str, length 1
        Indicates which of the following methods is to be used to assign
        scores to tied observations.

        `ties` = 'A'

          The average of the scores for tied observations is used.

        `ties` = 'L'

          The lowest score in the group of ties is used.

        `ties` = 'H'

          The highest score in the group of ties is used.

        `ties` = 'N'

          The nonrepeatable random number generator is used to randomly
          untie any group of tied observations.

        `ties` = 'R'

          The repeatable random number generator is used to randomly
          untie any group of tied observations.

        `ties` = 'I'

          Any ties are ignored, that is the scores are assigned to tied
          observations in the order that they appear in the data.

    n : int
        n, the number of observations.

    x : float, shape (`n`)
        The sample of observations, x_i, for i = 1,2,...,n.

    r : float, shape (`n`), modified in place
        On exit: contains the scores, s_i, for i = 1,2,...,n, as
        specified by `scores`.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `scores` = *<value>*.

            Constraint: `scores` = 'R', 'N', 'B', 'T', 'V' or 'S'.

        (`errno` 1)
            On entry, `ties` = *<value>*.

            Constraint: `ties` = 'A', 'L', 'H', 'R' or 'I'.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.ranks_and_scores'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01dhft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, scores(len=1).
        utils._BYREF_STRING, # Input, ties(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[n].
        utils._EngineFloat64ArrayType.c_type, # Output, r[n].
        utils._EngineIntArrayType.c_type, # Workspace, iwrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of scores.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of ties.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _scores = utils._EngineCharScalarType(
        scores, {'fun_name': fun_name, 'entity': 'scores'}, exp_chrlen=1,
    ).to_ctypes()
    _ties = utils._EngineCharScalarType(
        ties, {'fun_name': fun_name, 'entity': 'ties'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _r_cls = utils._EngineFloat64ArrayType(
        r, {'fun_name': fun_name, 'entity': 'r'},
        exp_shape=(_n_py,),
        check_contents=False,
    )
    _r = _r_cls.to_ctypes(can_reference_contents=False)
    _iwrk = utils._EngineIntArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _scores,
        _ties,
        _n,
        _x,
        _r,
        _iwrk,
        _errbuf,
        _ierr,
        _scores._length_,
        _ties._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, scores = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: scores = \'R\', \'N\', \'B\', \'T\', \'V\' or \'S\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ties = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ties = \'A\', \'L\', \'H\', \'R\' or \'I\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _r_cls.refresh_py(_r)
        return_data = None
    _err_class.manage()
    return return_data

def prob_normal(tail, x):
    u"""
    Computes probabilities for the standard Normal distribution.

    For full information please refer to the NAG Library document for
    g01ea

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01eaf.html

    Parameters
    ----------
    tail : str, length 1
        Indicates which tail the returned probability should represent.

        `tail` = 'L'

          The lower tail probability is returned, i.e., P(X <= x).

        `tail` = 'U'

          The upper tail probability is returned, i.e., P(X >= x).

        `tail` = 'S'

          The two tail (significance level) probability is returned,
          i.e., P(X >= |x|)+P(X <= -|x|).

        `tail` = 'C'

          The two tail (confidence interval) probability is returned,
          i.e., P(X <= |x|)-P(X <= -|x|).

    x : float
        x, the value of the standard Normal variate.

    Returns
    -------
    p : float
        A one or two tail probability for the standard Normal
        distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'L', 'U', 'S' or 'C'.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_normal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01eaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _tail,
        _x,
        _p,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'L\', \'U\', \'S\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_students_t(tail, t, df):
    u"""
    Computes probabilities for Student's t-distribution.

    For full information please refer to the NAG Library document for
    g01eb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01ebf.html

    Parameters
    ----------
    tail : str, length 1
        Indicates which tail the returned probability should represent.

        `tail` = 'U'

          The upper tail probability is returned, i.e., P(T >= t:nu).

        `tail` = 'S'

          The two tail (significance level) probability is returned,
          i.e., P(T >= |t|:nu)+P(T <= -|t|:nu).

        `tail` = 'C'

          The two tail (confidence interval) probability is returned,
          i.e., P(T <= |t|:nu)-P(T <= -|t|:nu).

        `tail` = 'L'

          The lower tail probability is returned, i.e., P(T <= t:nu).

    t : float
        t, the value of the Student's t variate.

    df : float
        nu, the degrees of freedom of the Student's t-distribution.

    Returns
    -------
    p : float
        Either the lower tail, upper tail or two tail probability for
        the Student's t-distribution, depending on the value of `tail`
        supplied.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'L', 'U', 'S' or 'C'.

        (`errno` 2)
            On entry, `df` = *<value>*.

            Constraint: `df` >= 1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_students_t'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01ebft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, df.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _df = utils._EngineFloat64ScalarType(
        df, {'fun_name': fun_name, 'entity': 'df'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _tail,
        _t,
        _df,
        _p,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'L\', \'U\', \'S\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: df >= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_chisq(tail, x, df):
    u"""
    Computes probabilities for chi^2 distribution.

    For full information please refer to the NAG Library document for
    g01ec

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01ecf.html

    Parameters
    ----------
    tail : str, length 1
        Indicates whether the upper or lower tail probability is
        required.

        `tail` = 'L'

          The lower tail probability is returned, i.e., P(X <= x:nu).

        `tail` = 'U'

          The upper tail probability is returned, i.e., P(X >= x:nu).

    x : float
        x, the value of the chi^2 variate with nu degrees of freedom.

    df : float
        nu, the degrees of freedom of the chi^2-distribution.

    Returns
    -------
    p : float
        The lower or upper tail probability for the chi^2-distribution,
        depending on the value of `tail` supplied.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'L' or 'U'.

        (`errno` 2)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.

        (`errno` 3)
            On entry, `df` = *<value>*.

            Constraint: `df` > 0.0.

        (`errno` 4)
            The series used to calculate the gamma probabilities has
            failed to converge. The result returned should represent an
            approximation to the solution.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_chisq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01ecft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, df.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _df = utils._EngineFloat64ScalarType(
        df, {'fun_name': fun_name, 'entity': 'df'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _tail,
        _x,
        _df,
        _p,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'L\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: df > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The series used to calculate the gamma probabilities has failed to converge.',
                    'The result returned should represent an approximation to the solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_f(tail, f, df1, df2):
    u"""
    Computes probabilities for F-distribution.

    For full information please refer to the NAG Library document for
    g01ed

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01edf.html

    Parameters
    ----------
    tail : str, length 1
        Indicates whether an upper or lower tail probability is
        required.

        `tail` = 'L'

          The lower tail probability is returned, i.e., P(F <=
          f:nu_1,nu_2).

        `tail` = 'U'

          The upper tail probability is returned, i.e., P(F >=
          f:nu_1,nu_2).

    f : float
        f, the value of the F variate.

    df1 : float
        The degrees of freedom of the numerator variance, nu_1.

    df2 : float
        The degrees of freedom of the denominator variance, nu_2.

    Returns
    -------
    p : float
        The probability for the lower or upper tail of the F or
        variance-ratio distribution with real degrees of freedom.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'L' or 'U'.

        (`errno` 2)
            On entry, `f` = *<value>*.

            Constraint: `f` >= 0.0.

        (`errno` 3)
            On entry, `df1` = *<value>* and `df2` = *<value>*.

            Constraint: `df1` > 0.0 and `df2` > 0.0.

        (`errno` 4)
            The probability is too close to 0.0 or 1.0. `f` is too far
            out into the tails for the probability to be evaluated
            exactly. The result tends to approach 1.0 if f is large, or
            0.0 if f is small. The result returned is a good
            approximation to the required solution.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_f'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01edft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_FLOAT64, # Input, f.
        utils._BYREF_FLOAT64, # Input, df1.
        utils._BYREF_FLOAT64, # Input, df2.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _f = utils._EngineFloat64ScalarType(
        f, {'fun_name': fun_name, 'entity': 'f'},
    ).to_ctypes()
    _df1 = utils._EngineFloat64ScalarType(
        df1, {'fun_name': fun_name, 'entity': 'df1'},
    ).to_ctypes()
    _df2 = utils._EngineFloat64ScalarType(
        df2, {'fun_name': fun_name, 'entity': 'df2'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _tail,
        _f,
        _df1,
        _df2,
        _p,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'L\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, f = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: f >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df1 = {:s} and df2 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: df1 > 0.0 and df2 > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The probability is too close to 0.0 or 1.0.',
                    'f is too far out into the tails for the probability to be evaluated exactly.',
                    'The result tends to approach 1.0 if f is large, or 0.0 if f is small.',
                    'The result returned is a good approximation to the required solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_beta(x, a, b):
    u"""
    Computes upper and lower tail probabilities and probability density
    function for the beta distribution.

    For full information please refer to the NAG Library document for
    g01ee

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01eef.html

    Parameters
    ----------
    x : float
        beta, the value of the beta variate.

    a : float
        a, the first parameter of the required beta distribution.

    b : float
        b, the second parameter of the required beta distribution.

    Returns
    -------
    p : float
        The lower tail probability, P(B <= beta:a,b).

    q : float
        The upper tail probability, P(B >= beta:a,b).

    pdf : float
        The probability density function, f(B:a,b).

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.

        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` <= 1.0.

        (`errno` 2)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `a` > 0.0.

        (`errno` 2)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `b` > 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_beta'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01eeft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Output, p.
        utils._BYREF_FLOAT64, # Output, q.
        utils._BYREF_FLOAT64, # Output, pdf.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _q = utils._EngineFloat64ScalarType.empty_ctypes()
    _pdf = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _a,
        _b,
        _p,
        _q,
        _pdf,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 11 and _ifmt == 11:
            _ierr_fl = 1
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= 0.0.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _ierr_fl = 1
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x <= 1.0.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _ierr_fl = 2
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a > 0.0.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 31 and _ifmt == 31:
            _ierr_fl = 2
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: b > 0.0.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        _q_py = utils._EngineFloat64ScalarType.to_py(_q)
        _pdf_py = utils._EngineFloat64ScalarType.to_py(_pdf)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['p', 'q', 'pdf'],
        )(
            _p_py,
            _q_py,
            _pdf_py,
        )
    _err_class.manage()
    return return_data

def prob_gamma(tail, g, a, b):
    u"""
    Computes probabilities for the gamma distribution.

    For full information please refer to the NAG Library document for
    g01ef

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01eff.html

    Parameters
    ----------
    tail : str, length 1
        Indicates whether an upper or lower tail probability is
        required.

        `tail` = 'L'

          The lower tail probability is returned, that is P(G <=
          g:alpha,beta).

        `tail` = 'U'

          The upper tail probability is returned, that is P(G >=
          g:alpha,beta).

    g : float
        g, the value of the gamma variate.

    a : float
        The parameter alpha of the gamma distribution.

    b : float
        The parameter beta of the gamma distribution.

    Returns
    -------
    p : float
        The lower or upper tail probability of the gamma distribution,
        with parameters alpha and beta.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'L' or 'U'.

        (`errno` 2)
            On entry, `g` = *<value>*.

            Constraint: `g` >= 0.0.

        (`errno` 3)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `a` > 0.0 and `b` > 0.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The algorithm has failed to converge in *<value>*
            iterations. The probability returned should be a reasonable
            approximation to the solution.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_gamma'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01efft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_FLOAT64, # Input, g.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _g = utils._EngineFloat64ScalarType(
        g, {'fun_name': fun_name, 'entity': 'g'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _tail,
        _g,
        _a,
        _b,
        _p,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'L\' or \'U\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, g = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: g >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a > 0.0 and b > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm has failed to converge in {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The probability returned should be a reasonable approximation to',
                    'the solution.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['p'],
            )(
                _p_py,
            )
    _err_class.manage()
    return return_data

def prob_studentized_range(q, v, ir):
    u"""
    Computes probability for the Studentized range statistic.

    For full information please refer to the NAG Library document for
    g01em

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01emf.html

    Parameters
    ----------
    q : float
        q, the Studentized range statistic.

    v : float
        v, the number of degrees of freedom for the experimental error.

    ir : int
        r, the number of groups.

    Returns
    -------
    p : float
        The probability associated with the lower tail of the
        distribution of the Studentized range statistic.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `q` = *<value>*.

            Constraint: `q` > 0.0.

        (`errno` 1)
            On entry, `ir` = *<value>*.

            Constraint: `ir` >= 2.

        (`errno` 1)
            On entry, `v` = *<value>*.

            Constraint: `v` >= 1.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            There is some doubt as to whether full accuracy has been
            achieved. The returned value should be a reasonable estimate
            of the true value.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_studentized_range'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01emft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, q.
        utils._BYREF_FLOAT64, # Input, v.
        utils._BYREF_INT, # Input, ir.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _q = utils._EngineFloat64ScalarType(
        q, {'fun_name': fun_name, 'entity': 'q'},
    ).to_ctypes()
    _v = utils._EngineFloat64ScalarType(
        v, {'fun_name': fun_name, 'entity': 'v'},
    ).to_ctypes()
    _ir = utils._EngineIntScalarType(
        ir, {'fun_name': fun_name, 'entity': 'ir'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _q,
        _v,
        _ir,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, q = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: q > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ir = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ir >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, v = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: v >= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There is some doubt as to whether full accuracy has been achieved.',
                    'The returned value should be a reasonable estimate of the true value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['p'],
            )(
                _p_py,
            )
    _err_class.manage()
    return return_data

def prob_durbin_watson(n, ip, d):
    u"""
    Computes bounds for the significance of a Durbin--Watson statistic.

    For full information please refer to the NAG Library document for
    g01ep

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01epf.html

    Parameters
    ----------
    n : int
        n, the number of observations used in calculating the
        Durbin--Watson statistic.

    ip : int
        p, the number of independent variables in the regression model,
        including the mean.

    d : float
        d, the Durbin--Watson statistic.

    Returns
    -------
    pdl : float
        Lower bound for the significance of the Durbin--Watson
        statistic, p_l.

    pdu : float
        Upper bound for the significance of the Durbin--Watson
        statistic, p_u.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `ip` = *<value>*.

            Constraint: `ip` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>* and `ip` = *<value>*.

            Constraint: `n` > `ip`.

        (`errno` 2)
            On entry, `d` = *<value>*.

            Constraint: `d` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_durbin_watson'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01epft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_INT, # Input, ip.
        utils._BYREF_FLOAT64, # Input, d.
        utils._BYREF_FLOAT64, # Output, pdl.
        utils._BYREF_FLOAT64, # Output, pdu.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ip = utils._EngineIntScalarType(
        ip, {'fun_name': fun_name, 'entity': 'ip'},
    ).to_ctypes()
    _d = utils._EngineFloat64ScalarType(
        d, {'fun_name': fun_name, 'entity': 'd'},
    ).to_ctypes()
    _pdl = utils._EngineFloat64ScalarType.empty_ctypes()
    _pdu = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _ip,
        _d,
        _pdl,
        _pdu,
        _work,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ip = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ip >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s} and ip = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n > ip.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, d = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: d >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _pdl_py = utils._EngineFloat64ScalarType.to_py(_pdl)
        _pdu_py = utils._EngineFloat64ScalarType.to_py(_pdu)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['pdl', 'pdu'],
        )(
            _pdl_py,
            _pdu_py,
        )
    _err_class.manage()
    return return_data

def prob_vonmises(t, vk):
    u"""
    Computes probability for von Mises distribution.

    For full information please refer to the NAG Library document for
    g01er

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01erf.html

    Parameters
    ----------
    t : float
        theta, the observed von Mises statistic measured in radians.

    vk : float
        The concentration parameter kappa, of the von Mises
        distribution.

    Returns
    -------
    p : float
        The probability associated with the lower tail of the von Mises
        distribution between -pi and pi.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `vk` = *<value>*.

            Constraint: `vk` >= 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_vonmises'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01erft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, vk.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _vk = utils._EngineFloat64ScalarType(
        vk, {'fun_name': fun_name, 'entity': 'vk'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _t,
        _vk,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, vk = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: vk >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_landau(x):
    u"""
    Landau distribution function.

    For full information please refer to the NAG Library document for
    g01et

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01etf.html

    Parameters
    ----------
    x : float
        The argument lambda of the function.

    Returns
    -------
    p : float
        The value of the Landau distribution function Phi(lambda).
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_landau'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01etft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, p.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _p,
    )
    utils._handle_cb_exception(excs)
    _p_py = utils._EngineFloat64ScalarType.to_py(_p)
    return_data = _p_py
    return return_data

def prob_vavilov(x, comm):
    u"""
    Vavilov distribution function.

    For full information please refer to the NAG Library document for
    g01eu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01euf.html

    Parameters
    ----------
    x : float
        The argument lambda of the function.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``init_vavilov``.

    Returns
    -------
    p : float
        The value of the Vavilov distribution function
        Phi_V(lambda;kappa,beta^2).

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_vavilov'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01euft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineFloat64ArrayType.c_type, # Input, rcomm[322].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rcomm',),
    )
    _rcomm = comm['rcomm']
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _p,
        _rcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the first call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_dickey_fuller_unit(method, ts_type, n, ts, nsamp, statecomm):
    u"""
    Computes probabilities for the Dickey--Fuller unit root test.

    For full information please refer to the NAG Library document for
    g01ew

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01ewf.html

    Parameters
    ----------
    method : int
        The method used to calculate the probability.

        `method` = 1

          The probability is interpolated from a look-up table, whose
          values were obtained via simulation.

        `method` = 2

          The probability is interpolated from a look-up table, whose
          values were obtained from Dickey.

        `method` = 3

          The probability is obtained via simulation.

        The probability calculated from the look-up table should give
        sufficient accuracy for most applications.

    ts_type : int
        The type of test statistic, supplied in `ts`.

    n : int
        n, the length of the time series used to calculate the test
        statistic.

    ts : float
        The Dickey--Fuller test statistic for which the probability is
        required. If

        `ts_type` = 1

          `ts` must contain tau.

        `ts_type` = 2

          `ts` must contain tau_mu.

        `ts_type` = 3

          `ts` must contain tau_tau.

        If the test statistic was calculated using
        ``tsa.uni_dickey_fuller_unit`` the value of `ts_type` and `n`
        must not change between calls to ``tsa.uni_dickey_fuller_unit``
        and ``prob_dickey_fuller_unit``.

    nsamp : int
        If `method` = 3, the number of samples used in the simulation;
        otherwise `nsamp` is not referenced and need not be set.

    statecomm : None or dict, RNG communication object, modified in
    place
        RNG communication structure.

        When `method` = 3, this argument must have been initialized by a
        prior call to ``rand.init_repeat`` or ``rand.init_nonrepeat``.

    Returns
    -------
    pint : float
        The probability associated with the lower tail of the
        distribution for the (augmented) Dickey--Fuller unit root test
        statistic supplied in `ts`.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `method` = *<value>*.

            Constraint: `method` = 1, 2 or 3.

        (`errno` 21)
            On entry, `ts_type` = *<value>*.

            Constraint: `ts_type` = 1, 2 or 3.

        (`errno` 31)
            On entry, `n` = *<value>*.

            Constraint: if `method` != 3, `n` > 0.

        (`errno` 31)
            On entry, `n` = *<value>*.

            Constraint: if `method` = 3 and `ts_type` = 1, `n` > 2.

        (`errno` 31)
            On entry, `n` = *<value>*.

            Constraint: if `method` = 3 and `ts_type` = 2, `n` > 3.

        (`errno` 31)
            On entry, `n` = *<value>*.

            Constraint: if `method` = 3 and `ts_type` = 3, `n` > 4.

        (`errno` 51)
            On entry, `nsamp` = *<value>*.

            Constraint: if `method` = 3, `nsamp` > 0.

        (`errno` 61)
            On entry, `method` = 3 and the `statecomm`['state'] vector
            has been corrupted or not initialized.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 201)
            The supplied input values were outside the range of at least
            one look-up table, therefore extrapolation was used.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_dickey_fuller_unit'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01ewft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, method.
        utils._BYREF_INT, # Input, ts_type.
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, ts.
        utils._BYREF_INT, # Input, nsamp.
        utils._EngineIntArrayType.c_type, # Input/Output, state[lstate].
        utils._BYREF_INT, # Input, lstate.
        utils._BYREF_INT, # Input, p.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_FLOAT64, # Output, pint.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _method = utils._EngineIntScalarType(
        method, {'fun_name': fun_name, 'entity': 'method'},
    ).to_ctypes()
    _ts_type = utils._EngineIntScalarType(
        ts_type, {'fun_name': fun_name, 'entity': 'ts_type'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ts = utils._EngineFloat64ScalarType(
        ts, {'fun_name': fun_name, 'entity': 'ts'},
    ).to_ctypes()
    _nsamp = utils._EngineIntScalarType(
        nsamp, {'fun_name': fun_name, 'entity': 'nsamp'},
    ).to_ctypes()
    _vl_dummy_state = utils._EngineIntArrayType.empty_ctypes(0)
    utils._check_comm(
        statecomm, {'fun_name': fun_name, 'entity': 'statecomm'},
        req_keys=('state',),
        allow_none=True,
    )
    if statecomm is None:
        _lstate_py = utils._EngineIntArrayType.get_shape(_vl_dummy_state, {'fun_name': fun_name, 'entity': '_vl_dummy_state'}, exp_rank=1)[0]
        _lstate = utils._EngineIntScalarType(
            _lstate_py, {'fun_name': fun_name, 'entity': '_lstate'},
        ).to_ctypes()
    else:
        _lstate_py = utils._EngineIntArrayType.get_shape(statecomm['state'], {'fun_name': fun_name, 'entity': 'statecomm[\'state\']'}, exp_rank=1)[0]
        _lstate = utils._EngineIntScalarType(
            _lstate_py, {'fun_name': fun_name, 'entity': '_lstate'},
        ).to_ctypes()
    _method_py = utils._EngineIntScalarType.to_py(_method)
    if _method_py == 3:
        if statecomm is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'statecomm'},
                'must not be None when method = 3',
            )
    if statecomm is None:
        _state = _vl_dummy_state
    else:
        _state = statecomm['state']
    _p = utils._EngineIntScalarType.c_type(1)
    _tol = utils._EngineFloat64ScalarType.c_type(0.0)
    _pint = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _method,
        _ts_type,
        _n,
        _ts,
        _nsamp,
        _state,
        _lstate,
        _p,
        _tol,
        _pint,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: method = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ts_type = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ts_type = 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 31 and _ifmt == 31:
            _ierr_fl = 31
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if method != 3, n > 0.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 32 and _ifmt == 32:
            _ierr_fl = 31
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if method = 3 and ts_type = 1, n > 2.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 33 and _ifmt == 33:
            _ierr_fl = 31
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if method = 3 and ts_type = 2, n > 3.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 34 and _ifmt == 34:
            _ierr_fl = 31
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if method = 3 and ts_type = 3, n > 4.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 51 and _ifmt == 51:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nsamp = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if method = 3, nsamp > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 61 and _ifmt == 61:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = 3 and the statecomm[\'state\'] vector has been',
                    'corrupted or not initialized.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 71 and _ifmt == 71:
            _ierr_fl = 61
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = 3 and the statecomm[\'state\'] vector has been',
                    'corrupted or not initialized.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 201 and _ifmt == 201:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The supplied input values were outside the range of at least one look-up',
                    'table, therefore extrapolation was used.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 202 and _ifmt == 202:
            _ierr_fl = 201
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The supplied input values were outside the range of at least one look-up',
                    'table, therefore extrapolation was used.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _pint_py = utils._EngineFloat64ScalarType.to_py(_pint)
        return_data = _pint_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['pint'],
            )(
                _pint_py,
            )
    _err_class.manage()
    return return_data

def prob_kolmogorov1(n, d):
    u"""
    Computes probabilities for the one-sample Kolmogorov--Smirnov
    distribution.

    For full information please refer to the NAG Library document for
    g01ey

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01eyf.html

    Parameters
    ----------
    n : int
        n, the number of observations in the sample.

    d : float
        Contains the test statistic, D_n^+ or D_n^-.

    Returns
    -------
    p : float
        The upper tail probability associated with the one sample
        Kolmogorov--Smirnov distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, `d` < 0.0 or `d` > 1.0: `d` = *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_kolmogorov1'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01eyft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, d.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _d = utils._EngineFloat64ScalarType(
        d, {'fun_name': fun_name, 'entity': 'd'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _d,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, d < 0.0 or d > 1.0: d = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_kolmogorov2(n1, n2, d):
    u"""
    Computes probabilities for the two-sample Kolmogorov--Smirnov
    distribution.

    For full information please refer to the NAG Library document for
    g01ez

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01ezf.html

    Parameters
    ----------
    n1 : int
        The number of observations in the first sample, n_1.

    n2 : int
        The number of observations in the second sample, n_2.

    d : float
        The test statistic D_{n_1,n_2}, for the two sample
        Kolmogorov--Smirnov goodness-of-fit test, that is the maximum
        difference between the empirical cumulative distribution
        functions (CDFs) of the two samples.

    Returns
    -------
    p : float
        The probability associated with the upper tail of the
        Kolmogorov--Smirnov two sample distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n1` = *<value>* and `n2` = *<value>*.

            Constraint: `n1` >= 1 and `n2` >= 1.

        (`errno` 2)
            On entry, `d` < 0.0 or `d` > 1.0: `d` = *<value>*.

        (`errno` 3)
            The Smirnov approximation used for large samples did not
            converge in 200 iterations. The probability is set to 1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_kolmogorov2'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01ezft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n1.
        utils._BYREF_INT, # Input, n2.
        utils._BYREF_FLOAT64, # Input, d.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n1 = utils._EngineIntScalarType(
        n1, {'fun_name': fun_name, 'entity': 'n1'},
    ).to_ctypes()
    _n2 = utils._EngineIntScalarType(
        n2, {'fun_name': fun_name, 'entity': 'n2'},
    ).to_ctypes()
    _d = utils._EngineFloat64ScalarType(
        d, {'fun_name': fun_name, 'entity': 'd'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n1,
        _n2,
        _d,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n1 = {:s} and n2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: n1 >= 1 and n2 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, d < 0.0 or d > 1.0: d = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The Smirnov approximation used for large samples did not converge in 200',
                    'iterations. The probability is set to 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def inv_cdf_normal(tail, p):
    u"""
    Computes deviates for the standard Normal distribution.

    For full information please refer to the NAG Library document for
    g01fa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01faf.html

    Parameters
    ----------
    tail : str, length 1
        Indicates which tail the supplied probability represents.

        `tail` = 'L'

          The lower probability, i.e., P(X <= x_p).

        `tail` = 'U'

          The upper probability, i.e., P(X >= x_p).

        `tail` = 'S'

          The two tail (significance level) probability, i.e., P(X >=
          |x_p|)+P(X <= -|x_p|).

        `tail` = 'C'

          The two tail (confidence interval) probability, i.e., P(X <=
          |x_p|)-P(X <= -|x_p|).

    p : float
        p, the probability from the standard Normal distribution as
        defined by `tail`.

    Returns
    -------
    x : float
        The deviate associated with the given probability of the
        standard Normal distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'L', 'U', 'S' or 'C'.

        (`errno` 2)
            On entry, `p` = *<value>*.

            Constraint: `p` < 1.0.

        (`errno` 2)
            On entry, `p` = *<value>*.

            Constraint: `p` > 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_normal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01faft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_FLOAT64, # Output, x.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _tail,
        _p,
        _x,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'L\', \'U\', \'S\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 89998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        return_data = _x_py
    _err_class.manage()
    return return_data

def inv_cdf_students_t(tail, p, df):
    u"""
    Computes deviates for Student's t-distribution.

    For full information please refer to the NAG Library document for
    g01fb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01fbf.html

    Parameters
    ----------
    tail : str, length 1
        Indicates which tail the supplied probability represents.

        `tail` = 'U'

          The upper tail probability, i.e., P(T >= t_p:nu).

        `tail` = 'L'

          The lower tail probability, i.e., P(T <= t_p:nu).

        `tail` = 'S'

          The two tail (significance level) probability, i.e., P(T >=
          |t_p|:nu)+P(T <= -|t_p|:nu).

        `tail` = 'C'

          The two tail (confidence interval) probability, i.e., P(T <=
          |t_p|:nu)-P(T <= -|t_p|:nu).

    p : float
        p, the probability from the required Student's t-distribution as
        defined by `tail`.

    df : float
        nu, the degrees of freedom of the Student's t-distribution.

    Returns
    -------
    x : float
        The deviate associated with the given tail probability of
        Student's t-distribution with real degrees of freedom.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `tail` = *<value>*.

            Constraint: `tail` = 'L', 'U', 'S' or 'C'.

        (`errno` 2)
            On entry, `p` = *<value>*.

            Constraint: `p` < 1.0.

        (`errno` 2)
            On entry, `p` = *<value>*.

            Constraint: `p` > 0.0.

        (`errno` 3)
            On entry, `df` = *<value>*.

            Constraint: `df` >= 1.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 5)
            The solution has failed to converge. However, the result
            should be a reasonable approximation.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_students_t'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01fbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_FLOAT64, # Input, df.
        utils._BYREF_FLOAT64, # Output, x.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _tail = utils._EngineCharScalarType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _df = utils._EngineFloat64ScalarType(
        df, {'fun_name': fun_name, 'entity': 'df'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _tail,
        _p,
        _df,
        _x,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'L\', \'U\', \'S\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 89999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: df >= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99996:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solution has failed to converge.',
                    'However, the result should be a reasonable approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        return_data = _x_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['x'],
            )(
                _x_py,
            )
    _err_class.manage()
    return return_data

def inv_cdf_chisq(p, df):
    u"""
    Computes deviates for the chi^2 distribution.

    For full information please refer to the NAG Library document for
    g01fc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01fcf.html

    Parameters
    ----------
    p : float
        p, the lower tail probability from the required
        chi^2-distribution.

    df : float
        nu, the degrees of freedom of the chi^2-distribution.

    Returns
    -------
    x : float
        The deviate associated with the given lower tail probability of
        the chi^2-distribution with real degrees of freedom.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` < 1.0.

        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` >= 0.0.

        (`errno` 2)
            On entry, `df` = *<value>*.

            Constraint: `df` > 0.0.

        (`errno` 3)
            The probability is too close to 0.0 or 1.0.

        (`errno` 5)
            The series used to calculate the gamma function has failed
            to converge. This is an unlikely error exit.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The algorithm has failed to converge in *<value>*
            iterations. The result should be a reasonable approximation.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_chisq'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01fcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_FLOAT64, # Input, df.
        utils._BYREF_FLOAT64, # Output, x.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _df = utils._EngineFloat64ScalarType(
        df, {'fun_name': fun_name, 'entity': 'df'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _p,
        _df,
        _x,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 89999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: df > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The probability is too close to 0.0 or 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm has failed to converge in {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'The result should be a reasonable approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The series used to calculate the gamma function has failed to converge.',
                    'This is an unlikely error exit.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        return_data = _x_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['x'],
            )(
                _x_py,
            )
    _err_class.manage()
    return return_data

def inv_cdf_f(p, df1, df2):
    u"""
    Computes deviates for the F-distribution.

    For full information please refer to the NAG Library document for
    g01fd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01fdf.html

    Parameters
    ----------
    p : float
        p, the lower tail probability from the required F-distribution.

    df1 : float
        The degrees of freedom of the numerator variance, nu_1.

    df2 : float
        The degrees of freedom of the denominator variance, nu_2.

    Returns
    -------
    x : float
        The deviate associated with the given lower tail probability of
        the F or variance-ratio distribution with real degrees of
        freedom.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` < 1.0.

        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` >= 0.0.

        (`errno` 2)
            On entry, `df1` = *<value>* and `df2` = *<value>*.

            Constraint: `df1` > 0.0 and `df2` > 0.0.

        (`errno` 4)
            The probability is too close to 0.0 or 1.0. The value of f_p
            cannot be computed. This will only occur when the large
            sample approximations are used.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The solution has failed to converge. However, the result
            should be a reasonable approximation. Alternatively,
            ``inv_cdf_beta`` can be used with a suitable setting of the
            argument tol.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_f'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01fdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_FLOAT64, # Input, df1.
        utils._BYREF_FLOAT64, # Input, df2.
        utils._BYREF_FLOAT64, # Output, x.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _df1 = utils._EngineFloat64ScalarType(
        df1, {'fun_name': fun_name, 'entity': 'df1'},
    ).to_ctypes()
    _df2 = utils._EngineFloat64ScalarType(
        df2, {'fun_name': fun_name, 'entity': 'df2'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _p,
        _df1,
        _df2,
        _x,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 89999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df1 = {:s} and df2 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: df1 > 0.0 and df2 > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solution has failed to converge.',
                    'However, the result should be a reasonable approximation.',
                    'Alternatively, inv_cdf_beta can be used with a suitable setting of the argument',
                    'tol.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The probability is too close to 0.0 or 1.0.',
                    'The value of f_p cannot be computed.',
                    'This will only occur when the large sample approximations are used.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        return_data = _x_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['x'],
            )(
                _x_py,
            )
    _err_class.manage()
    return return_data

def inv_cdf_beta(p, a, b, tol):
    u"""
    Computes deviates for the beta distribution.

    For full information please refer to the NAG Library document for
    g01fe

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01fef.html

    Parameters
    ----------
    p : float
        p, the lower tail probability from the required beta
        distribution.

    a : float
        a, the first parameter of the required beta distribution.

    b : float
        b, the second parameter of the required beta distribution.

    tol : float
        The relative accuracy required by you in the result. If
        ``inv_cdf_beta`` is entered with `tol` greater than or equal to
        1.0 or less than 10*machine precision (see
        ``machine.precision``), the value of 10*machine precision is
        used instead.

    Returns
    -------
    x : float
        The deviate associated with the given lower tail probability of
        the beta distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` >= 0.0.

        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` <= 1.0.

        (`errno` 2)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `a` > 0.0.

        (`errno` 2)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `a` <= 10^6.

        (`errno` 2)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `b` > 0.0.

        (`errno` 2)
            On entry, `a` = *<value>* and `b` = *<value>*.

            Constraint: `b` <= 10^6.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            The solution has failed to converge. However, the result
            should be a reasonable approximation. Requested accuracy not
            achieved when calculating beta probability. You should try
            setting `tol` larger.

        (`errno` 4)
            The requested accuracy has not been achieved. Use a larger
            value of `tol`. There is doubt concerning the accuracy of
            the computed result. 100 iterations of the Newton--Raphson
            method have been performed without satisfying the accuracy
            criterion (see [Further Comments]). The result should be a
            reasonable approximation of the solution.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_beta'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01feft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_FLOAT64, # Output, x.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _p,
        _a,
        _b,
        _tol,
        _x,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 11 and _ifmt == 11:
            _ierr_fl = 1
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p >= 0.0.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _ierr_fl = 1
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p <= 1.0.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _ierr_fl = 2
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a > 0.0.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 22 and _ifmt == 22:
            _ierr_fl = 2
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a <= 10^6.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 31 and _ifmt == 31:
            _ierr_fl = 2
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: b > 0.0.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 32 and _ifmt == 32:
            _ierr_fl = 2
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s} and b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: b <= 10^6.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 101 and _ifmt == 101:
            _ierr_fl = 3
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solution has failed to converge.',
                    'However, the result should be a reasonable approximation.',
                    'Requested accuracy not achieved when calculating beta probability.',
                    'You should try setting tol larger.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        elif _ierr_py == 102 and _ifmt == 102:
            _ierr_fl = 4
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The requested accuracy has not been achieved. Use a larger value of tol.',
                    'There is doubt concerning the accuracy of the computed result.',
                    '100 iterations of the Newton--Raphson method have been performed without',
                    'satisfying the accuracy criterion',
                    '(see [Further Comments]).',
                    'The result should be a reasonable approximation of the solution.',
                ],
                errcodes=(_ierr_fl, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        return_data = _x_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['x'],
            )(
                _x_py,
            )
    _err_class.manage()
    return return_data

def inv_cdf_gamma(p, a, b, tol):
    u"""
    Computes deviates for the gamma distribution.

    For full information please refer to the NAG Library document for
    g01ff

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01fff.html

    Parameters
    ----------
    p : float
        p, the lower tail probability from the required gamma
        distribution.

    a : float
        alpha, the shape parameter of the gamma distribution.

    b : float
        beta, the scale parameter of the gamma distribution.

    tol : float
        The relative accuracy required by you in the results. The
        smallest recommended value is 50*delta, where delta =
        max(10^(-18), machine precision). If ``inv_cdf_gamma`` is
        entered with `tol` less than 50*delta or greater or equal to
        1.0, then 50*delta is used instead.

    Returns
    -------
    x : float
        The deviate associated with the given lower tail probability of
        the gamma distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` < 1.0.

        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: `p` >= 0.0.

        (`errno` 2)
            On entry, `a` = *<value>*.

            Constraint: `a` <= 10^6.

        (`errno` 2)
            On entry, `b` = *<value>*.

            Constraint: `b` > 0.0.

        (`errno` 2)
            On entry, `a` = *<value>*.

            Constraint: `a` > 0.0.

        (`errno` 3)
            The probability is too close to 0.0 for the given `a` to
            enable the result to be calculated.

        (`errno` 5)
            The series used to calculate the gamma function has failed
            to converge. This is an unlikely error exit.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The algorithm has failed to converge in 100 iterations. A
            larger value of `tol` should be tried. The result may be a
            reasonable approximation.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_gamma'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01ffft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_FLOAT64, # Output, x.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _p,
        _a,
        _b,
        _tol,
        _x,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 89999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: p >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: a <= 10^6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: b > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: a > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The probability is too close to 0.0 for the given a to enable',
                    'the result to be calculated.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The algorithm has failed to converge in 100 iterations.',
                    'A larger value of tol should be tried.',
                    'The result may be a reasonable approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The series used to calculate the gamma function has failed to converge.',
                    'This is an unlikely error exit.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        return_data = _x_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['x'],
            )(
                _x_py,
            )
    _err_class.manage()
    return return_data

def inv_cdf_studentized_range(p, v, ir):
    u"""
    Computes deviates for the Studentized range statistic.

    For full information please refer to the NAG Library document for
    g01fm

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01fmf.html

    Parameters
    ----------
    p : float
        The lower tail probability for the Studentized range statistic,
        p_0.

    v : float
        v, the number of degrees of freedom.

    ir : int
        r, the number of groups.

    Returns
    -------
    x : float
        The deviate associated with the lower tail probability of the
        distribution of the Studentized range statistic.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `p` = *<value>*.

            Constraint: 0.0 < `p` < 1.0.

        (`errno` 1)
            On entry, `ir` = *<value>*.

            Constraint: `ir` >= 2.

        (`errno` 1)
            On entry, `v` = *<value>*.

            Constraint: `v` >= 1.0.

        (`errno` 2)
            The function was unable to find an upper bound for the value
            of q_0. This will be caused by p_0 being too close to 1.0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            There is some doubt as to whether full accuracy has been
            achieved. The returned value should be a reasonable estimate
            of the true value.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_studentized_range'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01fmft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, p.
        utils._BYREF_FLOAT64, # Input, v.
        utils._BYREF_INT, # Input, ir.
        utils._BYREF_FLOAT64, # Output, x.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _p = utils._EngineFloat64ScalarType(
        p, {'fun_name': fun_name, 'entity': 'p'},
    ).to_ctypes()
    _v = utils._EngineFloat64ScalarType(
        v, {'fun_name': fun_name, 'entity': 'v'},
    ).to_ctypes()
    _ir = utils._EngineIntScalarType(
        ir, {'fun_name': fun_name, 'entity': 'ir'},
    ).to_ctypes()
    _x = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _p,
        _v,
        _ir,
        _x,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, p = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 < p < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ir = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ir >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, v = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: v >= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The function was unable to find an upper bound for the value of q_0.',
                    'This will be caused by p_0 being too close to 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'There is some doubt as to whether full accuracy has been achieved.',
                    'The returned value should be a reasonable estimate of the true value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_py = utils._EngineFloat64ScalarType.to_py(_x)
        return_data = _x_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['x'],
            )(
                _x_py,
            )
    _err_class.manage()
    return return_data

def inv_cdf_landau(x):
    u"""
    Landau inverse function Psi(x).

    For full information please refer to the NAG Library document for
    g01ft

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01ftf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    invlan : float
        The value of the inverse Phi^-1(x) of the Landau distribution
        function.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` < 1.0.

        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` > 0.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_landau'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01ftft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, invlan.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _invlan = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _invlan,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x < 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _invlan_py = utils._EngineFloat64ScalarType.to_py(_invlan)
        return_data = _invlan_py
    _err_class.manage()
    return return_data

def prob_students_t_noncentral(t, df, delta, tol, maxit):
    u"""
    Computes probabilities for the non-central Student's t-distribution.

    For full information please refer to the NAG Library document for
    g01gb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01gbf.html

    Parameters
    ----------
    t : float
        t, the deviate from the Student's t-distribution with nu degrees
        of freedom.

    df : float
        nu, the degrees of freedom of the Student's t-distribution.

    delta : float
        delta, the noncentrality parameter of the Students
        t-distribution.

    tol : float
        The absolute accuracy required by you in the results. If
        ``prob_students_t_noncentral`` is entered with `tol` greater
        than or equal to 1.0 or less than 10*machine precision (see
        ``machine.precision``), the value of 10*machine precision is
        used instead.

    maxit : int
        The maximum number of terms that are used in each of the
        summations.

    Returns
    -------
    p : float
        The lower tail probability for the noncentral Student's
        t-distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `df` = *<value>*.

            Constraint: `df` >= 1.0.

        (`errno` 2)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 1.

        (`errno` 4)
            Unable to calculate the probability as it is too close to
            zero or one.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 3)
            One of the series has failed to converge with `maxit` =
            *<value>* and `tol` = *<value>*. Reconsider the requested
            tolerance and/or the maximum number of iterations.

        (`errno` 4)
            The probability is too close to 0 or 1. The returned value
            should be a reasonable estimate of the true value.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_students_t_noncentral'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01gbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, t.
        utils._BYREF_FLOAT64, # Input, df.
        utils._BYREF_FLOAT64, # Input, delta.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, maxit.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _t = utils._EngineFloat64ScalarType(
        t, {'fun_name': fun_name, 'entity': 't'},
    ).to_ctypes()
    _df = utils._EngineFloat64ScalarType(
        df, {'fun_name': fun_name, 'entity': 'df'},
    ).to_ctypes()
    _delta = utils._EngineFloat64ScalarType(
        delta, {'fun_name': fun_name, 'entity': 'delta'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _t,
        _df,
        _delta,
        _tol,
        _maxit,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: df >= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxit >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99997:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'One of the series has failed to converge with maxit = {:s}'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'and tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Reconsider the requested tolerance and/or the maximum number of iterations.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The probability is too close to 0 or 1.',
                    'The returned value should be a reasonable estimate of the true value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Unable to calculate the probability as it is too close to zero or one.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['p'],
            )(
                _p_py,
            )
    _err_class.manage()
    return return_data

def prob_chisq_noncentral(x, df, rlamda, tol, maxit):
    u"""
    Computes probabilities for the non-central chi^2 distribution.

    For full information please refer to the NAG Library document for
    g01gc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01gcf.html

    Parameters
    ----------
    x : float
        The deviate from the noncentral chi^2-distribution with nu
        degrees of freedom and noncentrality parameter lambda.

    df : float
        nu, the degrees of freedom of the noncentral chi^2-distribution.

    rlamda : float
        lambda, the noncentrality parameter of the noncentral
        chi^2-distribution.

    tol : float
        The required accuracy of the solution. If
        ``prob_chisq_noncentral`` is entered with `tol` greater than or
        equal to 1.0 or less than 10*machine precision (see
        ``machine.precision``), the value of 10*machine precision is
        used instead.

    maxit : int
        The maximum number of iterations to be performed.

    Returns
    -------
    p : float
        The probability associated with the lower tail of the noncentral
        chi^2-distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `df` = 0.0 and `rlamda` = 0.0.

            Constraint: `rlamda` > 0.0 if `df` = 0.0.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 1.

        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: `x` >= 0.0.

        (`errno` 1)
            On entry, `rlamda` = *<value>*.

            Constraint: `rlamda` >= 0.0.

        (`errno` 1)
            On entry, `df` = *<value>*.

            Constraint: `df` >= 0.0.

        (`errno` 2)
            The initial value of the Poisson weight used in the
            summation of Equation (1) (see [Description]) was too small
            to be calculated. The computed probability is likely to be
            zero.

        (`errno` 3)
            The solution has failed to converge in *<value>* iterations.
            Consider increasing `maxit` or `tol`.

        (`errno` 4)
            The value of a term required in Equation (2) (see
            [Description]) is too large to be evaluated accurately. The
            most likely cause of this error is both `x` and `rlamda` are
            too large.

        (`errno` 5)
            The calculations for the central chi-square probability has
            failed to converge. A larger value of `tol` should be used.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_chisq_noncentral'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01gcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, df.
        utils._BYREF_FLOAT64, # Input, rlamda.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, maxit.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _df = utils._EngineFloat64ScalarType(
        df, {'fun_name': fun_name, 'entity': 'df'},
    ).to_ctypes()
    _rlamda = utils._EngineFloat64ScalarType(
        rlamda, {'fun_name': fun_name, 'entity': 'rlamda'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _df,
        _rlamda,
        _tol,
        _maxit,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df = 0.0 and rlamda = 0.0.',
                    'Constraint: rlamda > 0.0 if df = 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxit >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: x >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rlamda = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rlamda >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: df >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initial value of the Poisson weight used in the summation of Equation (1)',
                    '(see [Description]) was too small to be calculated.',
                    'The computed probability is likely to be zero.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solution has failed to converge in {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Consider increasing maxit or tol.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The value of a term required in Equation (2)',
                    '(see [Description]) is too large to be evaluated',
                    'accurately. The most likely cause of this error is both x and rlamda are',
                    'too large.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The calculations for the central chi-square probability has failed',
                    'to converge. A larger value of tol should be used.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_f_noncentral(f, df1, df2, rlamda, tol, maxit):
    u"""
    Computes probabilities for the non-central F-distribution.

    For full information please refer to the NAG Library document for
    g01gd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01gdf.html

    Parameters
    ----------
    f : float
        f, the deviate from the noncentral F-distribution.

    df1 : float
        The degrees of freedom of the numerator variance, nu_1.

    df2 : float
        The degrees of freedom of the denominator variance, nu_2.

    rlamda : float
        lambda, the noncentrality parameter.

    tol : float
        The relative accuracy required by you in the results. If
        ``prob_f_noncentral`` is entered with `tol` greater than or
        equal to 1.0 or less than 10*machine precision (see
        ``machine.precision``), the value of 10*machine precision is
        used instead.

    maxit : int
        The maximum number of iterations to be used.

    Returns
    -------
    p : float
        The probability associated with the lower tail of the noncentral
        F or variance-ratio distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `df2` = *<value>*.

            Constraint: `df2` > 0.0.

        (`errno` 1)
            On entry, `rlamda` = *<value>*.

            Constraint: 0.0 <= `rlamda` <= -2.0*log(U), where U is the
            safe range parameter as defined by ``machine.real_safe``.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 1.

        (`errno` 1)
            On entry, `df1` = *<value>*.

            Constraint: 0.0 < `df1` <= 10^6.

        (`errno` 1)
            On entry, `f` = *<value>*.

            Constraint: `f` > 0.0.

        (`errno` 1)
            On entry, `df1` = *<value>*.

            Constraint: `df1` > 0.0.

        (`errno` 2)
            The solution has failed to converge in *<value>* iterations.
            Consider increasing `maxit` or `tol`.

        (`errno` 3)
            The required probability cannot be computed accurately. This
            may happen if the result would be very close to zero or one.
            Alternatively the values of `df1` and `f` may be too large.
            In the latter case you could try using a normal
            approximation, see Abramowitz and Stegun.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The required accuracy was not achieved when calculating the
            initial value of the central F or chi^2 probability. You
            should try a larger value of `tol`. If the chi^2
            approximation is being used then ``prob_f_noncentral``
            returns zero otherwise the value returned should be an
            approximation to the correct value.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_f_noncentral'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01gdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, f.
        utils._BYREF_FLOAT64, # Input, df1.
        utils._BYREF_FLOAT64, # Input, df2.
        utils._BYREF_FLOAT64, # Input, rlamda.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, maxit.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _f = utils._EngineFloat64ScalarType(
        f, {'fun_name': fun_name, 'entity': 'f'},
    ).to_ctypes()
    _df1 = utils._EngineFloat64ScalarType(
        df1, {'fun_name': fun_name, 'entity': 'df1'},
    ).to_ctypes()
    _df2 = utils._EngineFloat64ScalarType(
        df2, {'fun_name': fun_name, 'entity': 'df2'},
    ).to_ctypes()
    _rlamda = utils._EngineFloat64ScalarType(
        rlamda, {'fun_name': fun_name, 'entity': 'rlamda'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _f,
        _df1,
        _df2,
        _rlamda,
        _tol,
        _maxit,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 89999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df2 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: df2 > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rlamda = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 <= rlamda <= -2.0*log(U), where U is the safe range',
                    'parameter as defined by machine.real_safe.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxit >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df1 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 < df1 <= 10^6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rlamda = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 <= rlamda <= -2.0*log(U), where U is the safe range',
                    'parameter as defined by machine.real_safe.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, f = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: f > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df1 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: df1 > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solution has failed to converge in {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Consider increasing maxit or tol.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The required probability cannot be computed accurately.',
                    'This may happen if the result would be very close to zero or one.',
                    'Alternatively the values of df1 and f may be too large.',
                    'In the latter case you could try using a normal approximation, see',
                    'Abramowitz and Stegun.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99992:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The required accuracy was not achieved when calculating the initial value',
                    'of the central F or chi^2 probability.',
                    'You should try a larger value of tol.',
                    'If the chi^2 approximation is being used then prob_f_noncentral returns zero',
                    'otherwise the value returned should be an approximation to',
                    'the correct value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['p'],
            )(
                _p_py,
            )
    _err_class.manage()
    return return_data

def prob_beta_noncentral(x, a, b, rlamda, tol, maxit):
    u"""
    Computes probabilities for the non-central beta distribution.

    For full information please refer to the NAG Library document for
    g01ge

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01gef.html

    Parameters
    ----------
    x : float
        beta, the deviate from the beta distribution, for which the
        probability P(B <= beta:a,b;lambda) is to be found.

    a : float
        a, the first parameter of the required beta distribution.

    b : float
        b, the second parameter of the required beta distribution.

    rlamda : float
        lambda, the noncentrality parameter of the required beta
        distribution.

    tol : float
        The relative accuracy required by you in the results. If
        ``prob_beta_noncentral`` is entered with `tol` greater than or
        equal to 1.0 or less than 10*machine precision (see
        ``machine.precision``), the value of 10*machine precision is
        used instead.

        See [Accuracy] for the relationship between `tol` and `maxit`.

    maxit : int
        The maximum number of iterations that the algorithm should use.

        See [Accuracy] for suggestions as to suitable values for `maxit`
        for different values of the arguments.

    Returns
    -------
    p : float
        The probability associated with the lower tail of the noncentral
        beta distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `b` = *<value>*.

            Constraint: 0.0 < `b` <= 10^6.

        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 1.

        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: 0.0 < `a` <= 10^6.

        (`errno` 1)
            On entry, `x` = *<value>*.

            Constraint: 0.0 <= `x` <= 1.0.

        (`errno` 1)
            On entry, `rlamda` = *<value>*.

            Constraint: 0.0 <= `rlamda` <= -2.0log(U), where U is the
            safe range parameter as defined by ``machine.real_safe``.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 2)
            The solution has failed to converge in *<value>* iterations.
            Consider increasing `maxit` or `tol`. The returned value
            will be an approximation to the correct value.

        (`errno` 3)
            The probability is too close to 0.0 or 1.0 for the algorithm
            to be able to calculate the required probability.
            ``prob_beta_noncentral`` will return 0.0 or 1.0 as
            appropriate. This should be a reasonable approximation.

        (`errno` 4)
            The required accuracy was not achieved when calculating the
            initial value of the beta distribution. You should try a
            larger value of `tol`. The returned value will be an
            approximation to the correct value.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_beta_noncentral'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01geft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Input, rlamda.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, maxit.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _rlamda = utils._EngineFloat64ScalarType(
        rlamda, {'fun_name': fun_name, 'entity': 'rlamda'},
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _a,
        _b,
        _rlamda,
        _tol,
        _maxit,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 89993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 < b <= 10^6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 89998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 < b <= 10^6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxit >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 < a <= 10^6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, x = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 <= x <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rlamda = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 <= rlamda <= -2.0log(U), where U is the safe range',
                    'parameter as defined by machine.real_safe.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 0.0 < a <= 10^6.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solution has failed to converge in {:s} iterations.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Consider increasing maxit or tol. The returned value will be',
                    'an approximation to the correct value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The probability is too close to 0.0 or 1.0 for the algorithm to be able to',
                    'calculate the required probability.',
                    'prob_beta_noncentral will return 0.0 or 1.0 as appropriate.',
                    'This should be a reasonable approximation.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The required accuracy was not achieved when calculating the initial value',
                    'of the beta distribution. You should try a larger value of tol.',
                    'The returned value will be an approximation to the correct value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['p'],
            )(
                _p_py,
            )
    _err_class.manage()
    return return_data

def prob_bivariate_normal(x, y, rho):
    u"""
    Computes probability for the bivariate Normal distribution.

    For full information please refer to the NAG Library document for
    g01ha

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01haf.html

    Parameters
    ----------
    x : float
        x, the first argument for which the bivariate Normal
        distribution function is to be evaluated.

    y : float
        y, the second argument for which the bivariate Normal
        distribution function is to be evaluated.

    rho : float
        rho, the correlation coefficient.

    Returns
    -------
    p : float
        The lower tail probability for the bivariate Normal
        distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `rho` = *<value>*.

            Constraint: `rho` <= 1.0.

        (`errno` 1)
            On entry, `rho` = *<value>*.

            Constraint: `rho` >= -1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_bivariate_normal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01haft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, y.
        utils._BYREF_FLOAT64, # Input, rho.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _y = utils._EngineFloat64ScalarType(
        y, {'fun_name': fun_name, 'entity': 'y'},
    ).to_ctypes()
    _rho = utils._EngineFloat64ScalarType(
        rho, {'fun_name': fun_name, 'entity': 'rho'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _y,
        _rho,
        _p,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 89999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rho = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rho <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rho = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rho >= -1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_multi_normal(n, a, b, xmu, sig, tol):
    u"""
    Computes probabilities for the multivariate Normal distribution.

    For full information please refer to the NAG Library document for
    g01hb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01hbf.html

    Parameters
    ----------
    n : int
        n, the number of dimensions.

    a : None or float, shape (`n`)
        If upper tail or central probablilities are to be returned, `a`
        should supply the lower bounds, a_i, for i = 1,2,...,n.

    b : None or float, shape (`n`)
        If lower tail or central probablilities are to be returned, `b`
        should supply the upper bounds, b_i, for i = 1,2,...,n.

    xmu : float, shape (`n`)
        mu, the mean vector of the multivariate Normal distribution.

    sig : float, shape (`n`, `n`)
        Sigma, the variance-covariance matrix of the multivariate Normal
        distribution. Only the lower triangle is referenced.

    tol : float
        If n > 2 the relative accuracy required for the probability, and
        if the upper or the lower tail probability is requested then
        `tol` is also used to determine the cut-off points, see
        [Accuracy].

        If n = 1, `tol` is not referenced.

    Returns
    -------
    p : float
        The upper tail, lower tail or central probability associated
        with then multivariate Normal distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: 1 <= `n` <= 10.

        (`errno` 1)
            On entry, tail = *<value>*.

            Constraint: tail = 'L', 'U' or 'C'.

        (`errno` 1)
            On entry, `tol` = *<value>*.

            Constraint: `tol` > 0.0.

        (`errno` 2)
            On entry, the *<value>**<value>* value in `b` is less than
            or equal to the corresponding value in `a`.

        (`errno` 3)
            On entry, `sig` is not positive definite.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            Full accuracy not achieved, relative accuracy = *<value>*.

        (`errno` 5)
            Accuracy requested by `tol` is too strict: `tol` =
            *<value>*.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_multi_normal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01hbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, iemode.
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[n].
        utils._EngineFloat64ArrayType.c_type, # Input, xmu[n].
        utils._EngineFloat64ArrayType.c_type, # Input, sig[ldsig, n].
        utils._BYREF_INT, # Input, ldsig.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._BYREF_INT, # Input, lwk.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((sig, 'sig'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _iemode = utils._EngineIntScalarType.c_type(0)
    if b is None and a is not None:
        _tail = utils._EngineCharScalarType(
            'U', {'fun_name': fun_name, 'entity': '_tail'}, exp_chrlen=1,
        ).to_ctypes()
    elif a is None and b is not None:
        _tail = utils._EngineCharScalarType(
            'L', {'fun_name': fun_name, 'entity': '_tail'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _tail = utils._EngineCharScalarType(
            'C', {'fun_name': fun_name, 'entity': '_tail'}, exp_chrlen=1,
        ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    if a is None:
        _a = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    else:
        _a = utils._EngineFloat64ArrayType(
            a, {'fun_name': fun_name, 'entity': 'a'},
            exp_shape=(_n_py,),
            allow_none=True,
        ).to_ctypes()
    if b is None:
        _b = utils._EngineFloat64ArrayType.empty_ctypes(_n_py)
    else:
        _b = utils._EngineFloat64ArrayType(
            b, {'fun_name': fun_name, 'entity': 'b'},
            exp_shape=(_n_py,),
            allow_none=True,
        ).to_ctypes()
    _xmu = utils._EngineFloat64ArrayType(
        xmu, {'fun_name': fun_name, 'entity': 'xmu'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _ldsig_py = utils._EngineFloat64ArrayType.get_nd(sig, {'fun_name': fun_name, 'entity': 'sig'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldsig = utils._EngineIntScalarType(
        _ldsig_py, {'fun_name': fun_name, 'entity': '_ldsig'},
    ).to_ctypes()
    _sig = utils._EngineFloat64ArrayType(
        sig, {'fun_name': fun_name, 'entity': 'sig'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _lwk = utils._EngineIntScalarType.c_type(0)
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _iemode,
        _tail,
        _n,
        _a,
        _b,
        _xmu,
        _sig,
        _ldsig,
        _tol,
        _wk,
        _lwk,
        _p,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= n <= 10.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99994:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'L\', \'U\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tol > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 <= n <= 10.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the {:s}{:s} value in b is'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_STR.format(_eb_data[2]),
                    ),
                    'less than or equal to the corresponding value in a.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sig is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99995:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Full accuracy not achieved, relative accuracy = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99990:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Accuracy requested by tol is too strict: tol = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['p'],
            )(
                _p_py,
            )
    _err_class.manage()
    return return_data

def prob_bivariate_students_t(a, b, df, rho):
    u"""
    Computes probabilities for the bivariate Student's t-distribution.

    For full information please refer to the NAG Library document for
    g01hc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01hcf.html

    Parameters
    ----------
    a : None or float, shape (2)
        If upper tail or central probablilities are to be returned, `a`
        should supply the lower bounds, a_i, for i = 1,2,...,2.

    b : None or float, shape (2)
        If lower tail or central probablilities are to be returned, `b`
        should supply the upper bounds, b_i, for i = 1,2,...,2.

    df : int
        nu, the degrees of freedom of the bivariate Student's
        t-distribution.

    rho : float
        rho, the correlation of the bivariate Student's t-distribution.

    Returns
    -------
    p : float
        The probabilities for the bivariate Student's t-distribution.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, tail = *<value>*.

            Constraint: tail = 'L', 'U' or 'C'.

        (`errno` 3)
            On entry, `b`[i-1] <= `a`[i-1] for central probability, for
            some i = 1,2.

        (`errno` 4)
            On entry, `df` = *<value>*.

            Constraint: `df` >= 1.

        (`errno` 5)
            On entry, `rho` = *<value>*.

            Constraint: -1.0 <= `rho` <= 1.0.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_bivariate_students_t'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01hcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, tail(len=1).
        utils._EngineFloat64ArrayType.c_type, # Input, a[2].
        utils._EngineFloat64ArrayType.c_type, # Input, b[2].
        utils._BYREF_INT, # Input, df.
        utils._BYREF_FLOAT64, # Input, rho.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    if b is None and a is not None:
        _tail = utils._EngineCharScalarType(
            'U', {'fun_name': fun_name, 'entity': '_tail'}, exp_chrlen=1,
        ).to_ctypes()
    elif a is None and b is not None:
        _tail = utils._EngineCharScalarType(
            'L', {'fun_name': fun_name, 'entity': '_tail'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _tail = utils._EngineCharScalarType(
            'C', {'fun_name': fun_name, 'entity': '_tail'}, exp_chrlen=1,
        ).to_ctypes()
    if a is None:
        _a = utils._EngineFloat64ArrayType.empty_ctypes(2)
    else:
        _a = utils._EngineFloat64ArrayType(
            a, {'fun_name': fun_name, 'entity': 'a'},
            exp_shape=(2,),
            allow_none=True,
        ).to_ctypes()
    if b is None:
        _b = utils._EngineFloat64ArrayType.empty_ctypes(2)
    else:
        _b = utils._EngineFloat64ArrayType(
            b, {'fun_name': fun_name, 'entity': 'b'},
            exp_shape=(2,),
            allow_none=True,
        ).to_ctypes()
    _df = utils._EngineIntScalarType(
        df, {'fun_name': fun_name, 'entity': 'df'},
    ).to_ctypes()
    _rho = utils._EngineFloat64ScalarType(
        rho, {'fun_name': fun_name, 'entity': 'rho'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _tail,
        _a,
        _b,
        _df,
        _rho,
        _p,
        _errbuf,
        _ierr,
        _tail._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail = \'L\', \'U\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b[i-1] <= a[i-1] for central probability, for some i = 1,2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, df = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: df >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rho = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: -1.0 <= rho <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        return_data = _p_py
    _err_class.manage()
    return return_data

def prob_multi_students_t(n, tail, a, b, nu, delta, iscov, rc, epsabs, epsrel, numsub, nsampl, fmax):
    u"""
    Computes the probability for the multivariate Student's
    t-distribution.

    For full information please refer to the NAG Library document for
    g01hd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01hdf.html

    Parameters
    ----------
    n : int
        n, the number of dimensions.

    tail : str, length 1, shape (`n`)
        Defines the calculated probability, set `tail`[i-1] to:

        `tail`[i-1] = 'L'

          If the ith lower limit a_i is negative infinity.

        `tail`[i-1] = 'U'

          If the ith upper limit b_i is infinity.

        `tail`[i-1] = 'C'

          If both a_i and b_i are finite.

    a : float, shape (`n`)
        a_i, for i = 1,2,...,n, the lower integral limits of the
        calculation.

        If `tail`[i-1] = 'L', `a`[i-1] is not referenced and the ith
        lower limit of integration is - infinity.

    b : float, shape (`n`)
        b_i, for i = 1,2,...,n, the upper integral limits of the
        calculation.

        If `tail`[i-1] = 'U', `b`[i-1] is not referenced and the ith
        upper limit of integration is infinity.

    nu : float
        nu, the degrees of freedom.

    delta : float, shape (`n`)
        `delta`[i-1] the noncentrality parameter for the ith dimension,
        for i = 1,2,...,`n`; set `delta`[i-1] = 0 for the central
        probability.

    iscov : int
        Set `iscov` = 1 if the covariance matrix is supplied and `iscov`
        = 2 if the correlation matrix is supplied.

    rc : float, shape (`n`, `n`), modified in place
        On entry: the lower triangle of either the covariance matrix (if
        `iscov` = 1) or the correlation matrix (if `iscov` = 2). In
        either case the array elements corresponding to the upper
        triangle of the matrix need not be set.

        On exit: the strict upper triangle of `rc` contains the
        correlation matrix used in the calculations.

    epsabs : float
        epsilon_a, the absolute accuracy requested in the approximation.
        If `epsabs` is negative, the absolute value is used.

    epsrel : float
        epsilon_r, the relative accuracy requested in the approximation.
        If `epsrel` is negative, the absolute value is used.

    numsub : int
        If quadrature is used, the number of sub-intervals used by the
        quadrature algorithm; otherwise `numsub` is not referenced.

    nsampl : int
        If quadrature is used, `nsampl` is not referenced; otherwise
        `nsampl` is the number of samples used to estimate the error in
        the approximation.

    fmax : int
        If a number theoretic approach is used, the maximum number of
        evaluations for each integrand function.

    Returns
    -------
    p : float
        The probability associated with the multivariate Student's
        t-distribution.

    errest : float
        An estimate of the error in the calculated probability.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: 1 < `n` <= 1000.

        (`errno` 2)
            On entry, `tail`[k-1] = *<value>*.

            Constraint: `tail`[k-1] = 'L', 'U' or 'C'.

        (`errno` 4)
            On entry, k = *<value>*.

            Constraint: `b`[k-1] > `a`[k-1] for a central probability.

        (`errno` 5)
            On entry, `nu` = *<value>*.

            Constraint: degrees of freedom `nu` > 0.0.

        (`errno` 8)
            On entry, `iscov` = *<value>*.

            Constraint: `iscov` = 1 or 2.

        (`errno` 9)
            On entry, the information supplied in `rc` is invalid.

        (`errno` 12)
            On entry, `numsub` = *<value>*.

            Constraint: `numsub` >= 1.

        (`errno` 13)
            On entry, `nsampl` = *<value>*.

            Constraint: `nsampl` >= 1.

        (`errno` 14)
            On entry, `fmax` = *<value>*.

            Constraint: `fmax` >= 1.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_multi_students_t'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01hdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, n.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[n].
        utils._EngineFloat64ArrayType.c_type, # Input, a[n].
        utils._EngineFloat64ArrayType.c_type, # Input, b[n].
        utils._BYREF_FLOAT64, # Input, nu.
        utils._EngineFloat64ArrayType.c_type, # Input, delta[n].
        utils._BYREF_INT, # Input, iscov.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rc[ldrc, n].
        utils._BYREF_INT, # Input, ldrc.
        utils._BYREF_FLOAT64, # Input, epsabs.
        utils._BYREF_FLOAT64, # Input, epsrel.
        utils._BYREF_INT, # Input, numsub.
        utils._BYREF_INT, # Input, nsampl.
        utils._BYREF_INT, # Input, fmax.
        utils._BYREF_FLOAT64, # Output, errest.
        utils._BYREF_FLOAT64, # Output, p.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((rc, 'rc'),),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_n_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _nu = utils._EngineFloat64ScalarType(
        nu, {'fun_name': fun_name, 'entity': 'nu'},
    ).to_ctypes()
    _delta = utils._EngineFloat64ArrayType(
        delta, {'fun_name': fun_name, 'entity': 'delta'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _iscov = utils._EngineIntScalarType(
        iscov, {'fun_name': fun_name, 'entity': 'iscov'},
    ).to_ctypes()
    _ldrc_py = utils._EngineFloat64ArrayType.get_nd(rc, {'fun_name': fun_name, 'entity': 'rc'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldrc = utils._EngineIntScalarType(
        _ldrc_py, {'fun_name': fun_name, 'entity': '_ldrc'},
    ).to_ctypes()
    _rc_cls = utils._EngineFloat64ArrayType(
        rc, {'fun_name': fun_name, 'entity': 'rc'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    )
    _rc = _rc_cls.to_ctypes()
    _epsabs = utils._EngineFloat64ScalarType(
        epsabs, {'fun_name': fun_name, 'entity': 'epsabs'},
    ).to_ctypes()
    _epsrel = utils._EngineFloat64ScalarType(
        epsrel, {'fun_name': fun_name, 'entity': 'epsrel'},
    ).to_ctypes()
    _numsub = utils._EngineIntScalarType(
        numsub, {'fun_name': fun_name, 'entity': 'numsub'},
    ).to_ctypes()
    _nsampl = utils._EngineIntScalarType(
        nsampl, {'fun_name': fun_name, 'entity': 'nsampl'},
    ).to_ctypes()
    _fmax = utils._EngineIntScalarType(
        fmax, {'fun_name': fun_name, 'entity': 'fmax'},
    ).to_ctypes()
    _errest = utils._EngineFloat64ScalarType.empty_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _n,
        _tail,
        _a,
        _b,
        _nu,
        _delta,
        _iscov,
        _rc,
        _ldrc,
        _epsabs,
        _epsrel,
        _numsub,
        _nsampl,
        _fmax,
        _errest,
        _p,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: 1 < n <= 1000.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, tail[k-1] = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: tail[k-1] = \'L\', \'U\' or \'C\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: b[k-1] > a[k-1] for a central probability.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nu = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: degrees of freedom nu > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 8 and _ifmt == 8:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iscov = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: iscov = 1 or 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 9 and _ifmt == 9:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, the information supplied in rc is invalid.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, numsub = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: numsub >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 13 and _ifmt == 13:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nsampl = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nsampl >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 14 and _ifmt == 14:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, fmax = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: fmax >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rc_cls.refresh_py(_rc)
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        _errest_py = utils._EngineFloat64ScalarType.to_py(_errest)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['p', 'errest'],
        )(
            _p_py,
            _errest_py,
        )
    _err_class.manage()
    return return_data

def prob_chisq_noncentral_lincomb(a, mult, rlamda, n, c, tol, maxit):
    u"""
    Computes probability for a positive linear combination of chi^2
    variables.

    For full information please refer to the NAG Library document for
    g01jc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01jcf.html

    Parameters
    ----------
    a : float, shape (`n`)
        The weights, a_1,a_2,...,a_n.

    mult : int, shape (`n`)
        The degrees of freedom, m_1,m_2,...,m_n.

    rlamda : float, shape (`n`)
        The noncentrality parameters, lambda_1,lambda_2,...,lambda_n.

    n : int
        n, the number of chi^2 random variables in the combination,
        i.e., the number of terms in equation (1).

    c : float
        c, the point for which the lower tail probability is to be
        evaluated.

    tol : float
        The relative accuracy required by you in the results. If
        ``prob_chisq_noncentral_lincomb`` is entered with `tol` greater
        than or equal to 1.0 or less than 10*machine precision (see
        ``machine.precision``), the value of 10*machine precision is
        used instead.

    maxit : int
        The maximum number of terms that should be used during the
        summation.

    Returns
    -------
    p : float
        The lower tail probability associated with the linear
        combination of n chi^2 random variables with m_j degrees of
        freedom, and noncentrality parameters lambda_j, for j =
        1,2,...,n.

    pdf : float
        The value of the probability density function of the linear
        combination of chi^2 variables.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `maxit` = *<value>*.

            Constraint: `maxit` >= 1.

        (`errno` 1)
            On entry, `c` = *<value>*.

            Constraint: `c` >= 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, `rlamda`[*<value>*] = *<value>*.

            Constraint: `rlamda`[i] >= 0.0, for i = 0,...,`n`-1.

        (`errno` 2)
            On entry, `mult`[*<value>*] = *<value>*.

            Constraint: `mult`[i] >= 1, for i = 0,...,`n`-1.

        (`errno` 2)
            On entry, `a`[*<value>*] = *<value>*.

            Constraint: `a`[i] > 0.0, for i = 0,...,`n`-1.

        (`errno` 3)
            The central chi^2 calculation has failed to converge. This
            is an unlikely exit. A larger value of `tol` should be
            tried.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 4)
            The solution has failed to converge within `maxit`
            iterations. A larger value of `maxit` or `tol` should be
            used. The returned value should be a reasonable
            approximation to the correct value.

        (`errno` 5)
            The solution appears to be too close to 0 or 1 for accurate
            calculation. The value returned is 0 or 1 as appropriate.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_chisq_noncentral_lincomb'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01jcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._EngineFloat64ArrayType.c_type, # Input, a[n].
        utils._EngineIntArrayType.c_type, # Input, mult[n].
        utils._EngineFloat64ArrayType.c_type, # Input, rlamda[n].
        utils._BYREF_INT, # Input, n.
        utils._BYREF_FLOAT64, # Input, c.
        utils._BYREF_FLOAT64, # Output, p.
        utils._BYREF_FLOAT64, # Output, pdf.
        utils._BYREF_FLOAT64, # Input, tol.
        utils._BYREF_INT, # Input, maxit.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wrk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _mult = utils._EngineIntArrayType(
        mult, {'fun_name': fun_name, 'entity': 'mult'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _rlamda = utils._EngineFloat64ArrayType(
        rlamda, {'fun_name': fun_name, 'entity': 'rlamda'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _c = utils._EngineFloat64ScalarType(
        c, {'fun_name': fun_name, 'entity': 'c'},
    ).to_ctypes()
    _p = utils._EngineFloat64ScalarType.empty_ctypes()
    _pdf = utils._EngineFloat64ScalarType.empty_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _maxit = utils._EngineIntScalarType(
        maxit, {'fun_name': fun_name, 'entity': 'maxit'},
    ).to_ctypes()
    _wrk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _a,
        _mult,
        _rlamda,
        _n,
        _c,
        _p,
        _pdf,
        _tol,
        _maxit,
        _wrk,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, maxit = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: maxit >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, c = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: c >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rlamda[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: rlamda[i] >= 0.0, for i = 0,...,n-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mult[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: mult[i] >= 1, for i = 0,...,n-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: a[i] > 0.0, for i = 0,...,n-1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The central chi^2 calculation has failed to converge.',
                    'This is an unlikely exit.',
                    'A larger value of tol should be tried.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99994:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solution has failed to converge within maxit iterations.',
                    'A larger value of maxit or tol should be used. The',
                    'returned value should be a reasonable approximation to the correct value.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99993:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The solution appears to be too close to 0 or 1',
                    'for accurate calculation. The value returned is 0 or 1',
                    'as appropriate.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_py = utils._EngineFloat64ScalarType.to_py(_p)
        _pdf_py = utils._EngineFloat64ScalarType.to_py(_pdf)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['p', 'pdf'],
        )(
            _p_py,
            _pdf_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_chisq_lincomb(method, n, rlam, d, c):
    u"""
    Computes lower tail probability for a linear combination of
    (central) chi^2 variables.

    For full information please refer to the NAG Library document for
    g01jd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01jdf.html

    Parameters
    ----------
    method : str, length 1
        Indicates whether Pan's, Imhof's or an appropriately selected
        procedure is to be used.

        `method` = 'P'

          Pan's method is used.

        `method` = 'I'

          Imhof's method is used.

        `method` = 'D'

          Pan's method is used if lambda_i^*, for i = 1,2,...,n are at
          least 1% distinct and n <= 60; otherwise Imhof's method is
          used.

    n : int
        n, the number of independent standard Normal variates, (central
        chi^2 variates).

    rlam : float, shape (`n`)
        The weights, lambda_i, for i = 1,2,...,n, of the central chi^2
        variables.

    d : float
        d, the multiplier of the central chi^2 variables.

    c : float
        c, the value of the constant.

    Returns
    -------
    prob : float
        The lower tail probability for the linear combination of central
        chi^2 variables.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `method` = *<value>*.

            Constraint: `method` = 'P', 'I' or 'D'.

        (`errno` 1)
            On entry, `d` = *<value>*.

            Constraint: `d` >= 0.0.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 1.

        (`errno` 2)
            On entry, `rlam`[i-1] = `d` for all values of i, for i =
            1,2,...,n.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_chisq_lincomb'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01jdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, method(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, rlam[n].
        utils._BYREF_FLOAT64, # Input, d.
        utils._BYREF_FLOAT64, # Input, c.
        utils._BYREF_FLOAT64, # Output, prob.
        utils._EngineFloat64ArrayType.c_type, # Workspace, work.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of method.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _method = utils._EngineCharScalarType(
        method, {'fun_name': fun_name, 'entity': 'method'}, exp_chrlen=1,
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _rlam = utils._EngineFloat64ArrayType(
        rlam, {'fun_name': fun_name, 'entity': 'rlam'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _d = utils._EngineFloat64ScalarType(
        d, {'fun_name': fun_name, 'entity': 'd'},
    ).to_ctypes()
    _c = utils._EngineFloat64ScalarType(
        c, {'fun_name': fun_name, 'entity': 'c'},
    ).to_ctypes()
    _prob = utils._EngineFloat64ScalarType.empty_ctypes()
    _work = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _method,
        _n,
        _rlam,
        _d,
        _c,
        _prob,
        _work,
        _errbuf,
        _ierr,
        _method._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, method = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: method = \'P\', \'I\' or \'D\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, d = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: d >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rlam[i-1] = d for all values of i, for i = 1,2,...,n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _prob_py = utils._EngineFloat64ScalarType.to_py(_prob)
        return_data = _prob_py
    _err_class.manage()
    return return_data

def pdf_normal(x, xmean, xstd):
    u"""
    Calculates the value for the probability density function of the
    Normal distribution at a chosen point.

    For full information please refer to the NAG Library document for
    g01ka

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01kaf.html

    Parameters
    ----------
    x : float
        x, the value at which the PDF is to be evaluated.

    xmean : float
        mu, the mean of the Normal distribution.

    xstd : float
        sigma, the standard deviation of the Normal distribution.

    Returns
    -------
    pdf : float
        The value of the PDF.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `xstd` = *<value>*.

            Constraint: `xstd`*sqrt(2.0*pi) > U, where U is the safe
            range parameter as defined by ``machine.real_safe``.

        (`errno` 2)
            Computation abandoned owing to underflow of
            1/(sigma*sqrt(2*pi)).

        (`errno` 3)
            Computation abandoned owing to an internal calculation
            overflowing.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.pdf_normal'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01kaft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, xmean.
        utils._BYREF_FLOAT64, # Input, xstd.
        utils._BYREF_FLOAT64, # Output, pdf.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _xmean = utils._EngineFloat64ScalarType(
        xmean, {'fun_name': fun_name, 'entity': 'xmean'},
    ).to_ctypes()
    _xstd = utils._EngineFloat64ScalarType(
        xstd, {'fun_name': fun_name, 'entity': 'xstd'},
    ).to_ctypes()
    _pdf = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _xmean,
        _xstd,
        _pdf,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, xstd = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: xstd*sqrt(2.0*pi) > U, where U is the safe range',
                    'parameter as defined by machine.real_safe.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1020:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation abandoned owing to underflow of 1/(sigma*sqrt(2*pi)).',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1030:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation abandoned owing to an internal calculation overflowing.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _pdf_py = utils._EngineFloat64ScalarType.to_py(_pdf)
        return_data = _pdf_py
    _err_class.manage()
    return return_data

def pdf_gamma(x, a, b):
    u"""
    Calculates the value for the probability density function of the
    gamma distribution at a chosen point.

    For full information please refer to the NAG Library document for
    g01kf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01kff.html

    Parameters
    ----------
    x : float
        x, the value at which the PDF is to be evaluated.

    a : float
        alpha, the shape parameter of the gamma distribution.

    b : float
        beta, the scale parameter of the gamma distribution.

    Returns
    -------
    pdf : float
        The value of the PDF.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `a` = *<value>*.

            Constraint: `a` > 0.0.

        (`errno` 2)
            On entry, `b` = *<value>*.

            Constraint: `b` > 0.0.

        (`errno` 3)
            Computation abandoned owing to overflow due to extreme
            parameter values.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.pdf_gamma'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01kfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Input, a.
        utils._BYREF_FLOAT64, # Input, b.
        utils._BYREF_FLOAT64, # Output, pdf.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _a = utils._EngineFloat64ScalarType(
        a, {'fun_name': fun_name, 'entity': 'a'},
    ).to_ctypes()
    _b = utils._EngineFloat64ScalarType(
        b, {'fun_name': fun_name, 'entity': 'b'},
    ).to_ctypes()
    _pdf = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _a,
        _b,
        _pdf,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, a = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: a > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1020:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: b > 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1030:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Computation abandoned owing to overflow due to extreme parameter values.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _pdf_py = utils._EngineFloat64ScalarType.to_py(_pdf)
        return_data = _pdf_py
    _err_class.manage()
    return return_data

def pdf_gamma_vector(ilog, lx, x, la, a, lb, b, pdf, ivalid):
    u"""
    Computes a vector of values for the probability density function of
    the gamma distribution.

    For full information please refer to the NAG Library document for
    g01kk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01kkf.html

    Parameters
    ----------
    ilog : int
        The value of `ilog` determines whether the logarithmic value is
        returned in `pdf`.

        `ilog` = 0

          f(x_i, alpha_i, beta_i), the probability density function is
          returned.

        `ilog` = 1

          log(f(x_i, alpha_i, beta_i)), the logarithm of the probability
          density function is returned.

    lx : int
        The dimension of the array `x`.

    x : float, shape (`lx`)
        x_i, the values at which the PDF is to be evaluated.

    la : int
        The dimension of the array `a`.

    a : float, shape (`la`)
        alpha_i, the shape parameter.

    lb : int
        The dimension of the array `b`.

    b : float, shape (`lb`)
        beta_i, the scale parameter.

    pdf : float, shape (max(`lx`, `la`, `lb`)), modified in place
        On exit: f(x_i, alpha_i, beta_i) or log(f(x_i, alpha_i,
        beta_i)).

    ivalid : int, shape (max(`lx`, `la`, `lb`)), modified in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          alpha_i <= 0.0.

        `ivalid`[i-1] = 2

          beta_i <= 0.0.

        `ivalid`[i-1] = 3

          (x_i)/(beta_i) overflows, the value returned should be a
          reasonable approximation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `ilog` = *<value>*.

            Constraint: `ilog` = 0 or 1.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lx` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `la` > 0.

        (`errno` 5)
            On entry, array size = *<value>*.

            Constraint: `lb` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x`, `a` or `b` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.pdf_gamma_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01kkft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ilog.
        utils._BYREF_INT, # Input, lx.
        utils._EngineFloat64ArrayType.c_type, # Input, x[lx].
        utils._BYREF_INT, # Input, la.
        utils._EngineFloat64ArrayType.c_type, # Input, a[la].
        utils._BYREF_INT, # Input, lb.
        utils._EngineFloat64ArrayType.c_type, # Input, b[lb].
        utils._EngineFloat64ArrayType.c_type, # Output, pdf[max(lx, la, lb)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(lx, la, lb)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ilog = utils._EngineIntScalarType(
        ilog, {'fun_name': fun_name, 'entity': 'ilog'},
    ).to_ctypes()
    _lx = utils._EngineIntScalarType(
        lx, {'fun_name': fun_name, 'entity': 'lx'},
    ).to_ctypes()
    _lx_py = utils._EngineIntScalarType.to_py(_lx)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_lx_py,),
    ).to_ctypes()
    _la = utils._EngineIntScalarType(
        la, {'fun_name': fun_name, 'entity': 'la'},
    ).to_ctypes()
    _la_py = utils._EngineIntScalarType.to_py(_la)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_la_py,),
    ).to_ctypes()
    _lb = utils._EngineIntScalarType(
        lb, {'fun_name': fun_name, 'entity': 'lb'},
    ).to_ctypes()
    _lb_py = utils._EngineIntScalarType.to_py(_lb)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_lb_py,),
    ).to_ctypes()
    _pdf_cls = utils._EngineFloat64ArrayType(
        pdf, {'fun_name': fun_name, 'entity': 'pdf'},
        exp_shape=(max(_lx_py, _la_py, _lb_py),),
        check_contents=False,
    )
    _pdf = _pdf_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_lx_py, _la_py, _lb_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ilog,
        _lx,
        _x,
        _la,
        _a,
        _lb,
        _b,
        _pdf,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x, a or b was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1050:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilog = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ilog = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1020:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1030:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: la > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1040:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lb > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _pdf_cls.refresh_py(_pdf)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def pdf_normal_vector(ilog, lx, x, lxmu, xmu, lxstd, xstd, pdf, ivalid):
    u"""
    Computes a vector of values for the probability density function of
    the Normal distribution.

    For full information please refer to the NAG Library document for
    g01kq

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01kqf.html

    Parameters
    ----------
    ilog : int
        The value of `ilog` determines whether the logarithmic value is
        returned in PDF.

        `ilog` = 0

          f(x_i, mu_i, sigma_i), the probability density function is
          returned.

        `ilog` = 1

          log(f(x_i, mu_i, sigma_i)), the logarithm of the probability
          density function is returned.

    lx : int
        The dimension of the array `x`.

    x : float, shape (`lx`)
        x_i, the values at which the PDF is to be evaluated.

    lxmu : int
        The dimension of the array `xmu`.

    xmu : float, shape (`lxmu`)
        mu_i, the means.

    lxstd : int
        The dimension of the array `xstd`.

    xstd : float, shape (`lxstd`)
        sigma_i, the standard deviations.

    pdf : float, shape (max(`lx`, `lxstd`, `lxmu`)), modified in place
        On exit: f(x_i, mu_i, sigma_i) or log(f(x_i, mu_i, sigma_i)).

    ivalid : int, shape (max(`lx`, `lxstd`, `lxmu`)), modified in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          sigma_i < 0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `ilog` = *<value>*.

            Constraint: `ilog` = 0 or 1.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lx` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `lxmu` > 0.

        (`errno` 5)
            On entry, array size = *<value>*.

            Constraint: `lxstd` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `xstd` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.pdf_normal_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01kqft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ilog.
        utils._BYREF_INT, # Input, lx.
        utils._EngineFloat64ArrayType.c_type, # Input, x[lx].
        utils._BYREF_INT, # Input, lxmu.
        utils._EngineFloat64ArrayType.c_type, # Input, xmu[lxmu].
        utils._BYREF_INT, # Input, lxstd.
        utils._EngineFloat64ArrayType.c_type, # Input, xstd[lxstd].
        utils._EngineFloat64ArrayType.c_type, # Output, pdf[max(lx, lxstd, lxmu)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(lx, lxstd, lxmu)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ilog = utils._EngineIntScalarType(
        ilog, {'fun_name': fun_name, 'entity': 'ilog'},
    ).to_ctypes()
    _lx = utils._EngineIntScalarType(
        lx, {'fun_name': fun_name, 'entity': 'lx'},
    ).to_ctypes()
    _lx_py = utils._EngineIntScalarType.to_py(_lx)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_lx_py,),
    ).to_ctypes()
    _lxmu = utils._EngineIntScalarType(
        lxmu, {'fun_name': fun_name, 'entity': 'lxmu'},
    ).to_ctypes()
    _lxmu_py = utils._EngineIntScalarType.to_py(_lxmu)
    _xmu = utils._EngineFloat64ArrayType(
        xmu, {'fun_name': fun_name, 'entity': 'xmu'},
        exp_shape=(_lxmu_py,),
    ).to_ctypes()
    _lxstd = utils._EngineIntScalarType(
        lxstd, {'fun_name': fun_name, 'entity': 'lxstd'},
    ).to_ctypes()
    _lxstd_py = utils._EngineIntScalarType.to_py(_lxstd)
    _xstd = utils._EngineFloat64ArrayType(
        xstd, {'fun_name': fun_name, 'entity': 'xstd'},
        exp_shape=(_lxstd_py,),
    ).to_ctypes()
    _pdf_cls = utils._EngineFloat64ArrayType(
        pdf, {'fun_name': fun_name, 'entity': 'pdf'},
        exp_shape=(max(_lx_py, _lxstd_py, _lxmu_py),),
        check_contents=False,
    )
    _pdf = _pdf_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_lx_py, _lxstd_py, _lxmu_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ilog,
        _lx,
        _x,
        _lxmu,
        _xmu,
        _lxstd,
        _xstd,
        _pdf,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1010:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of xstd was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 1050:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilog = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ilog = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 1020:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 1030:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lxmu > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 1040:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lxstd > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _pdf_cls.refresh_py(_pdf)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def pdf_multi_normal_vector(ilog, k, n, x, xmu, iuld, sig, pdf):
    u"""
    Computes a vector of values for the probability density function of
    the multivariate Normal distribution.

    For full information please refer to the NAG Library document for
    g01lb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01lbf.html

    Parameters
    ----------
    ilog : int
        The value of `ilog` determines whether the logarithmic value is
        returned in PDF.

        `ilog` = 0

          f(X:mu, Sigma), the probability density function is returned.

        `ilog` = 1

          log(f(X:mu, Sigma)), the logarithm of the probability density
          function is returned.

    k : int
        k, the number of points the PDF is to be evaluated at.

    n : int
        n, the number of dimensions.

    x : float, shape (`n`, `k`)
        X, the matrix of k points at which to evaluate the probability
        density function, with the ith dimension for the jth point held
        in `x`[i-1,j-1].

    xmu : float, shape (`n`)
        mu, the mean vector of the multivariate Normal distribution.

    iuld : int
        Indicates the form of Sigma and how it is stored in `sig`.

        `iuld` = 1

          `sig` holds the lower triangular portion of Sigma.

        `iuld` = 2

          `sig` holds the upper triangular portion of Sigma.

        `iuld` = 3

          Sigma is a diagonal matrix and `sig` only holds the diagonal
          elements.

        `iuld` = 4

          `sig` holds the lower Cholesky decomposition, L such that
          L*L^T = Sigma.

        `iuld` = 5

          `sig` holds the upper Cholesky decomposition, U such that
          U^T*U = Sigma.

    sig : float, shape (:, `n`)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `iuld`=3: 1; otherwise: `n`.

        Information defining the variance-covariance matrix, Sigma.

        `iuld` = 1 or 2

          `sig` must hold the lower or upper portion of Sigma, with
          Sigma_{ij} held in `sig`[i-1,j-1]. The supplied
          variance-covariance matrix must be positive semidefinite.

        `iuld` = 3

          Sigma is a diagonal matrix and the ith diagonal element,
          Sigma_{ii}, must be held in `sig`[0,i-1]

        `iuld` = 4 or 5

          `sig` must hold L or U, the lower or upper Cholesky
          decomposition of Sigma, with L_{ij} or U_{ij} held in
          `sig`[i-1,j-1], depending on the value of `iuld`. No check is
          made that L*L^T or U^T*U is a valid variance-covariance
          matrix. The diagonal elements of the supplied L or U must be
          greater than zero

    pdf : float, shape (`k`), modified in place
        On exit: f(X:mu, Sigma) or log(f(X:mu, Sigma)) depending on the
        value of `ilog`.

    Returns
    -------
    rank : int
        r, rank of Sigma.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `ilog` = *<value>*.

            Constraint: `ilog` = 0 or 1.

        (`errno` 21)
            On entry, `k` = *<value>*.

            Constraint: `k` >= 0.

        (`errno` 31)
            On entry, `n` = *<value>*.

            Constraint: `n` >= 2.

        (`errno` 71)
            On entry, `iuld` = *<value>*.

            Constraint: `iuld` = 1, 2, 3, 4 or 5.

        (`errno` 81)
            On entry, Sigma is not positive semidefinite.

        (`errno` 82)
            On entry, at least one diagonal element of Sigma is less
            than or equal to 0.

        (`errno` 83)
            On entry, Sigma is not positive definite and eigenvalue
            decomposition failed.

        (`errno` 92)
            On entry, ldsig = *<value>*.

            Constraint: if `iuld` != 3, ldsig >= `n`.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.pdf_multi_normal_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01lbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ilog.
        utils._BYREF_INT, # Input, k.
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, x[ldx, k].
        utils._BYREF_INT, # Input, ldx.
        utils._EngineFloat64ArrayType.c_type, # Input, xmu[n].
        utils._BYREF_INT, # Input, iuld.
        utils._EngineFloat64ArrayType.c_type, # Input, sig[ldsig, n].
        utils._BYREF_INT, # Input, ldsig.
        utils._EngineFloat64ArrayType.c_type, # Output, pdf[k].
        utils._BYREF_INT, # Output, rank.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((x, 'x'), (sig, 'sig')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _ilog = utils._EngineIntScalarType(
        ilog, {'fun_name': fun_name, 'entity': 'ilog'},
    ).to_ctypes()
    _k = utils._EngineIntScalarType(
        k, {'fun_name': fun_name, 'entity': 'k'},
    ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _ldx_py = utils._EngineFloat64ArrayType.get_nd(x, {'fun_name': fun_name, 'entity': 'x'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldx = utils._EngineIntScalarType(
        _ldx_py, {'fun_name': fun_name, 'entity': '_ldx'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _k_py = utils._EngineIntScalarType.to_py(_k)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_n_py, _k_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _xmu = utils._EngineFloat64ArrayType(
        xmu, {'fun_name': fun_name, 'entity': 'xmu'},
        exp_shape=(_n_py,),
    ).to_ctypes()
    _iuld = utils._EngineIntScalarType(
        iuld, {'fun_name': fun_name, 'entity': 'iuld'},
    ).to_ctypes()
    _ldsig_py = utils._EngineFloat64ArrayType.get_nd(sig, {'fun_name': fun_name, 'entity': 'sig'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldsig = utils._EngineIntScalarType(
        _ldsig_py, {'fun_name': fun_name, 'entity': '_ldsig'},
    ).to_ctypes()
    _iuld_py = utils._EngineIntScalarType.to_py(_iuld)
    if _iuld_py == 3:
        _sig_dim1 = 1
    else:
        _sig_dim1 = _n_py
    exp_shape_sig = (_sig_dim1, _n_py)
    _sig = utils._EngineFloat64ArrayType(
        sig, {'fun_name': fun_name, 'entity': 'sig'},
        exp_shape=exp_shape_sig,
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _pdf_cls = utils._EngineFloat64ArrayType(
        pdf, {'fun_name': fun_name, 'entity': 'pdf'},
        exp_shape=(_k_py,),
        check_contents=False,
    )
    _pdf = _pdf_cls.to_ctypes(can_reference_contents=False)
    _rank = utils._EngineIntScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ilog,
        _k,
        _n,
        _x,
        _ldx,
        _xmu,
        _iuld,
        _sig,
        _ldsig,
        _pdf,
        _rank,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ilog = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ilog = 0 or 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, k = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: k >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 31 and _ifmt == 31:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n >= 2.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 71 and _ifmt == 71:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iuld = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: iuld = 1, 2, 3, 4 or 5.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 81 and _ifmt == 81:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, Sigma is not positive semidefinite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 82 and _ifmt == 82:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one diagonal element of Sigma is less than or equal to 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 83 and _ifmt == 83:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, Sigma is not positive definite and eigenvalue decomposition',
                    'failed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 92 and _ifmt == 92:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ldsig = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if iuld != 3, ldsig >= n.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _pdf_cls.refresh_py(_pdf)
        _rank_py = utils._EngineIntScalarType.to_py(_rank)
        return_data = _rank_py
    _err_class.manage()
    return return_data

def mills_ratio(x):
    u"""
    Computes reciprocal of Mills' Ratio.

    For full information please refer to the NAG Library document for
    g01mb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01mbf.html

    Parameters
    ----------
    x : float
        x, the argument of the reciprocal of Mills' Ratio.

    Returns
    -------
    rmr : float
        The reciprocal of Mills' Ratio.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.mills_ratio'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01mbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, rmr.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _rmr = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _rmr,
    )
    utils._handle_cb_exception(excs)
    _rmr_py = utils._EngineFloat64ScalarType.to_py(_rmr)
    return_data = _rmr_py
    return return_data

def pdf_landau(x):
    u"""
    Landau density function phi(lambda).

    For full information please refer to the NAG Library document for
    g01mt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01mtf.html

    Parameters
    ----------
    x : float
        The argument lambda of the function.

    Returns
    -------
    pdf : float
        The value of the Landau density function phi(lambda).
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.pdf_landau'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01mtft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, pdf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _pdf = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _pdf,
    )
    utils._handle_cb_exception(excs)
    _pdf_py = utils._EngineFloat64ScalarType.to_py(_pdf)
    return_data = _pdf_py
    return return_data

def pdf_vavilov(x, comm):
    u"""
    Vavilov density function phi_V(lambda;kappa,beta^2).

    For full information please refer to the NAG Library document for
    g01mu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01muf.html

    Parameters
    ----------
    x : float
        The argument lambda of the function.

    comm : dict, communication object
        Communication structure.

        This argument must have been initialized by a prior call to
        ``init_vavilov``.

    Returns
    -------
    pdf : float
        The value of the Vavilov density function
        phi_V(lambda;kappa,beta^2).

    Raises
    ------
    NagValueError
        (`errno` 1)
            Either the initialization function has not been called prior
            to the first call of this function or a communication array
            has become corrupted.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.pdf_vavilov'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01muft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._EngineFloat64ArrayType.c_type, # Input, rcomm[322].
        utils._BYREF_FLOAT64, # Output, pdf.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    utils._check_comm(
        comm, {'fun_name': fun_name, 'entity': 'comm'},
        req_keys=('rcomm',),
    )
    _rcomm = comm['rcomm']
    _pdf = utils._EngineFloat64ScalarType.empty_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _rcomm,
        _pdf,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 1:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Either the initialization function has not been called prior',
                    'to the first call of this function or',
                    'a communication array has become corrupted.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _pdf_py = utils._EngineFloat64ScalarType.to_py(_pdf)
        return_data = _pdf_py
    _err_class.manage()
    return return_data

def moments_quad_form(mom, n, a, emu, sigma, l, rkum, rmom):
    u"""
    Cumulants and moments of quadratic forms in Normal variables.

    For full information please refer to the NAG Library document for
    g01na

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01naf.html

    Parameters
    ----------
    mom : str, length 1
        Indicates if moments are computed in addition to cumulants.

        `mom` = 'C'

          Only cumulants are computed.

        `mom` = 'M'

          Moments are computed in addition to cumulants.

    n : int
        n, the dimension of the quadratic form.

    a : float, shape (`n`, `n`)
        The n by n symmetric matrix A. Only the lower triangle is
        referenced.

    emu : None or float, shape (:)
        Note: the required length for this argument is determined as
        follows: if mean='M': `n`; otherwise: 1.

        If mean = 'M', `emu` must contain the n elements of the vector
        mu.

        If mean = 'Z', `emu` is not referenced.

    sigma : float, shape (`n`, `n`)
        The n by n variance-covariance matrix Sigma. Only the lower
        triangle is referenced.

    l : int
        The required number of cumulants, and moments if specified.

    rkum : float, shape (`l`), modified in place
        On exit: the `l` cumulants of the quadratic form.

    rmom : float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `mom`='M': `l`; otherwise: 1.

        On exit: if `mom` = 'M', the `l` moments of the quadratic form.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `l` = *<value>*.

            Constraint: `l` <= 12.

        (`errno` 1)
            On entry, mean = *<value>*.

            Constraint: mean = 'Z' or 'M'.

        (`errno` 1)
            On entry, `mom` = *<value>*.

            Constraint: `mom` = 'C' or 'M'.

        (`errno` 1)
            On entry, `l` = *<value>*.

            Constraint: `l` >= 1.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 1.

        (`errno` 2)
            On entry, `sigma` is not positive definite.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.moments_quad_form'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01naft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, mom(len=1).
        utils._BYREF_STRING, # Input, mean(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, emu[*].
        utils._EngineFloat64ArrayType.c_type, # Input, sigma[ldsig, n].
        utils._BYREF_INT, # Input, ldsig.
        utils._BYREF_INT, # Input, l.
        utils._EngineFloat64ArrayType.c_type, # Output, rkum[l].
        utils._EngineFloat64ArrayType.c_type, # Output, rmom[*].
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mom.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (sigma, 'sigma')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    _mom = utils._EngineCharScalarType(
        mom, {'fun_name': fun_name, 'entity': 'mom'}, exp_chrlen=1,
    ).to_ctypes()
    if emu is None:
        _mean = utils._EngineCharScalarType(
            'Z', {'fun_name': fun_name, 'entity': '_mean'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _mean = utils._EngineCharScalarType(
            'M', {'fun_name': fun_name, 'entity': '_mean'}, exp_chrlen=1,
        ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _mean_py = utils._EngineCharScalarType.to_py(_mean)
    if _mean_py in ('M', 'm'):
        _emu_dim1 = _n_py
    else:
        _emu_dim1 = 1
    exp_shape_emu = (_emu_dim1,)
    _emu = utils._EngineFloat64ArrayType(
        emu, {'fun_name': fun_name, 'entity': 'emu'},
        exp_shape=exp_shape_emu,
        allow_none=True,
    ).to_ctypes()
    _ldsig_py = utils._EngineFloat64ArrayType.get_nd(sigma, {'fun_name': fun_name, 'entity': 'sigma'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldsig = utils._EngineIntScalarType(
        _ldsig_py, {'fun_name': fun_name, 'entity': '_ldsig'},
    ).to_ctypes()
    _sigma = utils._EngineFloat64ArrayType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _l = utils._EngineIntScalarType(
        l, {'fun_name': fun_name, 'entity': 'l'},
    ).to_ctypes()
    _l_py = utils._EngineIntScalarType.to_py(_l)
    _rkum_cls = utils._EngineFloat64ArrayType(
        rkum, {'fun_name': fun_name, 'entity': 'rkum'},
        exp_shape=(_l_py,),
        check_contents=False,
    )
    _rkum = _rkum_cls.to_ctypes(can_reference_contents=False)
    _mom_py = utils._EngineCharScalarType.to_py(_mom)
    if _mom_py in ('M', 'm'):
        _rmom_dim1 = _l_py
    else:
        _rmom_dim1 = 1
    exp_shape_rmom = (_rmom_dim1,)
    _rmom_cls = utils._EngineFloat64ArrayType(
        rmom, {'fun_name': fun_name, 'entity': 'rmom'},
        exp_shape=exp_shape_rmom,
        check_contents=False,
    )
    _rmom = _rmom_cls.to_ctypes(can_reference_contents=False)
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _mom,
        _mean,
        _n,
        _a,
        _lda,
        _emu,
        _sigma,
        _ldsig,
        _l,
        _rkum,
        _rmom,
        _wk,
        _errbuf,
        _ierr,
        _mom._length_,
        _mean._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: l <= 12.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mean = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mean = \'Z\' or \'M\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, mom = \'{:s}\'.'.format(
                        utils._STR_FMT_STR.format(_eb_data[1]),
                    ),
                    'Constraint: mom = \'C\' or \'M\'.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, l = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: l >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _rkum_cls.refresh_py(_rkum)
        _rmom_cls.refresh_py(_rmom)
        return_data = None
    _err_class.manage()
    return return_data

def moments_ratio_quad_forms(n, a, b, c, ela, emu, sigma, l1, l2, rmom, eps):
    u"""
    Moments of ratios of quadratic forms in Normal variables, and
    related statistics.

    For full information please refer to the NAG Library document for
    g01nb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01nbf.html

    Parameters
    ----------
    n : int
        n, the dimension of the quadratic form.

    a : float, shape (`n`, `n`)
        The n by n symmetric matrix A. Only the lower triangle is
        referenced.

    b : float, shape (`n`, `n`)
        The n by n positive semidefinite symmetric matrix B. Only the
        lower triangle is referenced.

    c : None or float, shape (:, `n`)
        Note: the required extent for this argument in dimension 1 is
        determined as follows: if `c` is not None: `n`; otherwise: 0.

        If `c` is not None, `c` must contain the n by n symmetric matrix
        C; only the lower triangle is referenced.

    ela : None or float, shape (`n`)
        If `ela` is not None, `ela` must contain the vector a of length
        n, otherwise `ela` is not referenced.

    emu : None or float, shape (`n`)
        If `emu` is not None, `emu` must contain the n elements of the
        vector mu.

    sigma : float, shape (`n`, `n`)
        The n by n variance-covariance matrix Sigma. Only the lower
        triangle is referenced.

    l1 : int
        The first moment to be computed, l_1.

    l2 : int
        The last moment to be computed, l_2.

    rmom : float, shape (`l2`-`l1`+1), modified in place
        On exit: the l_1 to l_MAX moments.

    eps : float
        The relative accuracy required for the moments, this value is
        also used in the checks for the existence of the moments.

        If `eps` = 0.0, a value of sqrt(epsilon) where epsilon is the
        machine precision used.

    Returns
    -------
    lmax : int
        The highest moment computed, l_MAX. This will be l_2 if no
        exception or warning is raised on exit.

    abserr : float
        The estimated maximum absolute error in any computed moment.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `l2` = *<value>*.

            Constraint: `l2` <= 12.

        (`errno` 1)
            On entry, `l1` = *<value>* and `l2` = *<value>*.

            Constraint: `l2` >= `l1`.

        (`errno` 1)
            On entry, `l1` = *<value>*.

            Constraint: `l1` >= 1.

        (`errno` 1)
            On entry, `eps` = *<value>*.

            Constraint: if `eps` != 0.0, `eps` >= machine precision.

        (`errno` 1)
            On entry, `n` = *<value>*.

            Constraint: `n` > 1.

        (`errno` 2)
            On entry, `sigma` is not positive definite.

        (`errno` 2)
            On entry, `b` is not positive semidefinite or is null.

        (`errno` 3)
            Only *<value>* moments exist, less than `l1` = *<value>*,
            therefore none of the required moments can be computed.

        (`errno` 4)
            The matrix L^TBL is not positive semidefinite or is null.

        (`errno` 5)
            The computation to compute the eigenvalues required in the
            calculation of moments has failed to converge: this is an
            unlikely error exit.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 6)
            Only some of the required moments have been computed, the
            highest is given by `lmax`.

        (`errno` 7)
            The required accuracy has not been achieved in the
            integration. An estimate of the accuracy is returned in
            `abserr`.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.moments_ratio_quad_forms'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01nbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_STRING, # Input, case(len=1).
        utils._BYREF_STRING, # Input, mean(len=1).
        utils._BYREF_INT, # Input, n.
        utils._EngineFloat64ArrayType.c_type, # Input, a[lda, n].
        utils._BYREF_INT, # Input, lda.
        utils._EngineFloat64ArrayType.c_type, # Input, b[ldb, n].
        utils._BYREF_INT, # Input, ldb.
        utils._EngineFloat64ArrayType.c_type, # Input, c[ldc, n].
        utils._BYREF_INT, # Input, ldc.
        utils._EngineFloat64ArrayType.c_type, # Input, ela[n].
        utils._EngineFloat64ArrayType.c_type, # Input, emu[n].
        utils._EngineFloat64ArrayType.c_type, # Input, sigma[ldsig, n].
        utils._BYREF_INT, # Input, ldsig.
        utils._BYREF_INT, # Input, l1.
        utils._BYREF_INT, # Input, l2.
        utils._BYREF_INT, # Output, lmax.
        utils._EngineFloat64ArrayType.c_type, # Output, rmom[l2-l1+1].
        utils._BYREF_FLOAT64, # Output, abserr.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._EngineFloat64ArrayType.c_type, # Workspace, wk.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of case.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of mean.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _m_sorder = utils._get_mutual_sorder(
        {'fun_name': fun_name},
        ((a, 'a'), (b, 'b'), (c, 'c'), (sigma, 'sigma')),
    )
    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec, sorder=_m_sorder)
    if ela is None and c is None:
        _case = utils._EngineCharScalarType(
            'R', {'fun_name': fun_name, 'entity': '_case'}, exp_chrlen=1,
        ).to_ctypes()
    elif ela is None:
        _case = utils._EngineCharScalarType(
            'Q', {'fun_name': fun_name, 'entity': '_case'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _case = utils._EngineCharScalarType(
            'L', {'fun_name': fun_name, 'entity': '_case'}, exp_chrlen=1,
        ).to_ctypes()
    if emu is None:
        _mean = utils._EngineCharScalarType(
            'Z', {'fun_name': fun_name, 'entity': '_mean'}, exp_chrlen=1,
        ).to_ctypes()
    else:
        _mean = utils._EngineCharScalarType(
            'M', {'fun_name': fun_name, 'entity': '_mean'}, exp_chrlen=1,
        ).to_ctypes()
    _n = utils._EngineIntScalarType(
        n, {'fun_name': fun_name, 'entity': 'n'},
    ).to_ctypes()
    _lda_py = utils._EngineFloat64ArrayType.get_nd(a, {'fun_name': fun_name, 'entity': 'a'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _lda = utils._EngineIntScalarType(
        _lda_py, {'fun_name': fun_name, 'entity': '_lda'},
    ).to_ctypes()
    _n_py = utils._EngineIntScalarType.to_py(_n)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldb_py = utils._EngineFloat64ArrayType.get_nd(b, {'fun_name': fun_name, 'entity': 'b'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldb = utils._EngineIntScalarType(
        _ldb_py, {'fun_name': fun_name, 'entity': '_ldb'},
    ).to_ctypes()
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _ldc_py = utils._EngineFloat64ArrayType.get_nd(c, {'fun_name': fun_name, 'entity': 'c'}, exp_rank=2, sorder=_m_sorder, dim_type='l', allow_none=True)
    _ldc = utils._EngineIntScalarType(
        _ldc_py, {'fun_name': fun_name, 'entity': '_ldc'},
    ).to_ctypes()
    if c is not None:
        _c_dim1 = _n_py
    else:
        _c_dim1 = 0
    exp_shape_c = (_c_dim1, _n_py)
    _c = utils._EngineFloat64ArrayType(
        c, {'fun_name': fun_name, 'entity': 'c'},
        exp_shape=exp_shape_c,
        mutual_sorder=_m_sorder,
        allow_none=True,
    ).to_ctypes()
    _ela = utils._EngineFloat64ArrayType(
        ela, {'fun_name': fun_name, 'entity': 'ela'},
        exp_shape=(_n_py,),
        allow_none=True,
    ).to_ctypes()
    _emu = utils._EngineFloat64ArrayType(
        emu, {'fun_name': fun_name, 'entity': 'emu'},
        exp_shape=(_n_py,),
        allow_none=True,
    ).to_ctypes()
    _ldsig_py = utils._EngineFloat64ArrayType.get_nd(sigma, {'fun_name': fun_name, 'entity': 'sigma'}, exp_rank=2, sorder=_m_sorder, dim_type='l')
    _ldsig = utils._EngineIntScalarType(
        _ldsig_py, {'fun_name': fun_name, 'entity': '_ldsig'},
    ).to_ctypes()
    _sigma = utils._EngineFloat64ArrayType(
        sigma, {'fun_name': fun_name, 'entity': 'sigma'},
        exp_shape=(_n_py, _n_py),
        mutual_sorder=_m_sorder,
    ).to_ctypes()
    _l1 = utils._EngineIntScalarType(
        l1, {'fun_name': fun_name, 'entity': 'l1'},
    ).to_ctypes()
    _l2 = utils._EngineIntScalarType(
        l2, {'fun_name': fun_name, 'entity': 'l2'},
    ).to_ctypes()
    _lmax = utils._EngineIntScalarType.empty_ctypes()
    _l2_py = utils._EngineIntScalarType.to_py(_l2)
    _l1_py = utils._EngineIntScalarType.to_py(_l1)
    _rmom_cls = utils._EngineFloat64ArrayType(
        rmom, {'fun_name': fun_name, 'entity': 'rmom'},
        exp_shape=(_l2_py-_l1_py+1,),
        check_contents=False,
    )
    _rmom = _rmom_cls.to_ctypes(can_reference_contents=False)
    _abserr = utils._EngineFloat64ScalarType.empty_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        eps, {'fun_name': fun_name, 'entity': 'eps'},
    ).to_ctypes()
    _wk = utils._EngineFloat64ArrayType.empty_ctypes(0)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _case,
        _mean,
        _n,
        _a,
        _lda,
        _b,
        _ldb,
        _c,
        _ldc,
        _ela,
        _emu,
        _sigma,
        _ldsig,
        _l1,
        _l2,
        _lmax,
        _rmom,
        _abserr,
        _eps,
        _wk,
        _errbuf,
        _ierr,
        _case._length_,
        _mean._length_,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99991:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, l2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: l2 <= 12.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99992:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, l1 = {:s} and l2 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: l2 >= l1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99993:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, l1 = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: l1 >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, eps = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if eps != 0.0, eps >= machine precision.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, n = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: n > 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99986:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sigma is not positive definite.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99987:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, b is not positive semidefinite or is null.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3 and _ifmt == 99983:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'Only {:s} moments exist,'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'less than l1 = {:s},'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'therefore none of the required moments can be computed.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4 and _ifmt == 99985:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The matrix L^TBL is not positive semidefinite or is null.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5 and _ifmt == 99988:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The computation to compute the eigenvalues required in the calculation',
                    'of moments has failed to converge: this is an unlikely error exit.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 6 and _ifmt == 99982:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'Only some of the required moments have been computed, the highest is',
                    'given by lmax.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 7 and _ifmt == 99984:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'The required accuracy has not been achieved in the integration. An estimate',
                    'of the accuracy is returned in abserr.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _rmom_cls.refresh_py(_rmom)
        _lmax_py = utils._EngineIntScalarType.to_py(_lmax)
        _abserr_py = utils._EngineFloat64ScalarType.to_py(_abserr)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['lmax', 'abserr'],
        )(
            _lmax_py,
            _abserr_py,
        )
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def pdf_landau_moment1(x):
    u"""
    Landau first moment function Phi_1(x) .

    For full information please refer to the NAG Library document for
    g01pt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01ptf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    mom1 : float
        The value of the first moment Phi_1(x) of the Landau density
        function.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.pdf_landau_moment1'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01ptft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, mom1.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _mom1 = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _mom1,
    )
    utils._handle_cb_exception(excs)
    _mom1_py = utils._EngineFloat64ScalarType.to_py(_mom1)
    return_data = _mom1_py
    return return_data

def pdf_landau_moment2(x):
    u"""
    Landau second moment function Phi_2(x) .

    For full information please refer to the NAG Library document for
    g01qt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01qtf.html

    Parameters
    ----------
    x : float
        The argument x of the function.

    Returns
    -------
    mom2 : float
        The value of the second moment Phi_2(x) of the Landau density
        function.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.pdf_landau_moment2'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01qtft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, mom2.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _mom2 = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _mom2,
    )
    utils._handle_cb_exception(excs)
    _mom2_py = utils._EngineFloat64ScalarType.to_py(_mom2)
    return_data = _mom2_py
    return return_data

def pdf_landau_deriv(x):
    u"""
    Landau derivative function phi'(lambda).

    For full information please refer to the NAG Library document for
    g01rt

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01rtf.html

    Parameters
    ----------
    x : float
        The argument lambda of the function.

    Returns
    -------
    deriv : float
        The value of the derivative phi'(lambda) of the Landau density
        function.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.pdf_landau_deriv'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01rtft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, x.
        utils._BYREF_FLOAT64, # Output, deriv.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _x = utils._EngineFloat64ScalarType(
        x, {'fun_name': fun_name, 'entity': 'x'},
    ).to_ctypes()
    _deriv = utils._EngineFloat64ScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _x,
        _deriv,
    )
    utils._handle_cb_exception(excs)
    _deriv_py = utils._EngineFloat64ScalarType.to_py(_deriv)
    return_data = _deriv_py
    return return_data

def prob_normal_vector(ltail, tail, lx, x, lxmu, xmu, lxstd, xstd, p, ivalid):
    u"""
    Computes a vector of probabilities for the standard Normal
    distribution.

    For full information please refer to the NAG Library document for
    g01sa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01saf.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates which tail the returned probabilities should
        represent. Letting Z denote a variate from a standard Normal
        distribution, and z_i = (x_i-mu_i)/(sigma_i), then for j =
        (mod(i-1, `ltail`)) +1 -1, for i = 1,2,...,max(`lx`, `ltail`,
        `lxmu`, `lxstd`):

        `tail`[j] = 'L'

          The lower tail probability is returned, i.e., p_i = P(Z <=
          z_i).

        `tail`[j] = 'U'

          The upper tail probability is returned, i.e., p_i = P(Z >=
          z_i).

        `tail`[j] = 'C'

          The two tail (confidence interval) probability is returned,
          i.e., p_i = P(Z <= |z_i|)-P(Z <= -|z_i|).

        `tail`[j] = 'S'

          The two tail (significance level) probability is returned,
          i.e., p_i = P(Z >= |z_i|)+P(Z <= -|z_i|).

    lx : int
        The dimension of the array `x`.

    x : float, shape (`lx`)
        x_i, the Normal variate values.

    lxmu : int
        The dimension of the array `xmu`.

    xmu : float, shape (`lxmu`)
        mu_i, the means.

    lxstd : int
        The dimension of the array `xstd`.

    xstd : float, shape (`lxstd`)
        sigma_i, the standard deviations.

    p : float, shape (max(`lx`, `ltail`, `lxmu`, `lxstd`)), modified in
    place
        On exit: p_i, the probabilities for the Normal distribution.

    ivalid : int, shape (max(`lx`, `ltail`, `lxmu`, `lxstd`)), modified
    in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          p_i.

        `ivalid`[i-1] = 2

          On entry, sigma_i <= 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, `ltail` = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, `lx` = *<value>*.

            Constraint: `lx` > 0.

        (`errno` 4)
            On entry, `lxmu` = *<value>*.

            Constraint: `lxmu` > 0.

        (`errno` 5)
            On entry, `lxstd` = *<value>*.

            Constraint: `lxstd` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `tail` or `xstd` was
            invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_normal_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01saft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lx.
        utils._EngineFloat64ArrayType.c_type, # Input, x[lx].
        utils._BYREF_INT, # Input, lxmu.
        utils._EngineFloat64ArrayType.c_type, # Input, xmu[lxmu].
        utils._BYREF_INT, # Input, lxstd.
        utils._EngineFloat64ArrayType.c_type, # Input, xstd[lxstd].
        utils._EngineFloat64ArrayType.c_type, # Output, p[max(lx, ltail, lxmu, lxstd)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(lx, ltail, lxmu, lxstd)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lx = utils._EngineIntScalarType(
        lx, {'fun_name': fun_name, 'entity': 'lx'},
    ).to_ctypes()
    _lx_py = utils._EngineIntScalarType.to_py(_lx)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_lx_py,),
    ).to_ctypes()
    _lxmu = utils._EngineIntScalarType(
        lxmu, {'fun_name': fun_name, 'entity': 'lxmu'},
    ).to_ctypes()
    _lxmu_py = utils._EngineIntScalarType.to_py(_lxmu)
    _xmu = utils._EngineFloat64ArrayType(
        xmu, {'fun_name': fun_name, 'entity': 'xmu'},
        exp_shape=(_lxmu_py,),
    ).to_ctypes()
    _lxstd = utils._EngineIntScalarType(
        lxstd, {'fun_name': fun_name, 'entity': 'lxstd'},
    ).to_ctypes()
    _lxstd_py = utils._EngineIntScalarType.to_py(_lxstd)
    _xstd = utils._EngineFloat64ArrayType(
        xstd, {'fun_name': fun_name, 'entity': 'xstd'},
        exp_shape=(_lxstd_py,),
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(max(_lx_py, _ltail_py, _lxmu_py, _lxstd_py),),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_lx_py, _ltail_py, _lxmu_py, _lxstd_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lx,
        _x,
        _lxmu,
        _xmu,
        _lxstd,
        _xstd,
        _p,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of tail or xstd was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, ltail = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lx = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lxmu = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lxmu > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, lxstd = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lxstd > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_cls.refresh_py(_p)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_students_t_vector(ltail, tail, lt, t, ldf, df, p, ivalid):
    u"""
    Computes a vector of probabilities for the Student's t-distribution.

    For full information please refer to the NAG Library document for
    g01sb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01sbf.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates which tail the returned probabilities should
        represent. For j = (mod(i-1, `ltail`)) +1 -1, for i =
        1,2,...,max(`ltail`, `lt`, `ldf`):

        `tail`[j] = 'L'

          The lower tail probability is returned, i.e., p_i = (T_i <=
          t_i:nu_i).

        `tail`[j] = 'U'

          The upper tail probability is returned, i.e., p_i = (T_i >=
          t_i:nu_i).

        `tail`[j] = 'C'

          The two tail (confidence interval) probability is returned,
          i.e., p_i = (T_i <= |t_i|:nu_i)-(T_i <= -|t_i|:nu_i).

        `tail`[j] = 'S'

          The two tail (significance level) probability is returned,
          i.e., p_i = (T_i >= |t_i|:nu_i)+(T_i <= -|t_i|:nu_i).

    lt : int
        The dimension of the array `t`.

    t : float, shape (`lt`)
        t_i, the values of the Student's t variates.

    ldf : int
        The dimension of the array `df`.

    df : float, shape (`ldf`)
        nu_i, the degrees of freedom of the Student's t-distribution.

    p : float, shape (max(`ltail`, `lt`, `ldf`)), modified in place
        On exit: p_i, the probabilities for the Student's t
        distribution.

    ivalid : int, shape (max(`ltail`, `lt`, `ldf`)), modified in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          p_i.

        `ivalid`[i-1] = 2

          On entry, nu_i < 1.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lt` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `ldf` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `tail` or `df` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_students_t_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01sbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lt.
        utils._EngineFloat64ArrayType.c_type, # Input, t[lt].
        utils._BYREF_INT, # Input, ldf.
        utils._EngineFloat64ArrayType.c_type, # Input, df[ldf].
        utils._EngineFloat64ArrayType.c_type, # Output, p[max(ltail, lt, ldf)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ltail, lt, ldf)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lt = utils._EngineIntScalarType(
        lt, {'fun_name': fun_name, 'entity': 'lt'},
    ).to_ctypes()
    _lt_py = utils._EngineIntScalarType.to_py(_lt)
    _t = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(_lt_py,),
    ).to_ctypes()
    _ldf = utils._EngineIntScalarType(
        ldf, {'fun_name': fun_name, 'entity': 'ldf'},
    ).to_ctypes()
    _ldf_py = utils._EngineIntScalarType.to_py(_ldf)
    _df = utils._EngineFloat64ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_ldf_py,),
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(max(_ltail_py, _lt_py, _ldf_py),),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ltail_py, _lt_py, _ldf_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lt,
        _t,
        _ldf,
        _df,
        _p,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of tail or df was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lt > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldf > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_cls.refresh_py(_p)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_chisq_vector(ltail, tail, lx, x, ldf, df, p, ivalid):
    u"""
    Computes a vector of probabilities for chi^2 distribution.

    For full information please refer to the NAG Library document for
    g01sc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01scf.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates whether the lower or upper tail probabilities are
        required. For j = (mod(i-1, `ltail`)) +1 -1, for i =
        1,2,...,max(`ltail`, `lx`, `ldf`):

        `tail`[j] = 'L'

          The lower tail probability is returned, i.e., p_i = (X_i <=
          x_i:nu_i).

        `tail`[j] = 'U'

          The upper tail probability is returned, i.e., p_i = (X_i >=
          x_i:nu_i).

    lx : int
        The dimension of the array `x`.

    x : float, shape (`lx`)
        x_i, the values of the chi^2 variates with nu_i degrees of
        freedom.

    ldf : int
        The dimension of the array `df`.

    df : float, shape (`ldf`)
        nu_i, the degrees of freedom of the chi^2-distribution.

    p : float, shape (max(`ltail`, `ldf`, `lx`)), modified in place
        On exit: p_i, the probabilities for the chi^2 distribution.

    ivalid : int, shape (max(`ltail`, `ldf`, `lx`)), modified in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          p_i.

        `ivalid`[i-1] = 2

          On entry, x_i < 0.0.

        `ivalid`[i-1] = 3

          On entry, nu_i <= 0.0.

        `ivalid`[i-1] = 4

          The solution has failed to converge while calculating the
          gamma variate. The result returned should represent an
          approximation to the solution.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lx` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `ldf` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `x`, `df` or `tail` was
            invalid, or the solution failed to converge.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_chisq_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01scft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lx.
        utils._EngineFloat64ArrayType.c_type, # Input, x[lx].
        utils._BYREF_INT, # Input, ldf.
        utils._EngineFloat64ArrayType.c_type, # Input, df[ldf].
        utils._EngineFloat64ArrayType.c_type, # Output, p[max(ltail, ldf, lx)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ltail, ldf, lx)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lx = utils._EngineIntScalarType(
        lx, {'fun_name': fun_name, 'entity': 'lx'},
    ).to_ctypes()
    _lx_py = utils._EngineIntScalarType.to_py(_lx)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_lx_py,),
    ).to_ctypes()
    _ldf = utils._EngineIntScalarType(
        ldf, {'fun_name': fun_name, 'entity': 'ldf'},
    ).to_ctypes()
    _ldf_py = utils._EngineIntScalarType.to_py(_ldf)
    _df = utils._EngineFloat64ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_ldf_py,),
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(max(_ltail_py, _ldf_py, _lx_py),),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ltail_py, _ldf_py, _lx_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lx,
        _x,
        _ldf,
        _df,
        _p,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of x, df or tail was invalid,',
                    'or the solution failed to converge.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lx > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldf > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_cls.refresh_py(_p)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_f_vector(ltail, tail, lf, f, ldf1, df1, ldf2, df2, p, ivalid):
    u"""
    Computes a vector of probabilities for F-distribution.

    For full information please refer to the NAG Library document for
    g01sd

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01sdf.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates whether the lower or upper tail probabilities are
        required. For j = (mod(i-1, `ltail`)) +1 -1, for i =
        1,2,...,max(`ltail`, `lf`, `ldf1`, `ldf2`):

        `tail`[j] = 'L'

          The lower tail probability is returned, i.e., p_i = (F_i <=
          f_i:u_i,v_i).

        `tail`[j] = 'U'

          The upper tail probability is returned, i.e., p_i = (F_i >=
          f_i:u_i,v_i).

    lf : int
        The dimension of the array `f`.

    f : float, shape (`lf`)
        f_i, the value of the F variate.

    ldf1 : int
        The dimension of the array `df1`.

    df1 : float, shape (`ldf1`)
        u_i, the degrees of freedom of the numerator variance.

    ldf2 : int
        The dimension of the array `df2`.

    df2 : float, shape (`ldf2`)
        v_i, the degrees of freedom of the denominator variance.

    p : float, shape (max(`ltail`, `lf`, `ldf1`, `ldf2`)), modified in
    place
        On exit: p_i, the probabilities for the F-distribution.

    ivalid : int, shape (max(`ltail`, `lf`, `ldf1`, `ldf2`)), modified
    in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          p_i.

        `ivalid`[i-1] = 2

          On entry, f_i < 0.0.

        `ivalid`[i-1] = 3

          On entry, u_i <= 0.0, or, v_i <= 0.0.

        `ivalid`[i-1] = 4

          The solution has failed to converge. The result returned
          should represent an approximation to the solution.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lf` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `ldf1` > 0.

        (`errno` 5)
            On entry, array size = *<value>*.

            Constraint: `ldf2` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `f`, `df1`, `df2` or `tail`
            was invalid, or the solution failed to converge.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_f_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01sdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lf.
        utils._EngineFloat64ArrayType.c_type, # Input, f[lf].
        utils._BYREF_INT, # Input, ldf1.
        utils._EngineFloat64ArrayType.c_type, # Input, df1[ldf1].
        utils._BYREF_INT, # Input, ldf2.
        utils._EngineFloat64ArrayType.c_type, # Input, df2[ldf2].
        utils._EngineFloat64ArrayType.c_type, # Output, p[max(ltail, lf, ldf1, ldf2)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ltail, lf, ldf1, ldf2)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lf = utils._EngineIntScalarType(
        lf, {'fun_name': fun_name, 'entity': 'lf'},
    ).to_ctypes()
    _lf_py = utils._EngineIntScalarType.to_py(_lf)
    _f = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(_lf_py,),
    ).to_ctypes()
    _ldf1 = utils._EngineIntScalarType(
        ldf1, {'fun_name': fun_name, 'entity': 'ldf1'},
    ).to_ctypes()
    _ldf1_py = utils._EngineIntScalarType.to_py(_ldf1)
    _df1 = utils._EngineFloat64ArrayType(
        df1, {'fun_name': fun_name, 'entity': 'df1'},
        exp_shape=(_ldf1_py,),
    ).to_ctypes()
    _ldf2 = utils._EngineIntScalarType(
        ldf2, {'fun_name': fun_name, 'entity': 'ldf2'},
    ).to_ctypes()
    _ldf2_py = utils._EngineIntScalarType.to_py(_ldf2)
    _df2 = utils._EngineFloat64ArrayType(
        df2, {'fun_name': fun_name, 'entity': 'df2'},
        exp_shape=(_ldf2_py,),
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(max(_ltail_py, _lf_py, _ldf1_py, _ldf2_py),),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ltail_py, _lf_py, _ldf1_py, _ldf2_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lf,
        _f,
        _ldf1,
        _df1,
        _ldf2,
        _df2,
        _p,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of f, df1, df2 or tail was invalid,',
                    'or the solution failed to converge.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lf > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldf1 > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldf2 > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_cls.refresh_py(_p)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_beta_vector(ltail, tail, lbeta, beta, la, a, lb, b, p, ivalid):
    u"""
    Computes a vector of probabilities for the beta distribution.

    For full information please refer to the NAG Library document for
    g01se

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01sef.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates whether a lower or upper tail probabilities are
        required. For j = (mod(i-1, `ltail`)) +1 -1, for i =
        1,2,...,max(`ltail`, `lbeta`, `la`, `lb`):

        `tail`[j] = 'L'

          The lower tail probability is returned, i.e., p_i = (B_i <=
          beta_i:a_i,b_i).

        `tail`[j] = 'U'

          The upper tail probability is returned, i.e., p_i = (B_i >=
          beta_i:a_i,b_i).

    lbeta : int
        The dimension of the array `beta`.

    beta : float, shape (`lbeta`)
        beta_i, the value of the beta variate.

    la : int
        The dimension of the array `a`.

    a : float, shape (`la`)
        a_i, the first parameter of the required beta distribution.

    lb : int
        The dimension of the array `b`.

    b : float, shape (`lb`)
        b_i, the second parameter of the required beta distribution.

    p : float, shape (max(`ltail`, `lbeta`, `la`, `lb`)), modified in
    place
        On exit: p_i, the probabilities for the beta distribution.

    ivalid : int, shape (max(`ltail`, `lbeta`, `la`, `lb`)), modified in
    place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          p_i.

        `ivalid`[i-1] = 2

          On entry, beta_i < 0.0, or, beta_i > 1.0.

        `ivalid`[i-1] = 3

          On entry, a_i <= 0.0, or, b_i <= 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lbeta` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `la` > 0.

        (`errno` 5)
            On entry, array size = *<value>*.

            Constraint: `lb` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `beta`, `a`, `b` or `tail`
            was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_beta_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01seft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lbeta.
        utils._EngineFloat64ArrayType.c_type, # Input, beta[lbeta].
        utils._BYREF_INT, # Input, la.
        utils._EngineFloat64ArrayType.c_type, # Input, a[la].
        utils._BYREF_INT, # Input, lb.
        utils._EngineFloat64ArrayType.c_type, # Input, b[lb].
        utils._EngineFloat64ArrayType.c_type, # Output, p[max(ltail, lbeta, la, lb)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ltail, lbeta, la, lb)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lbeta = utils._EngineIntScalarType(
        lbeta, {'fun_name': fun_name, 'entity': 'lbeta'},
    ).to_ctypes()
    _lbeta_py = utils._EngineIntScalarType.to_py(_lbeta)
    _beta = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(_lbeta_py,),
    ).to_ctypes()
    _la = utils._EngineIntScalarType(
        la, {'fun_name': fun_name, 'entity': 'la'},
    ).to_ctypes()
    _la_py = utils._EngineIntScalarType.to_py(_la)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_la_py,),
    ).to_ctypes()
    _lb = utils._EngineIntScalarType(
        lb, {'fun_name': fun_name, 'entity': 'lb'},
    ).to_ctypes()
    _lb_py = utils._EngineIntScalarType.to_py(_lb)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_lb_py,),
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(max(_ltail_py, _lbeta_py, _la_py, _lb_py),),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ltail_py, _lbeta_py, _la_py, _lb_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lbeta,
        _beta,
        _la,
        _a,
        _lb,
        _b,
        _p,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of beta, a, b or tail was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lbeta > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: la > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lb > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_cls.refresh_py(_p)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_gamma_vector(ltail, tail, lg, g, la, a, lb, b, p, ivalid):
    u"""
    Computes a vector of probabilities for the gamma distribution.

    For full information please refer to the NAG Library document for
    g01sf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01sff.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates whether a lower or upper tail probability is required.
        For j = (mod(i-1, `ltail`)) +1 -1, for i = 1,2,...,max(`ltail`,
        `lg`, `la`, `lb`):

        `tail`[j] = 'L'

          The lower tail probability is returned, i.e., p_i = (G_i <=
          g_i:alpha_i,beta_i).

        `tail`[j] = 'U'

          The upper tail probability is returned, i.e., p_i = (G_i >=
          g_i:alpha_i,beta_i).

    lg : int
        The dimension of the array `g`.

    g : float, shape (`lg`)
        g_i, the value of the gamma variate.

    la : int
        The dimension of the array `a`.

    a : float, shape (`la`)
        The parameter alpha_i of the gamma distribution.

    lb : int
        The dimension of the array `b`.

    b : float, shape (`lb`)
        The parameter beta_i of the gamma distribution.

    p : float, shape (max(`lg`, `la`, `lb`, `ltail`)), modified in place
        On exit: p_i, the probabilities of the beta distribution.

    ivalid : int, shape (max(`lg`, `la`, `lb`, `ltail`)), modified in
    place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          p_i.

        `ivalid`[i-1] = 2

          On entry, g_i < 0.0.

        `ivalid`[i-1] = 3

          On entry, alpha_i <= 0.0, or, beta_i <= 0.0.

        `ivalid`[i-1] = 4

          The solution did not converge in 600 iterations, see
          ``specfun.gamma_incomplete`` for details. The probability
          returned should be a reasonable approximation to the solution.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lg` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `la` > 0.

        (`errno` 5)
            On entry, array size = *<value>*.

            Constraint: `lb` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `g`, `a`, `b` or `tail` was
            invalid, or the solution did not converge.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_gamma_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01sfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lg.
        utils._EngineFloat64ArrayType.c_type, # Input, g[lg].
        utils._BYREF_INT, # Input, la.
        utils._EngineFloat64ArrayType.c_type, # Input, a[la].
        utils._BYREF_INT, # Input, lb.
        utils._EngineFloat64ArrayType.c_type, # Input, b[lb].
        utils._EngineFloat64ArrayType.c_type, # Output, p[max(lg, la, lb, ltail)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(lg, la, lb, ltail)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lg = utils._EngineIntScalarType(
        lg, {'fun_name': fun_name, 'entity': 'lg'},
    ).to_ctypes()
    _lg_py = utils._EngineIntScalarType.to_py(_lg)
    _g = utils._EngineFloat64ArrayType(
        g, {'fun_name': fun_name, 'entity': 'g'},
        exp_shape=(_lg_py,),
    ).to_ctypes()
    _la = utils._EngineIntScalarType(
        la, {'fun_name': fun_name, 'entity': 'la'},
    ).to_ctypes()
    _la_py = utils._EngineIntScalarType.to_py(_la)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_la_py,),
    ).to_ctypes()
    _lb = utils._EngineIntScalarType(
        lb, {'fun_name': fun_name, 'entity': 'lb'},
    ).to_ctypes()
    _lb_py = utils._EngineIntScalarType.to_py(_lb)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_lb_py,),
    ).to_ctypes()
    _p_cls = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(max(_lg_py, _la_py, _lb_py, _ltail_py),),
        check_contents=False,
    )
    _p = _p_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_lg_py, _la_py, _lb_py, _ltail_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lg,
        _g,
        _la,
        _a,
        _lb,
        _b,
        _p,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of g, a, b or tail was invalid,',
                    'or the solution did not converge.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lg > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: la > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lb > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _p_cls.refresh_py(_p)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_binomial_vector(ln, n, lp, p, lk, k, plek, pgtk, peqk, ivalid):
    u"""
    Computes a vector of probabilities for the binomial distribution.

    For full information please refer to the NAG Library document for
    g01sj

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01sjf.html

    Parameters
    ----------
    ln : int
        The dimension of the array `n`.

    n : int, shape (`ln`)
        n_i, the first parameter of the binomial distribution.

    lp : int
        The dimension of the array `p`.

    p : float, shape (`lp`)
        p_i, the second parameter of the binomial distribution.

    lk : int
        The dimension of the array `k`.

    k : int, shape (`lk`)
        k_i, the integer which defines the required probabilities.

    plek : float, shape (max(`ln`, `lp`, `lk`)), modified in place
        On exit: Prob{X_i <= k_i}, the lower tail probabilities.

    pgtk : float, shape (max(`ln`, `lp`, `lk`)), modified in place
        On exit: Prob{X_i > k_i}, the upper tail probabilities.

    peqk : float, shape (max(`ln`, `lp`, `lk`)), modified in place
        On exit: Prob{X_i = k_i}, the point probabilities.

    ivalid : int, shape (max(`ln`, `lp`, `lk`)), modified in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, n_i < 0.

        `ivalid`[i-1] = 2

          On entry, p_i <= 0.0, or, p_i >= 1.0.

        `ivalid`[i-1] = 3

          On entry, k_i < 0, or, k_i > n_i.

        `ivalid`[i-1] = 4

          On entry, n_i is too large to be represented exactly as a real
          number.

        `ivalid`[i-1] = 5

          On entry, the variance (= n_i p_i(1-p_i)) exceeds 10^6.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ln` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lp` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `lk` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `n`, `p` or `k` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_binomial_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01sjft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ln.
        utils._EngineIntArrayType.c_type, # Input, n[ln].
        utils._BYREF_INT, # Input, lp.
        utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
        utils._BYREF_INT, # Input, lk.
        utils._EngineIntArrayType.c_type, # Input, k[lk].
        utils._EngineFloat64ArrayType.c_type, # Output, plek[max(ln, lp, lk)].
        utils._EngineFloat64ArrayType.c_type, # Output, pgtk[max(ln, lp, lk)].
        utils._EngineFloat64ArrayType.c_type, # Output, peqk[max(ln, lp, lk)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ln, lp, lk)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ln = utils._EngineIntScalarType(
        ln, {'fun_name': fun_name, 'entity': 'ln'},
    ).to_ctypes()
    _ln_py = utils._EngineIntScalarType.to_py(_ln)
    _n = utils._EngineIntArrayType(
        n, {'fun_name': fun_name, 'entity': 'n'},
        exp_shape=(_ln_py,),
    ).to_ctypes()
    _lp = utils._EngineIntScalarType(
        lp, {'fun_name': fun_name, 'entity': 'lp'},
    ).to_ctypes()
    _lp_py = utils._EngineIntScalarType.to_py(_lp)
    _p = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_lp_py,),
    ).to_ctypes()
    _lk = utils._EngineIntScalarType(
        lk, {'fun_name': fun_name, 'entity': 'lk'},
    ).to_ctypes()
    _lk_py = utils._EngineIntScalarType.to_py(_lk)
    _k = utils._EngineIntArrayType(
        k, {'fun_name': fun_name, 'entity': 'k'},
        exp_shape=(_lk_py,),
    ).to_ctypes()
    _plek_cls = utils._EngineFloat64ArrayType(
        plek, {'fun_name': fun_name, 'entity': 'plek'},
        exp_shape=(max(_ln_py, _lp_py, _lk_py),),
        check_contents=False,
    )
    _plek = _plek_cls.to_ctypes(can_reference_contents=False)
    _pgtk_cls = utils._EngineFloat64ArrayType(
        pgtk, {'fun_name': fun_name, 'entity': 'pgtk'},
        exp_shape=(max(_ln_py, _lp_py, _lk_py),),
        check_contents=False,
    )
    _pgtk = _pgtk_cls.to_ctypes(can_reference_contents=False)
    _peqk_cls = utils._EngineFloat64ArrayType(
        peqk, {'fun_name': fun_name, 'entity': 'peqk'},
        exp_shape=(max(_ln_py, _lp_py, _lk_py),),
        check_contents=False,
    )
    _peqk = _peqk_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ln_py, _lp_py, _lk_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ln,
        _n,
        _lp,
        _p,
        _lk,
        _k,
        _plek,
        _pgtk,
        _peqk,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of n, p or k was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ln > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lp > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lk > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _plek_cls.refresh_py(_plek)
        _pgtk_cls.refresh_py(_pgtk)
        _peqk_cls.refresh_py(_peqk)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_poisson_vector(ll, l, lk, k, plek, pgtk, peqk, ivalid):
    u"""
    Computes a vector of probabilities for the Poisson distribution.

    For full information please refer to the NAG Library document for
    g01sk

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01skf.html

    Parameters
    ----------
    ll : int
        The dimension of the array `l`.

    l : float, shape (`ll`)
        lambda_i, the parameter of the Poisson distribution.

    lk : int
        The dimension of the array `k`.

    k : int, shape (`lk`)
        k_i, the integer which defines the required probabilities.

    plek : float, shape (max(`ll`, `lk`)), modified in place
        On exit: Prob{X_i <= k_i}, the lower tail probabilities.

    pgtk : float, shape (max(`ll`, `lk`)), modified in place
        On exit: Prob{X_i > k_i}, the upper tail probabilities.

    peqk : float, shape (max(`ll`, `lk`)), modified in place
        On exit: Prob{X_i = k_i}, the point probabilities.

    ivalid : int, shape (max(`ll`, `lk`)), modified in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, lambda_i <= 0.0.

        `ivalid`[i-1] = 2

          On entry, k_i < 0.

        `ivalid`[i-1] = 3

          On entry, lambda_i > 10^6.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ll` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lk` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `l` or `k` was invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_poisson_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01skft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ll.
        utils._EngineFloat64ArrayType.c_type, # Input, l[ll].
        utils._BYREF_INT, # Input, lk.
        utils._EngineIntArrayType.c_type, # Input, k[lk].
        utils._EngineFloat64ArrayType.c_type, # Output, plek[max(ll, lk)].
        utils._EngineFloat64ArrayType.c_type, # Output, pgtk[max(ll, lk)].
        utils._EngineFloat64ArrayType.c_type, # Output, peqk[max(ll, lk)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ll, lk)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ll = utils._EngineIntScalarType(
        ll, {'fun_name': fun_name, 'entity': 'll'},
    ).to_ctypes()
    _ll_py = utils._EngineIntScalarType.to_py(_ll)
    _l = utils._EngineFloat64ArrayType(
        l, {'fun_name': fun_name, 'entity': 'l'},
        exp_shape=(_ll_py,),
    ).to_ctypes()
    _lk = utils._EngineIntScalarType(
        lk, {'fun_name': fun_name, 'entity': 'lk'},
    ).to_ctypes()
    _lk_py = utils._EngineIntScalarType.to_py(_lk)
    _k = utils._EngineIntArrayType(
        k, {'fun_name': fun_name, 'entity': 'k'},
        exp_shape=(_lk_py,),
    ).to_ctypes()
    _plek_cls = utils._EngineFloat64ArrayType(
        plek, {'fun_name': fun_name, 'entity': 'plek'},
        exp_shape=(max(_ll_py, _lk_py),),
        check_contents=False,
    )
    _plek = _plek_cls.to_ctypes(can_reference_contents=False)
    _pgtk_cls = utils._EngineFloat64ArrayType(
        pgtk, {'fun_name': fun_name, 'entity': 'pgtk'},
        exp_shape=(max(_ll_py, _lk_py),),
        check_contents=False,
    )
    _pgtk = _pgtk_cls.to_ctypes(can_reference_contents=False)
    _peqk_cls = utils._EngineFloat64ArrayType(
        peqk, {'fun_name': fun_name, 'entity': 'peqk'},
        exp_shape=(max(_ll_py, _lk_py),),
        check_contents=False,
    )
    _peqk = _peqk_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ll_py, _lk_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ll,
        _l,
        _lk,
        _k,
        _plek,
        _pgtk,
        _peqk,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of l or k was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ll > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lk > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _plek_cls.refresh_py(_plek)
        _pgtk_cls.refresh_py(_pgtk)
        _peqk_cls.refresh_py(_peqk)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def prob_hypergeom_vector(ln, n, ll, l, lm, m, lk, k, plek, pgtk, peqk, ivalid):
    u"""
    Computes a vector of probabilities for the hypergeometric
    distribution.

    For full information please refer to the NAG Library document for
    g01sl

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01slf.html

    Parameters
    ----------
    ln : int
        The dimension of the array `n`.

    n : int, shape (`ln`)
        n_i, the parameter of the hypergeometric distribution.

    ll : int
        The dimension of the array `l`.

    l : int, shape (`ll`)
        l_i, the parameter of the hypergeometric distribution.

    lm : int
        The dimension of the array `m`.

    m : int, shape (`lm`)
        m_i, the parameter of the hypergeometric distribution.

    lk : int
        The dimension of the array `k`.

    k : int, shape (`lk`)
        k_i, the integer which defines the required probabilities.

    plek : float, shape (max(`ln`, `ll`, `lm`, `lk`)), modified in place
        On exit: Prob{X_i <= k_i}, the lower tail probabilities.

    pgtk : float, shape (max(`ln`, `ll`, `lm`, `lk`)), modified in place
        On exit: Prob{X_i > k_i}, the upper tail probabilities.

    peqk : float, shape (max(`ln`, `ll`, `lm`, `lk`)), modified in place
        On exit: Prob{X_i = k_i}, the point probabilities.

    ivalid : int, shape (max(`ln`, `ll`, `lm`, `lk`)), modified in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, n_i < 0.

        `ivalid`[i-1] = 2

          On entry, l_i < 0, or, l_i > n_i.

        `ivalid`[i-1] = 3

          On entry, m_i < 0, or, m_i > n_i.

        `ivalid`[i-1] = 4

          On entry, k_i < 0, or, k_i > l_i, or, k_i > m_i, or, k_i <
          l_i+m_i-n_i.

        `ivalid`[i-1] = 5

          On entry, n_i is too large to be represented exactly as a real
          number.

        `ivalid`[i-1] = 6

          On entry, the variance (see [Description]) exceeds 10^6.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ln` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `ll` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `lm` > 0.

        (`errno` 5)
            On entry, array size = *<value>*.

            Constraint: `lk` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `n`, `l`, `m` or `k` was
            invalid, or the variance was too large.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.prob_hypergeom_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01slft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ln.
        utils._EngineIntArrayType.c_type, # Input, n[ln].
        utils._BYREF_INT, # Input, ll.
        utils._EngineIntArrayType.c_type, # Input, l[ll].
        utils._BYREF_INT, # Input, lm.
        utils._EngineIntArrayType.c_type, # Input, m[lm].
        utils._BYREF_INT, # Input, lk.
        utils._EngineIntArrayType.c_type, # Input, k[lk].
        utils._EngineFloat64ArrayType.c_type, # Output, plek[max(ln, ll, lm, lk)].
        utils._EngineFloat64ArrayType.c_type, # Output, pgtk[max(ln, ll, lm, lk)].
        utils._EngineFloat64ArrayType.c_type, # Output, peqk[max(ln, ll, lm, lk)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ln, ll, lm, lk)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ln = utils._EngineIntScalarType(
        ln, {'fun_name': fun_name, 'entity': 'ln'},
    ).to_ctypes()
    _ln_py = utils._EngineIntScalarType.to_py(_ln)
    _n = utils._EngineIntArrayType(
        n, {'fun_name': fun_name, 'entity': 'n'},
        exp_shape=(_ln_py,),
    ).to_ctypes()
    _ll = utils._EngineIntScalarType(
        ll, {'fun_name': fun_name, 'entity': 'll'},
    ).to_ctypes()
    _ll_py = utils._EngineIntScalarType.to_py(_ll)
    _l = utils._EngineIntArrayType(
        l, {'fun_name': fun_name, 'entity': 'l'},
        exp_shape=(_ll_py,),
    ).to_ctypes()
    _lm = utils._EngineIntScalarType(
        lm, {'fun_name': fun_name, 'entity': 'lm'},
    ).to_ctypes()
    _lm_py = utils._EngineIntScalarType.to_py(_lm)
    _m = utils._EngineIntArrayType(
        m, {'fun_name': fun_name, 'entity': 'm'},
        exp_shape=(_lm_py,),
    ).to_ctypes()
    _lk = utils._EngineIntScalarType(
        lk, {'fun_name': fun_name, 'entity': 'lk'},
    ).to_ctypes()
    _lk_py = utils._EngineIntScalarType.to_py(_lk)
    _k = utils._EngineIntArrayType(
        k, {'fun_name': fun_name, 'entity': 'k'},
        exp_shape=(_lk_py,),
    ).to_ctypes()
    _plek_cls = utils._EngineFloat64ArrayType(
        plek, {'fun_name': fun_name, 'entity': 'plek'},
        exp_shape=(max(_ln_py, _ll_py, _lm_py, _lk_py),),
        check_contents=False,
    )
    _plek = _plek_cls.to_ctypes(can_reference_contents=False)
    _pgtk_cls = utils._EngineFloat64ArrayType(
        pgtk, {'fun_name': fun_name, 'entity': 'pgtk'},
        exp_shape=(max(_ln_py, _ll_py, _lm_py, _lk_py),),
        check_contents=False,
    )
    _pgtk = _pgtk_cls.to_ctypes(can_reference_contents=False)
    _peqk_cls = utils._EngineFloat64ArrayType(
        peqk, {'fun_name': fun_name, 'entity': 'peqk'},
        exp_shape=(max(_ln_py, _ll_py, _lm_py, _lk_py),),
        check_contents=False,
    )
    _peqk = _peqk_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ln_py, _ll_py, _lm_py, _lk_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ln,
        _n,
        _ll,
        _l,
        _lm,
        _m,
        _lk,
        _k,
        _plek,
        _pgtk,
        _peqk,
        _ivalid,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of n, l, m or k was invalid,',
                    'or the variance was too large.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ln > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ll > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lm > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lk > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _plek_cls.refresh_py(_plek)
        _pgtk_cls.refresh_py(_pgtk)
        _peqk_cls.refresh_py(_peqk)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def inv_cdf_normal_vector(ltail, tail, lp, p, lxmu, xmu, lxstd, xstd, x, ivalid):
    u"""
    Computes a vector of deviates for the standard Normal distribution.

    For full information please refer to the NAG Library document for
    g01ta

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01taf.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates which tail the supplied probabilities represent.
        Letting Z denote a variate from a standard Normal distribution,
        and z_i = (x_p_i-mu_i)/(sigma_i), then for j = (mod(i-1,
        `ltail`)) +1 -1, for i = 1,2,...,max(`ltail`, `lp`, `lxmu`,
        `lxstd`):

        `tail`[j] = 'L'

          The lower tail probability, i.e., p_i = P(Z <= z_i).

        `tail`[j] = 'U'

          The upper tail probability, i.e., p_i = P(Z >= z_i).

        `tail`[j] = 'C'

          The two tail (confidence interval) probability, i.e., p_i =
          P(Z <= |z_i|)-P(Z <= -|z_i|).

        `tail`[j] = 'S'

          The two tail (significance level) probability, i.e., p_i = P(Z
          >= |z_i|)+P(Z <= -|z_i|).

    lp : int
        The dimension of the array `p`.

    p : float, shape (`lp`)
        p_i, the probabilities for the Normal distribution as defined by
        `tail` with p_i = `p`[j], j = mod(i-1, `lp`).

    lxmu : int
        The dimension of the array `xmu`.

    xmu : float, shape (`lxmu`)
        mu_i, the means.

    lxstd : int
        The dimension of the array `xstd`.

    xstd : float, shape (`lxstd`)
        sigma_i, the standard deviations.

    x : float, shape (max(`ltail`, `lxmu`, `lxstd`, `lp`)), modified in
    place
        On exit: x_p_i, the deviates for the Normal distribution.

    ivalid : int, shape (max(`ltail`, `lxmu`, `lxstd`, `lp`)), modified
    in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          x_p_i.

        `ivalid`[i-1] = 2

          On entry, p_i <= 0.0, or, p_i >= 1.0.

        `ivalid`[i-1] = 3

          On entry, sigma_i <= 0.0.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lp` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `lxmu` > 0.

        (`errno` 5)
            On entry, array size = *<value>*.

            Constraint: `lxstd` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `tail`, `xstd` or `p` was
            invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_normal_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01taft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lp.
        utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
        utils._BYREF_INT, # Input, lxmu.
        utils._EngineFloat64ArrayType.c_type, # Input, xmu[lxmu].
        utils._BYREF_INT, # Input, lxstd.
        utils._EngineFloat64ArrayType.c_type, # Input, xstd[lxstd].
        utils._EngineFloat64ArrayType.c_type, # Output, x[max(ltail, lxmu, lxstd, lp)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ltail, lxmu, lxstd, lp)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lp = utils._EngineIntScalarType(
        lp, {'fun_name': fun_name, 'entity': 'lp'},
    ).to_ctypes()
    _lp_py = utils._EngineIntScalarType.to_py(_lp)
    _p = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_lp_py,),
    ).to_ctypes()
    _lxmu = utils._EngineIntScalarType(
        lxmu, {'fun_name': fun_name, 'entity': 'lxmu'},
    ).to_ctypes()
    _lxmu_py = utils._EngineIntScalarType.to_py(_lxmu)
    _xmu = utils._EngineFloat64ArrayType(
        xmu, {'fun_name': fun_name, 'entity': 'xmu'},
        exp_shape=(_lxmu_py,),
    ).to_ctypes()
    _lxstd = utils._EngineIntScalarType(
        lxstd, {'fun_name': fun_name, 'entity': 'lxstd'},
    ).to_ctypes()
    _lxstd_py = utils._EngineIntScalarType.to_py(_lxstd)
    _xstd = utils._EngineFloat64ArrayType(
        xstd, {'fun_name': fun_name, 'entity': 'xstd'},
        exp_shape=(_lxstd_py,),
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(max(_ltail_py, _lxmu_py, _lxstd_py, _lp_py),),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ltail_py, _lxmu_py, _lxstd_py, _lp_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lp,
        _p,
        _lxmu,
        _xmu,
        _lxstd,
        _xstd,
        _x,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of tail, xstd or p was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lp > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lxmu > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lxstd > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def inv_cdf_students_t_vector(ltail, tail, lp, p, ldf, df, t, ivalid):
    u"""
    Computes a vector of deviates for Student's t-distribution.

    For full information please refer to the NAG Library document for
    g01tb

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01tbf.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates which tail the supplied probabilities represent. For j
        = (mod(i-1, `ltail`)) +1 -1, for i = 1,2,...,max(`ltail`, `lp`,
        `ldf`):

        `tail`[j] = 'L'

          The lower tail probability, i.e., p_i = (T_i <= t_p_i:nu_i).

        `tail`[j] = 'U'

          The upper tail probability, i.e., p_i = (T_i >= t_p_i:nu_i).

        `tail`[j] = 'C'

          The two tail (confidence interval) probability, i.e., p_i =
          (T_i <= |t_p_i|:nu_i)-(T_i <= -|t_p_i|:nu_i).

        `tail`[j] = 'S'

          The two tail (significance level) probability, i.e., p_i =
          (T_i >= |t_p_i|:nu_i)+(T_i <= -|t_p_i|:nu_i).

    lp : int
        The dimension of the array `p`.

    p : float, shape (`lp`)
        p_i, the probability of the required Student's t-distribution as
        defined by `tail` .

    ldf : int
        The dimension of the array `df`.

    df : float, shape (`ldf`)
        nu_i, the degrees of freedom of the Student's t-distribution.

    t : float, shape (max(`ltail`, `lp`, `ldf`)), modified in place
        On exit: t_p_i, the deviates for the Student's t-distribution.

    ivalid : int, shape (max(`ltail`, `lp`, `ldf`)), modified in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          t_p_i.

        `ivalid`[i-1] = 2

          On entry, p_i <= 0.0, or, p_i >= 1.0.

        `ivalid`[i-1] = 3

          On entry, nu_i < 1.0.

        `ivalid`[i-1] = 4

          The solution has failed to converge. The result returned
          should represent an approximation to the solution.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lp` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `ldf` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `tail`, `p` or `df` was
            invalid, or the solution failed to converge.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_students_t_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01tbft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lp.
        utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
        utils._BYREF_INT, # Input, ldf.
        utils._EngineFloat64ArrayType.c_type, # Input, df[ldf].
        utils._EngineFloat64ArrayType.c_type, # Output, t[max(ltail, lp, ldf)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ltail, lp, ldf)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lp = utils._EngineIntScalarType(
        lp, {'fun_name': fun_name, 'entity': 'lp'},
    ).to_ctypes()
    _lp_py = utils._EngineIntScalarType.to_py(_lp)
    _p = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_lp_py,),
    ).to_ctypes()
    _ldf = utils._EngineIntScalarType(
        ldf, {'fun_name': fun_name, 'entity': 'ldf'},
    ).to_ctypes()
    _ldf_py = utils._EngineIntScalarType.to_py(_ldf)
    _df = utils._EngineFloat64ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_ldf_py,),
    ).to_ctypes()
    _t_cls = utils._EngineFloat64ArrayType(
        t, {'fun_name': fun_name, 'entity': 't'},
        exp_shape=(max(_ltail_py, _lp_py, _ldf_py),),
        check_contents=False,
    )
    _t = _t_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ltail_py, _lp_py, _ldf_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lp,
        _p,
        _ldf,
        _df,
        _t,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of tail, p or df was invalid,',
                    'or the solution failed to converge.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lp > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldf > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _t_cls.refresh_py(_t)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def inv_cdf_chisq_vector(ltail, tail, lp, p, ldf, df, x, ivalid):
    u"""
    Computes a vector of deviates for chi^2 distribution.

    For full information please refer to the NAG Library document for
    g01tc

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01tcf.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates which tail the supplied probabilities represent. For j
        = (mod(i-1, `ltail`)) +1 -1, for i = 1,2,...,max(`ltail`, `lp`,
        `ldf`):

        `tail`[j] = 'L'

          The lower tail probability, i.e., p_i = (X_i <= x_p_i:nu_i).

        `tail`[j] = 'U'

          The upper tail probability, i.e., p_i = (X_i >= x_p_i:nu_i).

    lp : int
        The dimension of the array `p`.

    p : float, shape (`lp`)
        p_i, the probability of the required chi^2-distribution as
        defined by `tail` .

    ldf : int
        The dimension of the array `df`.

    df : float, shape (`ldf`)
        nu_i, the degrees of freedom of the chi^2-distribution.

    x : float, shape (max(`ltail`, `lp`, `ldf`)), modified in place
        On exit: x_p_i, the deviates for the chi^2-distribution.

    ivalid : int, shape (max(`ltail`, `lp`, `ldf`)), modified in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          x_p_i.

        `ivalid`[i-1] = 2

          On entry, invalid value for p_i.

        `ivalid`[i-1] = 3

          On entry, nu_i <= 0.0.

        `ivalid`[i-1] = 4

          p_i is too close to 0.0 or 1.0 for the result to be
          calculated.

        `ivalid`[i-1] = 5

          The solution has failed to converge. The result should be a
          reasonable approximation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lp` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `ldf` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `tail`, `p` or `df` was
            invalid, or the solution failed to converge.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_chisq_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01tcft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lp.
        utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
        utils._BYREF_INT, # Input, ldf.
        utils._EngineFloat64ArrayType.c_type, # Input, df[ldf].
        utils._EngineFloat64ArrayType.c_type, # Output, x[max(ltail, lp, ldf)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ltail, lp, ldf)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lp = utils._EngineIntScalarType(
        lp, {'fun_name': fun_name, 'entity': 'lp'},
    ).to_ctypes()
    _lp_py = utils._EngineIntScalarType.to_py(_lp)
    _p = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_lp_py,),
    ).to_ctypes()
    _ldf = utils._EngineIntScalarType(
        ldf, {'fun_name': fun_name, 'entity': 'ldf'},
    ).to_ctypes()
    _ldf_py = utils._EngineIntScalarType.to_py(_ldf)
    _df = utils._EngineFloat64ArrayType(
        df, {'fun_name': fun_name, 'entity': 'df'},
        exp_shape=(_ldf_py,),
    ).to_ctypes()
    _x_cls = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(max(_ltail_py, _lp_py, _ldf_py),),
        check_contents=False,
    )
    _x = _x_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ltail_py, _lp_py, _ldf_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lp,
        _p,
        _ldf,
        _df,
        _x,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of tail, p or df was invalid,',
                    'or the solution failed to converge.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lp > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldf > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _x_cls.refresh_py(_x)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def inv_cdf_f_vector(ltail, tail, lp, p, ldf1, df1, ldf2, df2, f, ivalid):
    u"""
    Computes a vector of deviates for F-distribution.

    For full information please refer to the NAG Library document for
    g01td

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01tdf.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates which tail the supplied probabilities represent. For j
        = (mod(i-1, `ltail`)) +1 -1, for i = 1,2,...,max(`ltail`, `lp`,
        `ldf1`, `ldf2`):

        `tail`[j] = 'L'

          The lower tail probability, i.e., p_i = (F_i <=
          f_p_i:u_i,v_i).

        `tail`[j] = 'U'

          The upper tail probability, i.e., p_i = (F_i >=
          f_p_i:u_i,v_i).

    lp : int
        The dimension of the array `p`.

    p : float, shape (`lp`)
        p_i, the probability of the required F-distribution as defined
        by `tail` .

    ldf1 : int
        The dimension of the array `df1`.

    df1 : float, shape (`ldf1`)
        u_i, the degrees of freedom of the numerator variance.

    ldf2 : int
        The dimension of the array `df2`.

    df2 : float, shape (`ldf2`)
        v_i, the degrees of freedom of the denominator variance.

    f : float, shape (max(`ltail`, `lp`, `ldf1`, `ldf2`)), modified in
    place
        On exit: f_p_i, the deviates for the F-distribution.

    ivalid : int, shape (max(`ltail`, `lp`, `ldf1`, `ldf2`)), modified
    in place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          f_p_i.

        `ivalid`[i-1] = 2

          On entry, invalid value for p_i.

        `ivalid`[i-1] = 3

          On entry, u_i <= 0.0, or, v_i <= 0.0.

        `ivalid`[i-1] = 4

          The solution has not converged. The result should still be a
          reasonable approximation to the solution.

        `ivalid`[i-1] = 5

          The value of p_i is too close to 0.0 or 1.0 for the result to
          be computed. This will only occur when the large sample
          approximations are used.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lp` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `ldf1` > 0.

        (`errno` 5)
            On entry, array size = *<value>*.

            Constraint: `ldf2` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `tail`, `p`, `df1`, `df2`
            was invalid, or the solution failed to converge.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_f_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01tdft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lp.
        utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
        utils._BYREF_INT, # Input, ldf1.
        utils._EngineFloat64ArrayType.c_type, # Input, df1[ldf1].
        utils._BYREF_INT, # Input, ldf2.
        utils._EngineFloat64ArrayType.c_type, # Input, df2[ldf2].
        utils._EngineFloat64ArrayType.c_type, # Output, f[max(ltail, lp, ldf1, ldf2)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ltail, lp, ldf1, ldf2)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lp = utils._EngineIntScalarType(
        lp, {'fun_name': fun_name, 'entity': 'lp'},
    ).to_ctypes()
    _lp_py = utils._EngineIntScalarType.to_py(_lp)
    _p = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_lp_py,),
    ).to_ctypes()
    _ldf1 = utils._EngineIntScalarType(
        ldf1, {'fun_name': fun_name, 'entity': 'ldf1'},
    ).to_ctypes()
    _ldf1_py = utils._EngineIntScalarType.to_py(_ldf1)
    _df1 = utils._EngineFloat64ArrayType(
        df1, {'fun_name': fun_name, 'entity': 'df1'},
        exp_shape=(_ldf1_py,),
    ).to_ctypes()
    _ldf2 = utils._EngineIntScalarType(
        ldf2, {'fun_name': fun_name, 'entity': 'ldf2'},
    ).to_ctypes()
    _ldf2_py = utils._EngineIntScalarType.to_py(_ldf2)
    _df2 = utils._EngineFloat64ArrayType(
        df2, {'fun_name': fun_name, 'entity': 'df2'},
        exp_shape=(_ldf2_py,),
    ).to_ctypes()
    _f_cls = utils._EngineFloat64ArrayType(
        f, {'fun_name': fun_name, 'entity': 'f'},
        exp_shape=(max(_ltail_py, _lp_py, _ldf1_py, _ldf2_py),),
        check_contents=False,
    )
    _f = _f_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ltail_py, _lp_py, _ldf1_py, _ldf2_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lp,
        _p,
        _ldf1,
        _df1,
        _ldf2,
        _df2,
        _f,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of tail, p, df1, df2 was invalid,',
                    'or the solution failed to converge.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lp > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldf1 > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ldf2 > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _f_cls.refresh_py(_f)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def inv_cdf_beta_vector(ltail, tail, lp, p, la, a, lb, b, tol, beta, ivalid):
    u"""
    Computes a vector of deviates for the beta distribution.

    For full information please refer to the NAG Library document for
    g01te

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01tef.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates which tail the supplied probabilities represent. For j
        = (mod(i-1, `ltail`)) +1 -1, for i = 1,2,...,max(`ltail`, `lp`,
        `la`, `lb`):

        `tail`[j] = 'L'

          The lower tail probability, i.e., p_i = (B_i <=
          beta_p_i:a_i,b_i).

        `tail`[j] = 'U'

          The upper tail probability, i.e., p_i = (B_i >=
          beta_p_i:a_i,b_i).

    lp : int
        The dimension of the array `p`.

    p : float, shape (`lp`)
        p_i, the probability of the required beta distribution as
        defined by `tail` .

    la : int
        The dimension of the array `a`.

    a : float, shape (`la`)
        a_i, the first parameter of the required beta distribution.

    lb : int
        The dimension of the array `b`.

    b : float, shape (`lb`)
        b_i, the second parameter of the required beta distribution.

    tol : float
        The relative accuracy required by you in the results. If
        ``inv_cdf_beta_vector`` is entered with `tol` greater than or
        equal to 1.0 or less than 10*machine precision (see
        ``machine.precision``), the value of 10*machine precision is
        used instead.

    beta : float, shape (max(`ltail`, `lp`, `la`, `lb`)), modified in
    place
        On exit: beta_p_i, the deviates for the beta distribution.

    ivalid : int, shape (max(`ltail`, `lp`, `la`, `lb`)), modified in
    place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          beta_p_i.

        `ivalid`[i-1] = 2

          On entry, p_i < 0.0, or, p_i > 1.0.

        `ivalid`[i-1] = 3

          On entry, a_i <= 0.0, or, a_i > 10^6, or, b_i <= 0.0, or, b_i
          > 10^6.

        `ivalid`[i-1] = 4

          The solution has not converged but the result should be a
          reasonable approximation to the solution.

        `ivalid`[i-1] = 5

          Requested accuracy not achieved when calculating the beta
          probability. The result should be a reasonable approximation
          to the correct solution.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lp` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `la` > 0.

        (`errno` 5)
            On entry, array size = *<value>*.

            Constraint: `lb` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `tail`, `p`, `a`, or `b` was
            invalid, or the solution failed to converge.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_beta_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01teft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lp.
        utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
        utils._BYREF_INT, # Input, la.
        utils._EngineFloat64ArrayType.c_type, # Input, a[la].
        utils._BYREF_INT, # Input, lb.
        utils._EngineFloat64ArrayType.c_type, # Input, b[lb].
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Output, beta[max(ltail, lp, la, lb)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ltail, lp, la, lb)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lp = utils._EngineIntScalarType(
        lp, {'fun_name': fun_name, 'entity': 'lp'},
    ).to_ctypes()
    _lp_py = utils._EngineIntScalarType.to_py(_lp)
    _p = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_lp_py,),
    ).to_ctypes()
    _la = utils._EngineIntScalarType(
        la, {'fun_name': fun_name, 'entity': 'la'},
    ).to_ctypes()
    _la_py = utils._EngineIntScalarType.to_py(_la)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_la_py,),
    ).to_ctypes()
    _lb = utils._EngineIntScalarType(
        lb, {'fun_name': fun_name, 'entity': 'lb'},
    ).to_ctypes()
    _lb_py = utils._EngineIntScalarType.to_py(_lb)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_lb_py,),
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _beta_cls = utils._EngineFloat64ArrayType(
        beta, {'fun_name': fun_name, 'entity': 'beta'},
        exp_shape=(max(_ltail_py, _lp_py, _la_py, _lb_py),),
        check_contents=False,
    )
    _beta = _beta_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ltail_py, _lp_py, _la_py, _lb_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lp,
        _p,
        _la,
        _a,
        _lb,
        _b,
        _tol,
        _beta,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of tail, p, a, or b was invalid,',
                    'or the solution failed to converge.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lp > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: la > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lb > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _beta_cls.refresh_py(_beta)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def inv_cdf_gamma_vector(ltail, tail, lp, p, la, a, lb, b, tol, g, ivalid):
    u"""
    Computes a vector of deviates for the gamma distribution.

    For full information please refer to the NAG Library document for
    g01tf

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01tff.html

    Parameters
    ----------
    ltail : int
        The dimension of the array `tail`.

    tail : str, length 1, shape (`ltail`)
        Indicates which tail the supplied probabilities represent. For j
        = (mod(i-1, `ltail`)) +1 -1, for i = 1,2,...,max(`ltail`, `lp`,
        `la`, `lb`):

        `tail`[j] = 'L'

          The lower tail probability, i.e., p_i = (G_i <=
          g_p_i:alpha_i,beta_i).

        `tail`[j] = 'U'

          The upper tail probability, i.e., p_i = (G_i >=
          g_p_i:alpha_i,beta_i).

    lp : int
        The dimension of the array `p`.

    p : float, shape (`lp`)
        p_i, the probability of the required gamma distribution as
        defined by `tail` .

    la : int
        The dimension of the array `a`.

    a : float, shape (`la`)
        alpha_i, the first parameter of the required gamma distribution.

    lb : int
        The dimension of the array `b`.

    b : float, shape (`lb`)
        beta_i, the second parameter of the required gamma distribution.

    tol : float
        The relative accuracy required by you in the results. If
        ``inv_cdf_gamma_vector`` is entered with `tol` greater than or
        equal to 1.0 or less than 10*machine precision (see
        ``machine.precision``), the value of 10*machine precision is
        used instead.

    g : float, shape (max(`ltail`, `lp`, `la`, `lb`)), modified in place
        On exit: g_p_i, the deviates for the gamma distribution.

    ivalid : int, shape (max(`ltail`, `lp`, `la`, `lb`)), modified in
    place
        On exit: `ivalid`[i-1] indicates any errors with the input
        arguments, with

        `ivalid`[i-1] = 0

          No error.

        `ivalid`[i-1] = 1

          On entry, invalid value supplied in `tail` when calculating
          g_p_i.

        `ivalid`[i-1] = 2

          On entry, invalid value for p_i.

        `ivalid`[i-1] = 3

          On entry, alpha_i <= 0.0, or, alpha_i > 10^6, or, beta_i <=
          0.0.

        `ivalid`[i-1] = 4

          p_i is too close to 0.0 or 1.0 to enable the result to be
          calculated.

        `ivalid`[i-1] = 5

          The solution has failed to converge. The result may be a
          reasonable approximation.

    Raises
    ------
    NagValueError
        (`errno` 2)
            On entry, array size = *<value>*.

            Constraint: `ltail` > 0.

        (`errno` 3)
            On entry, array size = *<value>*.

            Constraint: `lp` > 0.

        (`errno` 4)
            On entry, array size = *<value>*.

            Constraint: `la` > 0.

        (`errno` 5)
            On entry, array size = *<value>*.

            Constraint: `lb` > 0.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 1)
            On entry, at least one value of `tail`, `p`, `a`, or `b` was
            invalid.

            Check `ivalid` for more information.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.inv_cdf_gamma_vector'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01tfft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, ltail.
        utils._EngineCharArrayType.c_type, # Input, tail(len=1)[ltail].
        utils._BYREF_INT, # Input, lp.
        utils._EngineFloat64ArrayType.c_type, # Input, p[lp].
        utils._BYREF_INT, # Input, la.
        utils._EngineFloat64ArrayType.c_type, # Input, a[la].
        utils._BYREF_INT, # Input, lb.
        utils._EngineFloat64ArrayType.c_type, # Input, b[lb].
        utils._BYREF_FLOAT64, # Input, tol.
        utils._EngineFloat64ArrayType.c_type, # Output, g[max(ltail, lp, la, lb)].
        utils._EngineIntArrayType.c_type, # Output, ivalid[max(ltail, lp, la, lb)].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of tail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _ltail = utils._EngineIntScalarType(
        ltail, {'fun_name': fun_name, 'entity': 'ltail'},
    ).to_ctypes()
    _ltail_py = utils._EngineIntScalarType.to_py(_ltail)
    _tail_cls = utils._EngineCharArrayType(
        tail, {'fun_name': fun_name, 'entity': 'tail'}, exp_chrlen=1,
        exp_shape=(_ltail_py,),
    )
    _tail = _tail_cls.to_ctypes()
    _tail_chrlen = _tail_cls.chrlen
    _lp = utils._EngineIntScalarType(
        lp, {'fun_name': fun_name, 'entity': 'lp'},
    ).to_ctypes()
    _lp_py = utils._EngineIntScalarType.to_py(_lp)
    _p = utils._EngineFloat64ArrayType(
        p, {'fun_name': fun_name, 'entity': 'p'},
        exp_shape=(_lp_py,),
    ).to_ctypes()
    _la = utils._EngineIntScalarType(
        la, {'fun_name': fun_name, 'entity': 'la'},
    ).to_ctypes()
    _la_py = utils._EngineIntScalarType.to_py(_la)
    _a = utils._EngineFloat64ArrayType(
        a, {'fun_name': fun_name, 'entity': 'a'},
        exp_shape=(_la_py,),
    ).to_ctypes()
    _lb = utils._EngineIntScalarType(
        lb, {'fun_name': fun_name, 'entity': 'lb'},
    ).to_ctypes()
    _lb_py = utils._EngineIntScalarType.to_py(_lb)
    _b = utils._EngineFloat64ArrayType(
        b, {'fun_name': fun_name, 'entity': 'b'},
        exp_shape=(_lb_py,),
    ).to_ctypes()
    _tol = utils._EngineFloat64ScalarType(
        tol, {'fun_name': fun_name, 'entity': 'tol'},
    ).to_ctypes()
    _g_cls = utils._EngineFloat64ArrayType(
        g, {'fun_name': fun_name, 'entity': 'g'},
        exp_shape=(max(_ltail_py, _lp_py, _la_py, _lb_py),),
        check_contents=False,
    )
    _g = _g_cls.to_ctypes(can_reference_contents=False)
    _ivalid_cls = utils._EngineIntArrayType(
        ivalid, {'fun_name': fun_name, 'entity': 'ivalid'},
        exp_shape=(max(_ltail_py, _lp_py, _la_py, _lb_py),),
        check_contents=False,
    )
    _ivalid = _ivalid_cls.to_ctypes(can_reference_contents=False)
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _ltail,
        _tail,
        _lp,
        _p,
        _la,
        _a,
        _lb,
        _b,
        _tol,
        _g,
        _ivalid,
        _errbuf,
        _ierr,
        _tail_chrlen,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one value of tail, p, a, or b was invalid.',
                    'Check ivalid for more information.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: ltail > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 3:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lp > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 4:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: la > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 5:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, array size = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: lb > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _g_cls.refresh_py(_g)
        _ivalid_cls.refresh_py(_ivalid)
        return_data = None
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = return_data
    _err_class.manage()
    return return_data

def moving_average(m, nb, x, iwt, wt, pn, rmean, rsd, comm):
    u"""
    Computes the mean and standard deviation using a rolling window.

    For full information please refer to the NAG Library document for
    g01wa

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01waf.html

    Parameters
    ----------
    m : int
        m, the length of the rolling window.

        If `pn` != 0, `m` must be unchanged since the last call to
        ``moving_average``.

    nb : int
        b, the number of observations in the current block of data. The
        size of the block of data supplied in `x` (and when `iwt` = 1,
        `wt`) can vary; therefore `nb` can change between calls to
        ``moving_average``.

    x : float, shape (`nb`)
        The current block of observations, corresponding to x_i, for i =
        k+1,...,k+b, where k is the number of observations processed so
        far and b is the size of the current block of data.

    iwt : int
        The type of weighting to use.

        `iwt` = 0

          No weights are used.

        `iwt` = 1

          Each observation has its own weight.

        `iwt` = 2

          Each position in the window has its own weight.

        `iwt` = 3

          Each position in the window has a weight equal to its position
          number.

        If `pn` != 0, `iwt` must be unchanged since the last call to
        ``moving_average``.

    wt : None or float, shape (:)
        Note: the required length for this argument is determined as
        follows: if `iwt`=1: `nb`; if `iwt`=2: `m`; otherwise: 0.

        The user-supplied weights.

        If `iwt` = 1, `wt`[i-1] = nu_{i+k}, for i = 1,2,...,b.

        If `iwt` = 2, `wt`[j-1] = w_j, for j = 1,2,...,m.

    pn : int
        k, the number of observations processed so far. On the first
        call to ``moving_average``, or when starting to summarise a new
        dataset, `pn` must be set to 0.

        If `pn` != 0, it must be the same value as returned by the last
        call to ``moving_average``.

    rmean : float, shape (max(0, `nb`+min(0, `pn`-`m`+1))), modified in
    place
        On exit: mu_l, the (weighted) moving averages, for l =
        1,2,...,b+min(0, k-m+1). Therefore, mu_l is the mean of the data
        in the window that ends on `x`[l+m-min(k, m-1)-2].

        If, on entry, `pn` >= `m`-1, i.e., at least one windows worth of
        data has been previously processed, then `rmean`[l-1] is the
        summary corresponding to the window that ends on `x`[l-1].

        On the other hand, if, on entry, `pn` = 0, i.e., no data has
        been previously processed, then `rmean`[l-1] is the summary
        corresponding to the window that ends on `x`[`m`+l-2] (or,
        equivalently, starts on `x`[l-1]).

    rsd : None or float, shape (:), modified in place
        Note: the required length for this argument is determined as
        follows: if `rsd` is not None: max(0, `nb`+min(0, `pn`-`m`+1));
        otherwise: 0.

        On exit, if not None on entry: if `rsd` is not None on entry
        then sigma_l, the (weighted) standard deviation. The ordering of
        `rsd` is the same as the ordering of `rmean`.

        If `rsd` is None, `rsd` is not referenced.

    comm : None or dict, communication object, modified in place
        Communication structure.

        If None all the data must be supplied in one go, otherwise need
        not be set.

    Returns
    -------
    pn : int
        k+b, the updated number of observations processed so far.

    Raises
    ------
    NagValueError
        (`errno` 11)
            On entry, `m` = *<value>*.

            Constraint: `m` >= 1.

        (`errno` 12)
            On entry, `m` = *<value>*.

            On entry at previous call, `m` = *<value>*.

            Constraint: if `pn` > 0, `m` must be unchanged since
            previous call.

        (`errno` 21)
            On entry, `nb` = *<value>*.

            Constraint: `nb` >= 0.

        (`errno` 22)
            On entry, `nb` = *<value>*, `m` = *<value>*.

            Constraint: if `comm` is None, `nb` >= `m`.

        (`errno` 41)
            On entry, `iwt` = *<value>*.

            Constraint: `iwt` = 0, 1, 2 or 3.

        (`errno` 42)
            On entry, `iwt` = *<value>*.

            On entry at previous call, `iwt` = *<value>*.

            Constraint: if `pn` > 0, `iwt` must be unchanged since
            previous call.

        (`errno` 51)
            On entry, `wt`[*<value>*] = *<value>*.

            Constraint: `wt`[i-1] >= 0.

        (`errno` 52)
            On entry, `wt`[0] = *<value>*.

            Constraint: if `iwt` = 2, `wt`[0] > 0.

        (`errno` 55)
            On entry, sum of weights supplied in `wt` is *<value>*.

            Constraint: if `iwt` = 2, the sum of the weights > 0.

        (`errno` 61)
            On entry, `pn` = *<value>*.

            Constraint: `pn` >= 0.

        (`errno` 62)
            On entry, `pn` = *<value>*.

            On exit from previous call, `pn` = *<value>*.

            Constraint: if `pn` > 0, `pn` must be unchanged since
            previous call.

        (`errno` 101)
            `comm`['rcomm'] has been corrupted between calls.

    Warns
    -----
    NagAlgorithmicWarning
        (`errno` 53)
            On entry, at least one window had all zero weights.

        (`errno` 54)
            On entry, unable to calculate at least one standard
            deviation due to the weights supplied.

    See Also
    --------
    :meth:`naginterfaces.library.examples.stat.moving_average_ex.main`
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.moving_average'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01waft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_INT, # Input, m.
        utils._BYREF_INT, # Input, nb.
        utils._EngineFloat64ArrayType.c_type, # Input, x[nb].
        utils._BYREF_INT, # Input, iwt.
        utils._EngineFloat64ArrayType.c_type, # Input, wt[lwt].
        utils._BYREF_INT, # Input/Output, pn.
        utils._EngineFloat64ArrayType.c_type, # Output, rmean[max(0, nb+min(0, pn-m+1))].
        utils._EngineFloat64ArrayType.c_type, # Output, rsd[lrsd].
        utils._BYREF_INT, # Input, lrsd.
        utils._EngineFloat64ArrayType.c_type, # Input/Output, rcomm[lrcomm].
        utils._BYREF_INT, # Input, lrcomm.
        utils._BYREF_INT, # Input, lwt.
        utils._BYREF_FLOAT64, # Input, eps.
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _m = utils._EngineIntScalarType(
        m, {'fun_name': fun_name, 'entity': 'm'},
    ).to_ctypes()
    _nb = utils._EngineIntScalarType(
        nb, {'fun_name': fun_name, 'entity': 'nb'},
    ).to_ctypes()
    _nb_py = utils._EngineIntScalarType.to_py(_nb)
    _x = utils._EngineFloat64ArrayType(
        x, {'fun_name': fun_name, 'entity': 'x'},
        exp_shape=(_nb_py,),
    ).to_ctypes()
    _iwt = utils._EngineIntScalarType(
        iwt, {'fun_name': fun_name, 'entity': 'iwt'},
    ).to_ctypes()
    _lwt_py = utils._EngineFloat64ArrayType.get_shape(wt, {'fun_name': fun_name, 'entity': 'wt'}, exp_rank=1, allow_none=True)[0]
    _lwt = utils._EngineIntScalarType(
        _lwt_py, {'fun_name': fun_name, 'entity': '_lwt'},
    ).to_ctypes()
    _iwt_py = utils._EngineIntScalarType.to_py(_iwt)
    if _iwt_py in (1, 2):
        if wt is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'wt'},
                'must not be None when iwt in (1, 2)',
            )
    _m_py = utils._EngineIntScalarType.to_py(_m)
    if _iwt_py == 1:
        _wt_dim1 = _nb_py
    elif _iwt_py == 2:
        _wt_dim1 = _m_py
    else:
        _wt_dim1 = 0
    exp_shape_wt = (_wt_dim1,)
    _wt = utils._EngineFloat64ArrayType(
        wt, {'fun_name': fun_name, 'entity': 'wt'},
        exp_shape=exp_shape_wt,
        check_contents=_iwt_py in (1, 2),
        allow_none=True,
    ).to_ctypes(can_reference_contents=_iwt_py in (1, 2))
    _pn = utils._EngineIntScalarType(
        pn, {'fun_name': fun_name, 'entity': 'pn'},
    ).to_ctypes()
    _pn_py = utils._EngineIntScalarType.to_py(_pn)
    _rmean_cls = utils._EngineFloat64ArrayType(
        rmean, {'fun_name': fun_name, 'entity': 'rmean'},
        exp_shape=(max(0, _nb_py+min(0, _pn_py-_m_py+1)),),
        check_contents=False,
    )
    _rmean = _rmean_cls.to_ctypes(can_reference_contents=False)
    if rsd is not None:
        _lrsd = utils._EngineIntScalarType(
            max(0, _nb_py+min(0, _pn_py-_m_py+1)), {'fun_name': fun_name, 'entity': '_lrsd'},
        ).to_ctypes()
    else:
        _lrsd = utils._EngineIntScalarType.c_type(0)
    if rsd is not None:
        _rsd_dim1 = max(0, _nb_py+min(0, _pn_py-_m_py+1))
    else:
        _rsd_dim1 = 0
    exp_shape_rsd = (_rsd_dim1,)
    _rsd_cls = utils._EngineFloat64ArrayType(
        rsd, {'fun_name': fun_name, 'entity': 'rsd'},
        exp_shape=exp_shape_rsd,
        check_contents=False,
        allow_none=True,
    )
    _rsd = _rsd_cls.to_ctypes(can_reference_contents=False)
    _vl_dummy_rcomm = utils._EngineFloat64ArrayType.empty_ctypes(0)
    if _pn_py != 0:
        if comm is None:
            raise utils.NagTypeError(
                {'fun_name': fun_name, 'entity': 'comm'},
                'must not be None when pn != 0',
            )
    if _pn_py != 0:
        _comm_needs_init = utils._check_comm(
            comm, {'fun_name': fun_name, 'entity': 'comm'},
            req_keys=('rcomm',),
        )
    else:
        _comm_needs_init = utils._check_comm(
            comm, {'fun_name': fun_name, 'entity': 'comm'},
            opt_keys=('rcomm',),
            allow_none=True,
        )
    if comm is None:
        _rcomm = _vl_dummy_rcomm
    else:
        if _comm_needs_init:
            if comm is None:
                _rcomm_dim1 = 0
            else:
                _rcomm_dim1 = 2*_m_py+20
            _rcomm = utils._EngineFloat64ArrayType.empty_ctypes(_rcomm_dim1)
            comm['rcomm'] = _rcomm
        else:
            _rcomm = comm['rcomm']
    if comm is None:
        _lrcomm_py = utils._EngineFloat64ArrayType.get_shape(_vl_dummy_rcomm, {'fun_name': fun_name, 'entity': '_vl_dummy_rcomm'}, exp_rank=1)[0]
        _lrcomm = utils._EngineIntScalarType(
            _lrcomm_py, {'fun_name': fun_name, 'entity': '_lrcomm'},
        ).to_ctypes()
    else:
        _lrcomm_py = utils._EngineFloat64ArrayType.get_shape(_rcomm, {'fun_name': fun_name, 'entity': '_rcomm'}, exp_rank=1, validate=False)[0]
        _lrcomm = utils._EngineIntScalarType(
            _lrcomm_py, {'fun_name': fun_name, 'entity': '_lrcomm'},
        ).to_ctypes()
    _eps = utils._EngineFloat64ScalarType(
        -1.0, {'fun_name': fun_name, 'entity': '_eps'},
    ).to_ctypes()
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _m,
        _nb,
        _x,
        _iwt,
        _wt,
        _pn,
        _rmean,
        _rsd,
        _lrsd,
        _rcomm,
        _lrcomm,
        _lwt,
        _eps,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 11 and _ifmt == 11:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: m >= 1.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 12 and _ifmt == 12:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On entry at previous call, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if pn > 0, m must be unchanged since previous call.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 21 and _ifmt == 21:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nb = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: nb >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 22 and _ifmt == 22:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, nb = {:s}, m = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if comm is None, nb >= m.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 41 and _ifmt == 41:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iwt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: iwt = 0, 1, 2 or 3.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 42 and _ifmt == 42:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, iwt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On entry at previous call, iwt = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if pn > 0, iwt must be unchanged since previous call.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 51 and _ifmt == 51:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wt[{:s}] = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: wt[i-1] >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 52 and _ifmt == 52:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, wt[0] = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if iwt = 2, wt[0] > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 53 and _ifmt == 53:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, at least one window had all zero weights.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 54 and _ifmt == 54:
            _err_class = utils._EngineWarningExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, unable to calculate at least one standard deviation',
                    'due to the weights supplied.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 55 and _ifmt == 55:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, sum of weights supplied in wt is {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: if iwt = 2, the sum of the weights > 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 61 and _ifmt == 61:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, pn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: pn >= 0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 62 and _ifmt == 62:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, pn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[1]),
                    ),
                    'On exit from previous call, pn = {:s}.'.format(
                        utils._STR_FMT_INT_AS_STR.format(_eb_data[2]),
                    ),
                    'Constraint: if pn > 0, pn must be unchanged since previous call.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 101 and _ifmt == 101:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'comm[\'rcomm\'] has been corrupted between calls.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, (utils._EngineNoErrorExit, utils._EngineWarningExit)):
        _pn_py = utils._EngineIntScalarType.to_py(_pn)
        _rmean_cls.refresh_py(_rmean)
        if rsd is not None:
            _rsd_cls.refresh_py(_rsd)
        return_data = _pn_py
        if isinstance(_err_class, utils._EngineWarningExit):
            _err_class.return_data = _collections.namedtuple(
                'nag_return_data',
                ['pn'],
            )(
                _pn_py,
            )
    _err_class.manage()
    return return_data

def init_vavilov(rkappa, beta2, mode):
    u"""
    Initialization function for ``pdf_vavilov`` and ``prob_vavilov``.

    For full information please refer to the NAG Library document for
    g01zu

    https://www.nag.com/numeric/nl/nagdoc_27/flhtml/g01/g01zuf.html

    Parameters
    ----------
    rkappa : float
        The argument kappa of the function.

    beta2 : float
        The argument beta^2 of the function.

    mode : int
        If `mode` = 0, ``pdf_vavilov`` is to be called after the call to
        ``init_vavilov``. Otherwise, ``prob_vavilov`` is to be called.

    Returns
    -------
    xl : float
        x_l, a threshold value below which phi_V(lambda;kappa,beta^2)
        will be set to zero by ``pdf_vavilov`` and
        Phi_V(lambda;kappa,beta^2) will be set to zero by
        ``prob_vavilov`` if lambda < x_l.

    xu : float
        x_u, a threshold value above which phi_V(lambda;kappa,beta^2)
        will be set to zero by ``pdf_vavilov`` and
        Phi_V(lambda;kappa,beta^2) will be set to unity by
        ``prob_vavilov`` if lambda > x_u.

    comm : dict, communication object
        Communication structure.

    Raises
    ------
    NagValueError
        (`errno` 1)
            On entry, `beta2` = *<value>*.

            Constraint: `beta2` <= 1.0.

        (`errno` 1)
            On entry, `beta2` = *<value>*.

            Constraint: `beta2` >= 0.0.

        (`errno` 1)
            On entry, `rkappa` = *<value>*.

            Constraint: `rkappa` <= 10.0.

        (`errno` 1)
            On entry, `rkappa` = *<value>*.

            Constraint: `rkappa` >= 0.01.

        (`errno` 2)
            The initialization has been abandoned due to an internal
            error. This error exit is unlikely to occur, but if it does
            change the values of `rkappa` and/or `beta2` and rerun
            ``init_vavilov``.
    """

    excs = []
    fun_name = 'naginterfaces.base.stat.init_vavilov'

    def cb_py_c_print_rec(en_data, _, rec, length_rec, ifail): # pragma: no cover
        # pylint: disable=missing-docstring
        import logging
        try:
            utils._py_print_rec(
                en_data,
                -1,
                rec[:utils._EngineIntScalarType.to_py(
                    length_rec.contents
                )],
                None,
                excs,
                ifail.contents,
                logger=logging.getLogger(fun_name),
            )
        except:
            utils._print_err(
                _sys.exc_info(),
                excs,
                en_data,
                ifail.contents,
            )

    engine_ctypes = utils._EngineState._get_symbol('g01zuft')
    engine_ctypes.restype = None
    engine_ctypes.argtypes = (
        utils._BYREF_ENGINEDATA,
        utils._BYREF_FLOAT64, # Input, rkappa.
        utils._BYREF_FLOAT64, # Input, beta2.
        utils._BYREF_INT, # Input, mode.
        utils._BYREF_FLOAT64, # Output, xl.
        utils._BYREF_FLOAT64, # Output, xu.
        utils._EngineFloat64ArrayType.c_type, # Output, rcomm[322].
        utils._EngineErrbuf,
        utils._BYREF_INT, # Output, ifail.
        utils._EngineState._chrlen_ctype, # Input, hidden chrlen of errbuf.
    )

    _en_data = utils._EngineData(py_c_print_rec=cb_py_c_print_rec)
    _rkappa = utils._EngineFloat64ScalarType(
        rkappa, {'fun_name': fun_name, 'entity': 'rkappa'},
    ).to_ctypes()
    _beta2 = utils._EngineFloat64ScalarType(
        beta2, {'fun_name': fun_name, 'entity': 'beta2'},
    ).to_ctypes()
    _mode = utils._EngineIntScalarType(
        mode, {'fun_name': fun_name, 'entity': 'mode'},
    ).to_ctypes()
    _xl = utils._EngineFloat64ScalarType.empty_ctypes()
    _xu = utils._EngineFloat64ScalarType.empty_ctypes()
    _rcomm = utils._EngineFloat64ArrayType.empty_ctypes(322)
    _comm_py = {}
    _comm_py['rcomm'] = _rcomm
    _errbuf = utils._EngineErrbuf()
    _ierr = utils._EngineIntScalarType.empty_ctypes()
    engine_ctypes(
        _en_data,
        _rkappa,
        _beta2,
        _mode,
        _xl,
        _xu,
        _rcomm,
        _errbuf,
        _ierr,
        _errbuf._length_,
    )
    utils._handle_cb_exception(excs)
    _ierr_py = utils._EngineIntScalarType.to_py(_ierr)
    if _ierr_py == utils._IERR_SUCCESS:
        _err_class = utils._EngineNoErrorExit()
    else:
        _ifmt, _eb_data = utils._EngineState._handle_global_errors(fun_name, _errbuf, _ierr_py)
        if _ierr_py == 1 and _ifmt == 99996:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, beta2 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: beta2 <= 1.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99997:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, beta2 = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: beta2 >= 0.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99998:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rkappa = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rkappa <= 10.0.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 1 and _ifmt == 99999:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'On entry, rkappa = {:s}.'.format(
                        utils._STR_FMT_FLOAT_AS_STR.format(_eb_data[1]),
                    ),
                    'Constraint: rkappa >= 0.01.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        elif _ierr_py == 2 and _ifmt == 99995:
            _err_class = utils._EngineErrorExit(
                fun_name=fun_name,
                msg_lines=[
                    'The initialization has been abandoned due to an internal error. This',
                    'error exit is unlikely to occur, but if it does change the values of',
                    'rkappa and/or beta2 and rerun init_vavilov.',
                ],
                errcodes=(_ierr_py, _ierr_py, _ifmt),
            )
        else:
            _err_class = utils._EngineUnexpectedExit(
                fun_name=fun_name,
                errcodes=(utils._IERR_UNEXPECTED, _ierr_py, _ifmt),
            )
    if isinstance(_err_class, utils._EngineNoErrorExit):
        _xl_py = utils._EngineFloat64ScalarType.to_py(_xl)
        _xu_py = utils._EngineFloat64ScalarType.to_py(_xu)
        return_data = _collections.namedtuple(
            'nag_return_data',
            ['xl', 'xu', 'comm'],
        )(
            _xl_py,
            _xu_py,
            _comm_py,
        )
    _err_class.manage()
    return return_data
